function ActualsData = VerifyActualsData(actuals,n,T)
% VerifyActualsData: Checks if the data in actuals is proper to use as actuals when forecasting.
%
% USAGE:
%
%       ActualsData = VerifyActualsData(actuals,n,T)
%
% REQUIRED INPUT:  actuals (vector) structure with fields "data" and "title" is proper.
%
%                  n (integer) with the number of observed variables.
%
%                  T (integet) with the number of observations for the data that the actuals can
%                    replace when forecasting.
%
% REQUIRED OUTPUT: ActualsData (vector) structure with the proper actuals.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   DG-R/MPR
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2017 European Central Bank.
%
%                       First version: June 12, 2015.
%                        This version: February 10, 2017.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 15-06-2015: Updated the documentation.
%
% * 20-01-2016: Updated the documentation.
%
% * 10-02-2017: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize
%
ActualsData = [];
if isempty(actuals)==1;
   return;
end;
m = length(actuals);
if isstruct(actuals)==0;
   return;
end;
j = 0;
for i=1:m;
   if isfield(actuals(i),'data')==1;
      j = j+1;
      %
      % test if the dimentions are correct
      %
      Data = actuals(i).data;
      if size(Data,1)~=n;
         Data = Data';
      end;
      [n1,T1] = size(Data);
      %
      % the data is considered proper if the number of variables match the
      % "original" dataset and the number of observations is greater than
      % or equal to the number of observations of the "original" dataset.
      %
      if (n1==n)&(T1>=T);
         ActualsData(j).data = Data;
         if isfield(actuals(i),'title')==1;
            if (isempty(actuals(i).title)==0)&(ischar(actuals(i).title(1,:))==1);
               ActualsData(j).title = StringTrim(actuals(i).title(1,:));
            else;
               ActualsData(j).title = ['Actuals dataset number ' int2str(j)];
            end;
         else;
            ActualsData(j).title = ['Actuals dataset number ' int2str(j)];
         end;
      end;
   end;
end;

%
% end of VerifyActualsData.m
%
