function DSGEModel = VerifyStateCondInformation(DSGEModel,DataStructure)
% VerifyStateCondInformation: Checks if there is valid information for running conditional
%                             forecasts with state variable assumptions in the DataStructure
%                             structure.
%
% USAGE:
%
%       DSGEModel = VerifyStateCondInformation(DSGEModel,DataStructure)
%
% REQUIRED INPUT:  DSGEModel (structure) whose fields contain all required information
%                            about the model, such as input files, variable and sample
%                            data. Actual observations are not stored in this file.
%
%                  DataStrcuture (structure) where this function looks at the fields zeta.data
%                            zeta.names, and zeta.K3.
%
% REQUIRED OUTPUT: DSGEModel (structure) whose fields contain all required information
%                            about the model, such as input files, variable and sample
%                            data. Actual observations are not stored in this file.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   DG-R/MPR
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2017 European Central Bank.
%
%                       First version: July 14, 2009.
%                        This version: February 10, 2017.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 15-07-2009: Updated the documentation.
%
% * 05-01-2010: Updated the documentation.
%
% * 05-01-2011: Updated the documentation.
%
% * 21-03-2011: The field "ZetaMixedConditionalShocks" is now supported.
%
% * 22-03-2011: Changed one of the tests for resetting the field "ZetaMixedConditionalShocks".
%
% * 03-01-2012: Updated the documentation.
%
% * 03-01-2013: Updated the documentation.
%
% * 27-01-2014: Updated the documentation.
%
% * 12-01-2015: Updated the documentation.
%
% * 20-01-2016: Updated the documentation.
%
% * 10-02-2017: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize
%
DSGEModel.Zeta = [];
DSGEModel.ZetaVariableNames = '';
DSGEModel.K3 = [];
%
if isfield(DSGEModel,'ZetaConditionalShocks')==0;
   DSGEModel.ZetaConditionalShocks = [];
end;
if isfield(DSGEModel,'ZetaMixedConditionalShocks')==0;
   DSGEModel.ZetaMixedConditionalShocks = [];
end;
if isfield(DSGEModel,'ZetaConditionalVariables')==0;
   DSGEModel.ZetaConditionalVariables = [];
end;
if isfield(DataStructure,'zeta')==0;
   DSGEModel.ZetaConditionalShocks = [];
   DSGEModel.ZetaConditionalVariables = [];
   DSGEModel.ZetaMixedConditionalShocks = [];
   return;
end;
if isfield(DataStructure.zeta,'data')==0;
   DSGEModel.ZetaConditionalShocks = [];
   DSGEModel.ZetaConditionalVariables = [];
   DSGEModel.ZetaMixedConditionalShocks = [];
   return;
end;
if isfield(DataStructure.zeta,'names')==0;
   DSGEModel.ZetaConditionalShocks = [];
   DSGEModel.ZetaConditionalVariables = [];
   DSGEModel.ZetaMixedConditionalShocks = [];
   return;
end;
if isfield(DataStructure.zeta,'K3')==0;
   DSGEModel.ZetaConditionalShocks = [];
   DSGEModel.ZetaConditionalVariables = [];
   DSGEModel.ZetaMixedConditionalShocks = [];
   return;
end;
if isfield(DSGEModel,'StateVariableNames')==0;
   return;
end;
r = size(DSGEModel.StateVariableNames,1);
if isfield(DSGEModel,'StateShockNames')==0;
   return;
end;
q = size(DSGEModel.StateShockNames,1);
%
% all fields are available.
%
% 1. check if dimensions match for Zeta, ZetaVariableNames, and K3
%
Zeta = DataStructure.zeta.data;
ZetaVariableNames = DataStructure.zeta.names;
K3 = DataStructure.zeta.K3;
[n m] = size(K3);
if size(Zeta,1)>size(Zeta,2);
   Zeta = Zeta';
end;
Tz = size(Zeta,2);
if (Tz==0)|(m==0)|(m~=size(Zeta,1))|(m~=length(ZetaVariableNames))|(m~=rank(K3))|(n~=r)|(m>min(DSGEModel.n,q));
   DSGEModel.ZetaConditionalShocks = [];
   DSGEModel.ZetaConditionalVariables = [];
   DSGEModel.ZetaMixedConditionalShocks = [];
   return;
end;
%
% Everything looks OK. we can now let the DSGEModel structure hold
% these data
%
DSGEModel.Zeta = Zeta;
DSGEModel.ZetaVariableNames = char(ZetaVariableNames);
DSGEModel.K3 = K3;
%
if (isempty(DSGEModel.ZetaConditionalVariables)==1)|(max(DSGEModel.ZetaConditionalVariables)>m);
   DSGEModel.ZetaConditionalVariables = (1:m);
end;
if (isempty(DSGEModel.ZetaConditionalShocks)==1)|(max(DSGEModel.ZetaConditionalShocks)>q);
   DSGEModel.ZetaConditionalShocks = (1:m);
end;
if (isempty(DSGEModel.ZetaMixedConditionalShocks)==1)|(max(DSGEModel.ZetaMixedConditionalShocks)>q)|(length(DSGEModel.ZetaMixedConditionalShocks)<=length(DSGEModel.ZetaConditionalShocks));
   DSGEModel.ZetaMixedConditionalShocks = (1:q-1);
end;
if length(DSGEModel.ZetaConditionalShocks)+1==q;
   DSGEModel.ZetaMixedConditionalShocks = [];
end;

%
% end of VerifyStateCondInformation.m
%
