function [XValues,YValues] = NormalDensityValues(mu,Sigma,NumGridPoints)
% NormalDensityValues: Computes the height of the normal density for a grid of X-axis values based
%                      on a normal distribution.
%
% USAGE:
%
%       [XValues,YValues] = NormalDensityValues(mu,Sigma,NumGridPoints)
%
% REQUIRED INPUT:  mu (matrix) of dimension nxh with the mean of n variables at h
%                     points in time.
%
%                  Sigma (matrix) of dimension nxh with the variance of n variables
%                     at h points in time.
%
%                  NumGridPoint (integer) with the number of grid points to use for
%                     the X-axis.
%
% REQUIRED OUTPUT: XValues (matrix) of dimension nxNumGridPointsxh with the X-axis
%                     values.
%
%                  YValues (matrix) of dimension nxNumGridPoints with the Y-axis
%                     values.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   DG-R/MPR
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2017 European Central Bank.
%
%                       First version: July 22, 2010.
%                        This version: February 10, 2017.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 27-09-2010: Updated the documentation.
%
% * 05-01-2011: Updated the documentation.
%
% * 02-01-2012: Updated the documentation.
%
% * 03-01-2013: Updated the documentation.
%
% * 27-01-2014: Updated the documentation.
%
% * 13-01-2015: Updated the documentation.
%
% * 20-01-2016: Updated the documentation.
%
% * 10-02-2017: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% intialize
%
[n,h] = size(mu);
XValues = zeros(n,NumGridPoints,h);
YValues = XValues;
%
for t=1:h;
   %
   % set up the grid points for time t
   %
   sig = sqrt(Sigma(:,t));
   MinVec = mu(:,t)-(4*sig);
   MaxVec = mu(:,t)+(4*sig);
   for i=1:n;
      XValues(i,:,t) = (MinVec(i):(MaxVec(i)-MinVec(i))/(NumGridPoints-1):MaxVec(i));
   end;
   for j=1:NumGridPoints;
      YValues(:,j,t) = exp(logNormalPDF(XValues(:,j,t),mu(:,t),sig));
   end;
   %
   % make sure that the YValues "integrate" to exactly unity over the different X-values
   %
   IntValue = sum(YValues(:,:,t),2).*(XValues(:,2,t)-XValues(:,1,t));
   YValues(:,:,t) = ((1./IntValue)*ones(1,NumGridPoints)).*YValues(:,:,t);
end;

%
% end of NormalDensityValues.m
%
