function SVEs = DSGEVARCorrSimulationDistribution(UsedDraws,CurrChain,IsPosterior,Weights,PME,lambda,NumPaths,DSGEModel,CurrINI,controls)
% DSGEVARCorrSimulationDistribution: Compute the distribution of the sample correlations of the observed
%                                    variables based on a DSGE-VAR model.
%
% USAGE:
%
%       SVEs = DSGEVARCorrSimulationDistribution(UsedDraws,CurrChain,IsPosterior,Weights,PME,lambda,NumPaths,...
%                                                DSGEModel,CurrINI,controls)
%
% REQUIRED INPUT:  UsedDraws (integer) with the number of parameter draws used for the correlations.
%
%                  CurrChain (integer) with the number for the current Markov chain.
%
%                  IsPosterior (boolean) that is 1 if draws from the posterior distribution
%                              are used and 0 if draws from the prior are supplied.
%
%                  Weights (vector) of dimension UsedDraws x 1 with the SMC based weights. Is empty for
%                              all MCMC samplers.
%
%                  PME (integer) that takes the value 1 if the DSGE model posterior mode estimates
%                              are used, 2 if the DSGE-VAR marginal posterior mode estimates were
%                              used, and 3 if the DSGE-VAR joint posterior mode estimates were used
%                              when generating the posterior draws.
%
%                  lambda (scalar) with the value of the lambda hyperparameter for the DSGE-VAR.
%
%                  NumPaths (integer) with the number of simulated paths per parameter value.
%
%                  DSGEModel (structure)  whose fields contain all required information about the
%                              model, such as input files, variable and sample data.
%
%                  controls (structure) whose fields provide handles to all the UI controls on YADA's
%                              main dialog.
%
% REQUIRED OUTPUT: SVEs (structure) with fields "Data", "Mean", "Std", "Corr" and "Quantiles". The
%                              Data field has sub-fields "Mean", "Std" and "Corr". Quantiles is
%                              a vector structure with sub-fields "Mean", "Std" and "percent".
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   DG-R/MPR
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2017 European Central Bank.
%
%                       First version: June 18, 2010.
%                        This version: September 11, 2017.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 21-06-2010: Updated the documentation.
%
% * 22-10-2010: The wait dialog now also display the number of lags of the DSGE-VAR.
%
% * 05-01-2011: Updated the documentation.
%
% * 11-01-2011: The function now supports unix and mac paths.
%
% * 25-11-2011: The function now sorts the skewness statistics.
%
% * 02-12-2011: The function now also handles kurtosis.
%
% * 03-01-2012: Updated the documentation.
%
% * 27-08-2012: Added support for higest probability density confidence bands.
%
% * 03-01-2013: Updated the documentation.
%
% * 04-09-2013: Added "try-catch" code around possibly too large matrices.
%
% * 27-01-2014: Updated the documentation.
%
% * 13-01-2015: Updated the documentation.
%
% * 21-01-2016: Updated the documentation.
%
% * 10-02-2017: Updated the documentation.
%
% * 11-09-2017: Added the input variable "Weights".
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize
%
SVEs = [];
if isempty(Weights)==1;
   Weights = ones(UsedDraws,1);
end;
Weights = kron(Weights,ones(NumPaths,1));
%
% 1. determine the number of draws of the DSGE model parameters
%
DrawsStr = get(controls.posterior.draws,'String');
NumDraws = str2num(StringTrim(strrep(DrawsStr(DSGEModel.PosteriorDrawsValue,:),',','')));
%
% 2. Number of parallel chains
%
ChainsStr = get(controls.posterior.chains,'String');
NumChains = str2num(StringTrim(ChainsStr(DSGEModel.ParallelChainsValue,:)));
n = DSGEModel.n;
k = DSGEModel.k;
p = DSGEModel.BVARLags;
%
[y,Y,HSample,x,X] = GetDSGEVARData(DSGEModel);
T = size(y,2);
%
% compute population covariances and autocorrelations
%
if strcmp(DSGEModel.DataFrequency,'q')==1;
   SampleFreq = 4;
elseif strcmp(DSGEModel.DataFrequency,'m')==1;
   SampleFreq = 12;
else;
   SampleFreq = 1;
end;
%
AutoCovHorizon = min([SampleFreq*3 T-20 18]);
%
%
% compute the data measures
%
if k>0;
   SVEs.Data.Mean = (y*x')/(x*x');
   y = y-(SVEs.Data.Mean*x);
else;
   SVEs.Data.Mean = (1/T)*sum(y,2);
   y = y-(SVEs.Data.Mean*ones(1,T));
end;
AutoCov = (1/T)*y*y';
StdVec = sqrt(diag(AutoCov));
SVEs.Data.Cov = AutoCov;
SVEs.Data.Std = StdVec;
SVEs.Data.Skew = ((1/T)*sum(y.*y.*y,2))./(StdVec.*StdVec.*StdVec);
SVEs.Data.Kurt = ((1/T)*sum(y.*y.*y.*y,2))./(StdVec.*StdVec.*StdVec.*StdVec);
SVEs.Data.Corr = zeros((n*(n+1))/2,1+(2*AutoCovHorizon));
SVEs.Data.Corr(:,AutoCovHorizon+1) = vech(AutoCov./(StdVec*StdVec'));
for c=1:AutoCovHorizon;
   ys = y(:,c+1:T);
   ylag = y(:,1:T-c);
   AutoCov = (1/T)*ys*ylag';
   CorrMat = AutoCov./(StdVec*StdVec');
   SVEs.Data.Corr(:,AutoCovHorizon+1+c) = vech(CorrMat');
   SVEs.Data.Corr(:,AutoCovHorizon+1-c) = vech(CorrMat);
end;
%
NumFiles = UsedDraws;
NumQuants = length(DSGEModel.Percentiles);
%
% initialize
%
if k>0;
   SVEs.Mean = zeros(n,k);
   Weightsk = Weights*ones(1,k);
else;
   SVEs.Mean = zeros(n,1);
   Weightsk = Weights;
end;
SVEs.Std = zeros(n,1);
SVEs.Skew = zeros(n,1);
SVEs.Kurt = zeros(n,1);
SVEs.Corr = zeros((n*(n+1))/2,1+(2*AutoCovHorizon));
for i=1:NumQuants;
   if k>0;
      SVEs.Quantiles(i).Mean = zeros(n,k);
   else;
      SVEs.Quantiles(i).Mean = zeros(n,1);
   end;
   SVEs.Quantiles(i).Std = zeros(n,1);
   SVEs.Quantiles(i).Skew = zeros(n,1);
   SVEs.Quantiles(i).Kurt = zeros(n,1);
   SVEs.Quantiles(i).Corr = zeros((n*(n+1))/2,1+(2*AutoCovHorizon));
   SVEs.Quantiles(i).percent = DSGEModel.Percentiles(i);
end;
%
PriorHeader = '';
if IsPosterior==0;
   PriorHeader = ' - Prior';
end;
%
% setup a wait dialog
%
txt = ['Please wait while YADA computes the distributions for the DSGE-VAR(lambda=' num2str(lambda,'%0.4f') ',p=' int2str(p) ') model based sample means, autocorrelations, skewness, and kurtosis of the observed variables. The data are loaded sequentially from disk and it may therefore take some time. Current variable: ' StringTrim(DSGEModel.VariableNames(1,:)) '.'];
WaitHandle = WaitDLG(txt,'information',['Observed variable Correlations' PriorHeader],600,400,CurrINI,0);
WaitControls = get(WaitHandle,'UserData');
drawnow;
pause(0.02);
%
for i=1:n;
   %
   % setup a wait dialog
   %
   txt = ['Please wait while YADA computes the distributions for the DSGE-VAR(lambda=' num2str(lambda,'%0.4f') ',p=' int2str(p) ') model based sample means, autocorrelations, skewness, and kurtosis of the observed variables. The data are loaded sequentially from disk and it may therefore take some time. Current variable: ' StringTrim(DSGEModel.VariableNames(i,:)) '.'];
   set(WaitControls.text,'String',txt);
   drawnow;
   %
   % set up a matrix for the data
   %
   if k>0;
      TempMean = zeros(UsedDraws*NumPaths,k);
   else;
      TempMean = zeros(UsedDraws*NumPaths,1);
   end;
   TempStd = zeros(UsedDraws*NumPaths,1);
   TempSkew = zeros(UsedDraws*NumPaths,1);
   TempKurt = zeros(UsedDraws*NumPaths,1);
   CurrSaves = 0;
   for ThisSave=1:NumFiles;
      if IsPosterior==1;
         file = FixFilePath([DSGEModel.OutputDirectory '\state\DSGEVAR-CorrY-' num2str(lambda,'%0.4f') '-' int2str(p) '-' DSGEModel.NameOfModel '-' int2str(CurrChain) '-' int2str(NumChains) '-' int2str(ThisSave) '-' int2str(NumFiles) '.' int2str(UsedDraws) '-' int2str(NumDraws) '-' int2str(NumPaths) '.mat']);
      else;
         file = FixFilePath([DSGEModel.OutputDirectory '\state\DSGEVAR-CorrY-Prior-' num2str(lambda,'%0.4f') '-' int2str(p) '-' DSGEModel.NameOfModel '-' int2str(ThisSave) '-' int2str(NumFiles) '.' int2str(UsedDraws) '-' int2str(NumPaths) '.mat']);
      end;
      SVEData = load(file);
      SVE = SVEData.SVE;
      if (i==1)&(ThisSave==1);
         IsSkewness = isfield(SVE.CorrData(1),'SkewY');
      end;
      for j=1:NumPaths;
         TempMean(CurrSaves+j,:) = SVE.CorrData(j).MeanY(i,:);
         TempStd(CurrSaves+j,1) = SVE.CorrData(j).StdY(i,1);
         if IsSkewness==1;
            TempSkew(CurrSaves+j,1) = SVE.CorrData(j).SkewY(i,1);
            TempKurt(CurrSaves+j,1) = SVE.CorrData(j).KurtY(i,1);
         else;
            TempSkew(CurrSaves+j,1) = NaN;
            TempKurt(CurrSaves+j,1) = NaN;
         end;
      end;
      CurrSaves = CurrSaves+NumPaths;
      drawnow;
   end;
   %
   % compute the mean
   %
   SVEs.Mean(i,:) = (1/(UsedDraws*NumPaths))*sum(TempMean.*Weightsk,1);
   SVEs.Std(i,1) = (1/(UsedDraws*NumPaths))*sum(TempStd.*Weights,1);
   SVEs.Skew(i,1) = (1/(UsedDraws*NumPaths))*sum(TempSkew.*Weights,1);
   SVEs.Kurt(i,1) = (1/(UsedDraws*NumPaths))*sum(TempKurt.*Weights,1);
   %
   % sort everything
   %
   TempMean = sort(TempMean);
   TempStd = sort(TempStd);
   TempSkew = sort(TempSkew);
   TempKurt = sort(TempKurt);
   if DSGEModel.ConfidenceRegionMethod==1;
      for l=1:NumQuants;
         if DSGEModel.Percentiles(l)<50;
            QuantVal = ceil((DSGEModel.Percentiles(l)/100)*UsedDraws*NumPaths);
         else;
            QuantVal = floor((DSGEModel.Percentiles(l)/100)*UsedDraws*NumPaths);
         end;
         SVEs.Quantiles(l).Std(i,1) = TempStd(QuantVal,1);
         SVEs.Quantiles(l).Skew(i,1) = TempSkew(QuantVal,1);
         SVEs.Quantiles(l).Kurt(i,1) = TempKurt(QuantVal,1);
         SVEs.Quantiles(l).Mean(i,:) = TempMean(QuantVal,:);
      end;
   else;
      for l=1:floor(NumQuants/2);
         ConfDraws = floor((DSGEModel.Percentiles(NumQuants-l+1)-DSGEModel.Percentiles(l))*UsedDraws*NumPaths/100);
         BoundsStd = HighestProbabilityDensityBand(TempStd,ConfDraws);
         SVEs.Quantiles(l).Std(i) = BoundsStd(1);
         SVEs.Quantiles(NumQuants-l+1).Std(i) = BoundsStd(2);
         BoundsSkew = HighestProbabilityDensityBand(TempSkew,ConfDraws);
         SVEs.Quantiles(l).Skew(i) = BoundsSkew(1);
         SVEs.Quantiles(NumQuants-l+1).Skew(i) = BoundsSkew(2);
         BoundsKurt = HighestProbabilityDensityBand(TempKurt,ConfDraws);
         SVEs.Quantiles(l).Kurt(i) = BoundsKurt(1);
         SVEs.Quantiles(NumQuants-l+1).Kurt(i) = BoundsKurt(2);
         for k1=1:k;
            BoundsMean = HighestProbabilityDensityBand(TempMean(:,k1),ConfDraws);
            SVEs.Quantiles(l).Mean(i,k1) = BoundsMean(1);
            SVEs.Quantiles(NumQuants-l+1).Mean(i,k1) = BoundsMean(2);
         end;
      end;
   end;
   drawnow;
end;
%
% now we need to deal with the autocorrelations
%
Weights = Weights*ones(1,(n*(n+1))/2);
SVEs.AutoCovHorizon = (-AutoCovHorizon:1:AutoCovHorizon);
for j=1:(2*AutoCovHorizon)+1;
   %
   % setup a wait dialog
   %
   txt = ['Please wait while YADA computes the distributions for the DSGE-VAR(lambda=' num2str(lambda,'%0.4f') ',p=' int2str(p) ') model based sample means, autocorrelations, skewness, and kurtosis of the observed variables. The data are loaded sequentially from disk and it may therefore take some time. Current autocorrelation horizon: ' int2str(j-(AutoCovHorizon+1)) '.'];
   set(WaitControls.text,'String',txt);
   drawnow;
   %
   try;
      TempCorr = zeros(UsedDraws*NumPaths,(n*(n+1))/2);
      CurrSaves = 0;
      for ThisSave=1:NumFiles;
         if IsPosterior==1;
            file = FixFilePath([DSGEModel.OutputDirectory '\state\DSGEVAR-CorrY-' num2str(lambda,'%0.4f') '-' int2str(p) '-' DSGEModel.NameOfModel '-' int2str(CurrChain) '-' int2str(NumChains) '-' int2str(ThisSave) '-' int2str(NumFiles) '.' int2str(UsedDraws) '-' int2str(NumDraws) '-' int2str(NumPaths) '.mat']);
         else;
            file = FixFilePath([DSGEModel.OutputDirectory '\state\DSGEVAR-CorrY-Prior-' num2str(lambda,'%0.4f') '-' int2str(p) '-' DSGEModel.NameOfModel '-' int2str(ThisSave) '-' int2str(NumFiles) '.' int2str(UsedDraws) '-' int2str(NumPaths) '.mat']);
         end;
         SVEData = load(file);
         SVE = SVEData.SVE;
         for s=1:NumPaths;
            TempCorr(CurrSaves+s,:) = SVE.CorrData(s).CorrY(:,j)';
         end;
         CurrSaves = CurrSaves+NumPaths;
         drawnow;
      end;
      %
      % compute the mean
      %
      SVEs.Corr(:,j) = (1/(UsedDraws*NumPaths))*sum(TempCorr.*Weightsn,1)';
      %
      % sort the data and determine the quantile values
      %
      TempCorr = sort(TempCorr);
      if DSGEModel.ConfidenceRegionMethod==1;
         for l=1:NumQuants;
            if DSGEModel.Percentiles(l)<50;
               QuantVal = ceil((DSGEModel.Percentiles(l)/100)*UsedDraws*NumPaths);
            else;
               QuantVal = floor((DSGEModel.Percentiles(l)/100)*UsedDraws*NumPaths);
            end;
            SVEs.Quantiles(l).Corr(:,j) = TempCorr(QuantVal,:)';
         end;
      else;
         for l=1:floor(NumQuants/2);
            ConfDraws = floor((DSGEModel.Percentiles(NumQuants-l+1)-DSGEModel.Percentiles(l))*UsedDraws*NumPaths/100);
            for t=1:(n*(n+1))/2;
               Bounds = HighestProbabilityDensityBand(TempCorr(:,t),ConfDraws);
               SVEs.Quantiles(l).Corr(t,j) = Bounds(1);
               SVEs.Quantiles(NumQuants-l+1).Corr(t,j) = Bounds(2);
            end;
         end;
      end;
   catch;
      for t=1:(n*(n+1))/2;
         TempCorr = zeros(UsedDraws*NumPaths,1);
         CurrSaves = 0;
         for ThisSave=1:NumFiles;
            if IsPosterior==1;
               file = FixFilePath([DSGEModel.OutputDirectory '\state\DSGEVAR-CorrY-' num2str(lambda,'%0.4f') '-' int2str(p) '-' DSGEModel.NameOfModel '-' int2str(CurrChain) '-' int2str(NumChains) '-' int2str(ThisSave) '-' int2str(NumFiles) '.' int2str(UsedDraws) '-' int2str(NumDraws) '-' int2str(NumPaths) '.mat']);
            else;
               file = FixFilePath([DSGEModel.OutputDirectory '\state\DSGEVAR-CorrY-Prior-' num2str(lambda,'%0.4f') '-' int2str(p) '-' DSGEModel.NameOfModel '-' int2str(ThisSave) '-' int2str(NumFiles) '.' int2str(UsedDraws) '-' int2str(NumPaths) '.mat']);
            end;
            SVEData = load(file);
            SVE = SVEData.SVE;
            for s=1:NumPaths;
               TempCorr(CurrSaves+s,1) = SVE.CorrData(s).CorrY(t,j);
            end;
            CurrSaves = CurrSaves+NumPaths;
            drawnow;
         end;
         %
         % compute the mean
         %
         SVEs.Corr(t,j) = (1/(UsedDraws*NumPaths))*sum(TempCorr.*Weights,1);
         %
         % sort the data and determine the quantile values
         %
         TempCorr = sort(TempCorr);
         if DSGEModel.ConfidenceRegionMethod==1;
            for l=1:NumQuants;
               if DSGEModel.Percentiles(l)<50;
                  QuantVal = ceil((DSGEModel.Percentiles(l)/100)*UsedDraws*NumPaths);
               else;
                  QuantVal = floor((DSGEModel.Percentiles(l)/100)*UsedDraws*NumPaths);
               end;
               SVEs.Quantiles(l).Corr(t,j) = TempCorr(QuantVal,1);
            end;
         else;
            for l=1:floor(NumQuants/2);
               ConfDraws = floor((DSGEModel.Percentiles(NumQuants-l+1)-DSGEModel.Percentiles(l))*UsedDraws*NumPaths/100);
               Bounds = HighestProbabilityDensityBand(TempCorr,ConfDraws);
               SVEs.Quantiles(l).Corr(t,j) = Bounds(1);
               SVEs.Quantiles(NumQuants-l+1).Corr(t,j) = Bounds(2);
            end;
         end;
      end;
   end;
   drawnow;
end;
%
% close the wait dialog
%
set(WaitControls.text,'UserData','done');
delete(WaitHandle);
drawnow;
pause(0.02);

%
% end of DSGEVARCorrSimulationDistribution.m
%
