function [A0,IdentifyStatus] = DSGEVARIdentifyShocks(SigmaEpsilon,H,B0,DSGEVARShocks,HSample)
% DSGEVARIdentifyShocks: Computes the contemporaneous responses from the structural shocks in the
%                        DSGE-VAR model.
%
% USAGE:
%
%       [A0,IdentifyStatus] = DSGEVARIdentifyShocks(SigmaEpsilon,H,B0,DSGEVARShocks,HSample)
%
% REQUIRED INPUT:  SigmaEpsilon (matrix) of dimension nxn with estimates of the residual covariance
%                               matrix of the VAR.
%
%                  H (rxnxT) matrix with cofficients on the state variables in the measurement equation.
%
%                  B0 (rxq) matrix with coefficients on the economic shocks in the state equation.
%
%                  DSGEVARShocks (vector) of dimension n with the positions of the shocks to the DSGE-VAR
%                               relative to the columns of B0, i.e., the shocks in the DSGE model.
%
%                  HSample (vector) which determines the positions in a 3D H-matrix where t=1 and t=T
%                               are located.
%
% REUIRED OUTPUT: A0 (matrix) of dimension nxn with the contemporaneous effects of the n structural
%                               shocks in the DSGE-VAR model.
%
%                 IdentifyStatus (boolean) that takes the value 1 if A0 could be determined and 0
%                               otherwise.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   DG-R/MPR
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2017 European Central Bank.
%
%                       First version: January 26, 2010.
%                        This version: February 10, 2017.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 28-01-2010: Updated the documentation.
%
% * 05-01-2011: Updated the documentation.
%
% * 18-04-2011: Made sure that "T = " command is closed when having a 3D H-matrix.
%
% * 03-01-2012: Updated the documentation.
%
% * 03-01-2013: Updated the documentation.
%
% * 27-01-2014: Updated the documentation.
%
% * 13-01-2015: Updated the documentation.
%
% * 21-01-2016: Updated the documentation.
%
% * 10-02-2017: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize
%
A0 = [];
IdentifyStatus = 0;
n = length(DSGEVARShocks);
q = size(B0,2);
%
% check if the shocks can be identified
%
if n>q;
   return;
end;
if IsPositiveDefinite(SigmaEpsilon)==0;
   return;
end;
%
% check if we should fix the H-matrix
%
if length(size(H))==3;
   T = HSample(2)-HSample(1)+1;
   H = (1/T)*sum(H(:,:,HSample(1):HSample(2)),3);
end;
%
% compute the Choleski decomposition of 
%
SigmaEpsilonCholeski = chol(SigmaEpsilon)';
%
% compute the contemporanous impact of the shocks
%
A = H'*B0(:,DSGEVARShocks);
%
% compute the Q-R decomposition of the transpose
%
[Q,R] = qr(A');
%
% where Q is orthonormal and R is upper triangular
%
% now make sure that the diagonal of R has only
% positive entries
%
Dvec = diag(R);
AllElem = (1:n)';
PosElem = (Dvec>0).*AllElem;
PosElem = PosElem(PosElem>0);
NegElem = setdiff(AllElem,PosElem);
S = zeros(n,1);
S(PosElem) = 1;
S(NegElem) = -1;
S = diag(S);
Q = Q*S;
A0 = SigmaEpsilonCholeski*Q';
IdentifyStatus = 1;

%
% end of DSGEVARIdentifyShocks.m
%
