function DoneCalc = DSGEVARImpulseRespDistribution(PhiDraws,A0Draws,h,IRType,CurrChain,IsPosterior,PME,lambda,DSGEModel,CurrINI,controls)
% DSGEVARImpulseRespDistribution: Calculates the impulse responses for the posterior or prior draws of the DSGE-VAR
%                                 model and stores them on disk.
%
% USAGE:
%
%       DoneCalc = DSGEVARImpulseRespDistribution(PhiDraws,A0Draws,h,IRType,CurrChain,IsPosterior,PME,lambda,...
%                                                 DSGEModel,CurrINI,controls)
%
% REQUIRED INPUT:  PhiDraws (matrix) of dimension UsedDraws x n(np+k) with the draws from the
%                           VAR parameters. Each row holds vec(Phi(0) Phi(1) ... Phi(p))'.
%
%                  A0Draws (matrix) of dimension UsedDraws x n^2 with the draws of the A0
%                           parameters. Each row holds vec(A0)'.
%
%                  h (integer) with the response horizon.
%
%                  IRType (integer) that takes the value 1 if the responses concern the
%                           original data, 2 if annualized, and 3 if levels.
%
%                  CurrChain (integer) with the number for the current Markov chain.
%
%                  IsPosterior (boolean) that is 1 if draws from the posterior distribution
%                           are used and 0 if draws from the prior are supplied.
%
%                  PME (integer) that takes the value 1 if the DSGE model posterior mode estimates
%                           are used, 2 if the DSGE-VAR marginal posterior mode estimates were
%                           used, and 3 if the DSGE-VAR joint posterior mode estimates were used
%                           when generating the posterior draws.
%
%                  lambda (scalar)  with the value of the lambda hyperparameter for the DSGE-VAR.
%
%                  DSGEModel (structure)  whose fields contain all required information about the
%                           model, such as input files, variable and sample data. Actual observations
%                           are not stored in this file.
%
%                  CurrINI (structure) with initialization information.
%
%                  controls (structure) whose fields provide handles to all the UI controls on YADA's
%                           main dialog.
%
% REQUIRED OUTPUT: DoneCalc (boolean) that is 1 if the calculations were completed and 0 otherwise.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   DG-R/MPR
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2017 European Central Bank.
%
%                       First version: April 15, 2010.
%                        This version: March 20, 2017.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 16-04-2010: Updated the documentation.
%
% * 26-04-2010: The function now makes sure that the output directory exists.
%
% * 21-05-2010: Updated the documentation.
%
% * 26-10-2010: Added info about the lag order on the wait dialog.
%
% * 05-01-2011: Updated the documentation.
%
% * 11-01-2011: Added support for unix and mac paths.
%
% * 12-01-2011: Added additional support for unix and mac paths.
%
% * 03-01-2012: Updated the documentation.
%
% * 03-01-2013: Updated the documentation.
%
% * 27-01-2014: Updated the documentation.
%
% * 13-01-2015: Updated the documentation.
%
% * 05-03-2015: Added code for parallel computations.
%
% * 17-03-2015: Switched to the new IsHandle function.
%
% * 08-06-2015: The code takes the CurrINI.GraphicsRoot field into account.
%
% * 21-01-2016: Updated the documentation.
%
% * 20-03-2017: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize
%
DoneCalc = 0;
UsedDraws = size(PhiDraws,1);
%
% 1. determine the number of draws of the DSGE model parameters
%
DrawsStr = get(controls.posterior.draws,'String');
NumDraws = str2num(StringTrim(strrep(DrawsStr(DSGEModel.PosteriorDrawsValue,:),',','')));
%
% 2. Number of parallel chains
%
ChainsStr = get(controls.posterior.chains,'String');
NumChains = str2num(StringTrim(ChainsStr(DSGEModel.ParallelChainsValue,:)));
%
% 3. the selected shocks
%
SelectedShockStr = ismember((1:length(DSGEModel.StateShockPositions)),DSGEModel.DSGEVARShocks);
SelectedShockStr = strrep(num2str(SelectedShockStr),' ','');
%
if IRType==1;
   DataType = 'Original';
elseif IRType==2;
   DataType = 'Annualized';
elseif IRType==3;
   DataType = 'Levels';
end;
%
% create the output directory
%
stat = MakeDir(DSGEModel.OutputDirectory,'irfs');
if stat~=1;
   if ispc==1;
      txt = ['YADA was for some reason unable to create the directory "' DSGEModel.OutputDirectory '\irfs". The computation of the impulse response distribution has therefore been aborted.'];
   else;
      txt = ['YADA was for some reason unable to create the directory "' DSGEModel.OutputDirectory '/irfs". The computation of the impulse response distribution has therefore been aborted.'];
   end;
   About(txt,'information','YADA - Directory Creation Problem',180,500,CurrINI);
   return;
end;
%
% initialize index for saving
%
SaveAfterDraws = min(UsedDraws,1000);
NumFiles = ceil(UsedDraws/SaveAfterDraws);
%
% begin the loop
%
p = DSGEModel.BVARLags;
n = DSGEModel.n;
k = DSGEModel.k;
%
PriorHeader = '';
if IsPosterior==0;
   PriorHeader = ' For Prior';
end;
if CurrINI.NumWorkers<2;
   abort = '';
   MeanSolutionTime = 0;
   LastDraw = 0;
   ThisSave = 0;
   IR = [];
   %
   % check if we should setup a wait handle
   %
   if DSGEModel.ShowProgress==1;
      ProgressStructure.title = ['Progress for ' num2str(UsedDraws,'%0.0f') ' parameter draws with lambda=' num2str(lambda,'%0.4f')];
      ProgressStructure.facecolor = CurrINI.progress_facecolor;
      ProgressStructure.startfacecolor = CurrINI.progress_startfacecolor;
      ProgressStructure.edgecolor = CurrINI.progress_edgecolor;
      ProgressStructure.bgcolor = CurrINI.progress_bgcolor;
      ProgressStructure.stop = 0;
      ProgressStructure.clock = DSGEModel.ShowProgressClock;
      ProgressStructure.label = 'Mean IRF computation time:';
      %
      ProgressStructure.name = [DataType ' Impulse Responses' PriorHeader];
      ProgressStructure.CurrINI = CurrINI;
      WaitHandle = ProgressDLG(0,ProgressStructure);
      set(WaitHandle,'Color',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'));
      drawnow;
   else;
      txt = ['Please wait while YADA computes impulse responses for the ' lower(DataType) ' data using a DSGE-VAR with lambda=' num2str(lambda,'%0.4f') ' and ' int2str(p) ' lags. Computations started at: ' StringTrim(datestr(now,14))];
      WaitHandle = WaitDLG(txt,'information',[DataType ' Impulse Responses' PriorHeader],500,200,CurrINI,0);
      WaitControls = get(WaitHandle,'UserData');
   end;
   %
   for it=1:UsedDraws;
      LastDraw = LastDraw+1;
      if DSGEModel.ShowProgress==1;
         abort = get(WaitHandle,'UserData');
         if strcmp(abort,'cancel')==1;
            break;
         else;
            ProgressDLG([it/UsedDraws MeanSolutionTime]);
         end;
      end;
      tic;
      Phi = reshape(PhiDraws(it,:)',n,(n*p)+k);
      A0 = reshape(A0Draws(it,:)',n,n);
      IR(LastDraw).IRF = DSGEVARImpulseResponses(Phi(:,k+1:(n*p)+k),A0,p,h,IRType,DSGEModel.levels,DSGEModel.annual,DSGEModel.annualscale);
      MeanSolutionTime = (((it-1)/it)*MeanSolutionTime)+((1/it)*toc);
      %
      % check if we should save to disk
      %
      if (LastDraw==SaveAfterDraws)|(it==UsedDraws);
         ThisSave = ThisSave+1;
         LastDraw = 0;
         %
         % set up the file name and then save
         %
         if IsPosterior==1;
            file = FixFilePath([DSGEModel.OutputDirectory '\irfs\DSGE-VAR-IR-' DataType '-' num2str(lambda,'%0.4f') '-' int2str(p) '-' int2str(PME) '-' SelectedShockStr '-' DSGEModel.NameOfModel '-' int2str(CurrChain) '.' int2str(NumChains) '-' int2str(h) '-' int2str(ThisSave) '-' int2str(NumFiles) '.' int2str(UsedDraws) '-' int2str(NumDraws) '.mat']);
         else;
            file = FixFilePath([DSGEModel.OutputDirectory '\irfs\DSGE-VAR-IR-Prior-' DataType '-' num2str(lambda,'%0.4f') '-' int2str(p) '-' SelectedShockStr '-' DSGEModel.NameOfModel '-' int2str(h) '-' int2str(ThisSave) '-' int2str(NumFiles) '.' int2str(UsedDraws) '.mat']);
         end;
         save(file,'IR');
         %
         % restore the data structure
         %
         IR = [];
      end;
      drawnow;
   end;
   %
   % close the wait dialog
   %
   if DSGEModel.ShowProgress==1;
      if IsHandle(WaitHandle)==1;
         set(WaitHandle,'UserData','done');
         close(WaitHandle);
         drawnow;
         pause(0.02);
      end;
   else;
      set(WaitControls.text,'UserData','done');
      delete(WaitHandle);
      drawnow;
      pause(0.02);
   end;
else;
   abort = '';
   txt = ['Please wait while YADA computes impulse responses for the ' lower(DataType) ' data using a DSGE-VAR with lambda=' num2str(lambda,'%0.4f') ' and ' int2str(p) ' lags. Computations started at: ' StringTrim(datestr(now,14))];
   WaitHandle = WaitDLG(txt,'information',[DataType ' Impulse Responses' PriorHeader],500,200,CurrINI,0);
   WaitControls = get(WaitHandle,'UserData');
   %
   % initialzing
   %
   for i=1:UsedDraws;
      IRF(i).IRF = zeros(n,n,h+1);
   end;
   parfor it=1:UsedDraws;
      Phi = reshape(PhiDraws(it,:)',n,(n*p)+k);
      A0 = reshape(A0Draws(it,:)',n,n);
      IRF(it).IRF = DSGEVARImpulseResponses(Phi(:,k+1:(n*p)+k),A0,p,h,IRType,DSGEModel.levels,DSGEModel.annual,DSGEModel.annualscale);
      drawnow;
   end;
   %
   % close the wait dialog
   %
   set(WaitControls.text,'UserData','done');
   delete(WaitHandle);
   drawnow;
   pause(0.02);
   %
   % save to file
   %
   for i=1:NumFiles;
      %
      % save the data to disk
      %
      IR = IRF(((i-1)*SaveAfterDraws)+1:min(SaveAfterDraws*i,UsedDraws));
      %
      % set up the file name and then save
      %
      if IsPosterior==1;
         file = FixFilePath([DSGEModel.OutputDirectory '\irfs\DSGE-VAR-IR-' DataType '-' num2str(lambda,'%0.4f') '-' int2str(p) '-' int2str(PME) '-' SelectedShockStr '-' DSGEModel.NameOfModel '-' int2str(CurrChain) '.' int2str(NumChains) '-' int2str(h) '-' int2str(i) '-' int2str(NumFiles) '.' int2str(UsedDraws) '-' int2str(NumDraws) '.mat']);
      else;
         file = FixFilePath([DSGEModel.OutputDirectory '\irfs\DSGE-VAR-IR-Prior-' DataType '-' num2str(lambda,'%0.4f') '-' int2str(p) '-' SelectedShockStr '-' DSGEModel.NameOfModel '-' int2str(h) '-' int2str(i) '-' int2str(NumFiles) '.' int2str(UsedDraws) '.mat']);
      end;
      save(file,'IR');
   end;
end;
%
% check if we have computed all output
%
if strcmp(abort,'cancel')==0;
   DoneCalc = 1;
end;

%
% end of DSGEVARImpulseRespDistribution.m
%
