function [lnLt,mcode,PriorStatus,CalcTime] = DSGEVARLogLikelihood(theta,ModelParameters,thetaPositions,DSGEModel,AIMData,lambda,PME,OrderQZ,NumWorkers)
% DSGEVARLogLikelihood: Computes the log-likelhood functions at all t=1,...,T for a DSGE-VAR model, where
%                       both the concentrated and the marginalized likelihood are supported.
%
% USAGE:
%
%       [lnLt,mcode,PriorStatus,CalcTime] = DSGEVARLogLikelihood(theta,ModelParameters,thetaPositions,DSGEModel,AIMData,...
%                                                       lambda,PME,OrderQZ,NumWorkers)
%
% REQUIRED INPUT:  theta (vector) with all parameters to be estimated.
%
%                  ModelParameters (structure) whose fields have names given by the parameter names in the DSGE model.
%
%                  thetaPositions (structure) with fields parameter and distribution. This function
%                                 needs the former field to know which parameter name corresponds to
%                                 position i in theta.
%
%                  DSGEModel (structure) whose fields contain all required information about the model, such as
%                                 input files, variable and sample data. Actual observations are not stored in
%                                 this file.
%
%                  AIMData (structure) with fields given by neq, nlag, nlead etc that is produced when running
%                                 "compute_aim_data". The neq, nlag, and nlead fields are needed by AiMSolver.
%
%                  lambda (real) hyperparameter value that is used to measure degree of misspecification.
%
%                  PME (integer) that takes the value 1 if the DSGE model posterior mode estimates are used,
%                                 2 if the DSGE-VAR marginal posterior mode estimates were used, and 3 if the
%                                 DSGE-VAR joint posterior mode estimates were used.
%
%                  OrderQZ (boolean) that is unity if "ordqz" is a built-in matlab function and zero otherwise.
%
%                  NumWorkers (integer) with the number of workers in a parallel pool.
%
% REQUIRED OUTPUT: lnLt (vector) of dimension T with the time t value of the log-likelihood in each element
%                                 for t=1,...,T.
%
%                  mcode (integer) Indicates whether the solution of the model is unique or not.
%                                  mcode is 1 when unique, 0 if not computed, and various other values when
%                                  there are problems with the solution. Specifically:
%
%                                     mcode = 2:      Roots not correctly computed by real_schur
%                                     mcode = 3:      Too many big roots
%                                     mcode = 35:     Too many big roots, and q(:,right) is singular
%                                     mcode = 4:      Too few big roots
%                                     mcode = 45:     Too few big roots, and q(:,right) is singular
%                                     mcode = 5:      q(:,right) is singular
%                                     mcode = 61:     Too many exact shiftrights
%                                     mcode = 62:     Too many numeric shiftrights
%                                     mcode = 7:      The a matrix has infinite or NaN values. Eigenvalues
%                                                     cannot be calculated.
%                                     mcode = 8:      The function "compute_aim_matrices" returns complex
%                                                     numbers.
%                                     mcode = -1:     No stable solution.
%                                     mcode = -2:     Too many large generalized eigenvalues.
%                                     mcode = -3:     Too few large generalized eigenvalues.
%                                     else            Return code not properly specified
%
%                  PriorStatus (integer) Indicates if the log-likelihood was calculated (PriorStatus=1), or not (PriorStatus=0).
%
%                  CalcTime (scalar) with the time it had taken to compute the log-likelihood.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   DG-R/MPR
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2017 European Central Bank.
%
%                       First version: March 12, 2014.
%                        This version: August 30, 2017.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 13-03-2014: Finished the code from the first "DSGEVARPrior" call and onwards.
%
% * 13-01-2015: Updated the documentation.
%
% * 21-01-2016: Updated the documentation.
%
% * 10-02-2017: Updated the documentation.
%
% * 12-06-2017: The fucntion now supports the system prior.
%
% * 30-08-2017: Added the "NumWorkers" input variable.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize
%
lnLt = [];
mcode = 0;
PriorStatus = 0;
CalcTime = [];
tic;
%
% Copy files to the tmp directory and load data from DSGEModel.AIMDataFile
%
[stat,msg] = CopyFile([GetPath(DSGEModel.AIMDataFile) 'compute_aim_matrices.m'],[pwd '\tmp']);
[stat,msg] = CopyFile(DSGEModel.MeasurementEquationFile,[pwd '\tmp']);
if FileExist(DSGEModel.UpdateParameterFile)==1;
   [stat,msg] = CopyFile(DSGEModel.UpdateParameterFile,[pwd '\tmp']);
end;
if FileExist(DSGEModel.InitializeParameterFile)==1;
   [stat,msg] = CopyFile(DSGEModel.InitializeParameterFile,[pwd '\tmp']);
end;
if FileExist(DSGEModel.SystemPriorFile)==1;
   [stat,msg] = CopyFile(DSGEModel.SystemPriorFile,[pwd '\tmp']);
end;
%
% update ModelParameters
%
ModelParameters = ThetaToModelParameters(ModelParameters,theta,thetaPositions);
%
% Update any parameters that need to be updated
%
if FileExist(DSGEModel.UpdateParameterFile)==1;
   eval(['ModelParameters = ' GetName(DSGEModel.UpdateParameterFile) '(ModelParameters);']);
end;
%
% See if we can solve the model
%
if DSGEModel.ModelSolver==1;
   [mcode,MaxAbsError,BMatrix,SMatrix] = AiMSolver(ModelParameters,AIMData.neq,AIMData.nlag,AIMData.nlead,DSGEModel.AIMTolerance);
elseif DSGEModel.ModelSolver==2;
   [F,B0,mcode] = KleinSolver(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,OrderQZ);
elseif DSGEModel.ModelSolver==3;
   [F,B0,mcode] = SimsSolver(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,OrderQZ);
end;
if mcode~=1;
   if DSGEModel.ModelSolver==1;
      [mcode,MaxAbsError,BMatrix,SMatrix,ModelEigenvalues] = AiMSolver(ModelParameters,AIMData.neq,AIMData.nlag,AIMData.nlead,DSGEModel.AIMTolerance);
   elseif DSGEModel.ModelSolver==2;
      [F,B0,mcode,MaxAbsError,ModelEigenvalues] = KleinSolver(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,OrderQZ);
   elseif DSGEModel.ModelSolver==3;
      [F,B0,mcode,MaxAbsError,ModelEigenvalues] = SimsSolver(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,OrderQZ);
   end;
   if NumWorkers<=1;
	   save(FixFilePath([pwd '\tmp\AiMErrorData.mat']),'mcode','ModelEigenvalues');
	end;
   CalcTime = toc;
   return;
else;
   if (NumWorkers<=1)&(FileExist(FixFilePath([pwd '\tmp\AiMErrorData.mat']))==1);
      delete(FixFilePath([pwd '\tmp\AiMErrorData.mat']));
   end;
end;
%
% transform into state space
%
if DSGEModel.ModelSolver==1;
   [F,B0] = AiMtoStateSpace(BMatrix,SMatrix,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions);
end;
%
% Evaluate the measurement equation file
%
eval(['[A,H,R] = ' GetName(DSGEModel.MeasurementEquationFile) '(ModelParameters,AIMData.endog(DSGEModel.StateVariablePositions,:),DSGEModel.VariableNames,DSGEModel.XVariableNames,DSGEModel.n,size(F,1),DSGEModel.k);']);
%
% determine the data
%
[y,Y,HSample,x,X] = GetDSGEVARData(DSGEModel);
[n,T] = size(y);
lnLt = zeros(1,T);
k = DSGEModel.k;
p = DSGEModel.BVARLags;
npk = (n*p)+k;
RecomputePrior = 1;
CheckRecomputePrior = 1;
%
if isinf(lambda)==0;
   tau = lambda/(1+lambda);
   for t=1:T;
      if t*lambda>=npk+n;
         if RecomputePrior==1;
            if k>0;
               DetProductMoments = CalcDeterministicProductMoments(x(:,1:t),X(:,1:t+p));
            else;
               DetProductMoments = [];
            end;
         end;
         GammaHatyy = (1/t)*y(:,1:t)*y(:,1:t)';
         GammaHatyY = (1/t)*y(:,1:t)*Y(:,1:t)';
         GammaHatYY = (1/t)*Y(:,1:t)*Y(:,1:t)';
         if PME<3;
            logGPR = LogGammaProductRatio(n,((1+lambda)*t)-npk,lambda*t-npk);
            if RecomputePrior==1;
               HSamplet = [HSample(1) HSample(2)+t-T];
               [Gammayy,GammaYY,GammayY,PriorStatus] = DSGEVARPrior(A,H,R,F,B0,DetProductMoments,p,DSGEModel.UseDoublingAlgorithm,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,HSamplet);
               %
               % compute the covariance matrix of the VAR residuals under the prior
               %
               if PriorStatus==1;
                  SigmaEpsilon = Gammayy-((GammayY/GammaYY)*GammayY');
               end;
               %
               % Check if we need to recompute the prior
               %
               if CheckRecomputePrior==1;
                  CheckRecomputePrior = 0;
                  if isempty(DetProductMoments)==1;
                     RecomputePrior = 0;
                  else;
                     if (k==1)&(sum(x==ones(1,T))==T);
                        RecomputePrior = 0;
                     end;
                  end;
               end;
            end;
            if PriorStatus==1;
               %
               % compute the covariance matrix of the VAR residuals under the posterior
               %
               GammaBaryy = (tau*Gammayy)+((1-tau)*GammaHatyy);
               GammaBaryY = (tau*GammayY)+((1-tau)*GammaHatyY);
               GammaBarYY = (tau*GammaYY)+((1-tau)*GammaHatYY);
               %
               % check if GammaBarYY is invertible
               %
               if rank(GammaBarYY)==npk;
                  SigmaHatEpsilon = GammaBaryy-((GammaBaryY/GammaBarYY)*GammaBaryY');
                  %
                  % and now the log-likelihood terms
                  %
                  FirstTerm = -((n/2)*LogDet(GammaYY+((1/lambda)*GammaHatYY))) + ((n/2)*LogDet(GammaYY));
                  SecondTerm = -((t+(t*lambda)-npk)/2)*LogDet((1+(1/lambda))*SigmaHatEpsilon);
                  ThirdTerm = (((t*lambda)-npk)/2)*LogDet(SigmaEpsilon);
                  lnLt(t) = FirstTerm+SecondTerm+ThirdTerm+logGPR-(((n*t)/2)*log(t*lambda*pi));
               end;
            end;
         else;
            logCLC = DSGEVARLogConcLikelihoodConstant(lambda,t,n,npk);
            if RecomputePrior==1;
               HSamplet = [HSample(1) HSample(2)+t-T];
               [Gammayy,GammaYY,GammayY,PriorStatus] = DSGEVARPrior(A,H,R,F,B0,DetProductMoments,p,DSGEModel.UseDoublingAlgorithm,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,HSamplet);
               %
               % compute the covariance matrix of the VAR residuals under the prior
               %
               if PriorStatus==1;
                  SigmaEpsilon = Gammayy-((GammayY/GammaYY)*GammayY');
               end;
               %
               % Check if we need to recompute the prior
               %
               if CheckRecomputePrior==1;
                  CheckRecomputePrior = 0;
                  if isempty(DetProductMoments)==1;
                     RecomputePrior = 0;
                  else;
                     if (k==1)&(sum(x==ones(1,T))==T);
                        RecomputePrior = 0;
                     end;
                  end;
               end;
            end;
            if PriorStatus==1;
               %
               % compute the covariance matrix of the VAR residuals under the posterior
               %
               GammaBaryy = (tau*Gammayy)+((1-tau)*GammaHatyy);
               GammaBaryY = (tau*GammayY)+((1-tau)*GammaHatyY);
               GammaBarYY = (tau*GammaYY)+((1-tau)*GammaHatYY);
               %
               % check if GammaBarYY is invertible
               %
               if rank(GammaBarYY)==npk;
                  SigmaHatEpsilon = GammaBaryy-((GammaBaryY/GammaBarYY)*GammaBaryY');
                  lnLt(t) = ((n/2)*LogDet(GammaYY))+((((lambda*t)-npk)/2)*LogDet(SigmaEpsilon)) ...
                               - (((t+(lambda*t)+n+1)/2)*LogDet((1+(1/lambda))*SigmaHatEpsilon))+logCLC;
               end;
            end;
         end;
      end;
   end;
else;
   for t=1:T;
      if RecomputePrior==1;
         if k>0;
            DetProductMoments = CalcDeterministicProductMoments(x(:,1:t),X(:,1:t+p));
         else;
            DetProductMoments = [];
         end;
      end;
      GammaHatyy = (1/t)*y(:,1:t)*y(:,1:t)';
      GammaHatyY = (1/t)*y(:,1:t)*Y(:,1:t)';
      GammaHatYY = (1/t)*Y(:,1:t)*Y(:,1:t)';
      if RecomputePrior==1;
         HSamplet = [HSample(1) HSample(2)+t-T];
         [Gammayy,GammaYY,GammayY,PriorStatus] = DSGEVARPrior(A,H,R,F,B0,DetProductMoments,p,DSGEModel.UseDoublingAlgorithm,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,HSamplet);
         %
         % compute the covariance matrix of the VAR residuals under the prior
         %
         if PriorStatus==1;
            SigmaEpsilon = Gammayy-((GammayY/GammaYY)*GammayY');
            Phi = GammayY/GammaYY;
         end;
         %
         % Check if we need to recompute the prior
         %
         if CheckRecomputePrior==1;
            CheckRecomputePrior = 0;
            if isempty(DetProductMoments)==1;
               RecomputePrior = 0;
            else;
               if (k==1)&(sum(x==ones(1,T))==T);
                  RecomputePrior = 0;
               end;
            end;
         end;
      end;
      if PriorStatus==1;
         %
         SigmaBarEpsilon = GammaHatyy+(Phi*GammaHatYY*Phi')-(GammaHatyY*Phi')-(Phi*GammaHatyY');
         %
         % and now the log-likelihood
         %
         lnLt(t) = -(((n*t)/2)*log(2*pi))-((t/2)*LogDet(SigmaEpsilon))-((t/2)*sum(diag(SigmaEpsilon\SigmaBarEpsilon)));
      end;
   end;
end;
lnLt = [lnLt(1) lnLt(2:T)-lnLt(1:T-1)];
CalcTime = toc;

%
% end of DSGEVARLogLikelihood.m
%
