function JointLambda = DSGEVARPosteriorModeEstimation(DSGEModel,CurrINI,controls,JointLambda)
% DSGEVARPosteriorModeEstimation: Computes the joint posterior mode for the DSGE model and the
%                                 DSGE-VAR parameters.
%
% USAGE:
%
%       JointLambda = DSGEVARPosteriorModeEstimation(DSGEModel,CurrINI,controls,JointLambda)
%
% REQUIRED INPUT:  DSGEModel (structure) whose fields contain all required information
%                            about the model, such as input files, variable and sample
%                            data. Actual observations are not stored in this file.
%
%                  CurrINI (structure), with initialization information.
%
%                  controls (structure), whose fields provide handles to all the UI
%                            controls in YADA's main dialog.
%
% OPTIONAL INPUT:  JointLambda (vector) with the positions of the selected lambda
%                            values for joint posterior mode estimation.
%
% REQUIRED OUTPUT: JointLambda (vector) with the positions of the selected lambda
%                            values for joint posterior mode estimation.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   DG-R/MPR
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2017 European Central Bank.
%
%                       First version: January 28, 2010.
%                        This version: August 30, 2017.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 29-01-2010: Continued work on the function from the point where the joint log
%               posterior is computed.
%
% * 03-02-2010: Continued work on the function through the call to "DSGEVARJointPosteriorInvHessian".
%
% * 04-02-2010: Continued the function after the call to the inverse Hessian function.
%
% * 05-02-2010: Took into account the "DSGEVARJointPosteriorInvHessian" now provides the "InvHessphi"
%               output variable.
%
% * 08-02-2010: Finished the function.
%
% * 11-02-2010: Fixed a bug when setting up the Y matrix.
%
% * 16-02-2010: The mat and text files now have names that depend on the lag order and on selected shocks.
%
% * 17-02-2010: Made sure that InverseHessian for the phi parameters is properly updated when finite differences
%               has successfully produced an inverse Hessian from the joint posterior.
%
% * 18-02-2010: Made sure that Inf is included in the Lambda vector.
%
% * 22-02-2010: Added 6 more variables that are saved in the mat-file with posterior mode results.
%
% * 24-02-2010: The function now writes an optimization error summary when the inverse Hessian is not positive
%               definite and finite difference methods fail to achieve this as well.
%
% * 25-02-2010: Updated the documentation.
%
% * 04-03-2010: Made sure that "DSGE" is only shown on the X-axis when the Laplace approximation is
%               available for the DSGE model.
%
% * 08-03-2010: The user can now select which lambda values to run the posterior mode estimation for. The
%               output variable "JointLambda" is used to make the selection persistent.
%
% * 09-03-2010: The marginal likelihood figure is only created if there are at least two valid values.
%
% * 17-03-2010: The figure with log marginal likelihoods is now given a filename that depends on the
%               lag order, selected shocks, and model name.
%
% * 03-11-2010: Fixed a bug where logCPC wasn't computed correctly unless the last lambda value was infinity.
%
% * 08-12-2010: The AiM solution time "SolveTime" is now computed until the state-space form has been
%               obtained.
%
% * 16-12-2010: The function now supports the Klein and Sims model solvers.
%
% * 17-12-2010: Took into account that "SolveDSGEModel" and "DSGEVARJointPosteriorInvHessian" take the input
%               variable "OrderQZ" from the CurrINI structure.
%
% * 05-01-2011: Updated the documentation.
%
% * 09-01-2011: Changed the dos delete commands to delete commands.
%
% * 10-01-2011: Changed all dos copy call to "CopyFile" calls.
%
% * 11-01-2011: Added support for unix and mac paths.
%
% * 12-01-2011: Adde additional support for unix and mac paths.
%
% * 24-06-2011: Added try-catch code around the call to csminwel.
%
% * 06-07-2011: Started work to make the function compatible with optimization of the original parameters.
%
% * 11-07-2011: Fixed a bug with the setting the value for "InitialInvHessMat". Began work on integrating
%               newrat.
%
% * 12-07-2011: Finished integrating newrat. Started integrating gmhmaxlik.
%
% * 14-07-2011: Finished integrating gmhmaxlik.
%
% * 24-08-2011: Made sure "F" and "B0" are included in the "ModelParameters" structure via the "YADA" field.
%
% * 03-01-2012: Updated the documentation.
%
% * 22-03-2012: The Laplace approximation of the log marginal likelihood is now computed using the finite
%               difference estimator of the inverse Hessian.
%
% * 28-03-2012: Took into account that the MargLikeLaplace function accepts a third variable.
%
% * 05-06-2012: Took into account that the solvers now take the "StateEquationPositions" input variable.
%
% * 03-01-2013: Updated the documentation.
%
% * 27-01-2014: Updated the documentation.
%
% * 28-01-2014: Switched the "inv" function for "\".
%
% * 03-12-2014: Took into account that "VillaniCheckOptimum" takes the boolean input variable "ShowFigs".
%
% * 13-01-2015: Updated the documentation.
%
% * 15-01-2015: Made "JointLambda" an optional input variable. Took the data in "CurrINI.Sequence.QueryAnswers"
%               into account.
%
% * 17-03-2015: Switched to the new IsHandle function.
%
% * 07-05-2015: Added log text information about the max absolute gradient.
%
% * 21-01-2016: Updated the documentation.
%
% * 10-02-2017: Updated the documentation.
%
% * 06-04-2017: Made it possible to skip the finite difference calculation of the inverse Hessian
%               and thereby the calculation of the Laplace approximation of the marginal likelihood.
%
% * 23-05-2017: Took into account that VillaniCheckOptimum now requires an additional input variable.
%
% * 12-06-2017: Worked on supporting the system prior for DSGE-VARs.
%
% * 13-06-2017: Continued building in support for the system prior.
%
% * 22-06-2017: Took into account that the system prior function now takes the input variable
%               OrderQZ instead of CurrINI.
%
% * 23-06-2017: Took into account that PrintDSGEVARPosteriorModeSummary now takes the maximum absolute
%               value of the gradient vector as input.
%
% * 10-08-2017: Made sure that ModelParameters is updated once the posterior mode has been calculated.
%
% * 18-08-2017: Updated the documentation.
%
% * 29-08-2017: Took into account that "logPosteriorPhiDSGE" requires the input variable "NumWorkers".
%
% * 30-08-2017: Took into account that "logConcPosteriorPhiDSGEVAR" and "logConcPosteriorThetaDSGEVAR" require
%               the input variable "NumWorkers".
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

if nargin==3;
   JointLambda = [];
end;
EstimationStatus = 0;
ScreenMessages = '';
if FileExist(FixFilePath([pwd '\tmp\AiMErrorData.mat']))==1;
   delete(FixFilePath([pwd '\tmp\AiMErrorData.mat']));
end;
%
% Check the file with prior information
%
SetLogText(strvcat(' ','Initializing DSGE-VAR joint posterior mode estimation...',[' * Checking "' GetFilename(DSGEModel.PriorFile) '" with data on prior...']));
%
[status,theta,thetaIndex,thetaDist,LowerBound,ModelParameters,thetaPositions,PriorDist,ParameterNames,UniformBounds] = VerifyPriorData(DSGEModel,CurrINI);
%
if (status==1)&(isempty(theta)==1);
   SetLogText(strvcat('   STATUS: No parameters to estimate!',' '));
   if isempty(CurrINI.Sequence)==1;
      txt = 'All parameters of your model have status calibrated. Hence, YADA doesn''t have any parameters to estimate.';
      About(txt,'information','Calibrated DSGE Model',200,500,CurrINI);
      drawnow;
   end;
   return;
elseif (status~=1)|(isempty(theta)==1);
   SetLogText(strvcat('   STATUS: Bad prior file or inadequate data.',' '));
   return;
end;
OriginalModelParameters = ModelParameters;
phi = ThetaToPhi(theta,thetaIndex,UniformBounds,LowerBound);
%
% Transform PriorDist.gamma and PriorDist.beta to the relevant parameters that are used by the logGammaPDF and
% the logBetaPDF functions
%
if isempty(PriorDist.beta)==0;
   [a,b] = MomentToParamStdbetaPDF((PriorDist.beta(:,1)-PriorDist.beta(:,3))./(PriorDist.beta(:,4)-PriorDist.beta(:,3)),PriorDist.beta(:,2)./(PriorDist.beta(:,4)-PriorDist.beta(:,3)));
   PriorDist.beta_ab = [a b];
else;
   PriorDist.beta_ab = [];
end;
%
if isempty(PriorDist.gamma)==0;
   [a,b] = MomentToParamGammaPDF(PriorDist.gamma(:,1)-PriorDist.gamma(:,3),PriorDist.gamma(:,2));
   PriorDist.gamma_ab = [a b];
else;
   PriorDist.gamma_ab = [];
end;
if isempty(PriorDist.logistic)==0;
   [a,b] = MomentToParamLogisticPDF(PriorDist.logistic(:,1),PriorDist.logistic(:,2),PriorDist.logistic(:,3));
   PriorDist.logistic_ab = [a b PriorDist.logistic(:,3)];
else;
   PriorDist.logistic_ab = [];
end;
if isempty(PriorDist.gumbel)==0;
   [a,b] = MomentToParamGumbelPDF(PriorDist.gumbel(:,1),PriorDist.gumbel(:,2));
   PriorDist.gumbel_ab = [a b];
else;
   PriorDist.gumbel_ab = [];
end;
%
% Now we need to copy files to the tmp directory and load data from DSGEModel.AIMDataFile
%
[stat,msg] = CopyFile([GetPath(DSGEModel.AIMDataFile) 'compute_aim_matrices.m'],[pwd '\tmp']);
[stat,msg] = CopyFile(DSGEModel.MeasurementEquationFile,[pwd '\tmp']);
if FileExist(DSGEModel.UpdateParameterFile)==1;
   [stat,msg] = CopyFile(DSGEModel.UpdateParameterFile,[pwd '\tmp']);
end;
if FileExist(DSGEModel.InitializeParameterFile)==1;
   [stat,msg] = CopyFile(DSGEModel.InitializeParameterFile,[pwd '\tmp']);
end;
if FileExist(DSGEModel.SystemPriorFile)==1;
   [stat,msg] = CopyFile(DSGEModel.SystemPriorFile,[pwd '\tmp']);
   IsSystemPrior = 1;
else;
   IsSystemPrior = 0;
end;
ErrorStr = '';
ErrorFile = '';
try;
   AIMData = load(DSGEModel.AIMDataFile);
catch;
   ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' lasterr];
   ErrorFile = DSGEModel.AIMDataFile;
end;
%
% The structure AIMData contains fields such as neq, nlead and nlag
%
firstUpdatePos = 0;
lastUpdatePos = 0;
firstInitializePos = 0;
lastInitializePos = 0;
if isempty(ErrorStr)==1;
   if DSGEModel.RunInitializeFirst==1;
      %
      % check if the file with additional parameters that only needs to be updated is OK
      %
      SetLogText(strvcat('   STATUS: OK',[' * Checking ' GetName(DSGEModel.InitializeParameterFile) '...']));
      %
      % we need to run the file DSGEModel.InitializeParameterFile first
      %
      try;
         firstInitializePos = length(fieldnames(ModelParameters))+1;
         eval(['ModelParameters = ' GetName(DSGEModel.InitializeParameterFile) '(ModelParameters);']);
         lastInitializePos = length(fieldnames(ModelParameters));
      catch;
         ErrorStr = ['Error found in "' GetName(DSGEModel.InitializeParameterFile) '.m". Message caught is: ' lasterr];
         ErrorFile = DSGEModel.InitializeParameterFile;
      end;
      if (FileExist(DSGEModel.UpdateParameterFile)==1)&(isempty(ErrorStr)==1);
         SetLogText(strvcat('   STATUS: OK',[' * Checking ' GetName(DSGEModel.UpdateParameterFile) '...']));
         try;
            firstUpdatePos = length(fieldnames(ModelParameters))+1;
            eval(['ModelParameters = ' GetName(DSGEModel.UpdateParameterFile) '(ModelParameters);']);
            lastUpdatePos = length(fieldnames(ModelParameters));
         catch;
            ErrorStr = ['Error found in "' GetName(DSGEModel.UpdateParameterFile) '.m". Message caught is: ' lasterr];
            ErrorFile = DSGEModel.UpdateParameterFile;
         end;
      end;
   else;
      if FileExist(DSGEModel.UpdateParameterFile)==1;
         SetLogText(strvcat('   STATUS: OK',[' * Checking ' GetName(DSGEModel.UpdateParameterFile) '...']));
         try;
            firstUpdatePos = length(fieldnames(ModelParameters))+1;
            eval(['ModelParameters = ' GetName(DSGEModel.UpdateParameterFile) '(ModelParameters);']);
            lastUpdatePos = length(fieldnames(ModelParameters));
         catch;
            ErrorStr = ['Error found in "' GetName(DSGEModel.UpdateParameterFile) '.m". Message caught is: ' lasterr];
            ErrorFile = DSGEModel.UpdateParameterFile;
         end;
      end;
      if (FileExist(DSGEModel.InitializeParameterFile)==1)&(isempty(ErrorStr)==1);
         SetLogText(strvcat('   STATUS: OK',[' * Checking ' GetName(DSGEModel.InitializeParameterFile) '...']));
         try;
            firstInitializePos = length(fieldnames(ModelParameters))+1;
            eval(['ModelParameters = ' GetName(DSGEModel.InitializeParameterFile) '(ModelParameters);']);
            lastInitializePos = length(fieldnames(ModelParameters));
         catch;
            ErrorStr = ['Error found in "' GetName(DSGEModel.InitializeParameterFile) '.m". Message caught is: ' lasterr];
            ErrorFile = DSGEModel.InitializeParameterFile;
         end;
      end;
   end;
end;
%
if isempty(ErrorStr)==0;
   SetLogText(strvcat(['   STATUS: Bad input file ' GetName(ErrorFile)],' '));
   if isempty(CurrINI.Sequence)==1;
      About(ErrorStr,'error',['Error - ' GetFilename(ErrorFile)],CurrINI.scrsz(4)-50,500,CurrINI);
      drawnow;
   end;
   return;
end;
%
% we can go on with checking if the initial values permit a unique solution of the DSGE model
%
SetLogText(strvcat('   STATUS: OK',' * Checking if the DSGE model has a unique convergent','   solution for initial parameter values...'));
%
% we can now run the AiM solver:
%
% To measure the solution time more accurately we call it twice, with the first call only being
% performed for checking if the model has a unique convergent solution at the initial values.
%
try;
   if DSGEModel.ModelSolver==1;
      [mcode,MaxAbsError,BMatrix,SMatrix] = AiMSolver(ModelParameters,AIMData.neq,AIMData.nlag,AIMData.nlead,DSGEModel.AIMTolerance);
   elseif DSGEModel.ModelSolver==2;
      [F,B0,mcode,MaxAbsError,ModelEigenvalues] = KleinSolver(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,CurrINI.OrderQZ);
   elseif DSGEModel.ModelSolver==3;
      [F,B0,mcode,MaxAbsError,ModelEigenvalues] = SimsSolver(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,CurrINI.OrderQZ);
   end;
catch;
   if DSGEModel.ModelSolver==1;
      ErrorStr = ['There was an error running AiM. YADA received the following message: ' lasterr];
   elseif DSGEModel.ModelSolver==2;
      ErrorStr = ['There was an error running solab (Klein solver). YADA received the following message: ' lasterr];
   elseif DSGEModel.ModelSolver==3;
      ErrorStr = ['There was an error running gensys (Sims solver). YADA received the following message: ' lasterr];
   end;
   if isempty(CurrINI.Sequence)==1;
      About(ErrorStr,'error','AiM Error',CurrINI.scrsz(4)-50,500,CurrINI);
      drawnow;
   end;
   return;
end;
%
if DSGEModel.ModelSolver==1;
   SolverStr = 'AiM';
elseif DSGEModel.ModelSolver==2;
   SolverStr = 'Klein';
elseif DSGEModel.ModelSolver==3;
   SolverStr = 'Gensys';
end;
%
% check if we have a unique convergent solution at the initial parameter values.
%
if mcode~=1;
   %
   % There doesn't exist a unique convergent solution for the initial parameter values.
   %
   if mcode==2;
      mcodeStr = 'Roots are not correctly computed by real_schur.'
   elseif mcode==3;
      mcodeStr = 'Too many big roots.';
   elseif mcode==35;
      mcodeStr = 'Too many big roots, and q(:,right) is singular.';
   elseif mcode==4;
      mcodeStr = 'Too few big roots.';
   elseif mcode==45;
      mcodeStr = 'Too few big roots, and q(:,right) is singular.';
   elseif mcode==5;
      mcodeStr = 'q(:,right) is singular.';
   elseif mcode==61;
      mcodeStr = 'Too many exact shiftrights.';
   elseif mcode==62;
      mcodeStr = 'Too many numeric shiftrights.';
   elseif mcode==7;
      mcodeStr = 'Infinite or NaN values detected.';
   elseif mcode==8;
      mcodeStr = 'The function "compute_aim_matrices" returns complex numbers.';
   elseif mcode==-1;
      mcodeStr = 'No stable solution.';
   elseif mcode==-2;
      mcodeStr = 'Too many large generalized eigenvalues.';
   elseif mcode==-3;
      mcodeStr = 'Too few large generalized eigenvalues.';
   else;
      mcodeStr = 'Return code not properly specified.';
   end;
   SetLogText(strvcat(['   STATUS: ' mcodeStr],' '));
   if isempty(CurrINI.Sequence)==1;
      txt = ['The ' SolverStr ' solver provided the return code: ' num2str(mcode) ', i.e., "' mcodeStr '" You have to change your initial parameter values.'];
      About(txt,'information','YADA - DSGE Model Problem',200,500,CurrINI);
      drawnow;
   end;
end;
tic;
if DSGEModel.ModelSolver==1;
   [mcode,MaxAbsError,BMatrix,SMatrix] = AiMSolver(ModelParameters,AIMData.neq,AIMData.nlag,AIMData.nlead,DSGEModel.AIMTolerance);
   %
   % check the state equations
   %
   [F,B0] = AiMtoStateSpace(BMatrix,SMatrix,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions);
elseif DSGEModel.ModelSolver==2;
   [F,B0,mcode] = KleinSolver(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,CurrINI.OrderQZ);
elseif DSGEModel.ModelSolver==3;
   [F,B0,mcode] = SimsSolver(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,CurrINI.OrderQZ);
end;
SolveTime = toc;
%
if (max(abs(eig(F)))>=1)|(IsAlmostEqual(max(abs(eig(F))),1)==1);
   SetLogText(strvcat('   STATUS: Non-stationary state equation',' '));
   if isempty(CurrINI.Sequence)==1;
      txt = ['The largest eigenvalue of the state equation is ' num2str(max(abs(eig(F)))) '. The state equation is therefore non-stationary. You need to change your initial parameter values.'];
      About(txt,'information','YADA - DSGE Model Problem',200,500,CurrINI);
      drawnow;
   end;
   return;
end;
%
% add the values of F and B0 to the ModelParameters.YADA field
%
[B0Alt,KeepVar] = RemoveRedundantColumns(B0);
ModelParameters.YADA.F = F;
ModelParameters.YADA.B0 = B0Alt;
%
% check the measurement equation file for internal validity
%
SetLogText(strvcat('   STATUS: OK',['   DSGE model solver:      ' SolverStr],['   Maximum absolute error: ' num2str(MaxAbsError)],['   Solution time:          ' num2str(SolveTime) ' seconds'],[' * Checking ' GetName(DSGEModel.MeasurementEquationFile) '...']));
try;
   eval(['[A,H,R] = ' GetName(DSGEModel.MeasurementEquationFile) '(ModelParameters,AIMData.endog(DSGEModel.StateVariablePositions,:),DSGEModel.VariableNames,DSGEModel.XVariableNames,DSGEModel.n,size(F,1),DSGEModel.k);']);
catch;
   ErrorStr = ['Error found in "' GetName(DSGEModel.MeasurementEquationFile) '.m". Message caught is: ' lasterr];
   ErrorFile = DSGEModel.MeasurementEquationFile;
   SetLogText(strvcat(['   STATUS: Bad input file ' GetName(ErrorFile)],' '));
   if isempty(CurrINI.Sequence)==1;
      About(ErrorStr,'error',['Error - ' GetFilename(ErrorFile)],CurrINI.scrsz(4)-50,500,CurrINI);
      drawnow;
   end;
   return;
end;
%
% checking the system prior file if it exists
%
if IsSystemPrior==1;
   SetLogText(strvcat('   STATUS: OK',' * Checking if the system prior can be calculated...'));
   try;
      eval(['lnSP = ' GetName(DSGEModel.SystemPriorFile) '(theta,thetaPositions,ModelParameters,AIMData,DSGEModel,CurrINI.OrderQZ);']);
   catch;
      ErrorStr  = ['Unable to calculate the log of the system prior. Message caught is: ' lasterr];
      ErrorFile = DSGEModel.SystemPriorFile;
      SetLogText(strvcat(['   STATUS: Bad input file ' GetName(ErrorFile)],' '));
      if isempty(CurrINI.Sequence)==1;
         About(ErrorStr,'error',['Error - ' GetFilename(ErrorFile)],CurrINI.scrsz(4)-50,500,CurrINI);
         drawnow;
      end;
      return;
   end;
   SetLogText(strvcat('   STATUS: OK',' * Checking if the system prior mode has been calculated...'));
   SystemPriorModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\SystemPriorMode-' DSGEModel.NameOfModel '.mat']);
   if FileExist(SystemPriorModeFile)==1;
      SystemPriorModeData = load(SystemPriorModeFile);
      %
      % we only need the Laplace approximation of the log marginal likelihood of the system prior
      %
      LogMargLikeSystemLaplace = SystemPriorModeData.LogMargLikeSystemLaplace;
      SetLogText('   STATUS: OK');
   else;
      %
      % we need to first compute the system prior mode
      %
      [SystemPriorStatus,LogMargLikeSystemLaplace] = ComputeSystemPriorMode(theta,thetaIndex,thetaDist,LowerBound,ModelParameters,thetaPositions,PriorDist,ParameterNames,UniformBounds,[firstUpdatePos lastUpdatePos firstInitializePos lastInitializePos],AIMData,controls,DSGEModel,CurrINI);
      if SystemPriorStatus==0;
         SetLogText('Prior mode estimation has been aborted.');
         return;
      end;
   end;
else;
   LogMargLikeSystemLaplace = 0;
end;
%
%
SetLogText(strvcat('   STATUS: OK',' * Checking if the DSGE-VAR prior can be calculated...'));
%
% The model has a unique solution for the initial values
%
MaxIterStr = get(controls.optimize.maximumiterations,'String');
MaxIterStr = StringTrim(strrep(MaxIterStr(DSGEModel.OptMaxIterationsValue,:),',',''));
MaxIter = str2num(MaxIterStr);
%
ToleranceStr = get(controls.optimize.tolerance,'String');
ToleranceStr = StringTrim(ToleranceStr(DSGEModel.OptToleranceValue,:));
ToleranceValue = str2num(ToleranceStr);
%
StepLengthStr = get(controls.optimize.steplength,'String');
StepLength = str2num(StringTrim(StepLengthStr(DSGEModel.StepLengthHessian,:)));
%
NumGridPointsStr = get(controls.optimize.numbergridpoints,'String');
NumGridPoints = str2num(StringTrim(NumGridPointsStr(DSGEModel.NumberOfGridPoints,:)));
%
% Collect additional parameters in ParameterNames.additional
%
ParameterNames = ReadAdditionalParameterNames(ParameterNames,ModelParameters,OriginalModelParameters,[firstUpdatePos lastUpdatePos firstInitializePos lastInitializePos]);
%
% set up vectors with calibrated parameters and additional parameters. These are only used for printing to the output file
%
[thetaCalibrated,thetaAdditionalInitial] = ReadAdditionalParameterValues(ModelParameters,ParameterNames);
%
% initialize dimensions
%
p = DSGEModel.BVARLags;
k = DSGEModel.k;
n = DSGEModel.n;
npk = (n*p)+k;
%
% create data matrices
%
[FirstPeriod,LastPeriod] = CreateSubSample(DSGEModel);
%
% Determine the first observation to use when computing log-likelihood
%
KalmanFirstObservation = DSGEModel.KalmanFirstObservation;
%
% Check if we should adjust this based on p, the lag order
%
if FirstPeriod+KalmanFirstObservation-1<p+1;
   KalmanFirstObservation = p+2-FirstPeriod;
end;
%
% actual sample that we will use
%
[NewStartYear,NewStartPeriod] = AdjustSampleStart(DSGEModel.SubBeginYear,DSGEModel.SubBeginPeriod,DSGEModel.DataFrequency,KalmanFirstObservation-1);
DSGEModel.FirstPeriod = FirstPeriod;
DSGEModel.LastPeriod = LastPeriod;
DSGEModel.KalmanFirstObservation = KalmanFirstObservation;
%
% Setup all the basic data matrices
%
y = DSGEModel.Y(:,FirstPeriod+KalmanFirstObservation-1:LastPeriod);
if k>0
   x = DSGEModel.X(:,FirstPeriod+KalmanFirstObservation-1:LastPeriod);
   X = DSGEModel.X(:,FirstPeriod+KalmanFirstObservation-(p+1):LastPeriod);
else;
   x = [];
   X = [];
end;
T = size(y,2);
Y = zeros((n*p),T);
for i=1:p;
   Y(1+(n*(i-1)):n*i,:) = DSGEModel.Y(:,FirstPeriod+KalmanFirstObservation-1-i:LastPeriod-i);
end;
Y = [x;Y];
HSample = [FirstPeriod+KalmanFirstObservation-1 LastPeriod];
%
% compute the lower bound for the lambda hyperparameter
%
LowBoundLambda = (n*(p+1)+k)/T;
%
% keep only the lambda values that are at least as large as the lower bound
%
Lambda = DSGEModel.Lambda(DSGEModel.Lambda>=LowBoundLambda);
if length(Lambda)<=1;
   Lambda = [LowBoundLambda Inf];
elseif Lambda(1)>LowBoundLambda;
   Lambda = [LowBoundLambda Lambda];
end;
if max(isinf(Lambda))==0;
   Lambda = [Lambda Inf];
end;
%
% transform the entries in Lambda such that T*Lambda(i) is an
% integer for all lambda(i) in Lambda.
%
lambda = ceil(Lambda*T)/T;
%
% compute the deterministic product moments
%
if k>0;
   DetProductMoments = CalcDeterministicProductMoments(x,X);
else;
   DetProductMoments = [];
end;
%
% now we attempt to compute the moments of the prior for the DSGE-VAR parameters
% conditional on theta.
%
[Gammayy,GammaYY,GammayY,PriorStatus] = DSGEVARPrior(A,H,R,F,B0,DetProductMoments,p,DSGEModel.UseDoublingAlgorithm,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,HSample);
if PriorStatus==0;
   txt = 'The sample average of the implied non-central population moment matrix GammaYY(theta) is not invertible at the initial parameter values. This matrix is required for setting up the DSGE-VAR prior. YADA has therefore aborted from joint posterior mode estimation of the DSGE-VAR models.';
   SetLogText(strvcat('   STATUS: Prior moment matrix GammaYY is not invertible.',' '));
   if isempty(CurrINI.Sequence)==1;
      About(txt,'information','YADA - DSGE Model Problem',300,500,CurrINI);
      drawnow;
   end;
   return;
end;
%
% compute the DSGE-VAR parameters that are common for all priors
%
PhiPriorMode = GammayY/GammaYY;
SigmaPriorEpsilon = Gammayy-(PhiPriorMode*GammayY');
A0PriorMode = zeros(n,n,length(lambda));
IdentifyPriorModeStatus = zeros(1,length(lambda));
%
% let the user select lambda values
%
if isempty(JointLambda)==1;
   SetLogText(strvcat('   STATUS: OK',' * Determining which lambda values to use...'));
   LambdaNames = '';
   for i=1:length(lambda);
      LambdaNames = strvcat(LambdaNames,['lambda=' num2str(lambda(i),'%0.4f')]);
   end;
   if isempty(DSGEModel.JointLambda)==0;
      if (length(DSGEModel.JointLambda)<=length(lambda))&(max(DSGEModel.JointLambda)<=length(lambda));
         positions = DSGEModel.JointLambda;
      else;
         positions = (1:length(lambda));
      end;
   else;
      positions = (1:length(lambda));
   end;
   [PosAction,positions] = SelectCondVarShockDLG('init','Joint Posterior Mode','Lambda Values For',LambdaNames,positions,CurrINI);
   if strcmp(lower(PosAction),'ok')==0;
      SetLogText(strvcat(' ','Estimation aborted.'));
      return;
   end;
   lambda = lambda(positions);
   JointLambda = positions;
else;
   lambda = lambda(JointLambda);
end;
%
% determine lower and upper bounds for the parameters
%
ParameterBounds = CreateBounds(thetaDist,LowerBound,UniformBounds);
ParameterBounds = ParameterBounds(:,1:2);
%
% check the parameter target type
%
if IsOdd(DSGEModel.MaximizeAlgorithmValue)==1;
   OptParameters = 'Transformed';
else;
   OptParameters = 'Original';
end;
%
SetLogText(strvcat(['   STATUS: User selected ' int2str(length(lambda)) ' lambda values'],' * Checking if the log posterior can be calculated...'));
%
% compute sample product moments
%
GammaHatyy = (1/T)*y*y';
GammaHatyY = (1/T)*y*Y';
GammaHatYY = (1/T)*Y*Y';
%
LogPosteriorPhiIV = zeros(length(lambda),1);
for i=1:length(lambda);
   %
   % setup some constants
   %
   lambdai = lambda(i);
   if isinf(lambdai)==0;
      logCPC = DSGEVARLogConcLikelihoodConstant(lambdai,T,n,npk);
   else;
      logCPC = [];
   end;
   ErrorStr = '';
   try;
      tic;
      if IsOdd(DSGEModel.MaximizeAlgorithmValue)==1;
         logPost = logConcPosteriorPhiDSGEVAR(phi,thetaIndex,UniformBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,DSGEModel,AIMData,lambdai,T,n,p,npk,GammaHatyy,GammaHatyY,GammaHatYY,DetProductMoments,HSample,logCPC,CurrINI.OrderQZ,1);
      else;
         logPost = logConcPosteriorThetaDSGEVAR(theta,thetaIndex,ParameterBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,DSGEModel,AIMData,lambdai,T,n,p,npk,GammaHatyy,GammaHatyY,GammaHatYY,DetProductMoments,HSample,logCPC,CurrINI.OrderQZ,1);
         if logPost~=1000000;
            logPost = logPost-logJacobian(phi,thetaIndex,UniformBounds);
         end;
      end;
      LogPostTime = toc;
      LogPosteriorPhiIV(i) = -logPost;
   catch;
      ErrorStr = ['Unable to calculate the log posterior of the DSGE-VAR at initial values for lambda=' num2str(lambdai,'%0.4f') '. Message caught is: ' lasterr];
   end;
      if ((exist('logPost')==0)|(logPost==1000000))&(isempty(ErrorStr)==1);
      %
      % report error
      %
      SetLogText(strvcat('   STATUS: Unable to compute log posterior at initial values',['           for lambda=' num2str(lambdai,'%0.4f')],' '));
      if isempty(CurrINI.Sequence)==1;
         txt = ['Unable to compute the log posterior of the DSGE-VAR at the initial values for lambda=' num2str(lambdai,'%0.4f') '. You probably need to change your initial parameter values.'];
         About(txt,'information','YADA - DSGE Model Problem',300,500,CurrINI);
         drawnow;
      end;
      return;
   elseif isempty(ErrorStr)==0;
      SetLogText(strvcat('   STATUS: Unable to compute log posterior at initial values',['           for lambda=' num2str(lambdai,'%0.4f')],' '));
      if isempty(CurrINI.Sequence)==1;
         About(ErrorStr,'error','Error - Log Posterior',200,500,CurrINI);
         drawnow;
      end;
      return;
   end;
   %
   % check if we can compute the gradient
   %
   if IsOdd(DSGEModel.MaximizeAlgorithmValue)==1;
      [g,badg] = YADAnumgrad('logConcPosteriorPhiDSGEVAR',phi,thetaIndex,UniformBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,DSGEModel,AIMData,lambdai,T,n,p,npk,GammaHatyy,GammaHatyY,GammaHatYY,DetProductMoments,HSample,logCPC,CurrINI.OrderQZ,1);
   else;
      [g,badg] = YADAnumgrad('logConcPosteriorThetaDSGEVAR',theta,thetaIndex,ParameterBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,DSGEModel,AIMData,lambdai,T,n,p,npk,GammaHatyy,GammaHatyY,GammaHatYY,DetProductMoments,HSample,logCPC,CurrINI.OrderQZ,1);
   end;
   if badg~=0;
      SetLogText(strvcat('   STATUS: Bad gradient of log posterior at initial values',['           for lambda=' num2str(lambdai,'%0.4f')],' '));
      [badpar,maxpos] = min(abs(g));
      %
      if badpar>0;
         ValueStr = num2str(badpar);
      else;
         ValueStr = 'NaN';
      end;
      %
      ParamStr = ['"' StringTrim(ParameterNames.estimated(maxpos(1),:)) '"'];
      AddParamStr = ' ';
      if length(maxpos)>=2;
         AddParamStr = 's ';
         for j=2:length(maxpos);
            ParamStr = [ParamStr ', "' StringTrim(ParameterNames.estimated(maxpos(j),:)) '"'];
         end;
      end;
      %
      if isempty(CurrINI.Sequence)==1;
         txt = ['Unable to compute a meaningful numerical gradient of the log posterior of the DSGE-VAR at the initial values for lambda=' num2str(lambdai,'%0.4f') '. The largest absolute value or an improper value (NaN or Inf) of the gradient is found for the parameter' AddParamStr ParamStr ', with value of ' ValueStr '. You probably need to change your initial parameter values.'];
         About(txt,'information','YADA - DSGE Model Problem',200,500,CurrINI);
         drawnow;
      else;
         SetLogText(strvcat('   STATUS: Unable to compute numerical gradient of the log-posterior',['           at initial values for lambda=' num2str(lambdai,'%0.4f')]));
      end;
      return;
   end;
   %
   % compute the prior mode values
   %
   if isinf(lambdai)==0;
      SigmaPriorModeEpsilon = ((lambdai*T)/((lambdai*T)+n+1))*SigmaPriorEpsilon;
   else;
      SigmaPriorModeEpsilon = SigmaPriorEpsilon;
   end;
   [A0Prior,IdentifyStatus] = DSGEVARIdentifyShocks(SigmaPriorModeEpsilon,H,B0,DSGEModel.DSGEVARShocks,HSample);
   if IdentifyStatus==1;
      IdentifyPriorModeStatus(i) = 1;
      A0PriorMode(:,:,i) = A0Prior;
   end;
   SetLogText(strvcat(['   STATUS: OK for lambda=' num2str(lambdai,'%0.4f')],['   Computation time:       ' num2str(LogPostTime) ' seconds']));
end;
%
% Starting the optimization
%
ReturnCodeText = '';
ErrorStr = '';
LogJacobianIV = logJacobian(phi,thetaIndex,UniformBounds);
LogPriorIV = logPriorDSGE(PhiToTheta(phi,thetaIndex,UniformBounds,LowerBound),thetaDist,PriorDist,LowerBound);
if IsSystemPrior==1;
   eval(['LogSPIV = ' GetName(DSGEModel.SystemPriorFile) '(theta,thetaPositions,ModelParameters,AIMData,DSGEModel,CurrINI.OrderQZ);']);
   SystemPriorStr = ['   - Log system prior value: ' num2str(LogSPIV,'%0.4f')];
else;
   LogSPIV = 0;
   SystemPriorStr = '';
end;
LogPosteriorThetaIV = LogPosteriorPhiIV-LogJacobianIV;
LogLikelihoodIV = LogPosteriorThetaIV-LogPriorIV-LogSPIV;
NumParam = length(theta);
TotalParam = (n*npk)+((n*(n+1))/2)+NumParam;
%
% compute the initial inverse Hessian if csminwel is used
%
if (DSGEModel.MaximizeAlgorithmValue==1)|(DSGEModel.MaximizeAlgorithmValue==2);
   if DSGEModel.InitializeHessian==1;
      InitialInvHessMat = 10*eye(NumParam);
   elseif DSGEModel.InitializeHessian==2;
      InitialInvHessMat = 1*eye(NumParam);
   elseif DSGEModel.InitializeHessian==3;
      InitialInvHessMat = 10^(-1)*eye(NumParam);
   elseif DSGEModel.InitializeHessian==4;
      InitialInvHessMat = 10^(-2)*eye(NumParam);
   elseif DSGEModel.InitializeHessian==5;
      InitialInvHessMat = 10^(-3)*eye(NumParam);
   elseif DSGEModel.InitializeHessian==6;
      InitialInvHessMat = 10^(-4)*eye(NumParam);
   elseif DSGEModel.InitializeHessian==7;
      InitialInvHessMat = 10^(-5)*eye(NumParam);
   else;
      %
      % sample from the prior
      %
      InitialInvHessMat = PriorHessianMatrix(thetaDist,thetaIndex,PriorDist,UniformBounds,LowerBound,DSGEModel.RandomNumberValue,IsOdd(DSGEModel.MaximizeAlgorithmValue));
   end;
   OptAlgorithm = 'csminwel';
elseif (DSGEModel.MaximizeAlgorithmValue==3)|(DSGEModel.MaximizeAlgorithmValue==4);
   %
   % use Marco Ratto's newrat
   %
   OptAlgorithm = 'newrat';
   DSGEModel.Data.y = y;
   DSGEModel.Data.Y = Y;
   %
   % compute the initial inverse Hessian
   %
   if DSGEModel.InitializeHessian==1;
      HessianMat = 10^(-1)*eye(NumParam);
   elseif DSGEModel.InitializeHessian==2;
      HessianMat = eye(NumParam);
   elseif DSGEModel.InitializeHessian==3;
      HessianMat = 10*eye(NumParam);
   elseif DSGEModel.InitializeHessian==4;
      HessianMat = 10^2*eye(NumParam);
   elseif DSGEModel.InitializeHessian==5;
      HessianMat = 10^3*eye(NumParam);
   elseif DSGEModel.InitializeHessian==6;
      HessianMat = 10^4*eye(NumParam);
   elseif DSGEModel.InitializeHessian==7;
      HessianMat = 10^5*eye(NumParam);
   else;
      %
      % sample from the prior
      %
      HessianMat = eye(NumParam)/PriorHessianMatrix(thetaDist,thetaIndex,PriorDist,UniformBounds,LowerBound,DSGEModel.RandomNumberValue,IsOdd(DSGEModel.MaximizeAlgorithmValue));
   end;
   if DSGEModel.MaximizeAlgorithmValue==3;
      ParamLUB = [-Inf*ones(NumParam,1) Inf*ones(NumParam,1)];
   else;
      ParamLUB = ParameterBounds;
   end;
   PriorStd = sqrt(diag(PriorHessianMatrix(thetaDist,thetaIndex,PriorDist,UniformBounds,LowerBound,DSGEModel.RandomNumberValue,IsOdd(DSGEModel.MaximizeAlgorithmValue))));
elseif (DSGEModel.MaximizeAlgorithmValue==5)|(DSGEModel.MaximizeAlgorithmValue==6);
   %
   % use Dynare's gmhmaxlik
   %
   OptAlgorithm = 'gmhmaxlik';
   %
   % determine the initial scale factor
   %
   ScaleStr = get(controls.posterior.scalefactor,'String');
   c = str2num(StringTrim(ScaleStr(DSGEModel.MHScaleFactor,:)));
   %
   % determine the number of draws
   %
   DrawsStr = get(controls.posterior.draws,'String');
   NumPostDraws = str2num(StringTrim(strrep(DrawsStr(DSGEModel.PosteriorDrawsValue,:),',','')));
   BurninStr = get(controls.posterior.burnin,'String');
   NumBurnin = str2num(strrep(StringTrim(BurninStr(DSGEModel.BurnInValue,:)),',',''));
   NumDraws = NumPostDraws-NumBurnin;
   %
   % initial covariance matrix
   %
   [InitialInvHessian,ParamMean] = PriorHessianMatrix(thetaDist,thetaIndex,PriorDist,UniformBounds,LowerBound,DSGEModel.RandomNumberValue,IsOdd(DSGEModel.MaximizeAlgorithmValue));;
   %
   % parameter bounds
   %
   if DSGEModel.MaximizeAlgorithmValue==5;
      ParamLUB = [-Inf*ones(NumParam,1) Inf*ones(NumParam,1)];
   else;
      ParamLUB = ParameterBounds;
   end;
elseif (DSGEModel.MaximizeAlgorithmValue==7)|(DSGEModel.MaximizeAlgorithmValue==8);
   %
   % set the options
   %
   Options = optimset('MaxIter',MaxIter,'MaxFunEvals',100*MaxIter,'Diagnostics','off','LargeScale','off','TolFun',ToleranceValue,'TolX',ToleranceValue);
   %
   OptAlgorithm = 'fminunc';
end;
%
% make sure that there is no AiMErrorData file lurking around in the "tmp" directory
%
if FileExist(FixFilePath([pwd '\tmp\AiMErrorData.mat']))==1;
   delete(FixFilePath([pwd '\tmp\AiMErrorData.mat']));
end;
%
% determine string with 1's and 0's for the selected shocks
%
SelectedShockStr = ismember((1:length(DSGEModel.StateShockPositions)),DSGEModel.DSGEVARShocks);
SelectedShockStr = strrep(num2str(SelectedShockStr),' ','');
%
% check if results already exist on disk and if so ask the user if they should be used
%
AbortEstimation = '';
FiniteQuery = '';
NumIter = 0;
ContinueEstimation = 1;
ExitFlag = [];
ReturnCode = [];
ModeFileDir = FixFilePath([DSGEModel.OutputDirectory '\mode']);
ResultsExist = zeros(1,length(lambda));
UseOldResults = zeros(1,length(lambda));
for i=1:length(lambda);
   ModeFile = FixFilePath([ModeFileDir '\JointPosteriorMode-DSGEVAR-' num2str(lambda(i),'%0.4f') '-' int2str(p) '-' SelectedShockStr '-' DSGEModel.NameOfModel '.mat']);
   if FileExist(ModeFile)==1;
      %
      % try to load the data
      %
      GoodData = 1;
      try;
         ModeData = load(ModeFile);
      catch;
         GoodData = 0;
      end;
      if (GoodData==1)&(isfield(ModeData,'LaplaceMargLikeLambda')==1);
         ResultsExist(i) = ModeData.LaplaceMargLikeLambda;
      end;
   end;
end;
if max(abs(ResultsExist))>0;
   %
   % ask if the user wishes to load previous results from disk
   %
   if isempty(CurrINI.Sequence)==1;
      txt = strvcat('YADA has located joint posterior mode results on disk for some (or all) of the DSGE-VAR models you want to estimate. Would you like to use these previous results?',' ','NOTE: If you answer No then YADA will reestmate all DSGE-VAR models.');
      answer = Query(txt,'question',200,'Joint Posterior Mode Estimation',500,'No',CurrINI);
   else;
      answer = CurrINI.Sequence.QueryAnswers{1};
   end;
   if strcmp(lower(answer),'yes')==1;
      UseOldResults = (abs(ResultsExist)>0);
      GraphDirectory = FixFilePath([DSGEModel.OutputDirectory '\graphs']);
      stat = MakeDir(DSGEModel.OutputDirectory,'graphs');
   end;
end;
%
% loop over the different lambda values
%
LaplaceMargLike = zeros(1,length(lambda));
FirstEstimate = 1;
for i=1:length(lambda);
   if UseOldResults(i)==0;
      %
      % setup some constants
      %
      lambdai = lambda(i);
      if isinf(lambdai)==0;
         logCPC = DSGEVARLogConcLikelihoodConstant(lambdai,T,n,npk);
      else;
         logCPC = [];
      end;
      EstimationStatus = 0;
      ContinueEstimation = 0;
      if FirstEstimate==1;
         if (DSGEModel.MaximizeAlgorithmValue==1)|(DSGEModel.MaximizeAlgorithmValue==2);
            SetLogText(strvcat(' ','Optimization routine:',sprintf('   Sims'' csminwel for %s parameters...',lower(OptParameters))));
         elseif (DSGEModel.MaximizeAlgorithmValue==3)|(DSGEModel.MaximizeAlgorithmValue==4);
            SetLogText(strvcat(' ','Optimization routine:',sprintf('   Ratto''s newrat for %s parameters...',lower(OptParameters))));
         elseif (DSGEModel.MaximizeAlgorithmValue==5)|(DSGEModel.MaximizeAlgorithmValue==6);
            SetLogText(strvcat(' ','Optimization routine:',sprintf('   Dynare''s gmhmaxlik for %s parameters...',lower(OptParameters))));
         elseif (DSGEModel.MaximizeAlgorithmValue==7)|(DSGEModel.MaximizeAlgorithmValue==8);
            SetLogText(strvcat(' ','Optimization routine:',sprintf('   Matlab''s fminunc for %s parameters...',lower(OptParameters))));
         end;
         if (DSGEModel.MaximizeAlgorithmValue==5)|(DSGEModel.MaximizeAlgorithmValue==6);
            SetLogText(strvcat([' * Number of parameters to estimate: ' num2str(NumParam)],[' * Total number of parameters:       ' num2str(size(ParameterNames.all,1),'%0.0f')],[' * Number of Monte Carlo draws:      ' int2str(NumDraws)],[' * Effective sample size:            ' num2str(T,'%0.0f')],[' * Estimation sample:                ' NewStartYear ':' NewStartPeriod ' - ' DSGEModel.SubEndYear ':' DSGEModel.SubEndPeriod],[' * Number of observed variables:     ' int2str(n)],[' * Number of lags:                   ' int2str(p)],[' * Number of exogenous variables:    ' int2str(k)]));
         else;
            SetLogText(strvcat([' * Number of parameters to estimate: ' num2str(NumParam)],[' * Total number of parameters:       ' num2str(size(ParameterNames.all,1),'%0.0f')],[' * Tolerance level:                  ' ToleranceStr],[' * Maximum number of iterations:     ' MaxIterStr],[' * Effective sample size:            ' num2str(T,'%0.0f')],[' * Estimation sample:                ' NewStartYear ':' NewStartPeriod ' - ' DSGEModel.SubEndYear ':' DSGEModel.SubEndPeriod],[' * Number of observed variables:     ' int2str(n)],[' * Number of lags:                   ' int2str(p)],[' * Number of exogenous variables:    ' int2str(k)]));
         end;
         FirstEstimate = 0;
         SetLogText(strvcat([' * Time:                             ' StringTrim(datestr(now,14))],' ',['   lambda:                 ' num2str(lambdai,'%0.4f')],'   Initial parameter values of DSGE yield',['   - Log posterior value:    ' num2str(LogPosteriorPhiIV(i),'%0.4f')],['   - Log prior value:        ' num2str(LogPriorIV,'%0.4f')],['   - Log Jacobian value:     ' num2str(LogJacobianIV,'%0.4f')],SystemPriorStr,['   - Log likelihood value:   ' num2str(LogLikelihoodIV(i),'%0.4f')]));
      else;
         SetLogText(strvcat(' ','*************************************************************************',' ',[' * Time:                             ' StringTrim(datestr(now,14))],' ',['   lambda:                 ' num2str(lambdai,'%0.4f')],'   Initial parameter values of DSGE yield',['   - Log posterior value:    ' num2str(LogPosteriorPhiIV(i),'%0.4f')],['   - Log prior value:        ' num2str(LogPriorIV,'%0.4f')],['   - Log Jacobian value:     ' num2str(LogJacobianIV,'%0.4f')],SystemPriorStr,['   - Log likelihood value:   ' num2str(LogLikelihoodIV(i),'%0.4f')]));
      end;
      %
      if (DSGEModel.MaximizeAlgorithmValue==1)|(DSGEModel.MaximizeAlgorithmValue==2);
         %
         % We use Sims' csminwel routine with transformed parameters.
         %
         try;
            tic;
            if DSGEModel.MaximizeAlgorithmValue==1;
               [fh,phiMode,gh,InverseHessian,NumIter,fcount,FuncDiff,AbortEstimation,OptResults,ScreenMessages,phiEstimates,ReturnCode] = YADAcsminwel('logConcPosteriorPhiDSGEVAR',phi,InitialInvHessMat,[],ToleranceValue,MaxIter,DSGEModel.ShowProgress,DSGEModel.ShowProgressClock,'Joint Posterior Mode Estimation - csminwel',CurrINI, ...
                  thetaIndex,UniformBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,DSGEModel,AIMData,lambdai,T,n,p,npk,GammaHatyy,GammaHatyY,GammaHatYY,DetProductMoments,HSample,logCPC,CurrINI.OrderQZ,1);
            else;
               [fh,thetaMode,gh,InverseHessian,NumIter,fcount,FuncDiff,AbortEstimation,OptResults,ScreenMessages,thetaEstimates,ReturnCode] = YADAcsminwel('logConcPosteriorThetaDSGEVAR',theta,InitialInvHessMat,[],ToleranceValue,MaxIter,DSGEModel.ShowProgress,DSGEModel.ShowProgressClock,'Joint Posterior Mode Estimation - csminwel',CurrINI, ...
                  thetaIndex,ParameterBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,DSGEModel,AIMData,lambdai,T,n,p,npk,GammaHatyy,GammaHatyY,GammaHatYY,DetProductMoments,HSample,logCPC,CurrINI.OrderQZ,1);
               %
               % make sure that the outputs are transformed into phi-space
               %
               phiMode = ThetaToPhi(thetaMode,thetaIndex,UniformBounds,LowerBound);
               phiEstimates = zeros(size(thetaEstimates));
               for j=1:size(thetaEstimates,2);
                  phiEstimates(:,j) = ThetaToPhi(thetaEstimates(:,j),thetaIndex,UniformBounds,LowerBound);;
               end;
               ThetaPhiPartial = PartialThetaPartialPhi(phiMode,thetaIndex,UniformBounds);
               InverseHessian = (ThetaPhiPartial\InverseHessian)/ThetaPhiPartial;
            end;
            MaxTime = toc;
            GradientVector = abs(gh);
         catch;
            if DSGEModel.ShowProgress==1;
               WaitHandle = findobj('Type','figure','Tag','TMWProgressDLG');
               if IsHandle(WaitHandle)==1;
                  set(WaitHandle,'UserData','done');
                  close(WaitHandle);
                  drawnow;
                  pause(0.02);
               end;
            else;
               WaitHandle = findobj('Type','figure','Tag','WaitDLG');
               WaitControls = get(WaitHandle,'UserData');
               set(WaitControls.text,'UserData','done');
               delete(WaitHandle);
               drawnow;
               pause(0.02);
            end;
            %
            SetLogText(strvcat(' ','Csminwel error: optimization ended abruptly.'));
            if isempty(CurrINI.Sequence)==1;
               ErrorStr = ['There was an error running csminwel. YADA caught the following error message: ' lasterr];
               About(ErrorStr,'information','Optimization Problem',400,500,CurrINI);
               drawnow;
            end;
            return;
         end;
         %
         % gh is the numerical gradient, fh the function value at minimum
         %
         % ReturnCode: 0 = minimum found
         %             1 = zero gradient
         %        2 or 4 = back and forth on step length never finished
         %             3 = smallest step still improving too slow
         %             5 = largest step still improving too fast
         %             6 = smallest step still improving too slow, reversed gradient
         %             7 = warning: possible inaccuracy in H matrix
         %
         if (ReturnCode==0)&(strcmp(lower(AbortEstimation),'cancel')~=1)&(NumIter<=MaxIter);
            %
            % we're OK
            %
            LogJacobianMode = logJacobian(phiMode,thetaIndex,UniformBounds);
            if IsOdd(DSGEModel.MaximizeAlgorithmValue)==1;
               LogPostDensity = -fh;
               thetaMode = PhiToTheta(phiMode,thetaIndex,UniformBounds,LowerBound);
            else;
               LogPostDensity = -fh+LogJacobianMode;
            end;
            %
            LogPriorMode = logPriorDSGE(thetaMode,thetaDist,PriorDist,LowerBound);
            %
            % make sure that ModelParameters is updated
            %
            ModelParameters = ThetaToModelParameters(ModelParameters,thetaMode,thetaPositions);
            if IsSystemPrior==1;
               eval(['LogSPMode = ' GetName(DSGEModel.SystemPriorFile) '(thetaMode,thetaPositions,ModelParameters,AIMData,DSGEModel,CurrINI.OrderQZ);']);
               SystemPriorStr = [' * Log system prior at mode:         ' num2str(LogSPMode,'%0.4f')];
            else;
               LogSPMode = 0;
               SystemPriorStr = '';
            end;
            LogLikelihoodMode = LogPostDensity-(LogPriorMode+LogJacobianMode+LogSPMode);
            %
            SetLogText(strvcat(' ','Optimization completed!',[' * Time:                             ' StringTrim(datestr(now,14))],[' * Optimization time:                ' SecondsToHoursStr(MaxTime) ' (hrs:min:sec)'],[' * Number of iterations:             ' num2str(NumIter)],[' * Convergence value:                ' num2str(FuncDiff)],[' * max(abs(gradient)):               ' num2str(OptResults(NumIter,4))],[' * Log posterior of phi at mode:     ' num2str(LogPostDensity,'%0.4f')],[' * Log posterior of theta at mode:   ' num2str(LogPostDensity-LogJacobianMode,'%0.4f')],[' * Log prior at mode:                ' num2str(LogPriorMode,'%0.4f')],[' * Log Jacobian at mode:             ' num2str(LogJacobianMode,'%0.4f')],SystemPriorStr,[' * Log likelihood at mode:           ' num2str(LogLikelihoodMode,'%0.4f')]));
            %
            EstimationStatus = 1;
            ContinueEstimation = 1;
         elseif strcmp(lower(AbortEstimation),'cancel')==1;
            %
            % posteriod mode estimation was aborted
            %
            SetLogText(strvcat(' ','Joint posterior mode estimation was aborted.'));
         elseif (ReturnCode==0)&(strcmp(lower(AbortEstimation),'cancel')~=1)&(NumIter>MaxIter);
            %
            % maximum number of iterations was reached
            %
            txt = strvcat(['The maximum number of iterations was reached without csminwel converging during joint posterior mode estimation of the DSGE-VAR with lambda=' num2str(lambdai,'%0.4f') '. Would you like to skip this lambda-value?'],' ','NOTE: If you answer No, then YADA will abort.');
            SetLogText(strvcat(' ','Maximum number of iteration reached.'));
            drawnow;
            if isempty(CurrINI.Sequence)==1;
               answer = Query(txt,'question',300,['Optimization Problem - Lambda=' num2str(lambdai,'%0.4f')],500,'no',CurrINI);
            else;
               answer = CurrINI.Sequence.QueryAnswers{2};
            end;
            if strcmp(lower(answer),'yes')==1;
               ContinueEstimation = 1;
               SetLogText(strvcat(['User decided to skip lambda=' num2str(lambdai,'%0.4f')],' '));
            else;
               SetLogText('Joint posterior mode estimation was aborted.');
            end;
            drawnow;
         elseif ReturnCode==1;
            SetLogText(strvcat(' ','Csminwel return code: zero gradient.'));
            if isempty(CurrINI.Sequence)==1;
               txt = ['Csminwel return code: zero gradient. YADA has aborted from joint posterior mode estimation. Writing optimization information to file: "OptErrorReport-Joint-DSGEVAR-' num2str(lambdai,'%0.4f') '-' int2str(p) '-' SelectedShockStr '-' DSGEModel.NameOfModel '.txt" in "' DSGEModel.OutputDirectory '".'];
               About(txt,'information','Optimization Problem',200,500,CurrINI);
               drawnow;
            end;
            ReturnCodeText = 'Csminwel return code: zeros gradient.';
         elseif (ReturnCode==2)|(ReturnCode==4);
            SetLogText(strvcat(' ','Csminwel return code: back and forth on step length never finished.'));
            if isempty(CurrINI.Sequence)==1;
               txt = ['Csminwel return code: back and forth on step length never finished. YADA has aborted from joint posterior mode estimation. Writing optimization information to file: "OptErrorReport-Joint-DSGEVAR-' num2str(lambdai,'%0.4f') '-' int2str(p) '-' SelectedShockStr '-' DSGEModel.NameOfModel '.txt" in "' DSGEModel.OutputDirectory '".'];
               About(txt,'information','Optimization Problem',200,500,CurrINI);
               drawnow;
            end;
            ReturnCodeText = 'Csminwel return code: back and forth on step length never finished.';
         elseif ReturnCode==3;
            SetLogText(strvcat(' ','Csminwel return code: smallest step still improving too slow.'));
            if isempty(CurrINI.Sequence)==1;
               txt = ['Csminwel return code: smallest step still improving too slow. YADA has aborted from joint posterior mode estimation. Writing optimization information to file: "OptErrorReport-Joint-DSGEVAR-' num2str(lambdai,'%0.4f') '-' int2str(p) '-' SelectedShockStr '-' DSGEModel.NameOfModel '.txt" in "' DSGEModel.OutputDirectory '".'];
               About(txt,'information','Optimization Problem',200,500,CurrINI);
               drawnow;
            end;
            ReturnCodeText = 'Csminwel return code: smallest step still improving too slow.';
         elseif ReturnCode==5;
            SetLogText(strvcat(' ','Csminwel return code: largest step still improving too fast.'));
            if isempty(CurrINI.Sequence)==1;
               txt = ['Csminwel return code: largest step still improving too fast. YADA has aborted from joint posterior mode estimation. Writing optimization information to file: "OptErrorReport-Joint-DSGEVAR-' num2str(lambdai,'%0.4f') '-' int2str(p) '-' SelectedShockStr '-' DSGEModel.NameOfModel '.txt" in "' DSGEModel.OutputDirectory '".'];
               About(txt,'information','Optimization Problem',200,500,CurrINI);
               drawnow;
            end;
            ReturnCodeText = 'Csminwel return code: largest step still improving too fast.';
         elseif ReturnCode==6;
            SetLogText(strvcat(' ','Csminwel return code: smallest step still improving too slow, reversed gradient.'));
            if isempty(CurrINI.Sequence)==1;
               txt = ['Csminwel return code: smallest step still improving too slow, reversed gradient. YADA has aborted from joint posterior mode estimation. Writing optimization information to file: "OptErrorReport-Joint-DSGEVAR-' num2str(lambdai,'%0.4f') '-' int2str(p) '-' SelectedShockStr '-' DSGEModel.NameOfModel '.txt" in "' DSGEModel.OutputDirectory '".'];
               About(txt,'information','Optimization Problem',200,500,CurrINI);
               drawnow;
            end;
            ReturnCodeText = 'Csminwel return code: smallest step still improving too slow, reversed gradient.';
         elseif ReturnCode==7;
            SetLogText(strvcat(' ','Csminwel return code: possible inaccuracy in inverse Hessian.'));
            if isempty(CurrINI.Sequence)==1;
               txt = ['Csminwel return code: possible inaccuracy in inverse Hessian. YADA has aborted from joint posterior mode estimation. Writing optimization information to file: "OptErrorReport-Joint-DSGEVAR-' num2str(lambdai,'%0.4f') '-' int2str(p) '-' SelectedShockStr '-' DSGEModel.NameOfModel '.txt" in "' DSGEModel.OutputDirectory '".'];
               About(txt,'information','Optimization Problem',200,500,CurrINI);
               drawnow;
            end;
            ReturnCodeText = 'Csminwel return code: possible inaccuracy in inverse Hessian.';
         end;
      elseif (DSGEModel.MaximizeAlgorithmValue==3)|(DSGEModel.MaximizeAlgorithmValue==4);
         %
         % We use Ratto's newrat routine with transformed parameters.
         %
         try;
            tic;
            if DSGEModel.MaximizeAlgorithmValue==3;
               [phiMode,HessianMat,gh,fh,InverseHessian,NumIter,fcount,FuncDiff,AbortEstimation,OptResults,ScreenMessages,phiEstimates,ReturnCode] = YADAnewrat('logConcPosteriorPhiDSGEVAR',phi,HessianMat,[],[],ToleranceValue,MaxIter,2,ParamLUB,PriorStd,DSGEModel.ShowProgress,DSGEModel.ShowProgressClock,'Joint Posterior Mode Estimation - newrat',CurrINI, ...
                  thetaIndex,UniformBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,DSGEModel,AIMData,lambdai,T,n,p,npk,GammaHatyy,GammaHatyY,GammaHatYY,DetProductMoments,HSample,logCPC,CurrINI.OrderQZ,1);
            else;
               [thetaMode,HessianMat,gh,fh,InverseHessian,NumIter,fcount,FuncDiff,AbortEstimation,OptResults,ScreenMessages,thetaEstimates,ReturnCode] = YADAnewrat('logConcPosteriorThetaDSGEVAR',theta,HessianMat,[],[],ToleranceValue,MaxIter,2,ParamLUB,PriorStd,DSGEModel.ShowProgress,DSGEModel.ShowProgressClock,'Joint Posterior Mode Estimation - newrat',CurrINI, ...
                  thetaIndex,ParameterBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,DSGEModel,AIMData,lambdai,T,n,p,npk,GammaHatyy,GammaHatyY,GammaHatYY,DetProductMoments,HSample,logCPC,CurrINI.OrderQZ,1);
               %
               % make sure that the outputs are transformed into phi-space
               %
               phiMode = ThetaToPhi(thetaMode,thetaIndex,UniformBounds,LowerBound);
               phiEstimates = zeros(size(thetaEstimates));
               for j=1:size(thetaEstimates,2);
                  phiEstimates(:,j) = ThetaToPhi(thetaEstimates(:,j),thetaIndex,UniformBounds,LowerBound);;
               end;
               ThetaPhiPartial = PartialThetaPartialPhi(phiMode,thetaIndex,UniformBounds);
               InverseHessian = (ThetaPhiPartial\InverseHessian)/ThetaPhiPartial;
            end;
            MaxTime = toc;
            GradientVector = abs(gh);
         catch;
            if DSGEModel.ShowProgress==1;
               WaitHandle = findobj('Type','figure','Tag','TMWProgressDLG');
               if IsHandle(WaitHandle)==1;
                  set(WaitHandle,'UserData','done');
                  close(WaitHandle);
                  drawnow;
                  pause(0.02);
               end;
            else;
               WaitHandle = findobj('Type','figure','Tag','WaitDLG');
               WaitControls = get(WaitHandle,'UserData');
               set(WaitControls.text,'UserData','done');
               delete(WaitHandle);
               drawnow;
               pause(0.02);
            end;
            SetLogText(strvcat(' ','Newrat error: optimization ended abruptly.'));
            if isempty(CurrINI.Sequence)==1;
               ErrorStr = ['There was an error running newrat. YADA caught the following error message: ' lasterr];
               About(ErrorStr,'information','Optimization Problem',400,500,CurrINI);
               drawnow;
            end;
            return;
         end;
         %
         % gh is the numerical gradient, fh the function value at minimum
         %
         % ReturnCode: 0 = minimum found
         %             1 = zero gradient
         %        2 or 4 = back and forth on step length never finished
         %             3 = smallest step still improving too slow
         %             5 = largest step still improving too fast
         %             6 = smallest step still improving too slow, reversed gradient
         %             7 = warning: possible inaccuracy in H matrix
         %
         if (ReturnCode==0)&(strcmp(lower(AbortEstimation),'cancel')~=1)&(NumIter<=MaxIter);
            %
            % we're OK
            %
            LogJacobianMode = logJacobian(phiMode,thetaIndex,UniformBounds);
            if IsOdd(DSGEModel.MaximizeAlgorithmValue)==1;
               LogPostDensity = -fh;
               thetaMode = PhiToTheta(phiMode,thetaIndex,UniformBounds,LowerBound);
            else;
               LogPostDensity = -fh+LogJacobianMode;
            end;
            %
            LogPriorMode = logPriorDSGE(thetaMode,thetaDist,PriorDist,LowerBound);
            %
            % make sure that ModelParameters is updated
            %
            ModelParameters = ThetaToModelParameters(ModelParameters,thetaMode,thetaPositions);
            if IsSystemPrior==1;
               eval(['LogSPMode = ' GetName(DSGEModel.SystemPriorFile) '(thetaMode,thetaPositions,ModelParameters,AIMData,DSGEModel,CurrINI.OrderQZ);']);
               SystemPriorStr = [' * Log system prior at mode:         ' num2str(LogSPMode,'%0.4f')];
            else;
               LogSPMode = 0;
               SystemPriorStr = '';
            end;
            LogLikelihoodMode = LogPostDensity-(LogPriorMode+LogJacobianMode+LogSPMode);
            %
            SetLogText(strvcat(' ','Optimization completed!',[' * Time:                             ' StringTrim(datestr(now,14))],[' * Optimization time:                ' SecondsToHoursStr(MaxTime) ' (hrs:min:sec)'],[' * Number of iterations:             ' num2str(NumIter)],[' * Convergence value:                ' num2str(FuncDiff)],[' * max(abs(gradient)):               ' num2str(OptResults(NumIter,4))],[' * Log posterior of phi at mode:     ' num2str(LogPostDensity,'%0.4f')],[' * Log posterior of theta at mode:   ' num2str(LogPostDensity-LogJacobianMode,'%0.4f')],[' * Log prior at mode:                ' num2str(LogPriorMode,'%0.4f')],[' * Log Jacobian at mode:             ' num2str(LogJacobianMode,'%0.4f')],SystemPriorStr,[' * Log likelihood at mode:           ' num2str(LogLikelihoodMode,'%0.4f')]));
            %
            EstimationStatus = 1;
            ContinueEstimation = 1;
         elseif strcmp(lower(AbortEstimation),'cancel')==1;
            %
            % posteriod mode estimation was aborted
            %
            SetLogText(strvcat(' ','Joint posterior mode estimation was aborted.'));
         elseif (ReturnCode==0)&(strcmp(lower(AbortEstimation),'cancel')~=1)&(NumIter>MaxIter);
            %
            % maximum number of iterations was reached
            %
            txt = strvcat(['The maximum number of iterations was reached without newrat converging during joint posterior mode estimation of the DSGE-VAR with lambda=' num2str(lambdai,'%0.4f') '. Would you like to skip this lambda-value?'],' ','NOTE: If you answer No, then YADA will abort.');
            SetLogText(strvcat(' ','Maximum number of iteration reached.'));
            drawnow;
            if isempty(CurrINI.Sequence)==1;
               answer = Query(txt,'question',300,['Optimization Problem - Lambda=' num2str(lambdai,'%0.4f')],500,'no',CurrINI);
            else;
               answer = CurrINI.Sequence.QueryAnswers{2};
            end;
            if strcmp(lower(answer),'yes')==1;
               ContinueEstimation = 1;
               SetLogText(strvcat(['User decided to skip lambda=' num2str(lambdai,'%0.4f')],' '));
            else;
               SetLogText('Joint posterior mode estimation was aborted.');
            end;
            drawnow;
         elseif ReturnCode==1;
            SetLogText(strvcat(' ','Newrat return code: zero gradient.'));
            if isempty(CurrINI.Sequence)==1;
               txt = ['Newrat return code: zero gradient. YADA has aborted from joint posterior mode estimation. Writing optimization information to file: "OptErrorReport-Joint-DSGEVAR-' num2str(lambdai,'%0.4f') '-' int2str(p) '-' SelectedShockStr '-' DSGEModel.NameOfModel '.txt" in "' DSGEModel.OutputDirectory '".'];
               About(txt,'information','Optimization Problem',200,500,CurrINI);
               drawnow;
            end;
            ReturnCodeText = 'Newrat return code: zeros gradient.';
         elseif (ReturnCode==2)|(ReturnCode==4);
            SetLogText(strvcat(' ','Newrat return code: back and forth on step length never finished.'));
            if isempty(CurrINI.Sequence)==1;
               txt = ['Newrat return code: back and forth on step length never finished. YADA has aborted from joint posterior mode estimation. Writing optimization information to file: "OptErrorReport-Joint-DSGEVAR-' num2str(lambdai,'%0.4f') '-' int2str(p) '-' SelectedShockStr '-' DSGEModel.NameOfModel '.txt" in "' DSGEModel.OutputDirectory '".'];
               About(txt,'information','Optimization Problem',200,500,CurrINI);
               drawnow;
            end;
            ReturnCodeText = 'Newrat return code: back and forth on step length never finished.';
         elseif ReturnCode==3;
            SetLogText(strvcat(' ','Newrat return code: smallest step still improving too slow.'));
            if isempty(CurrINI.Sequence)==1;
               txt = ['Newrat return code: smallest step still improving too slow. YADA has aborted from joint posterior mode estimation. Writing optimization information to file: "OptErrorReport-Joint-DSGEVAR-' num2str(lambdai,'%0.4f') '-' int2str(p) '-' SelectedShockStr '-' DSGEModel.NameOfModel '.txt" in "' DSGEModel.OutputDirectory '".'];
               About(txt,'information','Optimization Problem',200,500,CurrINI);
               drawnow;
            end;
            ReturnCodeText = 'Newrat return code: smallest step still improving too slow.';
         elseif ReturnCode==5;
            SetLogText(strvcat(' ','Newrat return code: largest step still improving too fast.'));
            if isempty(CurrINI.Sequence)==1;
               txt = ['Newrat return code: largest step still improving too fast. YADA has aborted from joint posterior mode estimation. Writing optimization information to file: "OptErrorReport-Joint-DSGEVAR-' num2str(lambdai,'%0.4f') '-' int2str(p) '-' SelectedShockStr '-' DSGEModel.NameOfModel '.txt" in "' DSGEModel.OutputDirectory '".'];
               About(txt,'information','Optimization Problem',200,500,CurrINI);
               drawnow;
            end;
            ReturnCodeText = 'Newrat return code: largest step still improving too fast.';
         elseif ReturnCode==6;
            SetLogText(strvcat(' ','Newrat return code: smallest step still improving too slow, reversed gradient.'));
            if isempty(CurrINI.Sequence)==1;
               txt = ['Newrat return code: smallest step still improving too slow, reversed gradient. YADA has aborted from joint posterior mode estimation. Writing optimization information to file: "OptErrorReport-Joint-DSGEVAR-' num2str(lambdai,'%0.4f') '-' int2str(p) '-' SelectedShockStr '-' DSGEModel.NameOfModel '.txt" in "' DSGEModel.OutputDirectory '".'];
               About(txt,'information','Optimization Problem',200,500,CurrINI);
               drawnow;
            end;
            ReturnCodeText = 'Newrat return code: smallest step still improving too slow, reversed gradient.';
         elseif ReturnCode==7;
            SetLogText(strvcat(' ','Newrat return code: possible inaccuracy in inverse Hessian.'));
            if isempty(CurrINI.Sequence)==1;
               txt = ['Newrat return code: possible inaccuracy in inverse Hessian. YADA has aborted from joint posterior mode estimation. Writing optimization information to file: "OptErrorReport-Joint-DSGEVAR-' num2str(lambdai,'%0.4f') '-' int2str(p) '-' SelectedShockStr '-' DSGEModel.NameOfModel '.txt" in "' DSGEModel.OutputDirectory '".'];
               About(txt,'information','Optimization Problem',200,500,CurrINI);
               drawnow;
            end;
            ReturnCodeText = 'Newrat return code: possible inaccuracy in inverse Hessian.';
         end;
      elseif (DSGEModel.MaximizeAlgorithmValue==5)|(DSGEModel.MaximizeAlgorithmValue==6);
         %
         % use Dynare's gmhmaxlik
         %
         try;
            if DSGEModel.MaximizeAlgorithmValue==5;
               [phiMode,InverseHessian,cEst,phiMean,fh,gh,fcount,FuncDiff,AbortEstimation,OptResults,ScreenMessages,phiEstimates,MaxTime] = YADAgmhmaxlik('logConcPosteriorPhiDSGEVAR',phi,ParamLUB,NumDraws,c,ParamMean,InitialInvHessian,DSGEModel.RandomNumberValue,DSGEModel.ShowProgress,DSGEModel.ShowProgressClock,'Joint Posterior Mode Estimation - gmhmaxlik',CurrINI, ...
                  thetaIndex,UniformBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,DSGEModel,AIMData,lambdai,T,n,p,npk,GammaHatyy,GammaHatyY,GammaHatYY,DetProductMoments,HSample,logCPC,CurrINI.OrderQZ,1);
            else;
               [thetaMode,InverseHessian,cEst,phiMean,fh,gh,fcount,FuncDiff,AbortEstimation,OptResults,ScreenMessages,thetaEstimates,MaxTime] = YADAgmhmaxlik('logConcPosteriorThetaDSGEVAR',theta,ParamLUB,NumDraws,c,ParamMean,InitialInvHessian,DSGEModel.RandomNumberValue,DSGEModel.ShowProgress,DSGEModel.ShowProgressClock,'Joint Posterior Mode Estimation - gmhmaxlik',CurrINI, ...
                  thetaIndex,ParameterBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,DSGEModel,AIMData,lambdai,T,n,p,npk,GammaHatyy,GammaHatyY,GammaHatYY,DetProductMoments,HSample,logCPC,CurrINI.OrderQZ,1);
               %
               % make sure that the outputs are transformed into phi-space
               %
               phiMode = ThetaToPhi(thetaMode,thetaIndex,UniformBounds,LowerBound);
               phiEstimates = zeros(size(thetaEstimates));
               for j=1:size(thetaEstimates,2);
                  phiEstimates(:,j) = ThetaToPhi(thetaEstimates(:,j),thetaIndex,UniformBounds,LowerBound);;
               end;
               ThetaPhiPartial = PartialThetaPartialPhi(phiMode,thetaIndex,UniformBounds);
               InverseHessian = (ThetaPhiPartial\InverseHessian)/ThetaPhiPartial;
            end;
            %
            GradientVector = abs(gh);
            ReturnCode = 0;
            NumIter = NumDraws;
         catch;
            if DSGEModel.ShowProgress==1;
               WaitHandle = findobj('Type','figure','Tag','TMWProgressDLG');
               if IsHandle(WaitHandle)==1;
                  set(WaitHandle,'UserData','done');
                  close(WaitHandle);
                  drawnow;
                  pause(0.02);
               else;
                  WaitHandle = findobj('Type','figure','Tag','WaitDLG');
                  if IsHandle(WaitHandle)==1;
                     WaitControls = get(WaitHandle,'UserData');
                     set(WaitControls.text,'UserData','done');
                     delete(WaitHandle);
                     drawnow;
                     pause(0.02);
                  end;
               end;
            else;
               WaitHandle = findobj('Type','figure','Tag','WaitDLG');
               if IsHandle(WaitHandle)==1;
                  WaitControls = get(WaitHandle,'UserData');
                  set(WaitControls.text,'UserData','done');
                  delete(WaitHandle);
                  drawnow;
                  pause(0.02);
               end;
            end;
            AbortEstimation = '';
            NumIter = [];
            GradientVector = [];
            SetLogText(strvcat(' ','Gmhmaxlik error: optimization ended abruptly.'));
            if isempty(CurrINI.Sequence)==1;
               ErrorStr = ['There was an error running gmhmaxlik. YADA caught the following error message: ' lasterr];
               About(ErrorStr,'information','Optimization Problem',400,500,CurrINI);
               drawnow;
            end;
            return;
         end;
         %
         % check what we should do
         %
         if (ReturnCode==0)&(strcmp(lower(AbortEstimation),'cancel')~=1);
            %
            % we're OK
            %
            LogJacobianMode = logJacobian(phiMode,thetaIndex,UniformBounds);
            if IsOdd(DSGEModel.MaximizeAlgorithmValue)==1;
               LogPostDensity = -fh;
               thetaMode = PhiToTheta(phiMode,thetaIndex,UniformBounds,LowerBound);
            else;
               LogPostDensity = -fh+LogJacobianMode;
            end;
            %
            % Calculate the log posterior at theta
            %
            LogPriorMode = logPriorDSGE(thetaMode,thetaDist,PriorDist,LowerBound);
            %
            % make sure that ModelParameters is updated
            %
            ModelParameters = ThetaToModelParameters(ModelParameters,thetaMode,thetaPositions);
            if IsSystemPrior==1;
               eval(['LogSPMode = ' GetName(DSGEModel.SystemPriorFile) '(thetaMode,thetaPositions,ModelParameters,AIMData,DSGEModel,CurrINI.OrderQZ);']);
               SystemPriorStr = [' * Log system prior at mode:         ' num2str(LogSPMode,'%0.4f')];
            else;
               LogSPMode = 0;
               SystemPriorStr = '';
            end;
            LogLikelihoodMode = LogPostDensity-(LogPriorMode+LogJacobianMode+LogSPMode);
            %
            SetLogText(strvcat(' ','Optimization completed!',[' * Time:                             ' StringTrim(datestr(now,14))],[' * Optimization time:                ' SecondsToHoursStr(MaxTime) ' (hrs:min:sec)'],[' * Objective function change:        ' num2str(FuncDiff)],[' * max(abs(gradient)):               ' num2str(OptResults(NumIter,4))],[' * Log posterior of phi at mode:     ' num2str(LogPostDensity,'%0.4f')],[' * Log posterior of theta at mode:   ' num2str(LogPostDensity-LogJacobianMode,'%0.4f')],[' * Log prior at mode:                ' num2str(LogPriorMode,'%0.4f')],[' * Log Jacobian at mode:             ' num2str(LogJacobianMode,'%0.4f')],SystemPriorStr,[' * Log likelihood at mode:           ' num2str(LogLikelihoodMode,'%0.4f')]));
            %
            EstimationStatus = 1;
            ContinueEstimation = 1;
         elseif (ReturnCode==0)&(strcmp(lower(AbortEstimation),'cancel')==1);
            %
            % the user selected to abort posterior mode estimation
            %
            SetLogText(strvcat(' ','Marginal posterior mode estimation was aborted.'));
         end;
      elseif (DSGEModel.MaximizeAlgorithmValue==7)|(DSGEModel.MaximizeAlgorithmValue==8);
         %
         % use matlab's fminunc
         %
         if MatlabNumber>=7;
            tic;
            if IsOdd(DSGEModel.MaximizeAlgorithmValue)==1;
               [phiMode,fh,ExitFlag,OUTPUT,Grad,HessianMatrix] = YADAfminunc7('logConcPosteriorPhiDSGEVAR',phi,Options,DSGEModel.ShowProgress,DSGEModel.ShowProgressClock,'Joint Posterior Mode Estimation - fminunc',CurrINI, ...
                  thetaIndex,UniformBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,DSGEModel,AIMData,lambdai,T,n,p,npk,GammaHatyy,GammaHatyY,GammaHatYY,DetProductMoments,HSample,logCPC,CurrINI.OrderQZ,1);
            else;
               [thetaMode,fh,ExitFlag,OUTPUT,Grad,HessianMatrix] = YADAfminunc7('logConcPosteriorThetaDSGEVAR',theta,Options,DSGEModel.ShowProgress,DSGEModel.ShowProgressClock,'Joint Posterior Mode Estimation - fminunc',CurrINI, ...
                  thetaIndex,ParameterBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,DSGEModel,AIMData,lambdai,T,n,p,npk,GammaHatyy,GammaHatyY,GammaHatYY,DetProductMoments,HSample,logCPC,CurrINI.OrderQZ,1);
               %
               % make sure that the outputs are transformed into phi-space
               %
               phiMode = ThetaToPhi(thetaMode,thetaIndex,UniformBounds,LowerBound);
               phiEstimates = zeros(size(OUTPUT.phiEstimates));
               for j=1:size(OUTPUT.phiEstimates,2);
                  phiEstimates(:,j) = ThetaToPhi(OUTPUT.phiEstimates(:,j),thetaIndex,UniformBounds,LowerBound);;
               end;
               OUTPUT.phiEstimates = phiEstimates;
               ThetaPhiPartial = PartialThetaPartialPhi(phiMode,thetaIndex,UniformBounds);
               HessianMatrix = ThetaPhiPartial*HessianMatrix*ThetaPhiPartial;
            end;
            MaxTime = toc;
            GradientVector = abs(Grad);
            AbortEstimation = OUTPUT.abort;
            NumIter = OUTPUT.iterations;
            OptResults = OUTPUT.OptResults;
            if (ExitFlag>0)&(strcmp(lower(AbortEstimation),'cancel')~=1);
               %
               % we're OK
               %
               FuncDiff = OUTPUT.convergence;
               LogJacobianMode = logJacobian(phiMode,thetaIndex,UniformBounds);
               if IsOdd(DSGEModel.MaximizeAlgorithmValue)==1;
                  LogPostDensity = -fh;
                  thetaMode = PhiToTheta(phiMode,thetaIndex,UniformBounds,LowerBound);
               else;
                  LogPostDensity = -fh+LogJacobianMode;
               end;
               InverseHessian = HessianMatrix\eye(NumParam);
               phiEstimates = OUTPUT.phiEstimates;
               %
               LogPriorMode = logPriorDSGE(thetaMode,thetaDist,PriorDist,LowerBound);
               %
               % make sure that ModelParameters is updated
               %
               ModelParameters = ThetaToModelParameters(ModelParameters,thetaMode,thetaPositions);
               if IsSystemPrior==1;
                  eval(['LogSPMode = ' GetName(DSGEModel.SystemPriorFile) '(thetaMode,thetaPositions,ModelParameters,AIMData,DSGEModel,CurrINI.OrderQZ);']);
                  SystemPriorStr = [' * Log system prior at mode:         ' num2str(LogSPMode,'%0.4f')];
               else;
                  LogSPMode = 0;
                  SystemPriorStr = '';
               end;
               LogLikelihoodMode = LogPostDensity-(LogPriorMode+LogJacobianMode+LogSPMode);
               %
               SetLogText(strvcat(' ','Optimization completed!',[' * Time:                             ' StringTrim(datestr(now,14))],[' * Optimization time:                ' SecondsToHoursStr(MaxTime) ' (hrs:min:sec)'],[' * Number of iterations:             ' num2str(NumIter)],[' * Convergence value:                ' num2str(FuncDiff)],[' * max(abs(gradient)):               ' num2str(OptResults(NumIter,4))],[' * Log posterior of phi at mode:     ' num2str(LogPostDensity,'%0.4f')],[' * Log posterior of theta at mode:   ' num2str(LogPostDensity-LogJacobianMode,'%0.4f')],[' * Log prior at mode:                ' num2str(LogPriorMode,'%0.4f')],[' * Log Jacobian at mode:             ' num2str(LogJacobianMode,'%0.4f')],SystemPriorStr,[' * Log likelihood at mode:           ' num2str(LogLikelihoodMode,'%0.4f')]));
               %
               EstimationStatus = 1;
               ContinueEstimation = 1;
            elseif strcmp(lower(AbortEstimation),'cancel')==1;
               %
               % posteriod mode estimation was aborted
               %
               SetLogText(strvcat(' ','Joint posterior mode estimation was aborted.'));
            elseif ExitFlag==0;
               %
               % Reached maximum number of iteration
               %
               SetLogText(strvcat(' ','Maximum number of function evaluations or iteration reached.',[' * Time:                             ' StringTrim(datestr(now,14))],[' * Optimization time:                ' SecondsToHoursStr(MaxTime) ' (hrs:min:sec)']));
               if isempty(CurrINI.Sequence)==1;
                  txt = ['The maximum number of function evaluations or iterations was reached without fminunc converging (ExitFlag is 0). YADA has aborted from joint posterior mode estimation of the DSGE-VAR with lambda=' num2str(lambdai,'%0.4f') '. Writing optimization information to file: "OptErrorReport-Joint-DSGEVAR-' num2str(lambdai,'%0.4f') '-' int2str(p) '-' SelectedShockStr '-' DSGEModel.NameOfModel '.txt" in "' DSGEModel.OutputDirectory '".'];
                  About(txt,'information','Optimization Problem',200,500,CurrINI);
                  drawnow;
               end;
               ReturnCodeText = 'Fminunc return code: the maximum number of function evaluations or iteration was reached without fminunc coverging (ExitFlag is 0).';
            elseif ExitFlag<0;
               %
               % no convergence
               %
               SetLogText(strvcat(' ',['Fminunc didn''t converge (ExitFlag is ' int2str(ExitFlag) ').'],[' * Time:                             ' StringTrim(datestr(now,14))],[' * Optimization time:                ' SecondsToHoursStr(MaxTime) ' (hrs:min:sec)']));
               if isempty(CurrINI.Sequence)==1;
                  txt = ['Fminunc didn''t converge (ExitFlag is ' int2str(ExitFlag) '). YADA has aborted from joint posterior mode estimation of the DSGE-VAR with lambda=' num2str(lambdai,'%0.4f') '. Writing optimization information to file: "OptErrorReport-Joint-DSGEVAR-' num2str(lambdai,'%0.4f') '-' int2str(p) '-' SelectedShockStr '-' DSGEModel.NameOfModel '.txt" in "' DSGEModel.OutputDirectory '".'];
                  About(txt,'information','Optimization Problem',200,500,CurrINI);
                  drawnow;
               end;
               ReturnCodeText = ['Fminunc return code: No convergence (ExitFlag is ' int2str(ExitFlag) ').'];
            end;
         else;
            tic;
            if IsOdd(DSGEModel.MaximizeAlgorithmValue)==1;
               [phiMode,fh,ExitFlag,OUTPUT,Grad,HessianMatrix] = YADAfminunc5('logConcPosteriorPhiDSGEVAR',phi,Options,DSGEModel.ShowProgress,DSGEModel.ShowProgressClock,'Joint Posterior Mode Estimation - fminunc',CurrINI, ...
                  thetaIndex,UniformBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,DSGEModel,AIMData,lambdai,T,n,p,npk,GammaHatyy,GammaHatyY,GammaHatYY,DetProductMoments,HSample,logCPC,CurrINI.OrderQZ,1);
            else;
               [thetaMode,fh,ExitFlag,OUTPUT,Grad,HessianMatrix] = YADAfminunc5('logConcPosteriorThetaDSGEVAR',theta,Options,DSGEModel.ShowProgress,DSGEModel.ShowProgressClock,'Joint Posterior Mode Estimation - fminunc',CurrINI, ...
                  thetaIndex,ParameterBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,DSGEModel,AIMData,lambdai,T,n,p,npk,GammaHatyy,GammaHatyY,GammaHatYY,DetProductMoments,HSample,logCPC,CurrINI.OrderQZ,1);
               %
               % make sure that the outputs are transformed into phi-space
               %
               phiMode = ThetaToPhi(thetaMode,thetaIndex,UniformBounds,LowerBound);
               phiEstimates = zeros(size(OUTPUT.phiEstimates));
               for j=1:size(OUTPUT.phiEstimates,2);
                  phiEstimates(:,j) = ThetaToPhi(OUTPUT.phiEstimates(:,j),thetaIndex,UniformBounds,LowerBound);;
               end;
               OUTPUT.phiEstimates = phiEstimates;
               ThetaPhiPartial = PartialThetaPartialPhi(phiMode,thetaIndex,UniformBounds);
               HessianMatrix = ThetaPhiPartial*HessianMatrix*ThetaPhiPartial;
            end;
            MaxTime = toc;
            AbortEstimation = OUTPUT.abort;
            NumIter = OUTPUT.iterations;
            OptResults = OUTPUT.OptResults;
            GradientVector = abs(Grad);
            if (ExitFlag>0)&(strcmp(lower(AbortEstimation),'cancel')~=1);
               %
               % we're OK
               %
               FuncDiff = OUTPUT.convergence;
               LogJacobianMode = logJacobian(phiMode,thetaIndex,UniformBounds);
               if IsOdd(DSGEModel.MaximizeAlgorithmValue)==1;
                  LogPostDensity = -fh;
                  thetaMode = PhiToTheta(phiMode,thetaIndex,UniformBounds,LowerBound);
               else;
                  LogPostDensity = -fh+LogJacobianMode;
               end;
               InverseHessian = HessianMatrix\eye(NumParam);
               phiEstimates = OUTPUT.phiEstimates;
               %
               LogPriorMode = logPriorDSGE(thetaMode,thetaDist,PriorDist,LowerBound);
               %
               % make sure that ModelParameters is updated
               %
               ModelParameters = ThetaToModelParameters(ModelParameters,thetaMode,thetaPositions);
               if IsSystemPrior==1;
                  eval(['LogSPMode = ' GetName(DSGEModel.SystemPriorFile) '(thetaMode,thetaPositions,ModelParameters,AIMData,DSGEModel,CurrINI.OrderQZ);']);
                  SystemPriorStr = [' * Log system prior at mode:         ' num2str(LogSPMode,'%0.4f')];
               else;
                  LogSPMode = 0;
                  SystemPriorStr = '';
               end;
               LogLikelihoodMode = LogPostDensity-(LogPriorMode+LogJacobianMode+LogSPMode);
               %
               SetLogText(strvcat(' ','Optimization completed!',[' * Time:                             ' StringTrim(datestr(now,14))],[' * Optimization time:                ' SecondsToHoursStr(MaxTime) ' (hrs:min:sec)'],[' * Number of iterations:             ' num2str(NumIter)],[' * Convergence value:                ' num2str(FuncDiff)],[' * max(abs(gradient)):               ' num2str(OptResults(NumIter,4))],[' * Log posterior of phi at mode:     ' num2str(LogPostDensity,'%0.4f')],[' * Log posterior of theta at mode:   ' num2str(LogPostDensity-LogJacobianMode,'%0.4f')],[' * Log prior at mode:                ' num2str(LogPriorMode,'%0.4f')],[' * Log Jacobian at mode:             ' num2str(LogJacobianMode,'%0.4f')],SystemPriorStr,[' * Log likelihood at mode:           ' num2str(LogLikelihoodMode,'%0.4f')]));
               %
               EstimationStatus = 1;
               ContinueEstimation = 1;
            elseif strcmp(lower(AbortEstimation),'cancel')==1;
               %
               % posteriod mode estimation was aborted
               %
               SetLogText(strvcat(' ','Joint posterior mode estimation was aborted.'));
            elseif ExitFlag==0;
               %
               % Reached maximum number of iteration
               %
               SetLogText(strvcat(' ','Maximum number of function evaluations or iteration reached.',[' * Time:                             ' StringTrim(datestr(now,14))],[' * Optimization time:                ' SecondsToHoursStr(MaxTime) ' (hrs:min:sec)']));
               if isempty(CurrINI.Sequence)==1;
                  txt = ['The maximum number of function evaluations or iterations was reached without fminunc converging (ExitFlag is 0). YADA has aborted from joint posterior mode estimation of the DSGE-VAR with lambda=' num2str(lambdai,'%0.4f') '. Writing optimization information to file: "OptErrorReport-Joint-DSGEVAR-' num2str(lambdai,'%0.4f') '-' int2str(p) '-' SelectedShockStr '-' DSGEModel.NameOfModel '.txt" in "' DSGEModel.OutputDirectory '".'];
                  About(txt,'information','Optimization Problem',200,500,CurrINI);
                  drawnow;
               end;
               ReturnCodeText = 'Fminunc return code: the maximum number of function evaluations or iteration was reached without fminunc coverging (ExitFlag is 0).';
            elseif ExitFlag<0;
               %
               % no convergence
               %
               SetLogText(strvcat(' ',['Fminunc didn''t converge (ExitFlag is ' int2str(ExitFlag) ').'],[' * Time:                             ' StringTrim(datestr(now,14))],[' * Optimization time:                ' SecondsToHoursStr(MaxTime) ' (hrs:min:sec)']));
               if isempty(CurrINI.Sequence)==1;
                  txt = ['Fminunc didn''t converge (ExitFlag is ' int2str(ExitFlag) '). YADA has aborted from joint posterior mode estimation of the DSGE-VAR with lambda=' num2str(lambdai,'%0.4f') '. Writing optimization information to file: "OptErrorReport-Joint-DSGEVAR-' num2str(lambdai,'%0.4f') '-' int2str(p) '-' SelectedShockStr '-' DSGEModel.NameOfModel '.txt" in "' DSGEModel.OutputDirectory '".'];
                  About(txt,'information','Optimization Problem',200,500,CurrINI);
                  drawnow;
               end;
               ReturnCodeText = ['Fminunc return code: No convergence (ExitFlag is ' int2str(ExitFlag) ').'];
            end;
         end;
      end;
      if ((EstimationStatus==0)&(ContinueEstimation==0))|(strcmp(lower(AbortEstimation),'cancel')==1);
         break;
      end;
      if FileExist(FixFilePath([pwd '\tmp\AiMErrorData.mat']))==1;
         delete(FixFilePath([pwd '\tmp\AiMErrorData.mat']));
      end;
      if (ContinueEstimation==1)&(EstimationStatus==1);
         %
         % Compute the partial derivatives of the original parameters wrt the transformed parameters, evaluated at the
         % marginal posterior mode
         %
         ThetaPhiPartial = PartialThetaPartialPhi(phiMode,thetaIndex,UniformBounds);
         %
         % compute the DSGE-VAR posterior mode
         %
         [A,H,R,F,B0] = SolveDSGEModel(DSGEModel,thetaMode,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
         [PhiMode,SigmaModeEpsilon,Gammayy,GammayY,GammaYY] = DSGEVARParameters(A,H,R,F,B0,lambdai,GammaHatyy,GammaHatyY,GammaHatYY,DetProductMoments,p,DSGEModel.UseDoublingAlgorithm,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,HSample);
         if isinf(lambdai)==0;
            SigmaModeEpsilon = (((1+lambdai)*T)/(((1+lambdai)*T)+n+1))*SigmaModeEpsilon;
         end;
         [A0Mode,IdentifyModeStatus] = DSGEVARIdentifyShocks(SigmaModeEpsilon,H,B0,DSGEModel.DSGEVARShocks,HSample);
         %
         % compute the inverse Hessian for the joint posterior
         %
         if isinf(lambdai)==0;
            [JInverseHessian,InvHessphi,InvHessStatus] = DSGEVARJointPosteriorInvHessian(phiMode,thetaIndex,UniformBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,DSGEModel,AIMData,lambdai,T,n,p,npk,GammaHatyy,GammaHatyY,GammaHatYY,DetProductMoments,HSample,StepLength,CurrINI.OrderQZ);
            if InvHessStatus==1;
               %
               % replace the inverse Hessian from the optimization routine
               %
               InverseHessian = InvHessphi;
            end;
         else;
            JInverseHessian = InverseHessian;
            InvHessStatus = 1;
         end;
         %
         % Check if we should compute the inverse Hessian with finite difference methods
         %
         if DSGEModel.FiniteDifferenceHessian==1;
            SetLogText(strvcat(' ','Computing posterior standard deviations from inverse Hessian','using finite differences...'));
            %
            % for the transformed parameters
            %
            VechHessian = YADAHessianMatrix('logConcPosteriorPhiDSGEVAR',phiMode,StepLength,thetaIndex,UniformBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,DSGEModel,AIMData,lambdai,T,n,p,npk,GammaHatyy,GammaHatyY,GammaHatYY,DetProductMoments,HSample,logCPC,CurrINI.OrderQZ,1);
            HessianMat = GeneralizedCholeski(reshape(VechHessian,NumParam,NumParam));
            if IsPositiveDefinite(HessianMat)==1;
               phiInverseHessianFinDiff = HessianMat\eye(NumParam);
            else;
               phiInverseHessianFinDiff = NaN;
            end;
            VechHessian = YADAHessianMatrix('logJointPosteriorPhiDSGEVAR',[vec(PhiMode);vech(SigmaModeEpsilon);phiMode],StepLength,thetaIndex,UniformBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,DSGEModel,AIMData,lambdai,T,n,p,npk,GammaHatyy,GammaHatyY,GammaHatYY,DetProductMoments,HSample,CurrINI.OrderQZ);
            if isinf(lambdai)==0;
               JInverseHessianFinDiff = GeneralizedCholeski(reshape(VechHessian,TotalParam,TotalParam));
            else;
               JInverseHessianFinDiff = GeneralizedCholeski(reshape(VechHessian,NumParam,NumParam));
            end;
            if rank(JInverseHessianFinDiff)==size(JInverseHessianFinDiff,1);
               if isinf(lambdai)==0;
                  JInverseHessianFinDiff = JInverseHessianFinDiff\eye(TotalParam);
                  if isnan(phiInverseHessianFinDiff)==1;
                     phiInverseHessianFinDiff = JInverseHessianFinDiff(TotalParam-NumParam+1:TotalParam,TotalParam-NumParam+1:TotalParam);
                  end;
               else;
                  JInverseHessianFinDiff = JInverseHessian\eye(NumParam);
                  if isnan(phiInverseHessianFinDiff)==1;
                     phiInverseHessianFinDiff = JInverseHessianFinDiff;
                  end;
               end;
               if isempty(JInverseHessian)==1;
                  JInverseHessian = JInverseHessianFinDiff;
                  InverseHessian = JInverseHessian(TotalParam-NumParam+1:TotalParam,TotalParam-NumParam+1:TotalParam);
               end;
               phiPostModeStdFinDiff = sqrt(diag(phiInverseHessianFinDiff));
            else;
               JInverseHessianFinDiff = NaN;
               phiPostModeStdFinDiff = NaN;
            end;
            %
            % for the original parameters
            %
            VechHessian = YADAHessianMatrix('logJointPosteriorThetaDSGEVAR',[vec(PhiMode);vech(SigmaModeEpsilon);thetaMode],StepLength,thetaIndex,ParameterBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,DSGEModel,AIMData,lambdai,T,n,p,npk,GammaHatyy,GammaHatyY,GammaHatYY,DetProductMoments,HSample,CurrINI.OrderQZ);
            if isinf(lambdai)==0;
               thetaJInverseHessianFinDiff = GeneralizedCholeski(reshape(VechHessian,TotalParam,TotalParam));
            else;
               thetaJInverseHessianFinDiff = GeneralizedCholeski(reshape(VechHessian,NumParam,NumParam));
            end;
            if rank(thetaJInverseHessianFinDiff)==size(thetaJInverseHessianFinDiff,1);
               if isinf(lambdai)==0;
                  thetaJInverseHessianFinDiff = thetaJInverseHessianFinDiff\eye(TotalParam);
                  thetaInverseHessianFinDiff = thetaJInverseHessianFinDiff(TotalParam-NumParam+1:TotalParam,TotalParam-NumParam+1:TotalParam);
               else;
                  thetaJInverseHessianFinDiff = thetaJInverseHessian\eye(NumParam);
                  thetaInverseHessianFinDiff = thetaJInverseHessianFinDiff;
               end;
               thetaPostModeStdFinDiff = sqrt(diag(thetaInverseHessianFinDiff));
            else;
               thetaJInverseHessianFinDiff = NaN;
               thetaInverseHessianFinDiff = NaN;
               thetaPostModeStdFinDiff = NaN;
            end;
         elseif (DSGEModel.FiniteDifferenceHessian==0)&(isinf(lambdai)==0)&(InvHessStatus==0);
            %
            % compute the inverse Hessian using finite differences
            %
            SetLogText(strvcat(' ','Bad inverse Hessian. Trying finite differences...'));
            VechHessian = YADAHessianMatrix('logConcPosteriorPhiDSGEVAR',phiMode,StepLength,thetaIndex,UniformBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,DSGEModel,AIMData,lambdai,T,n,p,npk,GammaHatyy,GammaHatyY,GammaHatYY,DetProductMoments,HSample,logCPC,CurrINI.OrderQZ,1);
            HessianMat = GeneralizedCholeski(reshape(VechHessian,NumParam,NumParam));
            if IsPositiveDefinite(HessianMat)==1;
               phiInverseHessianFinDiff = HessianMat\eye(NumParam);
            else;
               phiInverseHessianFinDiff = NaN;
            end;
            VechHessian = YADAHessianMatrix('logJointPosteriorPhiDSGEVAR',[vec(PhiMode);vech(SigmaModeEpsilon);phiMode],StepLength,thetaIndex,UniformBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,DSGEModel,AIMData,lambdai,T,n,p,npk,GammaHatyy,GammaHatyY,GammaHatYY,DetProductMoments,HSample,CurrINI.OrderQZ);
            JInverseHessian = reshape(VechHessian,TotalParam,TotalParam);
            if IsPositiveDefinite(JInverseHessian);
               JInverseHessian = JInverseHessian\eye(TotalParam);
               InverseHessian = JInverseHessian(TotalParam-NumParam+1:TotalParam,TotalParam-NumParam+1:TotalParam);
               SetLogText('STATUS: OK');
            else;
               JInverseHessian = [];
               SetLogText('STATUS: Failed');
            end;
            JInverseHessianFinDiff = NaN;
            phiPostModeStdFinDiff = NaN;
            thetaJInverseHessianFinDiff = NaN;
            thetaInverseHessianFinDiff = NaN;
            thetaPostModeStdFinDiff = NaN;
         else;
            %
            if isempty(CurrINI.Sequence)==1;
               if isempty(FiniteQuery)==1;
                  txt = 'Would you like to compute the inverse Hessian using finite differences and use it to estimate the marginal likelihood with the Laplace approximation?';
                  answer = Query(txt,'question',500,'Posterior Mode Estimation - Finite Differences',600,'yes',CurrINI);
                  FiniteQuery = answer;
               else;
                  answer = FiniteQuery;
               end;
            else;
               answer = CurrINI.Sequence.QueryAnswers{5};
            end;
            if strcmp(lower(answer),'yes')==1;
               SetLogText(strvcat(' ','Computing an inverse Hessian using finite differences...'));
               VechHessian = YADAHessianMatrix('logConcPosteriorPhiDSGEVAR',phiMode,StepLength,thetaIndex,UniformBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,DSGEModel,AIMData,lambdai,T,n,p,npk,GammaHatyy,GammaHatyY,GammaHatYY,DetProductMoments,HSample,logCPC,CurrINI.OrderQZ,1);
               HessianMat = GeneralizedCholeski(reshape(VechHessian,NumParam,NumParam));
               if IsPositiveDefinite(HessianMat)==1;
                  phiInverseHessianFinDiff = HessianMat\eye(NumParam);
                  SetLogText('* Matrix is positive definite.');
               else;
                  phiInverseHessianFinDiff = NaN;
                  SetLogText('* Matrix is not positive definite.');
               end;
            else;
               phiInverseHessianFinDiff = NaN;
            end;
            %
            JInverseHessianFinDiff = NaN;
            phiPostModeStdFinDiff = NaN;
            thetaJInverseHessianFinDiff = NaN;
            thetaInverseHessianFinDiff = NaN;
            thetaPostModeStdFinDiff = NaN;
         end;
         OldInverseHessian = InverseHessian;
         PDInvHess = 1;
         if (isnan(phiInverseHessianFinDiff)==1)&(IsPositiveDefinite(InverseHessian)==0);
            SetLogText(strvcat(' ','WARNING: The inverse Hessian at the joint posterior mode','         is not positive definite.'));
            DgInvHess = diag(InverseHessian);
            if length(DgInvHess(DgInvHess>0))==length(DgInvHess);
               if isempty(CurrINI.Sequence)==1;
                  txt = ['The inverse Hessian at the joint posterior mode is not positive definite for the DSGE-VAR model with lambda=' num2str(lambdai,'%0.4f') '. Do you want YADA to attempt to create a positive definite matrix from the inverse Hessian?'];
                  answer = Query(txt,'question',200,'YADA - Bad Inverse Hessian',500,'no',CurrINI);
               else;
                  answer = CurrINI.Sequence.QueryAnswers{3};
               end;
               if strcmp(lower(answer),'yes')==1;
                  SetLogText('         Attempting the create positive definite matrix...');
                  [InvHess,pdstatus] = CreatePositiveDefinite(InverseHessian);
                  if pdstatus==1;
                     SetLogText(strvcat('         STATUS: Successful. Replacing old inverse Hessian'));
                     InverseHessian = InvHess;
                  else;
                     SetLogText(strvcat('         STATUS: Failed. Keeping old inverse Hessian'));
                     PDInvHess = 0;
                     if isempty(CurrINI.Sequence)==1;
                        txt = strvcat(['Unable to create a positive definite inverse Hessian at the joint posterior for the DSGE-VAR model with lambda=' num2str(lambdai,'%0.4f') '. Would you like to skip this lambda-value?'],' ','NOTE: If you answer No, then YADA will abort.');
                        answer = Query(txt,'question',400,'YADA - Bad Inverse Hessian',500,'no',CurrINI);
                     else;
                        answer = CurrINI.Sequence.QueryAnswers{4};
                     end;
                     if strcmp(lower(answer),'yes')==1;
                        LaplaceMargLike(i) = -Inf;
                        SetLogText(strvcat(' ',['User decided to skip lambda=' num2str(lambdai,'%0.4f')],' '));
                     else;
                        SetLogText(strvcat(' ','Joint posterior mode estimation was aborted.'));
                        break;
                     end;
                  end;
               end;
            else;
               PDInvHess = 0;
               if isempty(CurrINI.Sequence)==1;
                  txt = strvcat(['The inverse Hessian at the joint posterior mode is not positive definite for the DSGE-VAR model with lambda=' num2str(lambdai,'%0.4f') '. Would you like to skip this lambda-value?'],' ','NOTE: If you answer No, then YADA will abort.');
                  answer = Query(txt,'question',400,'YADA - Bad Inverse Hessian',500,'no',CurrINI);
               else;
                  answer = CurrINI.Sequence.QueryAnswers{4};
               end;
               if strcmp(lower(answer),'yes')==1;
                  LaplaceMargLike(i) = -Inf;
                  SetLogText(strvcat(' ',['User decided to skip lambda=' num2str(lambdai,'%0.4f')],' '));
               else;
                  EstimationStatus = 0;
                  break;
               end;
            end;
         end;
         %
         if PDInvHess==1;
            %
            % compute the Laplace approximation of the log marginal likelihood for lambdai
            %
            if isnan(phiInverseHessianFinDiff)==0;
               LaplaceMargLike(i) = MargLikeLaplace(LogPostDensity,phiInverseHessianFinDiff,1)-LogMargLikeSystemLaplace;
               SetLogText(strvcat(' ','Computing Laplace approximation...',[' * Log marginal likelihood at mode:  ' num2str(LaplaceMargLike(i),'%0.4f')]));
            elseif isempty(JInverseHessian)==0;
               LaplaceMargLike(i) = MargLikeLaplace(LogPostDensity,JInverseHessian,1)-LogMargLikeSystemLaplace;
               SetLogText(strvcat(' ','Computing Laplace approximation...',[' * Log marginal likelihood at mode:  ' num2str(LaplaceMargLike(i),'%0.4f')]));
            else;
               LaplaceMargLike(i) = -Inf;
            end;
            %
            % Check if we should check the optimum
            %
            if (DSGEModel.CheckOptimum==1)&(IsPositiveDefinite(InverseHessian)==1);
               SetLogText(strvcat(' ','Checking the optimum...',' * Transformed parameters'));
               %
               % Check if GraphDirectory exists
               %
               GraphDirectory = FixFilePath([DSGEModel.OutputDirectory '\graphs']);
               MinMaxGrid = zeros(NumParam,2);
               stat = MakeDir(DSGEModel.OutputDirectory,'graphs');
               %
               % create lower and upper bounds from InverseHessian and phiMode
               %
               MinMaxGrid(:,1) = phiMode-(DSGEModel.GridWidth*sqrt(diag(InverseHessian)));
               MinMaxGrid(:,2) = phiMode+(DSGEModel.GridWidth*sqrt(diag(InverseHessian)));
               phiFittedStd = VillaniCheckOptimum('logJointPosteriorPhiDSGEVAR',phiMode,[vec(PhiMode);vech(SigmaModeEpsilon)],MinMaxGrid,InverseHessian,NumGridPoints,0,[],ParameterNames.estimated,'Checking Curvature at Joint Posterior Mode - Transformed Parameters',CurrINI,GraphDirectory,'phi',['DSGEVAR-' num2str(lambdai,'%0.4f') '-'],'Joint ',1,1, ...
                  thetaIndex,UniformBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,DSGEModel,AIMData,lambdai,T,n,p,npk,GammaHatyy,GammaHatyY,GammaHatYY,DetProductMoments,HSample,CurrINI.OrderQZ);
               if (DSGEModel.CheckTransformedOptimum==0)|(IsOdd(DSGEModel.MaximizeAlgorithmValue)==0);
                  SetLogText(' * Original parameters');
                  MinMaxGrid(:,1) = PhiToTheta(MinMaxGrid(:,1),thetaIndex,UniformBounds,LowerBound);
                  MinMaxGrid(:,2) = PhiToTheta(MinMaxGrid(:,2),thetaIndex,UniformBounds,LowerBound);
                  thetaFittedStd = VillaniCheckOptimum('logJointPosteriorThetaDSGEVAR',thetaMode,[vec(PhiMode);vech(SigmaModeEpsilon)],MinMaxGrid,ThetaPhiPartial*InverseHessian*ThetaPhiPartial,NumGridPoints,0,[],ParameterNames.estimated,'Checking Curvature at Joint Posterior Mode - Original Parameters',CurrINI,GraphDirectory,'theta',['DSGEVAR-' num2str(lambdai,'%0.4f') '-'],'Joint ',1,1, ...
                     thetaIndex,ParameterBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,DSGEModel,AIMData,lambdai,T,n,p,npk,GammaHatyy,GammaHatyY,GammaHatYY,DetProductMoments,HSample,CurrINI.OrderQZ);
               else;
                  thetaFittedStd = NaN;
               end;
            else;
               phiFittedStd = NaN;
               thetaFittedStd = NaN;
            end;
            if FileExist(FixFilePath([pwd '\tmp\AiMErrorData.mat']))==1;
               delete(FixFilePath([pwd '\tmp\AiMErrorData.mat']));
            end;
            %
            % check the parameters with most extreme correlations based on the inverse Hessian
            %
            ExtremeCorrBound = 0.7;
            CorrHessian = diag(1./sqrt(diag(InverseHessian)))*InverseHessian*diag(1./sqrt(diag(InverseHessian)));
            CorrHessian = tril(CorrHessian,-1);
            [RowIndex,ColIndex] = find(abs(CorrHessian)>=ExtremeCorrBound);
            if isempty(RowIndex)==0;
               ExtremeCorrParameters = cell(length(RowIndex),3);
               for j=1:length(RowIndex)
                  ExtremeCorrParameters{j,1} = ParameterNames.estimated(RowIndex(j),:);
                  ExtremeCorrParameters{j,2} = ParameterNames.estimated(ColIndex(j),:);
                  ExtremeCorrParameters{j,3} = CorrHessian(RowIndex(j),ColIndex(j));
               end;
            else;
               ExtremeCorrParameters = [];
            end;
            %
            % check other possible rank deficiencies with the inverse Hessian at the mode
            %
            [EigenVectorMatrix,EigenValues,ParameterOrdering] = DSGEIdentificationPatterns(CorrHessian);
            SmallEigenValues = EigenValues(EigenValues<0.01);
            if isempty(SmallEigenValues)==0;
               p1 = length(SmallEigenValues);
               p2 = size(EigenVectorMatrix,2);
               SmallEigenVectors = EigenVectorMatrix(:,p2-p1+1:p2);
            else;
               SmallEigenVectors = [];
            end;
            %
            % make sure that ModelParameters is updated
            %
            ModelParameters = ThetaToModelParameters(ModelParameters,thetaMode,thetaPositions);
            if FileExist(DSGEModel.UpdateParameterFile)==1;
               eval(['ModelParameters = ' GetName(DSGEModel.UpdateParameterFile) '(ModelParameters);']);
            end;
            %
            % get the mode values of any additional parameters
            %
            [thetaCalibratedMode,thetaAdditionalMode] = ReadAdditionalParameterValues(ModelParameters,ParameterNames);
            LogLikeDensity = LogLikelihoodMode;
            LogPriorDensity = LogPriorMode;
            LogJacobianValue = LogJacobianMode;
            LaplaceMargLikeLambda = LaplaceMargLike(i);
            %
            % save output to file
            %
            stat = MakeDir(DSGEModel.OutputDirectory,'mode');
            if stat==1;
               %
               % make some temporary name changes
               %
               TmpA0PriorMode = A0PriorMode;
               TmpIdentifyPriorModeStatus = IdentifyPriorModeStatus;
               if IdentifyPriorModeStatus(i)==1;
                  A0PriorMode = A0PriorMode(:,:,i);
               else;
                  A0PriorMode = [];
               end;
               IdentifyPriorModeStatus = IdentifyPriorModeStatus(i);
               SetLogText(strvcat(' ','Saving joint posterior mode data to disk using directory:',['   "' ModeFileDir '"...']));
               ModeFile = FixFilePath([ModeFileDir '\JointPosteriorMode-DSGEVAR-' num2str(lambdai,'%0.4f') '-' int2str(p) '-' SelectedShockStr '-' DSGEModel.NameOfModel '.mat']);
               save(ModeFile,'phiMode','thetaMode','phi','theta','phiEstimates','thetaCalibrated','thetaAdditionalInitial','thetaAdditionalMode','ParameterNames','ExtremeCorrParameters','LogPostDensity','LogLikeDensity',...
                    'LogPriorDensity','LogJacobianValue','InverseHessian','OldInverseHessian','ThetaPhiPartial','thetaIndex','LowerBound','thetaPositions','thetaDist','PriorDist','ModelParameters','ParameterBounds','UniformBounds',...
                    'DSGEModel','AIMData','phiInverseHessianFinDiff','phiPostModeStdFinDiff','thetaInverseHessianFinDiff','thetaPostModeStdFinDiff','phiFittedStd','thetaFittedStd','JInverseHessian','JInverseHessianFinDiff', ...
                    'thetaJInverseHessianFinDiff','KalmanFirstObservation','FirstPeriod','LastPeriod','SmallEigenValues','SmallEigenVectors','LaplaceMargLikeLambda','PhiMode','SigmaModeEpsilon','A0Mode','IdentifyModeStatus', ...
                    'T','GammaHatyy','GammaHatyY','GammaHatYY','DetProductMoments','HSample','PhiPriorMode','A0PriorMode','IdentifyPriorModeStatus','n','p','k','lambdai','LogSPIV','LogSPMode','LogMargLikeSystemLaplace');
               %
               A0PriorMode = TmpA0PriorMode;
               IdentifyPriorModeStatus = TmpIdentifyPriorModeStatus;
            end;
            %
            % write output to file
            %
            ModeTextFile = FixFilePath([DSGEModel.OutputDirectory '\JointPosteriorModeResults-DSGEVAR-' num2str(lambdai,'%0.4f') '-' int2str(p) '-' SelectedShockStr '-' DSGEModel.NameOfModel '.txt']);
            SetLogText(strvcat(' ','Writing joint posterior mode estimation results to text file:',['   Directory: ' DSGEModel.OutputDirectory],['   File name: JointPosteriorModeResults-DSGEVAR-' num2str(lambdai,'%0.4f') '-' int2str(p) '-' SelectedShockStr '-' DSGEModel.NameOfModel '.txt']));
            PrintDSGEVARPosteriorModeResults(ModeTextFile,DSGEModel,CurrINI,ParameterNames,phiMode,thetaMode,phi,theta,thetaCalibrated,thetaAdditionalInitial,thetaAdditionalMode,thetaDist,PriorDist,ThetaPhiPartial,InverseHessian,phiFittedStd,phiPostModeStdFinDiff,thetaFittedStd,thetaPostModeStdFinDiff,LogPostDensity,LogLikeDensity,LogPriorDensity,LogJacobianValue,ExtremeCorrParameters,SmallEigenValues,SmallEigenVectors,OptAlgorithm,OptParameters,MaxTime,ToleranceValue,MaxIter,NumIter,FuncDiff,OptResults,ScreenMessages,LogPosteriorPhiIV(i),LogPosteriorThetaIV(i),LogPriorIV,LaplaceMargLikeLambda,PhiMode,A0Mode,IdentifyModeStatus,PhiPriorMode,A0PriorMode(:,:,i),IdentifyPriorModeStatus(i),n,p,k,lambdai,LogSPIV,LogSPMode,IsSystemPrior,LogMargLikeSystemLaplace);
            %
            % and now a summary
            %
            ModeTextFile = FixFilePath([DSGEModel.OutputDirectory '\JointPosteriorModeSummary-DSGEVAR-' num2str(lambdai,'%0.4f') '-' int2str(p) '-' SelectedShockStr '-' DSGEModel.NameOfModel '.txt']);
            SetLogText(strvcat(' ','Writing joint posterior mode estimation summary results to text file:',['   Directory: ' DSGEModel.OutputDirectory],['   File name: JointPosteriorModeSummary-DSGEVAR-' num2str(lambdai,'%0.4f') '-' int2str(p) '-' SelectedShockStr '-' DSGEModel.NameOfModel '.txt']));
            PrintDSGEVARPosteriorModeSummary(ModeTextFile,DSGEModel,CurrINI,ParameterNames,thetaMode,theta,thetaDist,PriorDist,ThetaPhiPartial,InverseHessian,phiFittedStd,thetaFittedStd,LogPostDensity,LogLikeDensity,LogPriorDensity,LogJacobianValue,ExtremeCorrParameters,SmallEigenValues,SmallEigenVectors,OptAlgorithm,OptParameters,MaxTime,ToleranceValue,MaxIter,NumIter,FuncDiff,max(GradientVector),LogPosteriorPhiIV(i),LogPosteriorThetaIV(i),LogPriorIV,LaplaceMargLikeLambda,n,p,k,lambdai,LogSPIV,LogSPMode,IsSystemPrior,LogMargLikeSystemLaplace);
         end;
      elseif (ContinueEstimation==1)&(EstimationStatus==0);;
         LaplaceMargLike(i) = -Inf;
      end;
   else;
      %
      % we have already estimated the DSGE-VAR for lambdai and want to use the old result
      %
      EstimationStatus = 1;
      LaplaceMargLike(i) = ResultsExist(i);
   end;
end;
%
% check if we should write and optimization error report
%
if (EstimationStatus==0)&(strcmp(lower(AbortEstimation),'cancel')==0);
   SetLogText(strvcat('Writing optimization error summary to disk.',['   Directory: ' DSGEModel.OutputDirectory],['   File name: OptErrorReport-Joint-DSGEVAR-' num2str(lambdai,'%0.4f') '-' int2str(p) '-' SelectedShockStr '-' DSGEModel.NameOfModel '.txt']));
   %
   % transform phiMode to thetaMode taking into account that certain entries may be Inf or Nan's
   %
   NotFinite = 1-isfinite(phiMode);
   if sum(NotFinite)>=1;
      HelpIndex = (1:NumParam)';
      SwapParams = (NotFinite .* HelpIndex);
      SwapParams = SwapParams(SwapParams>0);
      phiModeOld = phiMode;
      phiMode(SwapParams) = 1;
   end;
   thetaMode = PhiToTheta(phiMode,thetaIndex,UniformBounds,LowerBound);
   %
   % compute the partial of theta
   %
   phiGradientVector = abs((PartialThetaPartialPhi(phiMode,thetaIndex,UniformBounds)\eye(NumParam))*GradientVector);
   %
   % Now we have to reset all those elelemts that were either NaN or Inf
   %
   if sum(NotFinite)>=1;
      thetaMode(SwapParams) = phiModeOld(SwapParams);
      phiGradientVector(SwapParams) = abs(phiModeOld(SwapParams));
   end;
   %
   % check what we can save to mat-file
   %
   if IsOdd(DSGEModel.MaximizeAlgorithmValue)==1;
      LogPostDensity = -fh;
   else;
      LogPostDensity = -fh+logJacobian(phiMode,thetaIndex,UniformBounds);
   end;
   if (DSGEModel.MaximizeAlgorithmValue==7)|(DSGEModel.MaximizeAlgorithmValue==8);
      if (isempty(HessianMatrix)==0)&(rank(HessianMatrix)==size(HessianMatrix,2));
         InverseHessian = HessianMatrix\eye(length(phiMode));
      else;
         InverseHessian = [];
      end;
      phiEstimates = OUTPUT.phiEstimates;
   end;
   %
   % see if we should check the optimum
   %
   if (DSGEModel.CheckOptimum==1)&(isempty(InverseHessian)==0)&(IsPositiveDefinite(InverseHessian)==1);
      %
      % compute the DSGE-VAR posterior mode approximation
      %
      ModelSolved = 1;
      try;
         [A,H,R,F,B0] = SolveDSGEModel(DSGEModel,thetaMode,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
         [PhiMode,SigmaModeEpsilon,Gammayy,GammayY,GammaYY] = DSGEVARParameters(A,H,R,F,B0,lambdai,GammaHatyy,GammaHatyY,GammaHatYY,DetProductMoments,p,DSGEModel.UseDoublingAlgorithm,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,HSample);
         if isinf(lambdai)==0;
            SigmaModeEpsilon = (((1+lambdai)*T)/(((1+lambdai)*T)+n+1))*SigmaModeEpsilon;
         end;
         [A0Mode,IdentifyModeStatus] = DSGEVARIdentifyShocks(SigmaModeEpsilon,H,B0,DSGEModel.DSGEVARShocks,HSample);
      catch;
         ModelSolved = 0;
      end;
      %
      if ModelSolved==1;
         %
         % Check if GraphDirectory exists
         %
         GraphDirectory = FixFilePath([DSGEModel.OutputDirectory '\graphs']);
         MinMaxGrid = zeros(NumParam,2);
         stat = MakeDir(DSGEModel.OutputDirectory,'graphs');
         %
         % create lower and upper bounds from InverseHessian and phiMode
         %
         MinMaxGrid(:,1) = phiMode-(DSGEModel.GridWidth*sqrt(diag(InverseHessian)));
         MinMaxGrid(:,2) = phiMode+(DSGEModel.GridWidth*sqrt(diag(InverseHessian)));
         try;
            phiFittedStd = VillaniCheckOptimum('logJointPosteriorPhiDSGEVAR',phiMode,[vec(PhiMode);vech(SigmaModeEpsilon)],MinMaxGrid,InverseHessian,NumGridPoints,0,[],ParameterNames.estimated,'Checking Curvature Of Log Joint Posterior - Transformed Parameters',CurrINI,GraphDirectory,'phi',['DSGEVAR-' num2str(lambdai,'%0.4f') '-'],'Joint ',1,1, ...
               thetaIndex,UniformBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,DSGEModel,AIMData,lambdai,T,n,p,npk,GammaHatyy,GammaHatyY,GammaHatYY,DetProductMoments,HSample,CurrINI.OrderQZ);
         catch;
            phiFittedStd = NaN;
         end;
      else;
         phiFittedStd = NaN;
      end;
   else;
      phiFittedStd = NaN;
   end;
   %
   % Save to disk
   %
   ModeFileDir = FixFilePath([DSGEModel.OutputDirectory '\mode']);
   stat = MakeDir(DSGEModel.OutputDirectory,'mode');
   if stat==1;
      SetLogText(strvcat(' ','Saving optimization data to disk using directory:',['   "' ModeFileDir '"...']));
      ModeFile = FixFilePath([ModeFileDir '\PosteriorModeError-Joint-DSGEVAR-' num2str(lambdai,'%0.4f') '-' int2str(p) '-' SelectedShockStr '-' DSGEModel.NameOfModel '.mat']);
      save(ModeFile,'phiMode','thetaMode','phi','theta','phiEstimates','ParameterNames','LogPostDensity','InverseHessian','phiFittedStd');
   end;
   %
   % print an error report
   %
   TxtFileName = FixFilePath([DSGEModel.OutputDirectory '\OptErrorReport-Joint-DSGEVAR-' num2str(lambdai,'%0.4f') '-' int2str(p) '-' SelectedShockStr '-' DSGEModel.NameOfModel '.txt']);
   PrintOptimizationErrorReport(TxtFileName,p,lambdai,DSGEModel,CurrINI,AIMData,ParameterNames,PriorDist,thetaDist,thetaMode,theta,phiGradientVector,GradientVector,rank(R),OptResults,ScreenMessages,ReturnCodeText);
   %
   % display the file
   %
   if isempty(CurrINI.Sequence)==1;
      TextGUI(TxtFileName,['Optimization Error Summary - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 32 1000 CurrINI.scrsz(4)-100],'Results',50,CurrINI,1,1);
   end;
   return;
elseif strcmp(lower(AbortEstimation),'cancel')==1;
   %
   % user has chosen to abort
   %
   return;
end;
%
% check which lambda obtains the highest marginal likelihood
%
[MaxMargLike,Position] = max(LaplaceMargLike);
MaxLambda = lambda(Position);
%
% save results to file
%
MatFile = FixFilePath([ModeFileDir '\JointPosteriorMode-DSGEVAR-MaxMargLike-' int2str(p) '-' SelectedShockStr '-' DSGEModel.NameOfModel '.mat']);
save(MatFile,'MaxLambda','LaplaceMargLike','lambda');
%
% check if we can compute the Laplace approximation at the posterior mode of the
% DSGE model
%
if isempty(CurrINI.Sequence)==1;
   ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
   if FileExist(ModeFile)==1;
      BadMode = 0;
      try;
         ModeData = load(ModeFile);
      catch;
         BadMode = 1;;
      end;
      if BadMode==0;
         phiDSGE = ModeData.phiMode;
         SigmaDSGE = ModeData.InverseHessian;
         if (isnan(ModeData.phiInverseHessianFinDiff)==0)|((DSGEModel.InverseHessianEstimator==3)&(sum(sum(isnan(ModeData.phiInverseHessianFinDiff)))==0)&(IsPositiveDefinite(ModeData.phiInverseHessianFinDiff)==1));
            SigmaDSGE = ModeData.phiInverseHessianFinDiff;
         end;
         %
         % set the InitialStateVector value
         %
         if DSGEModel.UseOwnInitialState==1;
            if length(DSGEModel.InitialStateValues)==size(AIMData.endog,1);
               DSGEModel.InitialStateVector = DSGEModel.InitialStateValues(DSGEModel.StateVariablePositions);
            else;
               DSGEModel.InitialStateVector = zeros(length(DSGEModel.StateVariablePositions),1);
            end;
         else;
            DSGEModel.InitialStateVector = zeros(length(DSGEModel.StateVariablePositions),1);
         end;
         %
         % compute the value of the log-posterior at the mode for the estimation sample and the
         %
         logPost = -logPosteriorPhiDSGE(phiDSGE,thetaIndex,UniformBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,DSGEModel,AIMData,CurrINI.OrderQZ,CurrINI.NumWorkers);
         %
         % compute the Laplace approximation
         %
         if isfield(ModeData,'LogMargLikeSystemLaplace')==1;
            LogMargLikeSystemLaplaceDSGE = ModeData.LogMargLikeSystemLaplace;
         else;
            LogMargLikeSystemLaplaceDSGE = LogMargLikeSystemLaplace;
         end;
         LaplaceDSGE = MargLikeLaplace(logPost,SigmaDSGE,1)-LogMargLikeSystemLaplaceDSGE;
      else;
         LaplaceDSGE = [];
      end;
   else;
      LaplaceDSGE = [];
   end;
   %
   % check which marginal likelihood values we shall use
   %
   tau = lambda./(1+lambda);
   tau(length(tau)) = 1;
   UsePos = isfinite(LaplaceMargLike).*(1:length(LaplaceMargLike));
   UsePos = UsePos(UsePos>0);
   if (isempty(UsePos)==0)&(length(UsePos)+(~isempty(LaplaceDSGE))>=2);
      lambda = lambda(UsePos);
      tau = tau(UsePos);
      LaplaceMargLike = LaplaceMargLike(UsePos);
      if isempty(LaplaceDSGE)==0;
         LaplaceMargLike = [LaplaceMargLike LaplaceDSGE];
      end;
      %
      % plot the marginal likelihood values against lambda/(1+lambda)
      %
      FigHandle = figure('Visible','off', ...
                         'Position',[(CurrINI.scrsz(3)-800)/2 (CurrINI.scrsz(4)-500)/2 800 500], ...
                         'Tag','MargLike', ...
                         'Name','Marginal Likelihood For DSGE-VARs - Laplace Approximation');
      %
      SetFigureProperties(FigHandle,CurrINI);
      %
      if isempty(LaplaceDSGE)==0;
         plot([tau 1.05],LaplaceMargLike,'x-','Color',[0.8 0 0],'LineWidth',2);
      else;
         plot(tau,LaplaceMargLike,'x-','Color',[0.8 0 0],'LineWidth',2);
      end;
      hold('on');
      axis('tight');
      %
      % fix the y-axis
      %
      ylim = get(gca,'YLim');
      AddOn = 0.1*(ylim(2)-ylim(1));
      set(gca,'YLim',ylim+[-AddOn AddOn]);
      %
      % fix the tick and ticklabel for the x-axis
      %
      XTickLabel = '';
      for j=1:length(lambda);
         if IsInteger(1000*lambda(j))==1;
            XTickLabel = strvcat(XTickLabel,num2str(lambda(j)));
         else;
            XTickLabel = strvcat(XTickLabel,num2str(lambda(j),'%0.2f'));
         end;
      end;
      LambdaValues = tau;
      if length(tau)>10;
         AvDiff = ceil(length(tau)/10);
         UseEntry = (1:AvDiff:length(tau));
         if UseEntry(length(UseEntry))<length(tau);
            if length(UseEntry)==10;
               UseEntry = [UseEntry(1:9) length(tau)];
            else;
               UseEntry = [UseEntry length(tau)];
            end;
         end;
         LambdaValues = LambdaValues(UseEntry);
         XTickLabel = XTickLabel(UseEntry,:);
      end;
      if isempty(LaplaceDSGE)==0;
         XTickLabel = strvcat(XTickLabel,'DSGE');
         LambdaValues = [LambdaValues 1.05];
      end;
      XTickLabel = cellstr(XTickLabel);
      set(gca,'XTick',LambdaValues,'XTickLabel',XTickLabel);
      %
      SetAxesFonts(CurrINI,'Title','Laplace approximation for DSGE-VAR(\lambda) models','XLabel','\lambda','YLabel','Log marginal likelihood');
      %
      % save plot to disk
      %
      if DirExist(GraphDirectory)==1;
         saveas(FigHandle,FixFilePath([GraphDirectory '\LogMarginalLikelihood-LaplaceApproximation-JointPostMode-' int2str(p) '-' SelectedShockStr '-' DSGEModel.NameOfModel '-DSGEVAR.fig']));
      end;
      %
      % display the graph
      %
      SetToolbarImages(FigHandle,CurrINI.images);
      set(FigHandle,'Visible','on','CloseRequestFcn','delete(gcf); drawnow; pause(0.02);');
      drawnow;
      pause(0.02);
   end;
   %
   % ask if the full results or a summary should be displayed
   %
   txt = 'Would you like to view only a summary of the joint posterior mode estimation output? [No means all printed results]';
   answer = Query(txt,'question',120,'Output Selection',500,'yes',CurrINI);
   drawnow;
   if strcmp(lower(answer),'yes')==1;
      ModeTextFile = FixFilePath([DSGEModel.OutputDirectory '\JointPosteriorModeSummary-DSGEVAR-' num2str(MaxLambda,'%0.4f') '-' int2str(p) '-' SelectedShockStr '-' DSGEModel.NameOfModel '.txt']);
   else;
      ModeTextFile = FixFilePath([DSGEModel.OutputDirectory '\JointPosteriorModeResults-DSGEVAR-' num2str(MaxLambda,'%0.4f') '-' int2str(p) '-' SelectedShockStr '-' DSGEModel.NameOfModel '.txt']);
   end;
   %
   % display the results
   %
   TextGUI(ModeTextFile,['DSGE-VAR Joint Posterior Mode Estimation - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 32 1000 CurrINI.scrsz(4)-100],'Results',50,CurrINI,1,1);
   if sum(UseOldResults)==length(UseOldResults);
      SetLogText(strvcat(' ','All done!'));
   else;
      SetLogText('All done!');
   end;
else;
   SetLogText(' ');
end;

%
% end of DSGEVARPosteriorModeEstimation.m
%
