function [LogPredLike,StdErr,PredVars,MinMaxCondLike,TestStd] = DSGEVARPredLikeMCI(SelectedYData,NumDraws,h,IsOriginal,IsMarginal,IsPosterior,Weights,lambda,PME,CurrChain,DSGEModel,controls)
% DSGEVARPredLikeMCI: Computes the predictive likelihood and the marginal likelihood for parameter draws using Monte
%                     Carlo integration for a DSGE-VAR model.
%
% USAGE:
%
%       [LogPredLike,StdErr,PredVars,MinMaxCondLike,TestStd] = DSGEVARPredLikeMCI(SelectedYData,NumDraws,h,IsOriginal,...
%                                                                           IsMarginal,IsPosterior,Weights,lambda,PME,...
%                                                                           CurrChain,DSGEModel,controls)
%
% REQUIRED INPUT:  SelectedYData (integer) with the selected set of actuals for the observed variables.
%
%                  NumDraws (integer) with the number of draws used.
%
%                  h (integer) with the length of the prediction horizon.
%
%                  IsOriginal (boolean) that takes the value 1 if original values should be predicted and 0 if
%                             annualized values should be ptredicted.
%
%                  IsMarginal (boolean) that takes the value 1 if the marginal predictive likelihood should be
%                             computed and 0 if the joint predictive likelihood is the object of interest.
%
%                  IsPosterior (boolean) that takes the value 1 if the parameter draws are taken from the posterior
%                             distribution and 0 otherwise.
%
%                  Weights (vector) of dimension NumDraws x 1 with the importance weights from the SMC with
%                             likelihood tempering posterior sampler.
%
%                  lambda (real) hyperparameter value that is used to measure degree of misspecification.
%
%                  PME (integer) that takes the value 1 if the DSGE model posterior mode estimates
%                             are used, 2 if the DSGE-VAR marginal posterior mode estimates were
%                             used, and 3 if the DSGE-VAR joint posterior mode estimates were used
%                             when generating the posterior draws.
%
%                  CurrChain (integer) for the selected MCMC chain when posterior draws are used.
%
%                  DSGEModel (structure) with fields containing all the information about a DSGE Model for YADA.
%
%                  controls (structure) whose fields provide handles to all the UI controls on YADA's
%                             main dialog.
%
% REQUIRED OUTPUT: LogPredLike (vector) of dimension h with the predictive likelihood for the forecast
%                             periods.
%
%                  StdErr (vector) of dimension h with the numerical standard errors of the log predictive
%                             likelihood.
%
%                  PredVars (cell array) with the positions of the predicted variables in the vector of observed
%                             variables for each prediction period.
%
%                  MinMaxCondLike (matrix) of dimension hx3 with the mean, the minimum, and the maximum of
%                             the log conditional likelihood in the first to third columns.
%
%                  TestStd (boolean) that takes the value 1 if the numerical standard errors are robust to small
%                             values for the conditional likelihood and 0 if not.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   DG-R/MPR
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2017 European Central Bank.
%
%                       First version: May 14, 2012.
%                        This version: September 11, 2017.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 15-05-2012: Updated the documentation.
%
% * 21-08-2012: Changed estimator to an importance sampling estimator.
%
% * 18-10-2012: Added numerical standard errors based on Newey-West.
%
% * 04-01-2013: Updated the documentation.
%
% * 28-01-2014: Updated the documentation.
%
% * 14-01-2015: Updated the documentation.
%
% * 26-02-2015: Added the output variables "MinMaxCondLike" and "TestStd". Changed the file name from
%               "DSGEVARPredLikeImportanceSampling" to "DSGEVARPredLikeMCI".
%
% * 15-06-2015: The function now takes the input variable "SelectedYData".
%
% * 21-01-2016: Updated the documentation.
%
% * 21-03-2017: Updated the documentation.
%
% * 11-09-2017: Added the input variable "Weights".
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize
%
LogPredLike = [];
PredVars = [];
StdErr = [];
MinMaxCondLike = [];
TestStd = 0;
if isempty(Weights)==1;
   Weights = ones(NumDraws,1);
end;
%
%NumProb = length(CovProb);
%[NumDraws,NumParam] = size(PostSample);
SelectedVarStr = ismember((1:DSGEModel.n),DSGEModel.PredictionVariables);
SelectedVarStr = strrep(num2str(SelectedVarStr),' ','');
if IsOriginal==1;
   TypeStr = 'Original';
else;
   TypeStr = 'Annualized';
end;
if IsMarginal==1;
   MargStr = 'Marginal';
else;
   MargStr = 'Joint';
end;
if IsPosterior==1;
   DistStr = 'Posterior';
else;
   DistStr = 'Prior';
end;
DrawsStr = get(controls.posterior.draws,'String');
TotalDraws = str2num(StringTrim(strrep(DrawsStr(DSGEModel.PosteriorDrawsValue,:),',','')));
SelectedVarStr = ismember((1:DSGEModel.n),DSGEModel.PredictionVariables);
SelectedVarStr = strrep(num2str(SelectedVarStr),' ','');
if IsPosterior==1;
   file = FixFilePath([DSGEModel.OutputDirectory '\predictions\' MargStr 'DSGEVARPredictiveLikelihood-' num2str(lambda,'%0.4f') '-' int2str(DSGEModel.BVARLags) '-' int2str(PME) '-' DSGEModel.NameOfModel '-' int2str(SelectedYData) '-' TypeStr '-' SelectedVarStr '-' int2str(h) '-' int2str(NumDraws) '.' int2str(TotalDraws) '-' int2str(CurrChain) '.mat']);
else;
   file = FixFilePath([DSGEModel.OutputDirectory '\predictions\' MargStr 'DSGEVARPredictiveLikelihood-' num2str(lambda,'%0.4f') '-' int2str(DSGEModel.BVARLags) '-' int2str(PME) '-' DSGEModel.NameOfModel '-' int2str(SelectedYData) '-' TypeStr '-' SelectedVarStr '-' int2str(h) '-' int2str(NumDraws) '.mat']);
end;
PredData = load(file);
PredVars = PredData.PredVars;
LogPosteriorData = PredData.LogPosteriorData;
clear('PredData');
%
% compute the predictive likelihood as the average of the conditional likelihood evaluated at the
% posterior values of the parameters.
%
h = size(LogPosteriorData,2);
Weightsh = Weights*ones(1,h);
meanLPD = sum(LogPosteriorData.*Weightsh,1)/NumDraws;
MinMaxCondLike = [meanLPD;min(LogPosteriorData);max(LogPosteriorData)]';
LPD = LogPosteriorData-(ones(NumDraws,1)*meanLPD);
%
LogPredLike = (log(sum(exp(LPD).*Weights)/NumDraws)+meanLPD)';
%
%LogPosteriorData = exp(LogPosteriorData);
%PredLike = sum(LogPosteriorData)'/NumDraws;
%LogPredLike = log(PredLike);
%
% compute numerical standard errors with Newey-West
%
StdErrMat = NeweyWestCovMat(exp(LPD),floor(NumDraws^(1/2.01)));
StdErr = (sqrt(diag(StdErrMat)).*exp(meanLPD'))./exp(LogPredLike);
%
% now test this against the original data
%
StdErrMat = NeweyWestCovMat(exp(LogPosteriorData),floor(NumDraws^(1/2.01)));
StdErrCheck = sqrt(diag(StdErrMat))./exp(LogPredLike);
TestStd = min(abs(StdErr-StdErrCheck)<1.0e-10);
%LogPosteriorData = exp(LogPosteriorData);
%PredLike = sum(LogPosteriorData)'/NumDraws;
%LogPredLike = log(PredLike);
%%
%% compute numerical standard errors with Newey-West
%%
%StdErrMat = NeweyWestCovMat(LogPosteriorData,floor(NumDraws^(1/2.01)));
%StdErr = sqrt(diag(StdErrMat))./PredLike;

%
% end of DSGEVARPredLikeMCI.m
%
