function [y,Y,HSample,xx,XX] = GetDSGEVARData(DSGEModel)
% GetDSGEVARData: Returns the matrices with data on endogenous, exogenous and lagged
%                 endogenous variables for the DSGE-VAR model.
%
% USAGE:
%
%       [y,Y,HSample,xx,XX] = GetDSGEVARData(DSGEModel)
%
% REQUIRED INPUT:  DSGEModel (structure) whose fields contain all required information
%                            about the model, such as input files, variable and sample
%                            data. Actual observations are not stored in this file.
%
% REQUIRED OUTPUT: y (matrix) of dimension nxT with the data on the endogenous variables
%                            for the DSGE-VAR.
%
%                  Y (matrix) of dimension (np+k)xT with the data on the exogenous and
%                            lagged endogenous variables of the DSGE-VAR.
%
%                  HSample (vector) of dimension 1x2 with integer giving the first and
%                            last observation in the sample that are used. The integers
%                            are equal to the columns in DSGEModel.Y where the first and
%                            last observation are taken from.
%
% OPTIONAL OUTPUT: xx (matrix) of dimension kxT with data on the exogenous variables.
%
%                  XX (matrix) of dimension kx(T+p) with data on the exogenous variables.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   DG-R/MPR
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2017 European Central Bank.
%
%                       First version: February 18, 2010.
%                        This version: February 10, 2017.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 19-02-2010: Updated the documentation.
%
% * 27-02-2010: Removed the optional yInitial output variable.
%
% * 16-06-2010: Updated the documentation.
%
% * 05-01-2011: Updated the documentation.
%
% * 03-01-2012: Updated the documentation.
%
% * 03-01-2013: Updated the documentation.
%
% * 27-01-2014: Updated the documentation.
%
% * 13-01-2015: Updated the documentation.
%
% * 21-01-2016: Updated the documentation.
%
% * 10-02-2017: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% collect some parameters for dimensions
%
p = DSGEModel.BVARLags;
k = DSGEModel.k;
n = DSGEModel.n;
%
% create data matrices
%
[FirstPeriod,LastPeriod] = CreateSubSample(DSGEModel);
%
% Determine the first observation to use when computing log-likelihood
%
KalmanFirstObservation = DSGEModel.KalmanFirstObservation;
%
% Check if we should adjust this based on p, the lag order
%
if FirstPeriod+KalmanFirstObservation-1<p+1;
   KalmanFirstObservation = p+2-FirstPeriod;
end;
%
% actual sample that we will use
%
[NewStartYear,NewStartPeriod] = AdjustSampleStart(DSGEModel.SubBeginYear,DSGEModel.SubBeginPeriod,DSGEModel.DataFrequency,KalmanFirstObservation-1);
%
% Setup all the basic data matrices
%
y = DSGEModel.Y(:,FirstPeriod+KalmanFirstObservation-1:LastPeriod);
if k>0
   x = DSGEModel.X(:,FirstPeriod+KalmanFirstObservation-1:LastPeriod);
   X = DSGEModel.X(:,FirstPeriod+KalmanFirstObservation-(p+1):LastPeriod);
else;
   x = [];
   X = [];
end;
T = size(y,2);
Y = zeros((n*p),T);
for i=1:p;
   Y(1+(n*(i-1)):n*i,:) = DSGEModel.Y(:,FirstPeriod+KalmanFirstObservation-1-i:LastPeriod-i);
end;
Y = [x;Y];
HSample = [FirstPeriod+KalmanFirstObservation-1 LastPeriod];
if nargout>3;
   xx = x;
   XX = X;
end;

%
% end of GetDSGEVARData.m
%
