function [status,Lambda,Files] = VerifyVARDSGEPriorDraws(DSGEModel)
% VerifyVARDSGEPriorDraws: checks if prior draws of the VAR parameters exists for a DSGE-VAR model.
%
% USAGE:
%
%       [status,Lambda,Files] = VerifyVARDSGEPriorDraws(DSGEModel)
%
% REQUIRED INPUT:  DSGEModel (structure) whose fields contain all required information about the
%                            model, such as input files, variable and sample data. Actual observations
%                            are not stored in this file.
%
% REQUIRED OUTPUT: status (boolean) that takes the value 1 if prior draws of the VAR parameters exist
%                            for at least one lambda value
%
% OPTIONAL OUTPUT: Lambda (vector) with the values of the hyperparameter lambda for which prior draws
%                            exist.
%
%                  Files (string matrix) whose rows hold the path and filename of the files with the
%                            prior draws.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   DG-R/MPR
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2017 European Central Bank.
%
%                       First version: April 16, 2010.
%                        This version: February 10, 2017.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 05-01-2011: Updated the documentation.
%
% * 11-01-2011: The function now supports unix and mac paths.
%
% * 03-01-2012: Updated the documentation.
%
% * 03-01-2013: Updated the documentation.
%
% * 27-01-2014: Updated the documentation.
%
% * 13-01-2015: Updated the documentation.
%
% * 21-01-2016: Updated the documentation.
%
% * 10-02-2017: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize
%
status = 0;
if nargout>1;
   Lambda = [];
   Files = '';
end;
%
% determine string with 1's and 0's for the selected shocks
%
SelectedShockStr = ismember((1:length(DSGEModel.StateShockPositions)),DSGEModel.DSGEVARShocks);
SelectedShockStr = strrep(num2str(SelectedShockStr),' ','');
p = DSGEModel.BVARLags;
%
lambda = GetLambda(DSGEModel);
for i=1:length(lambda);
   lambdai = lambda(i);
   MatFile = FixFilePath([DSGEModel.OutputDirectory '\priordraws\PriorDraws-DSGEVAR-' num2str(lambdai,'%0.4f') '-' int2str(p) '-' SelectedShockStr '-' DSGEModel.NameOfModel '.mat']);
   if FileExist(MatFile)==1;
      status = 1;
      if nargout>1;
         Lambda = [Lambda lambdai];
         Files = strvcat(Files,MatFile);
      end;
   end;
   if (status==1)&(nargout==1);
      break;
   end;
end;

%
% end of VerifyVARDSGEPriorDraws.m
%
