function StructureForData = DataConstFile2x
% DataConstFile: Shows how the data construction output should be provided for YADA. This example is based
%                on the An-Schorfheide model
%
% USAGE:
%
%       StructureForData = DataConstFile2x
%
% where StructureForData is a proper structure in matlab. The file is run in matlab via the eval
% function.
%
% INPUTS:          None!
%
% REQUIRED OUTPUT: StructureForData (structure). Any name for this structure will do since YADA renames it internally.
%                  REQUIRED FIELDS:  Y.data              (nxT) data-matrix of the n observable variables that should be evaluated by the
%                                                        Kalman filter
%                                    X.data              (kxT) data-matrix with exogenous data, may be an empty matrix, but must be stated
%                                    Y.names             (n dimensional cellstring) with variable names
%                                    X.names             (k dimensional cellstring) with names for exogenous variables (may be empty)
%                                    sample              (1x4 matrix) containing [start-year start-period end-yer end-period]. Must match
%                                                        the number of observations T.
%                                    frequency           (string) data frequency. Valid entries are: a(nnual), q(uarterly), m(onthly)
%                                                        if invalid frequency is reported, it is set to annual with sample being set
%                                                        to [1 1 T 1]
%                  OPTIONAL FIELDS:  Lambda              (vector) with lambda values to use for the DSGE-VAR models.
%                                    Y.levels            (1xn) vector, where 0 indicates a first differenced variable and 1 a levels
%                                                        variable
%                                    Y.annual            (1xn) vector, where 1 indicates that the variable should be annualized based
%                                                        on the data frequency, while 0 means that it should not be annualized (i.e.,
%                                                        can already be regarded as annualized).
%                                    Y.comparisondata    (nxT) matrix with comparison data for the observed variables.
%                                    Y.annualscale       (1xn) vector with scaling coefficient to multiply the annualized variables
%                                                        with.
%                                    Y.BVAR              (1xn1) vector where n1<=n that gives the indices for the variables in Y that
%                                                        should be included in a Bayesian VAR.
%                                    X.BVAR              (1xk1) vector where k1<=k that gives the indices for the variables in X that
%                                                        should be included in a Bayesian VAR.
%                                    X.extradata         (kxT1) data matrix with out-of-sample exogenous data.
%                                    Y.transformation    structure with subfields having the same names as the observed variables. For each
%                                                        such subfield, the fields "fcn", "partial", "annualizefcn", "annualizepartial",
%                                                        "initial", "x", "invertfcn", "invertinitial", "invertx", "exportfcn", "exportinitial",
%                                                        and "exportc" are available. The "fcn" and "annualizefcn" fields contain inline
%                                                        function strings determining how to transform the data for the variable, denoted
%                                                        e.g. by y. The fields "partial" and "annualizepartial" are string vectors with the
%                                                        partial derivative of the function in the "fcn" and "annualizefcn" fields, respectively,
%                                                        with respect the first variable in the function times the first variable in the
%                                                        function. The field "initial" contains a vector with all the initial values that
%                                                        should be taken into account. These are prepended to y before transforming the
%                                                        variable. The field "x" is a matrix of dimension dxTx with any additional variables
%                                                        that are required by the transformation. This can, for example, include the data for
%                                                        a linear trend. The field "invertfcn" holds a string with the function needed to
%                                                        invert the "fcn" field function. The fields "invertinitial" and "invertx" serve the
%                                                        same purpose as "initial" and "x", but are instead used by "invertfcn". Similarly,
%                                                        the field "exportfcn" holds the function that is used when exporting data from the menu
%                                                        functions for exporting. The fields "exportinitial" and "exportx" are used by this
%                                                        function like in the case of "initial" and "x" for the "fcn" function. Furthermore,
%                                                        the fields "exporttitle" and "exportname" are supported. They are only used when
%                                                        exporting the data to a spreadsheet where they replace the variable name.
%                                    Z.data              (mxTz) data matrix with conditioning information.
%                                    Z.names             (m dimensional cell string) with names of the variables in the conditioning
%                                                        information set.
%                                    Z.K1                (nxm) matrix used to map Y(t) into Z(t).
%                                    Z.K2                (nxm) matrix used to map Y(t-i) into Z(t).
%                                    U.data              (mxTu) matrix with data for initial conditions for conditional forecasting.
%                                    percentiles         (1xs) vector with positive intergers greater than 0 and less than 100. This
%                                                        is used when plotting percentiles from distributions.
%                                    C.data              (cxTc) matrix with data for observed variable scenarios.
%                                    C.comparisondata    (cxTc) matrix with comparison data for observed variable scenarios.
%                                    C.names             (c dimensional cell string) with names of the variables for observed variables
%                                                        scenarios.
%                                    C.transformation    structure with the same construction as Y.transformation.
%                                    C.C1                (nxc) natrix used to map Y(t) into C(t).
%                                    C.C2                (nxc) matrix used to mape Y(t-i) into C(t).
%                                    CBar.data           (cxTc) matrix with data for initial conditions for the observed variable
%                                                        scenarios.
%                                    CBar.comparisondata (cxTc) matrix with data for initial conditions for the comparison data for
%                                                        the observed variable scenarios.
%                                    C.exportprepend     (cxTd) matrix with data to prepend after transforming the export data.
%                                    C.export            structure with two subfields "data" and "names". The field "data" can hold a matrix
%                                                        of dimension mx(Tc+Td), while "names" is a cellarray of dimension 2xm. These subfields
%                                                        will be written to the export-file is present.
%                                    C.exportvariables   structure with fields "names" and "titles", where the former gives the name of
%                                                        the variable be be created and exported, while the latter is used for the title
%                                                        in the export file. Both fields hold cell arrays of string, where the number
%                                                        of entries should be equal to the number of variables to create. For each string
%                                                        in the names cell srray there should be a field with the same name. For each
%                                                        such variable field, there should be 5 sub-fields: fcn, x, y, subsample, and
%                                                        priordata. The fcn field is a string that defines the function to apply. The
%                                                        x field is a matrix with all the exogneous variables for the function, while
%                                                        the y-field is a cell array of strings with the names of the scenario variables
%                                                        to apply in the function. Since the variables will be exported it should be
%                                                        kept in mind that the scenario data variables are taken after they have been
%                                                        transformed by the export transform functions. The field subsample gives the
%                                                        positions in the output vector from the function that should be used, while
%                                                        priordata is a vector with fixed values to prepend to the used subsample of the
%                                                        created variable.
%                                    C.createdata        structure with field "names", a cell array of strings that hold the names of
%                                                        the variables to create from the scenario data and exogenous variables. For
%                                                        each string in "names" the structure should have fields with the same name.
%                                                        Each such field should have at least 5 sub-fields. Namely, "createbeforefcn",
%                                                        "createafterfcn", "createx", "createcompx", "createy", and "before". The
%                                                        "createbeforefcn" and "createafterfcn" fields hold string vectors with the
%                                                        functions to use for creating the variable before and after any input have been
%                                                        transformed. The field "createx" is a matrix with exogenous variables that the
%                                                        functions needs. For the comparison data the field "createcompx" holds the valid
%                                                        data for the exogenous variables. The number of observations should be equal to
%                                                        the number of observations on the scenario data (Tc). The field "createy" is a cell
%                                                        array with names for the scenario data variables to use. The last required variable
%                                                        field is "before", a boolean variable that is 1 if creation functions should only
%                                                        be applied before any transformation functions have been applied to the scenario
%                                                        variables, and 0 if the creation functions should be applied also after the
%                                                        transformation functions have been run on the scenario variable. In this case
%                                                        "createafterfcn" is used. If the field "before" is 1, then each variable must have
%                                                        8 additional fields. Those additional fields are: "fcn", "partial", "annualizefcn",
%                                                        "annualizepartial", "initial", "x", "compinitial", and "compx". The fields hold
%                                                        the data for applying a transformation function to the created variable.
%                                                        YADA assumes that this is only necessary if the inputs to the created variables
%                                                        have not been transformed yet. The data in these fields should be of the same
%                                                        type as the transformation functions in, e.g., C.transformation. When the comparison
%                                                        data is used instead of the scenario data, then "x" and "initial" are swapped with
%                                                        "compx" and "compinitial".
%                                    C.summaryorder      vector with c integers between 1 and c. This vector can be used to change the
%                                                        order of the secnario variables when displaying summary information.
%                                    C.startperiod       1x2 vector with the start-year start-period of the observed variable scenario.
%                                    State.S             rxs matrix used to map the subset of the state variables, zeta(t), from the
%                                                        full state vector ksi(t) for state variable scenarios.
%                                    State.data          sxTs matrix with scenario data on the state variables zeta(t).
%                                    State.names         (s) dimensional cell array of strings with the names of the zeta variables.
%                                    State.starperiod    1x2 vector with the start-year start-period of the state variable scenario.
%                                    zeta.K3             rxqz matrix used to map the subset of the state variables, zeta(t), from the
%                                                        full state vector ksi(t) for state variable conditioning assumptions.
%                                    zeta.data           qzxTz matrix with conditioning assumptions data for the state variables in
%                                                        zeta(t).
%                                    zeta.names          (qz) dimensional cell array of string with the names of the zeta variables
%                                                        used as conditioning assumptions.
%
%
% NOTICE:
%         * Any valid matlab functions on the path can be used by the function. How you transform any data you read in from
%           a spreadsheet, text file, of matlab mat-file is up to you.
%         * Any file format supported by matlab can be used for your data.
%
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   DG-R/MPR
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2017 European Central Bank.
%
%                       First version: July 31, 2006.
%                        This version: February 10, 2017.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 08-12-2006: The subfield "levels" was added to the field "Y". This vector indicates that variable is in levels
%               if 1, and in first differences if 0.
%
% * 12-01-2007: Added the subfield "BVAR" to the field "Y" and the field "X". These vectors indicate which variables
%               should be included in a Bayesian VAR estimation.
%
% * 22-01-2007: Added the subfield "annual" to the field "Y". This vector indicates how to annualize variables in
%               first differences. The value "4" means that the data are in quarerly changes terms and, hence, that
%               we should add "4" consequetive observations to obtain the annual growth rate. The value "1" means that
%               the variable is already annualized. If the variable is considered a levels variable, then the entry
%               in this vector is ignored.
%
% * 23-01-2007: The vector "StructureForData.Y.annual" has been changed such that 1 means that the corresponding
%               variable should be annualized based on the data frequency. The value 0 means that teh variable in
%               question should not be annualized. For quarterlu data, the value 1 means that the current and previous
%               3 observation are summed to get the annualized observation.
%
% * 25-01-2007: The subfield "annualscale" has been added to the field "Y". This vector holds constants to multiply
%               the annualized variables with. These constants must be non-zero to be valid. Any zeros that YADA locates
%               will be replaced with unit values. For the An and Schorfheide model, inflation has been 'annualized' at
%               the quarterly frequency by mutliplying the variable by 4. Hence, for a proper annualized inflation we
%               should multiply this variable by 1/4 and then add 4 consecutive quarters. This is indicated by 0.25
%               in the vector "StructureForData.Y.annualscale" below.
%
% * 16-02-2007: Added the subfield "extradata" to the field "X". This data is necessary to compute predictions outside
%               the available sample. The subfield is treated as optional.
%
% * 20-02-2007: Added the optional field "Percentiles". This data is used for settings up confidence bands in plots of
%               various distributions. The default is [5 15 85 95].
%
% * 21-02-2007: Changed the data in the "percentiles" field. Also, this field should use lower case characters.
%
% * 08-03-2007: Added the output necessary to conduct conditional forecasts.
%
% * 09-03-2007: Changed the length of the conditioning sample from 16 to 8 periods.
%
% * 26-04-2007: Added the output necessary for computing the observed variables scenario.
%
% * 04-09-2007: Edited the data for the observed variable scenario.
%
% * 10-09-2007: Allow for NaN's in the "C.data" matrix.
%
% * 18-10-2007: Increased the size of the set of conditional variables.
%
% * 22-10-2007: Added the sub-field "transformation" to "Y" and "C".
%
% * 23-10-2007: Added the sub-field "annualizefcn" as an additional transformation function string.
%
% * 30-10-2007: Added the sub-fields "inversefcn", "inverseinitial", and "inversex" as additional
%               transformation function data.
%
% * 07-11-2007: Updated the documentation.
%
% * 09-11-2007: The fields "exportfcn", "exportinitial", and "exportx" are added as additional
%               transformation functions.
%
% * 14-11-2007: The fields "x", "invertx", and "exportx" now has two sub-fields "data" and "numinitialobs".
%
% * 20-11-2007: removed the subfields "data" and "numinitialobs" for "x", "invertx", and "exportx".
%
% * 21-11-2007: Changed the observed variable scenario data slightly.
%
% * 09-01-2008: The "comparisondata" field was added to the field "C" for observed variable scenarios.
%
% * 10-01-2008: Updated the documentation.
%
% * 11-01-2008: Added the "comparisondata" field to the "Y" field for the observed variables.
%
% * 25-01-2008: Added the field "exportname" as a sub-sub-field to the transformation field.
%
% * 28-03-2008: Added the field "exporttitle" as a sub-sub-field the the transformation field. The field
%               export with subfields "data" and "names" were added to the "C" field. Moreover, the
%               field "summaryorder" was added to the "C" field.
%
% * 02-04-2008: Added the field "startperiod" to the "C" field.
%
% * 07-04-2008: Added the field "exportprepend" to the "C" field.
%
% * 24-07-2008: Made a change to the C.data matrix.
%
% * 31-07-2008: Added the partial field to the transformation structure.
%
% * 01-08-2008: Added partial function for the conditioning variables.
%
% * 08-08-2008: The last term of the partial function is now assumed to be treated as
%               as x-variable. The setting of the "x" field is not performed in the data
%               construction file, but internally by YADA. The reason is that the partial
%               function is used for decompositions of the variables and hence, the partial
%               derivative is evaluated at the sum of the components, while the level of the
%               variable is evaluated for the different components separately.
%
% * 12-08-2008: Added the annualizepartial field.
%
% * 20-08-2008: Added the exportvariables field.
%
% * 28-08-2008: Added the createdata field.
%
% * 23-09-2008: Added the "createafterpartial" field to the createdata structure.
%
% * 23-01-2009: Updated the documentation.
%
% * 26-02-2009: Changed the ZData matrix values.
%
% * 15-04-2009: Added the field "State", with sub-fields "S", "data", "names", and "startperiod".
%
% * 18-01-2010: Added the "Lambda" field for dealing with DSGE-VAR models.
%
% * 27-01-2010: Changed the values of the "Lambda" field.
%
% * 27-10-2010: Added an additional scenario-path for yhat.
%
% * 05-01-2011: Updated the documentation.
%
% * 12-01-2011: The function now supports unix and mac paths.
%
% * 25-01-2011: Added the field "TransMatrix" to the field "Y" for transformation using linear
%               combinations of the transformed observed variables.
%
% * 03-01-2012: Updated the documentation.
%
% * 17-01-2012: Added NaN's to the conditioning assumptions for forecasting.
%
% * 03-01-2013: Updated the documentation.
%
% * 27-01-2014: Updated the documentation.
%
% * 07-08-2014: The file supports two exogenous variables.
%
% * 13-01-2015: Updated the documentation.
%
% * 21-01-2016: Updated the documentation.
%
% * 10-02-2017: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% Read data from a file. Make sure that the full path is given since this file will be moved
% into the sub-driectory tmp of pwd (pwd = working directory, i.e. the directory where YADA.m
% is located)
%
% Here we read in simulated data from the An Schorfheide model
%
StructureForData.Y.data = wk1read(FixFilePath([pwd '\example\AnSchorfheide\data\AnSchorfheideData.wk1']));
%StructureForData.Y.data = wk1read([pwd '\data\AnSchorfheideData.wk1']);
%
% transpose the data since YADA prefers it that way
%
StructureForData.Y.data = StructureForData.Y.data';
StructureForData.Y.names = {'YGR' 'INFL' 'INT'};
%
StructureForData.Lambda = [0.25 0.5 0.75 1 1.25 1.5 1.75 2 2.5 3 3.5 4 4.5 5 6 7 8 9 10 25 Inf];
StructureForData.Y.levels = [0 0 1];
StructureForData.Y.annual = [1 1 0];
StructureForData.Y.annualscale = [1 0.25 1];
StructureForData.Y.BVAR = [1 2 3];
StructureForData.X.data = [ones(1,100);kron([0 0 0 1],ones(1,25))];
StructureForData.X.extradata = [ones(1,120);kron([0 0 0 1],ones(1,30))];
StructureForData.X.names = {'const' 'dummy'};
StructureForData.X.BVAR = [1 2];
StructureForData.sample = [1980 1 2004 4];
StructureForData.frequency = 'q';
StructureForData.percentiles = [2.5 5 15 25 75 85 95 97.5];

%
% end of DataConstFile2x.m
%
