function ModelParameters = UpdateSWParameters(ModelParameters)
% UpdateSWParameters: Shows how to define new parameters from previously defined parameters
%
% USAGE:
%
%       ModelParameters = UpdateSWParameters(ModelParameters)
%
% REQUIRED INPUT:  ModelParameters (structure) with fields corresponding to the parameters
%                                   you need for defining new parameters that should be
%                                   added to the structure.
%
% REQUIRED OUTPUT: ModelParameters (structure) with additions to the input structure.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   DG-R/MPR
%                                   European Central Bank
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2017 European Central Bank.
%
%                       First version: June 16, 2011.
%                        This version: February 10, 2017.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 17-06-2011: Continued adding parameter expressions to the function.
%
% * 20-06-2011: Fixed a bug in the calculation of the steady-state wage.
%
% * 21-06-2011: Simplified the expression for "ModelParameters.zy".
%
% * 28-06-2011: Simplified the expression for "ModelParameters.whlcstar".
%
% * 03-01-2012: Updated the documentation.
%
% * 03-01-2013: Updated the documentation.
%
% * 27-01-2014: Updated the documentation.
%
% * 13-01-2015: Updated the documentation.
%
% * 21-01-2016: Updated the documentation.
%
% * 10-02-2017: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% mode values in Smets and Wouters
%
%ModelParameters.varphi = 5.48;
%ModelParameters.sigmac = 1.39;
%ModelParameters.lambda = 0.71;
%ModelParameters.xiw = 0.73;
%ModelParameters.sigmal = 1.92;
%ModelParameters.xip = 0.65;
%ModelParameters.imathw = 0.59;
%ModelParameters.imathp = 0.22;
%ModelParameters.psi = 0.54;
%ModelParameters.phip = 1.61;
%ModelParameters.rpi = 2.03;
%ModelParameters.rho = 0.81;
%ModelParameters.ry = 0.08;
%ModelParameters.rdy = 0.22;
%ModelParameters.pibar = 0.81;
%ModelParameters.betainv = 0.16;
%ModelParameters.lbar = -0.1;
%ModelParameters.gammabar = 0.43;
%ModelParameters.alpha = 0.19;
%
% Parameter functions related to the steady-state of the Smets and Wouters model
%
ModelParameters.gamma = (ModelParameters.gammabar/100)+1;
ModelParameters.pistar = (ModelParameters.pibar/100)+1;
ModelParameters.beta = 1/((ModelParameters.betainv/100)+1);
ModelParameters.betabar = ModelParameters.beta*(ModelParameters.gamma^(-ModelParameters.sigmac));
ModelParameters.rbar = 100*((ModelParameters.pistar/ModelParameters.betabar)-1);
%
% steady-state rental rate of capital
%
ModelParameters.rkstar = (1/ModelParameters.betabar)+ModelParameters.delta-1;
%
% steady-state real wages
%
ModelParameters.w = (ModelParameters.alpha^ModelParameters.alpha*(1-ModelParameters.alpha)^(1-ModelParameters.alpha)/(ModelParameters.phip*ModelParameters.rkstar^ModelParameters.alpha))^(1/(1-ModelParameters.alpha));
%
% steady-state investment-capital ratio
%
ModelParameters.ik = ModelParameters.gamma+ModelParameters.delta-1;
%
% steady-state labor-capital ratio
%
ModelParameters.lk = ((1-ModelParameters.alpha)/ModelParameters.alpha)*(ModelParameters.rkstar/ModelParameters.w);
%
% steady-state capital-output ratio
%
ModelParameters.ky = ModelParameters.phip*(ModelParameters.lk^(ModelParameters.alpha-1));
%
% steady-state investment-output ratio
%
ModelParameters.iy = ModelParameters.ik*ModelParameters.ky;
%
% steady-state consumption-output ratio
%
ModelParameters.cy = 1-ModelParameters.gy-ModelParameters.iy;
%
% steady-state rental rate of capital times capital output ratio
%
% zy = rkstar*ky = alpha
%
ModelParameters.zy = ModelParameters.alpha;
%
% steady-state real-wage bill to consumption ratio 
%
ModelParameters.whlcstar = (1-ModelParameters.alpha)/(ModelParameters.phiw*ModelParameters.cy);
%
% additional parameters
%
ModelParameters.c1 = (ModelParameters.lambda/ModelParameters.gamma)/(1+(ModelParameters.lambda/ModelParameters.gamma));
ModelParameters.c2 = ((ModelParameters.sigmac-1)*ModelParameters.whlcstar)/(ModelParameters.sigmac*(1+(ModelParameters.lambda/ModelParameters.gamma)));
ModelParameters.c3 = (1-(ModelParameters.lambda/ModelParameters.gamma))/(ModelParameters.sigmac*(1+(ModelParameters.lambda/ModelParameters.gamma)));
ModelParameters.i1 = 1/(1+(ModelParameters.betabar*ModelParameters.gamma));
ModelParameters.i2 = ModelParameters.i1/(ModelParameters.gamma^2*ModelParameters.varphi);
ModelParameters.q1 = ModelParameters.betabar*(1-ModelParameters.delta);
ModelParameters.c3inv = 1/ModelParameters.c3;
ModelParameters.z1 = (1-ModelParameters.psi)/ModelParameters.psi;
ModelParameters.k1 = (1-ModelParameters.delta)/ModelParameters.gamma;
ModelParameters.k2 = (1-ModelParameters.k1)*(1+(ModelParameters.betabar*ModelParameters.gamma))*ModelParameters.gamma^2*ModelParameters.varphi;
ModelParameters.pi1 = ModelParameters.imathp/(1+(ModelParameters.betabar*ModelParameters.gamma*ModelParameters.imathp));
ModelParameters.pi2 = (ModelParameters.betabar*ModelParameters.gamma)/(1+(ModelParameters.betabar*ModelParameters.gamma*ModelParameters.imathp));
ModelParameters.pi3 = ((1-ModelParameters.xip)*(1-(ModelParameters.betabar*ModelParameters.gamma*ModelParameters.xip)))/((1+(ModelParameters.betabar*ModelParameters.gamma*ModelParameters.imathp))*ModelParameters.xip*((ModelParameters.phip-1)*ModelParameters.epsilonp+1));
ModelParameters.m1 = ModelParameters.sigmal;
ModelParameters.m2 = 1/(1-(ModelParameters.lambda/ModelParameters.gamma));
ModelParameters.m3 = ModelParameters.m2*ModelParameters.lambda/ModelParameters.gamma;
ModelParameters.w1 = 1/(1+(ModelParameters.betabar*ModelParameters.gamma));
ModelParameters.w2 = ModelParameters.w1*(1+(ModelParameters.betabar*ModelParameters.gamma*ModelParameters.imathw));
ModelParameters.w3 = ModelParameters.w1*ModelParameters.imathw;
ModelParameters.w4 = (ModelParameters.w1*(1-ModelParameters.xiw)*(1-(ModelParameters.betabar*ModelParameters.gamma*ModelParameters.xiw)))/(ModelParameters.xiw*((ModelParameters.phiw-1)*ModelParameters.epsilonw+1));

%
% end of UpdateSWParameters.m
%
