function AcceptanceRatioPlot(AcceptedDraws,NumBurnin,CurrChain,lambda,DSGEModel,CurrINI)
% AcceptanceRatioPlot: Plots the acceptance ratio from the random walk metropolis algorithm
%                      over the posterior draws.
%
% USAGE:
%
%       AcceptanceRatioPlot(AcceptedDraws,NumBurnin,CurrChain,lambda,DSGEModel,CurrINI)
%
% REQUIRED INPUT: AcceptedDraws (vector) of dimension (NumIterx1) with the number of accepted draws.
%
%                 NumBurnin (integer) with the number of burn-in draws.
%
%                 CurrChain (integer) with the number for the current MCMC chain.
%
%                 lambda (scalar) with the value of the lambda hyperparameter for the DSGE-VAR model.
%
%                 DSGEModel (structure)  whose fields contain all required information
%                                 about the model, such as input files, variable and sample
%                                 data. Actual observations are not stored in this file.
%
%                 CurrINI (structure) with initialization information.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   DG-R/MPR
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2017 European Central Bank.
%
%                       First version: April 17, 2007.
%                        This version: August 14, 2017.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 18-04-2007: Updated the documentation.
%
% * 09-11-2007: Updated the documentation.
%
% * 10-01-2008: Changed the last drawnow call to pause(0.02).
%
% * 10-03-2008: Made sure that the "CurrChain" variable affects the filename for the figure.
%
% * 27-03-2008: Made sure that the sequential grid has at least 2 values.
%
% * 23-05-2008: Updated the documentation.
%
% * 19-12-2008: Updated the documentation.
%
% * 02-05-2009: Updated the documentation.
%
% * 06-01-2010: Fixed a bug where Origin>NumIter was not properly accounted for.
%
% * 05-01-2011: Updated the documentation.
%
% * 11-01-2011: Added support for unix and mac paths.
%
% * 04-01-2012: Updated the documentation.
%
% * 02-01-2013: Updated the documentation.
%
% * 24-01-2014: Updated the documentation.
%
% * 25-02-2014: Added the "lambda" input variable, thereby taking DSGE-VAR models into account.
%
% * 27-02-2014: Made sure that different posterior samplers are properly covered.
%
% * 05-01-2015: Updated the documentation.
%
% * 27-08-2015: The function now supports the RWM posterior sampler with a Student-t proposal density.
%
% * 20-01-2016: Updated the documentation.
%
% * 10-02-2017: Updated the documentation.
%
% * 02-04-2017: The function now supports the fixed blocking RWM posterior sampler with a normal
%               proposal density.
%
% * 14-08-2017: The function now supports the SMC with likelihood tempering posterior sampler.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% create the acceptance ratio
%
NumIter = length(AcceptedDraws);
AcceptedDraws = AcceptedDraws' ./ (1:1:NumIter);
%
% locate the controls structure
%
maingui = findobj('Type','figure','Tag','YADA');
controls = get(maingui,'UserData');
%
% sequential marginal likelihood start value and step length
%
SeqStartStr = get(controls.dsge.sequentialstart,'String');
Origin = str2num(strrep(StringTrim(SeqStartStr(DSGEModel.SequentialStartIteration,:)),',',''));
SeqStepStr = get(controls.dsge.sequentialstep,'String');
StepLength = str2num(strrep(StringTrim(SeqStepStr(DSGEModel.SequentialStepLength,:)),',',''));
%
% make sure that Origin is less than NumIter
%
if Origin>=NumIter;
   Origin = floor(NumIter/10);
end;
%
% Create the sequential grid over which we will compute the marginal likelihood
%
SeqGrid = (Origin:StepLength:NumIter);
if isempty(SeqGrid)==1;
   SeqGrid = NumIter;
elseif SeqGrid(length(SeqGrid))<NumIter;
   SeqGrid = [SeqGrid NumIter];
end;
%
% select the subset of AcceptedDraws
%
AcceptedDraws = AcceptedDraws(SeqGrid);
%
if isempty(lambda)==1;
   NameStr = ['Sequential Acceptance Ratio - ' DSGEModel.NameOfModel ' Chain ' int2str(CurrChain)];
else;
   NameStr = ['DSGE-VAR(' num2str(lambda,'%0.4f') ') Sequential Acceptance Ratio - ' DSGEModel.NameOfModel ' Chain ' int2str(CurrChain)];
end;
%
% prepare the figure
%
GraphDirectory = FixFilePath([DSGEModel.OutputDirectory '\graphs']);
%
FigHandle = figure('Visible','off', ...
                   'Position',[(CurrINI.scrsz(3)-700)/2 (CurrINI.scrsz(4)-500)/2 700 500], ...
                   'Tag','Acceptance Ratio Plot', ...
                   'Name',NameStr);
%
SetFigureProperties(FigHandle,CurrINI);
%
plot(SeqGrid,AcceptedDraws,'LineStyle','-','Color',[0.75 0 0],'LineWidth',2,'Marker','.');
hold('on');
%
% check the YLim
%
ylim = get(gca,'YLim');
add_on = 2.0*(ylim(2)-ylim(1));
ylim = [ylim(1)-add_on ylim(2)+add_on];
set(gca,'XLim',[SeqGrid(1) NumIter],'YLim',[max(ylim(1),0) min(ylim(2),1)]);
%
% plot the number of burnin draws
%
plot(NumBurnin*ones(1,2),get(gca,'YLim'),'-','Color',[0 0 0],'LineWidth',1);
hold('off');
%
if isempty(lambda)==1;
   if (DSGEModel.PosteriorSampler==1)|((DSGEModel.PosteriorSampler>=3)&(DSGEModel.PosteriorSampler<=7));
      SetAxesFonts(CurrINI,'Title','Sequential acceptance ratio for DSGE model','XLabel',['RWM draw number (with ' num2str(NumBurnin,'%0.0f') ' burn-in draws)'],'YLabel','Ratio');
   elseif DSGEModel.PosteriorSampler==2;
      SetAxesFonts(CurrINI,'Title','Sequential acceptance ratio for DSGE model','XLabel',['Slice sampler draw number (with ' num2str(NumBurnin,'%0.0f') ' burn-in draws)'],'YLabel','Ratio');
   elseif DSGEModel.PosteriorSampler==8;
      SetAxesFonts(CurrINI,'Title','Sequential acceptance ratio for DSGE model','XLabel','SMC sampler draw number','YLabel','Ratio');
   end;
else;
   if (DSGEModel.PosteriorSampler==1)|((DSGEModel.PosteriorSampler>=3)&(DSGEModel.PosteriorSampler<=7));
      SetAxesFonts(CurrINI,'Title',['Sequential acceptance ratio for DSGE-VAR(' num2str(lambda,'%0.4f') ') model'],'XLabel',['RWM draw number (with ' num2str(NumBurnin,'%0.0f') ' burn-in draws)'],'YLabel','Ratio');
   elseif DSGEModel.PosteriorSampler==2;
      SetAxesFonts(CurrINI,'Title',['Sequential acceptance ratio for DSGE-VAR(' num2str(lambda,'%0.4f') ') model'],'XLabel',['Slice sampler draw number (with ' num2str(NumBurnin,'%0.0f') ' burn-in draws)'],'YLabel','Ratio');
   elseif DSGEModel.PosteriorSampler==8;
      SetAxesFonts(CurrINI,'Title',['Sequential acceptance ratio for DSGE-VAR(' num2str(lambda,'%0.4f') ') model'],'XLabel','SMC sampler draw number','YLabel','Ratio');
   end;
end;
%
% save the grasph
%
if DirExist(GraphDirectory);
   if isempty(lambda)==1;
      file = FixFilePath([GraphDirectory '\SequentialAcceptanceRatio-' num2str(NumIter,'%0.0f') '-' int2str(CurrChain) '.fig']);
   else;
      SelectedShockStr = ismember((1:length(DSGEModel.StateShockPositions)),DSGEModel.DSGEVARShocks);
      SelectedShockStr = strrep(num2str(SelectedShockStr),' ','');
      file = FixFilePath([GraphDirectory '\SequentialAcceptanceRatio-DSGE-VAR-' num2str(lambda,'%0.4f') '-' int2str(DSGEModel.BVARLags) '-' SelectedShockStr '-' num2str(NumIter,'%0.0f') '-' int2str(CurrChain) '.fig']);
   end;
   saveas(FigHandle,file);
end;
%
% display the graph
%
SetToolbarImages(FigHandle,CurrINI.images);
set(FigHandle,'Visible','on','CloseRequestFcn','delete(gcf); drawnow; pause(0.02);');
drawnow;

%
% end of AcceptanceRatioPlot.m
%
