function [action,ModelSequence] = ModelSequenceDLG(selector,CurrINI)
% ModelSequenceDLG: Dialog for selecting models included in a sequence and for redordering these models.
%
% USAGE:
%
%       [action,ModelSequence] = ModelSequenceDLG(selector,CurrINI)
%
% REQUIRED INPUT:  selector (string vector), with the possible values "init", "reorder", "reset", "open",
%                           "remove", and "quit".
%
% OPTIONAL INPUT:  CurrINI (structure), with initialization information.
%
% OPTIONAL OUTPUT: action (string vector) taking the value "OK" or "Cancel". Depends on which button was
%                           clicked on.
%
%                  ModelSequence (string matrix) with the selected sequence of models.
%
% NOTE: If selector = "init", then  all optional inputs and outputs are required.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   DG-R/MPR
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2017 European Central Bank.
%
%                       First version: October 28, 2014.
%                        This version: February 10, 2017.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 29-10-2014: Increased the font size on the listbox control.
%
% * 30-10-2014: Finished the function.
%
% * 05-01-2015: Updated the documentation.
%
% * 08-06-2015: The code takes the CurrINI.GraphicsRoot field into account.
%
% * 20-01-2016: Updated the documentation.
%
% * 10-02-2017: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

if strcmp(selector,'init')==1;
   ModelSequence = CurrINI.ModelSequence;
   scrsz = CurrINI.scrsz;
   ImageStructure = ReadImages;
   %
   % try to determine the width and the height
   %
   width = 800;
   height = 500;
else;
   ModelSequenceGUI = findobj('Type','figure','Tag','ModelSequenceDLG');
   controls = get(ModelSequenceGUI,'UserData');
   CurrINI = get(controls.cancel,'UserData');
end;
%
switch selector
   case 'init'
      ModelSequenceGUI = figure('Color',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'FileName','ModelSequenceDLG.m', ...
         'MenuBar','none', ...
         'Units','pixels', ...
         'Position',[(scrsz(3)-width)/2 (scrsz(4)-height)/2 width height], ...
         'Tag','ModelSequenceDLG', ...
         'Resize','off', ...
         'Visible','off', ...
         'Name','YADA - Select Model Sequence', ...
         'NumberTitle','off', ...
         'WindowStyle','modal', ...
         'CloseRequestFcn','ModelSequenceDLG quit', ...
         'ToolBar','none');
      %
      if MatlabNumber>=7.0;
         set(ModelSequenceGUI,'DockControl','off');
      end;
      %
      AxesBox([2 2 width-2 height-10],'Model Sequence Selection',100,[0.5 1],'on',CurrINI);
      %
      % Create a listbox
      %
      controls.variables = uicontrol('Units','pixels', ...
         'BackgroundColor',[1 1 1], ...
         'Position',[20 60 665 height-85], ...
         'FontSize',10, ...
         'FontName','FixedWidth', ...
         'Style','listbox', ...
         'Enable','on', ...
         'Min',0, ...
         'Max',1, ...
         'HorizontalAlignment','left', ...
         'String',ModelSequence, ...
         'UserData',ModelSequence, ...
         'Visible','on', ...
         'Tag','VariablesListBox',...
         'Value',1);
      %
      % move up buttons
      %
      controls.up = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[width-95 ((height-80)/2)+60 75 25], ...
         'TooltipString','Move Up', ...
         'CData',ImageStructure.up, ...
         'Enable','on', ...
         'Callback','ModelSequenceDLG reorder', ...
         'Tag','MoveUpButton');
      %
      % move down button
      %
      controls.down = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[width-95 ((height-140)/2)+60 75 25], ...
         'TooltipString','Move Up', ...
         'CData',ImageStructure.down, ...
         'Enable','on', ...
         'Callback','ModelSequenceDLG reorder', ...
         'Tag','MoveDownButton');
      %
      % Add new model
      %
      controls.open = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[20 20 75 25], ...
         'TooltipString','Add Model', ...
         'CData',ImageStructure.open, ...
         'Enable','on', ...
         'Callback','ModelSequenceDLG open', ...
         'Tag','AddModelButton');
      %
      % remove model
      %
      controls.remove = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[100 20 75 25], ...
         'TooltipString','Remove Model', ...
         'CData',ImageStructure.clear, ...
         'Enable','on', ...
         'Callback','ModelSequenceDLG remove', ...
         'Tag','RemoveModelButton');
      %
      % Three buttons to select action
      %
      controls.ok = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[width-255 20 75 25], ...
         'String','OK', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'Enable','on', ...
         'Callback','ModelSequenceDLG quit', ...
         'UserData','waiting', ...
         'Tag','ButtonOK');
      %
      controls.resetbutton = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[width-175 20 75 25], ...
         'String','Reset', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'Enable','off', ...
         'Callback','ModelSequenceDLG reset', ...
         'Tag','ButtonReset');
      %
      controls.cancel = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[width-95 20 75 25], ...
         'String','Cancel', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'Enable','on', ...
         'UserData',CurrINI', ...
         'Callback','ModelSequenceDLG quit', ...
         'Tag','ButtonCancel');
      %
      set(ModelSequenceGUI,'UserData',controls);
   case 'reset'
      ModelSequence = get(controls.variables,'UserData');
      set(controls.variables,'String',ModelSequence,'Value',1);
      set(controls.resetbutton,'Enable','off');
   case 'reorder'
      ModelSequence = get(controls.variables,'String');
      Value = get(controls.variables,'Value');
      n = size(ModelSequence,1);
      Order = (1:n);
      if (Value>1)&(gco==controls.up);
         %
         % move current file up
         %
         DownValue = Order(Value-1);
         Order(Value-1) = Value;
         Order(Value) = DownValue;
         set(controls.variables,'String',ModelSequence(Order,:),'Value',Value-1);
         if MatlabNumber>=7;
            uicontrol(controls.variables);
         end;
         set(controls.resetbutton,'Enable','on');
      elseif (Value<n)&(gco==controls.down);
         %
         % move current file down
         %
         UpValue = Order(Value+1);
         Order(Value+1) = Value;
         Order(Value) = UpValue;
         set(controls.variables,'String',ModelSequence(Order,:),'Value',Value+1);
         if MatlabNumber>=7;
            uicontrol(controls.variables);
         end;
         set(controls.resetbutton,'Enable','on');
      end;
   case 'open'
      ModelSequence = get(controls.variables,'String');
      if isempty(ModelSequence)==0;
         File = StringTrim(ModelSequence(size(ModelSequence,1),:));
      else;
         File = CurrINI.CurrentModelFile;
      end;
      [status,ModelFile] = FileBrowse(File,'Add DSGE Model File',[],'dsge',CurrINI);
      if status==1;
         ValidFile = 1;
         try;
            NewDSGEModel = ReadDSGEModel(ModelFile);
         catch;
            ValidFile = 0;
         end;
         if ValidFile==1;
            [NewDSGEModel.SubBeginYearValue,NewDSGEModel.SubBeginPeriodValue,NewDSGEModel.SubEndYearValue,NewDSGEModel.SubEndPeriodValue,NewDSGEModel.YearStrMatrix,NewDSGEModel.PeriodStrMatrix] = CreateEstimationSampleValues(NewDSGEModel);
            [ModelStatus,NewDSGEModel] = VerifyDSGEModel(NewDSGEModel,ModelFile,NewDSGEModel.DataConstructionFile,CurrINI);
            if ModelStatus==1;
               ModelSequence = get(controls.variables,'String');
               ModelSequence = strvcat(ModelSequence,ModelFile);
               n = size(ModelSequence,1);
               set(controls.variables,'String',ModelSequence,'Value',n);
               if MatlabNumber>=7;
                  uicontrol(controls.variables);
               end;
               set(controls.resetbutton,'Enable','on');
            end;
         else;
            txt = ['The DSGE model file you selected, "' GetFilename(ModelFile) '", couldn''t be loaded into YADA. The file has possibly been corrupted.'];
            About(txt,'information','Bad DSGE Model File',200,600,CurrINI);
         end;
      end;
   case 'remove'
      CurrModel = get(controls.variables,'Value');
      ModelSequence = get(controls.variables,'String');
      n = size(ModelSequence,1);
      if n>1;
         set(controls.variables,'String',ModelSequence(setdiff((1:n),CurrModel),:),'Value',1);
      else;
         set(controls.variables,'String','');
      end;
      set(controls.resetbutton,'Enable','on');
   case 'quit'
      set(controls.ok,'UserData','done');
end;

if strcmp(selector,'init')==1;
   set(ModelSequenceGUI,'Visible','on');
   drawnow;
   if MatlabNumber>=7;
      uicontrol(controls.variables);
   end;
   %
   waitfor(controls.ok,'UserData','done');
   if (isempty(get(ModelSequenceGUI,'CurrentObject'))==1)|(strcmp(get(get(ModelSequenceGUI,'CurrentObject'),'Type'),'uicontrol')==0);
      action = 'Cancel';
   else;
      action = get(get(ModelSequenceGUI,'CurrentObject'),'String');
      if (isempty(action)==1)|(strcmp(action,'OK')==0);
         action = 'Cancel';
      end;
   end;
   %
   ModelSequence = get(controls.variables,'String');
   %
   delete(ModelSequenceGUI);
   drawnow;
   drawnow;
   pause(0.02);
end;

%
% end of ModelSequenceDLG.m
%
