function PlotMultiANOVA(MPSRF,NumBurnin,NumChains,lambda,DSGEModel,CurrINI)
% PlotMultiANOVA: Prepares plot of the multivariate potential scale reduction factor and the
%                 determinants of the between-chain and the within-chain covariance.
%
% USAGE:
%
%       PlotMultiANOVA(MPSRF,NumBurnin,NumChains,lambda,DSGEModel,CurrINI)
%
% REQUIRED INPUT: MPSRF (matrix) of dimension NumSeqEst x 4, where NumSeqEst is the number
%                       of sequential estimates. The first column contains the number of
%                       observations included in the sequatial estimation, the second the
%                       Multivariate Potential Scale Reduction Factor, the third column the
%                       determinant of V (the total covariance matrix for the
%                       parameters), and the fourth column the determinant of W (the
%                       within-chain covariance matrix for the parameters).
%
%                 NumBurnin (integer) with the number of burn-in draws.
%
%                 NumChains (integer) with the number of parallel MCMC chains.
%
%                 lambda (scalar) with the value of the lambda hyperparameter for the DSGE-VAR model.
%
%                 DSGEModel (structure)  whose fields contain all required information
%                       about the model, such as input files, variable and sample data.
%                       Actual observations are not stored in this file.
%
%                 CurrINI (structure) with initialization information.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   DG-R/MPR
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2017 European Central Bank.
%
%                       First version: April 24, 2007.
%                        This version: August 14, 2017.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 09-11-2007: Updated the documentation.
%
% * 10-01-2008: Changed the last drawnow call to pause(0.02).
%
% * 23-05-2008: Updated the documentation.
%
% * 19-12-2008: Updated the documentation.
%
% * 02-05-2009: Updated the documentation.
%
% * 06-01-2010: Updated the documentation.
%
% * 05-01-2011: Updated the documentation.
%
% * 11-01-2011: The function now supports unix and mac paths.
%
% * 04-01-2012: Updated the documentation.
%
% * 02-01-2013: Updated the documentation.
%
% * 24-01-2014: Updated the documentation.
%
% * 27-02-2014: Added the "lambda" input variable to allow for DSGE-VAR models.
%
% * 05-01-2015: Updated the documentation.
%
% * 27-08-2015: The function now supports the RWM posterior sampler with a Student-t proposal density.
%
% * 20-01-2016: Updated the documentation.
%
% * 10-02-2017: Updated the documentation.
%
% * 02-04-2017: The function now supports the fixed blocking RWM posterior sampler with a normal
%               proposal density and all samplers that come thereafter.
%
% * 14-08-2017: The function now supports the SMC with likelihood tempering posterior sampler.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% set the data
%
Xvalues = MPSRF(:,1)+NumBurnin;
%
GraphDirectory = FixFilePath([DSGEModel.OutputDirectory '\graphs']);
%
if isempty(lambda)==1;
   NameStr = 'Multivariate ANOVA - MPSRF';
else;
   NameStr = ['DSGE-VAR(' num2str(lambda,'%0.4f') ') Multivariate ANOVA - MPSRF'];
end;
%
FigHandle = figure('Visible','off', ...
                   'Position',[(CurrINI.scrsz(3)-750)/2 (CurrINI.scrsz(4)-550)/2 750 550], ...
                   'Tag','MPSRF', ...
                   'Name',NameStr);
%
SetFigureProperties(FigHandle,CurrINI);
%
% plot the multivariate potential scale reduction factors against the number of
% draws from the posterior
%
subplot(2,1,1);
MPHandle = plot(Xvalues,MPSRF(:,2),'LineStyle','-','Color',[0.75 0 0],'LineWidth',2);
hold('on');
ylim = get(gca,'YLim');
ylim_2 = ylim(2);
%
NBHandle = plot(NumBurnin*ones(1,2),[1 ylim_2],'LineStyle','-','Color',[0 0 0]);
hold('on');
CLineHandle = plot([0 Xvalues(length(Xvalues))],1.1*ones(1,2),'LineStyle','-','Color',[0 0 0]);
hold('off');
set(gca,'YLim',[1 ylim_2],'XLim',[0 Xvalues(length(Xvalues))],'Children',[MPHandle NBHandle CLineHandle]);
%
% set the title
%
if (DSGEModel.PosteriorSampler==1)|((DSGEModel.PosteriorSampler>=3)&(DSGEModel.PosteriorSampler<=7));
   SetAxesFonts(CurrINI,'Title','Sequential estimates of the multivariate potential scale reduction factor (MPSRF)','XLabel',['RWM draw number (with ' num2str(NumBurnin,'%0.0f') ' burn in draws)'],'YLabel','MPSRF');
elseif DSGEModel.PosteriorSampler==2;
   SetAxesFonts(CurrINI,'Title','Sequential estimates of the multivariate potential scale reduction factor (MPSRF)','XLabel',['Slice sampler draw number (with ' num2str(NumBurnin,'%0.0f') ' burn in draws)'],'YLabel','MPSRF');
elseif DSGEModel.PosteriorSampler==8;
   SetAxesFonts(CurrINI,'Title','Sequential estimates of the multivariate potential scale reduction factor (MPSRF)','XLabel','SMC sampler draw number','YLabel','MPSRF');
end;
%
subplot(2,1,2);
WHandle = plot(Xvalues,MPSRF(:,4),'LineStyle','--','Color',[0.75 0 0],'LineWidth',2);
hold('on');
VHandle = plot(Xvalues,MPSRF(:,3),'LineStyle','-','Color',[0 0 0.75],'LineWidth',2);
hold('on');
ylim = get(gca,'YLim');
ylim_2 = 1.1*ylim(2);
%
NBHandle = plot(NumBurnin*ones(1,2),[0 ylim_2],'LineStyle','-','Color',[0 0 0]);
hold('off');
set(gca,'YLim',[0 ylim_2],'XLim',[0 Xvalues(length(Xvalues))],'Children',[VHandle WHandle NBHandle]);
%
% set the title
%
if (DSGEModel.PosteriorSampler==1)|((DSGEModel.PosteriorSampler>=3)&(DSGEModel.PosteriorSampler<=7));
   SetAxesFonts(CurrINI,'Title',['Total and within variation in the ' num2str(NumChains,'%0.0f') ' parallel MCMC chains'],'XLabel',['RWM draw number (with ' num2str(NumBurnin,'%0.0f') ' burn in draws)']);
elseif DSGEModel.PosteriorSampler==2;
   SetAxesFonts(CurrINI,'Title',['Total and within variation in the ' num2str(NumChains,'%0.0f') ' parallel MCMC chains'],'XLabel',['Slice sampler draw number (with ' num2str(NumBurnin,'%0.0f') ' burn in draws)']);
elseif DSGEModel.PosteriorSampler==8;
   SetAxesFonts(CurrINI,'Title',['Total and within variation in the ' num2str(NumChains,'%0.0f') ' parallel MCMC chains'],'XLabel','SMC sampler draw number');
end;
%
legHandle = legend([WHandle VHandle],strvcat('Determinant within variance','Determinant total variance'),2);
SetAxesFonts(CurrINI,legHandle);
%
% save to disk
%
if DirExist(GraphDirectory)==1;
   if isempty(lambda)==1;
      FigFile = FixFilePath([GraphDirectory '\MANOVA-' DSGEModel.NameOfModel '-' int2str(NumChains) '.' int2str(NumBurnin) '.' int2str(Xvalues(length(Xvalues))) '.fig']);
   else;
      SelectedShockStr = ismember((1:length(DSGEModel.StateShockPositions)),DSGEModel.DSGEVARShocks);
      SelectedShockStr = strrep(num2str(SelectedShockStr),' ','');
      FigFile = FixFilePath([GraphDirectory '\MANOVA-DSGE-VAR-' num2str(lambda,'%0.4f') '-' int2str(DSGEModel.BVARLags) '-' SelectedShockStr '-' DSGEModel.NameOfModel '-' int2str(NumChains) '.' int2str(NumBurnin) '.' int2str(Xvalues(length(Xvalues))) '.fig']);
   end;
   saveas(FigHandle,FigFile);
end;
%
% display the graph
%
SetToolbarImages(FigHandle,CurrINI.images);
set(FigHandle,'Visible','on','CloseRequestFcn','delete(gcf); drawnow; pause(0.02);');
drawnow;
pause(0.02);

%
% end of PlotMultiANOVA.m
%
