function RawLogPosteriorPlot(LogPost,thetaPostSample,NumBurnin,TotalDraws,CurrChain,DSGEModel,CurrINI)
% RawLogPosteriorPlot: Provides a plot of the log posterior kernel values over the posterior draws.
%
% USAGE:
%
%       RawLogPosteriorPlot(LogPost,thetaPostSample,NumBurnin,TotalDraws,CurrChain,TypeStr,DSGEModel,CurrINI)
%
% REQUIRED INPUT: LogPost (vector) of dimension NumIterx1 with all the log posterior values
%                         for the underlying log posterior draws.
%
%                 thetaPostSample (matrix) of dimension NumIterxNumParam with the original
%                         parameter values yielding the log posterior kernel values.
%
%                 NumBurnin (integer) with the number of burn-in draws.
%
%                 TotalDraws (integer) with the total number of available posterior draws
%                         that could have been plotted.
%
%                 CurrChain (integer) with the number for the used sampling chain.
%
%                 DSGEModel (structure)  whose fields contain all required information
%                         about the model, such as input files, variable and sample
%                         data. Actual observations are not stored in this file.
%
%                 CurrINI (structure) with initialization information.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   DG-R/MPR
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2017 European Central Bank.
%
%                       First version: February 20, 2014.
%                        This version: February 10, 2017.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 21-02-2014: Updated the documentation.
%
% * 05-01-2015: Updated the documentation.
%
% * 20-01-2016: Updated the documentation.
%
% * 10-02-2017: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

NumIter = length(LogPost);
%
% Try to load the posterior mode estimates
%
ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
ErrorStr = '';
try;
   ModeData = load(ModeFile);
catch;
   ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' lasterr];
end;
if isempty(ErrorStr)==0;
   About(ErrorStr,'information','YADA - Bad MAT File',120,500,CurrINI);
   return;
end;
LogPostDensity = ModeData.LogPostDensity;
%
if (TotalDraws==NumIter)|(max(LogPost)>LogPostDensity);
   txt = strvcat('Would you like to save the original parameter values to file at the maximum for the log posterior kernel values?',' ',['NOTE: The log posterior mode value of the kernel is ' num2str(LogPostDensity,'%0.4f') ' while the largest kernel value for the posterior draws is ' num2str(max(LogPost),'%0.4f') '.']);
   answer = Query(txt,'question',400,'Raw Posterior Draws - Log Posterior Kernel Values',600,'no',CurrINI);
   if strcmp(lower(answer),'yes')==1;
      [maxLP,Ind] = max(LogPost);
      thetaMode = thetaPostSample(Ind,:)';
      ParameterNames = ModeData.ParameterNames.estimated;
      LogPostKernel = max(LogPost);
      file = FixFilePath([DSGEModel.OutputDirectory '\mode\LogPosteriorKernel-Theta-' DSGEModel.NameOfModel '.mat']);
      save(file,'thetaMode','ParameterNames','LogPostKernel');
      %
      txt = ['The original parameter values have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '".'];
      About(txt,'information','Raw Posterior Draws - Log Posterior Kernel Values',200,500,CurrINI);
   end;
end;
%
% Directory where the plots are stored
%
GraphDirectory = FixFilePath([DSGEModel.OutputDirectory '\graphs']);
pos = [(CurrINI.scrsz(3)-1000)/2 (CurrINI.scrsz(4)-680)/2 1000 650];
Xvalues = (1:NumIter)';
%
FigHandle = figure('Visible','off','Position',pos,...
                   'Name',['Log Posterior Kernel Values For Posterior Draws - Chain ' num2str(CurrChain,'%0.0f')]);
%
SetFigureProperties(FigHandle,CurrINI);
dh = plot(Xvalues,LogPost,'-','Color',[0.75 0 0],'LineWidth',1);
hold('on');
SetAxesFonts(CurrINI,'Title',['Log posterior kernel values with mode value at ' num2str(LogPostDensity,'%0.6f')]);
axis('tight');
ylim = get(gca,'YLim');
add_on = 0.1*(ylim(2)-ylim(1));
ylim = [ylim(1)-add_on ylim(2)+add_on];
%
set(gca,'YLim',ylim,'XLim',[0 NumIter]);
if TotalDraws==NumIter;
   bi = plot(NumBurnin*ones(1,2),ylim,'-','Color',[0 0 0]);
   hold('on');
end;
%
% plot the log posterior mode value of the log posterior kernel
%
pme = plot([0 NumIter],LogPostDensity*ones(1,2),'-','Color',[0 0 0]);
hold('off');
drawnow;
%
if DirExist(GraphDirectory)==1;
   file = FixFilePath([GraphDirectory '\LogPosteriorKernelValues-' int2str(NumIter) '.' int2str(TotalDraws) '-' int2str(CurrChain) '.fig']);
   saveas(FigHandle,file);
end;
%
% Fix the toolbar cdata
%
SetToolbarImages(FigHandle,CurrINI.images);
%
% Display the current figure
%
set(FigHandle,'Visible','on','CloseRequestFcn','delete(gcf); drawnow; pause(0.02);');
drawnow;

%
% end of RawLogPosteriorPlot.m
%
