function RawWeightsPosteriorPlots(Weights,lambda,TotalDraws,CurrChain,DSGEModel,CurrINI,controls)
% RawWeightsPosteriorPlots: Provides a plot of the weights values over the posterior draws for
%                           the SMC with likelihood tempering posterior sampler.
%
% USAGE:
%
%       RawWeightsPosteriorPlots(Weights,lambda,TotalDraws,CurrChain,DSGEModel,CurrINI)
%
% REQUIRED INPUT: Weights (vector) of dimension TotalDraws x 1 with all the log posterior values
%                         for the underlying log posterior draws.
%
%                 lambda (scalar) which is empty for DSGE models and determines the weight on the
%                         prior for DSGE-VAR models.
%
%                 TotalDraws (integer) with the total number of available posterior draws
%                         that could have been plotted.
%
%                 CurrChain (integer) with the number for the used sampling chain.
%
%                 DSGEModel (structure)  whose fields contain all required information
%                         about the model, such as input files, variable and sample
%                         data. Actual observations are not stored in this file.
%
%                 CurrINI (structure) with initialization information.
%
%                 controls (structure) with handles to all controls on the YADA GUI.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   DG-R/MPR
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2017 European Central Bank.
%
%                       First version: August 28, 2017.
%                        This version: September 11, 2017.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 29-08-2017: Updated the documentation.
%
% * 11-09-2017: Added the input variable "lambda".
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% number of tempering stages
%
TempStagesStr = get(controls.posterior.temperingstages,'String');
TempStages = str2num(StringTrim(TempStagesStr(DSGEModel.TemperingStagesValue,:)));
%
% Directory where the plots are stored
%
GraphDirectory = FixFilePath([DSGEModel.OutputDirectory '\graphs']);
pos = [(CurrINI.scrsz(3)-1000)/2 (CurrINI.scrsz(4)-680)/2 1000 650];
Xvalues = (1:TotalDraws)';
%
FigHandle = figure('Visible','off','Position',pos,...
                   'Name',['Weights For Posterior Draws (Particles) - Chain ' int2str(CurrChain)]);
%
SetFigureProperties(FigHandle,CurrINI);
%
% plot a line at unity
%
pu = plot([0 TotalDraws],ones(1,2),'-','Color',[0 0 0]);
hold('on');
drawnow;
da = plot(Xvalues,cumsum(Weights)./(1:TotalDraws)','-','Color',[0 0 0.75],'LineWidth',1);
hold('on');
dw = plot(Xvalues,Weights,'-','Color',[0.75 0 0],'LineWidth',1);
hold('on');
%
SetAxesFonts(CurrINI,'Title','Weights to posterior draws');
axis('tight');
ylim = get(gca,'YLim');
add_on = 0.1*(ylim(2)-ylim(1));
ylim = [ylim(1)-add_on ylim(2)+add_on];
%
set(gca,'YLim',ylim,'XLim',[0 TotalDraws]);
legend([dw da],strvcat('Particle weights','Acc. average weights'),1);
%
if DirExist(GraphDirectory)==1;
   if isempty(lambda)==1;
      file = FixFilePath([GraphDirectory '\PosteriorDrawsWeights-' int2str(TempStages) '.' int2str(DSGEModel.NumParamBlocks) '.' int2str(DSGEModel.ResamplingAlgorithm) '-' int2str(TotalDraws) '-' int2str(CurrChain) '.fig']);
   else;
      file = FixFilePath([GraphDirectory '\PosteriorDrawsWeights-' num2str(lambda,'%0.4f') '.' int2str(DSGEModel.BVARLags) '-' int2str(TempStages) '.' int2str(DSGEModel.NumParamBlocks) '.' int2str(DSGEModel.ResamplingAlgorithm) '-' int2str(TotalDraws) '-' int2str(CurrChain) '.fig']);
   end;
   saveas(FigHandle,file);
end;
%
% Fix the toolbar cdata
%
SetToolbarImages(FigHandle,CurrINI.images);
%
% Display the current figure
%
set(FigHandle,'Visible','on','CloseRequestFcn','delete(gcf); drawnow; pause(0.02);');
drawnow;

%
% end of RawWeightsPosteriorPlots.m
%
