function [action,positions,ObsVarGroupName,ObsVarGroupColor] = SetObsVarGroupsDLG(selector,ObsVarGroupNumber,VariableNames,positions,ObsVarGroupName,ObsVarGroupColor,CurrINI)
% SetObsVarGroupsDLG: Dialog for either selecting which observed variables belong to a certain group.
%
% USAGE:
%
%       [action,positions,ObsVarGroupName,ObsVarGroupColor] = SetObsVarGroupsDLG(selector,ObsVarGroupNumber,VariableNames,positions,ObsVarGroupName,ObsVarGroupColor,CurrINI)
%
% REQUIRED INPUT:  selector (string vector), with the possible values "init", "checkname", "quit"
%
% OPTIONAL INPUT:  ObsVarGroupNumber (integer), representing the group number.
%
%                  VariableNames (string matrix), with the names of the observed variables.
%
%                  positions (vector), with the selected entries from VariableNames.
%
%                  ObsVarGroupName (string), with the current name of the observed variable group.
%
%                  ObsVarGroupColor (1x3 vector) with a color code for the observed variables group.
%
%                  CurrINI (structure), with initialization information.
%
% Optional OUTPUT: action (string vector) taking the value "OK" or "Cancel". Depends on which button was
%                         clicked on.
%
%                  positions (vector), with the selected entries from VariableNames.
%
%                  ObsVarGroupName (string vector) with the new observed variable group name.
%
%                  ObsVarGroupColor (1x3 vector) with a color code for the observed variable group.
%
% NOTE: If selector = "init", then  all optional inputs and outputs are required.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   DG-R/MPR
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2017 European Central Bank.
%
%                       First version: September 24, 2009.
%                        This version: February 10, 2017.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 25-09-2009: Updated the documentation.
%
% * 06-01-2010: Updated the documentation.
%
% * 28-01-2010: Took into account that "IsWinXP" now supports separate output
%               values for Vista and Win 7.
%
% * 05-01-2011: Updated the documentation.
%
% * 27-01-2011: Took into account that "IsWinXP" now supports a separate output
%               value for unix and mac.
%
% * 04-01-2012: Updated the documentation.
%
% * 02-01-2013: Updated the documentation.
%
% * 24-01-2014: Updated the documentation.
%
% * 29-01-2014: YADA now assumes that the oldest used OS under Windows is XP.
%
% * 05-01-2015: Updated the documentation.
%
% * 08-06-2015: The code takes the CurrINI.GraphicsRoot field into account.
%
% * 20-01-2016: Updated the documentation.
%
% * 10-02-2017: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

if strcmp(selector,'init')==1;
   scrsz = CurrINI.scrsz;
   %
   % try to determine the width and the height
   %
   gui_cols = min(ceil(sqrt(size(VariableNames,1))),9);
   if gui_cols<=min(4,size(VariableNames,1));
      gui_cols = min(4,size(VariableNames,1));
   end;
   gui_rows = ceil(size(VariableNames,1)/gui_cols);
   width = (100*gui_cols)+50+(5*(gui_cols-1));
   if width<500;
      width = 500;
   end;
   height = min(scrsz(4)-100,110+(20*gui_rows)+(5*(gui_rows-1)));
else;
   SetObsVarGroupsGUI = findobj('Type','figure','Tag','SetObsVarGroupsDLG');
   controls = get(SetObsVarGroupsGUI,'UserData');
   SV = get(controls.cancel,'UserData');
end;
%
switch selector
   case 'init'
      SetObsVarGroupsGUI = figure('Color',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'FileName','SetObsVarGroupsDLG.m', ...
         'MenuBar','none', ...
         'Units','pixels', ...
         'Position',[(scrsz(3)-width)/2 (scrsz(4)-height)/2 width height], ...
         'Tag','SetObsVarGroupsDLG', ...
         'Resize','off', ...
         'Visible','off', ...
         'Name',['Select Variables - Group ' num2str(ObsVarGroupNumber)], ...
         'NumberTitle','off', ...
         'WindowStyle','modal', ...
         'CloseRequestFcn','SetObsVarGroupsDLG quit', ...
         'ToolBar','none');
      %
      if MatlabNumber>=7.0;
         set(SetObsVarGroupsGUI,'DockControl','off');
      end;
      %
      AxesBox([2 2 width-2 height-10],'Observed Variable Group Selection',100,[0.5 1],'on',CurrINI);
      %
      % checkboxes with all variables from the AiM output
      %
      chkbx = [];
      posmat = zeros(size(VariableNames,1),2);
      num = 0;
      for i=1:gui_rows;
         for j=1:gui_cols;
            num = num+1;
            if num>size(VariableNames,1);
               break;
            end;
            pos = [20+(105*(j-1)) height-35-(20*i)];
            new_chkbx = uicontrol('Units','pixels', ...
               'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
               'Position',[pos 100 20], ...
               'FontSize',CurrINI.gui_fontsize, ...
               'FontName',CurrINI.gui_fontname, ...
               'FontWeight',CurrINI.gui_fontweight, ...
               'FontAngle',CurrINI.gui_fontangle, ...
               'Style','checkbox', ...
               'Visible','on', ...
               'HorizontalAlignment','left', ...
               'String',StringTrim(VariableNames(num,:)), ...
               'Value',positions(num), ...
               'Enable','on', ...
               'Tag','UICheckBox');
            chkbx = [chkbx new_chkbx];
            posmat(num,:) = pos;
         end;
      end;
      SV.posmat = posmat;
      SV.chkbx = chkbx;
      %
      % edit dialog with the name of the group
      %
      controls.groupnametext = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[20 40 width-210 20], ...
         'String','OK', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'Enable','on', ...
         'Style','text', ...
         'String','Set observed variable group name:', ...
         'HorizontalAlignment','left', ...
         'Tag','GroupNameText');
      %
      controls.groupname = uicontrol('Units','pixels', ...
         'BackgroundColor',[1 1 1], ...
         'Position',[20 20 width-300 25], ...
         'String','OK', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'Style','edit', ...
         'String',strrep(ObsVarGroupName,'_',' '), ...
         'UserData',strrep(ObsVarGroupName,'_',' '), ...
         'Enable','on', ...
         'HorizontalAlignment','left', ...
         'Callback','SetObsVarGroupsDLG checkname', ...
         'Tag','GroupNameEdit');
      %
      % button to select a new group color
      %
      controls.color = uicontrol('Units','pixels', ...
         'BackgroundColor',ObsVarGroupColor, ...
         'ForegroundColor',1-ObsVarGroupColor, ...
         'Position',[width-255 20 75 25], ...
         'String','Group Color', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'Enable','on', ...
         'Callback','SetObsVarGroupsDLG color', ...
         'UserData',ObsVarGroupColor, ...
         'Tag','ButtonColor');
      %
      if MatlabNumber>=7;
         %
         % set the CData property
         %
         Cdat(:,:,1) = ones(25,75)*ObsVarGroupColor(1);
         Cdat(:,:,2) = ones(25,75)*ObsVarGroupColor(2);
         Cdat(:,:,3) = ones(25,75)*ObsVarGroupColor(3);
         set(controls.color,'CData',Cdat);
      end;
      %
      % Two buttons to select action
      %
      controls.ok = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[width-175 20 75 25], ...
         'String','OK', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'Enable','on', ...
         'Callback','SetObsVarGroupsDLG quit', ...
         'UserData','waiting', ...
         'Tag','ButtonOK');
      %
      controls.cancel = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[width-95 20 75 25], ...
         'String','Cancel', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'Enable','on', ...
         'UserData',SV, ...
         'Callback','SetObsVarGroupsDLG quit', ...
         'Tag','ButtonCancel');
      %
      set(SetObsVarGroupsGUI,'UserData',controls);
   case 'color'
      %
      % change the color
      %
      CurrColor = get(controls.color,'BackgroundColor');
      NewColor = uisetcolor(CurrColor,'Select Group Color');
      if (isnumeric(NewColor)==1)&(length(NewColor)==3);
         set(controls.color,'BackgroundColor',NewColor,'ForegroundColor',1-NewColor);
         if MatlabNumber>=7;
            %
            % set the CData property
            %
            Cdat(:,:,1) = ones(25,75)*NewColor(1);
            Cdat(:,:,2) = ones(25,75)*NewColor(2);
            Cdat(:,:,3) = ones(25,75)*NewColor(3);
            set(controls.color,'CData',Cdat);
         end;
      end;
   case 'checkname'
      %
      % checks to make sure the observed variable group name is a valid text string. $ signs are not allowed!
      %
      newname = StringTrim(strrep(get(controls.groupname,'String'),'$',''));
      if isempty(newname)==1;
         set(controls.groupname,'String',get(controls.groupname,'UserData'));
      else;
         set(controls.groupname,'String',newname,'UserData',newname);
      end;
   case 'quit'
      set(controls.ok,'UserData','done');
end;

if strcmp(selector,'init')==1;
   set(SetObsVarGroupsGUI,'Visible','on');
   drawnow;
   if MatlabNumber>=7;
      uicontrol(controls.ok);
   end;
   %
   waitfor(controls.ok,'UserData','done');
   if (isempty(get(SetObsVarGroupsGUI,'CurrentObject'))==1)|(strcmp(get(get(SetObsVarGroupsGUI,'CurrentObject'),'Type'),'uicontrol')==0);
      action = 'Cancel';
   else;
      action = get(get(SetObsVarGroupsGUI,'CurrentObject'),'String');
      if (isempty(action)==1)|(strcmp(action,'OK')==0);
         action = 'Cancel';
      end;
   end;
   SV = get(controls.cancel,'UserData');
   %
   % determine output
   %
   positions = [];
   for i=1:length(SV.chkbx);
      positions = [positions get(SV.chkbx(i),'Value')];
   end;
   ObsVarGroupName = strrep(get(controls.groupname,'UserData'),' ','_');
   if strcmp(lower(action),'ok')==1;
      ObsVarGroupColor = get(controls.color,'BackgroundColor');
   else;
      ObsVarGroupColor = get(controls.color,'UserData');
   end;
   delete(SetObsVarGroupsGUI);
   drawnow;
   pause(0.02);
end;

%
% end of SetObsVarGroupsDLG.m
%
