function SetYADAUIControls(maingui,DSGEModel,CurrentModelFile,ImageStructure)
% SetYADAUIControls: Sets properties of YADA's GUI objects based on the DSGEModel structure!
%
% USAGE:
%
%       SetYADAUIControls(controls,DSGEModel,CurrentModelFile,ImageStructure)
%
% REQUIRED INPUTS: maingui (handle) to the YADA main dialog, with userdata giving handles to all the
%                           GUI objects on the YADA main dialog.
%
%                  DSGEModel (structure) with fields whose values should be set as properties of the
%                            GUI objects in controls
%
%                  CurrentModelFile (string vector) with full path and name of the model file.
%
%                  ImageStructure (structure) with fields whose entries are data for images
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   DG-R/MPR
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2017 European Central Bank.
%
%                       First version: August 2, 2006.
%                        This version: September 14, 2017.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 09-08-2006: Added the impact of the DSGEModel fields "DAMaximumIterationsValue" and "UseDoublingAlgorithm".
%
% * 14-08-2006: Added the impact of the fields "ShowProgress" and "ShowProgressClock"
%
% * 28-08-2006: Took the DSGEModel field "InitializeKalman" into account.
%
% * 30-08-2006: Added the impact of "OverwriteDraws", "MarginalLikelihoodValue", "SequentialStartIteration",
%               "SequentialStepLength", "SequentialML", "CoverageStart", "CoverageIncrement", and "CoverageEnd".
%
% * 17-11-2006: Added the impact of "DAToleranceValue", "RiccatiMaxIterations", "RiccatiToleranceValue",
%               and "StateCovConst".
%
% * 20-11-2006: Added the effect of "UseOwnInitialState".
%
% * 23-11-2006: Took the "AiM Model File" item on the View menu into account.
%
% * 28-11-2006: The the fields "PosteriorDrawsValue", "SampleBatchValue", "ParallelChainsValue", "BurnInValue",
%               "MHInitialScaleFactor", "MHScaleFactor", and "RandomWeightValue" into account.
%
% * 07-12-2006: Added the fields "OptMaxIterationsValue", "OptToleranceValue", "MaximizeAlgorithmValue", "StepLengthHessian",
%               "NumberOfGridPoints", "FiniteDifferenceHessian", "CheckOptimum", "CheckTransformedOptimum", "GridWidth",
%               "IRHorizon", "PriorKernel", "KernelDensityValue", and "KernelDensityEstimator".
%
% * 15-12-2006: Made sure that the impact of the value of the fields "PriorKernel" and "PosteriorDensityValue"
%               are taken into account.
%
% * 18-12-2006: Made sure that the controls that depend on the field "RandomNumberValue" are set correctly.
%
% * 21-12-2006: The field "MaxForecastHorizon" is taken into account.
%
% * 22-12-2006: The field "FreqParameterReest" is taken into account.
%
% * 11-01-2007: The fields "BVARLags", "HarmonicLagDecayValue", "CrossEqTightnessValue", "OverallTightnessValue"
%               "PriorType", and "PriorDiffMeanValue" are taken into account.
%
% * 12-01-2007: The fields "OmegaPriorType", "WishartType", "VarianceTightnessValue", "WishartDFValue",
%               and "SteadyStatePriorFile" are taken into account.
%
% * 17-01-2007: The full impact of the field "PriorType" is now taken into account where the necessary controls
%               are now enabled/disabled.
%
% * 22-02-2007: The fields "NumPredPathsValue" and "PostDrawsUsageValue" are now taken into account.
%
% * 26-02-2007: The field "RandomizeDraws" is now taken into account.
%
% * 03-04-2007: The field "PostDrawsPercentValue" is now taken into account.
%
% * 10-04-2007: The field "InverseHessianEstimator" is now taken into account.
%
% * 16-04-2007: The field "MaxCorrelationValue" is now taken into account.
%
% * 23-08-2007: The field "AdjustPredictionPaths" is now taken into account.
%
% * 03-09-2007: The controls for selecting the estimation sample have been changed from edit controls
%               to popup controls. The function has been updated to take this change into account.
%
% * 25-10-2007: The field "FreqParameterReest" is no longer supported.
%
% * 09-11-2007: Updated the documentation.
%
% * 17-12-2007: The field "ShockControlMethod" is now taken into account.
%
% * 02-01-2008: The field "KalmanFirstObservation" is now taken into account properly.
%
% * 23-01-2008: Made sure that the string property of controls.forecast.maxhorizon is now reset to handle the
%               new behavior of the control.
%
% * 24-01-2008: The field "KsiUseCondData" is now taken into account.
%
% * 18-02-2008: The field "ModifiedHessian" is now taken into account.
%
% * 21-02-2008: Updated the documentation and added the input "CurrentModelFile".
%
% * 19-05-2008: The fields "BeginYear", "BeginPeriod", "EndYear", and "EndPeriod" are no longer attached to
%               any GUI controls. The field "AIMTolerance" is now also taken into account.
%
% * 23-05-2008: Updated the documentation.
%
% * 21-07-2008: Made sure that "PriorKernel" can only be 1 if either "YADAbetarnd" or "betarnd" exists
%               as m-files.
%
% * 19-12-2008: Updated the documentation.
%
% * 29-01-2009: Made sure that "InitializeHessian" is now taken into account.
%
% * 12-02-2009: Made sure that "ParameterCovMatrix" is now taken into account.
%
% * 13-03-2009: Removed check if "YADAbetarnd" and "betarnd" exist. The "YADARndBeta" now allow all
%               systems to get random draws from the beta distribution.
%
% * 11-08-2009: The "outputdir" controls are now also enabled if the data construction file exists.
%
% * 09-11-2009: The controls "controls.optimize.numbergridpoints", "controls.optimize.numbergridpointstext",
%               "controls.optimize.numbergridpoints", "controls.optimize.numbergridpointstext",
%               "controls.optimize.checktransformed", and "controls.optimize.modifiedhessian" are no longer
%               disabled when "DSGEModel.CheckOptimum" is 0. The "DSGEModel" field "PosteriorSampler" is now
%               taken into account.
%
% * 06-01-2010: Updated the documentation.
%
% * 14-12-2010: Made sure that "CsminwelExtraRuns", "ModelSolver" and "KalmanAlgorithm" are taken into account.
%
% * 04-01-2011: Updated the documentation.
%
% * 22-03-2011: Made sure that "ShockControlMethod" is only considered when shock control method control is
%               enabled on YADA's main dialog.
%
% * 04-01-2012: Updated the documentation.
%
% * 05-06-2012: Took the controls "controls.stateeqmenu" and "controls.stateeq" into account.
%
% * 20-08-2012: The field "ConfidenceRegionMethod" is now taken into account.
%
% * 02-01-2013: Updated the documentation.
%
% * 28-08-2013: The function now takes into account that the number of posterior draws and the number of
%               burnin draws can be a user selected number.
%
% * 24-01-2014: Updated the documentation.
%
% * 05-01-2015: Updated the documentation.
%
% * 28-08-2015: Took into account that the number of degrees of freedom should appear on the text string
%               when a Student-t posterior sampler is used.
%
% * 20-01-2016: Updated the documentation.
%
% * 10-02-2017: Updated the documentation.
%
% * 02-04-2017: The function now supports the fixed blocking RWM posterior sampler with a normal
%               proposal density.
%
% * 06-04-2017: Made sure RWM is used instead if RWMH.
%
% * 10-04-2017: The function now supports the fixed blocking RWM posterior sampler with a Student-t
%               proposal density.
%
% * 13-04-2017: The function now supports the random blocking RWM posterior sampler with a normal
%               proposal density.
%
% * 19-04-2017: The function now supports the random blocking RWM posterior sampler with a Student-t
%               proposal density.
%
% * 15-05-2017: Took into account that the handle controls.dsge.systempriorfile should receive the
%               string "DSGEModel.SystemPriorFile".
%
% * 10-08-2017: Took into account that controls.posterior.temperingstages gets DSGEModel.TemperingStagesValue
%               and that controls.posterior.temperinglambda gets DSGEModel.TemperingLambdaValue.
%
% * 11-08-2017: Took into account that controls.posterior.numparamblocks gets DSGEModel.NumParamBlocks,
%               that controls.posterior.mixeddistweight gets DSGEModel.MixedDistWeightValue, that
%               controls.posterior.smcinitialscale gets DSGEModel.SMCInitialScaleFactor, that
%               controls.posterior.targetacceptancerate gets DSGEModel.TargetAcceptanceRateValue, and that
%               controls.posterior.resamplingalgorithm gets DSGEModel.ResamplingAlgorithm.
%
% * 28-08-2017: Took into account that controls.posterior.smcmhsteps gets DSGEModel.SMCNumMHSteps.
%
% * 14-09-2017: The control controls.posterior.postdrawsusage is no longer visible for the SMC with
%               likelihood tempering algorithm.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

if strcmp(DSGEModel.NameOfModel,'')==0;
   if FileExist(CurrentModelFile)==1;
      set(maingui,'Name',['YADA - ' DSGEModel.NameOfModel ' - [' GetFilename(CurrentModelFile) ']']);
   else;
      set(maingui,'Name',['YADA - ' DSGEModel.NameOfModel]);
   end;
else;
   set(maingui,'Name','YADA');
end;
%
controls = get(maingui,'UserData');
set(controls.dsge.aimfile,'String',DSGEModel.AIMFile);
if FileExist(DSGEModel.AIMFile)==1;
   %
   % enable AiM parser controls
   %
   set(controls.runaim,'Enable','on');
   set(controls.runaimmenu,'Enable','on');
   set(controls.aimmodelinfo,'Enable','on');
else;
   set(controls.runaim,'Enable','off');
   set(controls.runaimmenu,'Enable','off');
   set(controls.aimmodelinfo,'Enable','off');
end;
if (FileExist(DSGEModel.AIMFile)==1)|(FileExist(DSGEModel.DataConstructionFile)==1);
   %
   % Enable output directory controls
   %
   set(controls.outputdirtext,'Enable','on');
   set(controls.outputdir,'Enable','on');
   set(controls.outputbutton,'Enable','on','CData',ImageStructure.browse);
else;
   set(controls.outputdirtext,'Enable','off');
   set(controls.outputdir,'Enable','off');
   set(controls.outputbutton,'Enable','off','CData',ImageStructure.browsedisable);
end;
%
if FileExist(DSGEModel.AIMDataFile)==1;
   %
   % Enable controls for selecting state variables and state shocks
   %
   set(controls.statevar,'Enable','on');
   set(controls.statevarmenu,'Enable','on');
   set(controls.stateeq,'Enable','on');
   set(controls.stateeqmenu,'Enable','on');
   set(controls.stateshock,'Enable','on');
   set(controls.stateshockmenu,'Enable','on');
else;
   set(controls.statevar,'Enable','off');
   set(controls.statevarmenu,'Enable','off');
   set(controls.stateeq,'Enable','off');
   set(controls.stateeqmenu,'Enable','off');
   set(controls.stateshock,'Enable','off');
   set(controls.stateshockmenu,'Enable','off');
end;
set(controls.dsge.datafile,'String',DSGEModel.DataConstructionFile);
if FileExist(DSGEModel.DataConstructionFile)==1;
   %
   % endable control for viewing data construction information
   %
   set(controls.datainfo,'Enable','on');
else;
   set(controls.datainfo,'Enable','off');
end;
set(controls.dsge.initialparameterfile,'String',DSGEModel.InitializeParameterFile);
set(controls.dsge.modelname,'String',DSGEModel.NameOfModel);
set(controls.dsge.measurefile,'String',DSGEModel.MeasurementEquationFile);
set(controls.outputdir,'String',DSGEModel.OutputDirectory);
set(controls.dsge.priorfile,'String',DSGEModel.PriorFile);
set(controls.dsge.priorfilesheet,'String',['Sheet: ' DSGEModel.PriorFileSheet]);
set(controls.dsge.systempriorfile,'String',DSGEModel.SystemPriorFile);
set(controls.dsge.runinitialfirst,'Value',DSGEModel.RunInitializeFirst);
if FileExist(DSGEModel.InitializeParameterFile)==1;
   set(controls.dsge.runinitialfirst,'Enable','on');
else;
   set(controls.dsge.runinitialfirst,'Enable','off');
end;
set(controls.dsge.modelsolver,'Value',DSGEModel.ModelSolver);
set(controls.dsge.kalmanalgorithm,'Value',DSGEModel.KalmanAlgorithm);
set(controls.dsge.aimtolerance,'Value',DSGEModel.AIMTolerance-1);
set(controls.dsge.subbeginperiod,'String',DSGEModel.PeriodStrMatrix,'Value',DSGEModel.SubBeginPeriodValue);
set(controls.dsge.subbeginyear,'String',DSGEModel.YearStrMatrix,'Value',DSGEModel.SubBeginYearValue);
set(controls.dsge.subendperiod,'String',DSGEModel.PeriodStrMatrix,'Value',DSGEModel.SubEndPeriodValue);
set(controls.dsge.subendyear,'String',DSGEModel.YearStrMatrix,'Value',DSGEModel.SubEndYearValue);
set(controls.dsge.updateparameterfile,'String',DSGEModel.UpdateParameterFile);
%
set(controls.dsge.doublingalgorithm,'Value',DSGEModel.DAMaximumIterationsValue);
set(controls.dsge.usedoublingalgorithm,'Value',DSGEModel.UseDoublingAlgorithm);
if DSGEModel.UseDoublingAlgorithm==2;
   set(controls.dsge.doublingalgorithmtext,'Enable','on');
   set(controls.dsge.doublingalgorithm,'Enable','on');
   set(controls.dsge.doublingtolerancetext,'Enable','on');
   set(controls.dsge.doublingtolerance,'Enable','on');
   set(controls.dsge.statecovariancetext,'Enable','off');
   set(controls.dsge.statecovariance,'Enable','off');
elseif DSGEModel.UseDoublingAlgorithm==3;
   set(controls.dsge.doublingalgorithmtext,'Enable','off');
   set(controls.dsge.doublingalgorithm,'Enable','off');
   set(controls.dsge.doublingtolerancetext,'Enable','off');
   set(controls.dsge.doublingtolerance,'Enable','off');
   set(controls.dsge.statecovariancetext,'Enable','on');
   set(controls.dsge.statecovariance,'Enable','on');
else;
   set(controls.dsge.doublingalgorithmtext,'Enable','off');
   set(controls.dsge.doublingalgorithm,'Enable','off');
   set(controls.dsge.doublingtolerancetext,'Enable','off');
   set(controls.dsge.doublingtolerance,'Enable','off');
   set(controls.dsge.statecovariancetext,'Enable','off');
   set(controls.dsge.statecovariance,'Enable','off');
end;
set(controls.dsge.progress,'Value',DSGEModel.ShowProgress);
if DSGEModel.ShowProgress==1;
   set(controls.dsge.progressclock,'Enable','on');
else;
   set(controls.dsge.progressclock,'Enable','off');
end;
set(controls.dsge.progressclock,'Value',DSGEModel.ShowProgressClock);
[FirstPeriod,LastPeriod,EstSample] = CreateSubSample(DSGEModel);
if (DSGEModel.InitializeKalman>1)&(DSGEModel.KalmanFirstObservation==1);
   DSGEModel.KalmanFirstObservation = round(((DSGEModel.InitializeKalman-1)/100)*(LastPeriod-FirstPeriod+1))+1;
end;
set(controls.dsge.initializekalman,'String',EstSample,'Value',DSGEModel.KalmanFirstObservation);
set(controls.dsge.overwritedraws,'Value',DSGEModel.OverwriteDraws);
set(controls.dsge.marginallikelihood,'Value',DSGEModel.MarginalLikelihoodValue);
set(controls.dsge.sequentialstart,'Value',DSGEModel.SequentialStartIteration);
set(controls.dsge.sequentialstep,'Value',DSGEModel.SequentialStepLength);
set(controls.dsge.sequentialml,'Value',DSGEModel.SequentialML);
set(controls.dsge.coveragestart,'Value',DSGEModel.CoverageStart);
set(controls.dsge.coverageincrement,'Value',DSGEModel.CoverageIncrement);
set(controls.dsge.coverageend,'Value',DSGEModel.CoverageEnd);
set(controls.dsge.doublingtolerance,'Value',DSGEModel.DAToleranceValue);
set(controls.tools.riccatiiteration,'Value',DSGEModel.RiccatiMaxIterations);
set(controls.tools.riccatitolerance,'Value',DSGEModel.RiccatiToleranceValue);
set(controls.dsge.statecovariance,'Value',DSGEModel.StateCovConst);
set(controls.dsge.useowninitialstate,'Value',DSGEModel.UseOwnInitialState);
set(controls.dsge.allowunitroot,'Value',DSGEModel.AllowUnitRoot);
if (isempty(DSGEModel.StudenttDegFree)==0)&(DSGEModel.PosteriorSampler==3);
   StudenttString = ['Random Walk Metropolis, t proposal, ' int2str(DSGEModel.StudenttDegFree) ' df'];
else;
   StudenttString = 'Random Walk Metropolis, t proposal';
end;
if (isempty(DSGEModel.FixedNumParamBlocks)==0)&(DSGEModel.PosteriorSampler==4);
   FixedBlockNormalString = [int2str(DSGEModel.FixedNumParamBlocks) ' block RWM, normal proposal'];
else;
   FixedBlockNormalString = 'Fixed block RWM, normal proposal';
end;
if (isempty(DSGEModel.FixedNumParamBlocks)==0)&(isempty(DSGEModel.StudenttDegFree)==0)&(DSGEModel.PosteriorSampler==5);
   FixedBlockStudentString = [int2str(DSGEModel.FixedNumParamBlocks) ' block RWM, t proposal, ' int2str(DSGEModel.StudenttDegFree) ' df'];
else;
   FixedBlockStudentString = 'Fixed block RWM, t proposal';
end;
if (length(DSGEModel.BlockSize)==2)&(DSGEModel.PosteriorSampler==6);
   RandomBlockNormalString = ['Random block (' int2str(DSGEModel.BlockSize(1)) ' to ' int2str(DSGEModel.BlockSize(2)) ') RWM, normal proposal'];
else;
   RandomBlockNormalString = 'Random block RWM, normal proposal';
end;
if (length(DSGEModel.BlockSize)==2)&(isempty(DSGEModel.StudenttDegFree)==0)&(DSGEModel.PosteriorSampler==7);
   RandomBlockStudentString = ['Random block (' int2str(DSGEModel.BlockSize(1)) ' to ' int2str(DSGEModel.BlockSize(2)) ') RWM, t proposal, ' int2str(DSGEModel.StudenttDegFree) ' df'];
else;
   RandomBlockStudentString = 'Random block RWM, t proposal';
end;
set(controls.posterior.posteriorsampler,'String',['Random Walk Metropolis, normal proposal|Slice sampler|' StudenttString '|' FixedBlockNormalString '|' FixedBlockStudentString '|' RandomBlockNormalString '|' RandomBlockStudentString '|Sequential Monte Carlo, likelihood tempering'],'Value',DSGEModel.PosteriorSampler);
DrawsStr = cellstr(get(controls.posterior.draws,'String'));
if isempty(DSGEModel.PosteriorDraws)==1;
   DrawsStr{174} = 'Other';
else;
   DrawsStr{174} = int2str(DSGEModel.PosteriorDraws);
end;
set(controls.posterior.draws,'Value',DSGEModel.PosteriorDrawsValue,'String',char(DrawsStr));
set(controls.posterior.batch,'Value',DSGEModel.SampleBatchValue);
set(controls.posterior.temperingstages,'Value',DSGEModel.TemperingStagesValue)
set(controls.posterior.chains,'Value',DSGEModel.ParallelChainsValue);
DrawsStr = cellstr(get(controls.posterior.burnin,'String'));
if isempty(DSGEModel.BurnIn)==1;
   DrawsStr{142} = 'Other';
else;
   DrawsStr{142} = int2str(DSGEModel.BurnIn);
end;
set(controls.posterior.burnin,'Value',DSGEModel.BurnInValue,'String',char(DrawsStr));
set(controls.posterior.temperinglambda,'Value',DSGEModel.TemperingLambdaValue);
set(controls.posterior.invhessian,'Value',DSGEModel.InverseHessianEstimator,'String',['Output from optimization routine|Fit quadratic to evaluated log posterior|Finite difference|My estimate ("' GetName(DSGEModel.ParameterCovMatrix) '")']);
set(controls.posterior.numparamblocks,'Value',DSGEModel.NumParamBlocks);
set(controls.posterior.smcmhsteps,'Value',DSGEModel.SMCNumMHSteps);
set(controls.posterior.maxcorrelation,'Value',DSGEModel.MaxCorrelationValue);
set(controls.posterior.mixeddistweight,'Value',DSGEModel.MixedDistWeightValue);
set(controls.posterior.initialscale,'Value',DSGEModel.MHInitialScaleFactor);
set(controls.posterior.smcinitialscale,'Value',DSGEModel.SMCInitialScaleFactor);
set(controls.posterior.scalefactor,'Value',DSGEModel.MHScaleFactor);
set(controls.posterior.targetacceptancerate,'Value',DSGEModel.TargetAcceptanceRateValue);
set(controls.posterior.randomweight,'Value',DSGEModel.RandomWeightValue);
set(controls.posterior.resamplingalgorithm,'Value',DSGEModel.ResamplingAlgorithm);
set(controls.posterior.resamplingthreshold,'Value',DSGEModel.ResamplingThresholdValue);
set(controls.posterior.postdrawsusage,'Value',DSGEModel.PostDrawsUsageValue);
set(controls.posterior.usepostdraws,'Value',DSGEModel.PostDrawsPercentValue);
%
set(controls.optimize.maximumiterations,'Value',DSGEModel.OptMaxIterationsValue);
set(controls.optimize.initcsminwel,'Value',DSGEModel.InitializeHessian);
set(controls.optimize.csminwelextraruns,'Value',DSGEModel.CsminwelExtraRuns);
set(controls.optimize.tolerance,'Value',DSGEModel.OptToleranceValue);
set(controls.optimize.maxroutine,'Value',DSGEModel.MaximizeAlgorithmValue);
set(controls.optimize.steplength,'Value',DSGEModel.StepLengthHessian);
set(controls.optimize.numbergridpoints,'Value',DSGEModel.NumberOfGridPoints);
set(controls.optimize.finitediffhessian,'Value',DSGEModel.FiniteDifferenceHessian);
set(controls.optimize.checkoptimum,'Value',DSGEModel.CheckOptimum);
set(controls.optimize.checktransformed,'Value',DSGEModel.CheckTransformedOptimum);
set(controls.optimize.modifiedhessian,'Value',DSGEModel.ModifiedHessian);
set(controls.optimize.gridwidth,'Value',DSGEModel.GridWidth);
set(controls.tools.irhorizon,'Value',DSGEModel.IRHorizon);
set(controls.tools.confidencemethod,'Value',DSGEModel.ConfidenceRegionMethod);
set(controls.graphics.priorkernel,'Value',DSGEModel.PriorKernel);
set(controls.graphics.kerneldensity,'Value',DSGEModel.KernelDensityValue);
set(controls.graphics.posteriorkernel,'Value',DSGEModel.PosteriorDensityValue);
set(controls.tools.randomnumber,'Value',DSGEModel.RandomNumberValue);
set(controls.tools.randomizedraws,'Value',DSGEModel.RandomizeDraws);
set(controls.forecast.maxhorizon,'Value',DSGEModel.MaxForecastHorizon);
ForecastStr = get(controls.forecast.maxhorizon,'UserData');
if strcmp(lower(DSGEModel.DataFrequency),'q')==1;
   set(controls.forecast.maxhorizon,'String',ForecastStr.quarter);
elseif strcmp(lower(DSGEModel.DataFrequency),'m')==1;
   set(controls.forecast.maxhorizon,'String',ForecastStr.month);
else;
   set(controls.forecast.maxhorizon,'String',ForecastStr.annual);
end;
if strcmp(get(controls.forecast.shockcontrol,'Enable'),'on')==1;
   set(controls.forecast.shockcontrol,'Value',DSGEModel.ShockControlMethod);
else;
   set(controls.forecast.shockcontrol,'Value',1);
end;
%set(controls.forecast.freqreest,'Value',DSGEModel.FreqParameterReest);
set(controls.forecast.predpaths,'Value',DSGEModel.NumPredPathsValue);
set(controls.forecast.adjustpredictionpaths,'Value',DSGEModel.AdjustPredictionPaths);
set(controls.forecast.ksiuseconddata,'Value',DSGEModel.KsiUseCondData);
if DSGEModel.PriorKernel==1;
   set(controls.graphics.kerneldensitytext,'Enable','on');
   set(controls.graphics.kerneldensity,'Enable','on');
else;
   set(controls.graphics.kerneldensitytext,'Enable','off');
   set(controls.graphics.kerneldensity,'Enable','off');
end;
set(controls.bvar.steadystatefile,'String',DSGEModel.SteadyStatePriorFile);
%
set(controls.bvar.lag,'Value',DSGEModel.BVARLags);
set(controls.bvar.priorlevelmean,'Value',DSGEModel.PriorLevelMeanValue);
set(controls.bvar.priordiffmean,'Value',DSGEModel.PriorDiffMeanValue);
set(controls.bvar.priortype,'Value',DSGEModel.PriorType);
set(controls.bvar.overalltightness,'Value',DSGEModel.OverallTightnessValue);
set(controls.bvar.crosseqtightness,'Value',DSGEModel.CrossEqTightnessValue);
set(controls.bvar.harmoniclagdecay,'Value',DSGEModel.HarmonicLagDecayValue);
if DSGEModel.PriorType~=3;
   set(controls.bvar.priorlevelmeantext,'Enable','on');
   set(controls.bvar.priorlevelmean,'Enable','on');
   set(controls.bvar.priordiffmeantext,'Enable','on');
   set(controls.bvar.priordiffmean,'Enable','on');
   set(controls.bvar.overalltightnesstext,'Enable','on');
   set(controls.bvar.overalltightness,'Enable','on');
   set(controls.bvar.crosseqtightnesstext,'Enable','on');
   set(controls.bvar.crosseqtightness,'Enable','on');
   set(controls.bvar.harmoniclagdecaytext,'Enable','on');
   set(controls.bvar.harmoniclagdecay,'Enable','on');
else;
   set(controls.bvar.priorlevelmeantext,'Enable','off');
   set(controls.bvar.priorlevelmean,'Enable','off');
   set(controls.bvar.priordiffmeantext,'Enable','off');
   set(controls.bvar.priordiffmean,'Enable','off');
   set(controls.bvar.overalltightnesstext,'Enable','off');
   set(controls.bvar.overalltightness,'Enable','off');
   set(controls.bvar.crosseqtightnesstext,'Enable','off');
   set(controls.bvar.crosseqtightness,'Enable','off');
   set(controls.bvar.harmoniclagdecaytext,'Enable','off');
   set(controls.bvar.harmoniclagdecay,'Enable','off');
end;
set(controls.bvar.omegapriortype,'Value',DSGEModel.OmegaPriorType);
set(controls.bvar.wisharttype,'Value',DSGEModel.WishartType);
set(controls.bvar.variancetightness,'Value',DSGEModel.VarianceTightnessValue);
set(controls.bvar.wishartdf,'Value',DSGEModel.WishartDFValue);
if DSGEModel.OmegaPriorType==1;
   set(controls.bvar.wisharttypetext,'Enable','off');
   set(controls.bvar.wisharttype,'Enable','off');
   set(controls.bvar.variancetightnesstext,'Enable','off');
   set(controls.bvar.variancetightness,'Enable','off');
   set(controls.bvar.wishartdftext,'Enable','off');
   set(controls.bvar.wishartdf,'Enable','off');
else;
   set(controls.bvar.wisharttypetext,'Enable','on');
   set(controls.bvar.wisharttype,'Enable','on');
   set(controls.bvar.wishartdftext,'Enable','on');
   set(controls.bvar.wishartdf,'Enable','on');
   if DSGEModel.WishartType==1;
      set(controls.bvar.variancetightnesstext,'Enable','off');
      set(controls.bvar.variancetightness,'Enable','off');
   else;
      set(controls.bvar.variancetightnesstext,'Enable','on');
      set(controls.bvar.variancetightness,'Enable','on');
   end;
end;
%
% check if we should make sure that the controls on the options tab
% are shown correctly if that tab is the current tab
%
if get(controls.tab1,'UserData')==3;
	if DSGEModel.PosteriorSampler<=7;
   	set(controls.posterior.drawstext,'Visible','on');
   	set(controls.posterior.batchtext,'Visible','on');
   	set(controls.posterior.batch,'Visible','on');
   	set(controls.posterior.burnintext,'Visible','on');
   	set(controls.posterior.burnin,'Visible','on');
   	set(controls.posterior.invhessiantext,'Visible','on');
   	set(controls.posterior.invhessian,'Visible','on');
   	set(controls.posterior.maxcorrelationtext,'Visible','on');
   	set(controls.posterior.maxcorrelation,'Visible','on');
   	set(controls.posterior.initialscaletext,'Visible','on');
   	set(controls.posterior.initialscale,'Visible','on');
   	set(controls.posterior.scalefactortext,'Visible','on');
   	set(controls.posterior.scalefactor,'Visible','on');
   	set(controls.posterior.randomweighttext,'Visible','on');
   	set(controls.posterior.randomweight,'Visible','on');
   	set(controls.posterior.usepostdrawstext,'Visible','on');
   	set(controls.posterior.usepostdraws,'Visible','on');
   	set(controls.posterior.postdrawsusagetext,'Visible','on');
      set(controls.posterior.postdrawsusage,'Visible','on');
   	set(controls.posterior.smcdrawstext,'Visible','off');
   	set(controls.posterior.temperingstagestext,'Visible','off');
   	set(controls.posterior.temperingstages,'Visible','off');
   	set(controls.posterior.temperinglambdatext,'Visible','off');
   	set(controls.posterior.temperinglambda,'Visible','off');
   	set(controls.posterior.numparamblockstext,'Visible','off');
   	set(controls.posterior.numparamblocks,'Visible','off');
   	set(controls.posterior.smcmhstepstext,'Visible','off');
   	set(controls.posterior.smcmhsteps,'Visible','off');
   	set(controls.posterior.mixeddistweighttext,'Visible','off');
   	set(controls.posterior.mixeddistweight,'Visible','off');
   	set(controls.posterior.smcinitialscaletext,'Visible','off');
   	set(controls.posterior.smcinitialscale,'Visible','off');
   	set(controls.posterior.targetacceptanceratetext,'Visible','off');
   	set(controls.posterior.targetacceptancerate,'Visible','off');
   	set(controls.posterior.resamplingalgorithmtext,'Visible','off');
   	set(controls.posterior.resamplingalgorithm,'Visible','off');
   	set(controls.posterior.resamplingthresholdtext,'Visible','off');
      set(controls.posterior.resamplingthreshold,'Visible','off');
   elseif DSGEModel.PosteriorSampler==8;
   	set(controls.posterior.smcdrawstext,'Visible','on');
   	set(controls.posterior.temperingstagestext,'Visible','on');
   	set(controls.posterior.temperingstages,'Visible','on');
   	set(controls.posterior.temperinglambdatext,'Visible','on');
   	set(controls.posterior.temperinglambda,'Visible','on');
   	set(controls.posterior.numparamblockstext,'Visible','on');
   	set(controls.posterior.numparamblocks,'Visible','on');
   	set(controls.posterior.smcmhstepstext,'Visible','on');
   	set(controls.posterior.smcmhsteps,'Visible','on');
   	set(controls.posterior.mixeddistweighttext,'Visible','on');
   	set(controls.posterior.mixeddistweight,'Visible','on');
   	set(controls.posterior.smcinitialscaletext,'Visible','on');
   	set(controls.posterior.smcinitialscale,'Visible','on');
   	set(controls.posterior.targetacceptanceratetext,'Visible','on');
   	set(controls.posterior.targetacceptancerate,'Visible','on');
   	set(controls.posterior.resamplingalgorithmtext,'Visible','on');
   	set(controls.posterior.resamplingalgorithm,'Visible','on');
   	set(controls.posterior.resamplingthresholdtext,'Visible','on');
      set(controls.posterior.resamplingthreshold,'Visible','on');
   	set(controls.posterior.drawstext,'Visible','off');
   	set(controls.posterior.batchtext,'Visible','off');
   	set(controls.posterior.batch,'Visible','off');
   	set(controls.posterior.burnintext,'Visible','off');
   	set(controls.posterior.burnin,'Visible','off');
   	set(controls.posterior.invhessiantext,'Visible','off');
   	set(controls.posterior.invhessian,'Visible','off');
   	set(controls.posterior.maxcorrelationtext,'Visible','off');
   	set(controls.posterior.maxcorrelation,'Visible','off');
   	set(controls.posterior.initialscaletext,'Visible','off');
   	set(controls.posterior.initialscale,'Visible','off');
   	set(controls.posterior.scalefactortext,'Visible','off');
   	set(controls.posterior.scalefactor,'Visible','off');
   	set(controls.posterior.randomweighttext,'Visible','off');
   	set(controls.posterior.randomweight,'Visible','off');
   	set(controls.posterior.usepostdrawstext,'Visible','off');
   	set(controls.posterior.usepostdraws,'Visible','off');
   	set(controls.posterior.postdrawsusagetext,'Visible','off');
      set(controls.posterior.postdrawsusage,'Visible','off');
   end;
   drawnow;
end;

%
% end of SetYADAUIControls.m
%
