function StateVarDecompDLG(selector,StateVarStructure,NameStr,CurrINI,DSGEModel)
% StateVarDecompDLG: Displays graphs of state variable decomposition.
%
% USAGE:
%
%       StateVarDecompDLG(selector,StateVarStructure,NameStr,CurrINI,DSGEModel)
%
% REQUIRED INPUT: selector (string vector) taking on the values "init", "samplebegin", "sampleend" "showgraph",
%                          and "done".
%
% OPTIONAL INPUT: StateVarStructure (structure)  with fields "Y", "Yhat" containing the observed variables and
%                          the 1-step ahead forecasts of these, "Ksitt1", "Ksitt", and "KsitT" with the forecasted,
%                          updated, and smoothed values for the state variables. Furthermore, the field "MaxEigenvalue"
%                          contains the largest eigenvalue of the state transition matrix F, "etatT" the smooth
%                          projections of the unique state shocks, "wtT" the smooth projections of the measurement
%                          errors, and "wtNames" the names of the measurement errors (determined directly from the
%                          observed variable's names. The field "wthT" contains all the measurement errors.
%                          Furthermore, "TrainSample" is a boolean variable that indicates if a training sample
%                          was employed for the Kalman filter. The field "X" holds the exogneous variables, while
%                          the field "XiInit" contains the matrix with typical element H'*(F^t)*Ksi0T that is used
%                          for observed variable decompositions. Similarly, the field "etaDecomp" contains a 3D
%                          matrix with the parts of the observed variable decomposition that are explained by the
%                          q economic shocks. The field "KeepVar" contains the column number of B0 that are non-zero.
%                          The parameter matrices of the measurement equation are given by the fields "A", "H", and
%                          "R", while the fields "F" a "B0" contain the parameter matrices of the state equation.
%
%                 NameStr (string vector) taking on the values "Posterior Mode" or "Initial Values".
%
%                 CurrINI (structure), with initialization information.
%
%                 DSGEModel (structure) whose fields contain all required information about the model, such as
%                          input files, variable and sample data. Actual observations are not stored in this file.
%
% NOTE: When selector = "init", then all optional input arguments are required!
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   DG-R/MPR
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2017 European Central Bank.
%
%                       First version: January 4, 2007.
%                        This version: February 10, 2017.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 05-01-2007: Fixed a bug that made Ymat contain rows of zeros for shock groups that are zero because
%               all individual shocks are zero by assumption.
%
% * 27-02-2007: Added the call "SetAxesFonts(CurrINI,legHandle)" after the legend call.
%
% * 09-11-2007: Added the call to "SetXTickIntegers".
%
% * 02-01-2008: Removed code using "DSGEModel.InitializeKalman".
%
% * 10-01-2008: Changed the last drawnow call to pause(0.02).
%
% * 04-03-2008: Added the sample select code.
%
% * 19-03-2008: Added the option to save the data to file.
%
% * 23-05-2008: Updated the documentation.
%
% * 19-12-2008: Updated the documentation.
%
% * 25-02-2009: Added the option to have black edges when drawing the bars.
%
% * 06-01-2010: Updated the documentation.
%
% * 25-05-2010: Corrected a text issue on the dialog.
%
% * 05-01-2011: Updated the documentation.
%
% * 11-01-2011: Added support for unix and mac paths.
%
% * 21-04-2011: Added the options to decompose shock groups into individual shocks and
%               to use shock aliases instead of shock names.
%
% * 20-12-2011: When the number of shock groups is equal to the number of shocks, the function
%               can now use "ShockColors" and "ShockAliases" when "ShockGroupColors" is empty
%               and "ShockGroupNames" is equal to the "StateShockNames". It is now also possible
%               to remove the initial state share from the graphs.
%
% * 04-01-2012: Updated the documentation.
%
% * 06-01-2012: Made sure that ShockGroupColors is only used when it has as many rows as there are shock
%               groups.
%
% * 02-01-2013: Updated the documentation.
%
% * 30-01-2013: The user can now choose between shock group and individual shocks.
%
% * 18-02-2013: Added the option to also produce a decomposition of the change relative to the start
%               period (fixed point decomposition).
%
% * 24-01-2014: Updated the documentation.
%
% * 05-01-2015: Updated the documentation.
%
% * 08-06-2015: The code takes the CurrINI.GraphicsRoot field into account.
%
% * 10-09-2015: The function is now ready for Matlab 2014b and later.
%
% * 20-01-2016: Updated the documentation.
%
% * 10-02-2017: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

if strcmp(selector,'init')==1;
   DSGEModel.NameStr = NameStr;
   DSGEModel.StateVarStructure = StateVarStructure;
else;
   GraphGUI = findobj('Type','figure','Tag','StateVarDecompDLG');
   controls = get(GraphGUI,'UserData');
   CurrINI = get(controls.variables,'UserData');
   DSGEModel = get(controls.variablestext,'UserData');
end;
%
switch selector;
   case 'init';
      GraphGUI = figure('Color',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'FileName','StateVarDecompDLG.m', ...
         'MenuBar','none', ...
         'PaperUnits','points', ...
         'Units','pixels', ...
         'Position',[(CurrINI.scrsz(3)-500)/2 (CurrINI.scrsz(4)-250)/2 500 250], ...
         'Tag','StateVarDecompDLG', ...
         'Visible','off', ...
         'Resize','off', ...
         'Name',['State Variable Decompositions - ' NameStr], ...
         'NumberTitle','off', ...
         'CloseRequestFcn','StateVarDecompDLG done', ...
         'ToolBar','none');

      if MatlabNumber>=7.0;
         set(GraphGUI,'DockControl','off');
      end;
      %
      % Outer axis
      %
      AxesBox([2 2 498 240],'Graphics',45,[0.5 1],'on',CurrINI);
      %
      % Check where the sample of data actually begins
      %
      [FirstPeriod,LastPeriod] = CreateSubSample(DSGEModel);
      [NewStartYear,NewStartPeriod] = AdjustSampleStart(DSGEModel.SubBeginYear,DSGEModel.SubBeginPeriod,DSGEModel.DataFrequency,DSGEModel.KalmanFirstObservation-1);
      %
      % Select the sample
      %
      OldDSGEModel = DSGEModel;
      DSGEModel.SubBeginYear = NewStartYear;
      DSGEModel.SubBeginPeriod = NewStartPeriod;
      %
      [FirstPeriod,LastPeriod,SubSampleStrMat] = CreateSubSample(DSGEModel);
      %
      DSGEModel = OldDSGEModel;
      controls.sampletxt = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[24 195 360 20], ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'Style','text', ...
         'String','Selected sample:', ...
         'HorizontalAlignment','left', ...
         'Tag','SampleText');
      %
      % sample start and sample end
      %
      controls.samplebeg = uicontrol('Units','pixels', ...
         'BackgroundColor',[1 1 1], ...
         'Position',[185 200 87 20], ...
         'Style','popupmenu', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'String',SubSampleStrMat, ...
         'HorizontalAlignment','center', ...
         'Callback','StateVarDecompDLG samplebegin', ...
         'Tag','SampleBeginPopup',...
         'Value',1);
      %
      controls.sampleend = uicontrol('Units','pixels', ...
         'BackgroundColor',[1 1 1], ...
         'Position',[278 200 87 20], ...
         'Style','popupmenu', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'String',SubSampleStrMat, ...
         'HorizontalAlignment','center', ...
         'Callback','StateVarDecompDLG sampleend', ...
         'Tag','SampleEndPopup',...
         'Value',size(SubSampleStrMat,1));
      %
      % Select shock
      %
      controls.variablestext = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[24 165 180 20], ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'Style','text', ...
         'UserData',DSGEModel, ...
         'String','Select state variable:', ...
         'HorizontalAlignment','left', ...
         'Tag','ObsVarText');
      %
      controls.variables = uicontrol('Units','pixels', ...
         'BackgroundColor',[1 1 1], ...
         'Position',[185 170 180 20], ...
         'Style','popupmenu', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'UserData',CurrINI, ...
         'String',DSGEModel.StateVariableNames, ...
         'HorizontalAlignment','center', ...
         'Tag','ObsVarPopup',...
         'Value',1);
      %
      controls.useshockgroups = uicontrol('Units','pixels', ...
            'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
            'Position',[24 140 366 20], ...
            'FontSize',CurrINI.gui_fontsize, ...
            'FontName',CurrINI.gui_fontname, ...
            'FontWeight',CurrINI.gui_fontweight, ...
            'FontAngle',CurrINI.gui_fontangle, ...
            'Style','checkbox', ...
            'Visible','on', ...
            'HorizontalAlignment','left', ...
            'String','Use shock groups', ...
            'Callback','StateVarDecompDLG useshockgroups', ...
            'Value',0, ...
            'Enable','on', ...
            'Tag','UseShockGroupCheckBox');
      %
      controls.decomposegroups = uicontrol('Units','pixels', ...
            'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
            'Position',[24 120 366 20], ...
            'FontSize',CurrINI.gui_fontsize, ...
            'FontName',CurrINI.gui_fontname, ...
            'FontWeight',CurrINI.gui_fontweight, ...
            'FontAngle',CurrINI.gui_fontangle, ...
            'Style','checkbox', ...
            'Visible','on', ...
            'HorizontalAlignment','left', ...
            'String','Decompose shock groups into individual shocks', ...
            'Callback','StateVarDecompDLG fixcontrols', ...
            'Value',0, ...
            'Enable','off', ...
            'Tag','DecomposeShockGroupCheckBox');
      %
      controls.useshockaliases = uicontrol('Units','pixels', ...
            'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
            'Position',[40 100 366 20], ...
            'FontSize',CurrINI.gui_fontsize, ...
            'FontName',CurrINI.gui_fontname, ...
            'FontWeight',CurrINI.gui_fontweight, ...
            'FontAngle',CurrINI.gui_fontangle, ...
            'Style','checkbox', ...
            'Visible','on', ...
            'HorizontalAlignment','left', ...
            'String','Use shock aliases instead of shock names', ...
            'Value',0, ...
            'Enable','off', ...
            'Tag','UseShockAliasesCheckBox');
      %
      controls.skipstateerrors = uicontrol('Units','pixels', ...
            'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
            'Position',[24 80 366 20], ...
            'FontSize',CurrINI.gui_fontsize, ...
            'FontName',CurrINI.gui_fontname, ...
            'FontWeight',CurrINI.gui_fontweight, ...
            'FontAngle',CurrINI.gui_fontangle, ...
            'Style','checkbox', ...
            'Visible','on', ...
            'HorizontalAlignment','left', ...
            'String','Skip initial state share', ...
            'Value',0, ...
            'Enable','on', ...
            'Tag','SkipStateErrorsCheckBox');
      %
      controls.fixedpointdecomp = uicontrol('Units','pixels', ...
            'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
            'Position',[24 60 366 20], ...
            'FontSize',CurrINI.gui_fontsize, ...
            'FontName',CurrINI.gui_fontname, ...
            'FontWeight',CurrINI.gui_fontweight, ...
            'FontAngle',CurrINI.gui_fontangle, ...
            'Style','checkbox', ...
            'Visible','on', ...
            'HorizontalAlignment','left', ...
            'String','Display decomposition of change relative to sample start period', ...
            'Value',0, ...
            'Enable','on', ...
            'Tag','FixedPointChangeCheckBox');
      %
      controls.blackedgebars = uicontrol('Units','pixels', ...
            'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
            'Position',[24 40 366 20], ...
            'FontSize',CurrINI.gui_fontsize, ...
            'FontName',CurrINI.gui_fontname, ...
            'FontWeight',CurrINI.gui_fontweight, ...
            'FontAngle',CurrINI.gui_fontangle, ...
            'Style','checkbox', ...
            'Visible','on', ...
            'HorizontalAlignment','left', ...
            'String','Draw bars with black edges', ...
            'Value',1, ...
            'Enable','on', ...
            'Tag','BlackEdgeBarsCheckBox');
      %
      % save data to file
      %
      controls.savedatatofile = uicontrol('Units','pixels', ...
            'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
            'Position',[24 20 366 20], ...
            'FontSize',CurrINI.gui_fontsize, ...
            'FontName',CurrINI.gui_fontname, ...
            'FontWeight',CurrINI.gui_fontweight, ...
            'FontAngle',CurrINI.gui_fontangle, ...
            'Style','checkbox', ...
            'Visible','on', ...
            'HorizontalAlignment','left', ...
            'String','Save data to file', ...
            'Value',0, ...
            'Enable','on', ...
            'Tag','SaveDataToFileCheckBox');
      %
      % Create graph button
      %
      controls.show = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[395 45 90 20], ...
         'String','Display', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'CallBack','StateVarDecompDLG showgraph', ...
         'Enable','on', ...
         'Tag','Done');
      %
      %  Done button
      %
      controls.done = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[395 20 90 20], ...
         'String','Done', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'CallBack','StateVarDecompDLG done', ...
         'Enable','on', ...
         'UserData','waiting', ...
         'Tag','Done');

      set(GraphGUI,'UserData',controls,...
         'HandleVisibility','callback');
   case 'samplebegin'
      %
      % check to make sure that the sample begins before it ends!
      %
      samplebeg = get(controls.samplebeg,'Value');
      sampleend = get(controls.sampleend,'Value');
      if sampleend<samplebeg;
         set(controls.sampleend,'Value',samplebeg);
      end;
   case 'sampleend'
      %
      % check to make sure that the sample begins before it ends
      %
      samplebeg = get(controls.samplebeg,'Value');
      sampleend = get(controls.sampleend,'Value');
      if sampleend<samplebeg;
         set(controls.samplebeg,'Value',sampleend);
      end;
   case 'fixcontrols'
      ShockGroups = unique(DSGEModel.ShockGroups(DSGEModel.StateVarStructure.KeepVar));
      if (get(controls.decomposegroups,'Value')==1)&(length(ShockGroups)>1);
         set(controls.useshockaliases,'Enable','on');
      else;
         set(controls.useshockaliases,'Enable','off','Value',0);
      end;
   case 'useshockgroups'
      if get(controls.useshockgroups,'Value')==1;
         if length(unique(DSGEModel.ShockGroups(DSGEModel.StateVarStructure.KeepVar)))==length(DSGEModel.StateVarStructure.KeepVar);
            set(controls.decomposegroups,'Enable','off','Value',0);
            set(controls.useshockaliases,'Enable','off','Value',0);
         else;
            set(controls.decomposegroups,'Enable','on');
         end;
      else;
         set(controls.decomposegroups,'Enable','off','Value',0);
         set(controls.useshockaliases,'Enable','off','Value',0);
      end;
   case 'showgraph'
      VariableValue = get(controls.variables,'Value');
      CurrVariable = StringTrim(DSGEModel.StateVariableNames(VariableValue,:));
      if VariableValue<size(DSGEModel.StateVariableNames,1);
         set(controls.variables,'Value',VariableValue+1);
      else;
         set(controls.variables,'Value',1);
      end;
      %
      % check if the bars should have black edges or not
      %
      BlackEdgeBars = get(controls.blackedgebars,'Value');
      %
      % check if we should use shock groups or individual shocks
      %
      UseShockGroups = get(controls.useshockgroups,'Value');
      %
      % create a vector with dates
      %
      DateVector = MakeDateVector(DSGEModel.SubBeginYear,DSGEModel.SubBeginPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,DSGEModel.DataFrequency);
      %
      % Make sure that DateVector is not longer than the number of periods for the shocks
      %
      if length(DateVector)>length(DSGEModel.StateVarStructure.Y);
         DateVector = DateVector(1,length(DateVector)-length(DSGEModel.StateVarStructure.Y)+1:length(DateVector));
      end;
      %
      samplebeg = get(controls.samplebeg,'Value');
      sampleend = get(controls.sampleend,'Value');
      SampleStrMat = get(controls.samplebeg,'String');
      %
      % create the figure
      %
      FigHandle = figure('Visible','off', ...
                         'Position',[(CurrINI.scrsz(3)-750)/2 (CurrINI.scrsz(4)-500)/2 750 500], ...
                         'Tag',CurrVariable, ...
                         'Name',['Graph of ' CurrVariable]);
      %
      SetFigureProperties(FigHandle,CurrINI);
      %
      % Create the Y-data matrix for the shock groups
      %
      if UseShockGroups==1;
         Ymat = [DSGEModel.StateVarStructure.XietaInit(VariableValue,:);zeros(size(DSGEModel.ShockGroupNames,1),size(DSGEModel.StateVarStructure.Y,2))];
         ShockGroups = DSGEModel.ShockGroups(DSGEModel.StateVarStructure.KeepVar);
      else;
         ShockGroups = (1:length(DSGEModel.StateShockPositions));
         ShockGroups = ShockGroups(DSGEModel.StateVarStructure.KeepVar);
         Ymat = [DSGEModel.StateVarStructure.XietaInit(VariableValue,:);zeros(length(ShockGroups),size(DSGEModel.StateVarStructure.Y,2))];
      end;
      for i=1:length(DSGEModel.StateVarStructure.KeepVar);
         Ymat(1+ShockGroups(i),:) = Ymat(1+ShockGroups(i),:)+DSGEModel.StateVarStructure.XietaDecomp(VariableValue,:,i);
      end;
      %
      % remove rows from Ymat where the shock groups are zero because all individual shocks
      % in that group are zero for all t.
      %
      OldYmat = Ymat([1 1+unique(ShockGroups)],:);
      Ymat = Ymat([1 1+unique(ShockGroups)],samplebeg:sampleend);
      %
      % set up the color scheme for the bars
      %
      UseAliases = 0;
      if (UseShockGroups==1)&(isempty(DSGEModel.ShockGroupColors)==0)&(size(DSGEModel.ShockGroupColors,1)==length(unique(ShockGroups)));
         ShockGroupColors = DSGEModel.ShockGroupColors(unique(ShockGroups),:);
      else;
         if (isempty(DSGEModel.ShockColors)==0)&(size(Ymat,1)-1==size(DSGEModel.ShockColors(DSGEModel.StateVarStructure.KeepVar,:),1));
            ShockGroupColors = DSGEModel.ShockColors(DSGEModel.StateVarStructure.KeepVar,:);
         else;
            ShockGroupColors = CreateDefaultColors(size(Ymat,1)-1);
         end;
         if (UseShockGroups==0)|(isequal(DSGEModel.ShockGroupNames,DSGEModel.StateShockNames)==1);
            UseAliases = 1;
         end;
      end;
      %
      % check if we should remove the initial state
      %
      if get(controls.skipstateerrors,'Value')==1;
         Ymat = Ymat(2:size(Ymat,1),:);
      else;
         ShockGroupColors = [0.4 0.4 0.4;ShockGroupColors];
      end;
      NewYmat = Ymat;
      %
      % Create the bars and collect the handles
      %
      if MatlabNumber>8.3;
         BarHandles = bar(DateVector(samplebeg:sampleend)',Ymat','stacked');
      else;
         if MatlabNumber>=7.0;
            %
            % make sure the bars are written as patches
            %
            BarHandles = bar('v6',DateVector(samplebeg:sampleend)',Ymat','stacked');
         else;
            BarHandles = bar(DateVector(samplebeg:sampleend)',Ymat','stacked');
         end;
      end;
      %
      % remake the bar patches such that all bars are displayed properly.
      %
      BarHandles = RemakeBars(BarHandles,ShockGroupColors,BlackEdgeBars);
      %
      % the smoothed state variables data
      %
      LineHandle = plot(DateVector(samplebeg:sampleend),DSGEModel.StateVarStructure.KsitT(VariableValue,samplebeg:sampleend),'.-','Color',[0 0 0],'LineWidth',2);
      SetAxesFonts(CurrINI,'Title',['Smooth estimate of ' CurrVariable]);
      axis('tight');
      %
      % see if we should add a zero line
      %
      ylim = get(gca,'YLim');
      add_on = 0.1*(ylim(2)-ylim(1));
      ylim = [ylim(1)-add_on ylim(2)+add_on];
      set(gca,'YLim',ylim);
      if (ylim(1)<0)&(ylim(2)>0);
         hold('on');
         plot([DateVector(samplebeg) DateVector(sampleend)],[0 0],'-','Color',[0 0 0]);
         %
         % make sure the zero line is displayed below the observed variable
         %
         kids = get(gca,'Children');
         if size(kids,1)>size(kids,2);
            kids = kids';
         end;
         kids = [kids(2:length(kids)) kids(1)]';
         set(gca,'Children',kids);
      end;
      if UseAliases==0;
         ShockGroupNames = '';
         for i=1:size(DSGEModel.ShockGroupNames,1);
            ShockGroupNames = strvcat(ShockGroupNames,StringTrim(strrep(DSGEModel.ShockGroupNames(i,:),'_',' ')));
         end;
      else;
         ShockGroupNames = DSGEModel.ShockAliases(DSGEModel.StateVarStructure.KeepVar,:);
         for j=1:size(ShockGroupNames,1);
            ShockGroupNames(j,:) = strrep(ShockGroupNames(j,:),'_',' ');
         end;
      end;
      %
      % check is we should adjust ShockGroupNames
      %
      if (size(ShockGroupNames,1)>length(DSGEModel.StateVarStructure.KeepVar))&(size(ShockGroupNames,1)>=max(DSGEModel.StateVarStructure.KeepVar));
         ShockGroupNames = ShockGroupNames(DSGEModel.StateVarStructure.KeepVar,:);
      elseif (size(ShockGroupNames,1)>length(DSGEModel.StateVarStructure.KeepVar))&(size(ShockGroupNames,1)<max(DSGEModel.StateVarStructure.KeepVar));
         ShockGroupNames = ShockGroupNames(1:length(DSGEModel.StateVarStructure.KeepVar),:);
      end;
      if get(controls.skipstateerrors,'Value')==0;
         legHandle = legend([BarHandles LineHandle],strvcat('Initial State',ShockGroupNames,[CurrVariable '(t|T)']));
      else;
         legHandle = legend([BarHandles LineHandle],strvcat(ShockGroupNames,[CurrVariable '(t|T)']));
      end;
      SetAxesFonts(CurrINI,legHandle);
      hold('off');
      %
      % check if we should save data to file
      %
      if get(controls.savedatatofile,'Value')==1;
         if isempty(findstr(lower(DSGEModel.NameStr),'mode'))==0;
            matfile = FixFilePath([DSGEModel.OutputDirectory '\State-' CurrVariable '-ShockDecomp-PosteriorMode.mat']);
            mfile = FixFilePath([DSGEModel.OutputDirectory '\State' CurrVariable 'ShockDecompPostMode.m']);
         else;
            matfile = FixFilePath([DSGEModel.OutputDirectory '\State-' CurrVariable '-ShockDecomp-InitialValues.mat']);
            mfile = FixFilePath([DSGEModel.OutputDirectory '\State' CurrVariable 'ShockDecompInitVal.m']);
         end;
         Variable = strrep(CurrVariable,'-','');
         NamesShockGroup = strvcat('Initial State',ShockGroupNames);
         Ymat = OldYmat;
         save(matfile,'Ymat','DateVector','NamesShockGroup','ShockGroupColors');
         %
         % write to m-file
         %
         fid = fopen(mfile,'wt');
         fprintf(fid,'%%\n%% load the data in %s\n%%\n',GetFilename(matfile));
         fprintf(fid,'DataOVD = load(''%s'');\n',GetFilename(matfile));
         fprintf(fid,'%%\n%% matrix with decomposed values for variable determined by field name\n%%\n');
         fprintf(fid,'Ymat.%s = DataOVD.Ymat;\n',Variable);
         fprintf(fid,'%%\n%% vector with dates\n%%\n');
         fprintf(fid,'DateVector = DataOVD.DateVector;\n');
         fprintf(fid,'%%\n%% names for the different shockgroups\n%%\n');
         fprintf(fid,'ShockGroupNames = DataOVD.NamesShockGroup;\n');
         fprintf(fid,'%%\n%% colors for the shock groups\n%%\n');
         fprintf(fid,'ShockGroupColors = DataOVD.ShockGroupColors;\n');
      end;
      %
      SetXTickIntegers(gca);
      %
      % display the graph
      %
      SetToolbarImages(FigHandle,CurrINI.images);
      set(FigHandle,'Visible','on','CloseRequestFcn','delete(gcf); drawnow; pause(0.02);');
      drawnow;
      pause(0.02);
      %
      % check if we should decompose the shock groups into the individual shocks
      %
      if get(controls.decomposegroups,'Value')==1;
         if get(controls.useshockaliases,'Value')==0;
            AllShockNames = DSGEModel.StateShockNames(DSGEModel.StateVarStructure.KeepVar,:);
         else;
            AllShockNames = DSGEModel.ShockAliases(DSGEModel.StateVarStructure.KeepVar,:);
            for j=1:size(AllShockNames,1);
               AllShockNames(j,:) = strrep(AllShockNames(j,:),'_',' ');
            end;
         end;
         pos = [(CurrINI.scrsz(3)-750)/2 (CurrINI.scrsz(4)-500)/2];
         ShockMatrix = [];
         if isempty(DSGEModel.ShockColors)==0;
            AllShockColors = DSGEModel.ShockColors(DSGEModel.StateVarStructure.KeepVar,:);
         else;
            if max(DSGEModel.ShockGroups(DSGEModel.StateVarStructure.KeepVar))==length(DSGEModel.StateVarStructure.KeepVar);
               AllShockColors = CreateDefaultColors(length(DSGEModel.StateVarStructure.KeepVar));
            else;
               AllShockColors = CreateShockColors(DSGEModel.ShockGroups(DSGEModel.StateVarStructure.KeepVar),DSGEModel.ShockGroupColors);
            end;
         end;
         %
         for i=1:size(ShockGroupNames,1);
            %
            % fix the figure position
            %
            pos = pos+[22 -22];
            if pos(1)+750>CurrINI.scrsz(3);
               pos(1) = 0;
            end;
            if pos(2)<=32;
               pos(2) = CurrINI.scrsz(4)-550;
            end;
            %
            GroupValues = Ymat(1+i,:);
            %
            % find out which shocks appear in group i
            %
            GroupShocks = (ShockGroups==i) .* (1:length(ShockGroups));
            GroupShocks = GroupShocks(GroupShocks>0);
            %
            ShockMat = zeros(length(GroupShocks),size(DSGEModel.StateVarStructure.XietaDecomp,2));
            for j=1:length(GroupShocks);
               ShockMat(j,:) = DSGEModel.StateVarStructure.XietaDecomp(VariableValue,:,GroupShocks(j));
            end;
            %
            % get the colors of the shocks
            %
            ShockColors = AllShockColors(GroupShocks,:);
            %
            % get the names of the shocks
            %
            ShockNames = AllShockNames(GroupShocks,:);
            %
            % create the figure
            %
            FigHandle = figure('Visible','off', ...
                               'Position',[pos 750 500], ...
                               'Tag',CurrVariable, ...
                               'Name',['Graph of ' StringTrim(ShockGroupNames(i,:)) ' shock group for ' CurrVariable]);
            %
            SetFigureProperties(FigHandle,CurrINI);
            %
            % Create the bars and collect the handles
            %
            if MatlabNumber>8.3;
               BarHandles = bar(DateVector(samplebeg:sampleend)',ShockMat(:,samplebeg:sampleend)','stacked');
            else;
               if MatlabNumber>=7.0;
                  %
                  % make sure the bars are written as patches
                  %
                  BarHandles = bar('v6',DateVector(samplebeg:sampleend)',ShockMat(:,samplebeg:sampleend)','stacked');
               else;
                  BarHandles = bar(DateVector(samplebeg:sampleend)',ShockMat(:,samplebeg:sampleend)','stacked');
               end;
            end;
            %
            % remake the bar patches such that all bars are displayed properly.
            %
            BarHandles = RemakeBars(BarHandles,ShockColors,BlackEdgeBars);
            %
            % display the line with the combined shock groups values
            %
            LineHandle = plot(DateVector(samplebeg:sampleend),GroupValues,'.-','Color',[0 0 0],'LineWidth',2);
            %
            % set the title
            %
            SetAxesFonts(CurrINI,'Title',['Decomposition of ' StringTrim(ShockGroupNames(i,:)) ' shock group for ' CurrVariable ' (original data) at ' lower(DSGEModel.NameStr)]);
            axis('tight');
            ylim = get(gca,'YLim');
            %
            % see if we should add a zero line
            %
            add_on = 0.1*(ylim(2)-ylim(1));
            if add_on==0;
               Ylim = get(gca,'YLim');
               add_on = 0.1*(Ylim(2)-Ylim(1));
            end;
            ylim = [ylim(1)-add_on ylim(2)+add_on];
            set(gca,'YLim',ylim);
            if (ylim(1)<0)&(ylim(2)>0);
               hold('on');
               plot([DateVector(1) DateVector(length(DateVector))],[0 0],'-','Color',[0 0 0]);
               %
               % make sure the zero line is displayed below the observed variable
               %
               kids = get(gca,'Children');
               if size(kids,1)>size(kids,2);
                  kids = kids';
               end;
               kids = [kids(2:length(kids)) kids(1)]';
               set(gca,'Children',kids);
            end;
            %
            % make sure the X axis limits follow the sample selection
            %
            legHandle = legend([BarHandles LineHandle],strvcat(ShockNames,[StringTrim(ShockGroupNames(i,:)) ' shock group']),2);
            SetAxesFonts(CurrINI,legHandle);
            hold('off');
            %
            SetXTickIntegers(gca);
            %
            % check if we should save data to file
            %
            if get(controls.savedatatofile,'Value')==1;
               GroupName = strrep(ShockGroupNames(i,:),' ','');
               GroupName = strrep(GroupName,'-','');
               eval(['ShockMatrix.' Variable '.' GroupName '.Values = ShockMat;']);
               eval(['ShockMatrix.' Variable '.' GroupName '.Colors = ShockColors;']);
               eval(['ShockMatrix.' Variable '.' GroupName '.Names = ShockNames;']);
               %
               % write to m-file
               %
               fprintf(fid,'%%\n%% Values for individual shocks for variable %s in group "%s"\n%%\n',Variable,StringTrim(ShockGroupNames(i,:)));
               fprintf(fid,'ShockMatrix.%s.%s.Values = DataOVD.ShockMatrix.%s.%s.Values;\n',Variable,GroupName,Variable,GroupName);
               fprintf(fid,'%%\n%% Colors for individual shocks for variable %s in group "%s"\n%%\n',Variable,StringTrim(ShockGroupNames(i,:)));
               fprintf(fid,'ShockMatrix.%s.%s.Colors = DataOVD.ShockMatrix.%s.%s.Colors;\n',Variable,GroupName,Variable,GroupName);
               fprintf(fid,'%%\n%% Names for individual shocks for variable %s in group "%s"\n%%\n',Variable,StringTrim(ShockGroupNames(i,:)));
               fprintf(fid,'ShockMatrix.%s.%s.Names = DataOVD.ShockMatrix.%s.%s.Names;\n',Variable,GroupName,Variable,GroupName);
            end;
            %
            % display the graph
            %
            SetToolbarImages(FigHandle,CurrINI.images);
            set(FigHandle,'Visible','on','CloseRequestFcn','delete(gcf); drawnow; pause(0.02);');
            drawnow;
            pause(0.02);
         end;
      end;
      %
      if get(controls.savedatatofile,'Value')==1;
         if get(controls.decomposegroups,'Value')==1;
            save(matfile,'ShockMatrix','-append');
         end;
         fprintf(fid,'%%\n%% add your own code here\n%%\n\n\n');
         fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
         fclose(fid);
      end;
      if get(controls.fixedpointdecomp,'Value')==1;
         DYmat = NewYmat-(NewYmat(:,1)*ones(1,size(NewYmat,2)));
         SampleDate = SampleStrMat(samplebeg,:);
         %
         % fix the figure position
         %
         if exist('pos')==0
            pos = [(CurrINI.scrsz(3)-750)/2 (CurrINI.scrsz(4)-500)/2];
         end;
         pos = pos+[22 -22];
         if pos(1)+750>CurrINI.scrsz(3);
            pos(1) = 0;
         end;
         if pos(2)<=32;
            pos(2) = CurrINI.scrsz(4)-550;
         end;
         %
         % create the figure
         %
         FigHandleFPC = figure('Visible','off', ...
                            'Position',[pos 750 500], ...
                            'Tag',['FixedPointChange-' CurrVariable], ...
                            'Name',['Decomposition of Change relative to ' SampleDate ' for ' CurrVariable]);
         %
         SetFigureProperties(FigHandleFPC,CurrINI);
         %
         % Create the bars and collect the handles
         %
         if MatlabNumber>8.3;
            BarHandles = bar(DateVector(samplebeg:sampleend)',DYmat','stacked');
         else;
            if MatlabNumber>=7.0;
               %
               % make sure the bars are written as patches
               %
               BarHandles = bar('v6',DateVector(samplebeg:sampleend)',DYmat','stacked');
            else;
               BarHandles = bar(DateVector(samplebeg:sampleend)',DYmat','stacked');
            end;
         end;
         %
         % remake the bar patches such that all bars are displayed properly.
         %
         BarHandles = RemakeBars(BarHandles,ShockGroupColors,BlackEdgeBars);
         %
         % plot the actual change
         %
         LineHandle = plot(DateVector(samplebeg:sampleend),DSGEModel.StateVarStructure.KsitT(VariableValue,samplebeg:sampleend)-DSGEModel.StateVarStructure.KsitT(VariableValue,samplebeg),'.-','Color',[0 0 0],'LineWidth',2);
         SetAxesFonts(CurrINI,'Title',['Smooth estimate of ' CurrVariable ' in deviation from ' SampleDate ' value']);
         axis('tight');
         %
         add_on = 0.1*(ylim(2)-ylim(1));
         ylim = get(gca,'YLim');
         for h=1:length(BarHandles);
            if h==1;
               YData = get(BarHandles(h),'YData');
               if size(YData,1)~=4;
                  YData = YData';
               end;
               ylim(1) = min(min(YData));
               ylim(2) = max(max(YData));
            else;
               YData = get(BarHandles(h),'YData');
               if size(YData,1)~=4;
                  YData = YData';
               end;
               ylim(1) = min([min(YData) ylim(1)]);
               ylim(2) = max([max(YData) ylim(2)]);
            end;
         end;
         ylim = [ylim(1)-add_on ylim(2)+add_on];
         set(gca,'YLim',ylim);
         if (ylim(1)<0)&(ylim(2)>0);
            hold('on');
            plot([DateVector(samplebeg) DateVector(sampleend)],[0 0],'-','Color',[0 0 0]);
            %
            % make sure the zero line is displayed below the observed variable
            %
            kids = get(gca,'Children');
            if size(kids,1)>size(kids,2);
               kids = kids';
            end;
            kids = [kids(2:length(kids)) kids(1)]';
            set(gca,'Children',kids);
         end;
         %
         % make sure that only the selected sample is displayed
         %
         XData = get(BarHandles(1),'XData');
         if size(XData,1)>size(XData,2);
            XData = XData';
         end;
         set(gca,'XLim',[min(XData(:,1)) max(XData(:,size(XData,2)))]);
         if get(controls.skipstateerrors,'Value')==0;
            legHandle = legend([BarHandles LineHandle],strvcat('Initial State',ShockGroupNames,[CurrVariable '(t|T)']));
         else;
            legHandle = legend([BarHandles LineHandle],strvcat(ShockGroupNames,[CurrVariable '(t|T)']));
         end;
         SetAxesFonts(CurrINI,legHandle);
         hold('off');
         %
         % check if we should fix the X label
         %
         SetXTickIntegers(gca);
         %
         % display the graph
         %
         SetToolbarImages(FigHandleFPC,CurrINI.images);
         set(FigHandleFPC,'Visible','on','CloseRequestFcn','delete(gcf); drawnow; pause(0.02);');
         drawnow;
         pause(0.02);
      end;
   case 'done'
      set(controls.done,'UserData','done');
      delete(GraphGUI);
      drawnow;
      pause(0.02);
end;
%
if strcmp(selector,'init')==1;
   set(GraphGUI,'Visible','on');
   drawnow;
   if MatlabNumber>=7;
      uicontrol(controls.done);
   end;
   waitfor(controls.done,'UserData','done');
end;

%
% end of StateVarDecompDLG.m
%
