function controls = ActionsMenuControls(maingui,controls,AiMEnable,StateEnable)
% ActionsMenuControls: sets up the Actions menu.
%
% USAGE:
%
%       controls = ActionsMenuControls(maingui,controls,AiMEnable,StateEnable)
%
% REQUIRED INPUT:  maingui (handle) to YADA's main dialog.
%
%                  controls (structure) with handles to the controls on the YADA main dialog.
%
%                  AiMEnable (string) that takes on the value 'on' or 'off' if AiM should
%                            be enabled or not.
%
%                  StateEnable (string) that takes on the value 'on' or 'off' if the state
%                            variable and shock controls should be enabled or not.
%
% REQUIRED OUTPUT: controls (structure) with handles to the controls on the YADA main dialog.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2017 European Central Bank.
%
%                       First version: March 3, 2010.
%                        This version: September 15, 2017.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 04-03-2010: Updated the documentation.
%
% * 05-01-2011: Updated the documentation.
%
% * 21-03-2011: Added controls for setting the mixed conditioning shocks.
%
% * 05-04-2011: Added "Configure Shocks" control.
%
% * 02-01-2012: Updated the documentation.
%
% * 02-02-2012: Added controls for reordering conditioning shocks.
%
% * 06-02-2012: Changed the name of the control for selecting the shocks to use under the direct method
%               for conducting conditional forecasting.
%
% * 20-03-2012: Added a hyphen to the specify unit root state variables label.
%
% * 05-06-2012: Added control for setting the state equations.
%
% * 03-01-2013: Updated the documentation.
%
% * 17-07-2013: The function now fully supports the Waggoner-Zha approach to conditional forecasting.
%
% * 28-01-2014: Updated the documentation.
%
% * 14-01-2015: Updated the documentation.
%
% * 27-01-2016: Remember Shoah.
%
% * 21-03-2017: Updated the documentation.
%
% * 02-06-2017: Added the estimate system prior mode handle.
%
% * 15-09-2017: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% Actions menu
%
controls.actionsmenu = uimenu('Parent',maingui, ...
   'Label','&Actions', ...
   'Tag','ActionsMenu');
%
% Objects on the Actions menu
%
controls.runaimmenu = uimenu('Parent',controls.actionsmenu, ...
   'Label','Run &AiM Parser...', ...
   'Callback','ActionsMenuFunctions RunAiM', ...
   'Enable',AiMEnable, ...
   'Tag','RunAIMMenu');
%
controls.postmodemenu = uimenu('Parent',controls.actionsmenu, ...
   'Label','&Estimate Posterior Mode...', ...
   'Callback','ActionsMenuFunctions PosteriorModeEstimation', ...
   'Enable','off', ...
   'Tag','EstimatePostModeMenu');
%
controls.postsamplingmenu = uimenu('Parent',controls.actionsmenu, ...
   'Label','&Posterior Sampling...', ...
   'Callback','ActionsMenuFunctions PosteriorSampling', ...
   'Enable','off', ...
   'Tag','PosteriorSamplingMenu');
%
controls.priorsamplingmenu = uimenu('Parent',controls.actionsmenu, ...
   'Label','Prior Sa&mpling...', ...
   'Callback','ActionsMenuFunctions PriorSampling', ...
   'Separator','on', ...
   'Enable','off', ...
   'Tag','PriorSamplingMenu');
%
controls.systempriormodemenu = uimenu('Parent',controls.actionsmenu, ...
   'Label','Estimate S&ystem Prior Mode...', ...
   'Callback','ActionsMenuFunctions SystemPriorModeEstimation', ...
   'Enable','off', ...
   'Tag','EstimateSystemPriorModeMenu');
%
controls.statevarmenu = uimenu('Parent',controls.actionsmenu, ...
   'Label','Set State &Variables...', ...
   'Callback','ActionsMenuFunctions SetStateVariables', ...
   'Separator','on', ...
   'Enable',StateEnable, ...
   'Tag','StateVarsMenu');
%
controls.stateeqmenu = uimenu('Parent',controls.actionsmenu, ...
   'Label','Set State E&quations...', ...
   'Callback','ActionsMenuFunctions SetStateEquations', ...
   'Enable',StateEnable, ...
   'Tag','StateEqsMenu');
%
controls.stateshockmenu = uimenu('Parent',controls.actionsmenu, ...
   'Label','&Set State Shocks...', ...
   'Callback','ActionsMenuFunctions SetStateShocks', ...
   'Enable',StateEnable, ...
   'Tag','StateShocksMenu');
%
controls.configureshocksmenu = uimenu('Parent',controls.actionsmenu, ...
   'Label','Con&figure Shocks...', ...
   'Callback','ActionsMenuFunctions ConfigureShocks', ...
   'Separator','on', ...
   'Enable','off', ...
   'Tag','ConfigureShocksMenu');
%
controls.shockgroupmenu = uimenu('Parent',controls.actionsmenu, ...
   'Label','Set Shock &Groups...', ...
   'Callback','ActionsMenuFunctions SetShockGroups', ...
   'Enable','off', ...
   'Tag','ShockGroupMenu');
%
controls.obsvargroupsmenu = uimenu('Parent',controls.actionsmenu, ...
   'Label','Set Observed Variable G&roups...', ...
   'Callback','ActionsMenuFunctions SetObsVarGroups', ...
   'Separator','on', ...
   'Enable','off', ...
   'Tag','ObsVarGroupMenu');
%
controls.initialstatevaluesmenu = uimenu('Parent',controls.actionsmenu, ...
   'Label','Set &Initial State Values...', ...
   'Callback','ActionsMenuFunctions SetInitialStateValues', ...
   'Separator','on', ...
   'Enable','off', ...
   'Tag','InitialStateValuesMenu');
%
controls.unitrootstatemenu = uimenu('Parent',controls.actionsmenu, ...
   'Label','Specify &Unit-Root State Variables...', ...
   'Callback','ActionsMenuFunctions SetUnitRootStateVars', ...
   'Enable','off', ...
   'Tag','StateVarsMenu');
%
controls.confbandcolor = uimenu('Parent',controls.actionsmenu, ...
   'Label','&Confidence Band Base Color...', ...
   'Callback','ActionsMenuFunctions SetConfBandBaseColor', ...
   'Separator','on', ...
   'Enable','on', ...
   'Tag','ConfidenceBandBaseColorMenu');
%
controls.viewconditionalvars = uimenu('Parent',controls.actionsmenu, ...
   'Label','Vie&w Conditioning Variables...', ...
   'Callback','ActionsMenuFunctions ViewConditionalVars', ...
   'Separator','on', ...
   'Enable','off', ...
   'Tag','ViewConditionalVariablesMenu');
%
controls.setconditionalvars = uimenu('Parent',controls.actionsmenu, ...
   'Label','Se&lect Conditioning Variables...', ...
   'Callback','ActionsMenuFunctions SetConditionalVars', ...
   'Separator','off', ...
   'Enable','off', ...
   'Tag','SetConditionalVariablesMenu');
%
controls.setconditionalshocks = uimenu('Parent',controls.actionsmenu, ...
   'Label','Select C&onditioning Shocks', ...
   'Enable','off', ...
   'Tag','SetConditionalShocksMenu');
%
controls.setconditionalshocksdirect = uimenu('Parent',controls.setconditionalshocks, ...
   'Label','&Direct Method Shocks...', ...
   'Callback','ActionsMenuFunctions SetConditionalShocks', ...
   'Enable','on', ...
   'Tag','SetConditionalShocksDirectMenu');
%
controls.setconditionalshocksmixed = uimenu('Parent',controls.setconditionalshocks, ...
   'Label','&Subset Method Shocks...', ...
   'Callback','ActionsMenuFunctions SetMixedConditionalShocks', ...
   'Enable','off', ...
   'Tag','SetConditionalShocksMixedMenu');
%
% reorder shocks when using the direct method for conditional forecasting
%
controls.reorderconditionalshocks = uimenu('Parent',controls.actionsmenu, ...
   'Label','Reorder Conditioning S&hocks...', ...
   'Callback','ActionsMenuFunctions ReorderConditionalShocks', ...
   'Enable','off', ...
   'Tag','ReorderConditionalShocksMenu');
%
% controls for selecting state conditioning data
%
controls.stateconditionalvarsparent = uimenu('Parent',controls.actionsmenu, ...
      'Label','State Co&nditioning Variables', ...
      'Separator','on', ...
      'Tag','StateConditionalVariablesParent');
%
controls.viewstateconditionalvars = uimenu('Parent',controls.stateconditionalvarsparent, ...
   'Label','Vie&w Variables...', ...
   'Callback','CondStateFunction ViewStateConditionalVars', ...
   'Enable','off', ...
   'Tag','ViewStateConditionalVariablesMenu');
%
controls.setstateconditionalvars = uimenu('Parent',controls.stateconditionalvarsparent, ...
   'Label','Select &Variables...', ...
   'Callback','CondStateFunction SetStateConditionalVars', ...
   'Enable','off', ...
   'Tag','SetStateConditionalVariablesMenu');
%
controls.setstateconditionalshocks = uimenu('Parent',controls.stateconditionalvarsparent, ...
   'Label','&Select Shocks', ...
   'Enable','off', ...
   'Tag','SetStateConditionalShocksMenu');
%
controls.setstateconditionalshocksdirect = uimenu('Parent',controls.setstateconditionalshocks, ...
   'Label','&Direct Method Shocks...', ...
   'Callback','CondStateFunction SetStateConditionalShocks', ...
   'Enable','on', ...
   'Tag','SetStateConditionalShocksDirectMenu');
%
controls.setstateconditionalshocksmixed = uimenu('Parent',controls.setstateconditionalshocks, ...
   'Label','&Subset Method Shocks...', ...
   'Callback','CondStateFunction SetMixedStateConditionalShocks', ...
   'Enable','off', ...
   'Tag','SetStateConditionalShocksMixedMenu');
%
controls.reorderstateconditionalshocks = uimenu('Parent',controls.stateconditionalvarsparent, ...
   'Label','&Reorder Shocks...', ...
   'Callback','CondStateFunction ReorderStateConditionalShocks', ...
   'Enable','off', ...
   'Tag','ReorderStateConditionalShocksMenu');


%
% end of ActionsMenuControls.m
%
