function CondStateFunction(selector)
% CondStateFunction: provides all the necessary functions for running the forecasts conditional on
%                    the state variable assumptions.
%
% USAGE:
%
%       CondStateFunction(selector)
%
% REQUIRED INPUT: selector (string vector) that determines which sub-function should be applied.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2017 European Central Bank.
%
%                       First version: July 14, 2009.
%                        This version: September 15, 2017.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 15-04-2009: Added code for conditional forecasts with state variable assumptions for the
%               posterior mode.
%
% * 16-07-2009: Finished the posterior mode code.
%
% * 17-07-2009: Added the initial values code, the view conditioning assumptions, the select
%               conditional shocks and conditional variables code. Started work on the posterior
%               distribution code.
%
% * 20-07-2009: Continued work on the posterior distribution code. Took nore of the fact that it's
%               40 years since the eagle landed. Live long and prosper!
%
% * 21-07-2009: Continued work on the posterior distribution code.
%
% * 22-07-2009: Finished the posterior distribution code. Added all the prediction event code
%               and began work on the prior distribution code.
%
% * 23-07-2009: Finished the prior distribution code.
%
% * 31-07-2009: Took into account that the function "DSGEPredictionDistribution" now provides the
%               "CovPrediction" output variable.
%
% * 18-09-2009: Made sure that "lasterr" is reset to an empty string.
%
% * 25-09-2009: The function now displays the error message that was caught when attempting to load
%               posterior mode values under initial parameter values and loading failed.
%
% * 06-01-2010: Updated the documentation.
%
% * 07-03-2010: Changed cases of all the cases.
%
% * 26-04-2010: Extended the data stored to mat-file when running the predictions for initial values
%               and posterior mode values.
%
% * 08-07-2010: Changed the headers of About dialogs when "mcode" or "kalmanstatus" reports problems.
%
% * 22-07-2010: Took into account that "SummarizeDSGEPredictions" now provides the "TotalVar" output
%               variable.
%
% * 23-07-2010: Added calls to "NormalDensityValues" for a normal approximation of the marginal predictive
%               densities.
%
% * 18-11-2010: Took into account that "PredEventData" now has 7 columns.
%
% * 19-11-2010: And now that it has 8 columns.
%
% * 17-12-2010: The function now supports the Klein and Sims DSGE model solvers.
%
% * 04-01-2011: Updated the documentation.
%
% * 05-01-2011: The func now calls "YADAEnableControls" at the very end.
%
% * 11-01-2011: Made sure that "ModeFile" supports unix and mac paths. Added additional support for unix and
%               mac paths.
%
% * 25-01-2011: Took into account that "CalcTransformData" takes the input variable "TransMatrix".
%
% * 16-02-2011: Started work on the decompositions of the predictive uncertainty into state, shocks,
%               measurement error, and parameter uncertainty.
%
% * 17-02-2011: Finished the decompositions of the predictive uncertainty for both prior and posterior
%               draws based on state variable assumptions.
%
% * 24-03-2011: Began work on implementing the distribution for a subset of the shocks method of conditional
%               forecasting. Posterior mode values first.
%
% * 25-03-2011: Finished all remaining the distribution for a subset of the shocks method cases.
%
% * 21-12-2011: Made sure that it's not possible to select more state conditioning assumptions than there
%               are shocks in the model.
%
% * 02-01-2012: Updated the documentation.
%
% * 07-02-2012: Adjusted the code to allow for more general conditioning assumptions.
%
% * 08-02-2012: Made more adjustments to the selection of state conditioning assumptions as shocks,
%               including a new function for reordering the shocks when using the direct control of
%               the shocks method.
%
% * 09-02-2012: Made sure the code accepts the relaxed conditions for using the subset method.
%
% * 29-02-2012: Added code for allowing the number of simulated paths per parameter value to be unity.
%
% * 29-08-2012: Took into account that "DSGEPredictionDistribution" and "PredictionPathDistribution"
%               now take the input variable "ConfidenceRegionMethod".
%
% * 02-01-2013: Updated the documentation.
%
% * 28-01-2014: Updated the documentation.
%
% * 14-01-2015: Updated the documentation.
%
% * 02-09-2015: Fixed the information displayed when the kalmanstatus is -1.
%
% * 27-01-2016: Remember Shoah.
%
% * 21-03-2017: Updated the documentation.
%
% * 23-08-2017: Took into account that "LoadDSGEPosteriorDraws" now provides the output variable "Weights".
%
% * 24-08-2017: Took into account that "DSGEPredictionDistribution" takes the input variable "Weights".
%
% * 25-08-2017: Took into account that "LoadDSGEPredictionPaths", "StateCondPredictionPaths",
%               "StateCondPredictionPathsWZ", and "StateCondPredictionPathsMixed" require the
%               "Weights" input variable.
%
% * 15-09-2017: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize
%
maingui = findobj('Type','figure','Tag','YADA');
controls = get(maingui,'UserData');
CurrINI = get(controls.filemenu,'UserData');
DSGEModel = get(controls.open,'UserData');
lasterr = '';
%
% and now for the switch function
%
switch selector
   case 'InitialValues'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % check first if posterior mode results exists. If they do we load initial values from this file.
         % Otherwise we have to do a bunch of checking
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
         if FileExist(ModeFile)==1;
            answer = 'Yes';
         else;
            %
            % Ask if the user really wants to look at these before estimating the posterior mode
            %
            txt = 'Are you sure you want to view graphs of the conditional predictive distributions of the observed variables before running the posterior mode estimation routine?';
            answer = Query(txt,'question',150,'Conditional Predictive Distributions - Initial Values',500,'no',CurrINI);
         end;
         %
         % check if we should check transformations functions
         %
         if gcbo==controls.predict.stateconditional_transformed_initialvalues;
            ErrorStr = '';
            DTStatus = VerifyDataTransformation(DSGEModel.VariableNames,DSGEModel.YTransformation,0);
            if DTStatus==1;
               [YTStatus,YData,ErrorStr,HeaderStr] = CalcTransformData(DSGEModel.VariableNames,DSGEModel.Y',DSGEModel.YTransformation,DSGEModel.YTransMatrix);
            else;
               ErrorStr = 'You haven''t specified all the necessary fields for computing transformations of the observed variables.';
               HeaderStr = 'Missing Transformation Information';
            end;
            if isempty(ErrorStr)==0;
               About(ErrorStr,'information',HeaderStr,200,500,CurrINI);
               answer = 'no';
            end;
         end;
         %
         if strcmp(lower(answer),'yes')==1;
            %
            % posterior mode estimation has not been completed yet. We therefore need to check all the imput files
            %
            [InitStatus,theta,thetaPositions,ModelParameters] = InitializeDSGEModelSimulation(DSGEModel,CurrINI);
            if (InitStatus==0)&(FileExist(DSGEModel.PriorFile)==0);
               if FileExist(ModeFile)==1;
                  ErrorStr = '';
                  try;
                     ModeData = load(ModeFile);
                  catch;
                     ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' lasterr];
                  end;
                  if isempty(ErrorStr)==1;
                     theta = ModeData.theta;
                     thetaPositions = ModeData.thetaPositions;
                     ModelParameters = ModeData.ModelParameters;
                     ModelParameters = ThetaToModelParameters(ModelParameters,theta,thetaPositions);
                     InitStatus = 1;
                  else;
                     About(ErrorStr,'information','YADA - Bad MAT File',120,500,CurrINI);
                  end;
               end;
            end;
            %
            if InitStatus==1;
               [FirstPeriod,LastPeriod] = CreateSubSample(DSGEModel);
               %
               % check how many time periods to use for predictive distributions
               %
               [MaxPH,NumPredX] = CalcMaxPredictionHorizon(DSGEModel.MaxForecastHorizon,DSGEModel.DataFrequency,DSGEModel.EndYear,DSGEModel.EndPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,size(DSGEModel.PredictedX,2));
               %
               % matrix with exogenous variables
               %
               X = [];
               if DSGEModel.k>0;
                  if NumPredX<MaxPH;
                     X = DSGEModel.X(:,LastPeriod+1:LastPeriod+(MaxPH-NumPredX));
                  end;
                  if NumPredX>0;
                     X = [X DSGEModel.PredictedX(:,1:NumPredX)];
                  end;
               end;
               %
               % check the length of the conditioning sample
               %
               [status,NumPeriods] = CheckStateCondData(DSGEModel);
               %
               % check if we should adjust the Zeta matrix
               %
               if MaxPH<NumPeriods;
                  NumPeriods = MaxPH;
               end;
               %
               % collect the conditional variables and the selected conditioning sample
               %
               Zeta = DSGEModel.Zeta(DSGEModel.ZetaConditionalVariables,LastPeriod+1:LastPeriod+NumPeriods);
               %
               % check if there are conditioning assumptions for the observed variables
               %
               NumZeta = length(DSGEModel.ZetaConditionalVariables);
               SStr = '';
               if NumZeta>1;
                  SStr = 's';
               end;
               Z = [];
               U = [];
               CondTypeStr = 'State-';
               CTypeStr = 'S';
               if isempty(DSGEModel.Z)==0;
                  %
                  % check if we can use the data
                  %
                  [status,NumZPeriods] = CheckConditioningData(DSGEModel);
                  if (status==1)&(NumZPeriods>=NumPeriods);
                     %
                     % check if the number of conditioning assumptions exceeds the number of
                     % observed variables and shocks.
                     %
                     r = length(DSGEModel.StateVariablePositions);
                     EnoughVariables = ((length([DSGEModel.ConditionalVariables DSGEModel.ZetaConditionalVariables])<=min(DSGEModel.n+r,size(DSGEModel.StateShockNames,1))))&(DSGEModel.ZetaConditionalVariables<=r)&(length(DSGEModel.ConditionalVariables)<=DSGEModel.n);
                     %
                     % check if we have selected enough unique shocks
                     %
                     if DSGEModel.ShockControlMethod==1;
                        UniqueShocks = (length(unique([DSGEModel.ZetaConditionalShocks DSGEModel.ConditionalShocks]))==length([DSGEModel.ZetaConditionalShocks DSGEModel.ConditionalShocks]));
                     elseif DSGEModel.ShockControlMethod==2;
                        UniqueShocks = 1;
                     else;
                        NumShocks = length(unique([DSGEModel.ZetaMixedConditionalShocks DSGEModel.MixedConditionalShocks]));
                        NumAssumptions = length([DSGEModel.ConditionalVariables DSGEModel.ZetaConditionalVariables]);
                        UniqueShocks = (NumShocks>=NumAssumptions)&(NumShocks<=length(DSGEModel.StateShockPositions));
                     end;
                     %
                     % ask if we should include conditioning assumptions for the the observed variables
                     %
                     if (UniqueShocks==1)&(EnoughVariables==1);
                        Ostr = '';
                        ItStr = 'it';
                        if length(DSGEModel.ConditionalVariables)>1;
                           Ostr = 's';
                           ItStr = 'them';
                        end;
                        txt = ['YADA has located ' int2str(length(DSGEModel.ConditionalVariables)) ' conditioning assumption' Ostr ' for the observed variables. Would you like to append ' ItStr ' to your ' int2str(NumZeta) ' assumption' SStr ' for the state variables?'];
                        answer = Query(txt,'question',200,'Conditional Predictive Distribution - Initial Values',500,'no',CurrINI);
                        if strcmp(lower(answer),'yes')==1;
                           Z = DSGEModel.Z(DSGEModel.ConditionalVariables,LastPeriod+1:LastPeriod+NumPeriods);
                           U = DSGEModel.U(DSGEModel.ConditionalVariables,LastPeriod-NumPeriods+1:LastPeriod);
                           CondTypeStr = 'State-Obs-';
                           CTypeStr = 'SO';
                        end;
                     end;
                  end;
               end;
               %
               % check if we should annualize or not
               %
               if gcbo==controls.predict.stateconditional_original_initialvalues;
                  TransData = 0;
                  AnnualizeData = 0;
                  AnnualStr = 'original';
               elseif gcbo==controls.predict.stateconditional_annual_initialvalues;
                  TransData = 0;
                  AnnualizeData = 1;
                  AnnualStr = 'annual';
               else;
                  TransData = 1;
                  AnnualizeData = 0;
                  AnnualStr = 'transformed';
               end;
               %
               % determine how many prediction paths to use
               %
               NumPredPathsStr = get(controls.forecast.predpaths,'String');
               NumPredPaths = str2num(strrep(strrep(strrep(StringTrim(NumPredPathsStr(DSGEModel.NumPredPathsValue,:)),'path',''),'s',''),',',''));
               %
               % ask if we should conduct prediction event calculations
               %
               if NumPredPaths>1;
                  if (isfield(DSGEModel,'PredictionEvent')==0)|(isempty(DSGEModel.PredictionEvent)==1)|(size(DSGEModel.PredictionEvent,1)~=DSGEModel.n)|(size(DSGEModel.PredictionEvent,2)~=3);
                     DSGEModel.PredictionEvent = [Inf*ones(DSGEModel.n,1) zeros(DSGEModel.n,1) ones(DSGEModel.n,1)];
                  end;
                  txt = 'Would you like YADA to calculate prediction event probabilities and perform risk analysis?';
                  answer = Query(txt,'question',140,'Prediction Events',500,'no',CurrINI);
               else;
                  answer = 'No';
               end;
               if strcmp(lower(answer),'yes')==1;
                  %
                  % make sure that the prediction event length does not exceed the maximum prediction horizon
                  %
                  for i=1:DSGEModel.n;
                     if DSGEModel.PredictionEvent(i,3)>MaxPH;
                        DSGEModel.PredictionEvent(i,3) = MaxPH;
                     end;
                  end;
                  %
                  % display a dialog for setting up an prediction event
                  %
                  [action,PredictionEvent] = PredictionEventDLG('init',DSGEModel.VariableNames,DSGEModel.PredictionEvent,MaxPH,CurrINI);
                  if strcmp(lower(action),'ok')==1;
                     DSGEModel.PredictionEvent = PredictionEvent;
                     DSGEModel.RunPredictionEvent = 1;
                  else;
                     DSGEModel.RunPredictionEvent = 0;
                  end;
               else;
                  DSGEModel.RunPredictionEvent = 0;
               end;
               %
               % we can now compute the predictive distribution for the endogenous variables
               % up to period MaxPh
               %
               if DSGEModel.RandomNumberValue==1;
                  randn('state',0);
               else;
                  randn('state',sum(100*clock));
               end;
               %
               if DSGEModel.ShockControlMethod==1;
                  [PredPaths,PredData,PredEventData,YObsEventData,MultiModestyStat,UniModestyStat,MeanUniModestyLZ,mcode,kalmanstatus] = StateCondPredictionPathsTheta(theta,thetaPositions,ModelParameters,DSGEModel,CurrINI,X,Zeta,Z,U,MaxPH,NumPredPaths,AnnualizeData,TransData,' At Initial Values');
               elseif DSGEModel.ShockControlMethod==2;
                  [PredPaths,PredData,PredEventData,YObsEventData,MultiModestyStat,UniModestyStat,MeanUniModestyLZ,mcode,kalmanstatus] = StateCondPredictionPathsThetaWZ(theta,thetaPositions,ModelParameters,DSGEModel,CurrINI,X,Zeta,Z,U,MaxPH,NumPredPaths,AnnualizeData,TransData,' At Initial Values');
               else;
                  %
                  % check if the selected method can actually be used
                  %
                  CondTxt = '';
                  if isempty(DSGEModel.ZetaMixedConditionalShocks)==1;
                     CondTxt = 'You haven''t selected any shocks for use which the distribution of a subset of the shocks conditioning method.';
                  elseif length(DSGEModel.ZetaConditionalVariables)+1>=length(DSGEModel.StateShockPositions);
                     CondTxt = 'You have too many conditioning assumptions to be able to use the distribution of a subset of the shocks conditioning method.';
                  elseif length(DSGEModel.ZetaMixedConditionalShocks)<length(DSGEModel.ZetaConditionalVariables);
                     CondTxt = ['The number of shocks you have selected to use to ensure that the conditioning assumptions are satisfied under the distribution of a subset of the shocks conditioning method is not sufficient. You need at least ' int2str(length(DSGEModel.ZetaConditionalVariables)) ' such shocks.'];
                  end;
                  if isempty(CondTxt)==1;
                     [PredPaths,PredData,PredEventData,YObsEventData,MultiModestyStat,UniModestyStat,MeanUniModestyLZ,mcode,kalmanstatus] = StateCondPredictionPathsThetaMixed(theta,thetaPositions,ModelParameters,DSGEModel,CurrINI,X,Zeta,Z,U,MaxPH,NumPredPaths,AnnualizeData,TransData,' At Initial Values');
                  else;
                     About(CondTxt,'information','Bad Shocks Selection - Distribution of Subset Shocks Method',300,550,CurrINI);
                     PredPaths = [];
                  end;
               end;
               %
               if (mcode==1)&(kalmanstatus==0);
                  %
                  % we're hopefully OK
                  %
                  if isempty(PredPaths)==0;
                     %
                     % the computations went OK. Compute point estimates from predictive distribution and
                     % quantiles
                     %
                     [MeanPrediction,Quantiles] = PredictionPathDistribution(PredPaths,DSGEModel.Percentiles,DSGEModel.ConfidenceRegionMethod);
                     %
                     % compute the mean error bias in percent
                     %
                     MeanErrorBias = max(max(abs((PredData.PredMean-MeanPrediction)./PredData.PredMean)));
                     if DSGEModel.AdjustPredictionPaths==0;
                        %
                        % ask if the prediction distribution should be corrected
                        %
                        txt = ['The absolute mean error bias from using the simulated draws is ' num2str(MeanErrorBias,'%0.4f') ' percent relative to the theoretical mean. Would you like to adjust that predictive distribution such that it matches the theoretical mean?'];
                        answer = Query(txt,'question',140,'Mean Error Bias - Conditional Predictive Distributions',500,'yes',CurrINI);
                     else;
                        answer = 'Yes';
                     end;
                     AdjustMean = 0;
                     if strcmp(lower(answer),'yes')==1;
                        %
                        % adjust the quantiles to match
                        %
                        for i=1:length(Quantiles);
                           OlVal = Quantiles(i).Quantile;
                           Quantiles(i).Quantile = Quantiles(i).Quantile+(PredData.PredMean-MeanPrediction);
                        end;
                        MeanPrediction = PredData.PredMean;
                        PredData.MeanPredPath = PredData.OrigPredMean;
                        AdjustMean = 1;
                     end;
                     %
                     % compute the forecast errors based on the mean predictions
                     %
                     PredictionErrors = CalculatePredictionErrors(MeanPrediction,AnnualizeData,TransData,'DSGE',DSGEModel);
                     %
                     % check if we have modesty statistics to deal with
                     %
                     if AnnualizeData==0;
                        %
                        % check how often we have a greater value of column 1 than column 2 in MultiModestyStat
                        %
                        MultiPValue = sum(MultiModestyStat(:,1)<=MultiModestyStat(:,2))/NumPredPaths;
                        MeanMultiModesty = (1/NumPredPaths)*[sum(MultiModestyStat(:,1)) sum(MultiModestyStat(:,2))];
                        MeanUniModesty = zeros(1,DSGEModel.n);
                        for i=1:DSGEModel.n;
                           MeanUniModesty(1,i) = (1/NumPredPaths)*sum(UniModestyStat(:,i));
                        end;
                        if exist('YADAchi2cdf')==2;
                           ChiSqPValue = 1-YADAchi2cdf(MeanMultiModesty(1),DSGEModel.n);
                        else;
                           if exist('chi2cdf')==2;
                              ChiSqPValue = 1-chi2cdf(MeanMultiModesty(1),DSGEModel.n);
                           else;
                              ChiSqPValue = 1-YADACheapchi2cdf(MeanMultiModesty(1),DSGEModel.n);
                           end;
                        end;
                        ResFile = PrintDSGEModestyStatistics(DSGEModel,CurrINI,'Initial Values',1,MaxPH,NumPeriods,1,NumPredPaths,MultiPValue,MeanMultiModesty,MeanUniModesty,MeanUniModestyLZ,PredData.etaShocks,PredData.ShockNames,CondTypeStr);
                        %
                        % ask if the user wishes to view the modesty analysis
                        %
                        txt = ['YADA has written the modesty analysis results to the file "' GetFilename(ResFile) '" in the directory "' GetPath(ResFile) '". Would you like to view the results?'];
                        answer = Query(txt,'question',300,'Conditional Predictive Distributions - Modesty Statistics',600,'yes',CurrINI);
                        if strcmp(lower(answer),'yes')==1;
                           TextGUI(ResFile,'Modesty Statistics - Initial Values',[(CurrINI.scrsz(3)-1000)/2 32 1000 CurrINI.scrsz(4)-100],'Results',40,CurrINI,1,0);
                        end;
                     else;
                        MultiPValue = [];
                        MeanMultiModesty = [];
                        MeanUniModesty = [];
                        ChiSqPValue = [];
                     end;
                     %
                     % check if the user wants to save the data
                     %
                     txt = ['Would you like to save the conditional predictive distributions for the ' AnnualStr ' data at initial values to file?'];
                     answer = Query(txt,'question',140,'Save - Conditional Predictive Distributions',500,'no',CurrINI);
                     drawnow;
                     if strcmp(lower(answer),'yes')==1;
                        [PredStartYear,PredStartPeriod,PredEndYear,PredEndPeriod,PredDateVector] = CreatePredictionSample(DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,DSGEModel.DataFrequency,size(MeanPrediction,2));
                        file = FixFilePath([DSGEModel.OutputDirectory '\CondPredictiveDist-' CondTypeStr int2str(DSGEModel.ShockControlMethod) '-' AnnualStr '-' num2str(MaxPH,'%0.0f') '-' DSGEModel.SubEndYear '.' DSGEModel.SubEndPeriod '-InitialValues.mat']);
                        YNames = DSGEModel.VariableNames;
                        StateVariableNames = DSGEModel.StateVariableNames;
                        DateVector = MakeDateVector(DSGEModel.SubBeginYear,DSGEModel.SubBeginPeriod,PredEndYear,PredEndPeriod,DSGEModel.DataFrequency);
                        Y = DSGEModel.Y(:,FirstPeriod:min((LastPeriod+MaxPH),size(DSGEModel.Y,2)));
                        if (length(DateVector)>size(Y,2))&(TransData==0);
                           DateVector = DateVector(1,1:size(Y,2));
                        end;
                        if AnnualizeData==1;
                           Y = Annualize(Y,DSGEModel.annual,DSGEModel.annualscale);
                           DateVector = DateVector(length(DateVector)-size(Y,2)+1:length(DateVector));
                        end;
                        %
                        if TransData==1;
                           [YTStatus,YData,ErrorStr,HeaderStr] = CalcTransformData(DSGEModel.VariableNames,DSGEModel.Y',DSGEModel.YTransformation,DSGEModel.YTransMatrix);
                           Y = YData';
                           if size(Y,2)==DSGEModel.T;
                              Y = Y(:,FirstPeriod:min((LastPeriod+MaxPH),size(DSGEModel.Y,2)));
                           else;
                              T2 = size(Y,2);
                              if (FirstPeriod-(DSGEModel.T-T2))>=1;
                                 Y = Y(:,FirstPeriod-(DSGEModel.T-T2):min((LastPeriod+MaxPH),size(DSGEModel.Y,2)));
                              else;
                                 Y = Y(:,1:min((LastPeriod+MaxPH),size(DSGEModel.Y,2))-(DSGEModel.T-T2));
                                 DateVector = DateVector(length(DateVector)-size(Y,2)+1:length(DateVector));
                              end;
                           end;
                        end;
                        KernelX = PredData.KernelX;
                        KernelY = PredData.KernelY;
                        PredMeanKsiStates = PredData.xiStates;
                        PredMean = PredData.PredMean;
                        PredMeanEtaShocks = PredData.etaShocks;
                        %
                        [B0_h,KeepShocks] = RemoveRedundantColumns(PredData.B0);
                        ShockNames = DSGEModel.StateShockNames(KeepShocks,:);
                        %
                        if isempty(PredEventData)==1;
                           save(file,'MeanPrediction','PredMean','PredMeanKsiStates','PredMeanEtaShocks','PredictionErrors','Quantiles','KernelX','KernelY','PredDateVector','Y','DateVector','YNames','StateVariableNames','ShockNames','MeanMultiModesty','MultiPValue','ChiSqPValue','MeanUniModesty','MeanUniModestyLZ');
                        else;
                           save(file,'MeanPrediction','PredMean','PredMeanKsiStates','PredMeanEtaShocks','PredictionErrors','Quantiles','KernelX','KernelY','PredDateVector','Y','DateVector','YNames','StateVariableNames','ShockNames','MeanMultiModesty','MultiPValue','ChiSqPValue','MeanUniModesty','MeanUniModestyLZ','PredictionEvent','PredEventData');
                        end;
                        %
                        % create a matlab m-file to read the output.
                        %
                        mfile = FixFilePath([DSGEModel.OutputDirectory '\CondPD' CTypeStr int2str(DSGEModel.ShockControlMethod) AnnualStr num2str(MaxPH,'%0.0f') DSGEModel.SubEndYear DSGEModel.SubEndPeriod 'InitVal.m']);
                        fid = fopen(mfile,'wt');
                        fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                        fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                        fprintf(fid,'%%\n%% string matrix with the names of the observed variables\n%%\n');
                        fprintf(fid,'YNames = DataStructure.YNames;\n');
                        if AdjustMean==0;
                           fprintf(fid,'%%\n%% matrix with the sample mean of the simulated predictions of the observed variables\n%%\n');
                        else;
                           fprintf(fid,'%%\n%% matrix with the corrected sample mean of the simulated predictions of the observed variables\n%%\n');
                        end;
                        fprintf(fid,'MeanPrediction = DataStructure.MeanPrediction;\n');
                        fprintf(fid,'%%\n%% matrix with the population mean predictions of the observed variables\n%%\n');
                        fprintf(fid,'PredMean = DataStructure.PredMean;\n');
                        fprintf(fid,'%%\n%% string matrix with the names of the state variable variables\n%%\n');
                        fprintf(fid,'StateVariableNames = DataStructure.StateVariableNames;\n');
                        fprintf(fid,'%%\n%% string matrix with the names of the economic shocks\n%%\n');
                        fprintf(fid,'ShockNames = DataStructure.ShockNames;\n');
                        fprintf(fid,'%%\n%% matrix with the population mean predictions of the state variables\n%%\n');
                        fprintf(fid,'PredMeanKsiStates = DataStructure.PredMeanKsiStates;\n');
                        fprintf(fid,'%%\n%% matrix with the population mean of the predictions of the economic shocks\n%%\n');
                        fprintf(fid,'PredMeanEtaShocks = DataStructure.PredMeanEtaShocks;\n');
                        fprintf(fid,'%%\n%% matrix with the errors from the mean predictions of the observed variables\n%%\n');
                        fprintf(fid,'PredictionErrors = DataStructure.PredictionErrors;\n');
                        fprintf(fid,'%%\n%% vector structure with quantiles from the predictive distribution\n%% the structure has %0.0f entries with fields "Quantile" and "percent"\n%% the first field is a matrix and the second a positive number between 0 and 100\n%%\n',length(Quantiles));
                        fprintf(fid,'Quantiles = DataStructure.Quantiles;\n');
                        fprintf(fid,'%%\n%% matrix with x-axis values for kernel density estimates of marginal predictive distribution\n%%\n');
                        fprintf(fid,'KernelX = DataStructure.KernelX;\n');
                        fprintf(fid,'%%\n%% matrix with y-axis values for kernel density estimates of marginal predictive distribution\n%%\n');
                        fprintf(fid,'KernelY = DataStructure.KernelY;\n');
                        fprintf(fid,'%%\n%% vector with dates for the predictions\n%%\n');
                        fprintf(fid,'PredDateVector = DataStructure.PredDateVector;\n');
                        fprintf(fid,'%%\n%% observed variables\n%%\n');
                        fprintf(fid,'Y = DataStructure.Y;\n');
                        fprintf(fid,'%%\n%% vector with dates for the observed variables\n%%\n');
                        fprintf(fid,'DateVector = DataStructure.DateVector;\n');
                        if isempty(PredEventData)==0;
                           fprintf(fid,'%%\n%% matrix with the prediction event definitions. The first column\n%% holds the upper bound, the second column\n%% the lower bound, and third the length of the event.\n%%\n');
                           fprintf(fid,'PredictionEvent = DataStructure.PredictionEvent;\n');
                           fprintf(fid,'%%\n%% matrix with prediction event data. First column holds the number of true\%% events, the second the number of times below the lower bound of the event,\n%% the third the number of times above the upper bound of the event, the\n%% fourth the number of events, the fifth and sixth the sum and sum of\n%% squared deviations from the lower bound when below, and the seventh and\n%% eighth the sum and sum of squared deviations from the upper bound when above\n%%\n');
                           fprintf(fid,'PredEventData = DataStructure.PredEventData;\n');
                           NumEntries = '16';
                           ExtraEntry = 'PredictionEvent (matrix with the upper and lower bound of the event and the length of the event), PredEventData (matrix with the number of true events, the number of events below the lower bound for the event, the number of events above the upper bound for the event, the number of events tested in the columns, the sum and sum of squared deviations from lower bound when below lower bound, the sum and sum of squared deviations from upper bound when above upper bound), ';
                        else;
                           NumEntries = '14';
                           ExtraEntry = '';
                        end;
                        if AnnualizeData==0;
                           fprintf(fid,'%%\n%% vector with the mean of the multivariate modesty statistic and its reference statistic\n%%\n');
                           fprintf(fid,'MeanMultiModesty = DataStructure.MeanMultiModesty;\n');
                           fprintf(fid,'%%\n%% tail probability based on the reference statistic\n%%\n');
                           fprintf(fid,'MultiPValue = DataStructure.MultiPValue;\n');
                           fprintf(fid,'%%\n%% tail probability based on the chi-square distribution\n%%\n');
                           fprintf(fid,'ChiSqPValue = DataStructure.ChiSqPValue;\n');
                           fprintf(fid,'%%\n%% Adolfson, Lasen, Lind and Villani univariate modesty statistic\n%%\n');
                           fprintf(fid,'MeanUniModesty = DataStructure.MeanUniModesty;\n');
                           fprintf(fid,'%%\n%% Leeper and Zha univariate modesty statistic\n%%\n');
                           fprintf(fid,'MeanUniModestyLZ = DataStructure.MeanUniModestyLZ;\n');
                           NumEntries = num2str(str2num(NumEntries)+5,'%0.0f');
                           ExtraEntry = [ExtraEntry 'MeanMultiModesty (1x2 vector with the mean of the multivariate modesty statistic and the mean of the multivariate reference modesty statistic), MultiPValue (scalar with the tail probability of the modesty statistic), ChiSqPValue (scalar with the tail probability of the modesty statistic using the chi-square distribution), MeanUniModesty (vector with the univariate modesty statistic from ALLV), MeanUniModestyLZ (vector with the univariate Leeper-Zha modesty statistic), '];
                        end;
                        fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                        %
                        % end the script file
                        %
                        fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                        fclose(fid);
                        %
                        txt = ['The conditional predictive distributions have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains ' NumEntries ' entries: YNames (string matrix with the names of the observed variables), MeanPrediction (matrix with the mean predictions), PredMean (population mean prediction), PredMeanKsiStates (population mean of the state variables), PredMeanEtaShocks (population mean of the economic shocks), StateVariableNames (string matrix with the names of the state varaibles), ShockNames (string matrix with the names of the economic shocks), PredictionErrors (matrix with errors for the mean predictions), Quantiles (vector structure with 2 fields: Quantile and percent), KernelX (matrix with x-axis for kernel density estimates of marginal predictive distribution), KernelY (matrix with y-axis values for kernel density estimates of marginal predictive distribution), PredDateVector (vector with dates for the predictions), Y (matrix with the observed variables), ' ExtraEntry 'and DateVector (vector with dates for the observed variables). To access these data you may run the matlab script file "' GetFilename(mfile) '".'];
                        About(txt,'information','Conditional Predictive Distributions - Initial Values',500,600,CurrINI);
                     end;
                     if isempty(PredEventData)==0;
                        %
                        % write prediction event results to file
                        %
                        file = FixFilePath([DSGEModel.OutputDirectory '\CondPredictiveDist-' CondTypeStr int2str(DSGEModel.ShockControlMethod) '-' AnnualStr '-' num2str(MaxPH,'%0.0f') '-' DSGEModel.SubEndYear '.' DSGEModel.SubEndPeriod '-InitialValues.txt']);
                        [PredStartYear,PredStartPeriod,PredEndYear,PredEndPeriod] = CreatePredictionSample(DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,DSGEModel.DataFrequency,MaxPH);
                        PrintPredictionEvent(file,DSGEModel.VariableNames,PredEventData,YObsEventData,DSGEModel.PredictionEvent,[PredStartYear ':' PredStartPeriod '-' PredEndYear ':' PredEndPeriod],MaxPH,NumPredPaths,1,CurrINI);
                        %
                        % displays the file if it exists
                        %
                        if FileExist(file)==1;
                           TextGUI(file,['Prediction Events - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 32 1000 CurrINI.scrsz(4)-100],'Initial Parameter Values',50,CurrINI,1,0);
                        end;
                     end;
                     %
                     % Open a dialog for displaying predictive distributions
                     %
                     CondPredictiveDistributionDLG('init',DSGEModel,CurrINI,MeanPrediction,Quantiles,PredData.KernelX,PredData.KernelY,'Initial Values','DSGE',AnnualStr,1);
                  end;
               elseif (mcode==1)&(kalmanstatus~=0);
                  if kalmanstatus==1;
                     txt = 'The state variables are not stationary at the initial values.';
                  else;
                     txt = 'The covariance matrix of the forecast of the observed variables in period t conditional on data until t-1 is not positive definite for some time period t. This implies that the log-likelihood value is a NaN (or a complex number) in Matlab.';
                  end;
                  About(txt,'information','Conditional Predictive Distributions - Initial Values',200,500,CurrINI);
               elseif mcode~=1;
                  if mcode==2;
                     mcodeStr = 'Roots are not correctly computed by real_schur.'
                  elseif mcode==3;
                     mcodeStr = 'Too many big roots.';
                  elseif mcode==35;
                     mcodeStr = 'Too many big roots, and q(:,right) is singular.';
                  elseif mcode==4;
                     mcodeStr = 'Too few big roots.';
                  elseif mcode==45;
                     mcodeStr = 'Too few big roots, and q(:,right) is singular.';
                  elseif mcode==5;
                     mcodeStr = 'q(:,right) is singular.';
                  elseif mcode==61;
                     mcodeStr = 'Too many exact shiftrights.';
                  elseif mcode==62;
                     mcodeStr = 'Too many numeric shiftrights.';
                  elseif mcode==7;
                     mcodeStr = 'Infinite or NaN values detected.';
                  elseif mcode==8;
                     mcodeStr = 'The function "compute_aim_matrices" returns complex numbers.';
                  elseif mcode==-1;
                     mcodeStr = 'No stable solution.';
                  elseif mcode==-2;
                     mcodeStr = 'Too many large generalized eigenvalues.';
                  elseif mcode==-3;
                     mcodeStr = 'Too few large generalized eigenvalues.';
                  else;
                     mcodeStr = 'Return code not properly specified.';
                  end;
                  if DSGEModel.ModelSolver==1;
                     SolverStr = 'AiM';
                  elseif DSGEModel.ModelSolver==2;
                     SolverStr = 'Klein';
                  elseif DSGEModel.ModelSolver==3;
                     SolverStr = 'Gensys';
                  end;
                  SetLogText(strvcat(['   STATUS: ' mcodeStr],' '));
                  txt = ['The ' SolverStr ' solver provided the return code: ' num2str(mcode) ', i.e., "' mcodeStr '" You have to change your initial parameter values.'];
                  About(txt,'information','Conditional Predictive Distributions - Initial Values',200,500,CurrINI);
               end;
            end;
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'PosteriorMode'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         %
         % File with the data needed to calculate the predictive distr
         %
         drawnow;
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
         ErrorStr = '';
         HeaderStr = 'YADA - Bad MAT File';
         try;
            ModeData = load(ModeFile);
         catch;
            ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' lasterr];
         end;
         %
         % check if we should check transformations functions
         %
         if (isempty(ErrorStr)==1)&(gcbo==controls.predict.stateconditional_transformed_postmode);
            DTStatus = VerifyDataTransformation(DSGEModel.VariableNames,DSGEModel.YTransformation,0);
            if DTStatus==1;
               [YTStatus,YData,ErrorStr,HeaderStr] = CalcTransformData(DSGEModel.VariableNames,DSGEModel.Y',DSGEModel.YTransformation,DSGEModel.YTransMatrix);
            else;
               ErrorStr = 'You haven''t specified all the necessary fields for computing transformations of the observed variables.';
               HeaderStr = 'Missing Transformation Information';
            end;
         end;
         %
         if strcmp(ErrorStr,'')==1;
            theta = ModeData.thetaMode;
            thetaPositions = ModeData.thetaPositions;
            ModelParameters = ModeData.ModelParameters;
            [FirstPeriod,LastPeriod] = CreateSubSample(DSGEModel);
            %
            % check how many time periods to use for predictive distributions
            %
            [MaxPH,NumPredX] = CalcMaxPredictionHorizon(DSGEModel.MaxForecastHorizon,DSGEModel.DataFrequency,DSGEModel.EndYear,DSGEModel.EndPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,size(DSGEModel.PredictedX,2));
            %
            % matrix with exogenous variables
            %
            X = [];
            if DSGEModel.k>0;
               if NumPredX<MaxPH;
                  X = DSGEModel.X(:,LastPeriod+1:LastPeriod+(MaxPH-NumPredX));
               end;
               if NumPredX>0;
                  X = [X DSGEModel.PredictedX(:,1:NumPredX)];
               end;
            end;
            %
            % check the length of the conditioning sample
            %
            [status,NumPeriods] = CheckStateCondData(DSGEModel);
            %
            % check if we should adjust the Zeta matrix
            %
            if MaxPH<NumPeriods;
               NumPeriods = MaxPH;
            end;
            %
            % collect the conditional variables and the selected conditioning sample
            %
            Zeta = DSGEModel.Zeta(DSGEModel.ZetaConditionalVariables,LastPeriod+1:LastPeriod+NumPeriods);
            %
            % check if there are conditioning assumptions for the observed variables
            %
            NumZeta = length(DSGEModel.ZetaConditionalVariables);
            SStr = '';
            if NumZeta>1;
               SStr = 's';
            end;
            Z = [];
            U = [];
            CondTypeStr = 'State-';
            CTypeStr = 'S';
            if isempty(DSGEModel.Z)==0;
               %
               % check if we can use the data
               %
               [status,NumZPeriods] = CheckConditioningData(DSGEModel);
               if (status==1)&(NumZPeriods>=NumPeriods);
                  %
                  % check if the number of conditioning assumptions exceeds the number of
                  % observed variables and shocks.
                  %
                  r = length(DSGEModel.StateVariablePositions);
                  EnoughVariables = ((length([DSGEModel.ConditionalVariables DSGEModel.ZetaConditionalVariables])<=min(DSGEModel.n+r,size(DSGEModel.StateShockNames,1))))&(DSGEModel.ZetaConditionalVariables<=r)&(length(DSGEModel.ConditionalVariables)<=DSGEModel.n);
                  %
                  % check if we have selected enough unique shocks
                  %
                  if DSGEModel.ShockControlMethod==1;
                     UniqueShocks = (length(unique([DSGEModel.ZetaConditionalShocks DSGEModel.ConditionalShocks]))==length([DSGEModel.ZetaConditionalShocks DSGEModel.ConditionalShocks]));
                  elseif DSGEModel.ShockControlMethod==2;
                     UniqueShocks = 1;
                  else;
                     NumShocks = length(unique([DSGEModel.ZetaMixedConditionalShocks DSGEModel.MixedConditionalShocks]));
                     NumAssumptions = length([DSGEModel.ConditionalVariables DSGEModel.ZetaConditionalVariables]);
                     UniqueShocks = (NumShocks>=NumAssumptions)&(NumShocks<=length(DSGEModel.StateShockPositions));
                  end;
                  %
                  % ask if we should include conditioning assumptions for the the observed variables
                  %
                  if (UniqueShocks==1)&(EnoughVariables==1);
                     Ostr = '';
                     ItStr = 'it';
                     if length(DSGEModel.ConditionalVariables)>1;
                        Ostr = 's';
                        ItStr = 'them';
                     end;
                     txt = ['YADA has located ' int2str(length(DSGEModel.ConditionalVariables)) ' conditioning assumption' Ostr ' for the observed variables. Would you like to append ' ItStr ' to your ' int2str(NumZeta) ' assumption' SStr ' for the state variables?'];
                     answer = Query(txt,'question',200,'Conditional Predictive Distribution - Posterior Mode',500,'no',CurrINI);
                     if strcmp(lower(answer),'yes')==1;
                        Z = DSGEModel.Z(DSGEModel.ConditionalVariables,LastPeriod+1:LastPeriod+NumPeriods);
                        U = DSGEModel.U(DSGEModel.ConditionalVariables,LastPeriod-NumPeriods+1:LastPeriod);
                        CondTypeStr = 'State-Obs-';
                        CTypeStr = 'SO';
                     end;
                  end;
               end;
            end;
            %
            % check if we should annualize or not
            %
            if gcbo==controls.predict.stateconditional_original_postmode;
               TransData = 0;
               AnnualizeData = 0;
               AnnualStr = 'original';
            elseif gcbo==controls.predict.stateconditional_annual_postmode;
               TransData = 0;
               AnnualizeData = 1;
               AnnualStr = 'annual';
            else;
               TransData = 1;
               AnnualizeData = 0;
               AnnualStr = 'transformed';
            end;
            %
            % determine how many prediction paths to use
            %
            NumPredPathsStr = get(controls.forecast.predpaths,'String');
            NumPredPaths = str2num(strrep(strrep(strrep(StringTrim(NumPredPathsStr(DSGEModel.NumPredPathsValue,:)),'path',''),'s',''),',',''));
            %
            % ask if we should conduct prediction event calculations
            %
            if NumPredPaths>1;
               if (isfield(DSGEModel,'PredictionEvent')==0)|(isempty(DSGEModel.PredictionEvent)==1)|(size(DSGEModel.PredictionEvent,1)~=DSGEModel.n)|(size(DSGEModel.PredictionEvent,2)~=3);
                  DSGEModel.PredictionEvent = [Inf*ones(DSGEModel.n,1) zeros(DSGEModel.n,1) ones(DSGEModel.n,1)];
               end;
               txt = 'Would you like YADA to calculate prediction event probabilities and perform risk analysis?';
               answer = Query(txt,'question',140,'Prediction Events',500,'no',CurrINI);
            else;
               answer = 'No';
            end;
            if strcmp(lower(answer),'yes')==1;
               %
               % make sure that the prediction event length does not exceed the maximum prediction horizon
               %
               for i=1:DSGEModel.n;
                  if DSGEModel.PredictionEvent(i,3)>MaxPH;
                     DSGEModel.PredictionEvent(i,3) = MaxPH;
                  end;
               end;
               %
               % display a dialog for setting up an prediction event
               %
               [action,PredictionEvent] = PredictionEventDLG('init',DSGEModel.VariableNames,DSGEModel.PredictionEvent,MaxPH,CurrINI);
               if strcmp(lower(action),'ok')==1;
                  DSGEModel.PredictionEvent = PredictionEvent;
                  DSGEModel.RunPredictionEvent = 1;
               else;
                  DSGEModel.RunPredictionEvent = 0;
               end;
            else;
               DSGEModel.RunPredictionEvent = 0;
            end;
            %
            % we can now compute the predictive distribution for the endogenous variables
            % up to period MaxPh
            %
            if DSGEModel.RandomNumberValue==1;
               randn('state',0);
            else;
               randn('state',sum(100*clock));
            end;
            %
            if DSGEModel.ShockControlMethod==1;
               [PredPaths,PredData,PredEventData,YObsEventData,MultiModestyStat,UniModestyStat,MeanUniModestyLZ] = StateCondPredictionPathsTheta(theta,thetaPositions,ModelParameters,DSGEModel,CurrINI,X,Zeta,Z,U,MaxPH,NumPredPaths,AnnualizeData,TransData,' At Posterior Mode');
            elseif DSGEModel.ShockControlMethod==2;
               [PredPaths,PredData,PredEventData,YObsEventData,MultiModestyStat,UniModestyStat,MeanUniModestyLZ] = StateCondPredictionPathsThetaWZ(theta,thetaPositions,ModelParameters,DSGEModel,CurrINI,X,Zeta,Z,U,MaxPH,NumPredPaths,AnnualizeData,TransData,' At Posterior Mode');
            else;
               %
               % check if the selected method can actually be used
               %
               CondTxt = '';
               if isempty(DSGEModel.ZetaMixedConditionalShocks)==1;
                  CondTxt = 'You haven''t selected any shocks for use which the distribution of a subset of the shocks conditioning method.';
               elseif length(DSGEModel.ZetaConditionalVariables)+1>=length(DSGEModel.StateShockPositions);
                  CondTxt = 'You have too many conditioning assumptions to be able to use the distribution of a subset of the shocks conditioning method.';
               elseif length(DSGEModel.ZetaMixedConditionalShocks)<length(DSGEModel.ZetaConditionalVariables);
                  CondTxt = ['The number of shocks you have selected to use to ensure that the conditioning assumptions are satisfied under the distribution of a subset of the shocks conditioning method is not sufficient. You need at least ' int2str(length(DSGEModel.ZetaConditionalVariables)) ' such shocks.'];
               end;
               if isempty(CondTxt)==1;
                  [PredPaths,PredData,PredEventData,YObsEventData,MultiModestyStat,UniModestyStat,MeanUniModestyLZ] = StateCondPredictionPathsThetaMixed(theta,thetaPositions,ModelParameters,DSGEModel,CurrINI,X,Zeta,Z,U,MaxPH,NumPredPaths,AnnualizeData,TransData,' At Posterior Mode');
               else;
                  About(CondTxt,'information','Bad Shocks Selection - Distribution of Subset Shocks Method',300,550,CurrINI);
                  PredPaths = [];
               end;
            end;
            if isempty(PredPaths)==0;
               %
               % the computations went OK. Compute point estimates from predictive distribution and
               % quantiles
               %
               [MeanPrediction,Quantiles] = PredictionPathDistribution(PredPaths,DSGEModel.Percentiles,DSGEModel.ConfidenceRegionMethod);
               %
               % compute the mean error bias in percent
               %
               MeanErrorBias = max(max(abs((PredData.PredMean-MeanPrediction)./PredData.PredMean)));
               if DSGEModel.AdjustPredictionPaths==0;
                  %
                  % ask if the prediction distribution should be corrected
                  %
                  txt = ['The absolute mean error bias from using the simulated draws is ' num2str(MeanErrorBias,'%0.4f') ' percent relative to the theoretical mean. Would you like to adjust that predictive distribution such that it matches the theoretical mean?'];
                  answer = Query(txt,'question',140,'Mean Error Bias - Conditional Predictive Distributions',500,'yes',CurrINI);
               else;
                  answer = 'Yes';
               end;
               AdjustMean = 0;
               if strcmp(lower(answer),'yes')==1;
                  %
                  % adjust the quantiles to match
                  %
                  for i=1:length(Quantiles);
                     OlVal = Quantiles(i).Quantile;
                     Quantiles(i).Quantile = Quantiles(i).Quantile+(PredData.PredMean-MeanPrediction);
                  end;
                  MeanPrediction = PredData.PredMean;
                  PredData.MeanPredPath = PredData.OrigPredMean;
                  AdjustMean = 1;
               end;
               %
               % compute the forecast errors based on the mean predictions
               %
               PredictionErrors = CalculatePredictionErrors(MeanPrediction,AnnualizeData,TransData,'DSGE',DSGEModel);
               %
               % check if we have modesty statistics to deal with
               %
               if AnnualizeData==0;
                  %
                  % check how often we have a greater value of column 1 than column 2 in MultiModestyStat
                  %
                  MultiPValue = sum(MultiModestyStat(:,1)<=MultiModestyStat(:,2))/NumPredPaths;
                  MeanMultiModesty = (1/NumPredPaths)*[sum(MultiModestyStat(:,1)) sum(MultiModestyStat(:,2))];
                  MeanUniModesty = zeros(1,DSGEModel.n);
                  for i=1:DSGEModel.n;
                     MeanUniModesty(1,i) = (1/NumPredPaths)*sum(UniModestyStat(:,i));
                  end;
                  if exist('YADAchi2cdf')==2;
                     ChiSqPValue = 1-YADAchi2cdf(MeanMultiModesty(1),DSGEModel.n);
                  else;
                     if exist('chi2cdf')==2;
                        ChiSqPValue = 1-chi2cdf(MeanMultiModesty(1),DSGEModel.n);
                     else;
                        ChiSqPValue = 1-YADACheapchi2cdf(MeanMultiModesty(1),DSGEModel.n);
                     end;
                  end;
                  ResFile = PrintDSGEModestyStatistics(DSGEModel,CurrINI,'Posterior Mode',1,MaxPH,NumPeriods,1,NumPredPaths,MultiPValue,MeanMultiModesty,MeanUniModesty,MeanUniModestyLZ,PredData.etaShocks,PredData.ShockNames,CondTypeStr);
                  %
                  % ask if the user wishes to view the modesty analysis
                  %
                  txt = ['YADA has written the modesty analysis results to the file "' GetFilename(ResFile) '" in the directory "' GetPath(ResFile) '". Would you like to view the results?'];
                  answer = Query(txt,'question',300,'Conditional Predictive Distributions - Modesty Statistics',600,'yes',CurrINI);
                  if strcmp(lower(answer),'yes')==1;
                     TextGUI(ResFile,'Modesty Statistics - Posterior Mode',[(CurrINI.scrsz(3)-1000)/2 32 1000 CurrINI.scrsz(4)-100],'Results',40,CurrINI,1,0);
                  end;
               else;
                  MultiPValue = [];
                  MeanMultiModesty = [];
                  MeanUniModesty = [];
                  ChiSqPValue = [];
               end;
               %
               % check if the user wants to save the data
               %
               txt = ['Would you like to save the conditional predictive distributions for the ' AnnualStr ' data at posterior mode to file?'];
               answer = Query(txt,'question',140,'Save - Conditional Predictive Distributions',500,'no',CurrINI);
               drawnow;
               if strcmp(lower(answer),'yes')==1;
                  [PredStartYear,PredStartPeriod,PredEndYear,PredEndPeriod,PredDateVector] = CreatePredictionSample(DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,DSGEModel.DataFrequency,size(MeanPrediction,2));
                  file = FixFilePath([DSGEModel.OutputDirectory '\CondPredictiveDist-' CondTypeStr int2str(DSGEModel.ShockControlMethod) '-' AnnualStr '-' num2str(MaxPH,'%0.0f') '-' DSGEModel.SubEndYear '.' DSGEModel.SubEndPeriod '-PosteriorMode.mat']);
                  YNames = DSGEModel.VariableNames;
                  StateVariableNames = DSGEModel.StateVariableNames;
                  DateVector = MakeDateVector(DSGEModel.SubBeginYear,DSGEModel.SubBeginPeriod,PredEndYear,PredEndPeriod,DSGEModel.DataFrequency);
                  Y = DSGEModel.Y(:,FirstPeriod:min((LastPeriod+MaxPH),size(DSGEModel.Y,2)));
                  if (length(DateVector)>size(Y,2))&(TransData==0);
                     DateVector = DateVector(1,1:size(Y,2));
                  end;
                  if AnnualizeData==1;
                     Y = Annualize(Y,DSGEModel.annual,DSGEModel.annualscale);
                     DateVector = DateVector(length(DateVector)-size(Y,2)+1:length(DateVector));
                  end;
                  %
                  if TransData==1;
                     [YTStatus,YData,ErrorStr,HeaderStr] = CalcTransformData(DSGEModel.VariableNames,DSGEModel.Y',DSGEModel.YTransformation,DSGEModel.YTransMatrix);
                     Y = YData';
                     if size(Y,2)==DSGEModel.T;
                        Y = Y(:,FirstPeriod:min((LastPeriod+MaxPH),size(DSGEModel.Y,2)));
                     else;
                        T2 = size(Y,2);
                        if (FirstPeriod-(DSGEModel.T-T2))>=1;
                           Y = Y(:,FirstPeriod-(DSGEModel.T-T2):min((LastPeriod+MaxPH),size(DSGEModel.Y,2)));
                        else;
                           Y = Y(:,1:min((LastPeriod+MaxPH),size(DSGEModel.Y,2))-(DSGEModel.T-T2));
                           DateVector = DateVector(length(DateVector)-size(Y,2)+1:length(DateVector));
                        end;
                     end;
                  end;
                  KernelX = PredData.KernelX;
                  KernelY = PredData.KernelY;
                  PredMeanKsiStates = PredData.xiStates;
                  PredMean = PredData.PredMean;
                  PredMeanEtaShocks = PredData.etaShocks;
                  %
                  [B0_h,KeepShocks] = RemoveRedundantColumns(PredData.B0);
                  ShockNames = DSGEModel.StateShockNames(KeepShocks,:);
                  %
                  if isempty(PredEventData)==1;
                     save(file,'MeanPrediction','PredMean','PredMeanKsiStates','PredMeanEtaShocks','PredictionErrors','Quantiles','KernelX','KernelY','PredDateVector','Y','DateVector','YNames','StateVariableNames','ShockNames','MeanMultiModesty','MultiPValue','ChiSqPValue','MeanUniModesty','MeanUniModestyLZ');
                  else;
                     save(file,'MeanPrediction','PredMean','PredMeanKsiStates','PredMeanEtaShocks','PredictionErrors','Quantiles','KernelX','KernelY','PredDateVector','Y','DateVector','YNames','StateVariableNames','ShockNames','MeanMultiModesty','MultiPValue','ChiSqPValue','MeanUniModesty','MeanUniModestyLZ','PredictionEvent','PredEventData');
                  end;
                  %
                  % create a matlab m-file to read the output.
                  %
                  mfile = FixFilePath([DSGEModel.OutputDirectory '\CondPD' CTypeStr int2str(DSGEModel.ShockControlMethod) AnnualStr num2str(MaxPH,'%0.0f') DSGEModel.SubEndYear DSGEModel.SubEndPeriod 'PostMode.m']);
                  fid = fopen(mfile,'wt');
                  fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                  fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                  fprintf(fid,'%%\n%% string matrix with the names of the observed variables\n%%\n');
                  fprintf(fid,'YNames = DataStructure.YNames;\n');
                  if AdjustMean==0;
                     fprintf(fid,'%%\n%% matrix with the sample mean of the simulated predictions of the observed variables\n%%\n');
                  else;
                     fprintf(fid,'%%\n%% matrix with the corrected sample mean of the simulated predictions of the observed variables\n%%\n');
                  end;
                  fprintf(fid,'MeanPrediction = DataStructure.MeanPrediction;\n');
                  fprintf(fid,'%%\n%% matrix with the population mean predictions of the observed variables\n%%\n');
                  fprintf(fid,'PredMean = DataStructure.PredMean;\n');
                  fprintf(fid,'%%\n%% string matrix with the names of the state variable variables\n%%\n');
                  fprintf(fid,'StateVariableNames = DataStructure.StateVariableNames;\n');
                  fprintf(fid,'%%\n%% string matrix with the names of the economic shocks\n%%\n');
                  fprintf(fid,'ShockNames = DataStructure.ShockNames;\n');
                  fprintf(fid,'%%\n%% matrix with the population mean predictions of the state variables\n%%\n');
                  fprintf(fid,'PredMeanKsiStates = DataStructure.PredMeanKsiStates;\n');
                  fprintf(fid,'%%\n%% matrix with the population mean of the predictions of the economic shocks\n%%\n');
                  fprintf(fid,'PredMeanEtaShocks = DataStructure.PredMeanEtaShocks;\n');
                  fprintf(fid,'%%\n%% matrix with the errors from the mean predictions of the observed variables\n%%\n');
                  fprintf(fid,'PredictionErrors = DataStructure.PredictionErrors;\n');
                  fprintf(fid,'%%\n%% vector structure with quantiles from the predictive distribution\n%% the structure has %0.0f entries with fields "Quantile" and "percent"\n%% the first field is a matrix and the second a positive number between 0 and 100\n%%\n',length(Quantiles));
                  fprintf(fid,'Quantiles = DataStructure.Quantiles;\n');
                  fprintf(fid,'%%\n%% matrix with x-axis values for kernel density estimates of marginal predictive distribution\n%%\n');
                  fprintf(fid,'KernelX = DataStructure.KernelX;\n');
                  fprintf(fid,'%%\n%% matrix with y-axis values for kernel density estimates of marginal predictive distribution\n%%\n');
                  fprintf(fid,'KernelY = DataStructure.KernelY;\n');
                  fprintf(fid,'%%\n%% vector with dates for the predictions\n%%\n');
                  fprintf(fid,'PredDateVector = DataStructure.PredDateVector;\n');
                  fprintf(fid,'%%\n%% observed variables\n%%\n');
                  fprintf(fid,'Y = DataStructure.Y;\n');
                  fprintf(fid,'%%\n%% vector with dates for the observed variables\n%%\n');
                  fprintf(fid,'DateVector = DataStructure.DateVector;\n');
                  if isempty(PredEventData)==0;
                     fprintf(fid,'%%\n%% matrix with the prediction event definitions. The first column\n%% holds the upper bound, the second column\n%% the lower bound, and third the length of the event.\n%%\n');
                     fprintf(fid,'PredictionEvent = DataStructure.PredictionEvent;\n');
                     fprintf(fid,'%%\n%% matrix with prediction event data. First column holds the number of true\%% events, the second the number of times below the lower bound of the event,\n%% the third the number of times above the upper bound of the event, the\n%% fourth the number of events, the fifth and sixth the sum and sum of\n%% squared deviations from the lower bound when below, and the seventh and\n%% eighth the sum and sum of squared deviations from the upper bound when above\n%%\n');
                     fprintf(fid,'PredEventData = DataStructure.PredEventData;\n');
                     NumEntries = '16';
                     ExtraEntry = 'PredictionEvent (matrix with the upper and lower bound of the event and the length of the event), PredEventData (matrix with the number of true events, the number of events below the lower bound for the event, the number of events above the upper bound for the event, the number of events tested in the columns, the sum and sum of squared deviations from lower bound when below lower bound, the sum and sum of squared deviations from upper bound when above upper bound), ';
                  else;
                     NumEntries = '14';
                     ExtraEntry = '';
                  end;
                  if AnnualizeData==0;
                     fprintf(fid,'%%\n%% vector with the mean of the multivariate modesty statistic and its reference statistic\n%%\n');
                     fprintf(fid,'MeanMultiModesty = DataStructure.MeanMultiModesty;\n');
                     fprintf(fid,'%%\n%% tail probability based on the reference statistic\n%%\n');
                     fprintf(fid,'MultiPValue = DataStructure.MultiPValue;\n');
                     fprintf(fid,'%%\n%% tail probability based on the chi-square distribution\n%%\n');
                     fprintf(fid,'ChiSqPValue = DataStructure.ChiSqPValue;\n');
                     fprintf(fid,'%%\n%% Adolfson, Lasen, Lind and Villani univariate modesty statistic\n%%\n');
                     fprintf(fid,'MeanUniModesty = DataStructure.MeanUniModesty;\n');
                     fprintf(fid,'%%\n%% Leeper and Zha univariate modesty statistic\n%%\n');
                     fprintf(fid,'MeanUniModestyLZ = DataStructure.MeanUniModestyLZ;\n');
                     NumEntries = num2str(str2num(NumEntries)+5,'%0.0f');
                     ExtraEntry = [ExtraEntry 'MeanMultiModesty (1x2 vector with the mean of the multivariate modesty statistic and the mean of the multivariate reference modesty statistic), MultiPValue (scalar with the tail probability of the modesty statistic), ChiSqPValue (scalar with the tail probability of the modesty statistic using the chi-square distribution), MeanUniModesty (vector with the univariate modesty statistic from ALLV), MeanUniModestyLZ (vector with the univariate Leeper-Zha modesty statistic), '];
                  end;
                  fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                  %
                  % end the script file
                  %
                  fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                  fclose(fid);
                  %
                  txt = ['The conditional predictive distributions have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains ' NumEntries ' entries: YNames (string matrix with the names of the observed variables), MeanPrediction (matrix with the mean predictions), PredMean (population mean prediction), PredMeanKsiStates (population mean of the state variables), PredMeanEtaShocks (population mean of the economic shocks), StateVariableNames (string matrix with the names of the state variables), ShockNames (string matrix with the names of the economic shocks), PredictionErrors (matrix with errors for the mean predictions), Quantiles (vector structure with 2 fields: Quantile and percent), KernelX (matrix with x-axis for kernel density estimates of marginal predictive distribution), KernelY (matrix with y-axis values for kernel density estimates of marginal predictive distribution), PredDateVector (vector with dates for the predictions), Y (matrix with the observed variables), ' ExtraEntry 'and DateVector (vector with dates for the observed variables). To access these data you may run the matlab script file "' GetFilename(mfile) '".'];
                  About(txt,'information','Conditional Predictive Distributions - Posterior Mode',500,600,CurrINI);
               end;
               if isempty(PredEventData)==0;
                  %
                  % write prediction event results to file
                  %
                  file = FixFilePath([DSGEModel.OutputDirectory '\CondPredictiveDist-' CondTypeStr int2str(DSGEModel.ShockControlMethod) '-' AnnualStr '-' num2str(MaxPH,'%0.0f') '-' DSGEModel.SubEndYear '.' DSGEModel.SubEndPeriod '-PosteriorMode.txt']);
                  [PredStartYear,PredStartPeriod,PredEndYear,PredEndPeriod] = CreatePredictionSample(DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,DSGEModel.DataFrequency,MaxPH);
                  PrintPredictionEvent(file,DSGEModel.VariableNames,PredEventData,YObsEventData,DSGEModel.PredictionEvent,[PredStartYear ':' PredStartPeriod '-' PredEndYear ':' PredEndPeriod],MaxPH,NumPredPaths,1,CurrINI);
                  %
                  % displays the file if it exists
                  %
                  if FileExist(file)==1;
                     TextGUI(file,['Prediction Events - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 32 1000 CurrINI.scrsz(4)-100],'Posterior Mode Parameter Values',50,CurrINI,1,0);
                  end;
               end;
               %
               % Open a dialog for displaying predictive distributions
               %
               CondPredictiveDistributionDLG('init',DSGEModel,CurrINI,MeanPrediction,Quantiles,PredData.KernelX,PredData.KernelY,'Posterior Mode','DSGE',AnnualStr,1);
            end;
         else;
            About(ErrorStr,'information',HeaderStr,120,500,CurrINI);
         end;
         set(controls.about,'UserData',0);
      end;
   case 'PriorDistribution'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % check if we should check transformations functions
         %
         ErrorStr = '';
         if gcbo==controls.predict.conditional_transformed_posterior;
            DTStatus = VerifyDataTransformation(DSGEModel.VariableNames,DSGEModel.YTransformation,0);
            if DTStatus==1;
               [YTStatus,YData,ErrorStr,HeaderStr] = CalcTransformData(DSGEModel.VariableNames,DSGEModel.Y',DSGEModel.YTransformation,DSGEModel.YTransMatrix);
            else;
               ErrorStr = 'You haven''t specified all the necessary fields for computing transformations of the observed variables.';
               HeaderStr = 'Missing Transformation Information';
            end;
         end;
         %
         if isempty(ErrorStr)==1;
            %
            % are we using annualized data or the original
            %
            if gcbo==controls.predict.stateconditional_original_prior;
               TransData = 0;
               AnnualizeData = 0;
               AnnualStr = 'original';
               QueryHead = 'Conditional Predictive Distributions For Prior - Original Data';
            elseif gcbo==controls.predict.stateconditional_annual_prior;
               TransData = 0;
               AnnualizeData = 1;
               AnnualStr = 'annual';
               QueryHead = 'Conditional Predictive Distributions For Prior - Annualized Data';
            else;
               TransData = 1;
               AnnualizeData = 0;
               AnnualStr = 'transformed';
               QueryHead = 'Conditional Predictive Distributions For Prior - Transformed Data';
            end;
            %
            % check if the posterior mode file exists
            %
            ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
            if FileExist(ModeFile)==1;
               ErrorStr = '';
               try;
                  ModeData = load(ModeFile);
               catch;
                  ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' lasterr];
               end;
               if strcmp(ErrorStr,'')==1;
                  if FileExist(DSGEModel.PriorFile)==1;
                     txt = ['Would you like to use the prior information stored in the posterior mode results file "' GetFilename(ModeFile) '"? If you answer ''No'', YADA will reread the prior file "' GetFilename(DSGEModel.PriorFile) '".'];
                     answer = Query(txt,'question',200,QueryHead,500,'no',CurrINI);
                  else;
                     answer = 'Yes';
                  end;
                  if strcmp(lower(answer),'yes')==1;
                     InitStatus = 1;
                     theta = ModeData.theta;
                     thetaDist = ModeData.thetaDist;
                     PriorDist = ModeData.PriorDist;
                     ParameterNames = ModeData.ParameterNames;
                     LowerBound = ModeData.LowerBound;
                     ModelParameters = ModeData.ModelParameters;
                     thetaPositions = ModeData.thetaPositions;
                  else;
                     [InitStatus,theta,thetaPositions,ModelParameters,thetaDist,PriorDist,ParameterNames,thetaIndex,UniformBounds,LowerBound] = InitializeDSGEModelSimulation(DSGEModel,CurrINI);
                  end;
               else;
                  InitStatus = 0;
                  About(ErrorStr,'information','YADA - Bad MAT File',120,500,CurrINI);
               end;
            else;
               [InitStatus,theta,thetaPositions,ModelParameters,thetaDist,PriorDist,ParameterNames,thetaIndex,UniformBounds,LowerBound] = InitializeDSGEModelSimulation(DSGEModel,CurrINI);
            end;
            if InitStatus==1;
               %
               % load the data
               %
               PriorDrawsFile = FixFilePath([DSGEModel.OutputDirectory '\priordraws\PriorDraws-' DSGEModel.NameOfModel '.mat']);
               PriorData = load(PriorDrawsFile);
               thetaDraws = PriorData.thetaDraws;
               NumPriorDraws = size(thetaDraws,2);
               %
               % initialize the random number generator
               %
               if DSGEModel.RandomNumberValue==1;
                  randn('state',0);
                  rand('state',0);
               else;
                  randn('state',sum(100*clock));
                  rand('state',sum(100*clock));
               end;
               %
               % determine the sample and setup the data
               %
               [FirstPeriod,LastPeriod] = CreateSubSample(DSGEModel);
               %
               % check how many time periods to use for predictive distributions
               %
               [MaxPH,NumPredX] = CalcMaxPredictionHorizon(DSGEModel.MaxForecastHorizon,DSGEModel.DataFrequency,DSGEModel.EndYear,DSGEModel.EndPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,size(DSGEModel.PredictedX,2));
               %
               % matrix with exogenous variables
               %
               X = [];
               if DSGEModel.k>0;
                  if NumPredX<MaxPH;
                     X = DSGEModel.X(:,LastPeriod+1:LastPeriod+(MaxPH-NumPredX));
                  end;
                  if NumPredX>0;
                     X = [X DSGEModel.PredictedX(:,1:NumPredX)];
                  end;
               end;
               %
               % check the length of the conditioning sample
               %
               [status,NumPeriods] = CheckStateCondData(DSGEModel);
               %
               % check if we should adjust the Zeta matrix
               %
               if MaxPH<NumPeriods;
                  NumPeriods = MaxPH;
               end;
               %
               % collect the conditional variables and the selected conditioning sample
               %
               Zeta = DSGEModel.Zeta(DSGEModel.ZetaConditionalVariables,LastPeriod+1:LastPeriod+NumPeriods);
               %
               % check if there are conditioning assumptions for the observed variables
               %
               NumZeta = length(DSGEModel.ZetaConditionalVariables);
               SStr = '';
               if NumZeta>1;
                  SStr = 's';
               end;
               Z = [];
               U = [];
               CondTypeStr = 'State-';
               CTypeStr = 'S';
               if isempty(DSGEModel.Z)==0;
                  %
                  % check if we can use the data
                  %
                  [status,NumZPeriods] = CheckConditioningData(DSGEModel);
                  if (status==1)&(NumZPeriods>=NumPeriods);
                     %
                     % check if the number of conditioning assumptions exceeds the number of
                     % observed variables and shocks.
                     %
                     r = length(DSGEModel.StateVariablePositions);
                     EnoughVariables = ((length([DSGEModel.ConditionalVariables DSGEModel.ZetaConditionalVariables])<=min(DSGEModel.n+r,size(DSGEModel.StateShockNames,1))))&(DSGEModel.ZetaConditionalVariables<=r)&(length(DSGEModel.ConditionalVariables)<=DSGEModel.n);
                     %
                     % check if we have selected enough unique shocks
                     %
                     if DSGEModel.ShockControlMethod==1;
                        UniqueShocks = (length(unique([DSGEModel.ZetaConditionalShocks DSGEModel.ConditionalShocks]))==length([DSGEModel.ZetaConditionalShocks DSGEModel.ConditionalShocks]));
                     elseif DSGEModel.ShockControlMethod==2;
                        UniqueShocks = 1;
                     else;
                        NumShocks = length(unique([DSGEModel.ZetaMixedConditionalShocks DSGEModel.MixedConditionalShocks]));
                        NumAssumptions = length([DSGEModel.ConditionalVariables DSGEModel.ZetaConditionalVariables]);
                        UniqueShocks = (NumShocks>=NumAssumptions)&(NumShocks<=length(DSGEModel.StateShockPositions));
                     end;
                     %
                     % ask if we should include conditioning assumptions for the the observed variables
                     %
                     if (UniqueShocks==1)&(EnoughVariables==1);
                        Ostr = '';
                        ItStr = 'it';
                        if length(DSGEModel.ConditionalVariables)>1;
                           Ostr = 's';
                           ItStr = 'them';
                        end;
                        txt = ['YADA has located ' int2str(length(DSGEModel.ConditionalVariables)) ' conditioning assumption' Ostr ' for the observed variables. Would you like to append ' ItStr ' to your ' int2str(NumZeta) ' assumption' SStr ' for the state variables?'];
                        answer = Query(txt,'question',200,QueryHead,500,'no',CurrINI);
                        if strcmp(lower(answer),'yes')==1;
                           Z = DSGEModel.Z(DSGEModel.ConditionalVariables,LastPeriod+1:LastPeriod+NumPeriods);
                           U = DSGEModel.U(DSGEModel.ConditionalVariables,LastPeriod-NumPeriods+1:LastPeriod);
                           CondTypeStr = 'State-Obs-';
                           CTypeStr = 'SO';
                        end;
                     end;
                  end;
               end;
               %
               % check which method is used for dealing with the conditioning assumptions
               %
               if DSGEModel.ShockControlMethod==1;
                  ShockMethod = 'Direct-';
               elseif DSGEModel.ShockControlMethod==2;
                  ShockMethod = 'WaggonerZha-';
               else;
                  ShockMethod = 'Subset-';
               end;
               %
               % determine how many prediction paths to use
               %
               NumPredPathsStr = get(controls.forecast.predpaths,'String');
               NumPredPaths = str2num(strrep(strrep(strrep(StringTrim(NumPredPathsStr(DSGEModel.NumPredPathsValue,:)),'path',''),'s',''),',',''));
               %
               % check if we have already stored results to disk
               %
               if PredictionsExist(DSGEModel.OutputDirectory,'DSGE',['Cond' CondTypeStr],ShockMethod,DSGEModel.NameOfModel,MaxPH,NumPriorDraws,NumPredPaths,AnnualizeData,TransData,1,0)==1;
                  txt = 'YADA has located results for the conditional predictive distribution on disk. Would you like to use these results?';
                  answer = Query(txt,'question',150,QueryHead,500,'no',CurrINI);
               else;
                  answer = 'No';
               end;
               if strcmp(lower(answer),'yes')==1;
                  [PredEventData,PredictionEvent,ShockMean,ShockNames,StateCov,ShockCov,MeasureCov,ParameterCov] = LoadDSGEPredictionUncertainty(DSGEModel.OutputDirectory,['Cond' CondTypeStr],ShockMethod,DSGEModel.NameOfModel,MaxPH,NumPriorDraws,NumPredPaths,AnnualizeData,TransData,1,0);
                  DoneCalc = 1;
               else;
                  %
                  % ask if we should conduct prediction event calculations
                  %
                  if (isfield(DSGEModel,'PredictionEvent')==0)|(isempty(DSGEModel.PredictionEvent)==1)|(size(DSGEModel.PredictionEvent,1)~=DSGEModel.n)|(size(DSGEModel.PredictionEvent,2)~=3);
                     DSGEModel.PredictionEvent = [Inf*ones(DSGEModel.n,1) zeros(DSGEModel.n,1) ones(DSGEModel.n,1)];
                  end;
                  txt = 'Would you like YADA to calculate prediction event probabilities and perform risk analysis?';
                  answer = Query(txt,'question',140,'Prediction Events - Prior',500,'no',CurrINI);
                  if strcmp(lower(answer),'yes')==1;
                     %
                     % make sure that the prediction event length does not exceed the maximum prediction horizon
                     %
                     for i=1:DSGEModel.n;
                        if DSGEModel.PredictionEvent(i,3)>MaxPH;
                           DSGEModel.PredictionEvent(i,3) = MaxPH;
                        end;
                     end;
                     %
                     % display a dialog for setting up an prediction event
                     %
                     [action,PredictionEvent] = PredictionEventDLG('init',DSGEModel.VariableNames,DSGEModel.PredictionEvent,MaxPH,CurrINI);
                     if strcmp(lower(action),'ok')==1;
                        DSGEModel.PredictionEvent = PredictionEvent;
                        DSGEModel.RunPredictionEvent = 1;
                     else;
                        DSGEModel.RunPredictionEvent = 0;
                     end;
                  else;
                     DSGEModel.RunPredictionEvent = 0;
                  end;
                  %
                  % we can now compute the predictive distribution for the endogenous variables
                  % up to period MaxPH
                  %
                  if DSGEModel.ShockControlMethod==1;
                     [DoneCalc,PredEventData,ShockMean,ShockNames,StateCov,ShockCov,MeasureCov,ParameterCov] = StateCondPredictionPaths(thetaDraws',theta,thetaPositions,ModelParameters,DSGEModel,CurrINI,X,Zeta,Z,U,MaxPH,NumPredPaths,AnnualizeData,TransData,1,0,[],CondTypeStr);
                  elseif DSGEModel.ShockControlMethod==2;
                     [DoneCalc,PredEventData,ShockMean,ShockNames,StateCov,ShockCov,MeasureCov,ParameterCov] = StateCondPredictionPathsWZ(thetaDraws',theta,thetaPositions,ModelParameters,DSGEModel,CurrINI,X,Zeta,Z,U,MaxPH,NumPredPaths,AnnualizeData,TransData,1,0,[],CondTypeStr);
                  else;
                     %
                     % check if the selected method can actually be used
                     %
                     CondTxt = '';
                     if isempty(DSGEModel.ZetaMixedConditionalShocks)==1;
                        CondTxt = 'You haven''t selected any shocks for use which the distribution of a subset of the shocks conditioning method.';
                     elseif length(DSGEModel.ZetaConditionalVariables)+1>=length(DSGEModel.StateShockPositions);
                        CondTxt = 'You have too many conditioning assumptions to be able to use the distribution of a subset of the shocks conditioning method.';
                     elseif length(DSGEModel.ZetaMixedConditionalShocks)<length(DSGEModel.ZetaConditionalVariables);
                        CondTxt = ['The number of shocks you have selected to use to ensure that the conditioning assumptions are satisfied under the distribution of a subset of the shocks conditioning method is not sufficient. You need at least ' int2str(length(DSGEModel.ZetaConditionalVariables)) ' such shocks.'];
                     end;
                     if isempty(CondTxt)==1;
                        [DoneCalc,PredEventData,ShockMean,ShockNames,StateCov,ShockCov,MeasureCov,ParameterCov] = StateCondPredictionPathsMixed(thetaDraws',theta,thetaPositions,ModelParameters,DSGEModel,CurrINI,X,Zeta,Z,U,MaxPH,NumPredPaths,AnnualizeData,TransData,1,0,[],CondTypeStr);
                     else;
                        About(CondTxt,'information','Bad Shocks Selection - Distribution of Subset Shocks Method',300,550,CurrINI);
                        DoneCalc = 0;
                     end;
                  end;
               end;
               %
               % check if we should show any results
               %
               if DoneCalc==1;
                  %
                  % yes, now we need to calculate some distribution properties
                  %
                  [MeanPrediction,CovPrediction,Quantiles,KernelX,KernelY] = DSGEPredictionDistribution(DSGEModel.n,DSGEModel.OutputDirectory,['Cond' CondTypeStr],ShockMethod,DSGEModel.NameOfModel,MaxPH,NumPriorDraws,NumPredPaths,AnnualizeData,TransData,1,0,[],DSGEModel.Percentiles,DSGEModel.VariableNames,DSGEModel.ConfidenceRegionMethod,CurrINI);
                  %
                  % compute the forecast errors based on the mean predictions
                  %
                  PredictionErrors = CalculatePredictionErrors(MeanPrediction,AnnualizeData,TransData,'DSGE',DSGEModel);
                  %
                  % check if we should evaluate modesty test data
                  %
                  if AnnualizeData==0;
                     [MultiPValue,MeanMultiModesty,MeanUniModesty,MeanUniModestyLZ] = DSGEModestyStatistics(DSGEModel.OutputDirectory,ShockMethod,DSGEModel.NameOfModel,MaxPH,NumPriorDraws,NumPredPaths,1,0,DSGEModel.n,CondTypeStr);
                     if exist('YADAchi2cdf')==2;
                        ChiSqPValue = 1-YADAchi2cdf(MeanMultiModesty(1),DSGEModel.n);
                     else;
                        if exist('chi2cdf')==2;
                           ChiSqPValue = 1-chi2cdf(MeanMultiModesty(1),DSGEModel.n);
                        else;
                           ChiSqPValue = 1-YADACheapchi2cdf(MeanMultiModesty(1),DSGEModel.n);
                        end;
                     end;
                     %
                     % print results to file
                     %
                     ResFile = PrintDSGEModestyStatistics(DSGEModel,CurrINI,1,0,MaxPH,NumPeriods,NumPriorDraws,NumPredPaths,MultiPValue,MeanMultiModesty,MeanUniModesty,MeanUniModestyLZ,ShockMean,ShockNames,CondTypeStr);
                     %
                     % ask if the user wishes to view the modesty analysis
                     %
                     txt = ['YADA has written the modesty analysis results to the file "' GetFilename(ResFile) '" in the directory "' GetPath(ResFile) '". Would you like to view the results?'];
                     answer = Query(txt,'question',300,'Conditional Predictive Distributions For Prior - Modesty Statistics',600,'yes',CurrINI);
                     if strcmp(lower(answer),'yes')==1;
                        TextGUI(ResFile,'Modesty Statistics - Prior Distribution',[(CurrINI.scrsz(3)-1000)/2 32 1000 CurrINI.scrsz(4)-100],'Results',40,CurrINI,1,0);
                     end;
                  else;
                     ChiSqPValue = [];
                     MultiPValue = [];
                     MeanMultiModesty = [];
                     MeanUniModesty = [];
                     MeanUniModestyLZ = [];
                  end;
                  %
                  % check if the user wants to save the data
                  %
                  txt = ['Would you like to save the conditional predictive distributions for the ' AnnualStr ' data to file?'];
                  answer = Query(txt,'question',140,'Save - Conditional Predictive Distributions',500,'no',CurrINI);
                  if strcmp(lower(answer),'yes')==1;
                     [PredStartYear,PredStartPeriod,PredEndYear,PredEndPeriod,PredDateVector] = CreatePredictionSample(DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,DSGEModel.DataFrequency,size(MeanPrediction,2));
                     file = FixFilePath([DSGEModel.OutputDirectory '\CondPredictivePriorDist-' CondTypeStr int2str(DSGEModel.ShockControlMethod) '-' AnnualStr '-' num2str(MaxPH,'%0.0f') '-' num2str(NumPriorDraws,'%0.0f') '.' num2str(NumPredPaths,'%0.0f') '-' DSGEModel.SubEndYear '.' DSGEModel.SubEndPeriod '.mat']);
                     YNames = DSGEModel.VariableNames;
                     DateVector = MakeDateVector(DSGEModel.SubBeginYear,DSGEModel.SubBeginPeriod,PredEndYear,PredEndPeriod,DSGEModel.DataFrequency);
                     Y = DSGEModel.Y(:,FirstPeriod:min((LastPeriod+MaxPH),size(DSGEModel.Y,2)));
                     if (length(DateVector)>size(Y,2))&(TransData==0);
                        DateVector = DateVector(1,1:size(Y,2));
                     end;
                     EstimationStartPeriod = DSGEModel.KalmanFirstObservation;
                     EstimationStartPeriodOrig = EstimationStartPeriod;
                     YOrig = Y;
                     DateVectorOrig = DateVector;
                     if AnnualizeData==1;
                        Y = Annualize(Y,DSGEModel.annual,DSGEModel.annualscale);
                        EstimationStartPeriod = max(1,EstimationStartPeriod-(length(DateVector)-size(Y,2)));
                        DateVector = DateVector(length(DateVector)-size(Y,2)+1:length(DateVector));
                     end;
                     %
                     if TransData==1;
                        [YTStatus,YData,ErrorStr,HeaderStr] = CalcTransformData(DSGEModel.VariableNames,DSGEModel.Y',DSGEModel.YTransformation,DSGEModel.YTransMatrix);
                        Y = YData';
                        if size(Y,2)==DSGEModel.T;
                           Y = Y(:,FirstPeriod:min((LastPeriod+MaxPH),size(DSGEModel.Y,2)));
                        else;
                           T2 = size(Y,2);
                           if (FirstPeriod-(DSGEModel.T-T2))>=1;
                              Y = Y(:,FirstPeriod-(DSGEModel.T-T2):min((LastPeriod+MaxPH),size(DSGEModel.Y,2)));
                           else;
                              Y = Y(:,1:min((LastPeriod+MaxPH),size(DSGEModel.Y,2))-(DSGEModel.T-T2));
                              DateVector = DateVector(length(DateVector)-size(Y,2)+1:length(DateVector));
                           end;
                        end;
                     end;
                     %
                     if isempty(PredEventData)==1;
                        save(file,'MeanPrediction','CovPrediction','PredictionErrors','Quantiles','KernelX','KernelY','PredDateVector','Y','DateVector','YNames','StateCov','ShockCov','MeasureCov','ParameterCov','MeanMultiModesty','MultiPValue','ChiSqPValue','MeanUniModesty','MeanUniModestyLZ','EstimationStartPeriod','EstimationStartPeriodOrig','YOrig','DateVectorOrig');
                     else;
                        save(file,'MeanPrediction','CovPrediction','PredictionErrors','Quantiles','KernelX','KernelY','PredDateVector','Y','DateVector','YNames','StateCov','ShockCov','MeasureCov','ParameterCov','MeanMultiModesty','MultiPValue','ChiSqPValue','MeanUniModesty','MeanUniModestyLZ','EstimationStartPeriod','EstimationStartPeriodOrig','YOrig','DateVectorOrig','PredictionEvent','PredEventData');
                     end;
                     %
                     % create a matlab m-file to read the output.
                     %
                     mfile = FixFilePath([DSGEModel.OutputDirectory '\CondPPD' CTypeStr int2str(DSGEModel.ShockControlMethod) AnnualStr num2str(MaxPH,'%0.0f') num2str(NumPriorDraws,'%0.0f') num2str(NumPredPaths,'%0.0f') DSGEModel.SubEndYear DSGEModel.SubEndPeriod '.m']);
                     fid = fopen(mfile,'wt');
                     fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                     fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                     fprintf(fid,'%%\n%% string matrix with the names of the observed variables\n%%\n');
                     fprintf(fid,'YNames = DataStructure.YNames;\n');
                     fprintf(fid,'%%\n%% matrix with the mean predictions of the observed variables\n%%\n');
                     fprintf(fid,'MeanPrediction = DataStructure.MeanPrediction;\n');
                     fprintf(fid,'%%\n%% 3D matrix (nxnxh) with the sample covariance of the predictions of the observed variables\n%%\n');
                     fprintf(fid,'CovPrediction = DataStructure.CovPrediction;\n');
                     fprintf(fid,'%%\n%% matrix with the errors from the mean predictions of the observed variables\n%%\n');
                     fprintf(fid,'PredictionErrors = DataStructure.PredictionErrors;\n');
                     fprintf(fid,'%%\n%% vector structure with quantiles from the predictive distribution\n%% the structure has %0.0f entries with fields "Quantile" and "percent"\n%% the first field is a matrix and the second a positive number between 0 and 100\n%%\n',length(Quantiles));
                     fprintf(fid,'Quantiles = DataStructure.Quantiles;\n');
                     fprintf(fid,'%%\n%% matrix with x-axis values for kernel density estimates of marginal predictive distribution\n%%\n');
                     fprintf(fid,'KernelX = DataStructure.KernelX;\n');
                     fprintf(fid,'%%\n%% matrix with y-axis values for kernel density estimates of marginal predictive distribution\n%%\n');
                     fprintf(fid,'KernelY = DataStructure.KernelY;\n');
                     fprintf(fid,'%%\n%% vector with dates for the predictions\n%%\n');
                     fprintf(fid,'PredDateVector = DataStructure.PredDateVector;\n');
                     fprintf(fid,'%%\n%% observed variables\n%%\n');
                     fprintf(fid,'Y = DataStructure.Y;\n');
                     fprintf(fid,'%%\n%% vector with dates for the observed variables\n%%\n');
                     fprintf(fid,'DateVector = DataStructure.DateVector;\n');
                     if (AnnualizeData==0)&(TransData==0);
                        fprintf(fid,'%%\n%% 3D Error covariance matrix (nxnxh) part due to state uncertainty\n%%\n');
                        fprintf(fid,'StateCov = DataStructure.StateCov;\n');
                        fprintf(fid,'%%\n%% 3D Error covariance matrix (nxnxh) part due to shock uncertainty\n%%\n');
                        fprintf(fid,'ShockCov = DataStructure.ShockCov;\n');
                        fprintf(fid,'%%\n%% 2D Error covariance matrix (nxn) part due to measurement error uncertainty\n%%\n');
                        fprintf(fid,'MeasureCov = DataStructure.MeasureCov;\n');
                        fprintf(fid,'%%\n%% 3D Error covariance matrix (nxnxh) part due to parameter uncertainty\n%%\n');
                        fprintf(fid,'ParameterCov = DataStructure.ParameterCov;\n');
                        CovDecompStr = 'StateCov (3D error covariance matrix with part due to state uncertainty), ShockCov (3D error covariance matrix with part due to shock uncertainty), MeasureCov (2D error covariance matrix with part due to measurement error uncertainty), ParameterCov (3D covariance matrix with part due to parameter uncertainty), ';
                     else;
                        CovDecompStr = '';
                     end;
                     if isempty(PredEventData)==0;
                        fprintf(fid,'%%\n%% matrix with the prediction event definitions. The first column\n%% holds the upper bound, the second column\n%% the lower bound, and third the length of the event.\n%%\n');
                        fprintf(fid,'PredictionEvent = DataStructure.PredictionEvent;\n');
                        fprintf(fid,'%%\n%% matrix with prediction event data. First column holds the number of true\%% events, the second the number of times below the lower bound of the event,\n%% the third the number of times above the upper bound of the event, the\n%% fourth the number of events, the fifth and sixth the sum and sum of\n%% squared deviations from the lower bound when below, and the seventh and\n%% eighth the sum and sum of squared deviations from the upper bound when above\n%%\n');
                        fprintf(fid,'PredEventData = DataStructure.PredEventData;\n');
                        if (AnnualizeData==0)&(TransData==0);
                           NumEntries = '16';
                        else;
                           NumEntries = '12';
                        end;
                        ExtraEntry = 'PredictionEvent (matrix with the upper and lower bound of the event and the length of the event), PredEventData (matrix with the number of true events, the number of events below the lower bound for the event, the number of events above the upper bound for the event, the number of events tested in the columns, the sum and sum of squared deviations from lower bound when below lower bound, the sum and sum of squared deviations from upper bound when above upper bound), ';
                     else;
                        if (AnnualizeData==0)&(TransData==0);
                           NumEntries = '14';
                        else;
                           NumEntries = '10';
                        end;
                        ExtraEntry = '';
                     end;
                     if AnnualizeData==0;
                        fprintf(fid,'%%\n%% vector with the mean of the multivariate modesty statistic and its reference statistic\n%%\n');
                        fprintf(fid,'MeanMultiModesty = DataStructure.MeanMultiModesty;\n');
                        fprintf(fid,'%%\n%% tail probability based on the reference statistic\n%%\n');
                        fprintf(fid,'MultiPValue = DataStructure.MultiPValue;\n');
                        fprintf(fid,'%%\n%% tail probability based on the chi-square distribution\n%%\n');
                        fprintf(fid,'ChiSqPValue = DataStructure.ChiSqPValue;\n');
                        fprintf(fid,'%%\n%% Adolfson, Lasen, Lind and Villani univariate modesty statistic\n%%\n');
                        fprintf(fid,'MeanUniModesty = DataStructure.MeanUniModesty;\n');
                        fprintf(fid,'%%\n%% Leeper and Zha univariate modesty statistic\n%%\n');
                        fprintf(fid,'MeanUniModestyLZ = DataStructure.MeanUniModestyLZ;\n');
                        NumEntries = num2str(str2num(NumEntries)+5,'%0.0f');
                        ExtraEntry = [ExtraEntry 'MeanMultiModesty (1x2 vector with the mean of the multivariate modesty statistic and the mean of the multivariate reference modesty statistic), MultiPValue (scalar with the tail probability of the modesty statistic), ChiSqPValue (scalar with the tail probability of the modesty statistic using the chi-square distribution), MeanUniModesty (vector with the univariate modesty statistic from ALLV), MeanUniModestyLZ (vector with the univariate Leeper-Zha modesty statistic), '];
                     end;
                     fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                     %
                     % end the script file
                     %
                     fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                     fclose(fid);
                     %
                     txt = ['The conditional predictive distributions have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains ' NumEntries ' entries: YNames (string matrix with the names of the observed variables), MeanPrediction (matrix with the mean predictions), CovPrediction (3D matrix with the sample covariances of the predictions), PredictionErrors (matrix with errors for the mean predictions), Quantiles (vector structure with 2 fields: Quantile and percent), KernelX (matrix with x-axis for kernel density estimates of marginal predictive distribution), KernelY (matrix with y-axis values for kernel density estimates of marginal predictive distribution), PredDateVector (vector with dates for the predictions), Y (matrix with the observed variables), ' CovDecompStr ExtraEntry 'and DateVector (vector with dates for the observed variables). To access these data you may run the matlab script file "' GetFilename(mfile) '".'];
                     About(txt,'information','Conditional Predictive Distributions - Prior',500,600,CurrINI);
                  end;
                  if isempty(PredEventData)==0;
                     %
                     % write prediction event results to file
                     %
                     file = FixFilePath([DSGEModel.OutputDirectory '\CondPredictivePriorDist-' CondTypeStr int2str(DSGEModel.ShockControlMethod) '-' AnnualStr '-' num2str(MaxPH,'%0.0f') '-' num2str(NumPriorDraws,'%0.0f') '.' num2str(NumPredPaths,'%0.0f') '-' DSGEModel.SubEndYear '.' DSGEModel.SubEndPeriod '.txt']);
                     [PredStartYear,PredStartPeriod,PredEndYear,PredEndPeriod] = CreatePredictionSample(DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,DSGEModel.DataFrequency,MaxPH);
                     PrintPredictionEvent(file,DSGEModel.VariableNames,PredEventData,[],PredictionEvent,[PredStartYear ':' PredStartPeriod '-' PredEndYear ':' PredEndPeriod],MaxPH,NumPredPaths,NumPriorDraws,CurrINI);
                     %
                     % displays the file if it exists
                     %
                     if FileExist(file)==1;
                        TextGUI(file,['Prediction Events For Prior - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 32 1000 CurrINI.scrsz(4)-100],'Prior Distribution - Conditional Predictions',50,CurrINI,1,0);
                     end;
                  end;
                  %
                  % summarize the information in the uncertainty variables more conveniently
                  %
                  if isempty(StateCov)==0;
                     [StateVar,ShockVar,MeasureVar,ParameterVar,TotalVar] = SummarizeDSGEPredictions(StateCov,ShockCov,MeasureCov,ParameterCov);
                     %
                     % Open a dialog for displaying predictive distributions
                     %
                     UData.StateVar = StateVar;
                     UData.ShockVar = ShockVar;
                     UData.MeasureVar = MeasureVar;
                     UData.ParameterVar = ParameterVar;
                     [UData.KernelX,UData.KernelY] = NormalDensityValues(MeanPrediction,TotalVar,2^8);
                  else;
                     UData = [];
                  end;
                  %
                  % make sure that UData is empty until the forecast error variance decompositions have been solved
                  % under conditional forecasting
                  %
                  CondPredictiveDistributionDLG('init',DSGEModel,CurrINI,MeanPrediction,Quantiles,KernelX,KernelY,'','DSGE',AnnualStr,0,UData);
               end;
            end;
         else;
            About(ErrorStr,'information',HeaderStr,120,500,CurrINI);
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'PosteriorDistribution'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % first attempt to load some posterior mode data
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
         ErrorStr = '';
         HeaderStr = 'YADA - Bad MAT File';
         try;
            ModeData = load(ModeFile);
         catch;
            ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' lasterr];
         end;
         %
         % check if we should check transformations functions
         %
         if (isempty(ErrorStr)==1)&(gcbo==controls.predict.conditional_transformed_posterior);
            DTStatus = VerifyDataTransformation(DSGEModel.VariableNames,DSGEModel.YTransformation,0);
            if DTStatus==1;
               [YTStatus,YData,ErrorStr,HeaderStr] = CalcTransformData(DSGEModel.VariableNames,DSGEModel.Y',DSGEModel.YTransformation,DSGEModel.YTransMatrix);
            else;
               ErrorStr = 'You haven''t specified all the necessary fields for computing transformations of the observed variables.';
               HeaderStr = 'Missing Transformation Information';
            end;
         end;
         %
         if strcmp(ErrorStr,'')==1;
            thetaPositions = ModeData.thetaPositions;
            ModelParameters = ModeData.ModelParameters;
            [FirstPeriod,LastPeriod] = CreateSubSample(DSGEModel);
            %
            % check how many time periods to use for predictive distributions
            %
            [MaxPH,NumPredX] = CalcMaxPredictionHorizon(DSGEModel.MaxForecastHorizon,DSGEModel.DataFrequency,DSGEModel.EndYear,DSGEModel.EndPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,size(DSGEModel.PredictedX,2));
            %
            % matrix with exogenous variables
            %
            X = [];
            if DSGEModel.k>0;
               if NumPredX<MaxPH;
                  X = DSGEModel.X(:,LastPeriod+1:LastPeriod+(MaxPH-NumPredX));
               end;
               if NumPredX>0;
                  X = [X DSGEModel.PredictedX(:,1:NumPredX)];
               end;
            end;
            %
            % check the length of the conditioning sample
            %
            [status,NumPeriods] = CheckStateCondData(DSGEModel);
            %
            % check if we should adjust the Zeta matrix
            %
            if MaxPH<NumPeriods;
               NumPeriods = MaxPH;
            end;
            %
            % collect the conditional variables and the selected conditioning sample
            %
            Zeta = DSGEModel.Zeta(DSGEModel.ZetaConditionalVariables,LastPeriod+1:LastPeriod+NumPeriods);
            %
            % check if there are conditioning assumptions for the observed variables
            %
            NumZeta = length(DSGEModel.ZetaConditionalVariables);
            SStr = '';
            if NumZeta>1;
               SStr = 's';
            end;
            Z = [];
            U = [];
            CondTypeStr = 'State-';
            CTypeStr = 'S';
            if isempty(DSGEModel.Z)==0;
               %
               % check if we can use the data
               %
               [status,NumZPeriods] = CheckConditioningData(DSGEModel);
               if (status==1)&(NumZPeriods>=NumPeriods);
                  %
                  % check if the number of conditioning assumptions exceeds the number of
                  % observed variables and shocks.
                  %
                  r = length(DSGEModel.StateVariablePositions);
                  EnoughVariables = ((length([DSGEModel.ConditionalVariables DSGEModel.ZetaConditionalVariables])<=min(DSGEModel.n+r,size(DSGEModel.StateShockNames,1))))&(DSGEModel.ZetaConditionalVariables<=r)&(length(DSGEModel.ConditionalVariables)<=DSGEModel.n);
                  %
                  % check if we have selected enough unique shocks
                  %
                  if DSGEModel.ShockControlMethod==1;
                     UniqueShocks = (length(unique([DSGEModel.ZetaConditionalShocks DSGEModel.ConditionalShocks]))==length([DSGEModel.ZetaConditionalShocks DSGEModel.ConditionalShocks]));
                  elseif DSGEModel.ShockControlMethod==2;
                     UniqueShocks = 1;
                  else;
                     NumShocks = length(unique([DSGEModel.ZetaMixedConditionalShocks DSGEModel.MixedConditionalShocks]));
                     NumAssumptions = length([DSGEModel.ConditionalVariables DSGEModel.ZetaConditionalVariables]);
                     UniqueShocks = (NumShocks>=NumAssumptions)&(NumShocks<=length(DSGEModel.StateShockPositions));
                  end;
                  %
                  % ask if we should include conditioning assumptions for the the observed variables
                  %
                  if (UniqueShocks==1)&(EnoughVariables==1);
                     Ostr = '';
                     ItStr = 'it';
                     if length(DSGEModel.ConditionalVariables)>1;
                        Ostr = 's';
                        ItStr = 'them';
                     end;
                     txt = ['YADA has located ' int2str(length(DSGEModel.ConditionalVariables)) ' conditioning assumption' Ostr ' for the observed variables. Would you like to append ' ItStr ' to your ' int2str(NumZeta) ' assumption' SStr ' for the state variables?'];
                     answer = Query(txt,'question',200,'Conditional Predictive Distributions',500,'no',CurrINI);
                     if strcmp(lower(answer),'yes')==1;
                        Z = DSGEModel.Z(DSGEModel.ConditionalVariables,LastPeriod+1:LastPeriod+NumPeriods);
                        U = DSGEModel.U(DSGEModel.ConditionalVariables,LastPeriod-NumPeriods+1:LastPeriod);
                        CondTypeStr = 'State-Obs-';
                        CTypeStr = 'SO';
                     end;
                  end;
               end;
            end;
            %
            % if there are multiple chains, ask which one to load
            %
            ChainsStr = get(controls.posterior.chains,'String');
            NumChains = str2num(StringTrim(ChainsStr(DSGEModel.ParallelChainsValue,:)));
            %
            if NumChains>1;
               %
               % ask which chain to load
               %
               ChainsStr = '';
               for i=1:NumChains;
                  ChainsStr = strvcat(ChainsStr,['Posterior chain number ' num2str(i,'%0.0f')]);
               end;
               [action,CurrChain] = SelectionDlg(ChainsStr,1,'Select the posterior draws chain to load','MCMC Chain Selection','Conditional Predictive Distributions','','',CurrINI);
            else;
               CurrChain = 1;
               action = 'ok';
            end;
            %
            if strcmp(lower(action),'cancel')==0;
               %
               % check if the data actually exists
               %
               if VerifyDSGEPosteriorDraws(DSGEModel,CurrChain,controls)==1;
                  %
                  % load the data
                  %
                  [PostSample,thetaPostSample,LogPost,AcceptedDraws,NumBurnin,Weights] = LoadDSGEPosteriorDraws(DSGEModel,CurrINI,CurrChain);
                  NumDraws = length(LogPost);
                  clear('PostSample','LogPost','AcceptedDraws');
                  drawnow;
                  %
                  % initialize the random number generator
                  %
                  if DSGEModel.RandomNumberValue==1;
                     randn('state',0);
                     rand('state',0);
                  else;
                     randn('state',sum(100*clock));
                     rand('state',sum(100*clock));
                  end;
                  %
                  % select which posterior draws to use
                  %
                  thetaPostSample = thetaPostSample(NumBurnin+1:NumDraws,:);
                  NumDraws = NumDraws-NumBurnin;
                  %
                  % check how many draws we should pick among these
                  %
                  if DSGEModel.PosteriorSampler<=7;
                     PostDrawsUsageStr = get(controls.posterior.postdrawsusage,'String');
                     NumPostDrawsUsage = min(str2num(strrep(StringTrim(PostDrawsUsageStr(DSGEModel.PostDrawsUsageValue,:)),',','')),NumDraws);
                     %
                     % get the draws to use
                     %
                     if DSGEModel.PosteriorSampler<=7;
                        if DSGEModel.RandomizeDraws==0;
                           DrawFreq = round(NumDraws/NumPostDrawsUsage);
                           SelectDraws = (1:DrawFreq:NumDraws);
                           if length(SelectDraws)+1==NumPostDrawsUsage;
                              SelectDraws = [SelectDraws NumDraws];
                           elseif length(SelectDraws)>NumPostDrawsUsage;
                              SelectDraws = SelectDraws(1:NumPostDrawsUsage);
                           end;
                           thetaPostSample = thetaPostSample(SelectDraws,:);
                        else;
                           thetaPostSample = thetaPostSample(ceil(rand(NumPostDrawsUsage,1)*NumDraws),:);
                        end;
                     end;
                  end;
                  %
                  % are we using annualized data or the original
                  %
                  if gcbo==controls.predict.stateconditional_original_posterior;
                     TransData = 0;
                     AnnualizeData = 0;
                     AnnualStr = 'original';
                     QueryHead = 'Conditional Predictive Distributions - Original Data';
                  elseif gcbo==controls.predict.stateconditional_annual_posterior;
                     TransData = 0;
                     AnnualizeData = 1;
                     AnnualStr = 'annual';
                     QueryHead = 'Conditional Predictive Distributions - Annualized Data';
                  else;
                     TransData = 1;
                     AnnualizeData = 0;
                     AnnualStr = 'transformed';
                     QueryHead = 'Conditional Predictive Distributions - Transformed Data';
                  end;
                  %
                  % check which method is used for dealing with the conditioning assumptions
                  %
                  if DSGEModel.ShockControlMethod==1;
                     ShockMethod = 'Direct-';
                  elseif DSGEModel.ShockControlMethod==2;
                     ShockMethod = 'WaggonerZha-';
                  else;
                     ShockMethod = 'Subset-';
                  end;
                  %
                  % determine how many prediction paths to use
                  %
                  NumPredPathsStr = get(controls.forecast.predpaths,'String');
                  NumPredPaths = str2num(strrep(strrep(strrep(StringTrim(NumPredPathsStr(DSGEModel.NumPredPathsValue,:)),'path',''),'s',''),',',''));
                  %
                  % check if we have already stored results to disk
                  %
                  if PredictionsExist(DSGEModel.OutputDirectory,'DSGE',['Cond' CondTypeStr],ShockMethod,DSGEModel.NameOfModel,MaxPH,NumPostDrawsUsage,NumPredPaths,AnnualizeData,TransData,CurrChain,1)==1;
                     txt = 'YADA has located results for the conditional predictive distribution on disk. Would you like to use these results?';
                     answer = Query(txt,'question',150,QueryHead,450,'no',CurrINI);
                  else;
                     answer = 'No';
                  end;
                  if strcmp(lower(answer),'yes')==1;
                     [PredEventData,PredictionEvent,ShockMean,ShockNames,StateCov,ShockCov,MeasureCov,ParameterCov] = LoadDSGEPredictionUncertainty(DSGEModel.OutputDirectory,['Cond' CondTypeStr],ShockMethod,DSGEModel.NameOfModel,MaxPH,NumPostDrawsUsage,NumPredPaths,AnnualizeData,TransData,CurrChain,1);
                     DoneCalc = 1;
                  else;
                     %
                     % ask if we should conduct prediction event calculations
                     %
                     if (isfield(DSGEModel,'PredictionEvent')==0)|(isempty(DSGEModel.PredictionEvent)==1)|(size(DSGEModel.PredictionEvent,1)~=DSGEModel.n)|(size(DSGEModel.PredictionEvent,2)~=3);
                        DSGEModel.PredictionEvent = [Inf*ones(DSGEModel.n,1) zeros(DSGEModel.n,1) ones(DSGEModel.n,1)];
                     end;
                     txt = 'Would you like YADA to calculate prediction event probabilities and perform risk analysis?';
                     answer = Query(txt,'question',140,'Prediction Events',500,'no',CurrINI);
                     if strcmp(lower(answer),'yes')==1;
                        %
                        % make sure that the prediction event length does not exceed the maximum prediction horizon
                        %
                        for i=1:DSGEModel.n;
                           if DSGEModel.PredictionEvent(i,3)>MaxPH;
                              DSGEModel.PredictionEvent(i,3) = MaxPH;
                           end;
                        end;
                        %
                        % display a dialog for setting up an prediction event
                        %
                        [action,PredictionEvent] = PredictionEventDLG('init',DSGEModel.VariableNames,DSGEModel.PredictionEvent,MaxPH,CurrINI);
                        if strcmp(lower(action),'ok')==1;
                           DSGEModel.PredictionEvent = PredictionEvent;
                           DSGEModel.RunPredictionEvent = 1;
                        else;
                           DSGEModel.RunPredictionEvent = 0;
                        end;
                     else;
                        DSGEModel.RunPredictionEvent = 0;
                     end;
                     %
                     % we can now compute the predictive distribution for the endogenous variables
                     % up to period MaxPH
                     %
                     if DSGEModel.ShockControlMethod==1;
                        [DoneCalc,PredEventData,ShockMean,ShockNames,StateCov,ShockCov,MeasureCov,ParameterCov] = StateCondPredictionPaths(thetaPostSample,ModeData.thetaMode,thetaPositions,ModelParameters,DSGEModel,CurrINI,X,Zeta,Z,U,MaxPH,NumPredPaths,AnnualizeData,TransData,CurrChain,1,Weights,CondTypeStr);
                     elseif DSGEModel.ShockControlMethod==2;
                        [DoneCalc,PredEventData,ShockMean,ShockNames,StateCov,ShockCov,MeasureCov,ParameterCov] = StateCondPredictionPathsWZ(thetaPostSample,ModeData.thetaMode,thetaPositions,ModelParameters,DSGEModel,CurrINI,X,Zeta,Z,U,MaxPH,NumPredPaths,AnnualizeData,TransData,CurrChain,1,Weights,CondTypeStr);
                     else;
                        %
                        % check if the selected method can actually be used
                        %
                        CondTxt = '';
                        if isempty(DSGEModel.ZetaMixedConditionalShocks)==1;
                           CondTxt = 'You haven''t selected any shocks for use which the distribution of a subset of the shocks conditioning method.';
                        elseif length(DSGEModel.ZetaConditionalVariables)+1>=length(DSGEModel.StateShockPositions);
                           CondTxt = 'You have too many conditioning assumptions to be able to use the distribution of a subset of the shocks conditioning method.';
                        elseif length(DSGEModel.ZetaMixedConditionalShocks)<length(DSGEModel.ZetaConditionalVariables);
                           CondTxt = ['The number of shocks you have selected to use to ensure that the conditioning assumptions are satisfied under the distribution of a subset of the shocks conditioning method is not sufficient. You need at least ' int2str(length(DSGEModel.ZetaConditionalVariables)) ' such shocks.'];
                        end;
                        if isempty(CondTxt)==1;
                           [DoneCalc,PredEventData,ShockMean,ShockNames,StateCov,ShockCov,MeasureCov,ParameterCov] = StateCondPredictionPathsMixed(thetaPostSample,ModeData.thetaMode,thetaPositions,ModelParameters,DSGEModel,CurrINI,X,Zeta,Z,U,MaxPH,NumPredPaths,AnnualizeData,TransData,CurrChain,1,Weights,CondTypeStr);
                        else;
                           About(CondTxt,'information','Bad Shocks Selection - Distribution of Subset Shocks Method',300,550,CurrINI);
                           DoneCalc = 0;
                        end;
                     end;
                  end;
                  %
                  % clear everything
                  %
                  clear('thetaPostSample');
                  %
                  % check if we should show any results
                  %
                  if DoneCalc==1;
                     %
                     % yes, now we need to calculate some distribution properties
                     %
                     [MeanPrediction,CovPrediction,Quantiles,KernelX,KernelY] = DSGEPredictionDistribution(DSGEModel.n,DSGEModel.OutputDirectory,['Cond' CondTypeStr],ShockMethod,DSGEModel.NameOfModel,MaxPH,NumPostDrawsUsage,NumPredPaths,AnnualizeData,TransData,CurrChain,1,Weights,DSGEModel.Percentiles,DSGEModel.VariableNames,DSGEModel.ConfidenceRegionMethod,CurrINI);
                     %
                     % compute the forecast errors based on the mean predictions
                     %
                     PredictionErrors = CalculatePredictionErrors(MeanPrediction,AnnualizeData,TransData,'DSGE',DSGEModel);
                     %
                     % check if we should evaluate modesty test data
                     %
                     if AnnualizeData==0;
                        [MultiPValue,MeanMultiModesty,MeanUniModesty,MeanUniModestyLZ] = DSGEModestyStatistics(DSGEModel.OutputDirectory,ShockMethod,DSGEModel.NameOfModel,MaxPH,NumPostDrawsUsage,NumPredPaths,CurrChain,1,DSGEModel.n,CondTypeStr);
                        if exist('YADAchi2cdf')==2;
                           ChiSqPValue = 1-YADAchi2cdf(MeanMultiModesty(1),DSGEModel.n);
                        else;
                           if exist('chi2cdf')==2;
                              ChiSqPValue = 1-chi2cdf(MeanMultiModesty(1),DSGEModel.n);
                           else;
                              ChiSqPValue = 1-YADACheapchi2cdf(MeanMultiModesty(1),DSGEModel.n);
                           end;
                        end;
                        %
                        % print results to file
                        %
                        ResFile = PrintDSGEModestyStatistics(DSGEModel,CurrINI,CurrChain,1,MaxPH,NumPeriods,NumPostDrawsUsage,NumPredPaths,MultiPValue,MeanMultiModesty,MeanUniModesty,MeanUniModestyLZ,ShockMean,ShockNames,CondTypeStr);
                        %
                        % ask if the user wishes to view the modesty analysis
                        %
                        txt = ['YADA has written the modesty analysis results to the file "' GetFilename(ResFile) '" in the directory "' GetPath(ResFile) '". Would you like to view the results?'];
                        answer = Query(txt,'question',300,'Conditional Predictive Distributions - Modesty Statistics',600,'yes',CurrINI);
                        if strcmp(lower(answer),'yes')==1;
                           TextGUI(ResFile,['Modesty Statistics - Chain ' int2str(CurrChain)],[(CurrINI.scrsz(3)-1000)/2 32 1000 CurrINI.scrsz(4)-100],'Results',40,CurrINI,1,0);
                        end;
                     else;
                        ChiSqPValue = [];
                        MultiPValue = [];
                        MeanMultiModesty = [];
                        MeanUniModesty = [];
                        MeanUniModestyLZ = [];
                     end;
                     %
                     % check if the user wants to save the data
                     %
                     txt = ['Would you like to save the conditional predictive distributions for the ' AnnualStr ' data to file?'];
                     answer = Query(txt,'question',140,'Save - Conditional Predictive Distributions',500,'no',CurrINI);
                     if strcmp(lower(answer),'yes')==1;
                        [PredStartYear,PredStartPeriod,PredEndYear,PredEndPeriod,PredDateVector] = CreatePredictionSample(DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,DSGEModel.DataFrequency,size(MeanPrediction,2));
                        file = FixFilePath([DSGEModel.OutputDirectory '\CondPredictiveDist-' CondTypeStr int2str(DSGEModel.ShockControlMethod) '-' AnnualStr '-' num2str(CurrChain,'%0.0f') '-' num2str(MaxPH,'%0.0f') '-' num2str(NumPostDrawsUsage,'%0.0f') '.' num2str(NumPredPaths,'%0.0f') '-' DSGEModel.SubEndYear '.' DSGEModel.SubEndPeriod '.mat']);
                        YNames = DSGEModel.VariableNames;
                        DateVector = MakeDateVector(DSGEModel.SubBeginYear,DSGEModel.SubBeginPeriod,PredEndYear,PredEndPeriod,DSGEModel.DataFrequency);
                        Y = DSGEModel.Y(:,FirstPeriod:min((LastPeriod+MaxPH),size(DSGEModel.Y,2)));
                        if (length(DateVector)>size(Y,2))&(TransData==0);
                           DateVector = DateVector(1,1:size(Y,2));
                        end;
                        EstimationStartPeriod = DSGEModel.KalmanFirstObservation;
                        EstimationStartPeriodOrig = EstimationStartPeriod;
                        YOrig = Y;
                        DateVectorOrig = DateVector;
                        if AnnualizeData==1;
                           Y = Annualize(Y,DSGEModel.annual,DSGEModel.annualscale);
                           EstimationStartPeriod = max(1,EstimationStartPeriod-(length(DateVector)-size(Y,2)));
                           DateVector = DateVector(length(DateVector)-size(Y,2)+1:length(DateVector));
                        end;
                        %
                        if TransData==1;
                           [YTStatus,YData,ErrorStr,HeaderStr] = CalcTransformData(DSGEModel.VariableNames,DSGEModel.Y',DSGEModel.YTransformation,DSGEModel.YTransMatrix);
                           Y = YData';
                           if size(Y,2)==DSGEModel.T;
                              Y = Y(:,FirstPeriod:min((LastPeriod+MaxPH),size(DSGEModel.Y,2)));
                           else;
                              T2 = size(Y,2);
                              if (FirstPeriod-(DSGEModel.T-T2))>=1;
                                 Y = Y(:,FirstPeriod-(DSGEModel.T-T2):min((LastPeriod+MaxPH),size(DSGEModel.Y,2)));
                              else;
                                 Y = Y(:,1:min((LastPeriod+MaxPH),size(DSGEModel.Y,2))-(DSGEModel.T-T2));
                                 DateVector = DateVector(length(DateVector)-size(Y,2)+1:length(DateVector));
                              end;
                           end;
                        end;
                        %
                        if isempty(PredEventData)==1;
                           save(file,'MeanPrediction','CovPrediction','PredictionErrors','Quantiles','KernelX','KernelY','PredDateVector','Y','DateVector','YNames','StateCov','ShockCov','MeasureCov','ParameterCov','MeanMultiModesty','MultiPValue','ChiSqPValue','MeanUniModesty','MeanUniModestyLZ','EstimationStartPeriod','EstimationStartPeriodOrig','YOrig','DateVectorOrig');
                        else;
                           save(file,'MeanPrediction','CovPrediction','PredictionErrors','Quantiles','KernelX','KernelY','PredDateVector','Y','DateVector','YNames','StateCov','ShockCov','MeasureCov','ParameterCov','MeanMultiModesty','MultiPValue','ChiSqPValue','MeanUniModesty','MeanUniModestyLZ','EstimationStartPeriod','EstimationStartPeriodOrig','YOrig','DateVectorOrig','PredictionEvent','PredEventData');
                        end;
                        %
                        % create a matlab m-file to read the output.
                        %
                        mfile = FixFilePath([DSGEModel.OutputDirectory '\CondPD' CTypeStr int2str(DSGEModel.ShockControlMethod) AnnualStr num2str(CurrChain,'%0.0f') num2str(MaxPH,'%0.0f') num2str(NumPostDrawsUsage,'%0.0f') num2str(NumPredPaths,'%0.0f') DSGEModel.SubEndYear DSGEModel.SubEndPeriod '.m']);
                        fid = fopen(mfile,'wt');
                        fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                        fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                        fprintf(fid,'%%\n%% string matrix with the names of the observed variables\n%%\n');
                        fprintf(fid,'YNames = DataStructure.YNames;\n');
                        fprintf(fid,'%%\n%% matrix with the mean predictions of the observed variables\n%%\n');
                        fprintf(fid,'MeanPrediction = DataStructure.MeanPrediction;\n');
                        fprintf(fid,'%%\n%% 3D matrix (nxnxh) with the sample covariance of the predictions of the observed variables\n%%\n');
                        fprintf(fid,'CovPrediction = DataStructure.CovPrediction;\n');
                        fprintf(fid,'%%\n%% matrix with the errors from the mean predictions of the observed variables\n%%\n');
                        fprintf(fid,'PredictionErrors = DataStructure.PredictionErrors;\n');
                        fprintf(fid,'%%\n%% vector structure with quantiles from the predictive distribution\n%% the structure has %0.0f entries with fields "Quantile" and "percent"\n%% the first field is a matrix and the second a positive number between 0 and 100\n%%\n',length(Quantiles));
                        fprintf(fid,'Quantiles = DataStructure.Quantiles;\n');
                        fprintf(fid,'%%\n%% matrix with x-axis values for kernel density estimates of marginal predictive distribution\n%%\n');
                        fprintf(fid,'KernelX = DataStructure.KernelX;\n');
                        fprintf(fid,'%%\n%% matrix with y-axis values for kernel density estimates of marginal predictive distribution\n%%\n');
                        fprintf(fid,'KernelY = DataStructure.KernelY;\n');
                        fprintf(fid,'%%\n%% vector with dates for the predictions\n%%\n');
                        fprintf(fid,'PredDateVector = DataStructure.PredDateVector;\n');
                        fprintf(fid,'%%\n%% observed variables\n%%\n');
                        fprintf(fid,'Y = DataStructure.Y;\n');
                        fprintf(fid,'%%\n%% vector with dates for the observed variables\n%%\n');
                        fprintf(fid,'DateVector = DataStructure.DateVector;\n');
                        if (AnnualizeData==0)&(TransData==0);
                           fprintf(fid,'%%\n%% 3D Error covariance matrix (nxnxh) part due to state uncertainty\n%%\n');
                           fprintf(fid,'StateCov = DataStructure.StateCov;\n');
                           fprintf(fid,'%%\n%% 3D Error covariance matrix (nxnxh) part due to shock uncertainty\n%%\n');
                           fprintf(fid,'ShockCov = DataStructure.ShockCov;\n');
                           fprintf(fid,'%%\n%% 2D Error covariance matrix (nxn) part due to measurement error uncertainty\n%%\n');
                           fprintf(fid,'MeasureCov = DataStructure.MeasureCov;\n');
                           fprintf(fid,'%%\n%% 3D Error covariance matrix (nxnxh) part due to parameter uncertainty\n%%\n');
                           fprintf(fid,'ParameterCov = DataStructure.ParameterCov;\n');
                           CovDecompStr = 'StateCov (3D error covariance matrix with part due to state uncertainty), ShockCov (3D error covariance matrix with part due to shock uncertainty), MeasureCov (2D error covariance matrix with part due to measurement error uncertainty), ParameterCov (3D covariance matrix with part due to parameter uncertainty), ';
                        else;
                           CovDecompStr = '';
                        end;
                        if isempty(PredEventData)==0;
                           fprintf(fid,'%%\n%% matrix with the prediction event definitions. The first column\n%% holds the upper bound, the second column\n%% the lower bound, and third the length of the event.\n%%\n');
                           fprintf(fid,'PredictionEvent = DataStructure.PredictionEvent;\n');
                           fprintf(fid,'%%\n%% matrix with prediction event data. First column holds the number of true\%% events, the second the number of times below the lower bound of the event,\n%% the third the number of times above the upper bound of the event, the\n%% fourth the number of events, the fifth and sixth the sum and sum of\n%% squared deviations from the lower bound when below, and the seventh and\n%% eighth the sum and sum of squared deviations from the upper bound when above\n%%\n');
                           fprintf(fid,'PredEventData = DataStructure.PredEventData;\n');
                           if (AnnualizeData==0)&(TransData==0);
                              NumEntries = '16';
                           else;
                              NumEntries = '12';
                           end;
                           ExtraEntry = 'PredictionEvent (matrix with the upper and lower bound of the event and the length of the event), PredEventData (matrix with the number of true events, the number of events below the lower bound for the event, the number of events above the upper bound for the event, the number of events tested in the columns, the sum and sum of squared deviations from lower bound when below lower bound, the sum and sum of squared deviations from upper bound when above upper bound), ';
                        else;
                           if (AnnualizeData==0)&(TransData==0);
                              NumEntries = '14';
                           else;
                              NumEntries = '10';
                           end;
                           ExtraEntry = '';
                        end;
                        if AnnualizeData==0;
                           fprintf(fid,'%%\n%% vector with the mean of the multivariate modesty statistic and its reference statistic\n%%\n');
                           fprintf(fid,'MeanMultiModesty = DataStructure.MeanMultiModesty;\n');
                           fprintf(fid,'%%\n%% tail probability based on the reference statistic\n%%\n');
                           fprintf(fid,'MultiPValue = DataStructure.MultiPValue;\n');
                           fprintf(fid,'%%\n%% tail probability based on the chi-square distribution\n%%\n');
                           fprintf(fid,'ChiSqPValue = DataStructure.ChiSqPValue;\n');
                           fprintf(fid,'%%\n%% Adolfson, Lasen, Lind and Villani univariate modesty statistic\n%%\n');
                           fprintf(fid,'MeanUniModesty = DataStructure.MeanUniModesty;\n');
                           fprintf(fid,'%%\n%% Leeper and Zha univariate modesty statistic\n%%\n');
                           fprintf(fid,'MeanUniModestyLZ = DataStructure.MeanUniModestyLZ;\n');
                           NumEntries = num2str(str2num(NumEntries)+5,'%0.0f');
                           ExtraEntry = [ExtraEntry 'MeanMultiModesty (1x2 vector with the mean of the multivariate modesty statistic and the mean of the multivariate reference modesty statistic), MultiPValue (scalar with the tail probability of the modesty statistic), ChiSqPValue (scalar with the tail probability of the modesty statistic using the chi-square distribution), MeanUniModesty (vector with the univariate modesty statistic from ALLV), MeanUniModestyLZ (vector with the univariate Leeper-Zha modesty statistic), '];
                        end;
                        fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                        %
                        % end the script file
                        %
                        fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                        fclose(fid);
                        %
                        txt = ['The conditional predictive distributions have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains ' NumEntries ' entries: YNames (string matrix with the names of the observed variables), MeanPrediction (matrix with the mean predictions), CovPrediction (3D matrix with the sample covariances of the predictions), PredictionErrors (matrix with errors for the mean predictions), Quantiles (vector structure with 2 fields: Quantile and percent), KernelX (matrix with x-axis for kernel density estimates of marginal predictive distribution), KernelY (matrix with y-axis values for kernel density estimates of marginal predictive distribution), PredDateVector (vector with dates for the predictions), Y (matrix with the observed variables), ' CovDecompStr ExtraEntry 'and DateVector (vector with dates for the observed variables). To access these data you may run the matlab script file "' GetFilename(mfile) '".'];
                        About(txt,'information','Conditional Predictive Distributions',500,600,CurrINI);
                     end;
                     if isempty(PredEventData)==0;
                        %
                        % write prediction event results to file
                        %
                        file = FixFilePath([DSGEModel.OutputDirectory '\CondPredictiveDist-' CondTypeStr int2str(DSGEModel.ShockControlMethod) '-' AnnualStr '-' num2str(CurrChain,'%0.0f') '-' num2str(MaxPH,'%0.0f') '-' num2str(NumPostDrawsUsage,'%0.0f') '.' num2str(NumPredPaths,'%0.0f') '-' DSGEModel.SubEndYear '.' DSGEModel.SubEndPeriod '.txt']);
                        [PredStartYear,PredStartPeriod,PredEndYear,PredEndPeriod] = CreatePredictionSample(DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,DSGEModel.DataFrequency,MaxPH);
                        PrintPredictionEvent(file,DSGEModel.VariableNames,PredEventData,[],PredictionEvent,[PredStartYear ':' PredStartPeriod '-' PredEndYear ':' PredEndPeriod],MaxPH,NumPredPaths,NumPostDrawsUsage,CurrINI);
                        %
                        % displays the file if it exists
                        %
                        if FileExist(file)==1;
                           TextGUI(file,['Prediction Events - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 32 1000 CurrINI.scrsz(4)-100],'Posterior Distribution - Conditional Predictions',50,CurrINI,1,0);
                        end;
                     end;
                     %
                     % summarize the information in the uncertainty variables more conveniently
                     %
                     if isempty(StateCov)==0;
                        [StateVar,ShockVar,MeasureVar,ParameterVar,TotalVar] = SummarizeDSGEPredictions(StateCov,ShockCov,MeasureCov,ParameterCov);
                        %
                        % Open a dialog for displaying predictive distributions
                        %
                        UData.StateVar = StateVar;
                        UData.ShockVar = ShockVar;
                        UData.MeasureVar = MeasureVar;
                        UData.ParameterVar = ParameterVar;
                        [UData.KernelX,UData.KernelY] = NormalDensityValues(MeanPrediction,TotalVar,2^8);
                     else;
                        UData = [];
                     end;
                     %
                     % make sure that UData is empty until the forecast error variance decompositions have been solved
                     % under conditional forecasting
                     %
                     CondPredictiveDistributionDLG('init',DSGEModel,CurrINI,MeanPrediction,Quantiles,KernelX,KernelY,'','DSGE',AnnualStr,1,UData);
                  end;
               else;
                  txt = ['Some of the files for loading the posterior data for chain number ' num2str(CurrChain,'%0.0f') ' are not available.'];
                  About(txt,'error','Missing Posterior Draws',150,500,CurrINI);
               end;
            end;
         else;
            About(ErrorStr,'information',HeaderStr,120,500,CurrINI);
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'PredictionEvent'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % first attempt to load some posterior mode data
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
         ErrorStr = '';
         HeaderStr = 'YADA - Bad MAT File';
         try;
            ModeData = load(ModeFile);
         catch;
            ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' lasterr];
         end;
         %
         % check if we should check transformations functions
         %
         if (isempty(ErrorStr)==1)&(gcbo==controls.predict.conditional_transformed_predevent);
            DTStatus = VerifyDataTransformation(DSGEModel.VariableNames,DSGEModel.YTransformation,0);
            if DTStatus==1;
               [YTStatus,YData,ErrorStr,HeaderStr] = CalcTransformData(DSGEModel.VariableNames,DSGEModel.Y',DSGEModel.YTransformation,DSGEModel.YTransMatrix);
            else;
               ErrorStr = 'You haven''t specified all the necessary fields for computing transformations of the observed variables.';
               HeaderStr = 'Missing Transformation Information';
            end;
         end;
         %
         if strcmp(ErrorStr,'')==1;
            thetaPositions = ModeData.thetaPositions;
            ModelParameters = ModeData.ModelParameters;
            [FirstPeriod,LastPeriod] = CreateSubSample(DSGEModel);
            %
            % check how many time periods to use for predictive distributions
            %
            [MaxPH,NumPredX] = CalcMaxPredictionHorizon(DSGEModel.MaxForecastHorizon,DSGEModel.DataFrequency,DSGEModel.EndYear,DSGEModel.EndPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,size(DSGEModel.PredictedX,2));
            %
            % matrix with exogenous variables
            %
            X = [];
            if DSGEModel.k>0;
               if NumPredX<MaxPH;
                  X = DSGEModel.X(:,LastPeriod+1:LastPeriod+(MaxPH-NumPredX));
               end;
               if NumPredX>0;
                  X = [X DSGEModel.PredictedX(:,1:NumPredX)];
               end;
            end;
            %
            % check the length of the conditioning sample
            %
            [status,NumPeriods] = CheckStateCondData(DSGEModel);
            %
            % check if we should adjust the Zeta matrix
            %
            if MaxPH<NumPeriods;
               NumPeriods = MaxPH;
            end;
            %
            % collect the conditional variables and the selected conditioning sample
            %
            Zeta = DSGEModel.Zeta(DSGEModel.ZetaConditionalVariables,LastPeriod+1:LastPeriod+NumPeriods);
            %
            % check if there are conditioning assumptions for the observed variables
            %
            NumZeta = length(DSGEModel.ZetaConditionalVariables);
            SStr = '';
            if NumZeta>1;
               SStr = 's';
            end;
            Z = [];
            U = [];
            CondTypeStr = 'State-';
            CTypeStr = 'S';
            if isempty(DSGEModel.Z)==0;
               %
               % check if we can use the data
               %
               [status,NumZPeriods] = CheckConditioningData(DSGEModel);
               if (status==1)&(NumZPeriods>=NumPeriods);
                  %
                  % check if the number of conditioning assumptions exceeds the number of
                  % observed variables and shocks.
                  %
                  r = length(DSGEModel.StateVariablePositions);
                  EnoughVariables = ((length([DSGEModel.ConditionalVariables DSGEModel.ZetaConditionalVariables])<=min(DSGEModel.n+r,size(DSGEModel.StateShockNames,1))))&(DSGEModel.ZetaConditionalVariables<=r)&(length(DSGEModel.ConditionalVariables)<=DSGEModel.n);
                  %
                  % check if we have selected enough unique shocks
                  %
                  if DSGEModel.ShockControlMethod==1;
                     UniqueShocks = (length(unique([DSGEModel.ZetaConditionalShocks DSGEModel.ConditionalShocks]))==length([DSGEModel.ZetaConditionalShocks DSGEModel.ConditionalShocks]));
                  elseif DSGEModel.ShockControlMethod==2;
                     UniqueShocks = 1;
                  else;
                     NumShocks = length(unique([DSGEModel.ZetaMixedConditionalShocks DSGEModel.MixedConditionalShocks]));
                     NumAssumptions = length([DSGEModel.ConditionalVariables DSGEModel.ZetaConditionalVariables]);
                     UniqueShocks = (NumShocks>=NumAssumptions)&(NumShocks<=length(DSGEModel.StateShockPositions));
                  end;
                  %
                  % ask if we should include conditioning assumptions for the the observed variables
                  %
                  if (UniqueShocks==1)&(EnoughVariables==1);
                     Ostr = '';
                     ItStr = 'it';
                     if length(DSGEModel.ConditionalVariables)>1;
                        Ostr = 's';
                        ItStr = 'them';
                     end;
                     txt = ['YADA has located ' int2str(length(DSGEModel.ConditionalVariables)) ' conditioning assumption' Ostr ' for the observed variables. Would you like to append ' ItStr ' to your ' int2str(NumZeta) ' assumption' SStr ' for the state variables?'];
                     answer = Query(txt,'question',200,'Conditional Predictive Distributions - Prediction Event',500,'no',CurrINI);
                     if strcmp(lower(answer),'yes')==1;
                        Z = DSGEModel.Z(DSGEModel.ConditionalVariables,LastPeriod+1:LastPeriod+NumPeriods);
                        U = DSGEModel.U(DSGEModel.ConditionalVariables,LastPeriod-NumPeriods+1:LastPeriod);
                        CondTypeStr = 'State-Obs-';
                        CTypeStr = 'SO';
                     end;
                  end;
               end;
            end;
            %
            % if there are multiple chains, ask which one to load
            %
            ChainsStr = get(controls.posterior.chains,'String');
            NumChains = str2num(StringTrim(ChainsStr(DSGEModel.ParallelChainsValue,:)));
            %
            if NumChains>1;
               %
               % ask which chain to load
               %
               ChainsStr = '';
               for i=1:NumChains;
                  ChainsStr = strvcat(ChainsStr,['Posterior chain number ' num2str(i,'%0.0f')]);
               end;
               [action,CurrChain] = SelectionDlg(ChainsStr,1,'Select the posterior draws chain to load','MCMC Chain Selection','Prediction Event','','',CurrINI);
            else;
               CurrChain = 1;
               action = 'ok';
            end;
            %
            %
            if strcmp(lower(action),'cancel')==0;
               %
               % check if the data actually exists
               %
               if VerifyDSGEPosteriorDraws(DSGEModel,CurrChain,controls)==1;
                  %
                  % load the data
                  %
                  [PostSample,thetaPostSample,LogPost,AcceptedDraws,NumBurnin,Weights] = LoadDSGEPosteriorDraws(DSGEModel,CurrINI,CurrChain);
                  NumDraws = length(LogPost);
                  clear('PostSample','LogPost','AcceptedDraws');
                  drawnow;
                  %
                  % initialize the random number generator
                  %
                  if DSGEModel.RandomNumberValue==1;
                     randn('state',0);
                     rand('state',0);
                  else;
                     randn('state',sum(100*clock));
                     rand('state',sum(100*clock));
                  end;
                  %
                  % select which posterior draws to use
                  %
                  thetaPostSample = thetaPostSample(NumBurnin+1:NumDraws,:);
                  NumDraws = NumDraws-NumBurnin;
                  %
                  % check how many draws we should pick among these
                  %
                  if DSGEModel.PosteriorSampler<=7;
                     PostDrawsUsageStr = get(controls.posterior.postdrawsusage,'String');
                     NumPostDrawsUsage = min(str2num(strrep(StringTrim(PostDrawsUsageStr(DSGEModel.PostDrawsUsageValue,:)),',','')),NumDraws);
                     %
                     % get the draws to use
                     %
                     if DSGEModel.RandomizeDraws==0;
                        DrawFreq = round(NumDraws/NumPostDrawsUsage);
                        SelectDraws = (1:DrawFreq:NumDraws);
                        if length(SelectDraws)+1==NumPostDrawsUsage;
                           SelectDraws = [SelectDraws NumDraws];
                        elseif length(SelectDraws)>NumPostDrawsUsage;
                           SelectDraws = SelectDraws(1:NumPostDrawsUsage);
                        end;
                        thetaPostSample = thetaPostSample(SelectDraws,:);
                     else;
                        thetaPostSample = thetaPostSample(ceil(rand(NumPostDrawsUsage,1)*NumDraws),:);
                     end;
                  end;
                  %
                  % are we using annualized data or the original
                  %
                  if gcbo==controls.predict.stateconditional_original_predevent;
                     TransData = 0;
                     AnnualizeData = 0;
                     AnnualStr = 'original';
                     QueryHead = 'Conditional Predictive Distributions - Original Data';
                  elseif gcbo==controls.predict.stateconditional_annual_predevent;
                     TransData = 0;
                     AnnualizeData = 1;
                     AnnualStr = 'annual';
                     QueryHead = 'Conditional Predictive Distributions - Annualized Data';
                  else;
                     TransData = 1;
                     AnnualizeData = 0;
                     AnnualStr = 'transformed';
                     QueryHead = 'Conditional Predictive Distributions - Transformed Data';
                  end;
                  %
                  % prediction event calculations
                  %
                  if (isfield(DSGEModel,'PredictionEvent')==0)|(isempty(DSGEModel.PredictionEvent)==1)|(size(DSGEModel.PredictionEvent,1)~=DSGEModel.n)|(size(DSGEModel.PredictionEvent,2)~=3);
                     DSGEModel.PredictionEvent = [Inf*ones(DSGEModel.n,1) zeros(DSGEModel.n,1) ones(DSGEModel.n,1)];
                  end;
                  %
                  % make sure that the prediction event length does not exceed the maximum prediction horizon
                  %
                  for i=1:DSGEModel.n;
                     if DSGEModel.PredictionEvent(i,3)>MaxPH;
                        DSGEModel.PredictionEvent(i,3) = MaxPH;
                     end;
                  end;
                  %
                  % display a dialog for setting up an prediction event
                  %
                  [action,PredictionEvent] = PredictionEventDLG('init',DSGEModel.VariableNames,DSGEModel.PredictionEvent,MaxPH,CurrINI);
                  %
                  % check if user cancelled or not
                  %
                  if strcmp(lower(action),'ok')==1;
                     DSGEModel.PredictionEvent = PredictionEvent;
                     DSGEModel.RunPredictionEvent = 1;
                     %
                     % check which method is used for dealing with the conditioning assumptions
                     %
                     if DSGEModel.ShockControlMethod==1;
                        ShockMethod = 'Direct-';
                     elseif DSGEModel.ShockControlMethod==2;
                        ShockMethod = 'WaggonerZha-';
                     else;
                        ShockMethod = 'Subset-';
                     end;
                     %
                     % determine how many prediction paths to use
                     %
                     NumPredPathsStr = get(controls.forecast.predpaths,'String');
                     NumPredPaths = str2num(strrep(strrep(strrep(StringTrim(NumPredPathsStr(DSGEModel.NumPredPathsValue,:)),'path',''),'s',''),',',''));
                     %
                     % check if we have already stored results to disk
                     %
                     if PredictionsExist(DSGEModel.OutputDirectory,'DSGE',['Cond' CondTypeStr],ShockMethod,DSGEModel.NameOfModel,MaxPH,NumPostDrawsUsage,NumPredPaths,AnnualizeData,TransData,CurrChain,1)==1;
                        txt = 'YADA has located results for the conditional predictive distribution on disk. Would you like to use these results?';
                        answer = Query(txt,'question',150,QueryHead,450,'no',CurrINI);
                     else;
                        answer = 'No';
                     end;
                     if strcmp(lower(answer),'yes')==1;
                        %
                        % compute the prediction event results
                        %
                        PredEventData = LoadDSGEPredictionPaths(DSGEModel.PredictionEvent,DSGEModel.OutputDirectory,['Cond' CondTypeStr],ShockMethod,DSGEModel.NameOfModel,MaxPH,NumPostDrawsUsage,NumPredPaths,AnnualizeData,TransData,CurrChain,CurrINI,Weights);
                        DoneCalc = 1;
                     else;
                        %
                        % we can now compute the predictive distribution for the endogenous variables
                        % up to period MaxPH
                        %
                        if DSGEModel.ShockControlMethod==1;
                           [DoneCalc,PredEventData,ShockMean,ShockNames,StateCov,ShockCov,MeasureCov,ParameterCov] = StateCondPredictionPaths(thetaPostSample,ModeData.thetaMode,thetaPositions,ModelParameters,DSGEModel,CurrINI,X,Zeta,Z,U,MaxPH,NumPredPaths,AnnualizeData,TransData,CurrChain,1,Weights,CondTypeStr);
                        elseif DSGEModel.ShockControlMethod==2;
                           [DoneCalc,PredEventData,ShockMean,ShockNames,StateCov,ShockCov,MeasureCov,ParameterCov] = StateCondPredictionPathsWZ(thetaPostSample,ModeData.thetaMode,thetaPositions,ModelParameters,DSGEModel,CurrINI,X,Zeta,Z,U,MaxPH,NumPredPaths,AnnualizeData,TransData,CurrChain,1,Weights,CondTypeStr);
                        else;
                           %
                           % check if the selected method can actually be used
                           %
                           CondTxt = '';
                           if isempty(DSGEModel.ZetaMixedConditionalShocks)==1;
                              CondTxt = 'You haven''t selected any shocks for use which the distribution of a subset of the shocks conditioning method.';
                           elseif length(DSGEModel.ZetaConditionalVariables)+1>=length(DSGEModel.StateShockPositions);
                              CondTxt = 'You have too many conditioning assumptions to be able to use the distribution of a subset of the shocks conditioning method.';
                           elseif length(DSGEModel.ZetaMixedConditionalShocks)<length(DSGEModel.ZetaConditionalVariables);
                              CondTxt = ['The number of shocks you have selected to use to ensure that the conditioning assumptions are satisfied under the distribution of a subset of the shocks conditioning method is not sufficient. You need at least ' int2str(length(DSGEModel.ZetaConditionalVariables)) ' such shocks.'];
                           end;
                           if isempty(CondTxt)==1;
                              [DoneCalc,PredEventData,ShockMean,ShockNames,StateCov,ShockCov,MeasureCov,ParameterCov] = StateCondPredictionPathsMixed(thetaPostSample,ModeData.thetaMode,thetaPositions,ModelParameters,DSGEModel,CurrINI,X,Zeta,Z,U,MaxPH,NumPredPaths,AnnualizeData,TransData,CurrChain,1,Weights,CondTypeStr);
                           else;
                              About(CondTxt,'information','Bad Shocks Selection - Distribution of Subset Shocks Method',300,550,CurrINI);
                              DoneCalc = 0;
                           end;
                        end;
                     end;
                     %
                     % clear everything
                     %
                     clear('thetaPostSample');
                     %
                     % check if we should show any results
                     %
                     if DoneCalc==1;
                        %
                        % check if summary results already exist on disk
                        %
                        [PredStartYear,PredStartPeriod,PredEndYear,PredEndPeriod,PredDateVector] = CreatePredictionSample(DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,DSGEModel.DataFrequency,MaxPH);
                        file = FixFilePath([DSGEModel.OutputDirectory '\CondPredictiveDist-' CondTypeStr int2str(DSGEModel.ShockControlMethod) '-' AnnualStr '-' num2str(CurrChain,'%0.0f') '-' num2str(MaxPH,'%0.0f') '-' num2str(NumPostDrawsUsage,'%0.0f') '.' num2str(NumPredPaths,'%0.0f') '-' DSGEModel.SubEndYear '.' DSGEModel.SubEndPeriod '.mat']);
                        if FileExist(file)==1;
                           save(file,'PredictionEvent','PredEventData','-APPEND');
                           %
                           % create a matlab m-file to read the output.
                           %
                           mfile = FixFilePath([DSGEModel.OutputDirectory '\CondPD' CTypeStr int2str(DSGEModel.ShockControlMethod) AnnualStr num2str(CurrChain,'%0.0f') num2str(MaxPH,'%0.0f') num2str(NumPostDrawsUsage,'%0.0f') num2str(NumPredPaths,'%0.0f') DSGEModel.SubEndYear DSGEModel.SubEndPeriod '.m']);
                           fid = fopen(mfile,'wt');
                           fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                           fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                           fprintf(fid,'%%\n%% string matrix with the names of the observed variables\n%%\n');
                           fprintf(fid,'YNames = DataStructure.YNames;\n');
                           fprintf(fid,'%%\n%% matrix with the mean predictions of the observed variables\n%%\n');
                           fprintf(fid,'MeanPrediction = DataStructure.MeanPrediction;\n');
                           fprintf(fid,'%%\n%% matrix with the errors from the mean predictions of the observed variables\n%%\n');
                           fprintf(fid,'PredictionErrors = DataStructure.PredictionErrors;\n');
                           fprintf(fid,'%%\n%% vector structure with quantiles from the predictive distribution\n%% the structure has %0.0f entries with fields "Quantile" and "percent"\n%% the first field is a matrix and the second a positive number between 0 and 100\n%%\n',length(DSGEModel.Percentiles));
                           fprintf(fid,'Quantiles = DataStructure.Quantiles;\n');
                           fprintf(fid,'%%\n%% vector with dates for the predictions\n%%\n');
                           fprintf(fid,'PredDateVector = DataStructure.PredDateVector;\n');
                           fprintf(fid,'%%\n%% observed variables\n%%\n');
                           fprintf(fid,'Y = DataStructure.Y;\n');
                           fprintf(fid,'%%\n%% vector with dates for the observed variables\n%%\n');
                           fprintf(fid,'DateVector = DataStructure.DateVector;\n');
                           if (AnnualizeData==0)&(TransData==0);
                              fprintf(fid,'%%\n%% 3D Error covariance matrix (nxnxh) part due to state uncertainty\n%%\n');
                              fprintf(fid,'StateCov = DataStructure.StateCov;\n');
                              fprintf(fid,'%%\n%% 3D Error covariance matrix (nxnxh) part due to shock uncertainty\n%%\n');
                              fprintf(fid,'ShockCov = DataStructure.ShockCov;\n');
                              fprintf(fid,'%%\n%% 2D Error covariance matrix (nxn) part due to measurement error uncertainty\n%%\n');
                              fprintf(fid,'MeasureCov = DataStructure.MeasureCov;\n');
                              fprintf(fid,'%%\n%% 3D Error covariance matrix (nxnxh) part due to parameter uncertainty\n%%\n');
                              fprintf(fid,'ParameterCov = DataStructure.ParameterCov;\n');
                           end;
                           fprintf(fid,'%%\n%% matrix with the prediction event definitions. The first column\n%% holds the upper bound, the second column\n%% the lower bound, and third the length of the event.\n%%\n');
                           fprintf(fid,'PredictionEvent = DataStructure.PredictionEvent;\n');
                           fprintf(fid,'%%\n%% matrix with prediction event data. First column holds the number of true\%% events, the second the number of times below the lower bound of the event,\n%% the third the number of times above the upper bound of the event, the\n%% fourth the number of events, the fifth and sixth the sum and sum of\n%% squared deviations from the lower bound when below, and the seventh and\n%% eighth the sum and sum of squared deviations from the upper bound when above\n%%\n');
                           fprintf(fid,'PredEventData = DataStructure.PredEventData;\n');
                           if AnnualizeData==0;
                              fprintf(fid,'%%\n%% vector with the mean of the multivariate modesty statistic and its reference statistic\n%%\n');
                              fprintf(fid,'MeanMultiModesty = DataStructure.MeanMultiModesty;\n');
                              fprintf(fid,'%%\n%% tail probability based on the reference statistic\n%%\n');
                              fprintf(fid,'MultiPValue = DataStructure.MultiPValue;\n');
                              fprintf(fid,'%%\n%% tail probability based on the chi-square distribution\n%%\n');
                              fprintf(fid,'ChiSqPValue = DataStructure.ChiSqPValue;\n');
                              fprintf(fid,'%%\n%% Adolfson, Lasen, Lind and Villani univariate modesty statistic\n%%\n');
                              fprintf(fid,'MeanUniModesty = DataStructure.MeanUniModesty;\n');
                              fprintf(fid,'%%\n%% Leeper and Zha univariate modesty statistic\n%%\n');
                              fprintf(fid,'MeanUniModestyLZ = DataStructure.MeanUniModestyLZ;\n');
                           end;
                           fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                           %
                           % end the script file
                           %
                           fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                           fclose(fid);
                        end;
                        %
                        % write prediction event results to file
                        %
                        file = FixFilePath([DSGEModel.OutputDirectory '\CondPredictiveDist-' CondTypeStr int2str(DSGEModel.ShockControlMethod) '-' AnnualStr '-' num2str(CurrChain,'%0.0f') '-' num2str(MaxPH,'%0.0f') '-' num2str(NumPostDrawsUsage,'%0.0f') '.' num2str(NumPredPaths,'%0.0f') '-' DSGEModel.SubEndYear '.' DSGEModel.SubEndPeriod '.txt']);
                        [PredStartYear,PredStartPeriod,PredEndYear,PredEndPeriod] = CreatePredictionSample(DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,DSGEModel.DataFrequency,MaxPH);
                        PrintPredictionEvent(file,DSGEModel.VariableNames,PredEventData,[],PredictionEvent,[PredStartYear ':' PredStartPeriod '-' PredEndYear ':' PredEndPeriod],MaxPH,NumPredPaths,NumPostDrawsUsage,CurrINI);
                        %
                        % displays the file if it exists
                        %
                        if FileExist(file)==1;
                           TextGUI(file,['Prediction Events - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 32 1000 CurrINI.scrsz(4)-100],'Posterior Distribution - Conditional Predictions',50,CurrINI,1,0);
                        end;
                     end;
                  else;
                     %
                     % make sure that the posterior draws are released from memory
                     %
                     clear('thetaPostSample');
                  end;
               else;
                  txt = ['Some of the files for loading the posterior data for chain number ' num2str(CurrChain,'%0.0f') ' are not available.'];
                  About(txt,'error','Missing Posterior Draws',150,500,CurrINI);
               end;
            end;
         else;
            About(ErrorStr,'information',HeaderStr,120,500,CurrINI);
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'ViewStateConditionalVars'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % create a string matrix with all the dates
         %
         if strcmp(lower(DSGEModel.DataFrequency),'q')==1;
            SampleFreq = 4;
         elseif strcmp(lower(DSGEModel.DataFrequency),'m')==1;
            SampleFreq = 12;
         else;
            SampleFreq = 1;
         end;
         %
         % create a full sample string matrix
         %
         CondSampleStrMat = '';
         BeginPeriod = str2num(DSGEModel.BeginPeriod);
         DateVector = [];
         for j=BeginPeriod:SampleFreq;
            CondSampleStrMat = strvcat(CondSampleStrMat,[DSGEModel.BeginYear ':' int2str(j)]);
            if SampleFreq>1;
               DateVector = [DateVector str2num(DSGEModel.BeginYear)+((1+(2*(j-1)))/(2*SampleFreq))];
            else;
               DateVector = str2num(DSGEModel.BeginYear);
            end;
         end;
         DoLoop = 1;
         NumPeriods = 0;
         CurrYear = str2num(DSGEModel.BeginYear)+1;
         while DoLoop==1;
            for j=1:SampleFreq;
               CondSampleStrMat = strvcat(CondSampleStrMat,[int2str(CurrYear) ':' int2str(j)]);
               if SampleFreq>1;
                  DateVector = [DateVector CurrYear+((1+(2*(j-1)))/(2*SampleFreq))];
               else;
                  DateVector = [DateVector CurrYear];
               end;
            end;
            CurrYear = CurrYear+1;
            if size(CondSampleStrMat,1)>=size(DSGEModel.Zeta,2);
               DoLoop = 0;
            end;
         end;
         CondSampleStrMat = CondSampleStrMat(1:size(DSGEModel.Zeta,2),:);
         DateVector = DateVector(1:size(DSGEModel.Zeta,2));
         %
         FirstPeriod = CondSampleStrMat(1,:);
         LastPeriod = CondSampleStrMat(length(DateVector),:);
         %
         % we can now view the data
         %
         GraphEditDataDLG('init','State Conditioning Variables',DSGEModel.ZetaVariableNames,DSGEModel.Zeta',[],[],0,DateVector,CondSampleStrMat,[],[],DSGEModel.DataFrequency,CurrINI);
         %
         set(controls.about,'UserData',0);
      end;
   case 'SetStateConditionalVars'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % open the dialog for selecting variables
         %
         [action,positions] = SelectCondVarShockDLG('init','State Variables','Conditioning',DSGEModel.ZetaVariableNames,DSGEModel.ZetaConditionalVariables,CurrINI);
         %
         % check is the action taken was OK or Cancel
         %
         %
         % check is the action taken was OK or Cancel
         %
         if (strcmp(action,'OK')==1)&(isequal(DSGEModel.ZetaConditionalVariables,positions)==0);
            %
            % check if positions is empty
            %
            if isempty(positions)==0;
               DSGEModel.ZetaConditionalVariables = positions;
               Addtxt = '';
               if (isempty(DSGEModel.ZetaMixedConditionalShocks)==0)&(length(DSGEModel.ZetaMixedConditionalShocks)<length(positions));
                  AddMCS = setdiff((1:length(DSGEModel.StateShockPositions)),DSGEModel.ZetaMixedConditionalShocks);
                  DSGEModel.ZetaMixedConditionalShocks = [DSGEModel.ZetaMixedConditionalShocks AddMCS];
                  DSGEModel.ZetaMixedConditionalShocks = DSGEModel.ZetaMixedConditionalShocks(1:length(positions));
                  if isfield(controls,'setstateconditionalshocksmixed')==1;
                     Addtxt = 'Please notice that the increase in the number of conditioning assumptions used requires that the selection of shocks for using the distribution of a subset of the shocks conditioning method has also been altered.';
                  end;
               end;
               if (length(DSGEModel.ZetaConditionalVariables)>length(DSGEModel.ZetaConditionalShocks))&(length(DSGEModel.ZetaConditionalVariables)<=length(DSGEModel.StateShockPositions));
                  %
                  % make sure that the number of shocks for conditioning is increased
                  %
                  OldNumShocks = length(DSGEModel.ZetaConditionalShocks);
                  AddConditionalShocks = setdiff((1:length(DSGEModel.ZetaConditionalVariables)),DSGEModel.ZetaConditionalShocks);
                  DSGEModel.ZetaConditionalShocks = sort([DSGEModel.ZetaConditionalShocks AddConditionalShocks(1:length(DSGEModel.ZetaConditionalVariables)-length(DSGEModel.ZetaConditionalShocks))]);
                  txt = ['The number of shocks for conditioning has been increased from ' int2str(OldNumShocks) ' to ' int2str(length(DSGEModel.ZetaConditionalShocks)) '. Please make sure that the shocks you want to use are the correct ones.'];
                  if isempty(Addtxt)==0;
                     txt = strvcat(txt,' ',Addtxt);
                  end;
                  About(txt,'information','Conditional Shock Changes',400,550,CurrINI);
               elseif length(DSGEModel.ZetaConditionalVariables)<length(DSGEModel.ZetaConditionalShocks);
                  %
                  % make sure that the number of shocks for conditioning is decreased
                  %
                  OldNumShocks = length(DSGEModel.ZetaConditionalShocks);
                  DSGEModel.ZetaConditionalShocks = DSGEModel.ZetaConditionalShocks(1:length(DSGEModel.ZetaConditionalVariables));
                  %
                  % inform the user about the change
                  %
                  txt = ['The number of shocks for conditioning has been decreased from ' int2str(OldNumShocks) ' to ' int2str(length(DSGEModel.ZetaConditionalShocks)) '. Please make sure that the shocks you want to use are the correct ones.'];
                  if isempty(Addtxt)==0;
                     txt = strvcat(txt,' ',Addtxt);
                  end;
                  About(txt,'information','Conditional Shock Changes',400,550,CurrINI);
               elseif length(DSGEModel.ZetaConditionalVariables)>length(DSGEModel.StateShockPositions);
                  Addtxt = ['The number of conditioning variables you have selected (' int2str(length(DSGEModel.ZetaConditionalVariables)) ') is greater than the number of shocks in the model (' int2str(length(DSGEModel.StateShockPositions)) '). This is not a valid choice.'];
                  DSGEModel.ZetaConditionalVariables = OrigDSGEModel.ZetaConditionalVariables;
                  About(Addtxt,'information','Select State Conditioning Variables Error',300,550,CurrINI);
               else;
                  if isempty(Addtxt)==0;
                     About(Addtxt,'information','Conditional State Variable Changes',300,550,CurrINI);
                  end;
               end;
            else;
               txt = 'You didn''t select any conditioning state variables. This is not a valid choice in YADA. You must at least select one conditioning variable.';
               About(txt,'information','Bad Conditional State Variables Selection',300,550,CurrINI);
            end;
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'SetStateConditionalShocks'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % open the dialog for selecting shocks
         %
         [action,positions] = SelectCondVarShockDLG('init','Shocks','Conditioning',DSGEModel.StateShockNames,DSGEModel.ZetaConditionalShocks,CurrINI);
         %
         % check is the action taken was OK or Cancel
         %
         if strcmp(action,'OK')==1;
            %
            % check if positions is empty
            %
            if isempty(positions)==0;
               %
               % check to make sure that the number of selected shocks doesn't exceed the number of possible
               % conditioning variables
               %
               if length(positions)<=size(DSGEModel.Zeta,1);
                  if (isempty(DSGEModel.ZetaConditionalShocks)==1)|(isequal(positions,DSGEModel.ZetaConditionalShocks)==0)
                     DSGEModel.ZetaConditionalShocks = positions;
                     Addtxt = '';
                     if (isempty(DSGEModel.ZetaMixedConditionalShocks)==0)&(length(DSGEModel.ZetaMixedConditionalShocks)<length(positions));
                        AddMCS = setdiff((1:length(DSGEModel.StateShockPositions)),DSGEModel.ZetaMixedConditionalShocks);
                        DSGEModel.ZetaMixedConditionalShocks = [DSGEModel.ZetaMixedConditionalShocks AddMCS];
                        DSGEModel.ZetaMixedConditionalShocks = DSGEModel.ZetaMixedConditionalShocks(1:length(positions));
                        if isfield(controls,'setstateconditionalshocksmixed')==1;
                           Addtxt = 'Please notice that the increase in the number of shocks used for the direct values conditional forecasting method requires that the selection of shocks for using the distribution of a subset of the shocks conditioning method has also been altered.';
                        end;
                     end;
                     if length(DSGEModel.ZetaConditionalShocks)>length(DSGEModel.ZetaConditionalVariables);
                        %
                        % we need to increase the number of conditioning variables
                        %
                        OldNumVars = length(DSGEModel.ZetaConditionalVariables);
                        AddConditionalVariables = setdiff((1:length(DSGEModel.ZetaConditionalShocks)),DSGEModel.ZetaConditionalVariables);
                        DSGEModel.ZetaConditionalVariables = sort([DSGEModel.ZetaConditionalVariables AddConditionalVariables(1:length(DSGEModel.ZetaConditionalShocks)-length(DSGEModel.ZetaConditionalVariables))]);
                        txt = ['The number of state variables for conditioning has been increased from ' num2str(OldNumVars,'%0.0f') ' to ' num2str(length(DSGEModel.ZetaConditionalVariables),'%0.0f') '. Please make sure that the state variables you want to use are the correct ones.'];
                        if isempty(Addtxt)==0;
                           txt = strvcat(txt,' ',Addtxt);
                        end;
                        About(txt,'information','Conditional State Variable Changes',400,550,CurrINI);
                     elseif length(DSGEModel.ZetaConditionalShocks)<length(DSGEModel.ZetaConditionalVariables);
                        %
                        % we need to decrease the number of conditioning variables
                        %
                        OldNumVars = length(DSGEModel.ZetaConditionalVariables);
                        DSGEModel.ZetaConditionalVariables = DSGEModel.ZetaConditionalVariables(1:length(DSGEModel.ZetaConditionalShocks));
                        txt = ['The number of state variables for conditioning has been decreased from ' num2str(OldNumVars,'%0.0f') ' to ' num2str(length(DSGEModel.ZetaConditionalVariables),'%0.0f') '. Please make sure that the state variables you want to use are the correct ones.'];
                        if isempty(Addtxt)==0;
                           txt = strvcat(txt,' ',Addtxt);
                        end;
                        About(txt,'information','Conditional State Variable Changes',400,550,CurrINI);
                     else;
                        if isempty(Addtxt)==0;
                           About(Addtxt,'information','Conditional Shock Changes',300,550,CurrINI);
                        end;
                     end;
                  end;
               else;
                  txt = ['The number of selected conditioning shocks exceeds the number of possible conditioning state variables. This is not a valid choice in YADA. You can at most select ' num2str(size(DSGEModel.Zeta,1),'%0.0f') ' conditioning shocks.'];
                  About(txt,'information','Bad Conditional Shocks Selection',120,550,CurrINI);
               end;
            else;
               txt = 'You didn''t select any conditioning shocks. This is not a valid choice in YADA. You must at least select one conditioning shock.';
               About(txt,'information','Bad Conditional Shocks Selection',120,550,CurrINI);
            end;
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'SetMixedStateConditionalShocks'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % open the dialog for selecting shocks
         %
         [action,positions] = SelectCondVarShockDLG('init','Shocks','Subset Method Conditioning',DSGEModel.StateShockNames,DSGEModel.ZetaMixedConditionalShocks,CurrINI);
         %
         % check is the action taken was OK or Cancel
         %
         if strcmp(action,'OK')==1;
            %
            % check if positions is empty
            %
            if isempty(positions)==0;
               if length(positions)>=length(DSGEModel.ZetaConditionalShocks);
                  %
                  % we're duin OK
                  %
                  DSGEModel.ZetaMixedConditionalShocks = positions;
               else;
                  txt = ['The number of shocks for conditional forecasting with the distribution for a subset of the shocks method (' int2str(length(positions)) ') that you selected is less than the number of conditioning assumptions (' int2str(length(DSGEModel.ZetaConditionalShocks)) '). This is not a valid choice in YADA. You must select at least as many shocks in the subset as there are conditioning assumptions.'];
                  About(txt,'information','Bad Shocks Selection - Distribution of Subset Shocks Method',300,550,CurrINI);
               end;
            else;
               txt = 'You didn''t select any shocks for conditional forecasting with the distribution for a subset of the shocks method. This is not a valid choice in YADA. You must select at least one more shock in the subset than there are conditioning assumptions and fewer than the total number of shocks.';
               About(txt,'information','Bad Conditional Shocks Selection',300,550,CurrINI);
            end;
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'ReorderStateConditionalShocks'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % open the dialog for reordering shocks
         %
         [action,ZetaConditionalShocks] = ReOrderShockDLG('init','State Conditioning',DSGEModel.ZetaVariableNames(DSGEModel.ZetaConditionalVariables,:),DSGEModel.StateShockNames,DSGEModel.ZetaConditionalShocks,CurrINI);
         if strcmp(lower(action),'ok')==1;
            DSGEModel.ZetaConditionalShocks = ZetaConditionalShocks;
         end;
         %
         set(controls.about,'UserData',0);
      end;
end;
%
set(controls.open,'UserData',DSGEModel);
set(controls.filemenu,'UserData',CurrINI);
%
% make sure that controls are enabled and disabled as they should
%
YADAEnableControls(DSGEModel,controls);

%
% end of CondStateFunction.m
%
