function [LogPredLike,StdErr,PredVars,MinMaxCondLike,TestStd,LogPredLikeBackNow,StdErrBackNow,MinMaxCondLikeBackNow,TestStdBackNow] = PredLikeMCI(SelectedYData,NumDraws,h,IsOriginal,IsMarginal,IsPosterior,Weights,NumBurnin,TotalDraws,CurrChain,DSGEModel)
% PredLikeMCI: Computes the predictive likelihood and the marginal likelihood for parameter draws using Monte Carlo
%              integration for a DSGE model.
%
% USAGE:
%
%       [LogPredLike,StdErr,PredVars,MinMaxCondLike,TestStd] = PredLikeMCI(SelectedYData,NumDraws,h,IsOriginal,IsMarginal,...
%                                                                          IsPosterior,Weights,NumBurnin,TotalDraws,...
%                                                                          CurrChain,DSGEModel)
%
% REQUIRED INPUT:  SelectedYData (integer) with the selected set of actuals for the observed variables.
%
%                  NumDraws (integer) with the number of draws used.
%
%                  h (integer) with the length of the prediction horizon.
%
%                  IsOriginal (boolean) that takes the value 1 if original values should be predicted and 0 if
%                             annualized values should be ptredicted.
%
%                  IsMarginal (boolean) that takes the value 1 if the marginal predictive likelihood should be
%                             computed and 0 if the joint predictive likelihood is the object of interest.
%
%                  IsPosterior (boolean) that takes the value 1 if the parameter draws are taken from the posterior
%                             distribution and 0 otherwise.
%
%                  Weights (vector) of dimension NumDraws x 1 with the importance weights from the SMC with
%                             likelihood tempering posterior sampler.
%
%                  NumBurnin (integer) with the number of burn-in draws.
%
%                  TotalDraws (integer) with the total number of parameter draws that could have been used.
%
%                  CurrChain (integer) for the selected MCMC chain when posterior draws are used.
%
%                  DSGEModel (structure) with fields containing all the information about a DSGE Model for YADA.
%
% REQUIRED OUTPUT: LogPredLike (vector) of dimension h with the predictive likelihood for the forecast
%                             periods.
%
%                  StdErr (vector) of dimension h with the numerical standard errors of the log predictive
%                             likelihood.
%
%                  PredVars (cell array) with the positions of the predicted variables in the vector of observed
%                             variables for each prediction period.
%
%                  MinMaxCondLike (matrix) of dimension hx3 with the mean, the minimum, and the maximum of
%                             the log conditional likelihood in the first to third columns.
%
%                  TestStd (boolean) that takes the value 1 if the numerical standard errors are robust to small
%                             values for the conditional likelihood and 0 if not.
%
%                  LogPredLikeBackNow (vector) of dimension 2 with the predictive likelihood for the back- and
%                             nowcast periods.
%
%                  StdErrBackNow (vector) of dimension 2 with the numerical standard errors of the log predictive
%                             likelihood for the back- and nowcast periods.
%
%                  MinMaxCondLikeBackNow (matrix) of dimension 2x3 with the mean, the minimum, and the maximum of
%                             the log conditional likelihood for the back- and nowcast periods in the first to
%                             third columns.
%
%                  TestStdBackNow (boolean) that takes the value 1 if the numerical standard errors of the back- and
%                             nowcast periods are robust to small values for the conditional likelihood and 0 if not.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   DG-R/MPR
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2017 European Central Bank.
%
%                       First version: April 27, 2012.
%                        This version: September 4, 2017.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 30-04-2012: Finished the function.
%
% * 14-05-2012: Updated the documentation.
%
% * 21-08-2012: Changed estimator to an importance sampling estimator.
%
% * 18-10-2012: Added numerical standard errors based on Newey-West.
%
% * 03-01-2013: Updated the documentation.
%
% * 28-01-2014: Updated the documentation.
%
% * 14-01-2015: Updated the documentation.
%
% * 25-02-2015: Added an optional output variable for checking if the standard errors are numerically
%               sensitive to underflow issues when exp(a) is close to zero.
%
% * 26-02-2015: Added the output variable "MinMaxCondLike". Changed the file name from
%               "PredLikeImportanceSampling" to "PredLikeMCI".
%
% * 12-06-2015: The function now requires the input "SelectedYData".
%
% * 22-01-2016: Updated the documentation.
%
% * 15-03-2017: The function now computes the predictive likelihood of the back and nowcasts when they exist.
%               As a consequence, the function provides the additional output variables "LogPredLikeBackNow",
%               "StdErrBackNow", "MinMaxCondLikeBackNow", and "TestStdBackNow".
%
% * 18-07-2017: Fixed a bug when computing the Newey-West standard errors using the back and nowcast data.
%               NaN's have to be properly dealt with.
%
% * 25-08-2017: Added the "Weights" input variable.
%
% * 04-09-2017: Speeded up the Weights computations.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize
%
LogPredLike = [];
PredVars = [];
StdErr = [];
MinMaxCondLike = [];
TestStd = 0;
%
LogPredLikeBackNow = [];
StdErrBackNow = [];
MinMaxCondLikeBackNow = [];
TestStdBackNow = 0;
if isempty(Weights)==1;
   Weights = ones(NumDraws,1);
end;
%
SelectedVarStr = ismember((1:DSGEModel.n),DSGEModel.PredictionVariables);
SelectedVarStr = strrep(num2str(SelectedVarStr),' ','');
if IsOriginal==1;
   TypeStr = 'Original';
else;
   TypeStr = 'Annualized';
end;
if IsMarginal==1;
   MargStr = 'Marginal';
else;
   MargStr = 'Joint';
end;
if IsPosterior==1;
   DistStr = 'Posterior';
else;
   DistStr = 'Prior';
end;
%
if IsPosterior==1;
   file = FixFilePath([DSGEModel.OutputDirectory '\predictions\' MargStr 'DSGEPredictiveLikelihood-' DSGEModel.NameOfModel '-' int2str(SelectedYData) '-' TypeStr '-' SelectedVarStr '-' int2str(h) '-' int2str(NumDraws) '.' int2str(NumBurnin) '.' int2str(TotalDraws) '-' int2str(CurrChain) '.mat']);
else;
   file = FixFilePath([DSGEModel.OutputDirectory '\predictions\' MargStr 'DSGEPredictiveLikelihood-' DSGEModel.NameOfModel '-' int2str(SelectedYData) '-' TypeStr '-' SelectedVarStr '-' int2str(h) '-' int2str(NumDraws) '.mat']);
end;
PredData = load(file);
PredVars = PredData.PredVars;
LogPosteriorData = PredData.LogPosteriorData;
if (IsMarginal==1)&(IsOriginal==1);
   LogPosteriorBackNowcastData = PredData.LogPosteriorBackNowcastData;
end;
clear('PredData');
%
% compute the predictive likelihood as the average of the conditional likelihood evaluated at the
% posterior values of the parameters.
%
h = size(LogPosteriorData,2);
Weightsh = Weights*ones(1,h);
meanLPD = sum(LogPosteriorData.*Weightsh,1)/NumDraws;
MinMaxCondLike = [meanLPD;min(LogPosteriorData);max(LogPosteriorData)]';
LPD = LogPosteriorData-(ones(NumDraws,1)*meanLPD);
%
LogPredLike = (log(sum(exp(LPD).*Weightsh)/NumDraws)+meanLPD)';
%
%LogPosteriorData = exp(LogPosteriorData);
%PredLike = sum(LogPosteriorData)'/NumDraws;
%LogPredLike = log(PredLike);
%
% compute numerical standard errors with Newey-West
%
StdErrMat = NeweyWestCovMat(exp(LPD),floor(NumDraws^(1/2.01)));
StdErr = (sqrt(diag(StdErrMat)).*exp(meanLPD'))./exp(LogPredLike);
%
% now test this against the original data
%
StdErrMat = NeweyWestCovMat(exp(LogPosteriorData),floor(NumDraws^(1/2.01)));
StdErrCheck = sqrt(diag(StdErrMat))./exp(LogPredLike);
TestStd = min(abs(StdErr-StdErrCheck)<1.0e-10);
%
% deal with back and nowcasts
%
if (IsMarginal==1)&(IsOriginal==1);
   b = size(LogPosteriorBackNowcastData,2);
   Weightsb = Weights*ones(1,b);
   LogPosteriorBackNowcastData = real(LogPosteriorBackNowcastData);
   meanLPDBackNow = sum(LogPosteriorBackNowcastData.*Weightsb,1)/NumDraws;
   MinMaxCondLikeBackNow = [meanLPDBackNow;min(LogPosteriorBackNowcastData);max(LogPosteriorBackNowcastData)]';
   LPDBackNow = LogPosteriorBackNowcastData-(ones(NumDraws,1)*meanLPDBackNow);
   %
   LogPredLikeBackNow = (log(sum(exp(LPDBackNow).*Weightsb)/NumDraws)+meanLPDBackNow)';
   %
   % and now the numerical standard errors
   %
   % check for NaN's
   %
   NaNPositions = isnan(LPDBackNow);
   if max(max(NaNPositions))==1;
      LPDBackNow(NaNPositions) = log(eps);
      LogPosteriorBackNowcastData(NaNPositions) = log(eps);
   end;
   %
   StdErrMatBackNow = NeweyWestCovMat(exp(LPDBackNow),floor(NumDraws^(1/2.01)));
   StdErrBackNow = (sqrt(diag(StdErrMatBackNow)).*exp(meanLPDBackNow'))./exp(LogPredLikeBackNow);
   %
   % now test this against the original data
   %
   StdErrMatBackNow = NeweyWestCovMat(exp(LogPosteriorBackNowcastData),floor(NumDraws^(1/2.01)));
   StdErrCheckBackNow = sqrt(diag(StdErrMatBackNow))./exp(LogPredLikeBackNow);
   TestStdBackNow = min(abs(StdErrBackNow-StdErrCheckBackNow)<1.0e-10);
end;

%
% end of PredLikeMCI.m
%
