function FittedStd = VillaniCheckOptimum(ObjFuncStr,CenterPoint,OtherParameters,MinMaxGrid,InvHessian,NumberOfEval,SameVerticalScale,SubplotStructure,EstParTeXNames,FigTitle,CurrINI,GraphDirectory,ParamName,ModelString,MargStr,ShowFigs,IsPosterior,varargin)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% PURPOSE:     Computing and plotting perturbations of the log posterior around a given point (CenterPoint). The pertubations
%              are one parameter at the time keeping the other parameters fixed at CenterPoint. 
%              
%              NaN is allowed in both CenterPoint and MinMaxGrid, but not at the same time for a given row (parameter). 
%              If CenterPoint(i)=nan, then CenterPoint(i) becomes the midpoint of the interval specified by MinMaxGrid(i,:)
%              If any of the bounds in MinMaxGrid(i,:) is nan, then the bounds are +- 4 posterior standard deviations, using the InvHessian 
%              to compute approximate posterior standard deviations.
%
% INPUT:       ObjFuncStr      (string)            String with name of .m file containing the objective function
%              OtherParameters (vector)            with values of other parameters that are kept fixed.
%              CenterPoint     (1-by-Npara)        Point where the normal approximation of the log posterior is centered.
%              MinMaxGrid      (Npara-by-2)        MinMaxGrid(i,:) contains the bounds for the i:th parameter. May contain nan's or MinMaxGrid=[]. See PURPOSE. 
%              InvHessian      (Npara-by-Npara)    InvHessian of the log posterior.
%              NumberOfEval    (scalar)            Number of point in the grid.
%              SameVerticalScale (scalar)          If SameVerticalScale==1, then the same scale is used on the vertical axis for each parameter.
%              EstParTeXNames  (Npara-by-Something)EstParTeXNames(i,:) contains the i:ths parameter's name in TeX code.
%              SubplotStructure (1-by-2)           Determines the structure of the Subplots (the number of rows and cols)
%              varargin                            List of additional arguments needed to compute the objective function in ObjFuncStr
%      
% OUTPUT:      FittedStd       (Npara-by-1)        Estimated standard deviation (local curvature) from fit of quadratic around optimum.
%
% AUTHOR:      Mattias Villani, Research Department, Sveriges Riksbank and
%              Department of Statistics, Stockholm University. 
%              E-mail: mattias.villani@riksbank.se
%
% REVISED:     2005-11-03
%
% ADAPTED for YADA by Anders Warne, August 11-29, September 22, November 10, 2006, February 8, 2007,
%              January 10, 2008, February 18-19, 2008, February 25-26, 2008, October 23, 2008,
%              November 28, 2008, January 21, 2010, January 26, 2010, February 5, 2010,
%              February 11, 2010, February 12, 2010, February 22, 2010, June 7, 2010,
%              January 12, 2011, July 6, 2011, December 3, 2014 (ShowFigs), March 18, 2015,
%              June 8, 2015 (groot), June 9, 2015, June 23, 2015, September 9, 2015, August 29, 2016,
%              May 23, 2017 (IsPosterior).
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%
NumberOfParameters = length(CenterPoint);
if isempty(SubplotStructure);
   if NumberOfParameters<=16;
      SubplotStructure=[ceil(sqrt(NumberOfParameters)) ceil(sqrt(NumberOfParameters))];
   else;
      SubplotStructure = [4 4];
   end;
end
NplotsPerFigure=prod(SubplotStructure);
NFigures=ceil(NumberOfParameters/NplotsPerFigure);
PriorFigHandles=cell(NFigures,1);
%
if NFigures==1;
   [Nrows,Ncols] = VillaniConstructOptimalSubplot(NumberOfParameters);
else;
   Nrows=SubplotStructure(1);
   Ncols=SubplotStructure(2);
end;
%
FittedStd = zeros(NumberOfParameters,1);
%
% Computing approximate conditional standard deviations
%
ApproxStd = zeros(NumberOfParameters,1);
S = InvHessian;
FullSet = (1:NumberOfParameters);
for i=1:NumberOfParameters;
   iExcluded = setdiff(FullSet,i);
   ApproxStd(i) = sqrt(S(i,i)-S(i,iExcluded)*inv(S(iExcluded,iExcluded))*S(iExcluded,i));
end;
%
if isempty(MinMaxGrid);
   LowerBoundGrid=CenterPoint-4*ApproxStd;
   UpperBoundGrid=CenterPoint+4*ApproxStd;
   StepLength=(8*ApproxStd)/(NumberOfEval-1);
else;
   for i=1:NumberOfParameters;
      if any(isnan(MinMaxGrid(i,:)));
         LowerBoundGrid(i) = CenterPoint(i)-4*ApproxStd(i);
         UpperBoundGrid(i) = CenterPoint(i)+4*ApproxStd(i);
         StepLength(i) = (8*ApproxStd(i))/(NumberOfEval-1);
         if isnan(CenterPoint(i));
            return;
         end;
      else;
         LowerBoundGrid(i) = MinMaxGrid(i,1);
         UpperBoundGrid(i) = MinMaxGrid(i,2);
         StepLength(i) = (UpperBoundGrid(i)-LowerBoundGrid(i))/(NumberOfEval-1);
         if isnan(CenterPoint(i));
            CenterPoint(i) = (LowerBoundGrid(i)+UpperBoundGrid(i))/2;
         end;
         if (CenterPoint(i)<LowerBoundGrid(i))|(CenterPoint(i)>UpperBoundGrid(i));
            CenterPoint(i) = (LowerBoundGrid(i)+UpperBoundGrid(i))/2;
         end;
      end;
   end;
end;
%
if IsPosterior==1;
   PostStr = 'Posterior';
   PostShort = 'Post';
else;
   PostStr = 'Prior';
   PostShort = 'Prior';
end;
%
if (strcmp(ObjFuncStr,'logPosteriorPhiDSGE')==1)|(strcmp(ObjFuncStr,'logPosteriorPhiDSGEVAR')==1)|(strcmp(ObjFuncStr,'logJointPosteriorPhiDSGEVAR')==1)|(strcmp(ObjFuncStr,'logSystemPriorPhiDSGE')==1);
   NameStr = 'Transformed';
else;
   NameStr = 'Original';
end;
%if exist('groot')==0;
%   GraphicsRoot = 0;
%else;
%   if exist('isvalid')==0;
%      GraphicsRoot = 0;
%   else;
%      if isvalid(groot)==1;
%         GraphicsRoot = groot;
%      else;
%         GraphicsRoot = 0;
%      end;
%   end;
%end;
%
% create a wait dialog
%
txt = ['Please wait while YADA computes the log ' lower(PostStr) ' for "' StringTrim(EstParTeXNames(1,:)) '" with grid values (' num2str(LowerBoundGrid(1),'%0.4f') ':' num2str(StepLength(1),'%0.4f') ':' num2str(UpperBoundGrid(1),'%0.4f') ').'];
WaitHandle = WaitDLG(txt,'information',['Check ' MargStr PostStr ' Mode - ' NameStr ' Parameters'],500,150,CurrINI,0);
WaitControls = get(WaitHandle,'UserData');
%
GridLength = length(LowerBoundGrid(1):StepLength(1):UpperBoundGrid(1))+1;
LogLikeMatrix = NaN*ones(NumberOfParameters,GridLength);
LogPostMatrix = LogLikeMatrix;
GridMatrix = LogLikeMatrix;
XLimMatrix = zeros(NumberOfParameters,2);
YLimMatrix = zeros(NumberOfParameters,2);
Minimum = inf;
Maximum = -inf;
scrsz = get(CurrINI.GraphicsRoot,'ScreenSize');
pos = [(scrsz(3)-1000)/2 (scrsz(4)-680)/2 1000 650];
if isempty(InvHessian)==0;
   LineHandles = zeros(NFigures,3);
else;
   LineHandles = zeros(NFigures,2);
end;
for j=1:NFigures;
   if j>1;
      pos(1) = pos(1)+22;
      pos(2) = pos(2)-22;
      if pos(1)+1000>scrsz(3);
         pos(1) = 0;
      end;
      if pos(2)<=32;
         pos(2) = scrsz(4)-700;
      end;
   end;
   PriorFigHandles{j} = figure('Visible','off','Position',pos,'CloseRequestFcn','delete(gcf); drawnow; pause(0.02);');
   SetFigureProperties(PriorFigHandles{j},CurrINI);
   k = 0;
   if j<NFigures;
      PlotCounter=1+(j-1)*NplotsPerFigure:j*NplotsPerFigure;
   else;
      PlotCounter=NplotsPerFigure*(NFigures-1)+1:NumberOfParameters;
   end;
   % 
   for i=PlotCounter;
      k = k+1;
      %
      % update the wait dialog
      %
      txt = ['Please wait while YADA computes the log ' lower(PostStr) ' for "' StringTrim(EstParTeXNames(i,:)) '" with grid values (' num2str(LowerBoundGrid(i),'%0.4f') ':' num2str(StepLength(i),'%0.4f') ':' num2str(UpperBoundGrid(i),'%0.4f') ').'];
      set(WaitControls.text,'String',txt);
      drawnow;
      pause(0.02);
      %
      % make sure that PriorFigHandles{j} is the current figure
      %
      set(CurrINI.GraphicsRoot,'CurrentFigure',PriorFigHandles{j});
      subplot(Nrows,Ncols,k);
      %
      % Computes the log posterior or prior around the supposed optimum
      %
      GridAroundOpt=LowerBoundGrid(i):StepLength(i):UpperBoundGrid(i);
      if isempty(MinMaxGrid);
         if rem(NumberOfEval,2)==0;
            GridAroundOpt = [GridAroundOpt(1:NumberOfEval/2) CenterPoint(i) GridAroundOpt(NumberOfEval/2+1:NumberOfEval)];
            CenterPointLoc = NumberOfEval/2+1;
         else;
            GridAroundOpt = [GridAroundOpt(1:floor(NumberOfEval/2)) CenterPoint(i) GridAroundOpt(floor(NumberOfEval/2)+1:NumberOfEval)];
            CenterPointLoc = floor(NumberOfEval/2)+1;
         end;
      else;
         NLowerThanCenter = sum(GridAroundOpt<=CenterPoint(i));
         NHigherThanCenter = sum(GridAroundOpt>=CenterPoint(i));
         if NLowerThanCenter+NHigherThanCenter>length(GridAroundOpt); % Center is one of the grid points
            GridAroundOpt(GridAroundOpt~=CenterPoint(i)); % Exclude centerpoint
         end;
         GridAroundOpt = [GridAroundOpt(1:NLowerThanCenter) CenterPoint(i) GridAroundOpt(NLowerThanCenter+1:length(GridAroundOpt))];
         CenterPointLoc = NLowerThanCenter+1;
      end;
      LogPostOnGrid = zeros(1,length(GridAroundOpt));
      Solved = ones(1,length(GridAroundOpt));
      TempPoint = CenterPoint;
      u = 0;
      [NegLogPost,LogLikePost] = feval(ObjFuncStr,[OtherParameters;TempPoint],varargin{:});
      LogPostOnGridMode = -NegLogPost;
      for q=GridAroundOpt;
         u = u+1;
         TempPoint(i)=q;
         [NegLogPost,LogLike] = feval(ObjFuncStr,[OtherParameters;TempPoint],varargin{:});
         LogPostOnGrid(u) = -NegLogPost;
         if NegLogPost~=1000000;
            %
            % only select the values for which the log posterior or prior is sufficiently close to the mode
            %
            if LogPostOnGridMode+NegLogPost<10;
               LogLikeMatrix(i,u) = LogLike+LogPostOnGridMode-LogLikePost;
               LogPostMatrix(i,u) = -NegLogPost;
               GridMatrix(i,u) = q;
            end;
         else;
            Solved(i) = 0;
         end;
      end;
      %
      LogPostOnGrid = LogPostOnGrid(Solved==1); % Discarding non-solvable grid points
      GridAroundOpt = GridAroundOpt(Solved==1);
      CloseToMode = (LogPostOnGridMode-LogPostOnGrid<10); % Plotting only grid point with log posterior or prior
      LogPostOnGrid = LogPostOnGrid(CloseToMode==1);      % not more than 10 units smaller than Log posterior mode 
      GridAroundOpt = GridAroundOpt(CloseToMode==1);
      Minimum = min(Minimum,min(LogPostOnGrid));
      Maximum = max(Maximum,max(LogPostOnGrid));
      %
      % make sure the current figure is properly set
      %
      set(CurrINI.GraphicsRoot,'CurrentFigure',PriorFigHandles{j});
      p1 = plot(GridAroundOpt,LogPostOnGrid,'.-','LineWidth',2,'Color',[0 0 0.75],'Tag','logposterior');
%      ApproxLogPDFMode = logNormalPDF(CenterPoint(i),CenterPoint(i),sqrt(InvHessian(i,i)));
      ApproxLogPDFMode = logNormalPDF(CenterPoint(i),CenterPoint(i),ApproxStd(i));
      hold('on');
      if isempty(InvHessian)==0;
%         ApproxLogPDF = logNormalPDF(GridAroundOpt,CenterPoint(i),sqrt(InvHessian(i,i)));
         ApproxLogPDF = logNormalPDF(GridAroundOpt,CenterPoint(i),ApproxStd(i));
         p2 = plot(GridAroundOpt,ApproxLogPDF-ApproxLogPDFMode+LogPostOnGridMode,':','LineWidth',2,'Color',[0.75 0 0],'Tag','hessian');
         hold('on');
      end;
      %LaplaceStuff=ApproxLogPDF-ApproxLogPDFMode+LogPostOnGridMode;
      %
      % Modified InvHessian by fitting quadratic to evaluated LogPost
      %
      y = LogPostOnGrid(:);
      x = GridAroundOpt(:);
      Z = [ones(length(x),1) x x.^2];
      Bols = inv(Z'*Z)*Z'*y;
      FittedStd(i) = 1/(sqrt(abs(2*Bols(3))));
      ApproxLogPDFMode = logNormalPDF(CenterPoint(i),CenterPoint(i),FittedStd(i));
      ApproxLogPDF = logNormalPDF(GridAroundOpt,CenterPoint(i),FittedStd(i));
      p3 = plot(GridAroundOpt,ApproxLogPDF-ApproxLogPDFMode+LogPostOnGridMode,'--','LineWidth',2,'Color',[0 0.75 0],'Tag','modhessian');
      hold('on');
   end;
   %
   % collect the last set of handles
   %
   if isempty(InvHessian)==0;
      LineHandles(j,:) = [p1 p2 p3];
   else;
      LineHandles(j,:) = [p1 p3];
   end;
end
%
% close the wait dialog
%
set(WaitControls.text,'UserData','done');
delete(WaitHandle);
drawnow;
pause(0.02);
%
ParamCount=1;
for j=1:NFigures;
   set(CurrINI.GraphicsRoot,'CurrentFigure',PriorFigHandles{j});
   % figure(PriorFigHandles{j});
   if NFigures>1;
      set(PriorFigHandles{j},'name',[FigTitle ' No. ' int2str(j)]);
   else;
      set(PriorFigHandles{j},'name',FigTitle);
   end;
   k = 0;
   for Row=1:Nrows;
      for Col=1:Ncols;
         if ParamCount<=NumberOfParameters;
            k = k+1;
            subplot(Nrows,Ncols,k);
            if SameVerticalScale==1;
               set(gca,'ylim',[Minimum Maximum]);
            else;
               YLimits = get(gca,'ylim');
               Minimum = YLimits(1);
               Maximum = YLimits(2);
            end;
            %
            % get the data for the log posterior or prior and the modified hessian
            %
            LogPostHandle = findobj(gca,'Type','line','Tag','logposterior');
            ModHessianHandle = findobj(gca,'Type','line','Tag','modhessian');
            if (IsHandle(LogPostHandle)==1)&(IsHandle(ModHessianHandle)==1);
               YLogPostData = get(LogPostHandle,'YData');
               YModHessianData = get(ModHessianHandle,'YData');
               YLim = [min([YLogPostData YModHessianData]) max([YLogPostData YModHessianData])];
               Maximum = LogPostOnGridMode;
               AddYLim = 0.1*(YLim(2)-YLim(1));
               if AddYLim==0;
                  AddYLim = 0.1*YLim(2);
               end;
               YLim = YLim+[0 AddYLim];
            else;
               YLim = [];
            end;
            %
            LineHandle = line([CenterPoint(ParamCount) CenterPoint(ParamCount)],[Minimum Maximum]);
            set(LineHandle,'color',[0 0 0],'LineWidth',1);
            SetAxesFonts(CurrINI,'Title',StringTrim(EstParTeXNames(ParamCount,:)));
            axis('tight');
            %
            % fix the y-axes
            %
            if (isempty(YLim)==0)&(max(isnan(YLim))==0)&(max(isinf(YLim))==0)&(YLim(2)>YLim(1));
               set(gca,'YLim',YLim);
            end;
            XLimMatrix(ParamCount,:) = get(gca,'XLim');
            YLimMatrix(ParamCount,:) = get(gca,'YLim');
         end;
         ParamCount = ParamCount+1;
      end;
   end;
   %
   % add a legend
   %
   if size(LineHandles,2)==3;
      legHandle = legend([LineHandle LineHandles(j,:)],strvcat('Mode',['Log ' PostStr],'Norm Approx - Hessian','Norm Approx - Modified Hessian'));
   else;
      legHandle = legend([LineHandle LineHandles(j,:)],strvcat('Mode',['Log ' PostStr],'Norm Approx - Modified Hessian'));
   end;
   SetAxesFonts(CurrINI,legHandle);
   %
   % transform units temporarily into pixels
   %
   OldAxesUnits = get(gca,'Units');
   OldLegUnits = get(legHandle,'Units');
   set(gca,'Units','pixels');
   set(legHandle,'Units','pixels');
   %
   % check if we can move legend
   %
   LegPos = get(legHandle,'Position');
   AxPos = get(gca,'Position');
   if LegPos(1)<AxPos(1)+AxPos(3);
      if AxPos(1)+AxPos(3)+10+LegPos(3)<1000;
         LegPos(1) = AxPos(1)+AxPos(3)+10;
         set(legHandle,'Position',LegPos);
      end;
   end;
   set(gca,'Units',OldAxesUnits);
   set(legHandle,'Units',OldLegUnits);
   %
   % save it to disk before
   %
   if DirExist(GraphDirectory)==1;
      if NFigures>1;
         file = FixFilePath([GraphDirectory '\' StringTrim(MargStr) PostStr 'ModeOptimum-' ModelString ParamName '-' num2str(j,'%0.0f') '.fig']);
      else;
         file = FixFilePath([GraphDirectory '\' StringTrim(MargStr) PostStr 'ModeOptimum-' ModelString ParamName '.fig']);
      end;
      SetLogText(strvcat('   Saving plot to:',['   ' GraphDirectory '...']));
      saveas(PriorFigHandles{j},file);
   end;
   if (ShowFigs==0)|(isempty(ModelString)==0);
      close(PriorFigHandles{j});
   end;
end;
%
MinLike = min(min(LogLikeMatrix));
MinPost = min(min(LogPostMatrix));
MaxLike = max(max(LogLikeMatrix));
MaxPost = max(max(LogPostMatrix));
%
% now we create plots of the log posterior or prior versus the log-likelihood
%
FigTitle = strrep(FigTitle,'Checking Curvature','Comparing Curvatures');
ParamCount=1;
for j=1:NFigures;
   pos(1) = pos(1)+22;
   pos(2) = pos(2)-22;
   if pos(1)+1000>scrsz(3);
      pos(1) = 0;
   end;
   if pos(2)<=32;
      pos(2) = scrsz(4)-700;
   end;
   %
   SecondFigHandles{j} = figure('Visible','off','Position',pos,'CloseRequestFcn','delete(gcf); drawnow; pause(0.02);');
   SetFigureProperties(SecondFigHandles{j},CurrINI);
   k = 0;
   if j<NFigures;
      PlotCounter = 1+(j-1)*NplotsPerFigure:j*NplotsPerFigure;
   else;
      PlotCounter = NplotsPerFigure*(NFigures-1)+1:NumberOfParameters;
   end;
   if NFigures>1;
      set(SecondFigHandles{j},'name',[FigTitle ' No. ' int2str(j)]);
   else;
      set(SecondFigHandles{j},'name',FigTitle);
   end;
   k = 0;
   for Row=1:Nrows;
      for Col=1:Ncols;
         if ParamCount<=NumberOfParameters;
            k = k+1;
            %
            % ensure that the handle to the current figure is correct (HG2 issue)
            %
            set(CurrINI.GraphicsRoot,'CurrentFigure',SecondFigHandles{j});
            subplot(Nrows,Ncols,k);
            %
            % plot the log posterior or prior
            %
            p1 = plot(GridMatrix(ParamCount,:),LogPostMatrix(ParamCount,:),'.-','LineWidth',2,'Color',[0 0 0.75],'Tag','logposterior');
            hold('on');
            %
            % plot the log likelihood
            %
            p2 = plot(GridMatrix(ParamCount,:),LogLikeMatrix(ParamCount,:),'--','LineWidth',2,'Color',[0.75 0 0],'Tag','loglikelihood');
            hold('on');
            %
            LineHandle = line([CenterPoint(ParamCount) CenterPoint(ParamCount)],[YLimMatrix(ParamCount,1) Maximum]);
            set(LineHandle,'color',[0 0 0],'LineWidth',1);
            SetAxesFonts(CurrINI,'Title',StringTrim(EstParTeXNames(ParamCount,:)));
            %
            if SameVerticalScale==1;
               set(gca,'XLim',XLimMatrix(ParamCount,:),'YLim',YLimMatrix(ParamCount,:));
            else;
               CurrYLim = get(gca,'YLim');
               if CurrYLim(2)>=YLimMatrix(ParamCount,2);
                  YLim = [YLimMatrix(ParamCount,1) CurrYLim(2)];
               else;
                  YLim = YLimMatrix(ParamCount,:);
               end;
               set(gca,'XLim',XLimMatrix(ParamCount,:),'YLim',YLim);
            end;
            hold('off');
         end;
         ParamCount = ParamCount+1;
      end;
   end;
   %
   % add a legend
   %
   if IsPosterior==1;
      legHandle = legend([LineHandle p1 p2],strvcat('Mode','Log Posterior','Log Likelihood (scaled)'));
   else;
      legHandle = legend([LineHandle p1 p2],strvcat('Mode','Log Joint Prior','Log System Prior (scaled)'));
   end;
   SetAxesFonts(CurrINI,legHandle);
   %
   % transform units temporarily into pixels
   %
   OldAxesUnits = get(gca,'Units');
   OldLegUnits = get(legHandle,'Units');
   set(gca,'Units','pixels');
   set(legHandle,'Units','pixels');
   %
   % check if we can move legend
   %
   LegPos = get(legHandle,'Position');
   AxPos = get(gca,'Position');
   if LegPos(1)<AxPos(1)+AxPos(3);
      if AxPos(1)+AxPos(3)+10+LegPos(3)<1000;
         LegPos(1) = AxPos(1)+AxPos(3)+10;
         set(legHandle,'Position',LegPos);
      end;
   end;
   set(gca,'Units',OldAxesUnits);
   set(legHandle,'Units',OldLegUnits);
   %
   % save it to disk before
   %
   if DirExist(GraphDirectory)==1;
      if NFigures>1;
         file = FixFilePath([GraphDirectory '\' StringTrim(MargStr) 'CompLike' PostShort 'Optimum-' ModelString ParamName '-' num2str(j,'%0.0f') '.fig']);
      else;
         file = FixFilePath([GraphDirectory '\' StringTrim(MargStr) 'CompLike' PostShort 'Optimum-' ModelString ParamName '.fig']);
      end;
      SetLogText(strvcat('   Saving plot to:',['   ' GraphDirectory '...']));
      saveas(SecondFigHandles{j},file);
   end;
   if (ShowFigs==0)|(isempty(ModelString)==0);
      close(SecondFigHandles{j});
   end;
end;
%
% display the figure that haven't been deleted
%
if (ShowFigs==1)&(isempty(ModelString)==1);
   for j=1:NFigures;
      SetToolbarImages(PriorFigHandles{j},CurrINI.images);
      set(PriorFigHandles{j},'Visible','on');
      drawnow;
      pause(0.02);
   end;
   for j=1:NFigures;
      SetToolbarImages(SecondFigHandles{j},CurrINI.images);
      set(SecondFigHandles{j},'Visible','on');
      drawnow;
      pause(0.02);
   end;
end;

%
% end of VillaniCheckOptimum.m
%
