function PrintDSGEMomentsResponseTheta(ResFile,MeanResp,StdResp,CorrResp,ParameterNames,ParamNumber,OutputType,EstStr,DSGEModel,CurrINI)
% PrintDSGEMomentsResponseTheta: Prints the numerical partial derivatives of the means, standard deviations, and correlations
%                                with respect to a change in one structural parameters.
%
% USAGE:
%
%       PrintDSGEMomentsResponseTheta(ResFile,MeanResp,StdResp,CorrResp,ParameterNames,ParamNumber,OutputType, ...
%                                     EstStr,DSGEModel,CurrINI)
%
% REQUIRED INPUT: ResFile (string vector) with the full path+name of the file to be printed.
%
%                 MeanResp (matrix) of dimension nxk with the responses of the exogenous variables from the change
%                          change in one structural parameter. This matrix is an r dimensional zero vector for
%                          state variables.
%
%                 StdResp (vector) of dimension n for observed variables and r for state variables with the response
%                          of the standard deviations of the variables.
%
%                 CorrResp (vector) of dimension n(n-1)/2 for observed variables and r(r-1)/2 for state variables
%                          with the responses of the correlations (stacking of elements below the diagonal of the
%                          correlation matrix).
%
%                 ParameterNames (string matrix) with the names of the parameters.
%
%                 ParamNumber (integer) with the position of the parameter in theta to change.
%
%                 OutputType (boolean) that takes the value 1 if responses to the moments should be computed
%                          for the observed variables, and 0 if the responses should be computed for the state
%                          variables.
%
%                 EstStr (string vector) that takes the values "Initial Values" or "Posterior Mode".
%
%                 DSGEModel (structure) with fields containing all the information about a DSGE Model for YADA.
%
%                 CurrINI (structure) with initialization information.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   DG-R/MPR
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2017 European Central Bank.
%
%                       First version: March 21, 2014.
%                        This version: March 22, 2017.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 24-03-2014: Finished the function.
%
% * 14-01-2015: Updated the documentation.
%
% * 22-02-2016: Updated the documentation.
%
% * 22-03-2017: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

fid = fopen(ResFile,'wt');
fprintf(fid,'********************************************************************************\n');
fprintf(fid,'*                                                                              *\n');
fprintf(fid,'*    M O M E N T   R E S P O N S E S   T O   A   C H A N G E   I N   O N E     *\n');
fprintf(fid,'* S T R U C T U R A L   P A R A M E T E R   O F   T H E   D S G E   M O D E L  *\n');
fprintf(fid,'*                                                                              *\n');
fprintf(fid,'********************************************************************************\n\n');
%
% initializing the formatting of output numbers. we usually apply the 12.6f format.
%
PrtVal = ['%' num2str(6+CurrINI.decimals,'%0.0f') '.' num2str(CurrINI.decimals,'%0.0f') 'f'];
%
k = DSGEModel.k;
if OutputType==1;
   VarStr = 'Observed';
   [n,vn] = size(DSGEModel.VariableNames);
   VariableNames = DSGEModel.VariableNames;
else;
   VarStr = 'State';
   [n,vn] = size(DSGEModel.StateVariableNames);
   VariableNames = DSGEModel.StateVariableNames;
end;
if vn<20;
   AddNameStr = SpaceStr(20-vn);
else;
   AddNameStr = '';
end;
fprintf(fid,'NOTE: The moment responses are calculated as numerical partial derivatives of the\n');
fprintf(fid,'      (mean,) standard deviation and correlation of the %s variables with \n',lower(VarStr));
fprintf(fid,'      respect to the structural parameter: "%s".\n\n',StringTrim(ParameterNames(ParamNumber,:)));
%
% check if we should write the effects on the means
%
if (OutputType==1)&(k>0);
   PrtStr = '%s%s';
   TopStr = MultiCharStr('=',21+(15*k));
   DivStr = MultiCharStr('-',21+(15*k));
   kn = size(DSGEModel.XVariableNames,2);
   if kn<15;
      AddXVar = SpaceStr(15-kn);
   else;
      AddXVar = '';
   end;
   ConstStr = '                    ';
   for i=1:k;
      PrtStr = [PrtStr '   ' PrtVal];
      ConstStr = [ConstStr AddXVar DSGEModel.XVariableNames(i,:)];
   end;
   fprintf(fid,[TopStr '\n']);
   fprintf(fid,' Response of the means:\n');
   fprintf(fid,[ConstStr '\n']);
   fprintf(fid,'            Variable \n');
   fprintf(fid,[DivStr '\n']);
   for i=1:n;
      fprintf(fid,[PrtStr '\n'],AddNameStr,VariableNames(i,:),MeanResp(i,:)');
   end;
   fprintf(fid,[TopStr '\n\n']);
end;
%
% write the effects on the standard deviations
%
PrtStr = ['%s%s   ' PrtVal '\n'];
TopStr = MultiCharStr('=',36);
DivStr = MultiCharStr('-',36);
fprintf(fid,[TopStr '\n']);
fprintf(fid,' Responses of the standard devs:\n');
fprintf(fid,'            Variable \n');
fprintf(fid,[DivStr '\n']);
for i=1:n;
   fprintf(fid,PrtStr,AddNameStr,VariableNames(i,:),StdResp(i));
end;
fprintf(fid,[TopStr '\n\n']);
%
% and now the correlations
%
if n>1;
   VarStrMat = '';
   for j=1:n-1;
      for i=j+1:n;
         VarStrMat = strvcat(VarStrMat,[VariableNames(j,:) ' ' VariableNames(i,:)]);
      end;
   end;
   TopStr = MultiCharStr('=',57);
   DivStr = MultiCharStr('-',57);
   fprintf(fid,[TopStr '\n']);
   fprintf(fid,' Responses of the correlations:\n');
   fprintf(fid,'          Variable 1/Variable 2\n');
   fprintf(fid,[DivStr '\n']);
   for i=1:(n*(n-1))/2;
      fprintf(fid,['%s%s%s   ' PrtVal '\n'],AddNameStr,VarStrMat(i,:),AddNameStr,CorrResp(i));
   end;
   fprintf(fid,[TopStr '\n\n']);
end;
%
% bottom file data
%
fprintf(fid,'Directory for file: %s\n',GetPath(ResFile));
fprintf(fid,'Name of file:       %s\n',GetFilename(ResFile));
fprintf(fid,'Output created on:  %s\n\n',datestr(now,0));
fclose(fid);

%
% end of PrintDSGEMomentsResponseTheta.m
%
