function PrintKolmogorovSmirnovTwoSample(ResFile,ResMat,VariableNames,TotalDraws,AddDraws,DSGEModel,CurrINI)
% PrintKolmogorovSmirnovTwoSample: prints the results of two-sample Kolmogorov-Smirnov tests to file.
%
% USAGE:
%
%       PrintKolmogorovSmirnovTwoSample(ResFile,ResMat,VariableNames,TotalDraws,AddDraws,DSGEModel,CurrINI)
%
% REQUIRED INPUT: ResFile (string) with the full path and name of the text file to print to.
%
%                 ResMat (matrix) of dimension mx2xh with the test-values and p-values of the
%                        Kolmogorov-Smirnov tests for the m variables over potentially h horizons.
%
%                 VariableNames (string matrix) with the names of the m variables.
%
%                 TotalDraws (integer) with the total number of draws for both samples.
%
%                 AddDraws (integer) with the number of draws for the second sample.
%
%                 DSGEModel (structure)  whose fields contain all required information about the model,
%                        such as input files, variable and sample data.
%
%                 CurrINI (structure) with initialization information.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   DG-R/MPR
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2017 European Central Bank.
%
%                       First version: September 16, 2014.
%                        This version: March 22, 2017.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 17-09-2014: Updated the documentation.
%
% * 23-09-2014: Added more information under the NOTE part of the output.
%
% * 23-10-2014: Took into account that the test is not computed when one distribution is missing.
%
% * 14-01-2015: Updated the documentation.
%
% * 22-02-2016: Updated the documentation.
%
% * 22-03-2017: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

q = size(ResMat,1);
h = 1;
if length(size(ResMat))==3;
   h = size(ResMat,3);
end;
%
fid = fopen(ResFile,'wt');
fprintf(fid,'********************************************************************************\n');
fprintf(fid,'*                                                                              *\n');
fprintf(fid,'*               K O L M O G O R O V - S M I R N O V   T E S T S                *\n');
fprintf(fid,'*                                                                              *\n');
fprintf(fid,'********************************************************************************\n\n');
%
%
% some information about the prediction sample
%
fprintf(fid,'Total number of draws:                          %0.0f\n',TotalDraws);
fprintf(fid,'Number of draws from the original distribution: %0.0f\n',TotalDraws-AddDraws);
fprintf(fid,'Number of draws from the second distribution:   %0.0f\n\n',AddDraws);
%
%
% write the test results to file
%
ns = size(VariableNames,2);
if ns<30;
   AddStr = SpaceStr(30-ns);
else;
   AddStr = '';
end;
PrtStr = '%s%s   %11.4f   %0.4f \n';
for i=1:h;
   fprintf(fid,'======================================================\n');
   if h>1;
      fprintf(fid,' Time period: %2.0f\n',i);
   end;
   fprintf(fid,'                      variable       KS stat  p-value \n');
   fprintf(fid,'------------------------------------------------------\n');
   if h>1;
      for j=1:q;
         fprintf(fid,PrtStr,AddStr,VariableNames(j,:),ResMat(j,:,i)');
      end;
   else;
      for j=1:q;
         fprintf(fid,PrtStr,AddStr,VariableNames(j,:),ResMat(j,:)');
      end;
   end;
   fprintf(fid,'======================================================\n\n');
end;
%
% write some information concerning the tests
%
fprintf(fid,'NOTE: The null hypothesis is that the original and second distribution\n');
fprintf(fid,'      are equal. It is assumed that the underlying draws are taken from\n');
fprintf(fid,'      from the same distribution, but that the draws from the original\n');
fprintf(fid,'      distribution were used for the calculations, while the draws from\n');
fprintf(fid,'      the second distribution were discarded. The test therefore checks\n');
fprintf(fid,'      if the draws that were accepted and those that were discarded can\n');
fprintf(fid,'      be treated as being taken from the same distribution or not. NaN''s\n');
fprintf(fid,'      reflect that the test was not conducted due to the second distri-\n');
fprintf(fid,'      bution being empty.\n\n');
%
fprintf(fid,'********************************************************************************\n\n');
%
% Some information about the filename and its location
%
fprintf(fid,'Directory for file: %s\n',GetPath(ResFile));
fprintf(fid,'Name of file:       %s\n',GetFilename(ResFile));
fprintf(fid,'Output created on:  %s\n\n',datestr(now,0));
fclose(fid);

%
% end of PrintKolmogorovSmirnovTwoSample.m
%
