function PrintSystemPriorModeResults(ResultsFile,DSGEModel,CurrINI,ParameterNames,thetaMode,theta,thetaDist,PriorDist,ThetaPhiPartial,InverseHessian,InverseHessianFinDiff,phiFittedStd,LogSPDensity,LogPriorDensity,LogJacobianValue,LogSPIV,LogPriorIV,LogJacobianIV,LogMargLikeSystemLaplace,thetaCalibrated,thetaAdditionalInitial,thetaAdditionalMode,OptResults,ScreenMessages,NumMeasureErrors,OptAlgorithm,OptParameters,MaxTime,ToleranceValue,MaxIter,NumIter,FuncDiff)
% PrintSystemPriorModeResults: prints the prior mode results to file for the system prior.
%
% USAGE:
%
%       PrintSystemPriorModeResults(ResultsFile,DSGEModel,CurrINI,ParameterNames,thetaMode,theta,...
%                       thetaDist,PriorDist,ThetaPhiPartial,InverseHessian,InverseHessianFinDiff,...
%                       phiFittedStd,LogSPDensity,LogPriorDensity,LogJacobianValue,LogSPIV,LogPriorIV,...
%                       LogJacobianIV,LogMargLikeSystemLaplace,thetaCalibrated,thetaAdditionalInitial,...
%                       thetaAdditionalMode,OptResults,ScreenMessages,NumMeasureErrors,OptAlgorithm,...
%                       OptParameters,MaxTime,ToleranceValue,MaxIter,NumIter,FuncDiff)
%
% REQUIRED INPUT:  ResultsFile (string) with the full path and name of the output file.
%
%                  DSGEModel (structure) whose fields contain all required information about the model, such as
%                            input files, variable and sample data.
%
%                  CurrINI (structure) with initialization information.
%
%                  ParameterNames (structure) with fields "calibrated", "beta", "gamma", "normal", "invgamma", "truncnormal",
%                            "uniform", "student", "cauchy", "logistic", "gumbel", "pareto", "all" and "estimated". Each
%                            field returns a string matrix with the parameter names.
%
%                  thetaMode (vector) with the prior mode of the original parameters using the system prior.
%
%                  theta (vector) with the initial values of the original parameters.
%
%                  thetaDist (vector) with integers 1 if gamma, 2 if beta, 3 if normal, 4 if invgamma, 5 if
%                            truncnormal, 6 if uniform, 7 if student, 8 if cauchy, 9 if logistic, 10 if gumbel,
%                            and 11 if pareto.
%
%                  PriorDist (structure) with fields "beta", "gamma", "normal", "invgamma", "truncnormal", "uniform",
%                            "student", "cauchy", "logistic", "gumbel", and "pareto", where the values are matrices with the
%                            parameter values for the distributions. In the case of the normal, beta, gamma, logistic 
%                            and gumbel these parameter values are the mean and the standard deviation. For the invgamma
%                            it is the s and q parameters, where q is a positive integer (degrees of freedom). For the left
%                            truncated normal there's a third parameter, the lower bound. For the uniform the parameter
%                            values are the lower and the upper bound. For the student-t they are the location (mode),
%                            the scale and the degrees of freedom parameter, while for the Cauchy only the location
%                            and scale parameters are needed. For the Pareto the parameter values are for the mode
%                            and the shape parameter.
%
%                  ThetaPhiPartial (matrix) where the diagonal entries give the partial derivatives of the original
%                            parameters wrt the transformed parameters. Evalues at the mode.
%
%                  InverseHessian (matrix), the inverse of the Hessian at the mode of the transformed parameters.
%                            This matrix is provides directly from the optimization routine.
%
%                  InverseHessianFinDiff (matrix) with minus the inverse Hessian at the mode of the transformed
%                            parameters using finite differences. This matrix is used for the Laplace approximation.
%
%                  phiFittedStd (vector) of standard deviations of the transformed parameters. This vector is given
%                            as output by "VillaniCheckOptimum".
%
%                  LogSPDensity (scalar) with the value of the conditional system prior at the prior mode.
%
%                  LogPriorDensity (scalar) with the value of the marginal prior of the original
%                            parameters at the prior mode.
%
%                  LogJacobianValue (scalar) with the value of the log Jacobian of the parameter
%                            transformation at the prior mode.
%
%                  LogSPIV (scalar) with the value of the system prior at the initial values.
%
%                  LogPriorIV (scalar) with the value of the marginal prior of the original
%                            parameters at the initial values.
%
%                  LogJacobianIV (scalar) with the value of the log Jacobian at the initial values.
%
%                  LogMargLikeSystemLaplace (scalar) with the value of the Laplace approximation of
%                            the log marginal likelihood of the system prior.
%
%                  thetaCalibrated (vector) with values of any calibrated DSGE model parameters.
%
%                  thetaAdditionalInitial (vector) with initial values of any additional DSGE model parameters.
%
%                  thetaAdditionalMode (vector) with the prior mode values of any additional DSGE model parameters.
%
%                  OptResults (matrix), of dimension (NumIter x 4) containing iteration no, value of log posterior, value
%                            of convergence measure, and the largest absolute value of the gradient.
%
%                  ScreenMessages (string matrix) with all the screen messages that csminwel could have produced.
%
%                  NumMeasureErrors (integer) giving the number of measurement errors in the model.
%
%                  OptAlgorithm (string vector), with either "fminunc" or "csminwel" being the optimization algorithm
%                            used to calculate the posterior mode.
%
%                  OptParameters (string vector) that takes the value "Transformed" or "Original" and refers to
%                            which parameters were used as target during optimization.
%
%                  MaxTime (scalar), the time in seconds to complete the optimization problem.
%
%                  ToleranceValue (scalar), the tolerance value used when evaluating if the optimization algorithm has
%                            converged or not.
%
%                  MaxIter (positive integer) the maximum number of iterations considered by the optimization algorithm.
%
%                  NumIter (positive integer) the actual number of iteration used by the optimization algorithm.
%
%                  FuncDiff (scalar) the value of the convergence measure.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   DG-R/MPR
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2017 European Central Bank.
%
%                       First version: June 2, 2017.
%                        This version: June 6, 2017.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 06-06-2017: Added the input variables "thetaCalibrated,thetaAdditionalInitial,thetaAdditionalMode,
%               OptResults,ScreenMessages".
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% open a file
%
fid = fopen(ResultsFile,'wt');
%
% the header information
%
fprintf(fid,'********************************************************************************\n');
fprintf(fid,'*                                                                              *\n');
fprintf(fid,'* S Y S T E M   P R I O R   M O D E   E S T I M A T I O N   W I T H   Y A D A  *\n');
fprintf(fid,'*                                                                              *\n');
fprintf(fid,'********************************************************************************\n\n');
%
% Model info
%
if DSGEModel.ModelSolver==1;
   SolverStr = 'Anderson & Moore (AiM)';
elseif DSGEModel.ModelSolver==2;
   SolverStr = 'Klein (solab)';
elseif DSGEModel.ModelSolver==3;
   SolverStr = 'Sims (gensys)';
end;
fprintf(fid,'Name of model:                    %s\n',DSGEModel.NameOfModel);
fprintf(fid,'DSGE model solver:                %s\n',SolverStr);
fprintf(fid,'Data construction file:           %s\n',GetFilename(DSGEModel.DataConstructionFile));
fprintf(fid,'Measurement equation file:        %s\n',GetFilename(DSGEModel.MeasurementEquationFile));
fprintf(fid,'Prior distribution data file:     %s\n',GetFilename(DSGEModel.PriorFile));
fprintf(fid,'Sheet name:                       %s\n',DSGEModel.PriorFileSheet);
fprintf(fid,'System prior density file:        %s\n',GetFilename(DSGEModel.SystemPriorFile));
if FileExist(DSGEModel.UpdateParameterFile)==1;
   fprintf(fid,'Update parameters file:           %s\n',GetFilename(DSGEModel.UpdateParameterFile));
end;
if FileExist(DSGEModel.InitializeParameterFile)==1;
   fprintf(fid,'Initialize parameters file:       %s\n',GetFilename(DSGEModel.InitializeParameterFile));
end;
fprintf(fid,'AiM model file:                   %s\n\n',GetFilename(DSGEModel.AIMFile));
%
fprintf(fid,'Number of observed variables:     %0.0f\n',min(size(DSGEModel.Y)));
fprintf(fid,'Number of exogenous variables:    %0.0f\n',min(size(DSGEModel.X)));
fprintf(fid,'Number of state variables:        %0.0f\n',size(DSGEModel.StateVariableNames,1));
fprintf(fid,'Number of state shocks:           %0.0f\n',size(DSGEModel.StateShockNames,1));
fprintf(fid,'Number of measurement errors:     %0.0f\n\n',NumMeasureErrors);
%
% observed variable names, state variable names and state shocks
%
VarNames = StrWrap(vecstr(DSGEModel.VariableNames,' '),40);
fprintf(fid,'Names of observed variables:      %s\n',VarNames(1,:));
for i=2:size(VarNames,1);
   if strcmp(StringTrim(VarNames(i,:)),'')==0;
      fprintf(fid,'                                  %s\n',VarNames(i,:));
   end;
end;
if min(size(DSGEModel.X))>0;
   XVarNames = StrWrap(vecstr(DSGEModel.XVariableNames,' '),40);
   fprintf(fid,'Names of exogenous variables:     %s\n',XVarNames(1,:));
   for i=2:size(XVarNames,1);
      if strcmp(StringTrim(XVarNames(i,:)),'')==0;
         fprintf(fid,'                                  %s\n',XVarNames(i,:));
      end;
   end;
end;
StateVarNames = StrWrap(vecstr(DSGEModel.StateVariableNames,' '),40);
fprintf(fid,'Names of state variables:         %s\n',StateVarNames(1,:));
for i=2:size(StateVarNames,1);
   if strcmp(StringTrim(StateVarNames(i,:)),'')==0;
      fprintf(fid,'                                  %s\n',StateVarNames(i,:));
   end;
end;
StateShockNames = StrWrap(vecstr(DSGEModel.StateShockNames,' '),40);
fprintf(fid,'Names of the state shocks:        %s\n',StateShockNames(1,:));
for i=2:size(StateShockNames,1);
   if strcmp(StringTrim(StateShockNames(i,:)),'')==0;
      fprintf(fid,'                                  %s\n',StateShockNames(i,:));
   end;
end;
%
fprintf(fid,'\n********************************************************************************\n');
fprintf(fid,'*                                                                              *\n');
fprintf(fid,'*               O P T I M I Z A T I O N   I N F O R M A T I O N                *\n');
fprintf(fid,'*                                                                              *\n');
fprintf(fid,'********************************************************************************\n\n');
%
% optimization information
%
fprintf(fid,'Optimization algorithm:           %s\n',OptAlgorithm);
fprintf(fid,'Parameter target type:            %s\n',lower(OptParameters));
fprintf(fid,'Time:                             %s (hrs:min:sec)\n',SecondsToHoursStr(MaxTime));
if strcmp(lower(OptAlgorithm),'gmhmaxlik')==0;
   fprintf(fid,'Tolerance level:                  %0.10f\n',ToleranceValue);
   fprintf(fid,'Maximum number of iterations:     %0.0f\n',MaxIter);
   fprintf(fid,'Number of iterations:             %0.0f\n',NumIter);
   fprintf(fid,'Convergence value:                %0.10f\n\n',FuncDiff);
else;
   fprintf(fid,'Number of parameter draws:        %0.0f\n',NumIter);
   fprintf(fid,'Objective function change:        %0.10f\n\n',FuncDiff);
end;
%
% check if we have screen messages from csminwel to provide.
%
%if isempty(ScreenMessages)==0;
%   fprintf(fid,'********************************************************************************\n');
%   fprintf(fid,'*                                                                              *\n');
%   if strcmp(lower(OptAlgorithm),'csminwel')==1;
%      fprintf(fid,'*               C S M I N W E L   S C R E E N   M E S S A G E S                *\n');
%   else;
%      fprintf(fid,'*                        S C R E E N   M E S S A G E S                         *\n');
%   end;
%   fprintf(fid,'*                                                                              *\n');
%   fprintf(fid,'********************************************************************************\n\n');
%   for i=1:size(ScreenMessages,1);
%      fprintf(fid,'%s\n',ScreenMessages(i,:));
%   end;
%   fprintf(fid,'\n');
%end;
%
fprintf(fid,'********************************************************************************\n');
fprintf(fid,'*                                                                              *\n');
fprintf(fid,'*                  P R I O R   D E N S I T Y   R E S U L T S                   *\n');
fprintf(fid,'*                                                                              *\n');
fprintf(fid,'********************************************************************************\n\n');
%
% LogSPDensity,LogPriorDensity,LogJacobianValue,LogSPIV,LogPriorIV,LogJacobianIV
%
prt_val = ['%' num2str(9+CurrINI.decimals,'%0.0f') '.' num2str(CurrINI.decimals,'%0.0f') 'f'];
fprintf(fid,'--------------------------------------------------------------------------------\n');
fprintf(fid,'|                   Log System Prior Density Kernel at Mode                    |\n');
fprintf(fid,'|------------------------------------------------------------------------------|\n');
fprintf(fid,['| Value of the full log system prior kernel of phi at mode:   ' prt_val '  |\n'],LogSPDensity+LogPriorDensity+LogJacobianValue);
fprintf(fid,['| Value of the full log system prior kernel of theta at mode: ' prt_val '  |\n'],LogSPDensity+LogPriorDensity);
fprintf(fid,['| Value of the conditional system prior at theta mode:        ' prt_val '  |\n'],LogSPDensity);
fprintf(fid,['| Value of the marginal log prior density of phi at mode:     ' prt_val '  |\n'],LogPriorDensity+LogJacobianValue);
fprintf(fid,['| Value of the marginal log prior density of theta at mode:   ' prt_val '  |\n'],LogPriorDensity);
fprintf(fid,['| Value of the log Jacobian (dtheta/dphi) at mode:            ' prt_val '  |\n'],LogJacobianValue);
fprintf(fid,'|                                                                              |\n');
fprintf(fid,'| Value of the Laplace approximation of the log marginal                       |\n');
fprintf(fid,['| likelihood based on the posterior mode of phi:              ' prt_val '  |\n'],LogMargLikeSystemLaplace);
fprintf(fid,'|------------------------------------------------------------------------------|\n');
fprintf(fid,'|                  Log System Prior Kernel at Initial Value                    |\n');
fprintf(fid,'|------------------------------------------------------------------------------|\n');
fprintf(fid,['| Initial value of the full log system prior kernel of phi:   ' prt_val '  |\n'],LogSPIV+LogPriorIV+LogJacobianIV);
fprintf(fid,['| Initial value of the full log system prior kernel of theta: ' prt_val '  |\n'],LogSPIV+LogPriorIV);
fprintf(fid,['| Initial value of the conditional system prior at theta:     ' prt_val '  |\n'],LogSPIV);
fprintf(fid,['| Initial value of the marginal log prior density of phi:     ' prt_val '  |\n'],LogPriorIV+LogJacobianIV);
fprintf(fid,['| Initial value of the marginal log prior density of theta:   ' prt_val '  |\n'],LogPriorIV);
fprintf(fid,['| Initial value of the log Jacobian (dtheta/dphi):            ' prt_val '  |\n'],LogJacobianIV);
fprintf(fid,'--------------------------------------------------------------------------------\n\n');
fprintf(fid,'NOTE: the full log system prior kernel of phi is equal to the sum of the\n      conditional system prior and the marginal log prior of phi. The full\n      log system prior kernal of theta is obtained by subtracting the log\n      Jacobian, and analogously for the marginal log prior of theta.\n\n');
%
fprintf(fid,'NOTE: The Laplace approximation of the log marginal likelihood is calculated\n');
if isnan(InverseHessianFinDiff)==0;
   fprintf(fid,'      using a finite difference estimator of minus the inverse Hessian at the\n      prior mode.\n\n');
else;
   fprintf(fid,'      using the inverse Hessian at the mode from the optimization routine.\n\n');
end;
%
% Optimization results
%
prt_val = ['%' num2str(6+CurrINI.decimals,'%0.0f') '.' num2str(CurrINI.decimals,'%0.0f') 'f'];
prt_str = ['|    %10.0f          ' prt_val '           ' prt_val '         ' prt_val '     |\n'];
fprintf(fid,'---------------------------------------------------------------------------------------\n');
if IsOdd(DSGEModel.MaximizeAlgorithmValue)==1;
   fprintf(fid,'| Iteration Number     log prior of phi      convergence measure   max(abs(gradient)) |\n');
else;
   fprintf(fid,'| Iteration Number    log prior of theta     convergence measure   max(abs(gradient)) |\n');
end;
fprintf(fid,'|-------------------------------------------------------------------------------------|\n');
fprintf(fid,prt_str,OptResults');
fprintf(fid,'---------------------------------------------------------------------------------------\n\n');
%
prt_val = ['%' num2str(6+CurrINI.decimals,'%0.0f') '.' num2str(CurrINI.decimals,'%0.0f') 'f'];
fprintf(fid,'********************************************************************************\n');
fprintf(fid,'*                                                                              *\n');
fprintf(fid,'*                             P A R A M E T E R S                              *\n');
fprintf(fid,'*                                                                              *\n');
fprintf(fid,'********************************************************************************\n\n');
%
% The parameter estimates
%
fprintf(fid,'Number of parameters to estimate: %0.0f\n',length(theta));
fprintf(fid,'Total number of parameters:       %0.0f\n\n',size(ParameterNames.all,1));
%
% check the length of parameter names string
%
ns = size(ParameterNames.estimated,2);
if ns<20;
   add_str = SpaceStr(20-ns);
else;
   add_str = '';
end;
prt_str_1 = ['%s%s    ' prt_val '    ' prt_val '    ' prt_val];
prt_str_2 = ['        %s(%s    ' prt_val '    ' prt_val];
%
% Calculate an approximation of the inverse Hessian for theta, the original parameters, using the
% inverse Hessian at the mode for phi, the transformed parameters, and the partial derivatives of
% theta wrt phi
%
thetaInverseHessian = ThetaPhiPartial*InverseHessian*ThetaPhiPartial;
%
fprintf(fid,'============================================================================\n');
fprintf(fid,'                            Original Parameters                             \n');
fprintf(fid,'      parameter          theta mode        st dev         init val\n');
if (isempty(PriorDist.truncnormal)==1)&(isempty(PriorDist.invgamma)==1)&(isempty(PriorDist.gamma)==1)&(isempty(PriorDist.beta)==1)&(isempty(PriorDist.student)==1)&(isempty(PriorDist.pareto)==1);
   fprintf(fid,'     (prior dist        parameter 1     parameter 2)\n');
else;
   if isempty(PriorDist.beta)==1;
      fprintf(fid,'     (prior dist        parameter 1     parameter 2     parameter 3)\n');
   else;
      fprintf(fid,'     (prior dist        parameter 1     parameter 2     parameter 3\n');
      fprintf(fid,'                                                        parameter 4)\n');
   end;
end;
%
n_gamma = 0;
n_beta = 0;
n_normal = 0;
n_invgamma = 0;
n_truncnormal = 0;
n_uniform = 0;
n_student = 0;
n_cauchy = 0;
n_logistic = 0;
n_gumbel = 0;
n_pareto = 0;
for i=1:length(thetaMode);
   if thetaDist(i)==1;
      dist_str = 'gamma';
      dist_add = SpaceStr(6);
      n_gamma = n_gamma+1;
      prior_val = PriorDist.gamma(n_gamma,:);
      if length(prior_val)<3;
         prior_val = [prior_val 0];
      end;
   elseif thetaDist(i)==2;
      dist_str = 'beta';
      dist_add = SpaceStr(7);
      n_beta = n_beta+1;
      prior_val = PriorDist.beta(n_beta,:);
      if length(prior_val)<3;
         prior_val = [prior_val 0 1];
      end;
   elseif thetaDist(i)==3;
      dist_str = 'normal';
      dist_add = SpaceStr(5);
      n_normal = n_normal+1;
      prior_val = PriorDist.normal(n_normal,:);
   elseif thetaDist(i)==4;
      dist_str = 'invgamma';
      dist_add = SpaceStr(3);
      n_invgamma = n_invgamma+1;
      prior_val = PriorDist.invgamma(n_invgamma,:);
      if length(prior_val)<3;
         prior_val = [prior_val 0];
      end;
   elseif thetaDist(i)==5;
      dist_str = 'truncnormal';
      dist_add = '';
      n_truncnormal = n_truncnormal+1;
      prior_val = PriorDist.truncnormal(n_truncnormal,:);
   elseif thetaDist(i)==6;
      dist_str = 'uniform';
      dist_add = SpaceStr(4);
      n_uniform = n_uniform+1;
      prior_val = PriorDist.uniform(n_uniform,:);
   elseif thetaDist(i)==7;
      dist_str = 'student';
      dist_add = SpaceStr(4);
      n_student = n_student+1;
      prior_val = PriorDist.student(n_student,:);
   elseif thetaDist(i)==8;
      dist_str = 'cauchy';
      dist_add = SpaceStr(5);
      n_cauchy = n_cauchy+1;
      prior_val = PriorDist.cauchy(n_cauchy,:);
   elseif thetaDist(i)==9;
      dist_str = 'logistic';
      dist_add = SpaceStr(3);
      n_logistic = n_logistic+1;
      prior_val = PriorDist.logistic(n_logistic,:);
   elseif thetaDist(i)==10;
      dist_str = 'gumbel';
      dist_add = SpaceStr(5);
      n_gumbel = n_gumbel+1;
      prior_val = PriorDist.gumbel(n_gumbel,:);
   elseif thetaDist(i)==11;
      dist_str = 'pareto';
      dist_add = SpaceStr(5);
      n_pareto = n_pareto+1;
      prior_val = PriorDist.pareto(n_pareto,:);
   end;
   fprintf(fid,'----------------------------------------------------------------------------\n');
   fprintf(fid,[prt_str_1 '\n'],add_str,ParameterNames.estimated(i,:),thetaMode(i),sqrt(thetaInverseHessian(i,i)),theta(i));
   if max(isnan(phiFittedStd))==0;
      fprintf(fid,['                                        ' prt_val '\n'],ThetaPhiPartial(i,i)*phiFittedStd(i));
   end;
   if (strcmp(dist_str,'truncnormal')==0)&(strcmp(dist_str,'invgamma')==0)&(strcmp(dist_str,'gamma')==0)&(strcmp(dist_str,'beta')==0)&(strcmp(dist_str,'student')==0)&(strcmp(dist_str,'pareto')==0)&(strcmp(dist_str,'logistic')==0);
      fprintf(fid,[prt_str_2 ')\n'],dist_add,dist_str,prior_val(1,1),prior_val(1,2));
   else;
      if strcmp(dist_str,'beta')==0;
         fprintf(fid,[prt_str_2 '    ' prt_val ')\n'],dist_add,dist_str,prior_val(1,1),prior_val(1,2),prior_val(1,3));
      else;
         fprintf(fid,[prt_str_2 '    ' prt_val '\n'],dist_add,dist_str,prior_val(1,1),prior_val(1,2),prior_val(1,3));
         fprintf(fid,['                                                        ' prt_val ')\n'],prior_val(1,4));
      end;
   end;
end;
if max(isnan(phiFittedStd))==0;
   AddNote = ' partial derivatives. The second standard deviation\n      is based on fitting a quadratic to the evaluated log system prior and\n      using partial derivatives.\n\n';
else;
   AddNote = ' partial derivatives.\n\n';
end;
%
fprintf(fid,'============================================================================\n\n');
fprintf(fid,'NOTE: The standard deviation is computed from the inverse Hessian at the\n');
fprintf(fid,['      posterior mode. It has been computed from the inverse Hessian of phi\n      at the mode using' AddNote]);
%
% Print the calibrated parameters and their values
%
if isempty(thetaCalibrated)==0;
   ns = size(ParameterNames.calibrated,2);
   if ns<20;
      cal_add_str = SpaceStr(20-ns);
   else;
      cal_add_str = '';
   end;
   %
   fprintf(fid,'==============================================================\n');
   fprintf(fid,'        Calibrated Parameters\n');
   fprintf(fid,'     parameter              value\n');
   prt_str = ['%s%s    ' prt_val '\n'];
   for i=1:length(thetaCalibrated);
      fprintf(fid,'--------------------------------------------------------------\n');
      fprintf(fid,prt_str,cal_add_str,ParameterNames.calibrated(i,:),thetaCalibrated(i));
   end;
   fprintf(fid,'==============================================================\n\n');
end;
%
% print any additional parameters values
%
if isempty(thetaAdditionalInitial)==0;
   ns = size(ParameterNames.additional,2);
   if ns<20;
      upd_add_str = SpaceStr(20-ns);
   else;
      upd_add_str = '';
   end;
   %
   fprintf(fid,'==============================================================\n');
   fprintf(fid,'               Additional Parameters\n');
   fprintf(fid,'     parameter              mode         init value\n');
   prt_str = ['%s%s    ' prt_val '    ' prt_val '\n'];
   for i=1:length(thetaAdditionalInitial);
      fprintf(fid,'--------------------------------------------------------------\n');
      fprintf(fid,prt_str,upd_add_str,ParameterNames.additional(i,:),thetaAdditionalMode(i),thetaAdditionalInitial(i));
   end;
   fprintf(fid,'==============================================================\n\n');
end;
%
% Some information about the filename and its location
%
%fprintf(fid,'================================================================================\n\n');
fprintf(fid,'Directory for file: %s\n',GetPath(ResultsFile));
fprintf(fid,'Name of file:       %s\n',GetFilename(ResultsFile));
fprintf(fid,'Output created on:  %s\n\n',datestr(now,0));
fclose(fid);

%
% end of PrintSystemPriorModeResults.m
%
