function PrintSystemPriorSamplingSummary(PriorSampleTextFile,DSGEModel,CurrINI,ParameterNames,PriorDist,thetaDist,ParData,thetaMargMode,NumPriorDraws,NumMeasureErrors,UseNormalProposal,ScaleInitial,ScaleProp,PriorSampleTime,AcceptRate,LogMargsMH,CovProb,NWStdErrMH,LogMargLikeSystemLaplace,StepLength,MaxCorr,thetaInit)
% PrintSystemPriorSamplingSummary: Writes summarey information about the prior sampling with a system prior for
%                                  the original parameters.
%
% USAGE:
%
%       PrintSystemPriorSamplingSummary(PriorSampleTextFile,DSGEModel,CurrINI,ParameterNames,PriorDist,thetaDist,ParData,...
%                                       thetaMargMode,NumPriorDraws,NumMeasureErrors,UseNormalProposal,ScaleInitial,...
%                                       ScaleProp,PriorSampleTime,AcceptRate,LogMargsMH,CovProb,NWStdErrMH,...
%                                       LogMargLikeSystemLaplace,StepLength,MaxCorr,thetaInit)
%
% REQUIRED INPUT: PriorSampleTextFile (string) with the full path and name of the text output file.
%
%                 DSGEModel (structure) whose fields contain all required information
%                           about the model, such as input files, variable and sample
%                           data. Actual observations are not stored in this structure.
%
%                 CurrINI (structure) with initialization information.
%
%                 ParameterNames (structure) with fields "calibrated", "beta", "gamma", "normal", "invgamma", "truncnormal",
%                           "uniform", "student", "cauchy", "logistic", "gumbel", "pareto", "all" and "estimated". Each
%                           field returns a string matrix with the parameter names.
%
%                 PriorDist (structure) with fields "beta", "gamma", "normal", "invgamma", "truncnormal", "uniform",
%                           "student", "cauchy", "logistic", "gumbel", and "pareto", where the values are matrices with the
%                           parameter values for the distributions. In the case of the normal, beta, gamma, logistic 
%                           and gumbel these parameter values are the mean and the standard deviation. For the invgamma
%                           it is the s and q parameters, where q is a positive integer (degrees of freedom). For the left
%                           truncated normal there's a third parameter, the lower bound. For the uniform the parameter
%                           values are the lower and the upper bound. For the student-t they are the location (mode),
%                           the scale and the degrees of freedom parameter, while for the Cauchy only the location
%                           and scale parameters are needed. For the Pareto the parameter values are for the mode
%                           and the shape parameter.
%
%                 thetaDist (vector) with integers 1 if gamma, 2 if beta, 3 if normal, 4 if invgamma, 5 if
%                           truncnormal, 6 if uniform, 7 if student, 8 if cauchy, 9 if logistic, 10 if gumbel,
%                           and 11 if pareto.
%
%                 ParData (structure), with fields "SampleMean" containing the sample
%                           mean of the different parameters, "SampleStd" with the sample
%                           standard deviations, and "SampleMedian" with the sample median
%                           parameter values. The field "SampleStdErr" contains numerical
%                           standard errors of the SampleMean estimate.
%
%                 thetaMargMode (vector) with the marginal posterior mode estimates of the original
%                           parameters.
%
%                 NumPriorDraws (integer) with the number of prior draws.
%
%                 NumMeasureErrors (integer) with the number of measurement errors in the model.
%
%                 UseNormalProposal (boolean) which is unity if the random-walk Metropolis sampler
%                           with a normal proposal is used and 0 if the marginal priors are used
%                           as proposal densities.
%
%                 ScaleInitial (non-negative real number), giving the scale factor for the initial phi value (c0)
%
%                 ScaleProp (positive real number), giving the scale factor for the proposal density (c).
%
%                 PriorSampleTime (time) with the length it took the posterior sampler to finish from where
%                           it started.
%
%                 AcceptRate (positive real number), giving the acceptance rate for the RWM posterior sampler
%                           and the average number of log posterior evaluations for the slice sampler.
%
%                 LogMargsMH (matrix) with the log marginal likelihood of the system prior using Geweke's
%                           modified harmonic mean estimator. First column is number of draws, while the remaining
%                           columns contain the estimated values for the CovProb values.
%
%                 CovProb (vector) with coverage probabilities.
%
%                 NWStdErrMH (vector) with numerical standard errors for the log marginal likelihood estimated using
%                           Geweke's modified harmonic mean for the full sample only.
%
%                 LogMargLikeSystemLaplace (scalar) with the log marginal likelihood of the system prior using the
%                           Laplace approximation.
%
%                 StepLength (positive scalar) with the step length that is used for the finite difference estimator
%                           of the inverse Hessian at the posterior mode.
%
%                 MaxCorr (scalar) with the maximum absolute correlation scale factor.
%
%                 thetaInit (vector)  with the initial values for the original parameters.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   DG-R/MPR
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2017 European Central Bank.
%
%                       First version: May 30, 2017.
%                        This version: May 31, 2017.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 31-05-2017: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize
%
NumParam = length(ParData.SampleMean);
fid = fopen(PriorSampleTextFile,'wt');
%
% the header information
%
fprintf(fid,'********************************************************************************\n');
fprintf(fid,'*                                                                              *\n');
fprintf(fid,'*  S Y S T E M   P R I O R   S A M P L I N G   W I T H   Y A D A   U S I N G   *\n');
if UseNormalProposal==1;
   fprintf(fid,'*   T H E   R A N D O M - W A L K   M E T R O P O L I S   A L G O R I T H M    *\n');
   fprintf(fid,'*         W I T H   A   N O R M A L   P R O P O S A L   D E N S I T Y          *\n');
else;
   fprintf(fid,'*      T H E   M E T R O P O L I S - H A S T I N G S   A L G O R I T H M       *\n');
   fprintf(fid,'*    W I T H   T H E   M A R G I N A L   P R I O R   A S   P R O P O S A L     *\n');
end;
fprintf(fid,'*                                                                              *\n');
fprintf(fid,'********************************************************************************\n\n');
%
% Model info
%
if DSGEModel.ModelSolver==1;
   SolverStr = 'Anderson & Moore (AiM)';
elseif DSGEModel.ModelSolver==2;
   SolverStr = 'Klein (solab)';
elseif DSGEModel.ModelSolver==3;
   SolverStr = 'Sims (gensys)';
end;
fprintf(fid,'Name of model:                    %s\n',DSGEModel.NameOfModel);
fprintf(fid,'DSGE model solver:                %s\n',SolverStr);
fprintf(fid,'Data construction file:           %s\n',GetFilename(DSGEModel.DataConstructionFile));
fprintf(fid,'Measurement equation file:        %s\n',GetFilename(DSGEModel.MeasurementEquationFile));
fprintf(fid,'Marginal prior dist. data file:   %s\n',GetFilename(DSGEModel.PriorFile));
fprintf(fid,'Sheet name:                       %s\n',DSGEModel.PriorFileSheet);
fprintf(fid,'System prior file:                %s\n',GetFilename(DSGEModel.SystemPriorFile));
if FileExist(DSGEModel.UpdateParameterFile)==1;
   fprintf(fid,'Update parameters file:           %s\n',GetFilename(DSGEModel.UpdateParameterFile));
end;
if FileExist(DSGEModel.InitializeParameterFile)==1;
   fprintf(fid,'Initialize parameters file:       %s\n',GetFilename(DSGEModel.InitializeParameterFile));
end;
fprintf(fid,'AiM model file:                   %s\n\n',GetFilename(DSGEModel.AIMFile));
%
fprintf(fid,'Number of observed variables:     %0.0f\n',min(size(DSGEModel.Y)));
fprintf(fid,'Number of exogenous variables:    %0.0f\n',min(size(DSGEModel.X)));
fprintf(fid,'Number of state variables:        %0.0f\n',size(DSGEModel.StateVariableNames,1));
fprintf(fid,'Number of state shocks:           %0.0f\n',size(DSGEModel.StateShockNames,1));
fprintf(fid,'Number of measurement errors:     %0.0f\n\n',NumMeasureErrors);
%
% observed variable names, state variable names and state shocks
%
VarNames = StrWrap(vecstr(DSGEModel.VariableNames,' '),40);
fprintf(fid,'Names of observed variables:      %s\n',VarNames(1,:));
for i=2:size(VarNames,1);
   if strcmp(StringTrim(VarNames(i,:)),'')==0;
      fprintf(fid,'                                  %s\n',VarNames(i,:));
   end;
end;
if min(size(DSGEModel.X))>0;
   XVarNames = StrWrap(vecstr(DSGEModel.XVariableNames,' '),40);
   fprintf(fid,'Names of exogenous variables:     %s\n',XVarNames(1,:));
   for i=2:size(XVarNames,1);
      if strcmp(StringTrim(XVarNames(i,:)),'')==0;
         fprintf(fid,'                                  %s\n',XVarNames(i,:));
      end;
   end;
end;
StateVarNames = StrWrap(vecstr(DSGEModel.StateVariableNames,' '),40);
fprintf(fid,'Names of state variables:         %s\n',StateVarNames(1,:));
for i=2:size(StateVarNames,1);
   if strcmp(StringTrim(StateVarNames(i,:)),'')==0;
      fprintf(fid,'                                  %s\n',StateVarNames(i,:));
   end;
end;
StateShockNames = StrWrap(vecstr(DSGEModel.StateShockNames,' '),40);
fprintf(fid,'Names of the state shocks:        %s\n',StateShockNames(1,:));
for i=2:size(StateShockNames,1);
   if strcmp(StringTrim(StateShockNames(i,:)),'')==0;
      fprintf(fid,'                                  %s\n',StateShockNames(i,:));
   end;
end;
%
DivStr = MultiCharStr('=',80);
%
% prior sampler information
%
fprintf(fid,'\n********************************************************************************\n');
fprintf(fid,'*                                                                              *\n');
fprintf(fid,'*                P R I O R   D R A W S   I N F O R M A T I O N                 *\n');
fprintf(fid,'*                                                                              *\n');
fprintf(fid,'********************************************************************************\n\n');
%
fprintf(fid,'Number of prior draws:                 %0.0f\n',NumPriorDraws);
if UseNormalProposal==1;
   if DSGEModel.InverseHessianEstimator==1;
      InvHessianTxt = 'Optimization routine';
   elseif DSGEModel.InverseHessianEstimator==2;
      InvHessianTxt = 'Quadratic modification';
   elseif DSGEModel.InverseHessianEstimator==3;
      InvHessianTxt = 'Finite difference';
   else;
      InvHessianTxt = 'Own matrix';
   end;
   fprintf(fid,'Inverse Hessian estimator:             %s\n',InvHessianTxt);
   if DSGEModel.InverseHessianEstimator==3;
      fprintf(fid,'Step length:                           %s\n',StepLength);
   end;
   fprintf(fid,'Max absolute correlation scale factor: %0.4f\n',MaxCorr);
   fprintf(fid,'Scale factor for initial phi density:  %0.4f\n',ScaleInitial);
   fprintf(fid,'Scale factor for proposal phi density: %0.4f\n',ScaleProp);
else;
   fprintf(fid,'Marginal independent priors as prorposal density\n');
end;
fprintf(fid,'Prior sampling time:                   %s (hrs:min:sec)\n',SecondsToHoursStr(PriorSampleTime));
fprintf(fid,'Acceptance rate:                       %0.4f\n\n',AcceptRate);
prt_val = ['%' num2str(9+CurrINI.decimals,'%0.0f') '.' num2str(CurrINI.decimals,'%0.0f') 'f'];
%
% write results for log marginal likelihoods
%
fprintf(fid,'********************************************************************************\n');
fprintf(fid,'*                                                                              *\n');
fprintf(fid,'*         M A R G I N A L   L I K E L I H O O D   E S T I M A T I O N          *\n');
fprintf(fid,'*                                                                              *\n');
fprintf(fid,'********************************************************************************\n\n');
if isempty(LogMargsMH)==0;
   %
   % Geweke's modified harmonic mean estimator
   %
   prt_str = 'Geweke''s Harmonic Mean';
   top_str = '                      ';
   std_str = 'Num. standard errors: ';
   for i=1:length(CovProb);
      top_str = [top_str '        ' num2str(CovProb(i)*100,'%0.0f') ' percent'];
      prt_str = [prt_str '   ' prt_val];
      std_str = [std_str '   ' prt_val];
   end;
   TopStr = MultiCharStr('=',length(top_str));
   MidStr = MultiCharStr('-',length(top_str));
   fprintf(fid,'%s\n',TopStr);
   fprintf(fid,'                              Coverage probabilities for the estimator\n');
   fprintf(fid,[top_str '\n']);
   fprintf(fid,'%s\n',MidStr);
   fprintf(fid,[prt_str '\n'],LogMargsMH(size(LogMargsMH,1),2:size(LogMargsMH,2))');
   if isempty(NWStdErrMH)==0;
      fprintf(fid,[std_str '\n'],NWStdErrMH');
   end;
   fprintf(fid,[MidStr '\n']);
   fprintf(fid,['Mean:                    ' prt_val '\n'],sum(LogMargsMH(1,2:size(LogMargsMH,2)))/length(CovProb));
   fprintf(fid,['Median:                  ' prt_val '\n'],median(LogMargsMH(1,2:size(LogMargsMH,2))));
   fprintf(fid,'%s\n\n',TopStr);
end;
fprintf(fid,'Laplace approximation:         %0.6f\n\n',LogMargLikeSystemLaplace);
fprintf(fid,'NOTE: The Laplace approximation of the log marginal likelihood of\n      the system prior is computed from the prior mode of phi, the\n      transformed parameters, and the finite difference estimator\n      of the inverse Hessian at the mode.\n\n');
fprintf(fid,'NOTE: If the harmonic mean estimates differ greatly from the Laplace\n      approximation estimate, you probably need more prior draws.\n\n');
%
% deal with the parameters
%
fprintf(fid,'********************************************************************************\n');
fprintf(fid,'*                                                                              *\n');
fprintf(fid,'*                             P A R A M E T E R S                              *\n');
fprintf(fid,'*                                                                              *\n');
fprintf(fid,'********************************************************************************\n\n');
%
fprintf(fid,'Number of parameters to estimate: %0.0f\n',NumParam);
if size(ParameterNames.additional,1)>0;
   fprintf(fid,'Number of update parameters:      %0.0f\n',size(ParameterNames.additional,1));
end;
if size(ParameterNames.calibrated,1)>0;
   fprintf(fid,'Number of calibrated parameters:  %0.0f\n',size(ParameterNames.calibrated,1));
end;
fprintf(fid,'Total number of parameters:       %0.0f\n\n',size(ParameterNames.all,1));
ns = size(ParameterNames.estimated,2);
if ns<20;
   add_str = SpaceStr(20-ns);
else;
   add_str = '';
end;
prt_val = ['%' num2str(6+CurrINI.decimals,'%0.0f') '.' num2str(CurrINI.decimals,'%0.0f') 'f'];
prt_str = ['%s%s   ' prt_val '   ' prt_val '   ' prt_val '   |   ' prt_val '   ' prt_val '   ' prt_val '   ' prt_val];
NewDivStr = MultiCharStr('-',129);
fprintf(fid,'=================================================================================================================================\n');
fprintf(fid,'                                                       Original Parameters\n');
fprintf(fid,'           parameter       mean          median          mode       |      st. dev.   auto st. dev.       RNE             IF\n');
fprintf(fid,'                     [std err of mean]\n');
fprintf(fid,'                       initial value\n');
if (isempty(PriorDist.truncnormal)==1)&(isempty(PriorDist.invgamma)==1)&(isempty(PriorDist.gamma)==1)&(isempty(PriorDist.beta)==1)&(isempty(PriorDist.student)==1)&(isempty(PriorDist.pareto)==1)&(isempty(PriorDist.logistic)==1);
   fprintf(fid,'         (prior dist    parameter 1    parameter 2)\n');
else;
   if isempty(PriorDist.beta)==1;
      fprintf(fid,'         (prior dist    parameter 1    parameter 2    parameter 3)\n');
   else;
      fprintf(fid,'         (prior dist    parameter 1    parameter 2    parameter 3\n');
      fprintf(fid,'                                                      parameter 4)\n');
   end;
end;
%
n_gamma = 0;
n_beta = 0;
n_normal = 0;
n_invgamma = 0;
n_truncnormal = 0;
n_uniform = 0;
n_student = 0;
n_cauchy = 0;
n_logistic = 0;
n_gumbel = 0;
n_pareto = 0;
for i=1:NumParam;
   if i>1;
      fprintf(fid,'%s\n',NewDivStr);
   else;
      fprintf(fid,'=================================================================================================================================\n');
   end;
   if thetaDist(i)==1;
      dist_str = 'gamma';
      dist_add = SpaceStr(6);
      n_gamma = n_gamma+1;
      prior_val = PriorDist.gamma(n_gamma,:);
      if length(prior_val)<3;
         prior_val = [prior_val 0];
      end;
   elseif thetaDist(i)==2;
      dist_str = 'beta';
      dist_add = SpaceStr(7);
      n_beta = n_beta+1;
      prior_val = PriorDist.beta(n_beta,:);
      if length(prior_val)<3;
         prior_val = [prior_val 0 1];
      end;
   elseif thetaDist(i)==3;
      dist_str = 'normal';
      dist_add = SpaceStr(5);
      n_normal = n_normal+1;
      prior_val = PriorDist.normal(n_normal,:);
   elseif thetaDist(i)==4;
      dist_str = 'invgamma';
      dist_add = SpaceStr(3);
      n_invgamma = n_invgamma+1;
      prior_val = PriorDist.invgamma(n_invgamma,:);
      if length(prior_val)<3;
         prior_val = [prior_val 0];
      end;
   elseif thetaDist(i)==5;
      dist_str = 'truncnormal';
      dist_add = '';
      n_truncnormal = n_truncnormal+1;
      prior_val = PriorDist.truncnormal(n_truncnormal,:);
   elseif thetaDist(i)==6;
      dist_str = 'uniform';
      dist_add = SpaceStr(4);
      n_uniform = n_uniform+1;
      prior_val = PriorDist.uniform(n_uniform,:);
   elseif thetaDist(i)==7;
      dist_str = 'student';
      dist_add = SpaceStr(4);
      n_student = n_student+1;
      prior_val = PriorDist.student(n_student,:);
   elseif thetaDist(i)==8;
      dist_str = 'cauchy';
      dist_add = SpaceStr(5);
      n_cauchy = n_cauchy+1;
      prior_val = PriorDist.cauchy(n_cauchy,:);
   elseif thetaDist(i)==9;
      dist_str = 'logistic';
      dist_add = SpaceStr(3);
      n_logistic = n_logistic+1;
      prior_val = PriorDist.logistic(n_logistic,:);
   elseif thetaDist(i)==10;
      dist_str = 'gumbel';
      dist_add = SpaceStr(5);
      n_gumbel = n_gumbel+1;
      prior_val = PriorDist.gumbel(n_gumbel,:);
   elseif thetaDist(i)==11;
      dist_str = 'pareto';
      dist_add = SpaceStr(5);
      n_pareto = n_pareto+1;
      prior_val = PriorDist.pareto(n_pareto,:);
   end;
   fprintf(fid,[prt_str '\n'],add_str,ParameterNames.estimated(i,:),ParData.SampleMean(i),ParData.SampleMedian(i),thetaMargMode(i),ParData.SampleStd(i),ParData.SampleStdAuto(i),(ParData.SampleStd(i)/ParData.SampleStdAuto(i))^2,(ParData.SampleStdAuto(i)/ParData.SampleStd(i))^2);
   fprintf(fid,['                      [' prt_val ']\n'],ParData.SampleStdErr(i));
   fprintf(fid,['                       ' prt_val '\n'],thetaInit(i));
   if (strcmp(dist_str,'truncnormal')==0)&(strcmp(dist_str,'invgamma')==0)&(strcmp(dist_str,'gamma')==0)&(strcmp(dist_str,'beta')==0)&(strcmp(dist_str,'student')==0)&(strcmp(dist_str,'pareto')==0)&(strcmp(dist_str,'logistic')==0);
      fprintf(fid,['        %s(%s   ' prt_val '   ' prt_val ')\n'],dist_add,dist_str,prior_val(1,1),prior_val(1,2));
   else;
      if strcmp(dist_str,'beta')==0;
         fprintf(fid,['        %s(%s   ' prt_val '   ' prt_val '   ' prt_val ')\n'],dist_add,dist_str,prior_val(1,1),prior_val(1,2),prior_val(1,3));
      else;
         fprintf(fid,['        %s(%s   ' prt_val '   ' prt_val '   ' prt_val '\n'],dist_add,dist_str,prior_val(1,1),prior_val(1,2),prior_val(1,3));
         fprintf(fid,['                                                     ' prt_val ')\n'],prior_val(1,4));
      end;
   end;
end;
fprintf(fid,'=================================================================================================================================\n\n');
%
fprintf(fid,'NOTE: RNE measures the relative numerical efficiency. It is defined as the ratio between the variance\n');
fprintf(fid,'      of the draws under the assumption that they are iid and the variance of the draws when autocorrelation\n');
fprintf(fid,'      is taken into account. This value is equal to the ratio between the previous two columns squared. The\n');
fprintf(fid,'      inverse of the RNE is sometimes called the inefficiency factor (IF) and is equal to 1+2*sum(1,K) rho(k),\n');
fprintf(fid,'      where rho(k) is the autocorrelation at lag k, and K is %0.0f.\n\n',floor((NumPriorDraws)^(1/2.01)));
%
% print parameter quantiles
%
prt_str = ['%s%s   ' prt_val '   ' prt_val '   ' prt_val '   ' prt_val '   ' prt_val '   ' prt_val '   ' prt_val '   ' prt_val '   ' prt_val];
fprintf(fid,'===========================================================================================================================================================\n');
fprintf(fid,'                                                        Prior Quantiles for the Original Parameters                                                      \n');
fprintf(fid,'           parameter        2.5%%           5%%            10%%            20%%            50%%            80%%            90%%            95%%            97.5%%\n');
for i=1:NumParam;
   fprintf(fid,'-----------------------------------------------------------------------------------------------------------------------------------------------------------\n');
   fprintf(fid,[prt_str '\n'],add_str,ParameterNames.estimated(i,:),ParData.Quantiles(i,:)');
end;
fprintf(fid,'===========================================================================================================================================================\n\n');
%
% Some information about the filename and its location
%
fprintf(fid,'Directory for file: %s\n',GetPath(PriorSampleTextFile));
fprintf(fid,'Name of file:       %s\n',GetFilename(PriorSampleTextFile));
fprintf(fid,'Output created on:  %s\n\n',datestr(now,0));
fclose(fid);

%
% end of PrintSystemPriorSamplingSummary.m
%
