function thetaRnd = DSGEPriorSampling(PriorDist,thetaDist,LowerBound,NumPriorDraws,HeaderStr,DSGEModel,CurrINI)
% DSGEPriorSampling: Draws values of the parameters to be estimated from the prior
%                    distribution.
%
% USAGE:
%
%       thetaRnd = DSGEPriorSampling(PriorDist,thetaDist,LowerBound,NumPriorDraws,HeaderStr,DSGEModel,CurrINI)
%
% REQUIRED INPUT:  PriorDist (structure) with fields "beta", "gamma", "normal", "invgamma", "truncnormal", "uniform",
%                            "student", "cauchy", "logistic", "gumbel", and "pareto", where the values are matrices with the
%                            parameter values for the distributions. In the case of the normal, beta, gamma, logistic 
%                            and gumbel these parameter values are the mean and the standard deviation. For the invgamma
%                            it is the s and q parameters, where q is a positive integer (degrees of freedom). For the left
%                            truncated normal there's a third parameter, the lower bound. For the uniform the parameter
%                            values are the lower and the upper bound. For the student-t they are the location (mode),
%                            the scale and the degrees of freedom parameter, while for the Cauchy only the location
%                            and scale parameters are needed. For the Pareto the parameter values are for the mode
%                            and the shape parameter.
%
%                  thetaDist (vector) with integers 1 if gamma, 2 if beta, 3 if normal, 4 if invgamma, 5 if
%                            truncnormal, 6 if uniform, 7 if student, 8 if cauchy, 9 if logistic, 10 if gumbel,
%                            and 11 if pareto.
%
%                  LowerBound (vector) with lower bound values for the theta vector.
%
%                  NumPriorDraws (integer) with the number of draws to take from the prior distribution.
%
%                  DSGEModel (structure) whose fields contain all required information about the model,
%                            such as input files, variable and sample data.
%
%                  CurrINI (structure) with initialization information.
%
% OPTIONAL OUTPUT: thetaRnd (matrix) of dimension NumParamxNumPriorDraws with the draws from the prior
%                            distribution of the DSGE model parameters. If the variable is used, then
%                            prior draws are not stored to disk.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   DG-R/MPR
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2017 European Central Bank.
%
%                       First version: March 22, 2009.
%                        This version: May 24, 2017.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 31-03-2009: Draws greater than or less than 6 times the scale factor relative to
%               the location parameter are now discarded for the Student-t and Cauchy
%               distributions. Made sure that the origin parameter of the Pareto
%               distribution is taken into account.
%
% * 02-04-2009: Took into account that the function "LogisticRndFcn" now accepts a
%               shape parameter. Added code for the Gumbel distribution.
%
% * 05-01-2010: Updated the documentation.
%
% * 18-02-2010: Added the optional "thetaRnd" output variable. Also, the wait dialog should
%               not show a cancel button.
%
% * 17-03-2010: Updated the documentation.
%
% * 05-01-2011: Updated the documentation.
%
% * 12-01-2011: Added support for unix and mac paths.
%
% * 14-12-2011: Added the "HeaderStr" input variable.
%
% * 04-01-2012: Updated the documentation.
%
% * 03-01-2013: Updated the documentation.
%
% * 27-01-2014: Updated the documentation.
%
% * 12-01-2015: Updated the documentation.
%
% * 20-01-2016: Updated the documentation.
%
% * 10-02-2017: Updated the documentation.
%
% * 24-05-2017: The function better allows for having just 1 draw from the marginal prior.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

if nargout==1;
   thetaRnd = [];
end;
%
% create the output directory
%
if nargout~=1;
   stat = MakeDir(DSGEModel.OutputDirectory,'priordraws');
   if stat~=1;
      if ispc==1;
         txt = ['YADA was for some reason unable to create the directory "' DSGEModel.OutputDirectory '\priordraws". The prior sampling has therefore been aborted.'];
      else;
         txt = ['YADA was for some reason unable to create the directory "' DSGEModel.OutputDirectory '/priordraws". The prior sampling has therefore been aborted.'];
      end;
      About(txt,'information','YADA - Directory Creation Problem',180,500,CurrINI);
      return;
   end;
end;
%
% draw from the prior distributions of the parameters to use
%
if NumPriorDraws>1;
   if DSGEModel.RandomNumberValue==1;
      %
      % intialize random number generators based on fixed state/seed
      %
      rand('state',0);
      randn('state',0);
   else;
      %
      % initialize random number generators based on clock based state/seed
      %
      rand('state',sum(100*clock));
      randn('state',sum(100*clock));
   end;
end;
NumGamma = 0;
NumBeta = 0;
NumNormal = 0;
NumInvGamma = 0;
NumLTN = 0;
NumUniform = 0;
NumStudent = 0;
NumCauchy = 0;
NumLogistic = 0;
NumGumbel = 0;
NumPareto = 0;
%
NumParam = length(thetaDist);
thetaDraws = zeros(NumParam,NumPriorDraws);
%
% open a wait dialog
%
if NumPriorDraws>=100;
   txt = ['Please wait while YADA draws from the prior distribution. Sampling started at: ' StringTrim(datestr(now,14)) '.'];
   WaitHandle = WaitDLG(txt,'information',[HeaderStr ' - ' int2str(NumPriorDraws) ' Draws'],500,200,CurrINI,0);
   WaitControls = get(WaitHandle,'UserData');
   drawnow;
end;
%
for i=1:NumParam;
   %
   if thetaDist(i)==1;
      %
      % gamma dist
      %
      NumGamma = NumGamma+1;
      thetaDraws(i,:) = LowerBound(i)+GammaRndFcn(PriorDist.gamma_ab(NumGamma,1),PriorDist.gamma_ab(NumGamma,2),NumPriorDraws);
   elseif thetaDist(i)==2;
      %
      % beta dist
      %
      NumBeta = NumBeta+1;
      if size(PriorDist.beta,2)==4;
         %
         % get the lower and upper bounds (if available from PriorDist.)
         %
         c = PriorDist.beta(NumBeta,3);
         d = PriorDist.beta(NumBeta,4);
      else;
         c = 0;
         d = 1;
      end;
      thetaDraws(i,:) = BetaRndFcn(PriorDist.beta_ab(NumBeta,1),PriorDist.beta_ab(NumBeta,2),c,d,NumPriorDraws);
   elseif thetaDist(i)==3;
      %
      % normal
      %
      NumNormal = NumNormal+1;
      thetaDraws(i,:) = NormalRndFcn(PriorDist.normal(NumNormal,1),PriorDist.normal(NumNormal,2),NumPriorDraws);
   elseif thetaDist(i)==4;
      %
      % inverted gamma
      %
      NumInvGamma = NumInvGamma+1;
      thetaDraws(i,:) = LowerBound(i)+InvGammaRndFcn(PriorDist.invgamma(NumInvGamma,1)-LowerBound(i),PriorDist.invgamma(NumInvGamma,2),NumPriorDraws);
   elseif thetaDist(i)==5;
      %
      % left truncated normal
      %
      NumLTN = NumLTN+1;
      thetaDraws(i,:) = LTNormalRndFcn(PriorDist.truncnormal(NumLTN,1),PriorDist.truncnormal(NumLTN,2),PriorDist.truncnormal(NumLTN,3),NumPriorDraws);
   elseif thetaDist(i)==6;
      %
      % uniform
      %
      NumUniform = NumUniform+1;
      thetaDraws(i,:) = UniformRndFcn(PriorDist.uniform(NumUniform,1),PriorDist.uniform(NumUniform,2),NumPriorDraws);
   elseif thetaDist(i)==7;
      %
      % Student-t
      %
      NumStudent = NumStudent+1;
      Numz = 0;
      Oldz = [];
      lb = PriorDist.student(NumStudent,1)-(6*PriorDist.student(NumStudent,2));
      ub = PriorDist.student(NumStudent,1)+(6*PriorDist.student(NumStudent,2));
      while Numz<NumPriorDraws;
         z = StudentTAltRndFcn(PriorDist.student(NumStudent,1),PriorDist.student(NumStudent,2),PriorDist.student(NumStudent,3),2*NumPriorDraws);
         %
         % remove value of z that are too big or too small
         %
         if sum(z>=lb)<length(z);
            z = z(z>=lb);
         end;
         if sum(z<=ub)<length(z);
            z = z(z<=ub);
         end;
         Oldz = [Oldz z];
         z = Oldz;
         if length(z)>NumPriorDraws;
            z = z(1:NumPriorDraws);
         end;
         Numz = length(z);
      end;
      thetaDraws(i,:) = z;
   elseif thetaDist(i)==8;
      %
      % Cauchy
      %
      NumCauchy = NumCauchy+1;
      Numz = 0;
      Oldz = [];
      lb = PriorDist.cauchy(NumCauchy,1)-(6*PriorDist.cauchy(NumCauchy,2));
      ub = PriorDist.cauchy(NumCauchy,1)+(6*PriorDist.cauchy(NumCauchy,2));
      while Numz<NumPriorDraws;
         z = CauchyRndFcn(PriorDist.cauchy(NumCauchy,1),PriorDist.cauchy(NumCauchy,2),2*NumPriorDraws);
         %
         % remove value of z that are too big or too small
         %
         if sum(z>=lb)<length(z);
            z = z(z>=lb);
         end;
         if sum(z<=ub)<length(z);
            z = z(z<=ub);
         end;
         Oldz = [Oldz z];
         z = Oldz;
         if length(z)>NumPriorDraws;
            z = z(1:NumPriorDraws);
         end;
         Numz = length(z);
      end;
      thetaDraws(i,:) = z;
   elseif thetaDist(i)==9;
      %
      % Logistic
      %
      NumLogistic = NumLogistic+1;
      thetaDraws(i,:) = LogisticRndFcn(PriorDist.logistic_ab(NumLogistic,1),PriorDist.logistic_ab(NumLogistic,2),PriorDist.logistic_ab(NumLogistic,3),NumPriorDraws);
   elseif thetaDist(i)==10;
      %
      % Gumbel
      %
      NumGumbel = NumGumbel+1;
      thetaDraws(i,:) = GumbelRndFcn(PriorDist.gumbel_ab(NumGumbel,1),PriorDist.gumbel_ab(NumGumbel,2),NumPriorDraws);
   elseif thetaDist(i)==11;
      %
      % Pareto
      %
      NumPareto = NumPareto+1;
      thetaDraws(i,:) = PriorDist.pareto(NumPareto,3)+ParetoRndFcn(PriorDist.pareto(NumPareto,1),PriorDist.pareto(NumPareto,2),NumPriorDraws);
   end;
   drawnow;
end;
%
% close the wait dialog
%
if NumPriorDraws>=100;
   set(WaitControls.text,'UserData','done');
   delete(WaitHandle);
   drawnow;
   pause(0.02);
end;
%
% check if we should provide an output
%
if nargout==1;
   thetaRnd = thetaDraws;
else;
   %
   % save the draws to file
   %
   PriorDrawsFile = FixFilePath([DSGEModel.OutputDirectory '\priordraws\PriorDraws-' DSGEModel.NameOfModel '.mat']);
   save(PriorDrawsFile,'thetaDraws');
   %
   txt = ['YADA has created ' int2str(NumPriorDraws) ' draws from the prior distributions of the ' int2str(NumParam) ' parameters that can be estimated. The draws are stored in the file "' GetFilename(PriorDrawsFile) '", located in the directory "' GetPath(PriorDrawsFile) '".'];
   About(txt,'information','Prior Sampling',200,500,CurrINI);
end;

%
% end of DSGEPriorSampling.m
%
