function thetaRnd = DSGESystemPriorSampling(theta,thetaPositions,ModelParameters,thetaDist,PriorDist,ParameterNames,thetaIndex,UniformBounds,LowerBound,NumPriorDraws,DSGEModel,CurrINI,controls)
% DSGESystemPriorSampling: Draws parameters to be estimated from the system prior distribution using RWM with
%                          a normal proposal density.
%
% USAGE:
%
%       thetaRnd = DSGESystemPriorSampling(theta,thetaPositions,ModelParameters,thetaDist,PriorDist,ParameterNames,...
%                               thetaIndex,UniformBounds,LowerBound,NumPriorDraws,DSGEModel,CurrINI,controls)
%
% REQUIRED INPUT:  theta (vector), with initial values of the parameters to be estimated.
%
%                  thetaPositions (structure) where thetaPositions(i).parameter is a string vector giving the name
%                            of the model parameter in position i of theta. The field "distribution" has a string
%                            vector giving the assumed prior distribution of the parameter.
%
%                  ModelParameters (structure) whose fields have names given by the parameter names in the
%                            DSGE model.
%
%                  thetaDist (vector) with integers 1 if gamma, 2 if beta, 3 if normal, 4 if invgamma, 5 if
%                            truncnormal, 6 if uniform, 7 if student, 8 if cauchy, 9 if logistic, 10 if gumbel,
%                            and 11 if pareto.
%
%                  PriorDist (structure) with fields "beta", "gamma", "normal", "invgamma", "truncnormal", "uniform",
%                            "student", "cauchy", "logistic", "gumbel", and "pareto", where the values are matrices with the
%                            parameter values for the distributions. In the case of the normal, beta, gamma, logistic 
%                            and gumbel these parameter values are the mean and the standard deviation. For the invgamma
%                            it is the s and q parameters, where q is a positive integer (degrees of freedom). For the left
%                            truncated normal there's a third parameter, the lower bound. For the uniform the parameter
%                            values are the lower and the upper bound. For the student-t they are the location (mode),
%                            the scale and the degrees of freedom parameter, while for the Cauchy only the location
%                            and scale parameters are needed. For the Pareto the parameter values are for the mode
%                            and the shape parameter.
%
%                  ParameterNames (structure) with fields "calibrated", "beta", "gamma", "normal", "invgamma", "truncnormal",
%                            "uniform", "student", "cauchy", "logistic", "gumbel", "pareto", "all" and "estimated". Each field returns
%                            a string matrix with the parameter names.
%
%                  thetaIndex (vector) with integers where 1=gamma, invgamma, truncnormal, pareto 2=beta, 3=uniform, and
%                            0=normal, student-t, cauchy, logistic, gumbel. This is used to determine the type of transformation
%                            that may need to be performed for the theta parameters.
%
%                  UniformBounds (matrix) with the same number of rows as parameters and 2 columns. The columns are 0 1 except
%                            for the uniform and the beta distribution, where they represent the upper and lower bounds.
%
%                  LowerBound (vector) with lower bound values for the theta vector.
%
%                  NumPriorDraws (integer) with the number of prior draws.
%
%                  DSGEModel (structure) whose fields contain all required information
%                            about the model, such as input files, variable and sample
%                            data. Actual observations are not stored in this file.
%
%                  CurrINI (structure) with initialization information.
%
%                  controls (structure) whose fields provide handles to all the UI
%                            controls in YADA's main dialog.
%
% OPTIONAL OUTPUT: thetaRnd (matrix) of dimension NumParamxNumPriorDraws with the draws from the prior
%                            distribution of the DSGE model parameters. If the variable is used, then
%                            prior draws are not stored to disk.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   DG-R/MPR
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2017 European Central Bank.
%
%                       First version: May 17, 2017.
%                        This version: August 10, 2017.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 18-05-2017: Continued working on the function from the prior mode estimation.
%
% * 19-05-2017: The function now checks if prior mode data already exist on disk.
%
% * 22-05-2017: Continied the function from the prior mode estimation point.
%
% * 23-05-2017: Took into account that PriorModeEstimation provides additional output
%               variables.
%
% * 24-05-2017: Allowed to the same type of construction of the covariance matrix for the
%               RWM prior sampler like in the case of the posterior sampler.
%
% * 29-05-2017: Reorganized the function at the end such that it now resembles the posterior
%               samplers, but it forces the marginal likelihood estimation based on Geweke's
%               truncated normal approach to the modified harmonic mean estimator.
%
% * 30-05-2017: Finished the function by adding printing to a text file.
%
% * 02-06-2017: Took into account that PriorModeEstimation provides the additional output variables
%               "MaxTime,ToleranceValue,MaxIter,NumIter,FuncDiff".
%
% * 06-06-2017: Took into account that PriorModeEstimation provides the additional output variables
%               "ParameterNames,thetaCalibrated,thetaAdditionalInitial,thetaAdditionalMode,OptResults,
%               ScreenMessages".
%
% * 07-06-2017: The variable "CovProb" from the modified harmonic mean estimation is now also saved
%               to the output file.
%
% * 26-06-2017: Added the optional output variable thetaRnd.
%
% * 10-08-2017: Took into account that PriorModeEstimation now provides ModelParameters as an output variable.
%               The function also makes it possible to view the raw prior draws.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

if nargout==1;
   thetaRnd = [];
   ShowLogText = 0;
else;
   ShowLogText = 1;
end;
%
% create the output directory
%
stat = MakeDir(DSGEModel.OutputDirectory,'priordraws');
if stat~=1;
   if ispc==1;
      txt = ['YADA was for some reason unable to create the directory "' DSGEModel.OutputDirectory '\priordraws". The prior sampling has therefore been aborted.'];
   else;
      txt = ['YADA was for some reason unable to create the directory "' DSGEModel.OutputDirectory '/priordraws". The prior sampling has therefore been aborted.'];
   end;
   About(txt,'information','YADA - Directory Creation Problem',180,500,CurrINI);
   return;
end;
%
% draw from the prior distributions of the parameters to use
%
if DSGEModel.RandomNumberValue==1;
   %
   % intialize random number generators based on fixed state/seed
   %
   rand('state',0);
   randn('state',0);
else;
   %
   % initialize random number generators based on clock based state/seed
   %
   rand('state',sum(100*clock));
   randn('state',sum(100*clock));
end;
%
% Now we need to copy files to the tmp directory and load data from DSGEModel.AIMDataFile
%
[stat,msg] = CopyFile([GetPath(DSGEModel.AIMDataFile) 'compute_aim_matrices.m'],[pwd '\tmp']);
[stat,msg] = CopyFile(DSGEModel.MeasurementEquationFile,[pwd '\tmp']);
if FileExist(DSGEModel.UpdateParameterFile)==1;
   [stat,msg] = CopyFile(DSGEModel.UpdateParameterFile,[pwd '\tmp']);
end;
if FileExist(DSGEModel.InitializeParameterFile)==1;
   [stat,msg] = CopyFile(DSGEModel.InitializeParameterFile,[pwd '\tmp']);
end;
[stat,msg] = CopyFile(DSGEModel.SystemPriorFile,[pwd '\tmp']);
ErrorStr = '';
ErrorFile = '';
try;
   AIMData = load(DSGEModel.AIMDataFile);
catch;
   ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' lasterr];
   ErrorFile = DSGEModel.AIMDataFile;
end;
%
if isempty(ErrorStr)==0;
   About(ErrorStr,'error',['Error - ' GetFilename(ErrorFile)],CurrINI.scrsz(4)-50,500,CurrINI);
   drawnow;
   return;
end;
%
% ask if we should use a normal proposal density or the marginal prior proposal density
%
txt = strvcat('Would you like to use the independent marginal prior densities as proposal densities for the prior sampler?',' ','NOTE: If you answer Yes, the joint proposal density can''t be tuned. If you choose No, then YADA will use a normal proposal density using a prior mode based estimate of the covariance matrix and otherwise tune the proposal as in the case of the posterior sampler.');
answer = Query(txt,'question',500,'Prior Sampling',600,'no',CurrINI);
if strcmp(lower(answer),'no')==1;
   UseNormalProposal = 1;
else;
   UseNormalProposal = 0;
end;
%
% remember the initial value of the original parameters
%
thetaInit = theta;
%
% compute the prior mode of the system prior if the file doesn't exist
%
ModeFileDir = FixFilePath([DSGEModel.OutputDirectory '\mode']);
ModeFile = FixFilePath([ModeFileDir '\SystemPriorMode-' DSGEModel.NameOfModel '.mat']);
if FileExist(ModeFile)==1;
   %
   % ask if the user wants to use the previous estimated data
   %
   txt = 'YADA has located previous estimates of the system prior mode. Would you like to use these results?';
   answer = Query(txt,'question',500,'Prior Mode Estimation - csminwel',600,'yes',CurrINI);
   if strcmp(lower(answer),'yes')==1;
      EstimationStatus = 1;
      PriorModeData = load(ModeFile);
      phiMode = PriorModeData.phi;
      InverseHessian = PriorModeData.InverseHessian;
      InverseHessianFinDiff = PriorModeData.InverseHessianFinDiff;
      phiFittedStd = PriorModeData.phiFittedStd;
      thetaFittedStd = PriorModeData.thetaFittedStd;
      LogMargLikeSystemLaplace = PriorModeData.LogMargLikeSystemLaplace;
      R = PriorModeData.R;
   else;
      [phiMode,ModelParameters,InverseHessian,InverseHessianFinDiff,phiFittedStd,thetaFittedStd,LogMargLikeSystemLaplace,ParameterNames,thetaCalibrated,thetaAdditionalInitial,thetaAdditionalMode,OptResults,ScreenMessages,R,MaxTime,ToleranceValue,MaxIter,NumIter,FuncDiff,EstimationStatus] = PriorModeEstimation(theta,thetaPositions,ModelParameters,thetaDist,PriorDist,ParameterNames,thetaIndex,UniformBounds,LowerBound,AIMData,DSGEModel,CurrINI,controls);
   end;
else;
   [phiMode,ModelParameters,InverseHessian,InverseHessianFinDiff,phiFittedStd,thetaFittedStd,LogMargLikeSystemLaplace,ParameterNames,thetaCalibrated,thetaAdditionalInitial,thetaAdditionalMode,OptResults,ScreenMessages,R,MaxTime,ToleranceValue,MaxIter,NumIter,FuncDiff,EstimationStatus] = PriorModeEstimation(theta,thetaPositions,ModelParameters,thetaDist,PriorDist,ParameterNames,thetaIndex,UniformBounds,LowerBound,AIMData,DSGEModel,CurrINI,controls);
end;
%
% we can now begin sampling using the RWM sampler with a normal proposal density.
%
if EstimationStatus==0;
   return;
end;
%
% now select the covariance matrix for the RWM sampler with a normal proposal density
%
if DSGEModel.InverseHessianEstimator==1;
   CovPhi = InverseHessian;
elseif DSGEModel.InverseHessianEstimator==2;
   if sum(isnan(phiFittedStd))==0;
      SigmaStd = sqrt(abs(diag(InverseHessian)));
      SigmaCorr = InverseHessian ./ (SigmaStd * SigmaStd');
      if DSGEModel.ModifiedHessian==1;
         FullSet = (1:length(phiFittedStd));
         for i=1:length(phiFittedStd);
            iExcluded = setdiff(FullSet,i);
%            phiFittedStd(i) = phiFittedStd(i)/sqrt(1-(SigmaCorr(i,iExcluded)*inv(SigmaCorr(iExcluded,iExcluded))*SigmaCorr(iExcluded,i)));
            phiFittedStd(i) = phiFittedStd(i)/sqrt(1-(SigmaCorr(i,iExcluded)*(SigmaCorr(iExcluded,iExcluded)\SigmaCorr(iExcluded,i))));
         end;
      end;
      CovPhi = SigmaCorr .* (phiFittedStd * phiFittedStd');
   else;
      CovPhi = InverseHessianFinDiff;
   end;
else;
   CovPhi = InverseHessianFinDiff;
end;
%
% Coverage probabilities for the Modified Harmonic Mean estimator of
% the marginal likelihood
%
CovStartStr = get(controls.dsge.coveragestart,'String');
DSGEModel.CovStartValue = str2num(StringTrim(CovStartStr(DSGEModel.CoverageStart,:)));
CovIncStr = get(controls.dsge.coverageincrement,'String');
DSGEModel.CovIncValue = str2num(StringTrim(CovIncStr(DSGEModel.CoverageIncrement,:)));
CovEndStr = get(controls.dsge.coverageend,'String');
DSGEModel.CovEndValue = str2num(StringTrim(CovEndStr(DSGEModel.CoverageEnd,:)));
%
% check if we should manipulate the correlations for the proposal density
%
if UseNormalProposal==1;
   if DSGEModel.MaxCorrelationValue>1;
      MaxCorrStr = get(controls.posterior.maxcorrelation,'String');
      MaxCorr = str2num(StringTrim(MaxCorrStr(DSGEModel.MaxCorrelationValue,:)));
      %
      % create the correlation matrix
      %
      StdVec = sqrt(diag(CovPhi));
      CorrMat = CovPhi ./ (StdVec*StdVec');
      for i=1:length(StdVec);
         CorrMat(i,i) = 0;
      end;
      ScaleFactor = max(max(abs(CorrMat)));
      ScaleMatrix = min(1,MaxCorr/ScaleFactor)*ones(length(StdVec),length(StdVec));
      for i=1:length(StdVec);
         ScaleMatrix(i,i) = 1;
      end;
      CovPhi = CovPhi .* ScaleMatrix;
   else;
      MaxCorr = 1;
   end;
else;
   MaxCorr = 1;
end;
%
% 5. Scale factor for initializing the random walk Metropolis algorithm, i.e., c0 in "YADA Manual - Computational Details".
%
InitScaleStr = get(controls.posterior.initialscale,'String');
c0 = str2num(StringTrim(InitScaleStr(DSGEModel.MHInitialScaleFactor,:)));
%
% 6. Scale factor for the random walk Metropolis algorithm, i.e., c in "YADA Manual - Computational Details".
%
ScaleStr = get(controls.posterior.scalefactor,'String');
c = str2num(StringTrim(ScaleStr(DSGEModel.MHScaleFactor,:)));
%
% information for finite difference estimate of inverse Hessian
%
StepLengthStr = get(controls.optimize.steplength,'String');
StepLength = str2num(StringTrim(StepLengthStr(DSGEModel.StepLengthHessian,:)));
%
if ShowLogText==1;
   %
   % fix the YADA gui
   %
   set(controls.about,'UserData',0);
   maingui = findobj('Type','figure','Tag','YADA');
   oldtab = get(controls.tab1,'UserData');
   set(maingui,'CurrentObject',controls.tab6);
   YADAGUI('tab');
   if MatlabNumber>=7;
      uicontrol(controls.tab6);
   end;
   set(controls.about,'UserData',1);
   drawnow;
   %
   % check if we should add some diving line to the log-text control
   %
   LogText = get(controls.estimation_log,'String');
   if isempty(LogText)==0;
      SetLogText(strvcat(' ','*************************************************************************','*    Y    A    D    A       Y    A    D    A       Y    A    D    A     *','*************************************************************************'));
   end;
end;
%
NumParam = length(phiMode);
%
if ShowLogText==1;
   if UseNormalProposal==1;
      SetLogText(strvcat(' ','Initializing system prior sampling using random walk Metropolis','with a normal proposal density...',' ',[' * Time:                               ' StringTrim(datestr(now,14)) ', ' GetCurrentDate],[' * Number of prior draws:              ' num2str(NumPriorDraws,'%0.0f')],[' * Maximum absolute correlation:       ' num2str(MaxCorr)],[' * Scale factor for initial phi value: ' num2str(c0)],[' * Scale factor in proposal density:   ' num2str(c)],[' * Number of parameters to estimate:   ' num2str(NumParam)],[' * Total number of parameters:         ' num2str(size(ParameterNames.all,1),'%0.0f')],[' * Number of observed variables:       ' num2str(DSGEModel.n,'%0.0f')],[' * Number of exogenous variables:      ' num2str(DSGEModel.k,'%0.0f')],[' * Number of state variables:          ' num2str(length(DSGEModel.StateVariablePositions),'%0.0f')],[' * Number of state shocks:             ' num2str(size(DSGEModel.StateShockNames,1),'%0.0f')],[' * Number of measurement errors:       ' num2str(rank(R),'%0.0f')]));
   else;
      SetLogText(strvcat(' ','Initializing system prior sampling using Metropolis-Hastings','with a proposal density based on the marginal prior...',' ',[' * Time:                               ' StringTrim(datestr(now,14)) ', ' GetCurrentDate],[' * Number of prior draws:              ' num2str(NumPriorDraws,'%0.0f')],[' * Number of parameters to estimate:   ' num2str(NumParam)],[' * Total number of parameters:         ' num2str(size(ParameterNames.all,1),'%0.0f')],[' * Number of observed variables:       ' num2str(DSGEModel.n,'%0.0f')],[' * Number of exogenous variables:      ' num2str(DSGEModel.k,'%0.0f')],[' * Number of state variables:          ' num2str(length(DSGEModel.StateVariablePositions),'%0.0f')],[' * Number of state shocks:             ' num2str(size(DSGEModel.StateShockNames,1),'%0.0f')],[' * Number of measurement errors:       ' num2str(rank(R),'%0.0f')]));
   end;
end;
%
% compute the Choleski decomposition of the covariance matrix of the proposal density
%
if UseNormalProposal==1;
   CholeskiSigma = chol(CovPhi)';
   %
   if c0>0;
      phi = phiMode+(c0*CholeskiSigma*randn(NumParam,1));
   else;
      phi = phiMode;
   end;
else;
   %
   % need to draw all at once
   %
   theta = DSGEPriorSampling(PriorDist,thetaDist,LowerBound,1,'',DSGEModel,CurrINI);
   phi = ThetaToPhi(theta,thetaIndex,UniformBounds,LowerBound);
end;
theta = PhiToTheta(phi,thetaIndex,UniformBounds,LowerBound);
%
% Now evaluate the log posterior at the starting value
%
logPriorInit = -logSystemPriorPhiDSGE(phi,thetaPositions,ModelParameters,thetaDist,PriorDist,thetaIndex,UniformBounds,LowerBound,AIMData,DSGEModel,CurrINI);
logPriorOld = logPriorInit;
%
% compute the height of the marginal prior
%
logMargPriorOld = -logPriorPhiDSGE(phi,thetaIndex,thetaDist,PriorDist,LowerBound,UniformBounds);
%
% check if we should show a progress dialog
%
if DSGEModel.ShowProgress==1;
   ProgressStructure.title = ['Progress for ' num2str(NumPriorDraws,'%0.0f') ' prior draws'];
   ProgressStructure.facecolor = CurrINI.progress_facecolor;
   ProgressStructure.startfacecolor = CurrINI.progress_startfacecolor;
   ProgressStructure.edgecolor = CurrINI.progress_edgecolor;
   ProgressStructure.bgcolor = CurrINI.progress_bgcolor;
   ProgressStructure.stop = 0;
   ProgressStructure.clock = DSGEModel.ShowProgressClock;
   ProgressStructure.label = 'Acceptance Rate:';
   %
   if UseNormalProposal==1;
      ProgressStructure.name = 'Random Walk Metropolis With Normal Proposal - Prior Draws';
   else;
      ProgressStructure.name = 'Metropolis-Hastings With Marginal Prior Proposal - Prior Draws';
   end;
   ProgressStructure.CurrINI = CurrINI;
   mywait = ProgressDLG(0,ProgressStructure,600);
   set(mywait,'Color',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'));
   drawnow;
else;
   %
   % show a wait dialog
   %
   if UseNormalProposal==1;
      txt = ['Please wait while YADA runs the random walk Metropolis sampler with a normal proposal density for the prior draws. The total number of prior draws is ' int2str(NumPriorDraws) '. Calculation was started at: ' StringTrim(datestr(now,14)) ' on ' GetCurrentDate '.'];
      WaitHandle = WaitDLG(txt,'information','Random Walk Metropolis - Prior Draws',500,200,CurrINI,0);
   else;
      txt = ['Please wait while YADA runs the Metropolis-Hastings sampler with the marginal prior as proposal density for the prior draws. The total number of prior draws is ' int2str(NumPriorDraws) '. Calculation was started at: ' StringTrim(datestr(now,14)) ' on ' GetCurrentDate '.'];
      WaitHandle = WaitDLG(txt,'information','Metropolis-Hastings - Prior Draws',500,200,CurrINI,0);
   end;
   WaitControls = get(WaitHandle,'UserData');
end;
%
% start the sampler
%
abort = '';
phiDraws = zeros(NumParam,NumPriorDraws);
thetaDraws = zeros(NumParam,NumPriorDraws);
logSystemPriorDensity = zeros(1,NumPriorDraws);
AcceptedDraws = zeros(1,NumPriorDraws);
tic;
for NPriorDraw=1:NumPriorDraws;
   %
   % draw a proposal phi
   %
   if UseNormalProposal==1;
      phiProp = phi+(c*CholeskiSigma*randn(NumParam,1));
      logPriorProp = -logSystemPriorPhiDSGE(phiProp,thetaPositions,ModelParameters,thetaDist,PriorDist,thetaIndex,UniformBounds,LowerBound,AIMData,DSGEModel,CurrINI);
      if isnan(logPriorProp)==0;
         AcceptProb = min(1,exp(logPriorProp-logPriorOld));
      else;
         AcceptProb = 0;
      end;
   else;
      thetaProp = DSGEPriorSampling(PriorDist,thetaDist,LowerBound,1,'',DSGEModel,CurrINI);
      phiProp = ThetaToPhi(thetaProp,thetaIndex,UniformBounds,LowerBound);
      logPriorProp = -logSystemPriorPhiDSGE(phiProp,thetaPositions,ModelParameters,thetaDist,PriorDist,thetaIndex,UniformBounds,LowerBound,AIMData,DSGEModel,CurrINI);
      if isnan(logPriorProp)==0;
         logMargPriorProp = -logPriorPhiDSGE(phiProp,thetaIndex,thetaDist,PriorDist,LowerBound,UniformBounds);
         AcceptProb = min(1,exp(logPriorProp+logMargPriorOld-logPriorOld-logMargPriorProp));
      else;
         AcceptProb = 0;
      end;
   end;
   TestProb = rand;
   if TestProb<AcceptProb;
      if NPriorDraw>1;
         AcceptedDraws(NPriorDraw) = AcceptedDraws(NPriorDraw-1)+1;
      else;
         AcceptedDraws(1) = 1;
      end;
      phi = phiProp;
      logPriorOld = logPriorProp;
      theta = PhiToTheta(phi,thetaIndex,UniformBounds,LowerBound);
      if UseNormalProposal==0;
         logMargPriorOld = logMargPriorProp;
      end;
   else;
      if NPriorDraw>1;
         AcceptedDraws(NPriorDraw) = AcceptedDraws(NPriorDraw-1);
      else;
         AcceptedDraws(1) = 0;
      end;
   end;
   %
   % update the progress dialog
   %
   if DSGEModel.ShowProgress==1;
      abort = get(mywait,'UserData');
      if (strcmp(abort,'cancel')==1)|(strcmp(abort,'done')==1);
         break;
      else;
         if NPriorDraw<=NumPriorDraws;
            ProgressDLG([NPriorDraw/NumPriorDraws AcceptedDraws(NPriorDraw)/NPriorDraw]);
         end;
      end;
   end;
   %
   % store the results
   %
   phiDraws(:,NPriorDraw) = phi;
   thetaDraws(:,NPriorDraw) = theta;
   logSystemPriorDensity(NPriorDraw) = logPriorOld;
   drawnow;
end;
PriorSampleTime = toc;
%
% close the current waitbar
%
if DSGEModel.ShowProgress==1;
   if IsHandle(mywait)==1;
      set(mywait,'UserData','done');
      close(mywait);
      drawnow;
      pause(0.02);
   end;
else;
   if IsHandle(WaitHandle)==1;
      set(WaitControls.text,'UserData','done');
      delete(WaitHandle);
      drawnow;
      pause(0.02);
   end;
end;
%
%
% check if we should provide an output
%
if nargout==1;
   thetaRnd = thetaDraws;
   return;
end;
%
% compute certain features from the prior mode estimation
%
NumAcceptedDraws = AcceptedDraws(NumPriorDraws)/NumPriorDraws;
if strcmp(abort,'cancel')==0;
   if UseNormalProposal==1;
      SetLogText(strvcat(' ','Prior sampling with the random walk Metropolis completed!',[' * Time:                               ' StringTrim(datestr(now,14)) ', ' GetCurrentDate],[' * Prior sampling time:                ' SecondsToHoursStr(PriorSampleTime) ' (hrs:min:sec)'],[' * Acceptance rate:                    ' num2str(NumAcceptedDraws)]));
   else;
      SetLogText(strvcat(' ','Prior sampling with the Metropolis-Hastings sampler completed!',[' * Time:                               ' StringTrim(datestr(now,14)) ', ' GetCurrentDate],[' * Prior sampling time:                ' SecondsToHoursStr(PriorSampleTime) ' (hrs:min:sec)'],[' * Acceptance rate:                    ' num2str(NumAcceptedDraws)]));
   end;
else;
   SetLogText('Prior sampling aborted.');
   return;
end;
%
% prepare some statistics on the drawn parameters
%
SetLogText(strvcat(' ','Computing summary information about the parameters...',[' * Time:                               ' StringTrim(datestr(now,14))]));
%
tic;
ParData = ParameterSummary(thetaDraws');
ParTime = toc;
SetLogText(strvcat('Computation completed!',[' * Estimation time:                    ' SecondsToHoursStr(ParTime) ' (hrs:min:sec)']));
%
if DSGEModel.PosteriorDensityValue==1;
   ModeAlgorithm = ' * Algorithm:                          Gaussian';
elseif DSGEModel.PosteriorDensityValue==2;
   ModeAlgorithm = strvcat(' * Algorithm:                          Silverman-type and','                                       Skld-Roberts correction');
elseif DSGEModel.PosteriorDensityValue==3;
   ModeAlgorithm = ' * Algorithm:                          Sheather-Jones bandwidth';
elseif DSGEModel.PosteriorDensityValue==4;
   ModeAlgorithm = ' * Algorithm:                          Bump killing bandwidth';
end;
SetLogText(strvcat(' ','Computing the mode of the marginal system prior distributions...',ModeAlgorithm,[' * Time:                               ' StringTrim(datestr(now,14))]));
tic;
%
% display graphs of the marginal system prior and the marginal prior
%
[thetaMargMode,UpdateMargMode] = GraphPosteriorDensity(thetaDraws',0,[],[],[],[],[],[],[],DSGEModel,CurrINI);
ModeTime = toc;
SetLogText(strvcat('Calculations completed!',[' * Estimation time:                    ' SecondsToHoursStr(ModeTime) ' (hrs:min:sec)']));
%
% calculate the marginal likelihood for the system prior with the modified harmonic mean estimator
% based on Geweke's idea of a truncated normal
%
SetLogText(strvcat(' ','Computing the marginal likelihood...'));
SetLogText(strvcat(' * Geweke''s modified harmonic mean estimator',[' * Time:                               ' StringTrim(datestr(now,14))]));
tic;
[LogMargsMH,CovProb,NWStdErrMH] = MargLikeModifiedHarmonic(phiDraws',logSystemPriorDensity',0,[],DSGEModel);
ModHarmTime = toc;
SetLogText(strvcat('Estimation completed!',[' * Estimation time:                    ' SecondsToHoursStr(ModHarmTime) ' (hrs:min:sec)']));
%
% save the prior draws
%
PriorDrawsFile = FixFilePath([DSGEModel.OutputDirectory '\priordraws\PriorDraws-' DSGEModel.NameOfModel '.mat']);
save(PriorDrawsFile,'thetaDraws','phiDraws','logSystemPriorDensity','AcceptedDraws','LogMargLikeSystemLaplace','LogMargsMH','NWStdErrMH','CovProb');
%
% print the results to a text file
%
PriorSampleTextFile = FixFilePath([DSGEModel.OutputDirectory '\PriorSamplingSummary-' DSGEModel.NameOfModel '-' num2str(DSGEModel.InverseHessianEstimator,'%0.0f') '-' num2str(NumPriorDraws,'%0.0f') '.txt']);
SetLogText(strvcat(' ','Writing system prior sampling summary results to text file:',['   Directory: ' DSGEModel.OutputDirectory],['   File name: ' GetFilename(PriorSampleTextFile)]));
PrintSystemPriorSamplingSummary(PriorSampleTextFile,DSGEModel,CurrINI,ParameterNames,PriorDist,thetaDist,ParData,thetaMargMode,NumPriorDraws,rank(R),UseNormalProposal,c0,c,PriorSampleTime,NumAcceptedDraws,LogMargsMH,CovProb,NWStdErrMH,LogMargLikeSystemLaplace,StepLength,MaxCorr,thetaInit);
%
% display the results
%
if FileExist(PriorSampleTextFile)==1;
   TextGUI(PriorSampleTextFile,['System Prior Sampling - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 32 1000 CurrINI.scrsz(4)-100],'Results',50,CurrINI,1,1);
   drawnow;
end;
%
% ask if the user would like to see the raw prior draws
%
txt = strvcat('Would you like to see plots of the raw prior draws?',' ','NOTE: If you answer yes, YADA produces plots of both the transformed and the original parameters.');
answer = Query(txt,'question',300,'Prior Sampling - Raw Prior Draws',500,'yes',CurrINI);
if strcmp(lower(answer),'yes')==1;
   RawPosteriorPlots(phiDraws',0,NumPriorDraws,[],'Transformed',DSGEModel,CurrINI);
   drawnow;
   RawPosteriorPlots(thetaDraws',0,NumPriorDraws,[],'Original',DSGEModel,CurrINI);
   drawnow;
end;
%
% ask if the user would like to see plots of the prior
%
txt = 'Would you like to see plots of the prior densitities?';
answer = Query(txt,'question',300,'Prior Sampling - Prior Densities',500,'yes',CurrINI);
if strcmp(lower(answer),'yes')==1;
   GraphPosteriorDensity(thetaDraws',0,[],[],[],[],[],[],[],DSGEModel,CurrINI);
   drawnow;
end;

%
% end of DSGESystemPriorSampling.m
%
