function [phi,ModelParameters,InverseHessian,InverseHessianFinDiff,phiFittedStd,thetaFittedStd,LogMargLikeSystemLaplace,ParameterNames,thetaCalibrated,thetaAdditionalInitial,thetaAdditionalMode,OptResults,ScreenMessages,R,MaxTime,ToleranceValue,MaxIter,NumIter,FuncDiff,EstimationStatus] = PriorModeEstimation(theta,thetaPositions,ModelParameters,thetaDist,PriorDist,ParameterNames,thetaIndex,UniformBounds,LowerBound,AIMData,DSGEModel,CurrINI,controls)
% PriorModeEstimation: Computes the prior mode numerically for the system prior using csminwel for either
%                      the original or the transformed parameters.
%
% USAGE:
%
%       [phi,ModelParameters,InverseHessian,InverseHessianFinDiff,phiFittedStd,thetaFittedStd,LogMargLikeSystemLaplace,...
%                                 ParameterNames,thetaCalibrated,thetaAdditionalInitial,thetaAdditionalMode,...
%                                 OptResults,ScreenMessages,R,MaxTime,ToleranceValue,MaxIter,NumIter,FuncDiff,...
%                                 EstimationStatus] = PriorModeEstimation(theta,thetaPositions,...
%                                                             ModelParameters,thetaDist,PriorDist,...
%                                                             ParameterNames,thetaIndex,UniformBounds,...
%                                                             LowerBound,AIMData,DSGEModel,CurrINI,controls)
%
% REQUIRED INPUT:  theta (vector), with initial values of the parameters to be estimated.
%
%                  thetaPositions (structure) where thetaPositions(i).parameter is a string vector giving the name
%                            of the model parameter in position i of theta. The field "distribution" has a string
%                            vector giving the assumed prior distribution of the parameter.
%
%                  ModelParameters (structure) whose fields have names given by the parameter names in the
%                            DSGE model.
%
%                  thetaDist (vector) with integers 1 if gamma, 2 if beta, 3 if normal, 4 if invgamma, 5 if
%                            truncnormal, 6 if uniform, 7 if student, 8 if cauchy, 9 if logistic, 10 if gumbel,
%                            and 11 if pareto.
%
%                  PriorDist (structure) with fields "beta", "gamma", "normal", "invgamma", "truncnormal", "uniform",
%                            "student", "cauchy", "logistic", "gumbel", and "pareto", where the values are matrices with the
%                            parameter values for the distributions. In the case of the normal, beta, gamma, logistic 
%                            and gumbel these parameter values are the mean and the standard deviation. For the invgamma
%                            it is the s and q parameters, where q is a positive integer (degrees of freedom). For the left
%                            truncated normal there's a third parameter, the lower bound. For the uniform the parameter
%                            values are the lower and the upper bound. For the student-t they are the location (mode),
%                            the scale and the degrees of freedom parameter, while for the Cauchy only the location
%                            and scale parameters are needed. For the Pareto the parameter values are for the mode
%                            and the shape parameter.
%
%                  ParameterNames (structure) with fields "calibrated", "beta", "gamma", "normal", "invgamma", "truncnormal",
%                            "uniform", "student", "cauchy", "logistic", "gumbel", "pareto", "all" and "estimated". Each field returns
%                            a string matrix with the parameter names.
%
%                  thetaIndex (vector) with integers where 1=gamma, invgamma, truncnormal, pareto 2=beta, 3=uniform, and
%                            0=normal, student-t, cauchy, logistic, gumbel. This is used to determine the type of transformation
%                            that may need to be performed for the theta parameters.
%
%                  UniformBounds (matrix) with the same number of rows as parameters and 2 columns. The columns are 0 1 except
%                            for the uniform and the beta distribution, where they represent the upper and lower bounds.
%
%                  LowerBound (vector) with lower bound values for the theta vector.
%
%                  AIMData (structure) with data collected by AIM about the model.
%
%                  DSGEModel (structure) whose fields contain all required information about the model, such as input files,
%                            variable and sample data.
%
%                  CurrINI (structure) with initialization information.
%
%                  controls (structure) whose fields provide handles to all the UI
%                            controls in YADA's main dialog.
%
% REQUIRED OUTPUT: phi (vector) of dimension mx1 with the prior mode of the parameters.
%
%                  ModelParameters (structure) whose fields have names given by the parameter names in the
%                            DSGE model.
%
%                  InverseHessian (matrix) of dimension mxm with the inverse Hessian at the prior
%                            mode.
%
%                  InverseHessianFinDiff (matrix) of dimension mxm with the inverse Hessian at the
%                            prior mode calculated with finite differences.
%
%                  phiFittedStd (vector) of dimension mx1 with the estimated standard deviations from
%                            fit of quadratic around the prior mode for the transformed parameters.
%
%                  thetaFittedStd (vector) of dimension mx1 with the estimated standard deviations from
%                            fit of quadratic around the prior mode for the original parameters.
%
%                  LogMargLikeSystemLaplace (scalar) with the log marginal likelihood for the system
%                            prior based on the Laplace approximation and the finite difference
%                            estimate of the inverse Hessian (when it is well-behaved).
%
%                  thetaCalibrated (vector) with values of any calibrated DSGE model parameters.
%
%                  thetaAdditionalInitial (vector) with initial values of any additional DSGE model parameters.
%
%                  thetaAdditionalMode (vector) with the prior mode values of any additional DSGE model parameters.
%
%                  OptResults (matrix), of dimension (NumIter x 4) containing iteration no, value of log posterior, value
%                            of convergence measure, and the largest absolute value of the gradient.
%
%                  ScreenMessages (string matrix) with all the screen messages that csminwel could have produced.
%
%                  R (matrix) of dimension nxn with the measurement error covariances.
%
%                  MaxTime (time) for the full optimization with csminwel.
%
%                  ToleranceValue (scalar) with the tolerance value used the the optimization routine.
%
%                  MaxIter (integer) with the maximum number of iterations.
%
%                  NumIter (integer) with the number of iterations used by the optimization routine.
%
%                  FuncDiff (scalar) with the value of the difference in function values when the
%                            optimization routine is satisfied.
%
%                  EstimationStatus (boolean) that takes the value 1 if the prior mode
%                            estimation was successful, and 0 otherwise.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   DG-R/MPR
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2017 European Central Bank.
%
%                       First version: May 18, 2017.
%                        This version: August 10, 2017.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 19-05-2017: Continued the function from the calculation of the marginal log prior at the initial
%               parameter values.
%
% * 22-05-2017: Added the R output variable.
%
% * 23-05-2017: Added computations based on "VillaniCheckOptimum". These results are saved to the prior mode
%               file and provided as output from the function.
%
% * 24-05-2017: Added the output variable "LogMargLikeSystemLaplace".
%
% * 01-06-2017: Updated the documentation.
%
% * 02-06-2017: Added the output variables "MaxTime,ToleranceValue,MaxIter,NumIter,FuncDiff".
%
% * 06-06-2017: Added the output variables "ParameterNames,thetaCalibrated,thetaAdditionalInitial,
%               thetaAdditionalMode,OptResults,ScreenMessages".
%
% * 22-06-2017: Took into account that the system prior function now takes the input variable
%               OrderQZ instead of CurrINI.
%
% * 10-08-2017: Made sure that "ModelParameters" is also an output variable.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialze
%
phi = [];
R = [];
InverseHessian = [];
InverseHessianFinDiff = [];
phiFittedStd = [];
thetaFittedStd = [];
LogMargLikeSystemLaplace = [];
EstimationStatus = 0;
ScreenMessages = '';
MaxTime = [];
ToleranceValue = [];
MaxIter = [];
NumIter = [];
FuncDiff = [];
thetaCalibrated = [];
thetaAdditionalInitial = [];
thetaAdditionalMode = [];
OptResults = [];
ScreenMessages = '';
if FileExist(FixFilePath([pwd '\tmp\AiMErrorData.mat']))==1;
   delete(FixFilePath([pwd '\tmp\AiMErrorData.mat']));
end;
OriginalModelParameters = ModelParameters;
%
if isfield(PriorDist,'beta_ab')==0;
   %
   % Transform PriorDist.gamma and PriorDist.beta to the relevant parameters that are used by the logGammaPDF and
   % the logBetaPDF functions
   %
   if isempty(PriorDist.beta)==0;
      [a,b] = MomentToParamStdbetaPDF((PriorDist.beta(:,1)-PriorDist.beta(:,3))./(PriorDist.beta(:,4)-PriorDist.beta(:,3)),PriorDist.beta(:,2)./(PriorDist.beta(:,4)-PriorDist.beta(:,3)));
      PriorDist.beta_ab = [a b];
   else;
      PriorDist.beta_ab = [];
   end;
   %
   if isempty(PriorDist.gamma)==0;
      [a,b] = MomentToParamGammaPDF(PriorDist.gamma(:,1)-PriorDist.gamma(:,3),PriorDist.gamma(:,2));
      PriorDist.gamma_ab = [a b];
   else;
      PriorDist.gamma_ab = [];
   end;
   if isempty(PriorDist.logistic)==0;
      [a,b] = MomentToParamLogisticPDF(PriorDist.logistic(:,1),PriorDist.logistic(:,2),PriorDist.logistic(:,3));
      PriorDist.logistic_ab = [a b PriorDist.logistic(:,3)];
   else;
      PriorDist.logistic_ab = [];
   end;
   if isempty(PriorDist.gumbel)==0;
      [a,b] = MomentToParamGumbelPDF(PriorDist.gumbel(:,1),PriorDist.gumbel(:,2));
      PriorDist.gumbel_ab = [a b];
   else;
      PriorDist.gumbel_ab = [];
   end;
end;
%
% fix the YADA gui
%
set(controls.about,'UserData',0);
maingui = findobj('Type','figure','Tag','YADA');
oldtab = get(controls.tab1,'UserData');
set(maingui,'CurrentObject',controls.tab6);
YADAGUI('tab');
if MatlabNumber>=7;
   uicontrol(controls.tab6);
end;
set(controls.about,'UserData',1);
drawnow;
%
% check if we should add some diving line to the log-text control
%
LogText = get(controls.estimation_log,'String');
if isempty(LogText)==0;
   SetLogText(strvcat(' ','*************************************************************************','*    Y    A    D    A       Y    A    D    A       Y    A    D    A     *','*************************************************************************'));
end;
%
% Check the file with prior information
%
SetLogText(strvcat(' ','Initializing system prior mode estimation...'));
%
phi = ThetaToPhi(theta,thetaIndex,UniformBounds,LowerBound);
ModelParameters = ThetaToModelParameters(ModelParameters,theta,thetaPositions);
ErrorStr  = '';
%
firstUpdatePos = 0;
lastUpdatePos = 0;
firstInitializePos = 0;
lastInitializePos = 0;
if DSGEModel.RunInitializeFirst==1;
   %
   % check if the file with additional parameters that only needs to be updated is OK
   %
   SetLogText(strvcat([' * Checking ' GetName(DSGEModel.InitializeParameterFile) '...']));
   %
   % we need to run the file DSGEModel.InitializeParameterFile first
   %
   try;
      firstInitializePos = length(fieldnames(ModelParameters))+1;
      eval(['ModelParameters = ' GetName(DSGEModel.InitializeParameterFile) '(ModelParameters);']);
      lastInitializePos = length(fieldnames(ModelParameters));
   catch;
      ErrorStr = ['Error found in "' GetName(DSGEModel.InitializeParameterFile) '.m". Message caught is: ' lasterr];
      ErrorFile = DSGEModel.InitializeParameterFile;
   end;
   if (FileExist(DSGEModel.UpdateParameterFile)==1)&(isempty(ErrorStr)==1);
      SetLogText(strvcat('   STATUS: OK',[' * Checking ' GetName(DSGEModel.UpdateParameterFile) '...']));
      try;
         firstUpdatePos = length(fieldnames(ModelParameters))+1;
         eval(['ModelParameters = ' GetName(DSGEModel.UpdateParameterFile) '(ModelParameters);']);
         lastUpdatePos = length(fieldnames(ModelParameters));
      catch;
         ErrorStr = ['Error found in "' GetName(DSGEModel.UpdateParameterFile) '.m". Message caught is: ' lasterr];
         ErrorFile = DSGEModel.UpdateParameterFile;
      end;
   end;
else;
   if FileExist(DSGEModel.UpdateParameterFile)==1;
      SetLogText(strvcat([' * Checking ' GetName(DSGEModel.UpdateParameterFile) '...']));
      try;
         firstUpdatePos = length(fieldnames(ModelParameters))+1;
         eval(['ModelParameters = ' GetName(DSGEModel.UpdateParameterFile) '(ModelParameters);']);
         lastUpdatePos = length(fieldnames(ModelParameters));
      catch;
         ErrorStr = ['Error found in "' GetName(DSGEModel.UpdateParameterFile) '.m". Message caught is: ' lasterr];
         ErrorFile = DSGEModel.UpdateParameterFile;
      end;
   end;
   if (FileExist(DSGEModel.InitializeParameterFile)==1)&(isempty(ErrorStr)==1);
      SetLogText(strvcat('   STATUS: OK',[' * Checking ' GetName(DSGEModel.InitializeParameterFile) '...']));
      try;
         firstInitializePos = length(fieldnames(ModelParameters))+1;
         eval(['ModelParameters = ' GetName(DSGEModel.InitializeParameterFile) '(ModelParameters);']);
         lastInitializePos = length(fieldnames(ModelParameters));
      catch;
         ErrorStr = ['Error found in "' GetName(DSGEModel.InitializeParameterFile) '.m". Message caught is: ' lasterr];
         ErrorFile = DSGEModel.InitializeParameterFile;
      end;
   end;
end;
%
if isempty(ErrorStr)==0;
   SetLogText(strvcat(['   STATUS: Bad input file ' GetName(ErrorFile)],' '));
   About(ErrorStr,'error',['Error - ' GetFilename(ErrorFile)],CurrINI.scrsz(4)-50,500,CurrINI);
   drawnow;
   return;
end;
%
% we can go on with checking if the initial values permit a unique solution of the DSGE model
%
SetLogText(strvcat('   STATUS: OK',' * Checking if the DSGE model has a unique convergent','   solution for initial parameter values...'));
%
% we can now run the DSGE model solver:
%
if DSGEModel.ModelSolver==1;
   SolverStr = 'AiM';
elseif DSGEModel.ModelSolver==2;
   SolverStr = 'Klein';
elseif DSGEModel.ModelSolver==3;
   SolverStr = 'Gensys';
end;
%
% To measure the solution time more accurately we call it twice, with the first call only being
% performed for checking if the model has a unique convergent solution at the initial values.
%
try;
   if DSGEModel.ModelSolver==1;
      [mcode,MaxAbsError,BMatrix,SMatrix] = AiMSolver(ModelParameters,AIMData.neq,AIMData.nlag,AIMData.nlead,DSGEModel.AIMTolerance);
   elseif DSGEModel.ModelSolver==2;
      [F,B0,mcode,MaxAbsError,ModelEigenvalues] = KleinSolver(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,CurrINI.OrderQZ);
   elseif DSGEModel.ModelSolver==3;
      [F,B0,mcode,MaxAbsError,ModelEigenvalues] = SimsSolver(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,CurrINI.OrderQZ);
   end;
catch;
   if DSGEModel.ModelSolver==1;
      ErrorStr = ['There was an error running AiM. YADA received the following message: ' lasterr];
   elseif DSGEModel.ModelSolver==2;
      ErrorStr = ['There was an error running Klein''s solver. YADA received the following message: ' lasterr];
   elseif DSGEModel.ModelSolver==3;
      ErrorStr = ['There was an error running Sims'' gensys solver. YADA received the following message: ' lasterr];
   end;
   if isempty(CurrINI.Sequence)==1;
      About(ErrorStr,'error','AiM Error',CurrINI.scrsz(4)-50,500,CurrINI);
      drawnow;
   else;
      SetLogText(ErrorStr);
   end;
   return;
end;
if mcode~=1;
   %
   % There doesn't exist a unique convergent solution for the initial parameter values.
   %
   if mcode==2;
      mcodeStr = 'Roots are not correctly computed by real_schur.'
   elseif mcode==3;
      mcodeStr = 'Too many big roots.';
   elseif mcode==35;
      mcodeStr = 'Too many big roots, and q(:,right) is singular.';
   elseif mcode==4;
      mcodeStr = 'Too few big roots.';
   elseif mcode==45;
      mcodeStr = 'Too few big roots, and q(:,right) is singular.';
   elseif mcode==5;
      mcodeStr = 'q(:,right) is singular.';
   elseif mcode==61;
      mcodeStr = 'Too many exact shiftrights.';
   elseif mcode==62;
      mcodeStr = 'Too many numeric shiftrights.';
   elseif mcode==7;
      mcodeStr = 'Infinite or NaN values detected.';
   elseif mcode==8;
      mcodeStr = 'The function "compute_aim_matrices" returns complex numbers.';
   elseif mcode==-1;
      mcodeStr = 'No stable solution.';
   elseif mcode==-2;
      mcodeStr = 'Too many large generalized eigenvalues.';
   elseif mcode==-3;
      mcodeStr = 'Too few large generalized eigenvalues.';
   else;
      mcodeStr = 'Return code not properly specified.';
   end;
   SetLogText(strvcat(['   STATUS: ' mcodeStr],' '));
   if isempty(CurrINI.Sequence)==1;
      txt = ['The ' SolverStr ' solver provided the return code: ' num2str(mcode) ', i.e., "' mcodeStr '" You have to change your initial parameter values.'];
      About(txt,'information','YADA - DSGE Model Problem',200,500,CurrINI);
      drawnow;
   end;
   return;
end;
if DSGEModel.ModelSolver==1;
   [F,B0] = AiMtoStateSpace(BMatrix,SMatrix,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions);
end;
%
% check for explosive or unsupported unit roots
%
if isempty(DSGEModel.UnitRootStates)==1;
   F1 = F;
else;
   KeepPos = GetPosition(DSGEModel.StateVariablePositions,DSGEModel.UnitRootStates);
   F1 = F(KeepPos,KeepPos);
end;
%
if ~((max(abs(eig(F1)))<1)&(IsAlmostEqual(max(abs(eig(F1))),1)==0))&~((isempty(DSGEModel.UnitRootStates)==1)&(max(abs(eig(F1)))<=1));
   SetLogText(strvcat('   STATUS: Non-stationary state equation',' '));
   if isempty(CurrINI.Sequence)==1;
      txt = ['The largest eigenvalue of the state equation is ' num2str(max(abs(eig(F1)))) '. The state equation is therefore non-stationary. You need to change your initial parameter values.'];
      About(txt,'information','YADA - DSGE Model Problem',200,500,CurrINI);
      drawnow;
   end;
   return;
end;
%
% check how long it takes to solve the model
%
tic;
if DSGEModel.ModelSolver==1;
   [mcode,MaxAbsError,BMatrix,SMatrix] = AiMSolver(ModelParameters,AIMData.neq,AIMData.nlag,AIMData.nlead,DSGEModel.AIMTolerance);
   [F,B0] = AiMtoStateSpace(BMatrix,SMatrix,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions);
elseif DSGEModel.ModelSolver==2;
   [F,B0,mcode] = KleinSolver(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,CurrINI.OrderQZ);
elseif DSGEModel.ModelSolver==3;
   [F,B0,mcode] = SimsSolver(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,CurrINI.OrderQZ);
end;
SolveTime = toc;
%
% add the values of F and B0 to the ModelParameters.YADA field
%
[B0Alt,KeepVar] = RemoveRedundantColumns(B0);
ModelParameters.YADA.F = F;
ModelParameters.YADA.B0 = B0Alt;
%
% check the measurement equation file for internal validity
%
SetLogText(strvcat('   STATUS: OK',['   DSGE model solver:      ' SolverStr],['   Maximum absolute error: ' num2str(MaxAbsError)],['   Solution time:          ' num2str(SolveTime) ' seconds'],[' * Checking ' GetName(DSGEModel.MeasurementEquationFile) '...']));
try;
   eval(['[A,H,R] = ' GetName(DSGEModel.MeasurementEquationFile) '(ModelParameters,AIMData.endog(DSGEModel.StateVariablePositions,:),DSGEModel.VariableNames,DSGEModel.XVariableNames,DSGEModel.n,size(F,1),DSGEModel.k);']);
catch;
   ErrorStr = ['Error found in "' GetName(DSGEModel.MeasurementEquationFile) '.m". Message caught is: ' lasterr];
   ErrorFile = DSGEModel.MeasurementEquationFile;
   SetLogText(strvcat(['   STATUS: Bad input file ' GetName(ErrorFile)],' '));
   if isempty(CurrINI.Sequence)==1;
      About(ErrorStr,'error',['Error - ' GetFilename(ErrorFile)],CurrINI.scrsz(4)-50,500,CurrINI);
      drawnow;
   end;
   return;
end;
%
% we can now check the marginal log prior
%
SetLogText(strvcat('   STATUS: OK',' * Checking the marginal log prior...'));
%
logPrior = logPriorDSGE(theta,thetaDist,PriorDist,LowerBound);
lnjac = logJacobian(phi,thetaIndex,UniformBounds);
%
% check if the log prior of the transformed parameters can be computed
%
if (isnan(lnjac)==1)|(isnan(logPrior)==1);
   ErrorStr = ['The marginal log prior of the original parameters is ' num2str(logPrior) ', while the log Jacobian is ' num2str(lnjac) '. YADA is therefore unable to compute the marginal log prior of the transformed parameters.'];
   SetLogText(strvcat('   STATUS: The marginal log prior is not a number.',' '));
   About(ErrorStr,'error','Error - Log Prior',400,500,CurrINI);
   drawnow;
   return;
end;
%
SetLogText(strvcat('   STATUS: OK',' * Checking if the log system prior can be calculated...'));
%
try;
   eval(['lnSP = ' GetName(DSGEModel.SystemPriorFile) '(theta,thetaPositions,ModelParameters,AIMData,DSGEModel,CurrINI.OrderQZ);']);
catch;
   ErrorStr  = ['Unable to calculate the log of the system prior. Message caught is: ' lasterr];
end;
%
if (isempty(ErrorStr)==1)&(isnan(lnSP)==1);
   ErrorStr = ['The log of the system prior is not a number at the initial values. YADA is therefore unable to estimate the prior mode.'];
end;
%
if isempty(ErrorStr)==0;
   SetLogText(strvcat('   STATUS: The marginal log prior is not a number.',' '));
   About(ErrorStr,'error','Error - Log Prior',400,500,CurrINI);
   drawnow;
   return;
end;
%
% compute the value of the full log prior
%
SetLogText(strvcat('   STATUS: OK',' * Checking the speed of computing the full system log prior...'));
%
% check the parameter target type
%
if IsOdd(DSGEModel.MaximizeAlgorithmValue)==1;
   OptParameters = 'Transformed';
else;
   OptParameters = 'Original';
end;
%
% Collect additional parameters in ParameterNames.additional
%
ParameterNames = ReadAdditionalParameterNames(ParameterNames,ModelParameters,OriginalModelParameters,[firstUpdatePos lastUpdatePos firstInitializePos lastInitializePos]);
%
% set up vectors with calibrated parameters and additional parameters. These are only used for printing to the output file
%
[thetaCalibrated,thetaAdditionalInitial] = ReadAdditionalParameterValues(ModelParameters,ParameterNames);
%
NumParam = length(theta);
%
% get the upper and lower bounds of the parameters
%
ParameterBounds = CreateBounds(thetaDist,LowerBound,UniformBounds);
ParameterBounds = ParameterBounds(:,1:2);
tic;
if IsOdd(DSGEModel.MaximizeAlgorithmValue)==1;
   logPrior = logSystemPriorPhiDSGE(phi,thetaPositions,ModelParameters,thetaDist,PriorDist,thetaIndex,UniformBounds,LowerBound,AIMData,DSGEModel,CurrINI);
else;
   logPrior = logSystemPriorThetaDSGE(theta,thetaPositions,ModelParameters,thetaDist,PriorDist,thetaIndex,ParameterBounds,LowerBound,AIMData,DSGEModel,CurrINI);
   logPrior = logPrior-logJacobian(phi,thetaIndex,UniformBounds);
end;
LogPriorTime = toc;
%
SetLogText(strvcat(['   Computation time:       ' num2str(LogPriorTime) ' seconds'],' * Checking the numerical gradient of the full system log prior'));
%
% Compute a numerical gradient
%
if IsOdd(DSGEModel.MaximizeAlgorithmValue)==1;
   [g,badg] = YADAnumgrad('logSystemPriorPhiDSGE',phi,thetaPositions,ModelParameters,thetaDist,PriorDist,thetaIndex,UniformBounds,LowerBound,AIMData,DSGEModel,CurrINI);
else;
   [g,badg] = YADAnumgrad('logSystemPriorThetaDSGE',theta,thetaPositions,ModelParameters,thetaDist,PriorDist,thetaIndex,ParameterBounds,LowerBound,AIMData,DSGEModel,CurrINI);
end;
%
if badg~=0;
   SetLogText(strvcat('   STATUS: Bad gradient of log prior at initial values',' '));
   [badpar,maxpos] = min(abs(g));
   %
   if badpar>0;
      ValueStr = num2str(badpar);
   else;
      ValueStr = 'NaN';
   end;
   %
   ParamStr = ['"' StringTrim(ParameterNames.estimated(maxpos(1),:)) '"'];
   AddParamStr = ' ';
   if length(maxpos)>=2;
      AddParamStr = 's ';
      for i=2:length(maxpos);
         ParamStr = [ParamStr ', "' StringTrim(ParameterNames.estimated(maxpos(i),:)) '"'];
      end;
   end;
   %
   txt = ['Unable to compute a meaningful numerical gradient of the log posterior at the initial values. The largest absolute value or an improper value (NaN or Inf) of the gradient is found for the parameter' AddParamStr ParamStr ', with value of ' ValueStr '. You probably need to change your initial parameter values.'];
   About(txt,'information','YADA - DSGE Model Problem',200,500,CurrINI);
   drawnow;
   return;
end;
%
% We use Sims' csminwel routine with transformed parameters.
%
OptAlgorithm = 'csminwel';
%
LogJacobianIV = lnjac;
LogThetaPriorIV = logPriorDSGE(PhiToTheta(phi,thetaIndex,UniformBounds,LowerBound),thetaDist,PriorDist,LowerBound);
LogSystemPriorIV = lnSP;
%
% compute the initial inverse Hessian
%
if DSGEModel.InitializeHessian==1;
   InitialInvHessMat = 10*eye(NumParam);
elseif DSGEModel.InitializeHessian==2;
   InitialInvHessMat = 1*eye(NumParam);
elseif DSGEModel.InitializeHessian==3;
   InitialInvHessMat = 10^(-1)*eye(NumParam);
elseif DSGEModel.InitializeHessian==4;
   InitialInvHessMat = 10^(-2)*eye(NumParam);
elseif DSGEModel.InitializeHessian==5;
   InitialInvHessMat = 10^(-3)*eye(NumParam);
elseif DSGEModel.InitializeHessian==6;
   InitialInvHessMat = 10^(-4)*eye(NumParam);
elseif DSGEModel.InitializeHessian==7;
   InitialInvHessMat = 10^(-5)*eye(NumParam);
elseif DSGEModel.InitializeHessian==8;
   %
   % sample from the prior
   %
   InitialInvHessMat = PriorHessianMatrix(thetaDist,thetaIndex,PriorDist,UniformBounds,LowerBound,DSGEModel.RandomNumberValue,IsOdd(DSGEModel.MaximizeAlgorithmValue));
end;
%
% The model has a unique solution for the initial values
%
MaxIterStr = get(controls.optimize.maximumiterations,'String');
MaxIterStr = StringTrim(strrep(MaxIterStr(DSGEModel.OptMaxIterationsValue,:),',',''));
MaxIter = str2num(MaxIterStr);
%
ToleranceStr = get(controls.optimize.tolerance,'String');
ToleranceStr = StringTrim(ToleranceStr(DSGEModel.OptToleranceValue,:));
ToleranceValue = str2num(ToleranceStr);
%
StepLengthStr = get(controls.optimize.steplength,'String');
StepLength = str2num(StringTrim(StepLengthStr(DSGEModel.StepLengthHessian,:)));
%
NumGridPointsStr = get(controls.optimize.numbergridpoints,'String');
NumGridPoints = str2num(StringTrim(NumGridPointsStr(DSGEModel.NumberOfGridPoints,:)));
%
if IsOdd(DSGEModel.MaximizeAlgorithmValue)==1;
   SetLogText(strvcat('   STATUS: OK',['   Log system prior value: ' num2str(LogSystemPriorIV,'%0.4f')],['   Log theta prior value:  ' num2str(LogThetaPriorIV,'%0.4f')],['   Log Jacobian value:     ' num2str(LogJacobianIV,'%0.4f')],['   Total log prior value:  ' num2str(LogSystemPriorIV+LogThetaPriorIV+LogJacobianIV,'%0.4f')],' ','Optimization routine:',sprintf('   Sims'' csminwel for %s parameters...',lower(OptParameters)),[' * Time:                             ' StringTrim(datestr(now,14))],[' * Number of parameters to estimate: ' int2str(NumParam)],[' * Total number of parameters:       ' int2str(size(ParameterNames.all,1))],[' * Tolerance level:                  ' ToleranceStr],[' * Maximum number of iterations:     ' MaxIterStr],[' * Number of observed variables:     ' num2str(DSGEModel.n,'%0.0f')],[' * Number of exogenous variables:    ' num2str(DSGEModel.k,'%0.0f')],[' * Number of state variables:        ' num2str(length(DSGEModel.StateVariablePositions),'%0.0f')],[' * Number of state shocks:           ' num2str(size(DSGEModel.StateShockNames,1),'%0.0f')],[' * Number of measurement errors:     ' num2str(rank(R),'%0.0f')]));
   try;
      tic;
      [fh,phiMode,gh,InverseHessian,NumIter,fcount,FuncDiff,AbortEstimation,OptResults,ScreenMessages,phiEstimates,ReturnCode] = YADAcsminwel('logSystemPriorPhiDSGE',phi,InitialInvHessMat,[],ToleranceValue,MaxIter,DSGEModel.ShowProgress,DSGEModel.ShowProgressClock,'Prior Mode Estimation - csminwel',CurrINI, ...
            thetaPositions,ModelParameters,thetaDist,PriorDist,thetaIndex,UniformBounds,LowerBound,AIMData,DSGEModel,CurrINI);
      GradientVector = abs(gh);
      ThetaPhiPartial = PartialThetaPartialPhi(phiMode,thetaIndex,UniformBounds);
      MaxTime = toc;
   catch;
      if DSGEModel.ShowProgress==1;
         WaitHandle = findobj('Type','figure','Tag','TMWProgressDLG');
         if IsHandle(WaitHandle)==1;
            set(WaitHandle,'UserData','done');
            close(WaitHandle);
            drawnow;
            pause(0.02);
         end;
      else;
         WaitHandle = findobj('Type','figure','Tag','WaitDLG');
         WaitControls = get(WaitHandle,'UserData');
         set(WaitControls.text,'UserData','done');
         delete(WaitHandle);
         drawnow;
         pause(0.02);
      end;
      ReturnCode = -1;
      AbortEstimation = '';
      NumIter = [];
      GradientVector = [];
      ErrorStr = ['There was an error running csminwel. YADA caught the following error message: ' lasterr];
   end;
else;
   SetLogText(strvcat('   STATUS: OK',['   Log system prior value: ' num2str(LogSystemPriorIV,'%0.4f')],['   Log theta prior value:  ' num2str(LogThetaPriorIV,'%0.4f')],['   Total log prior value:  ' num2str(LogSystemPriorIV+LogThetaPriorIV,'%0.4f')],' ','Optimization routine:',sprintf('   Sims'' csminwel for %s parameters...',lower(OptParameters)),[' * Time:                             ' StringTrim(datestr(now,14))],[' * Number of parameters to estimate: ' int2str(NumParam)],[' * Total number of parameters:       ' int2str(size(ParameterNames.all,1))],[' * Tolerance level:                  ' ToleranceStr],[' * Maximum number of iterations:     ' MaxIterStr],[' * Number of observed variables:     ' num2str(DSGEModel.n,'%0.0f')],[' * Number of exogenous variables:    ' num2str(DSGEModel.k,'%0.0f')],[' * Number of state variables:        ' num2str(length(DSGEModel.StateVariablePositions),'%0.0f')],[' * Number of state shocks:           ' num2str(size(DSGEModel.StateShockNames,1),'%0.0f')],[' * Number of measurement errors:     ' num2str(rank(R),'%0.0f')]));
   try;
      tic;
      [fh,thetaMode,gh,InverseHessian,NumIter,fcount,FuncDiff,AbortEstimation,OptResults,ScreenMessages,phiEstimates,ReturnCode] = YADAcsminwel('logSystemPriorThetaDSGE',theta,InitialInvHessMat,[],ToleranceValue,MaxIter,DSGEModel.ShowProgress,DSGEModel.ShowProgressClock,'Prior Mode Estimation - csminwel',CurrINI, ...
            thetaPositions,ModelParameters,thetaDist,PriorDist,thetaIndex,ParameterBounds,LowerBound,AIMData,DSGEModel,CurrINI);
      %
      % make sure that the outputs are transformed into phi-space
      %
      phiMode = ThetaToPhi(thetaMode,thetaIndex,UniformBounds,LowerBound);
      phiEstimates = zeros(size(thetaEstimates));
      for i=1:size(thetaEstimates,2);
         phiEstimates(:,i) = ThetaToPhi(thetaEstimates(:,i),thetaIndex,UniformBounds,LowerBound);;
      end;
      ThetaPhiPartial = PartialThetaPartialPhi(phiMode,thetaIndex,UniformBounds);
      InverseHessian = (ThetaPhiPartial\InverseHessian)/ThetaPhiPartial;
      GradientVector = abs(gh);
      MaxTime = toc;
   catch;
      if DSGEModel.ShowProgress==1;
         WaitHandle = findobj('Type','figure','Tag','TMWProgressDLG');
         if IsHandle(WaitHandle)==1;
            set(WaitHandle,'UserData','done');
            close(WaitHandle);
            drawnow;
            pause(0.02);
         end;
      else;
         WaitHandle = findobj('Type','figure','Tag','WaitDLG');
         WaitControls = get(WaitHandle,'UserData');
         set(WaitControls.text,'UserData','done');
         delete(WaitHandle);
         drawnow;
         pause(0.02);
      end;
      ReturnCode = -1;
      AbortEstimation = '';
      NumIter = [];
      GradientVector = [];
      ErrorStr = ['There was an error running csminwel. YADA caught the following error message: ' lasterr];
   end;
end;
if (ReturnCode==0)&(strcmp(lower(AbortEstimation),'cancel')~=1)&(NumIter<=MaxIter);
   %
   % we're OK
   %
   thetaMode = PhiToTheta(phiMode,thetaIndex,UniformBounds,LowerBound);
   LogJacobianMode = logJacobian(phiMode,thetaIndex,UniformBounds);
   LogPriorMode = logPriorDSGE(thetaMode,thetaDist,PriorDist,LowerBound);
   if IsOdd(DSGEModel.MaximizeAlgorithmValue)==1;
      LogJointPriorDensity = -fh;
   else;
      LogJointPriorDensity = -fh+LogJacobianMode;
   end;
   LogSystemPriorMode = LogJointPriorDensity-(LogPriorMode+LogJacobianMode);
   %
   SetLogText(strvcat(' ','Optimization completed!',[' * Time:                             ' StringTrim(datestr(now,14))],[' * Optimization time:                ' SecondsToHoursStr(MaxTime) ' (hrs:min:sec)'],[' * Number of iterations:             ' num2str(NumIter)],[' * Convergence value:                ' num2str(FuncDiff)],[' * max(abs(gradient)):               ' num2str(OptResults(NumIter,4))],[' * Total log prior of phi at mode:   ' num2str(LogJointPriorDensity,'%0.4f')],[' * Total log prior of theta at mode: ' num2str(LogJointPriorDensity-LogJacobianMode,'%0.4f')],[' * Log prior at mode:                ' num2str(LogPriorMode,'%0.4f')],[' * Log Jacobian at mode:             ' num2str(LogJacobianMode,'%0.4f')],[' * Log system prior at mode:         ' num2str(LogSystemPriorMode,'%0.4f')]));
   EstimationStatus = 1;
elseif strcmp(lower(AbortEstimation),'cancel')==1;
   %
   % posteriod mode estimation was aborted
   %
   SetLogText(strvcat(' ','Posterior mode estimation was aborted.'));
   return;
elseif (ReturnCode==0)&(strcmp(lower(AbortEstimation),'cancel')~=1)&(NumIter>MaxIter);
   %
   % maximum number of iterations was reached
   %
   txt = ['The maximum number of iterations was reached without csminwel converging. YADA has aborted from posterior mode estimation. Writing optimization information to file: "OptErrorReport-' DSGEModel.NameOfModel '.txt" in "' DSGEModel.OutputDirectory '".'];
   SetLogText(strvcat(' ','Maximum number of iteration reached.'));
   About(txt,'information','Optimization Problem',200,500,CurrINI);
   drawnow;
   return;
elseif ReturnCode==1;
   txt = ['Csminwel return code: zero gradient. YADA has aborted from posterior mode estimation. Writing optimization information to file: "OptErrorReport-' DSGEModel.NameOfModel '.txt" in "' DSGEModel.OutputDirectory '".'];
   SetLogText(strvcat(' ','Csminwel return code: zero gradient.'));
   About(txt,'information','Optimization Problem',200,500,CurrINI);
   drawnow;
   return;
elseif (ReturnCode==2)|(ReturnCode==4);
   txt = ['Csminwel return code: back and forth on step length never finished. YADA has aborted from posterior mode estimation. Writing optimization information to file: "OptErrorReport-' DSGEModel.NameOfModel '.txt" in "' DSGEModel.OutputDirectory '".'];
   SetLogText(strvcat(' ','Csminwel return code: back and forth on step length never finished.'));
   About(txt,'information','Optimization Problem',200,500,CurrINI);
   drawnow;
   return;
elseif ReturnCode==3;
   txt = ['Csminwel return code: smallest step still improving too slow. YADA has aborted from posterior mode estimation. Writing optimization information to file: "OptErrorReport-' DSGEModel.NameOfModel '.txt" in "' DSGEModel.OutputDirectory '".'];
   SetLogText(strvcat(' ','Csminwel return code: smallest step still improving too slow.'));
   About(txt,'information','Optimization Problem',200,500,CurrINI);
   drawnow;
   return;
elseif ReturnCode==5;
   txt = ['Csminwel return code: largest step still improving too fast. YADA has aborted from posterior mode estimation. Writing optimization information to file: "OptErrorReport-' DSGEModel.NameOfModel '.txt" in "' DSGEModel.OutputDirectory '".'];
   SetLogText(strvcat(' ','Csminwel return code: largest step still improving too fast.'));
   About(txt,'information','Optimization Problem',200,500,CurrINI);
   drawnow;
   return;
elseif ReturnCode==6;
   txt = ['Csminwel return code: smallest step still improving too slow, reversed gradient. YADA has aborted from posterior mode estimation. Writing optimization information to file: "OptErrorReport-' DSGEModel.NameOfModel '.txt" in "' DSGEModel.OutputDirectory '".'];
   SetLogText(strvcat(' ','Csminwel return code: smallest step still improving too slow, reversed gradient.'));
   About(txt,'information','Optimization Problem',200,500,CurrINI);
   drawnow;
   return;
elseif ReturnCode==7;
   txt = ['Csminwel return code: possible inaccuracy in inverse Hessian. YADA has aborted from posterior mode estimation. Writing optimization information to file: "OptErrorReport-' DSGEModel.NameOfModel '.txt" in "' DSGEModel.OutputDirectory '".'];
   SetLogText(strvcat(' ','Csminwel return code: possible inaccuracy in inverse Hessian.'));
   About(txt,'information','Optimization Problem',200,500,CurrINI);
   drawnow;
   return;
elseif ReturnCode==-1;
   SetLogText(strvcat(' ','Csminwel error: optimization ended abruptly.'));
   About(ErrorStr,'information','Optimization Problem',400,500,CurrINI);
   drawnow;
   return;
end;
%
SetLogText(strvcat(' ','Computing the inverse Hessian with finite differences...'));
%
% compute the finite difference estimate of the inverse Hessian
%
if IsOdd(DSGEModel.MaximizeAlgorithmValue)==1;
   %
   % for the transformed parameters
   %
   VechHessian = YADAHessianMatrix('logSystemPriorPhiDSGE',phiMode,StepLength,thetaPositions,ModelParameters,thetaDist,PriorDist,thetaIndex,UniformBounds,LowerBound,AIMData,DSGEModel,CurrINI);
   HessianMat = GeneralizedCholeski(reshape(VechHessian,NumParam,NumParam));
   InverseHessianFinDiff = HessianMat\eye(NumParam);
else;
   %
   % for the original parameters
   %
   VechHessian = YADAHessianMatrix('logSystemPriorThetaDSGE',thetaMode,StepLength,thetaPositions,ModelParameters,thetaDist,PriorDist,thetaIndex,ParameterBounds,LowerBound,AIMData,DSGEModel,CurrINI);
   HessianMat = GeneralizedCholeski(reshape(VechHessian,NumParam,NumParam));
   InverseHessianFinDiff = HessianMat\eye(NumParam);
   InverseHessianFinDiff = (ThetaPhiPartial\InverseHessianFinDiff)/ThetaPhiPartial;
end;
%
% compute the Laplace approximation
%
phi = phiMode;
if IsPositiveDefinite(InverseHessianFinDiff)==1;
   LogMargLikeSystemLaplace = MargLikeLaplace(LogJointPriorDensity,InverseHessianFinDiff,1);
else;
   LogMargLikeSystemLaplace = MargLikeLaplace(LogJointPriorDensity,InverseHessian,1);
end;
%
SetLogText(strvcat([' * Log sys prior marg like, Laplace: ' num2str(LogMargLikeSystemLaplace,'%0.4f')],' ','Checking the optimum...',' * Transformed parameters'));
%
% make sure that ModelParameters is updated with the prior mode value of theta
%
ModelParameters = ThetaToModelParameters(ModelParameters,thetaMode,thetaPositions);
%
% update the additional parameters
%
[thetaCalibratedMode,thetaAdditionalMode] = ReadAdditionalParameterValues(ModelParameters,ParameterNames);
%
% Check if GraphDirectory exists
%
GraphDirectory = FixFilePath([DSGEModel.OutputDirectory '\graphs']);
MinMaxGrid = zeros(NumParam,2);
stat = MakeDir(DSGEModel.OutputDirectory,'graphs');
%
% create lower and upper bounds from InverseHessian and phiMode
%
MinMaxGrid(:,1) = phiMode-(DSGEModel.GridWidth*sqrt(diag(InverseHessian)));
MinMaxGrid(:,2) = phiMode+(DSGEModel.GridWidth*sqrt(diag(InverseHessian)));
phiFittedStd = VillaniCheckOptimum('logSystemPriorPhiDSGE',phiMode,[],MinMaxGrid,InverseHessian,NumGridPoints,0,[],ParameterNames.estimated,'Checking Curvature at Prior Mode - Transformed Parameters',CurrINI,GraphDirectory,'phi','','',0,0, ...
   thetaPositions,ModelParameters,thetaDist,PriorDist,thetaIndex,UniformBounds,LowerBound,AIMData,DSGEModel,CurrINI);
%
SetLogText(' * Original parameters');
MinMaxGrid(:,1) = PhiToTheta(MinMaxGrid(:,1),thetaIndex,UniformBounds,LowerBound);
MinMaxGrid(:,2) = PhiToTheta(MinMaxGrid(:,2),thetaIndex,UniformBounds,LowerBound);
thetaFittedStd = VillaniCheckOptimum('logSystemPriorThetaDSGE',thetaMode,[],MinMaxGrid,ThetaPhiPartial*InverseHessian*ThetaPhiPartial,NumGridPoints,0,[],ParameterNames.estimated,'Checking Curvature at Prior Mode - Original Parameters',CurrINI,GraphDirectory,'theta','','',0,0, ...
   thetaPositions,ModelParameters,thetaDist,PriorDist,thetaIndex,ParameterBounds,LowerBound,AIMData,DSGEModel,CurrINI);
%
% now we can save the results to the prior mode file
%
ModeFileDir = FixFilePath([DSGEModel.OutputDirectory '\mode']);
stat = MakeDir(DSGEModel.OutputDirectory,'mode');
if stat==1;
   SetLogText(strvcat(' ','Saving prior mode data to disk using directory:',['   "' ModeFileDir '"...'],' ','System prior mode estimation completed.'));
   ModeFile = FixFilePath([ModeFileDir '\SystemPriorMode-' DSGEModel.NameOfModel '.mat']);
   save(ModeFile,'phi','LogMargLikeSystemLaplace','InverseHessian','InverseHessianFinDiff','phiFittedStd','thetaFittedStd','R','theta','thetaMode','ParameterNames','thetaDist','PriorDist','LowerBound','thetaPositions','ModelParameters','thetaIndex','UniformBounds');
else;
   SetLogText(strvcat(' ','Unable to create the directory:',['   "' ModeFileDir '"...'],' ','System prior mode results have not been saved to disk.'));
end;

% end of PriorModeEstimation.m
%
