function SpatDist = DSGESpatDistDistribution(DSGEModel,CurrINI,SelectedParameters,NumDraws,TotalDraws,CurrChain,IsPosterior,Weights)
% DSGESpatDistDistribution: Computes the mean, quantiles and shortest
%                           confidence bands of the spatial distortions
%
% USAGE:
%
%       SpatDist = DSGESpatDistDistribution(DSGEModel,CurrINI,SelectedParameters,NumDraws,TotalDraws,CurrChain,IsPosterior,Weights)
%
% REQUIRED INPUT:  DSGEModel (structure) with fields containing all the information about a DSGE Model for YADA.
%
%                  CurrINI (structure), with initialization information.
%
%                  SelectedParameters (vector) of dimension NumParam which indicates which parameters can vary from
%                                     one draw to the next (1) and which parameters are fixed at posterior mode (0).
%
%                  NumDraws (integer) with the number of effective draws from the posterior distribution of the
%                                     parameters.
%
%                  TotalDraws (integer) with the number of draws from the posterior distribution of the parameters
%                                     that could have been used.
%
%                  CurrChain (integer) with the number for the current Markov chain.
%
%                  IsPosterior (boolean) that is 1 if draws from the posterior distribution are used and 0
%                                     if draws from the prior are supplied.
%
%                  Weights (vector) of dimension NumDraws with the weights from the SMC with likelihood tempering
%                                     posterior sampler. Can be empty.
%
% REQUIRED OUTPUT: SpatDist (structure) with fields Mean, Quantiles and ShortestConfidence.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   DG-R/MPR
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2017 European Central Bank.
%
%                       First version: June 12, 2009.
%                        This version: September 4, 2017.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 08-07-2009: Updated the documentation.
%
% * 05-01-2010: Updated the documentation.
%
% * 05-01-2011: Updated the documentation.
%
% * 12-01-2011: The function now supports unix and mac paths.
%
% * 05-01-2012: Updated the documentation.
%
% * 03-01-2013: Updated the documentation.
%
% * 27-01-2014: Updated the documentation.
%
% * 13-01-2015: Updated the documentation.
%
% * 22-02-2016: Updated the documentation.
%
% * 22-03-2017: Updated the documentation.
%
% * 04-09-2017: Added the Weights input variable.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% determine parameters
%
SelParamStr = strrep(num2str(SelectedParameters),' ','');
NumFiles = ceil(NumDraws/min(NumDraws,1000));
NumQuants = length(DSGEModel.Percentiles);
if isempty(Weights)==1;
   Weights = ones(NumDraws,1);
end;
%
if IsPosterior==1;
   file = FixFilePath([DSGEModel.OutputDirectory '\spatdist\DSGE-SD-' DSGEModel.NameOfModel '-' int2str(CurrChain) '-' SelParamStr '-1-' int2str(NumFiles) '.' int2str(NumDraws) '-' int2str(TotalDraws) '.mat']);
else;
   file = FixFilePath([DSGEModel.OutputDirectory '\spatdist\DSGE-SD-Prior-' DSGEModel.NameOfModel '-' SelParamStr '-1-' int2str(NumFiles) '.' int2str(TotalDraws) '.mat']);
end;
%
SDData = load(file);
[n,r] = size(SDData.SptlDstrtns(1).SpatDist);
%
% initialize output
%
SpatDist.Mean = zeros(n,r);
for i=1:NumQuants;
   SpatDist.Quantiles(i).Mean = zeros(n,r);
   SpatDist.Quantiles(i).percent = DSGEModel.Percentiles(i);
end;
for i=1:(NumQuants/2);
   SpatDist.ShortestConfidence(i).UpperBound = zeros(n,r);
   SpatDist.ShortestConfidence(i).LowerBound = zeros(n,r);
   SpatDist.ShortestConfidence(i).percent = DSGEModel.Percentiles(NumQuants-i+1)-DSGEModel.Percentiles(i);
end;
PriorHeader = '';
if IsPosterior==0;
   PriorHeader = ' - Prior';
end;
%
% setup a wait dialog
%
txt = ['Please wait while YADA computes the mean spatial distortions as well as percentile values. The data are loaded sequentially from disk and it may therefore take some time. Current pair: ' StringTrim(DSGEModel.VariableNames(1,:)) ',' StringTrim(DSGEModel.StateVariableNames(1,:)) '.'];
WaitHandle = WaitDLG(txt,'information',['Spatial Distortions' PriorHeader],500,200,CurrINI,0);
WaitControls = get(WaitHandle,'UserData');
drawnow;
pause(0.02);
%
for i=1:n;
   for j=1:r;
      %
      % update the wait dialog
      %
      txt = ['Please wait while YADA computes the mean spatial distortions as well as percentile values. The data are loaded sequentially from disk and it may therefore take some time. Current pair: ' StringTrim(DSGEModel.VariableNames(i,:)) ',' StringTrim(DSGEModel.StateVariableNames(j,:)) '.'];
      set(WaitControls.text,'String',txt);
      drawnow;
      %
      % load the results
      %
      Temp = zeros(NumDraws,1);
      CurrSaves = 0;
      for ThisSave=1:NumFiles;
         if IsPosterior==1;
            file = FixFilePath([DSGEModel.OutputDirectory '\spatdist\DSGE-SD-' DSGEModel.NameOfModel '-' int2str(CurrChain) '-' SelParamStr '-' int2str(ThisSave) '-' int2str(NumFiles) '.' int2str(NumDraws) '-' int2str(TotalDraws) '.mat']);
         else;
            file = FixFilePath([DSGEModel.OutputDirectory '\spatdist\DSGE-SD-Prior-' DSGEModel.NameOfModel '-' SelParamStr '-' int2str(ThisSave) '-' int2str(NumFiles) '.' int2str(TotalDraws) '.mat']);
         end;
         SDData = load(file);
         NumSaves = length(SDData.SptlDstrtns);
         for s=1:NumSaves;
            Temp(CurrSaves+s,1) = SDData.SptlDstrtns(s).SpatDist(i,j);
         end;
         CurrSaves = CurrSaves+NumSaves;
         drawnow;
      end;
      %
      % compute the mean
      %
      SpatDist.Mean(i,j) = (1/NumDraws)*sum(Temp.*Weights);
      %
      % sort the results
      %
      Temp = sort(Temp);
      %
      % determine the quantiles
      %
      for l=1:NumQuants;
         if DSGEModel.Percentiles(l)<50;
            QuantVal = ceil((DSGEModel.Percentiles(l)/100)*NumDraws);
         else;
            QuantVal = floor((DSGEModel.Percentiles(l)/100)*NumDraws);
         end;
         SpatDist.Quantiles(l).Mean(i,j) = Temp(QuantVal,1);
      end;
      %
      % determine shortest confidence bands
      %
      for l=1:floor(NumQuants/2);
         %
         % determine number of elements in confidence band
         %
         ConfDraws = floor((SpatDist.ShortestConfidence(l).percent/100)*NumDraws);
         %
         UpperBound = Temp(ConfDraws,1);
         LowerBound = Temp(1,1);
         for k=2:NumDraws-ConfDraws+1;
            if (Temp(ConfDraws+k-1,1)-Temp(k,1))<(UpperBound-LowerBound);
               UpperBound = Temp(ConfDraws+k-1,1);
               LowerBound = Temp(k,1);
            end;
         end;
         %
         % store the shortest confidence bands
         %
         SpatDist.ShortestConfidence(l).UpperBound(i,j) = UpperBound;
         SpatDist.ShortestConfidence(l).LowerBound(i,j) = LowerBound;
      end;
   end;
end;
%
% close the wait dialog
%
set(WaitControls.text,'UserData','done');
delete(WaitHandle);
drawnow;
pause(0.02);

%
% end of DSGESpatDistDistribution.m
%
