function SpatialDistortionsDLG(selector,DSGEModel,CurrINI,SpatDist,TypeStr)
% SpatialDistortionsDLG: Displays scatter-plots of the spatial distortions
%
% USAGE:
%
%       SpatialDistortionsDLG(selector,DSGEModel,CurrINI,SpatDist,TypeStr)
%
% REQUIRED INPUT: selector (string) that takes on the values "init", "showgraph", or "done".
%
%                 DSGEModel (structure) with fields containing all the information about a DSGE Model for YADA.
%
%                 CurrINI (structure) with initialization information.
%
%                 SpatDist (matrix or structure) with the results on the spatial distortions.
%
%                 TypeStr (string) that takes on the values "Initial Values", "Posterior Mode",
%                         "Prior Distribution", or "Posterior Distribution".
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   DG-R/MPR
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2017 European Central Bank.
%
%                       First version: June 12, 2009.
%                        This version: March 22, 2017.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 08-07-2009: Updated the documentation.
%
% * 05-01-2010: Updated the documentation.
%
% * 05-01-2011: Updated the documentation.
%
% * 05-01-2012: Updated the documentation.
%
% * 03-01-2013: Updated the documentation.
%
% * 27-01-2014: Updated the documentation.
%
% * 13-01-2015: Updated the documentation.
%
% * 09-06-2015: The code takes the CurrINI.GraphicsRoot field into account.
%
% * 10-09-2015: The function is now ready for Matlab 2014b and later.
%
% * 14-09-2015: Fixed the units for the scatter plot function with Matlab 2014b and later.
%
% * 22-02-2016: Updated the documentation.
%
% * 22-03-2017: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

if strcmp(selector,'init')==1;
   DSGEModel.SpatDist = SpatDist;
   DSGEModel.TypeStr = TypeStr;
else;
   SDGUI = findobj('Type','figure','Tag','SpatialDistortionsDLG');
   SDControls = get(SDGUI,'UserData');
   DSGEModel = get(SDControls.show,'UserData');
   CurrINI = get(SDControls.variablex,'UserData');
end;
%
switch selector
   case 'init'
      %
      % create the dialog
      %
      SDGUI = figure('Color',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
                        'FileName','SpatialDistortionsDLG.m', ...
                        'MenuBar','none', ...
                        'PaperUnits','points', ...
                        'Units','pixels', ...
                        'Position',[(CurrINI.scrsz(3)-500)/2 ..
                                    (CurrINI.scrsz(4)-120)/2 500 120], ...
                        'Tag','SpatialDistortionsDLG', ...
                        'Visible','off', ...
                        'Resize','off', ...
                        'Name',['Spatial Distortions - ' TypeStr], ...
                        'NumberTitle','off', ...
                        'CloseRequestFcn','SpatialDistortionsDLG done', ...
                        'ToolBar','none');
      %
      if MatlabNumber>=7.0;
         set(SDGUI,'DockControl','off');
      end;
      %
      % Outer axis
      %
      AxesBox([2 2 498 110],'Graphics',45,[0.5 1],'on',CurrINI);
      %
      % Check where the sample of data actually begins
      %
      [NewStartYear,NewStartPeriod] = AdjustSampleStart(DSGEModel.SubBeginYear,DSGEModel.SubBeginPeriod,DSGEModel.DataFrequency,DSGEModel.KalmanFirstObservation-1);
      %
      % display the sample
      %
      SDontrols.sample = uicontrol('Units','pixels', ...
            'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
            'Position',[24 75 360 20], ...
            'FontSize',CurrINI.gui_fontsize, ...
            'FontName',CurrINI.gui_fontname, ...
            'FontWeight',CurrINI.gui_fontweight, ...
            'FontAngle',CurrINI.gui_fontangle, ...
            'Style','text', ...
            'String',['Selected Sample: ' NewStartYear ':' NewStartPeriod ' - ' DSGEModel.SubEndYear ':' DSGEModel.SubEndPeriod], ...
            'HorizontalAlignment','left', ...
            'Tag','SampleText');
      %
      % Select variables controls
      %
      SDControls.variableytext = uicontrol('Units','pixels', ...
            'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
            'Position',[24 45 176 20], ...
            'FontSize',CurrINI.gui_fontsize, ...
            'FontName',CurrINI.gui_fontname, ...
            'FontWeight',CurrINI.gui_fontweight, ...
            'FontAngle',CurrINI.gui_fontangle, ...
            'Style','text', ...
            'String','Select variable for Y-axis:', ...
            'HorizontalAlignment','left', ...
            'Tag','VariableYText');
      %
      SDControls.variabley = uicontrol('Units','pixels', ...
            'BackgroundColor',[1 1 1], ...
            'Position',[200 50 165 20], ...
            'Style','popupmenu', ...
            'FontSize',CurrINI.gui_fontsize, ...
            'FontName',CurrINI.gui_fontname, ...
            'FontWeight',CurrINI.gui_fontweight, ...
            'FontAngle',CurrINI.gui_fontangle, ...
            'UserData',CurrINI, ...
            'String',DSGEModel.VariableNames, ...
            'HorizontalAlignment','center', ...
            'Tag','VariableYPopup',...
            'Value',1);
      %
      SDControls.variablextext = uicontrol('Units','pixels', ...
            'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
            'Position',[24 15 176 20], ...
            'FontSize',CurrINI.gui_fontsize, ...
            'FontName',CurrINI.gui_fontname, ...
            'FontWeight',CurrINI.gui_fontweight, ...
            'FontAngle',CurrINI.gui_fontangle, ...
            'Style','text', ...
            'String','Select variable for X-axis:', ...
            'HorizontalAlignment','left', ...
            'Tag','VariableXText');
      %
      SDControls.variablex = uicontrol('Units','pixels', ...
            'BackgroundColor',[1 1 1], ...
            'Position',[200 20 165 20], ...
            'Style','popupmenu', ...
            'FontSize',CurrINI.gui_fontsize, ...
            'FontName',CurrINI.gui_fontname, ...
            'FontWeight',CurrINI.gui_fontweight, ...
            'FontAngle',CurrINI.gui_fontangle, ...
            'UserData',CurrINI, ...
            'String',DSGEModel.VariableNames, ...
            'HorizontalAlignment','center', ...
            'Tag','VariableXPopup',...
            'Value',2);
      %
      % show graph button
      %
      SDControls.show = uicontrol('Units','pixels', ...
            'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
            'Position',[395 50 90 20], ...
            'String','Display', ...
            'FontSize',CurrINI.gui_fontsize, ...
            'FontName',CurrINI.gui_fontname, ...
            'FontWeight',CurrINI.gui_fontweight, ...
            'FontAngle',CurrINI.gui_fontangle, ...
            'CallBack','SpatialDistortionsDLG showgraph', ...
            'UserData',DSGEModel, ...
            'Enable','on', ...
            'Tag','Done');
      %
      % done button
      %
      SDControls.done = uicontrol('Units','pixels', ...
            'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
            'Position',[395 20 90 20], ...
            'String','Done', ...
            'FontSize',CurrINI.gui_fontsize, ...
            'FontName',CurrINI.gui_fontname, ...
            'FontWeight',CurrINI.gui_fontweight, ...
            'FontAngle',CurrINI.gui_fontangle, ...
            'CallBack','SpatialDistortionsDLG done', ...
            'Enable','on', ...
            'UserData','waiting', ...
            'Tag','Done');
      %
      % set UserData property of the dialog
      %
      set(ScatterPostDrawsGUI,'UserData',SDControls,'HandleVisibility','callback');
   case 'showgraph'
      %
      % check which variable to plot
      %
      VariableValueX = get(SDControls.variablex,'Value');
      CurrVariableX = StringTrim(DSGEModel.VariableNames(VariableValueX,:));
      VariableValueY = get(SDControls.variabley,'Value');
      CurrVariableY = StringTrim(DSGEModel.VariableNames(VariableValueY,:));
      if VariableValueX<size(DSGEModel.VariableNames,1);
         set(SDControls.variablex,'Value',VariableValueX+1);
      else;
         set(SDControls.variablex,'Value',1);
         if CurrVariableY<size(DSGEModel.VariableNames,1);
            set(SDControls.variabley,'Value',VariableValueY+1);
         else;
            set(SDControls.variabley,'Value',2);
         end;
      end;
      %
      % create the figure
      %
      FigHandle = figure('Visible','off', ...
                         'Position',[(CurrINI.scrsz(3)-650)/2 (CurrINI.scrsz(4)-500)/2 650 500], ...
                         'Units','pixels', ...
                         'Tag',['ScatterPlot-' CurrVariableX '-' CurrVariableY], ...
                         'Name',['Spatial Distortions Of ' CurrVariable ' And ' CurrVariableY ' - ' DSGEModel.TypeStr]);
      %
      SetFigureProperties(FigHandle,CurrINI);
      %
      % create the scatter-plot
      %
      if isstruct(DSGEModel.SpatDist)==1;
         x = DSGEModel.SpatDist.Mean(VariableValueX,:);
         y = DSGEModel.SpatDist.Mean(VariableValueY,:);
      else;
         x = DSGEModel.SpatDist(VariableValueX,:);
         y = DSGEModel.SpatDist(VariableValueY,:);
      end;
      if MatlabNumber>8.3;
         ScatHandle = scatter(x,y,25,'filled');
      else;
         if MatlabNumber>=7;
            ScatHandle = scatter('v6',x,y,4,'filled');
         else;
            ScatHandle = scatter(x,y,4,'filled');
         end;
      end;
      %
      % Show title and labels
      %
      SetAxesFonts(CurrINI,'Title',['Spatial distortions for ' lower(DSGEModel.TypeStr)],'XLabel',CurrVariableX,'Ylabel',CurrVariableY);
      if strcmp(get(gca,'YGrid'),'on')==1;
         set(gca,'XGrid','on');
      end;
      %
      % make the figure visible
      %
      SetToolbarImages(FigHandle,CurrINI.images);
      set(FigHandle,'Visible','on','CloseRequestFcn','delete(gcf); drawnow; pause(0.02);');
      drawnow;
      pause(0.02);
   case 'done'
      set(SDControls.done,'UserData','done');
      delete(SDGUI);
      pause(0.02);
      drawnow;
end;
%
if strcmp(selector,'init')==1;
   %
   set(SDGUI,'Visible','on');
   drawnow;
   if MatlabNumber>=7;
      uicontrol(SDControls.done);
   end;
   waitfor(SDControls.done,'UserData','done');
end;

%
% end of SpatialDistortionsDLG.m
%