function phiProp = MixedDistributionDraw(phi,phiTilde,CholSigma,CholDiagSigma,alpha,cstar,NumParam)
% MixedDistributionDraw: Computes a parameter draws from a mixture of normal distributions
%                        for SMC algorithms.
%
% USAGE:
%
%       phiProp = MixedDistributionDraw(phi,phiTilde,CholSigma,CholDiagSigma,alpha,cstar,NumParam)
%
% REQUIRED INPUT:  phi (vector) of dimension NumParam x 1 with location values
%                      for the parameter draw.
%
%                  phiTilde (vector) of dimension NumParam X 1 with other location values
%                      for the parameter draw.
%
%                  CholSigma (matrix) of dimension NumParam x NumParam with the Choleski
%                      decomposition of the basic covariance matrix for the parameter draw.
%
%                  CholDiagSigma (matrix) of dimension NumParam x NumParam with the Choleski
%                      decomposition of the diagonal of the basic covariance matrix for the
%                      parameter draw.
%
%                  alpha (scalar) with the mixing weight for the draws. Takes values between
%                      zero and unity.
%
%                  cstar (scalar) with the scaling factor for the covariance matrix for the
%                      parameter draw.
%
%                  NumParam (integer) with the number of parameters.
%
% REQUIRED OUTPUT: phiProp (vector) with the parameter draw.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   DG-R/MPR
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2017 European Central Bank.
%
%                       First version: August 18, 2017.
%                        This version: August 28, 2017.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 21-08-2017: Updated the documentation.
%
% * 28-08-2017: Changed the single input variable "SigmaphiTilde" to the two input variables "CholSigma"
%               and "CholDiagSigma" to allow for faster computations.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% compute the alpha part
%
phiProp = phi+(cstar*CholSigma*randn(NumParam,1));
if alpha<1;
   phi1 = phi+(cstar*CholDiagSigma*randn(NumParam,1));
   phi2 = phiTilde+(cstar*CholSigma*randn(NumParam,1));
   phiProp = (alpha*phiProp)+(((1-alpha)/2)*(phi1+phi2));
end;

%
% end of MixedDistributionDraw.m
%
