function PredEventData = CalculatePredictionEvents(PredPaths,PredictionEvent,Weight)
% CalculatePredictionEvents: Computes the frequency with which the prediction evenst are true
%                            for the prediction paths.
%
% USAGE:
%
%       PredEventData = CalculatePredictionEvents(PredPaths,PredictionEvent,Weight)
%
% REQUIRED INPUT:  PredPaths (n x h x NumPaths) 3D matrix with the NumPaths possible paths for the n
%                            endogenous variables.
%
%                  PredictionEvent (n x 3) matrix with the prediction event information. The first
%                            column holds the upper bound, the second the lower bound for the event,
%                            while the third holds the event length in periods.
%
% OPTIONAL INPUT:  Weight (scalar) with a positive value for weighting in the summations. Default is
%                            unity, but alternative values may be used when supporting the SMC with
%                            likelihood tempering posterior sampler.
%
% REQUIRED OUTPUT: PredEventData (n x 8) matrix where the first column holds the number of times
%                            the PredPaths data satisfies the event, the second hold the number
%                            of times the PredPaths data is below the lower bound of the event,
%                            the third the number of time the data is above the upper bound,
%                            while the fourth column holds the number of times the event has been
%                            tested. The fourth and fifth column measure the average deviation
%                            from the lower bound when below the lower bound and the square of
%                            this, while the sizth and seventh columns hold similar data for
%                            the upper bound of the event when the variables are above the
%                            upper bound.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   DG-R/MPR
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2017 European Central Bank.
%
%                       First version: May 3, 2007.
%                        This version: August 25, 2017.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 04-05-2007: The function now tests if the upper bound is satisfied as well as the lower bound.
%
% * 06-06-2007: The PredEventData now has the second column function. Previously, only the first
%               and the "third" were included.
%
% * 13-11-2007: Updated the documentation.
%
% * 04-04-2008: Updated the documentation.
%
% * 23-05-2008: Updated the documentation.
%
% * 22-01-2009: Updated the documentation.
%
% * 05-01-2010: Updated the documentation.
%
% * 17-11-2010: Added 4 columns to "PredEventData".
%
% * 19-11-2010: Added the "missing" third column.
%
% * 05-01-2011: Updated the documentation.
%
% * 05-01-2012: Updated the documentation.
%
% * 03-01-2013: Updated the documentation.
%
% * 28-01-2014: Updated the documentation.
%
% * 14-01-2015: Updated the documentation.
%
% * 22-02-2016: Updated the documentation.
%
% * 29-03-2017: Updated the documentation.
%
% * 25-08-2017: Added the optional input variable "Weight".
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize
%
[n,h,NumPaths] = size(PredPaths);
PredEventData = zeros(n,8);
if nargin==2;
   Weight = 1;
end;
%
% we check the events variable by variable
%
for i=1:n;
   EventDataUpper = PredictionEvent(i,1);
   EventDataLower = PredictionEvent(i,2);
   EventLength = round(PredictionEvent(i,3));
   NumTests = h-EventLength+1;
   for p=1:NumPaths;
      for j=1:NumTests;
         PredData = PredPaths(i,j:EventLength+j-1,p);
         PredEventData(i,1) = PredEventData(i,1)+min(min(PredData>=EventDataLower),min(PredData<=EventDataUpper));
         PredEventData(i,2) = PredEventData(i,2)+min(PredData<EventDataLower);
         PredEventData(i,3) = PredEventData(i,3)+min(PredData>EventDataUpper);
         if (isinf(EventDataLower)==0)&(all(PredData<EventDataLower)==1);
            PredEventData(i,5) = PredEventData(i,5)+((Weight/EventLength)*sum(PredData-EventDataLower));
            PredEventData(i,6) = PredEventData(i,6)+((Weight/EventLength)*sum((EventDataLower-PredData).^2));
         elseif (isinf(EventDataUpper)==0)&(all(PredData>EventDataUpper)==1);
            PredEventData(i,7) = PredEventData(i,7)+((Weight/EventLength)*sum(PredData-EventDataUpper));
            PredEventData(i,8) = PredEventData(i,8)+((Weight/EventLength)*sum((PredData-EventDataUpper).^2));
         end;
      end;
   end;
   PredEventData(i,4) = NumTests*NumPaths;
end;

%
% end of CalculatePredictionEvents.m
%
