function [SOmega,Omega,Quantiles] = DSGECoherenceDistribution(DSGEModel,CurrINI,TypeStr,VariableNames,NumDraws,TotalDraws,CurrChain,IsPosterior,Weights)
% DSGECoherenceDistribution: Computes the marginal distributions of the coherence statistics for the observed or the
%                            state variables based on parameter draws from the prior or the posterior distribution.
%                            of a DSGE model.
%
% USAGE:
%
%       [SOmega,Omega,Quantiles] = DSGECoherenceDistribution(DSGEModel,CurrINI,TypeStr,VariableNames,NumDraws, ...
%                                                            TotalDraws,CurrChain,IsPosterior,Weights)
%
% REQUIRED INPUT:  DSGEModel (structure) with fields containing all the information about a DSGE Model for YADA.
%
%                  CurrINI (structure), with initialization information.
%
%                  TypeStr (string vector) that takes the value "Observed" or "State".
%
%                  VariableNames (string matrix) with the names of the variables.
%
%                  NumDraws (integer) with the number of effective draws from the posterior distribution of the
%                          parameters.
%
%                  TotalDraws (integer) with the total number of parameter draws that could have been used.
%
%                  CurrChain (integer) with the number for the current Markov chain.
%
%                  IsPosterior (boolean) that is 1 if draws from the posterior distribution are used and 0
%                          if draws from the prior are supplied.
%
%                  Weights (vector) of dimension NumDraws x 1 with the importance weights from the SMC with
%                                 likelihood tempering posterior sampler.
%
% REQUIRED OUTPUT: SOmega (matrix) of dimension m(m-1)/2 x f with the m(m-1)/2 unique coherence statistics for
%                          f frequencies from 0 to pi of the m variables.
%
%                  Omega (vector) of dimension f with the frequencies from 0 to pi.
%
%                  Quantiles (vector structure) with fields "SOmega" and "percent".
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   DG-R/MPR
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2017 European Central Bank.
%
%                       First version: May 14, 2010.
%                        This version: September 4, 2017.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 16-05-2010: Updated the documentation.
%
% * 22-06-2010: Updated the documentation.
%
% * 05-01-2011: Updated the documentation.
%
% * 12-01-2011: The function now supports unix and mac paths.
%
% * 05-01-2012: Updated the documentation.
%
% * 29-08-2012: Added support for higest probability density confidence bands.
%
% * 03-01-2013: Updated the documentation.
%
% * 03-09-2013: Added "try-catch" code around possibly too large matrices.
%
% * 28-01-2014: Updated the documentation.
%
% * 14-01-2015: Updated the documentation.
%
% * 22-02-2016: Updated the documentation.
%
% * 29-03-2017: Updated the documentation.
%
% * 24-08-2017: Added the Weights input variable.
%
% * 04-09-2017: Speeded up the Weights computations.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize
%
NumFiles = ceil(NumDraws/min(NumDraws,1000));
if strcmp(lower(TypeStr),'observed')==1;
   m = DSGEModel.n;
else;
   m = length(DSGEModel.StateVariablePositions);
end;
NumQuants = length(DSGEModel.Percentiles);
if isempty(Weights)==1;
   Weights = ones(NumDraws,1);
end;
%
% load the first file to determine Omega
%
if IsPosterior==1;
   file = FixFilePath([DSGEModel.OutputDirectory '\coherence\DSGE-Coherence-' TypeStr 'Variables-' DSGEModel.NameOfModel '-' num2str(CurrChain,'%0.0f') '-1-' num2str(NumFiles,'%0.0f') '.' num2str(NumDraws,'%0.0f') '-' num2str(TotalDraws,'%0.0f') '.mat']);
else;
   file = FixFilePath([DSGEModel.OutputDirectory '\coherence\DSGE-Coherence-Prior-' TypeStr 'Variables-' DSGEModel.NameOfModel '-1-' num2str(NumFiles,'%0.0f') '.' num2str(NumDraws,'%0.0f') '.mat']);
end;
%
CoherenceData = load(file);
Omega = CoherenceData.Omega;
f = length(Omega);
%
% intialize output
%
SOmega = zeros(m*(m-1)/2,f);
for i=1:NumQuants;
   Quantiles(i).SOmega = zeros(m*(m-1)/2,f);
   Quantiles(i).percent = DSGEModel.Percentiles(i);
end;
%
PriorHeader = '';
if IsPosterior==0;
   PriorHeader = ' For Prior';
end;
Weightsf = Weights*ones(1,f);
%
%
% set up a wait dialog
%
if IsPosterior==1;
   txt = ['Please wait while YADA computes the posterior mean coherence statistics as well as percentile values for the ' lower(TypeStr) ' variables. The data are loaded sequentially from disk and it may therefore take some time. Current variable pair: (' StringTrim(VariableNames(1,:)) ',' StringTrim(VariableNames(2,:)) ')'];
else;
   txt = ['Please wait while YADA computes the prior mean coherence statistics as well as percentile values for the ' lower(TypeStr) ' variables. The data are loaded sequentially from disk and it may therefore take some time. Current variable pair: (' StringTrim(VariableNames(1,:)) ',' StringTrim(VariableNames(2,:)) ')'];
end;
WaitHandle = WaitDLG(txt,'information',['Coherence' PriorHeader ' - ' TypeStr ' Variables'],500,150,CurrINI,0);
WaitControls = get(WaitHandle,'UserData');
drawnow;
pause(0.02);
%
num = 0;
for i=1:m-1;
   for j=i+1:m;
      num = num+1;
      if IsPosterior==1;
         txt = ['Please wait while YADA computes the posterior mean coherence statistics as well as percentile values for the ' lower(TypeStr) ' variables. The data are loaded sequentially from disk and it may therefore take some time. Current variable pair: (' StringTrim(VariableNames(i,:)) ',' StringTrim(VariableNames(j,:)) ')'];
      else;
         txt = ['Please wait while YADA computes the prior mean coherence statistics as well as percentile values for the ' lower(TypeStr) ' variables. The data are loaded sequentially from disk and it may therefore take some time. Current variable pair: (' StringTrim(VariableNames(i,:)) ',' StringTrim(VariableNames(j,:)) ')'];
      end;
      set(WaitControls.text,'String',txt);
      drawnow;
      %
      try;
         TempVar = zeros(NumDraws,f);
         CurrSaves = 0;
         for ThisSave=1:NumFiles;
            if IsPosterior==1;
               file = FixFilePath([DSGEModel.OutputDirectory '\coherence\DSGE-Coherence-' TypeStr 'Variables-' DSGEModel.NameOfModel '-' num2str(CurrChain,'%0.0f') '-' num2str(ThisSave,'%0.0f') '-' num2str(NumFiles,'%0.0f') '.' num2str(NumDraws,'%0.0f') '-' num2str(TotalDraws,'%0.0f') '.mat']);
            else;
               file = FixFilePath([DSGEModel.OutputDirectory '\coherence\DSGE-Coherence-Prior-' TypeStr 'Variables-' DSGEModel.NameOfModel '-' num2str(ThisSave,'%0.0f') '-' num2str(NumFiles,'%0.0f') '.' num2str(NumDraws,'%0.0f') '.mat']);
            end;
            CoherenceData = load(file);
            Coherence = CoherenceData.Coherence;
            NumSaves = length(Coherence);
            if strcmp(lower(TypeStr),'observed')==1;
               for s=1:NumSaves;
                  TempVar(CurrSaves+s,:) = Coherence(s).YCoherence(num,:);
               end;
            else;
               for s=1:NumSaves;
                  TempVar(CurrSaves+s,:) = Coherence(s).SCoherence(num,:);
               end;
            end;
            CurrSaves = CurrSaves+NumSaves;
            drawnow;
         end;
         %
         % compute the mean
         %
         SOmega(num,:) = (1/NumDraws)*sum(TempVar.*Weightsf,1);
         %
         % sort each column
         %
         TempVar = sort(TempVar,1);
         %
         if DSGEModel.ConfidenceRegionMethod==1;
            for k=1:NumQuants;
               if DSGEModel.Percentiles(k)<50;
                  QuantVal = ceil((DSGEModel.Percentiles(k)/100)*NumDraws);
               else;
                  QuantVal = floor((DSGEModel.Percentiles(k)/100)*NumDraws);
               end;
               Quantiles(k).SOmega(num,:) = TempVar(QuantVal,:);
            end;
         else;
            for k=1:floor(NumQuants/2);
               ConfDraws = floor((DSGEModel.Percentiles(NumQuants-k+1)-DSGEModel.Percentiles(k))*NumDraws/100);
               for t=1:f;
                  Bounds = HighestProbabilityDensityBand(TempVar(:,t),ConfDraws);
                  Quantiles(k).SOmega(num,t) = Bounds(1);
                  Quantiles(NumQuants-k+1).SOmega(num,t) = Bounds(2);
               end;
            end;
         end;
      catch;
         for t=1:f;
            TempVar = zeros(NumDraws,1);
            CurrSaves = 0;
            for ThisSave=1:NumFiles;
               if IsPosterior==1;
                  file = FixFilePath([DSGEModel.OutputDirectory '\coherence\DSGE-Coherence-' TypeStr 'Variables-' DSGEModel.NameOfModel '-' num2str(CurrChain,'%0.0f') '-' num2str(ThisSave,'%0.0f') '-' num2str(NumFiles,'%0.0f') '.' num2str(NumDraws,'%0.0f') '-' num2str(TotalDraws,'%0.0f') '.mat']);
               else;
                  file = FixFilePath([DSGEModel.OutputDirectory '\coherence\DSGE-Coherence-Prior-' TypeStr 'Variables-' DSGEModel.NameOfModel '-' num2str(ThisSave,'%0.0f') '-' num2str(NumFiles,'%0.0f') '.' num2str(NumDraws,'%0.0f') '.mat']);
               end;
               CoherenceData = load(file);
               Coherence = CoherenceData.Coherence;
               NumSaves = length(Coherence);
               if strcmp(lower(TypeStr),'observed')==1;
                  for s=1:NumSaves;
                     TempVar(CurrSaves+s,1) = Coherence(s).YCoherence(num,t);
                  end;
               else;
                  for s=1:NumSaves;
                     TempVar(CurrSaves+s,1) = Coherence(s).SCoherence(num,t);
                  end;
               end;
               CurrSaves = CurrSaves+NumSaves;
               drawnow;
            end;
            %
            % compute the mean
            %
            SOmega(num,t) = (1/NumDraws)*sum(TempVar.*Weights,1);
            %
            % sort each column
            %
            TempVar = sort(TempVar);
            if DSGEModel.ConfidenceRegionMethod==1;
               for k=1:NumQuants;
                  if DSGEModel.Percentiles(k)<50;
                     QuantVal = ceil((DSGEModel.Percentiles(k)/100)*NumDraws);
                  else;
                     QuantVal = floor((DSGEModel.Percentiles(k)/100)*NumDraws);
                  end;
                  Quantiles(k).SOmega(num,t) = TempVar(QuantVal,1);
               end;
            else;
               for k=1:floor(NumQuants/2);
                  ConfDraws = floor((DSGEModel.Percentiles(NumQuants-k+1)-DSGEModel.Percentiles(k))*NumDraws/100);
                  Bounds = HighestProbabilityDensityBand(TempVar,ConfDraws);
                  Quantiles(k).SOmega(num,t) = Bounds(1);
                  Quantiles(NumQuants-k+1).SOmega(num,t) = Bounds(2);
               end;
            end;
         end;
      end;
      drawnow;
   end;
end;
%
% close the wait dialog
%
set(WaitControls.text,'UserData','done');
delete(WaitHandle);
drawnow;
pause(0.02);

%
% end of DSGECoherenceDistribution.m
%
