function DoneCalc = DSGEConditionalFEVD(thetaPostSample,thetaMode,thetaPositions,ModelParameters,DSGEModel,CurrINI,AnnualizeData,LevelsData,TotalDraws,CurrChain,IsPosterior)
% DSGEConditionalFEVD: Calculates the conditional forecast error variance decompositions for the posterior draws
%                      of the DSGE model and stores them on disk.
%
% USAGE:
%
%       DoneCalc = DSGEConditionalFEVD(thetaPostSample,thetaMode,thetaPositions,ModelParameters,DSGEModel,...
%                                      CurrINI,AnnualizeData,LevelsData,TotalDraws,CurrChain,IsPosterior)
%
% REQUIRED INPUT:  thetaPostSample (matrix) of dimension NumDrawsxNumParam with draws from the posterior or
%                                  the prior distribution of the original DSGE model parameters.
%
%                  thetaMode (vector) of dimension NumParam with the posterior mode or initial values
%                            of the original parameters of the DSGE model.
%
%                  thetaPositions (structure) with fields parameter and distribution. This function
%                            needs the former field to know which parameter name corresponds to
%                            position i in theta.
%
%                  ModelParameters (structure) whose fields have names given by the parameter names in the
%                            DSGE model.
%
%                  DSGEModel (structure) with fields containing all the information about a DSGE Model for YADA.
%
%                  CurrINI (structure), with initialization information.
%
%                  AnnualizeData (boolean) that takes on the value 1 if the impulse responses should be annualized
%                            and 0 otherwise.
%
%                  LevelsData (boolean) that takes on the value 1 if the impulse responses should be computed for
%                            levels variables and 0 otherwise.
%
%                  TotalDraws (integer) with the total number of parameter draws that could have been used.
%
%                  CurrChain (integer) with the number for the current Markov chain.
%
%                  IsPosterior (boolean) that is 1 if draws from the posterior distribution are used and 0
%                            if draws from the prior are supplied.
%
% REQUIRED OUTPUT: DoneCalc (boolean) that is 1 if the calculations were completed and 0 otherwise.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   DG-R/MPR
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2017 European Central Bank.
%
%                       First version: April 4, 2007.
%                        This version: May 4, 2017.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 13-11-2007: Updated the documentation.
%
% * 21-11-2007: Fixed a spelling error.
%
% * 10-01-2008: Changed the last drawnow call to pause(0.02).
%
% * 23-05-2008: Updated the documentation.
%
% * 09-07-2008: Updated the documentation.
%
% * 23-01-2009: Updated the documentation.
%
% * 09-03-2009: Added quotation marks around paths for the dos copy commands.
%
% * 26-03-2009: Made sure that values close to zero in B0 are set to zero. The input variable
%               thetaMode was added as well as making sure that the optional output variable
%               status/mcode was received from the "SolveDSGEModel" function.
%
% * 27-03-2009: Added the input variable "IsPosterior".
%
% * 07-04-2009: Changed the maximum height of the "YADA - Error Message" message box.
%
% * 05-01-2010: Updated the documentation.
%
% * 14-05-2010: Took into account that it's not necessary to resolve the model each time
%               the theta draws do not have a unique convergent solution.
%
% * 17-12-2010: Took into account that "SolveDSGEModel" takes int the input variable "OrderQZ"
%               from the CurrINI structure.
%
% * 05-01-2011: Updated the documentation.
%
% * 10-01-2011: Changed all dos copy calls to "CopyFile" calls.
%
% * 12-01-2011: Added support for unix and mac paths.
%
% * 05-01-2012: Updated the documentation.
%
% * 02-01-2013: Took into account that "SolveDSGEModel" now supports "ErrMsg".
%
% * 17-09-2013: Took into account that "DSGECondVarianceDecompFcn" now requires two additional input variables
%               and provides the long-run conditional forecast error variance decomposition.
%
% * 28-01-2014: Updated the documentation.
%
% * 14-01-2015: Updated the documentation.
%
% * 18-03-2015: Switched to the new IsHandle function.
%
% * 09-06-2015: The code takes the CurrINI.GraphicsRoot field into account.
%
% * 23-02-2016: Updated the documentation.
%
% * 28-10-2016: The function now supports parallel calculations.
%
% * 29-03-2017: Updated the documentation.
%
% * 04-05-2017: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize the output
%
NumDraws = size(thetaPostSample,1);
DoneCalc = 0;
%
% copy files to the tmp folder
%
[stat,msg] = CopyFile([GetPath(DSGEModel.AIMDataFile) 'compute_aim_matrices.m'],[pwd '\tmp']);
[stat,msg] = CopyFile(DSGEModel.MeasurementEquationFile,[pwd '\tmp']);
if FileExist(DSGEModel.UpdateParameterFile)==1;
   [stat,msg] = CopyFile(DSGEModel.UpdateParameterFile,[pwd '\tmp']);
end;
%
% load the AIMData file
%
ErrorStr = '';
try;
   AIMData = load(DSGEModel.AIMDataFile);
catch;
   ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' lasterr];
end;
if isempty(ErrorStr)==0;
   About(ErrorStr,'information','YADA - Bad MAT File',120,500,CurrINI);
   return;
end;
%
% create the output directory
%
stat = MakeDir(DSGEModel.OutputDirectory,'cvd');
if stat~=1;
   if ispc==1;
      txt = ['YADA was for some reason unable to create the directory "' DSGEModel.OutputDirectory '\cvd". The computation of the conditional forecast error variance decomposition distributions has therefore been aborted.'];
   else;
      txt = ['YADA was for some reason unable to create the directory "' DSGEModel.OutputDirectory '/cvd". The computation of the conditional forecast error variance decomposition distributions has therefore been aborted.'];
   end;
   About(txt,'information','YADA - Directory Creation Problem',180,500,CurrINI);
   return;
end;
%
% Determine the impulse response horizon
%
if strcmp(DSGEModel.DataFrequency,'q')==1;
   SampleFreq = 4;
elseif strcmp(DSGEModel.DataFrequency,'m')==1;
   SampleFreq = 12;
else;
   SampleFreq = 1;
end;
h = SampleFreq*DSGEModel.IRHorizon;
TypeStr = 'Original';
AccMat = 0;
if AnnualizeData==1;
   AccMat = diag(DSGEModel.annual==SampleFreq);
   TypeStr = 'Annualized';
end;
if LevelsData==1;
   AccMat = diag(1-DSGEModel.levels);
   TypeStr = 'Levels';
end;
%
% initialize index for saving
%
SaveAfterDraws = min(NumDraws,1000);
NumFiles = ceil(NumDraws/SaveAfterDraws);
q = length(DSGEModel.StateShockPositions);
n = DSGEModel.n;
r = length(DSGEModel.StateVariablePositions);
%
% check if H is time-varying
%
ErrorStr = '';
try;
   [A,H,R,F,B0,mcode,ErrMsg] = SolveDSGEModel(DSGEModel,thetaMode,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
   if isempty(ErrMsg)==0;
      ErrorStr = ['YADA caught an error when trying to solve the DSGE model. Message caught is:' ErrMsg];
   end;
catch;
   ErrorStr = ['YADA caught an error when trying to solve the DSGE model. Message caught is:' lasterr];
end;
if isempty(ErrorStr)==0;
   About(ErrorStr,'information','YADA - Error Message',CurrINI.scrsz(4)-50,500,CurrINI);
   return;
end;
if mcode~=1;
   if mcode==2;
      mcodeStr = 'Roots are not correctly computed by real_schur.'
   elseif mcode==3;
      mcodeStr = 'Too many big roots.';
   elseif mcode==35;
      mcodeStr = 'Too many big roots, and q(:,right) is singular.';
   elseif mcode==4;
      mcodeStr = 'Too few big roots.';
   elseif mcode==45;
      mcodeStr = 'Too few big roots, and q(:,right) is singular.';
   elseif mcode==5;
      mcodeStr = 'q(:,right) is singular.';
   elseif mcode==61;
      mcodeStr = 'Too many exact shiftrights.';
   elseif mcode==62;
      mcodeStr = 'Too many numeric shiftrights.';
   elseif mcode==7;
      mcodeStr = 'Infinite or NaN values detected.';
   elseif mcode==8;
      mcodeStr = 'The function "compute_aim_matrices" returns complex numbers.';
   elseif mcode==-1;
      mcodeStr = 'No stable solution.';
   elseif mcode==-2;
      mcodeStr = 'Too many large generalized eigenvalues.';
   elseif mcode==-3;
      mcodeStr = 'Too few large generalized eigenvalues.';
   else;
      mcodeStr = 'Return code not properly specified.';
   end;
   if DSGEModel.ModelSolver==1;
      SolverStr = 'AiM';
   elseif DSGEModel.ModelSolver==2;
      SolverStr = 'Klein';
   elseif DSGEModel.ModelSolver==3;
      SolverStr = 'Gensys';
   end;
   txt = ['The ' SolverStr ' solver provided the return code: ' num2str(mcode) ', i.e., "' mcodeStr '"'];
   About(txt,'information',['YADA - ' SolverStr ' Solver Error'],200,500,CurrINI);
   return;
end;
%
SelPos = [];
if length(size(H))==3;
   %
   % check if H has the required size
   %
   [FirstPeriod,LastPeriod] = CreateSubSample(DSGEModel);
   if size(H,3)>=LastPeriod;
      %
      % pick a certain period for the H matrix
      %
      YADAHandle = findobj('Type','figure','Tag','YADA');
      controls = get(YADAHandle,'UserData');
      SubSampleMat = get(controls.dsge.initializekalman,'String');
      SubSampleMat = SubSampleMat(DSGEModel.KalmanFirstObservation:size(SubSampleMat,1),:);
      [action,SelectedPosition] = SelectionDlg(SubSampleMat,size(SubSampleMat,1),'Select time period for the measurement matrix H','Measurement Matrix',['Conditional Variance Decompositions - ' TypeStr ' Data'],'','',CurrINI);
      %
      SelPos = FirstPeriod+DSGEModel.KalmanFirstObservation-1+(SelectedPosition-1);
      if strcmp(lower(action),'cancel')==1;
         return;
      end;
   else;
      ErrorStr = ['The number of time periods for the time-varying measurement matrix H (' int2str(size(H,3)) ') is lower than the number of observations (T = ' int2str(LastPeriod) '). YADA has therefore aborted from computing the conditional variance decompositions.'];
      About(ErrorStr,'error','Error - Measurement Equation',200,500,CurrINI);
      return;
   end;
end;
%
% store the posterior mode or initial values matrices
%
Astar = A;
Hstar = H;
Rstar = R;
Fstar = F;
B0star = B0;
%
PriorHeader = 'ositions';
AddWidth = 0;
if IsPosterior==0;
   PriorHeader = ' For Prior';
   AddWidth = 50;
end;
%
% begin the loop
%
if CurrINI.NumWorkers<2;
   abort = '';
   MeanSolutionTime = 0;
   LastDraw = 0;
   ThisSave = 0;
   CVD = [];
   %
   % check if we should setup a wait handle
   %
   if DSGEModel.ShowProgress==1;
      ProgressStructure.title = ['Progress for ' num2str(NumDraws,'%0.0f') ' parameter draws'];
      ProgressStructure.facecolor = CurrINI.progress_facecolor;
      ProgressStructure.startfacecolor = CurrINI.progress_startfacecolor;
      ProgressStructure.edgecolor = CurrINI.progress_edgecolor;
      ProgressStructure.bgcolor = CurrINI.progress_bgcolor;
      ProgressStructure.stop = 0;
      ProgressStructure.clock = DSGEModel.ShowProgressClock;
      ProgressStructure.label = 'Mean DSGE model solution time:';
      %
      ProgressStructure.name = ['Cond Variance Decomp' PriorHeader ' - ' TypeStr ' Data'];
      ProgressStructure.CurrINI = CurrINI;
      WaitHandle = ProgressDLG(0,ProgressStructure);
      set(WaitHandle,'Color',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'));
      drawnow;
   else;
      txt = ['Please wait while YADA computes conditional variance decompositions for ' lower(TypeStr) ' data. Computations started at: ' StringTrim(datestr(now,14))];
      WaitHandle = WaitDLG(txt,'information',['Conditional Variance Decompositions' PriorHeader ' - ' TypeStr ' Data'],500+AddWidth,150,CurrINI,0);
      WaitControls = get(WaitHandle,'UserData');
   end;
   %
   for it=1:NumDraws;
      LastDraw = LastDraw+1;
      %
      % update the progress dialog if one is displayed
      %
      if DSGEModel.ShowProgress==1;
         abort = get(WaitHandle,'UserData');
         if strcmp(abort,'cancel')==1;
            break;
         else;
            ProgressDLG([it/NumDraws MeanSolutionTime]);
         end;
      end;
      %
      % Solve the model for the current parameter vector
      %
      theta = thetaPostSample(it,:)';
      tic;
      [A,H,R,F,B0,status] = SolveDSGEModel(DSGEModel,theta,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
      if status~=1;
         %
         % use the posterior mode or initial values solution of the model
         %
         A = Astar;
         H = Hstar;
         R = Rstar;
         F = Fstar;
         B0 = B0star;
      end;
      MeanSolutionTime = (((it-1)/it)*MeanSolutionTime)+((1/it)*toc);
      CVD(LastDraw).FEVD = zeros(n,q,h);
      if length(H)==3;
         H = H(:,:,SelPos);
      end;
      %
      % replace small numbers with zero
      %
      Bpos = (B0>1.0e-12);
      Bneg = (B0<-1.0e-12);
      B0 = (B0 .* Bpos)+(B0 .* Bneg);
      %
      % check which type of variance decomposition should be calculated
      %
      if (AnnualizeData+LevelsData)==0;
         %
         % we have the original data
         %
         CVD(LastDraw).FEVD = DSGECondVarianceDecompFcn(H,F,B0,h,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue);
      elseif AnnualizeData==1;
         %
         % we have annualized data to consider
         %
         CVD(LastDraw).FEVD = DSGECondAnnVarianceDecompFcn(H,F,B0,AccMat,h,SampleFreq);
      else;
         %
         % we have levels data to use
         %
         CVD(LastDraw).FEVD = DSGECondLevVarianceDecompFcn(H,F,B0,AccMat,h);
      end;
      %
      % check if we should save to disk
      %
      if (LastDraw==SaveAfterDraws)|(it==NumDraws);
         ThisSave = ThisSave+1;
         LastDraw = 0;
         %
         % set up the file name and then save
         %
         if IsPosterior==1;
            file = FixFilePath([DSGEModel.OutputDirectory '\cvd\DSGE-CVD-' TypeStr '-' DSGEModel.NameOfModel '-' num2str(CurrChain,'%0.0f') '-' num2str(h,'%0.0f') '-' num2str(ThisSave,'%0.0f') '-' num2str(NumFiles,'%0.0f') '.' num2str(NumDraws,'%0.0f') '-' num2str(TotalDraws,'%0.0f') '.mat']);
         else;
            file = FixFilePath([DSGEModel.OutputDirectory '\cvd\DSGE-CVD-Prior-' TypeStr '-' DSGEModel.NameOfModel '-' num2str(h,'%0.0f') '-' num2str(ThisSave,'%0.0f') '-' num2str(NumFiles,'%0.0f') '.' num2str(NumDraws,'%0.0f') '.mat']);
         end;
         save(file,'CVD');
         %
         % restore the data structure
         %
         CVD = [];
      end;
      drawnow;
   end;
   %
   % close the wait dialog
   %
   if DSGEModel.ShowProgress==1;
      if IsHandle(WaitHandle)==1;
         set(WaitHandle,'UserData','done');
         close(WaitHandle);
         drawnow;
         pause(0.02);
      end;
   else;
      set(WaitControls.text,'UserData','done');
      delete(WaitHandle);
      drawnow;
      pause(0.02);
   end;
else;
   %
   % run the parallel version of the coinditional FEVDs
   %
   abort = '';
   txt = ['Please wait while YADA computes conditional variance decompositions for ' lower(TypeStr) ' data using ' int2str(CurrINI.NumWorkers) ' parallel computational threads. Computations started at: ' StringTrim(datestr(now,14))];
   WaitHandle = WaitDLG(txt,'information',['Conditional Variance Decompositions' PriorHeader ' - ' TypeStr ' Data'],500+AddWidth,150,CurrINI,0);
   WaitControls = get(WaitHandle,'UserData');
   %
   % initializing
   %
   for i=1:NumDraws;
      VD(i).FEVD = zeros(n,q,h+1);
   end;
   parfor it=1:NumDraws;
      %
      % Solve the model for the current parameter vector
      %
      theta = thetaPostSample(it,:)';
      [A,H,R,F,B0,status] = SolveDSGEModel(DSGEModel,theta,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
      if status~=1;
         %
         % use the posterior mode or initial values solution of the model
         %
         A = Astar;
         H = Hstar;
         R = Rstar;
         F = Fstar;
         B0 = B0star;
      end;
      if length(H)==3;
         H = H(:,:,SelPos);
      end;
      %
      % replace small numbers with zero
      %
      Bpos = (B0>1.0e-12);
      Bneg = (B0<-1.0e-12);
      B0 = (B0 .* Bpos)+(B0 .* Bneg);
      %
      % check which type of variance decomposition should be calculated
      %
      if (AnnualizeData+LevelsData)==0;
         %
         % we have the original data
         %
         VD(it).FEVD = DSGECondVarianceDecompFcn(H,F,B0,h,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue);
      elseif AnnualizeData==1;
         %
         % we have annualized data to consider
         %
         VD(it).FEVD = DSGECondAnnVarianceDecompFcn(H,F,B0,AccMat,h,SampleFreq);
      else;
         %
         % we have levels data to use
         %
         VD(it).FEVD = DSGECondLevVarianceDecompFcn(H,F,B0,AccMat,h);
      end;
      drawnow;
   end;
   %
   % close the wait dialog
   %
   set(WaitControls.text,'UserData','done');
   delete(WaitHandle);
   drawnow;
   pause(0.02);
   %
   % save the results to files
   %
   for i=1:NumFiles;
      %
      % save the data to disk
      %
      CVD = VD(((i-1)*SaveAfterDraws)+1:min(SaveAfterDraws*i,NumDraws));
      if IsPosterior==1;
         file = FixFilePath([DSGEModel.OutputDirectory '\cvd\DSGE-CVD-' TypeStr '-' DSGEModel.NameOfModel '-' num2str(CurrChain,'%0.0f') '-' num2str(h,'%0.0f') '-' num2str(i,'%0.0f') '-' num2str(NumFiles,'%0.0f') '.' num2str(NumDraws,'%0.0f') '-' num2str(TotalDraws,'%0.0f') '.mat']);
      else;
         file = FixFilePath([DSGEModel.OutputDirectory '\cvd\DSGE-CVD-Prior-' TypeStr '-' DSGEModel.NameOfModel '-' num2str(h,'%0.0f') '-' num2str(i,'%0.0f') '-' num2str(NumFiles,'%0.0f') '.' num2str(NumDraws,'%0.0f') '.mat']);
      end;
      save(file,'CVD');
   end;
end;
%
% check if we have computed all output
%
if strcmp(abort,'cancel')==0;
   DoneCalc = 1;
end;

%
% end of DSGEConditionalFEVD.m
%
