function [InformationMatrix,ParamIncluded,ParamExcluded] = DSGEFrequencyDomainInfMat(theta,thetaPositions,ModelParameters,ParameterNames,DSGEModel,CurrINI)
% DSGEFrequencyDomainInfMat: Computes Fisher's information matrix in the frequency domain
%                            for the parameters that affect the population spectrum.
%
% USAGE:
%
%       [InformationMatrix,ParamIncluded,ParamExcluded] = DSGEFrequencyDomainInfMat(theta,thetaPositions,ModelParameters,...
%                                                                     ParameterNames,DSGEModel,CurrINI)
%
% REQUIRED INPUT:  theta (vector) with values of the original DSGE model parameters
%
%                  thetaPositions (structure) with fields parameter and distribution. This function
%                                 needs the former field to know which parameter name corresponds to
%                                 position i in theta.
%
%                  ModelParameters (structure) whose fields have names given by the parameter names in the
%                                 DSGE model.
%
%                  ParameterNames (string matrix) with the names of the parameters.
%
%                  DSGEModel (structure) with fields containing all the information about a DSGE Model for YADA.
%
%                  CurrINI (structure) with initialization information.
%
% REQUIRED OUTPUT: InformationMatrix (matrix) of dimension pxp, where p is the number of parameters that
%                                 affect the population spectrum, i.e., which affect H,R,F or B0 of the
%                                 state-space representation of the DSGE model.
%
%                  ParamIncluded (vector) with the positions in theta (and the rows of ParameterNames) of
%                                 the parameters that do affect the population spectrum.
%
%                  ParamExcluded (vector) with the positions in theta (and the rows of ParameterNames) of
%                                 the parametersn that don't affect the population spectrum.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   DG-R/MPR
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2017 European Central Bank.
%
%                       First version: May 5, 2010.
%                        This version: March 29, 2017.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 06-05-2010: Finished the function.
%
% * 12-05-2010: Made sure that the standard transpose is applied to complex matrices
%               rather than the conjugate, i.e., we use .' instead of ' to get the standard
%               transpose.
%
% * 14-05-2010: Updated the documentation.
%
% * 17-12-2010: Took into account that "SolveDSGEModel" takes the input variable "OrderQZ"
%               from the CurrINI structure.
%
% * 05-01-2011: Updated the documentation.
%
% * 10-01-2011: Changed all dos copy calls to "CopyFile" calls.
%
% * 05-01-2012: Updated the documentation.
%
% * 02-01-2013: Took into account that "SolveDSGEModel" now supports "ErrMsg".
%
% * 28-01-2014: Updated the documentation.
%
% * 14-01-2015: Updated the documentation.
%
% * 22-02-2016: Updated the documentation.
%
% * 29-03-2017: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize
%
InformationMatrix = [];
ParamIncluded = [];
ParamExcluded = [];
%
% copy files to the tmp folder
%
[stat,msg] = CopyFile([GetPath(DSGEModel.AIMDataFile) 'compute_aim_matrices.m'],[pwd '\tmp']);
[stat,msg] = CopyFile(DSGEModel.MeasurementEquationFile,[pwd '\tmp']);
if FileExist(DSGEModel.UpdateParameterFile)==1;
   [stat,msg] = CopyFile(DSGEModel.UpdateParameterFile,[pwd '\tmp']);
end;
%
% load the AIMData file
%
ErrorStr = '';
try;
   AIMData = load(DSGEModel.AIMDataFile);
catch;
   ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' lasterr];
end;
if isempty(ErrorStr)==1;
   %
   % check if we can solve the model
   %
   ErrorStr = '';
   try;
      [A,H,R,F,B0,mcode,ErrMsg] = SolveDSGEModel(DSGEModel,theta,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
      if isempty(ErrMsg)==0;
         ErrorStr = ['YADA caught an error when trying to solve the DSGE model. Message caught is:' ErrMsg];
      end;
   catch;
      ErrorStr = ['YADA caught an error when trying to solve the DSGE model. Message caught is:' lasterr];
   end;
   if isempty(ErrorStr)==0;
      About(ErrorStr,'information','YADA - Error Message',CurrINI.scrsz(4)-50,500,CurrINI);
      return;
   end;
   if mcode~=1;
      if mcode==2;
         mcodeStr = 'Roots are not correctly computed by real_schur.'
      elseif mcode==3;
         mcodeStr = 'Too many big roots.';
      elseif mcode==35;
         mcodeStr = 'Too many big roots, and q(:,right) is singular.';
      elseif mcode==4;
         mcodeStr = 'Too few big roots.';
      elseif mcode==45;
         mcodeStr = 'Too few big roots, and q(:,right) is singular.';
      elseif mcode==5;
         mcodeStr = 'q(:,right) is singular.';
      elseif mcode==61;
         mcodeStr = 'Too many exact shiftrights.';
      elseif mcode==62;
         mcodeStr = 'Too many numeric shiftrights.';
      elseif mcode==7;
         mcodeStr = 'Infinite or NaN values detected.';
      elseif mcode==8;
         mcodeStr = 'The function "compute_aim_matrices" returns complex numbers.';
      elseif mcode==-1;
         mcodeStr = 'No stable solution.';
      elseif mcode==-2;
         mcodeStr = 'Too many large generalized eigenvalues.';
      elseif mcode==-3;
         mcodeStr = 'Too few large generalized eigenvalues.';
      else;
         mcodeStr = 'Return code not properly specified.';
      end;
      if DSGEModel.ModelSolver==1;
         SolverStr = 'AiM';
      elseif DSGEModel.ModelSolver==2;
         SolverStr = 'Klein';
      elseif DSGEModel.ModelSolver==3;
         SolverStr = 'Gensys';
      end;
      txt = ['The ' SolverStr ' solver provided the return code: ' num2str(mcode) ', i.e., "' mcodeStr '"'];
      About(txt,'information','Information Matrix',120,500,CurrINI);
      return;
   end;
else;
   About(ErrorStr,'information','YADA - Bad MAT File',120,500,CurrINI);
   return;
end;
%
% check if H is time-varying.
%
if length(size(H))==3;
   if size(H,3)<DSGEModel.T;
      ErrorStr = ['The number of time periods for the time-varying measurement matrix H (' int2str(size(H,3)) ') is less than the number of observations (T = ' int2str(DSGEModel.T) '). YADA has therefore aborted from the frequency domain estimation of the information matrix.'];
      About(ErrorStr,'error','Error - Measurement Equation',200,500,CurrINI);
      return;
   end;
   %
   % determine the sample
   %
   [FirstPeriod,LastPeriod,SubSampleStrMat] = CreateSubSample(DSGEModel);
   %
   % compute the average H-matrix
   %
   H = H(:,:,FirstPeriod+DSGEModel.KalmanFirstObservation-1:LastPeriod);
   H = (1/size(H,3))*sum(H,3);
end;
%
% remove redunant columns from B0
%
[B0,KeepVar] = RemoveRedundantColumns(B0);
if isempty(KeepVar)==1;
   ErrorStr = 'The DSGE model doesn''t have any shocks with non-zero parameters in the B0 matrix. The spectral density of the state variables is therefore zero.';
   About(ErrorStr,'error','Error - State Shocks',200,500,CurrINI);
   return;
end;
%
% compute the matrix with partial derivatives of H, R, F, and B0 wrt theta
%
thetaChange = 0.001*abs(theta);
thetaAltChange = 0.0001*abs(theta);
StopCalculation = 0;
p = length(theta);
IdentMat = eye(p);
n = DSGEModel.n;
[r,q] = size(B0);
if size(R,1)~=n;
   R = zeros(n,n);
end;
%
% initialize the partial derivatives
%
Rpartial = zeros(n*n,p);
Hpartial = zeros(n*r,p);
Fpartial = zeros(r*r,p);
B0partial = zeros(r*q,p);
%
% loop over all parameters
%
for i=1:p;
   %
   % create a new parameter vector
   %
   ParamDiff = thetaChange(i);
   thetaNew = theta+(ParamDiff*IdentMat(:,i));
   %
   % try to solve the model at this value
   %
   [ANew,HNew,RNew,FNew,B0New,mcode] = SolveDSGEModel(DSGEModel,thetaNew,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
   if mcode~=1;
      %
      % try the alternative 
      %
      ParamDiff = thetaAltChange(NumParam);
      thetaNew = theta+(ParamDiff(NumParam)*IdentMat(:,NumParam));
      [ANew,HNew,RNew,FNew,B0New,mcode] = SolveDSGEModel(DSGEModel,thetaNew,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
   end;
   if mcode==1;
      B0New = B0New(:,KeepVar);
      if length(size(HNew))==3;
         HNew = HNew(:,:,FirstPeriod+DSGEModel.KalmanFirstObservation-1:LastPeriod);
         HNew = (1/size(HNew,3))*sum(HNew,3);
      end;
      if size(RNew,1)~=n;
         RNew = zeros(n,n);
      end;
      %
      Rpartial(:,i) = (1/ParamDiff)*vec(RNew-R);
      Hpartial(:,i) = (1/ParamDiff)*vec(HNew-H);
      Fpartial(:,i) = (1/ParamDiff)*vec(FNew-F);
      B0partial(:,i) = (1/ParamDiff)*vec(B0New-B0);
   else;
      StopCalculation = 1;
      txt = ['YADA was unable to compute the numerical partial derivatives of the state-space matrices with respect to the theta parameters since the DSGE model couldn''t be solved for the tested new values of the "' StringTrim(ParameterNames(i,:)) '" parameter.'];
      About(ErrorStr,'information','YADA - Bad Numerical Partial Derivatives',120,500,CurrINI);
      break;
   end;
end;
%
% check if the numerical partial derivatives could be calculated.
%
if StopCalculation==1;
   return;
end;
%
% test which columns of the stacked partials that are zero
%
[TMat,ParamIncluded] = RemoveRedundantColumns([Rpartial;Hpartial;Fpartial;B0partial]);
ParamExcluded = setdiff((1:p),ParamIncluded);
%
% make sure that we only keep the columns that are non-zero in at least one of the
% matrices with partial derivatives
%
Rpartial = Rpartial(:,ParamIncluded);
Hpartial = Hpartial(:,ParamIncluded);
Fpartial = Fpartial(:,ParamIncluded);
B0partial = B0partial(:,ParamIncluded);
%
% we can now compute the population spectrum over various frequencies
%
[SyOmega,Omega,SxiOmega,Fi,Fmi,ei,emi] = DSGEPopulationSpectrum(H,R,F,B0);
%
% define some additional matrices
%
Knn = CommutationMatrix(n,n);
B0Ir = (1/(2*pi))*(eye(r*r)+CommutationMatrix(r,r))*kron(B0,eye(r));
In = eye(n);
dSydRdtheta = (1/(2*pi))*eye(n*n)*Rpartial;
dOmega = Omega(2)-Omega(1);
%
% compute the Kernel matrices
%
f = length(Omega);
p = length(ParamIncluded);
InformationMatrix = zeros(p,p);
for freq=1:f;
   %
   % make sure that only full rank spectral densities are used
   %
   if rank(SyOmega{freq})==n;
      dSydH = (Knn*kron(In,H'*SxiOmega{freq}.'))+kron(In,H'*SxiOmega{freq});
      dSydF = (kron(H'*SxiOmega{freq}.',H'*Fmi{freq})*emi(freq)) + (Knn*kron(H'*SxiOmega{freq},H'*Fi{freq})*ei(freq));
      dSydB0 = kron(H'*Fi{freq},H'*Fmi{freq})*B0Ir;
      dSydtheta = dSydRdtheta+(dSydH*Hpartial)+(dSydF*Fpartial)+(dSydB0*B0partial);
      dSydtrtheta = Knn*dSydtheta;
      %
      Sy = SyOmega{freq}\In;
      KOmega = dSydtrtheta.'*kron(Sy.',Sy)*dSydtheta;
      %
      % take into account that KOmega is Hermitian and that KOmega+KOmega' is real (and symmetric).
      %
      % NOTE: K(Omega)' is equal to K(-Omega) when Omega is greater than 0.
      %
      if freq>1;
         InformationMatrix = InformationMatrix+(real(KOmega+KOmega')*dOmega);
      else;
         InformationMatrix = real(KOmega)*dOmega;
      end;
   end;
   drawnow;
end;
InformationMatrix = (1/(4*pi))*InformationMatrix;

%
% end of DSGEFrequencyDomainInfMat.m
%
