function [MeanResp,StdResp,CorrResp] = DSGEMomentsResponseTheta(theta,thetaPositions,ModelParameters,ParameterNames,ParamNumber,OutputType,EstStr,DSGEModel,CurrINI)
% DSGEMomentsResponseTheta: Computes the numerical partial derivatives of the means, standard deviations, and correlations
%                           with respect to a change in one structural parameters.
%
% USAGE:
%
%       [MeanResp,StdResp,CorrResp] = DSGEMomentsResponseTheta(theta,thetaPositions,ModelParameters,ParameterNames, ...
%                                                              ParamNumber,OutputType,EstStr,DSGEModel,CurrINI)
%
% REQUIRED INPUT:  theta (vector) with values of the original DSGE model parameters
%
%                  thetaPositions (structure) with fields parameter and distribution. This function
%                                 needs the former field to know which parameter name corresponds to
%                                 position i in theta.
%
%                  ModelParameters (structure) whose fields have names given by the parameter names in the
%                                 DSGE model.
%
%                  ParameterNames (string matrix) with the names of the parameters.
%
%                  ParamNumber (integer) with the position of the parameter in theta to change.
%
%                  OutputType (boolean) that takes the value 1 if responses to the moments should be computed
%                                 for the observed variables, and 0 if the responses should be computed for
%                                 the state variables.
%
%                  EstStr (string vector) that takes the values "Initial Values" or "Posterior Mode".
%
%                  DSGEModel (structure) with fields containing all the information about a DSGE Model for YADA.
%
%                  CurrINI (structure) with initialization information.
%
% REQUIRED OUTPUT: MeanResp (matrix) of dimension nxk with the responses of the exogenous variables from the change
%                                 change in one structural parameter. This matrix is an r dimensional zero vector
%                                 for state variables.
%
%                  StdResp (vector) of dimension n for observed variables and r for state variables with the response
%                                 of the standard deviations of the variables.
%
%                  CorrResp (vector) of dimension n(n-1)/2 for observed variables and r(r-1)/2 for state variables
%                                 with the responses of the correlations (stacking of elements below the diagonal
%                                 of the correlation matrix).
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   DG-R/MPR
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2017 European Central Bank.
%
%                       First version: March 21, 2014.
%                        This version: March 29, 2017.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 24-03-2014: Updated the documentation.
%
% * 14-01-2015: Updated the documentation.
%
% * 23-02-2016: Updated the documentation.
%
% * 29-03-2017: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize output
%
MeanResp = [];
StdResp = [];
CorrResp = [];
%
% copy files to the tmp folder
%
[stat,msg] = CopyFile([GetPath(DSGEModel.AIMDataFile) 'compute_aim_matrices.m'],[pwd '\tmp']);
[stat,msg] = CopyFile(DSGEModel.MeasurementEquationFile,[pwd '\tmp']);
if FileExist(DSGEModel.UpdateParameterFile)==1;
   [stat,msg] = CopyFile(DSGEModel.UpdateParameterFile,[pwd '\tmp']);
end;
%
% load the AIMData file
%
ErrorStr = '';
try;
   AIMData = load(DSGEModel.AIMDataFile);
catch;
   ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' lasterr];
end;
if isempty(ErrorStr)==1;
   %
   % check if we can solve the model
   %
   ErrorStr = '';
   try;
      [A,H,R,F,B0,mcode,ErrMsg] = SolveDSGEModel(DSGEModel,theta,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
      if isempty(ErrMsg)==0;
         ErrorStr = ['YADA caught an error when trying to solve the DSGE model. Message caught is:' ErrMsg];
      end;
   catch;
      ErrorStr = ['YADA caught an error when trying to solve the DSGE model. Message caught is:' lasterr];
   end;
   if isempty(ErrorStr)==0;
      About(ErrorStr,'information','YADA - Error Message',CurrINI.scrsz(4)-50,500,CurrINI);
      return;
   end;
   if mcode~=1;
      if mcode==2;
         mcodeStr = 'Roots are not correctly computed by real_schur.'
      elseif mcode==3;
         mcodeStr = 'Too many big roots.';
      elseif mcode==35;
         mcodeStr = 'Too many big roots, and q(:,right) is singular.';
      elseif mcode==4;
         mcodeStr = 'Too few big roots.';
      elseif mcode==45;
         mcodeStr = 'Too few big roots, and q(:,right) is singular.';
      elseif mcode==5;
         mcodeStr = 'q(:,right) is singular.';
      elseif mcode==61;
         mcodeStr = 'Too many exact shiftrights.';
      elseif mcode==62;
         mcodeStr = 'Too many numeric shiftrights.';
      elseif mcode==7;
         mcodeStr = 'Infinite or NaN values detected.';
      elseif mcode==8;
         mcodeStr = 'The function "compute_aim_matrices" returns complex numbers.';
      elseif mcode==-1;
         mcodeStr = 'No stable solution.';
      elseif mcode==-2;
         mcodeStr = 'Too many large generalized eigenvalues.';
      elseif mcode==-3;
         mcodeStr = 'Too few large generalized eigenvalues.';
      else;
         mcodeStr = 'Return code not properly specified.';
      end;
      if DSGEModel.ModelSolver==1;
         SolverStr = 'AiM';
      elseif DSGEModel.ModelSolver==2;
         SolverStr = 'Klein';
      elseif DSGEModel.ModelSolver==3;
         SolverStr = 'Gensys';
      end;
      txt = ['The ' SolverStr ' solver provided the return code: ' num2str(mcode) ', i.e., "' mcodeStr '"'];
      About(txt,'information',['Parameter Change - ' EstStr],120,500,CurrINI);
      return;
   end;
else;
   About(ErrorStr,'information','YADA - Bad MAT File',120,500,CurrINI);
   return;
end;
%
k = DSGEModel.k;
n = DSGEModel.n;
[r,q] = size(B0);
p = length(theta);
%
if OutputType==1;
   VarStr = 'observed';
   if k>0;
      MeanMat = A';
   end;
   [CovMat,stat]  = DoublingAlgorithmLyapunov(F,B0*B0',DSGEModel.DAMaximumIterationsValue*100,10^(-(DSGEModel.DAToleranceValue+1)));
   if stat==1;
      CovMat = reshape((eye(r*r)-kron(F,F))\vec(B0*B0'),r,r)
   end;
   if length(size(H))==2;
      CovMat = (H'*CovMat*H)+R;
   else;
      [FirstPeriod,LastPeriod] = CreateSubSample(DSGEModel);
      H = H(:,:,FirstPeriod+DSGEModel.KalmanFirstObservation-1:LastPeriod);
      H = (1/size(H,3))*sum(H,3);
      CovMat = (H'*CovMat*H)+R;
   end;
else;
   VarStr = 'state';
   [CovMat,stat]  = DoublingAlgorithmLyapunov(F,B0*B0',DSGEModel.DAMaximumIterationsValue*100,10^(-(DSGEModel.DAToleranceValue+1)));
   if stat==1;
      CovMat = reshape((eye(r*r)-kron(F,F))\vec(B0*B0'),r,r);
   end;
end;
StdVec = real(sqrt(diag(CovMat)));
if length(StdVec)>1;
   CorrVec = vechl(CovMat./(StdVec*StdVec'));
end;
%
% now make the parameter change
%
thetaChange = 0.001*abs(theta);
thetaAltChange = 0.0001*abs(theta);
IdentMat = eye(p);
ParamDiff = thetaChange(ParamNumber);
thetaNew = theta+(ParamDiff*IdentMat(:,ParamNumber));
%
% try to solve the model at the new value
%
[A,H,R,F,B0,mcode] = SolveDSGEModel(DSGEModel,thetaNew,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
if mcode~=1;
   ParamDiff = thetaAltChange(ParamNumber);
   thetaNew = theta+(ParamDiff*IdentMat(:,ParamNumber));
   [A,H,R,F,B0,mcode] = SolveDSGEModel(DSGEModel,thetaNew,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
   if mcode~=1;
      txt = ['YADA was unable to compute the numerical partial derivatives of the first and second moments of the ' VarStr ' variables since the DSGE model couldn''t be solved for the tested new values of the "' StringTrim(ParameterNames(ParamNumber,:)) '" parameter.'];
      About(ErrorStr,'information','YADA - Bad Numerical Partial Derivatives',120,500,CurrINI);
      return;
   end;
end;
%
% compute the moments at the new values
%
if OutputType==1;
   if k>0;
      MeanResp = (A'-MeanMat)./ParamDiff;
   else;
      MeanResp = zeros(n,1);
   end;
   [CovMat,stat]  = DoublingAlgorithmLyapunov(F,B0*B0',DSGEModel.DAMaximumIterationsValue*100,10^(-(DSGEModel.DAToleranceValue+1)));
   if stat==1;
      CovMat = reshape((eye(r*r)-kron(F,F))\vec(B0*B0'),r,r)
   end;
   if length(size(H))==2;
      CovMat = (H'*CovMat*H)+R;
   else;
      [FirstPeriod,LastPeriod] = CreateSubSample(DSGEModel);
      H = H(:,:,FirstPeriod+DSGEModel.KalmanFirstObservation-1:LastPeriod);
      H = (1/size(H,3))*sum(H,3);
      COvMat = (H'*CovMat*H)+R;
   end;
else;
   MeanResp = zeros(r,1);
   [CovMat,stat]  = DoublingAlgorithmLyapunov(F,B0*B0',DSGEModel.DAMaximumIterationsValue*100,10^(-(DSGEModel.DAToleranceValue+1)));
   if stat==1;
      CovMat = reshape((eye(r*r)-kron(F,F))\vec(B0*B0'),r,r);
   end;
end;
StdVecNew = real(sqrt(diag(CovMat)));
if length(StdVecNew)>1;
   CorrVecNew = vechl(CovMat./(StdVecNew*StdVecNew'));
end;
%
% now compute the change in standard deviations and correlations
%
StdResp = (StdVecNew-StdVec)./ParamDiff;
if length(StdVecNew)>1;
   CorrResp = (CorrVecNew-CorrVec)./ParamDiff;
end;

%
% end of DSGEMomentsResponseTheta.m
%
