function DoneCalc = DSGEOneStepAheadPredictiveChecks(theta,thetaPostSample,thetaPositions,ModelParameters,SelectedParameters,IsPosterior,TotalDraws,CurrChain,DSGEModel,CurrINI)
% DSGEOneStepAheadPredictiveChecks: Computes distributions of one-step-ahead forecast errors for actual data and simulated
%                                   data for prediictive checks.
%
% USAGE:
%
%       DoneCalc = DSGEOneStepAheadPredictiveChecks(theta,thetaPostSample,thetaPositions,ModelParameters, ...
%                                      SelectedParameters,IsPosterior,TotalDraws,CurrChain,DSGEModel,CurrINI)
%
% REQUIRED INPUT:  theta (vector) with the posterior mode estimates or the initial values of the parameters.
%
%                  thetaPostSample (matrix) of dimension NumDrawsxNumParam with draws from the posterior or the
%                                  prior distribution of the original DSGE model parameters.
%
%                  thetaPositions (structure) where thetaPositions(i).parameter is a string vector giving the name
%                                  of the model parameter in position i of theta. The field "distribution" has a string
%                                  vector giving the assumed prior distribution of the parameter.
%
%                  ModelParameters (structure) whose fields have names given by the parameter names in the
%                                  DSGE model.
%
%                  SelectedParameters (vector) of dimension NumParam which indicates which parameters can vary from
%                                  one draw to the next (1) and which parameters are fixed at posterior mode (0).
%
%                  IsPosterior (boolean) that is 1 if draws from the posterior distribution are used and 0
%                                  if draws from the prior are supplied.
%
%                  TotalDraws (integer) with the total number of parameter draws that could have been used.
%
%                  CurrChain (integer) with the number for the current Markov chain.
%
%                  DSGEModel (structure) with fields containing all the information about a DSGE Model for YADA.
%
%                  CurrINI (structure), with initialization information.
%
% REQUIRED OUTPUT: DoneCalc (boolean) that is 1 if the calculations were completed and 0 otherwise.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   DG-R/MPR
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2017 European Central Bank.
%
%                       First version: August 13, 2013.
%                        This version: March 29, 2017.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 14-08-2013: Continued work on the function.
%
% * 15-08-2013: Finished the function.
%
% * 20-08-2013: Made sure that "KeepShocks" is only applied to B0 when status is 1.
%
% * 03-09-2013: Updated the documentation.
%
% * 28-01-2014: Updated the documentation.
%
% * 14-01-2015: Updated the documentation.
%
% * 18-03-2015: Switched to the new IsHandle function.
%
% * 09-06-2016: The code takes the CurrINI.GraphicsRoot field into account.
%
% * 23-02-2016: Updated the documentation.
%
% * 29-03-2017: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize
%
DoneCalc = 0;
%
% Prepare variables for selecting the parameters that are updated through the
% posterior draws
%
ScenarioParameters = SelectedParameters .* (1:length(theta));
ScenarioParameters = ScenarioParameters(ScenarioParameters>0);
SelParamStr = strrep(num2str(SelectedParameters),' ','');
thetaMode = theta;
%
% copy files to the tmp folder
%
[stat,msg] = CopyFile([GetPath(DSGEModel.AIMDataFile) 'compute_aim_matrices.m'],[pwd '\tmp']);
[stat,msg] = CopyFile(DSGEModel.MeasurementEquationFile,[pwd '\tmp']);
if FileExist(DSGEModel.UpdateParameterFile)==1;
   [stat,msg] = CopyFile(DSGEModel.UpdateParameterFile,[pwd '\tmp']);
end;
%
% load the AIMData file
%
ErrorStr = '';
try;
   AIMData = load(DSGEModel.AIMDataFile);
catch;
   ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' lasterr];
end;
if isempty(ErrorStr)==0;
   About(ErrorStr,'information','YADA - Bad MAT File',120,500,CurrINI);
   return;
end;
%
% create the output directory
%
stat = MakeDir(DSGEModel.OutputDirectory,'predcheck');
if stat~=1;
   if ispc==1;
      txt = ['YADA was for some reason unable to create the directory "' DSGEModel.OutputDirectory '\predcheck". The computation of predictive checks for the one-step-ahead forecast errors has therefore been aborted.'];
   else;
      txt = ['YADA was for some reason unable to create the directory "' DSGEModel.OutputDirectory '/predcheck". The computation of predictive checks for the one-step-ahead forecast errors has therefore been aborted.'];
   end;
   About(txt,'information','YADA - Directory Creation Problem',180,500,CurrINI);
   return;
end;
%
% set the InitialStateVector value
%
if DSGEModel.UseOwnInitialState==1;
   if length(DSGEModel.InitialStateValues)==size(AIMData.endog,1);
      DSGEModel.InitialStateVector = DSGEModel.InitialStateValues(DSGEModel.StateVariablePositions);
   else;
      DSGEModel.InitialStateVector = zeros(length(DSGEModel.StateVariablePositions),1);
   end;
else;
   DSGEModel.InitialStateVector = zeros(length(DSGEModel.StateVariablePositions),1);
end;
%
% Get the sub-sample
%
[DSGEModel.FirstPeriod,DSGEModel.LastPeriod] = CreateSubSample(DSGEModel);
%
% set up Y and X
%
Y = DSGEModel.Y(:,DSGEModel.FirstPeriod+DSGEModel.KalmanFirstObservation-1:DSGEModel.LastPeriod);
if isempty(DSGEModel.X)==0;
   X = DSGEModel.X(:,DSGEModel.FirstPeriod+DSGEModel.KalmanFirstObservation-1:DSGEModel.LastPeriod);
   XSim = DSGEModel.X(:,DSGEModel.FirstPeriod:DSGEModel.LastPeriod);
else;
   X = [];
   XSim = [];
end;
T = DSGEModel.LastPeriod-DSGEModel.FirstPeriod+1;
Tsum = T-DSGEModel.KalmanFirstObservation+1;
NumDraws = size(thetaPostSample,1);
n = DSGEModel.n;
k = DSGEModel.k;
%
% define certain matrices for forecast error covariance calculations when there are missing obs
%
if DSGEModel.YNaNs==1;
   NaNY = isnan(Y);
   Tvec = Tsum-sum(NaNY,2);
   Tmat = zeros(n,n);
   for i=1:n;
      Tmat(i,i) = 1/Tvec(i);
      for j=i+1:n;
         Tmat(i,j) = 1/min(Tvec(i),Tvec(j));
         Tmat(j,i) = Tmat(i,j);
      end;
   end;
   NaNYSim = isnan(DSGEModel.Y(:,DSGEModel.FirstPeriod:DSGEModel.LastPeriod));
end;
%
% solve the model for the posterior mode estimate
%
ErrorStr = '';
try;
   [A,H,R,F,B0,mcode,ErrMsg] = SolveDSGEModel(DSGEModel,theta,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
   if isempty(ErrMsg)==0;
      ErrorStr = ['YADA caught an error when trying to solve the DSGE model. Message caught is:' ErrMsg];
   end;
catch;
   ErrorStr = ['YADA caught an error when trying to solve the DSGE model. Message caught is:' lasterr];
end;
if isempty(ErrorStr)==0;
   About(ErrorStr,'information','YADA - Error Message',CurrINI.scrsz(4)-50,500,CurrINI);
   return;
end;
if mcode~=1;
   if mcode==2;
      mcodeStr = 'Roots are not correctly computed by real_schur.'
   elseif mcode==3;
      mcodeStr = 'Too many big roots.';
   elseif mcode==35;
      mcodeStr = 'Too many big roots, and q(:,right) is singular.';
   elseif mcode==4;
      mcodeStr = 'Too few big roots.';
   elseif mcode==45;
      mcodeStr = 'Too few big roots, and q(:,right) is singular.';
   elseif mcode==5;
      mcodeStr = 'q(:,right) is singular.';
   elseif mcode==61;
      mcodeStr = 'Too many exact shiftrights.';
   elseif mcode==62;
      mcodeStr = 'Too many numeric shiftrights.';
   elseif mcode==7;
      mcodeStr = 'Infinite or NaN values detected.';
   elseif mcode==8;
      mcodeStr = 'The function "compute_aim_matrices" returns complex numbers.';
   elseif mcode==-1;
      mcodeStr = 'No stable solution.';
   elseif mcode==-2;
      mcodeStr = 'Too many large generalized eigenvalues.';
   elseif mcode==-3;
      mcodeStr = 'Too few large generalized eigenvalues.';
   else;
      mcodeStr = 'Return code not properly specified.';
   end;
   if DSGEModel.ModelSolver==1;
      SolverStr = 'AiM';
   elseif DSGEModel.ModelSolver==2;
      SolverStr = 'Klein';
   elseif DSGEModel.ModelSolver==3;
      SolverStr = 'Gensys';
   end;
   txt = ['The ' SolverStr ' solver provided the return code: ' num2str(mcode) ', i.e., "' mcodeStr '"'];
   About(txt,'information',['YADA - ' SolverStr ' Solver Error'],200,500,CurrINI);
   return;
end;
%
% check if we have a time-varying measurement matrix H
%
if length(size(H))==3;
   %
   % check if H has the required size
   %
   if size(H,3)<DSGEModel.LastPeriod;
      ErrorStr = ['The number of time periods for the time-varying measurement matrix H (' int2str(size(H,3)) ') is lower than the number of observations (T = ' int2str(DSGEModel.LastPeriod) '). YADA is therefore unable to compute predictive checks for the one-step-ahead forecast errors.'];
      About(ErrorStr,'error','Error - Measurement Equation',200,500,CurrINI);
      return;
   end;
end;
%
% make sure that only non-zero shocks are kept
%
[B0,KeepShocks] = RemoveRedundantColumns(B0);
[r,q] = size(B0);
%
% store the mode values
%
B0M = B0;
FM = F;
AM = A;
HM = H;
RM = R;
%
% set up the date vector
%
DateVector = MakeDateVector(DSGEModel.BeginYear,DSGEModel.BeginPeriod,DSGEModel.EndYear,DSGEModel.EndPeriod,DSGEModel.DataFrequency);
DateVector = DateVector(DSGEModel.FirstPeriod+DSGEModel.KalmanFirstObservation-1:DSGEModel.LastPeriod);
%
% initialize the random number generator
%
if DSGEModel.RandomNumberValue==1;
   randn('state',0);
   rand('state',0);
else;
   randn('state',sum(100*clock));
   rand('state',sum(100*clock));
end;
%
% begin the loop
%
abort = '';
LastDraw = 0;
ThisSave = 0;
MeanEstimationTime = 0;
PredCheck = [];
%
% initialize index for saving
%
SaveAfterDraws = min(NumDraws,1000);
NumFiles = ceil(NumDraws/SaveAfterDraws);
%
% check if we should setup a wait handle
%
if IsPosterior==1;
   PriorHeader = 'Posterior';
else;
   PriorHeader = 'Prior';
end;
if DSGEModel.ShowProgress==1;
   ProgressStructure.title = ['Progress for ' num2str(NumDraws,'%0.0f') ' parameter draws'];
   ProgressStructure.facecolor = CurrINI.progress_facecolor;
   ProgressStructure.startfacecolor = CurrINI.progress_startfacecolor;
   ProgressStructure.edgecolor = CurrINI.progress_edgecolor;
   ProgressStructure.bgcolor = CurrINI.progress_bgcolor;
   ProgressStructure.stop = 0;
   ProgressStructure.clock = DSGEModel.ShowProgressClock;
   ProgressStructure.label = 'Mean estimation time:';
   %
   ProgressStructure.name = ['One-Step-Ahead ' PriorHeader ' Predictive Checks'];
   ProgressStructure.CurrINI = CurrINI;
   WaitHandle = ProgressDLG(0,ProgressStructure);
   set(WaitHandle,'Color',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'));
   drawnow;
else;
   txt = ['Please wait while YADA computes the ' lower(PriorHeader) ' predictive checks for the one-step-ahead forecast error covariances. Computations started at: ' StringTrim(datestr(now,14))];
   WaitHandle = WaitDLG(txt,'information',['One-Step-Ahead ' PriorHeader ' Predictive Checks'],500,150,CurrINI,0);
   WaitControls = get(WaitHandle,'UserData');
end;
%
% start the simulation
%
StopSimulation = 0;
if length(size(H))==2;
   for it=1:NumDraws;
      if DSGEModel.ShowProgress==1;
         abort = get(WaitHandle,'UserData');
         if strcmp(abort,'cancel')==1;
            StopSimulation = 1;
            break;
         else;
            ProgressDLG([it/NumDraws MeanEstimationTime]);
         end;
      end;
      LastDraw = LastDraw+1;
      %
      % Solve the model for the current parameter vector
      %
      theta(ScenarioParameters) = thetaPostSample(it,ScenarioParameters)';
      tic;
      [A,H,R,F,B0,status] = SolveDSGEModel(DSGEModel,theta,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
      if status==1;
         B0 = B0(:,KeepShocks);
      else;
         A = AM;
         H = HM;
         R = RM;
         F = FM;
         B0 = B0M;
      end;
      %
      % 1. compute the one-step-ahead forecasts using the actual data
      %
      % run the Kalman filter to get the estimate of the state forecasts
      %
      [logLikeValue,stat,lnLt,Yerror,Yhat,SigmaSqRoot,MSEY,InvMSEY,KalmanGain,Ksitt1,Ptt1] = RunKalmanFilter(DSGEModel.Y,DSGEModel.X,DSGEModel.FirstPeriod,DSGEModel.LastPeriod,A,H,F,B0,R,DSGEModel.InitialStateVector,DSGEModel.UseDoublingAlgorithm,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,DSGEModel.KalmanFirstObservation,DSGEModel.AllowUnitRoot,DSGEModel.StateCovConstValue,GetPosition(DSGEModel.StateVariablePositions,DSGEModel.UnitRootStates),DSGEModel.YNaNs,DSGEModel.KalmanAlgorithm);
      %
      % compute the forecast errors and take into account if there are NaNs
      %
      FE = Y-Yhat;
      if DSGEModel.YNaNs==0;
         %
         % there are no missing observation
         %
         OmegaObs = (1/Tsum)*FE*FE';
      else;
         FE(NaNY) = 0;
         OmegaObs = Tmat.*(FE*FE');
      end;
      %
      % 2. Simulate data
      %
      % initial values for the mean and covariance matrix of the distribution of the state variables
      %
      if isempty(DSGEModel.UnitRootStates)==1;
         [P0,stat] = DoublingAlgorithmLyapunov(F,B0*B0',DSGEModel.DAMaximumIterationsValue*100,10^(-(DSGEModel.DAToleranceValue+1)));
      else;
         StationaryPos = GetPosition(DSGEModel.StateVariablePositions,DSGEModel.UnitRootStates);
         P0 = DSGEModel.StateCovConstValue*eye(r);
         F1 = F(StationaryPos,StationaryPos);
         Q1 = B0(StationaryPos,:)*B0(StationaryPos,:)';
         [Pstat,stat] = DoublingAlgorithmLyapunov(F1,Q1,DSGEModel.DAMaximumIterationsValue*100,10^(-(DSGEModel.DAToleranceValue+1)));
         P0(StationaryPos,StationaryPos) = Pstat;
      end;
      Ksi0 = DSGEModel.InitialStateVector;
      [S,L] = SchurDecomposition(P0);
      %
      % deal with measurement errors
      %
      if rank(R)>0;
         [Rm,Lm] = SchurDecomposition(R);
         Rmeasure = Rm*sqrt(Lm);
         NumMeasure = size(Lm,1);
      else;
         Rmeasure = [];
         wSim = 0;
         NumMeasure = 0;
      end;
      %
      % draw the economic shocks
      %
      etaSim = randn(q,T);
      %
      % draw the measurement errors
      %
      if isempty(Rmeasure)==0;
         wSim = Rmeasure*randn(NumMeasure,T);
      end;
      %
      % draw an initial state
      %
      StateSim = zeros(r,T+1);
      StateSim(:,1) = Ksi0+(S*sqrt(L)*randn(size(L,2),1));
      %
      % generate the state matrix
      %
      for t=1:T;
         StateSim(:,t+1) = (F*StateSim(:,t))+(B0*etaSim(:,t));
      end;
      drawnow;
      %
      % remove the initial value of the state vector
      %
      StateSim = StateSim(:,2:T+1);
      %
      % create the observed variables
      %
      if k>0;
         YSim = (A'*XSim)+(H'*StateSim)+wSim;
      else;
         YSim = (H'*StateSim)+wSim;
      end;
      %
      % make sure that the simulated data has the same missing obs positions as the actual data
      %
      if DSGEModel.YNaNs==1;
         YSim(NaNYSim) = NaN;
      end;
      %
      % now run the Kalman filter
      %
      [logLikeValueSim,statSim,lnLtSim,YerrorSim,YhatSim,SigmaSqRootSim,MSEYSim,InvMSEYSim,KalmanGainSim,Ksitt1Sim,Ptt1Sim] = RunKalmanFilter(YSim,XSim,1,T,A,H,F,B0,R,DSGEModel.InitialStateVector,DSGEModel.UseDoublingAlgorithm,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,DSGEModel.KalmanFirstObservation,DSGEModel.AllowUnitRoot,DSGEModel.StateCovConstValue,GetPosition(DSGEModel.StateVariablePositions,DSGEModel.UnitRootStates),DSGEModel.YNaNs,DSGEModel.KalmanAlgorithm);
      %
      % compute the forecast errors and take into account if there are NaNs
      %
      FESim = YSim(:,DSGEModel.KalmanFirstObservation:T)-YhatSim;
      if DSGEModel.YNaNs==0;
         %
         % there are no missing observation
         %
         OmegaSim = (1/Tsum)*FESim*FESim';
      else;
         FESim(NaNY) = 0;
         OmegaSim = Tmat.*(FESim*FESim');
      end;
      PredCheck.Omega(LastDraw).Obs = vech(OmegaObs)';
      PredCheck.Omega(LastDraw).Sim = vech(OmegaSim)';
      MeanEstimationTime = (((it-1)/it)*MeanEstimationTime)+((1/it)*toc);
      if (LastDraw==SaveAfterDraws)|(it==NumDraws);
         ThisSave = ThisSave+1;
         LastDraw = 0;
         %
         % determine string for file to save in
         %
         if IsPosterior==1;
            file = FixFilePath([DSGEModel.OutputDirectory '\predcheck\DSGE-OSA-' DSGEModel.NameOfModel '-' num2str(CurrChain,'%0.0f') '-' SelParamStr '-' num2str(ThisSave,'%0.0f') '-' num2str(NumFiles,'%0.0f') '.' num2str(NumDraws,'%0.0f') '-' num2str(TotalDraws,'%0.0f') '.mat']);
         else;
            file = FixFilePath([DSGEModel.OutputDirectory '\predcheck\DSGE-OSA-Prior-' DSGEModel.NameOfModel '-' SelParamStr '-' num2str(ThisSave,'%0.0f') '-' num2str(NumFiles,'%0.0f') '.' num2str(NumDraws,'%0.0f') '.mat']);
         end;
         save(file,'PredCheck');
         %
         % restore the data structure
         %
         PredCheck = [];
      end;
      drawnow;
   end;
else;
   for it=1:NumDraws;
      if DSGEModel.ShowProgress==1;
         abort = get(WaitHandle,'UserData');
         if strcmp(abort,'cancel')==1;
            StopSimulation = 1;
            break;
         else;
            ProgressDLG([it/NumDraws MeanEstimationTime]);
         end;
      end;
      LastDraw = LastDraw+1;
      %
      % Solve the model for the current parameter vector
      %
      theta(ScenarioParameters) = thetaPostSample(it,ScenarioParameters)';
      tic;
      [A,H,R,F,B0,status] = SolveDSGEModel(DSGEModel,theta,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
      if status==1;
         B0 = B0(:,KeepShocks);
      else;
         A = AM;
         H = HM;
         R = RM;
         F = FM;
         B0 = B0M;
      end;
      %
      % 1. compute the one-step-ahead forecasts using the actual data
      %
      % run the Kalman filter to get the estimate of the state forecasts
      %
      [logLikeValue,stat,lnLt,Yerror,Yhat,SigmaSqRoot,MSEY,InvMSEY,KalmanGain,Ksitt1,Ptt1] = RunKalmanFilter(DSGEModel.Y,DSGEModel.X,DSGEModel.FirstPeriod,DSGEModel.LastPeriod,A,H,F,B0,R,DSGEModel.InitialStateVector,DSGEModel.UseDoublingAlgorithm,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,DSGEModel.KalmanFirstObservation,DSGEModel.AllowUnitRoot,DSGEModel.StateCovConstValue,GetPosition(DSGEModel.StateVariablePositions,DSGEModel.UnitRootStates),DSGEModel.YNaNs,DSGEModel.KalmanAlgorithm);
      %
      % compute the forecast errors and take into account if there are NaNs
      %
      FE = Y-Yhat;
      if DSGEModel.YNaNs==0;
         %
         % there are no missing observation
         %
         OmegaObs = (1/Tsum)*FE*FE';
      else;
         FE(NaNY) = 0;
         OmegaObs = Tmat.*(FE*FE');
      end;
      %
      % initial values for the mean and covariance matrix of the distribution of the state variables
      %
      if isempty(DSGEModel.UnitRootStates)==1;
         [P0,stat] = DoublingAlgorithmLyapunov(F,B0*B0',DSGEModel.DAMaximumIterationsValue*100,10^(-(DSGEModel.DAToleranceValue+1)));
      else;
         StationaryPos = GetPosition(DSGEModel.StateVariablePositions,DSGEModel.UnitRootStates);
         P0 = DSGEModel.StateCovConstValue*eye(r);
         F1 = F(StationaryPos,StationaryPos);
         Q1 = B0(StationaryPos,:)*B0(StationaryPos,:)';
         [Pstat,stat] = DoublingAlgorithmLyapunov(F1,Q1,DSGEModel.DAMaximumIterationsValue*100,10^(-(DSGEModel.DAToleranceValue+1)));
         P0(StationaryPos,StationaryPos) = Pstat;
      end;
      Ksi0 = DSGEModel.InitialStateVector;
      [S,L] = SchurDecomposition(P0);
      %
      % deal with measurement errors
      %
      if rank(R)>0;
         [Rm,Lm] = SchurDecomposition(R);
         Rmeasure = Rm*sqrt(Lm);
         NumMeasure = size(Lm,1);
      else;
         Rmeasure = [];
         wSim = 0;
         NumMeasure = 0;
      end;
      %
      % draw the economic shocks
      %
      etaSim = randn(q,T);
      %
      % draw the measurement errors
      %
      if isempty(Rmeasure)==0;
         wSim = Rmeasure*randn(NumMeasure,T);
      else;
         wSim = zeros(n,T);
      end;
      %
      % draw an initial state
      %
      StateSim = zeros(r,T+1);
      StateSim(:,1) = Ksi0+(S*sqrt(L)*randn(size(L,2),1));
      %
      % generate the state matrix
      %
      for t=1:T;
         StateSim(:,t+1) = (F*StateSim(:,t))+(B0*etaSim(:,t));
      end;
      drawnow;
      %
      % remove the initial value of the state vector
      %
      StateSim = StateSim(:,2:T+1);
      %
      % create the observed variables
      %
      if k>0;
         for t=DSGEModel.FirstPeriod:DSGEModel.LastPeriod;
            YSim(:,t-DSGEModel.FirstPeriod+1) = (A'*XSim(:,t-DSGEModel.FirstPeriod+1))+(H(:,:,t)'*StateSim(:,t-DSGEModel.FirstPeriod+1))+wSim(:,t-DSGEModel.FirstPeriod+1);
         end;
      else;
         for t=DSGEModel.FirstPeriod:DSGEModel.LastPeriod;
            YSim(:,t-DSGEModel.FirstPeriod+1) = (H(:,:,t)'*StateSim(:,t-DSGEModel.FirstPeriod+1))+wSim(:,t-DSGEModel.FirstPeriod+1);
         end;
      end;
      %
      % make sure that the simulated data has the same missing obs positions as the actual data
      %
      if DSGEModel.YNaNs==1;
         YSim(NaNYSim) = NaN;
      end;
      %
      % now run the Kalman filter
      %
      [logLikeValueSim,statSim,lnLtSim,YerrorSim,YhatSim,SigmaSqRootSim,MSEYSim,InvMSEYSim,KalmanGainSim,Ksitt1Sim,Ptt1Sim] = RunKalmanFilter(YSim,XSim,1,T,A,H,F,B0,R,DSGEModel.InitialStateVector,DSGEModel.UseDoublingAlgorithm,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,DSGEModel.KalmanFirstObservation,DSGEModel.AllowUnitRoot,DSGEModel.StateCovConstValue,GetPosition(DSGEModel.StateVariablePositions,DSGEModel.UnitRootStates),DSGEModel.YNaNs,DSGEModel.KalmanAlgorithm);
      %
      % compute the forecast errors and take into account if there are NaNs
      %
      FESim = YSim(:,DSGEModel.KalmanFirstObservation:T)-YhatSim;
      if DSGEModel.YNaNs==0;
         %
         % there are no missing observation
         %
         OmegaSim = (1/Tsum)*FESim*FESim';
      else;
         FESim(NaNY) = 0;
         OmegaSim = Tmat.*(FESim*FESim');
      end;
      PredCheck.Omega(LastDraw).Obs = vech(OmegaObs)';
      PredCheck.Omega(LastDraw).Sim = vech(OmegaSim)';
      MeanEstimationTime = (((it-1)/it)*MeanEstimationTime)+((1/it)*toc);
      if (LastDraw==SaveAfterDraws)|(it==NumDraws);
         ThisSave = ThisSave+1;
         LastDraw = 0;
         %
         % determine string for file to save in
         %
         if IsPosterior==1;
            file = FixFilePath([DSGEModel.OutputDirectory '\predcheck\DSGE-OSA-' DSGEModel.NameOfModel '-' num2str(CurrChain,'%0.0f') '-' SelParamStr '-' num2str(ThisSave,'%0.0f') '-' num2str(NumFiles,'%0.0f') '.' num2str(NumDraws,'%0.0f') '-' num2str(TotalDraws,'%0.0f') '.mat']);
         else;
            file = FixFilePath([DSGEModel.OutputDirectory '\predcheck\DSGE-OSA-Prior-' DSGEModel.NameOfModel '-' SelParamStr '-' num2str(ThisSave,'%0.0f') '-' num2str(NumFiles,'%0.0f') '.' num2str(NumDraws,'%0.0f') '.mat']);
         end;
         save(file,'PredCheck');
         %
         % restore the data structure
         %
         PredCheck = [];
      end;
      drawnow;
   end;
end;
%
% close the wait dialog
%
if DSGEModel.ShowProgress==1;
   if IsHandle(WaitHandle)==1;
      set(WaitHandle,'UserData','done');
      close(WaitHandle);
      drawnow;
      pause(0.02);
   end;
else;
   set(WaitControls.text,'UserData','done');
   delete(WaitHandle);
   drawnow;
   pause(0.02);
end;
%
% check if we have computed all output
%
if strcmp(abort,'cancel')==0;
   DoneCalc = 1;
end;

%
% end of DSGEOneStepAheadPredictiveChecks.m
%
