function [SyOmega,Omega,SxiOmega,Fi,Fmi,ei,emi] = DSGEPopulationSpectrum(H,R,F,B0)
% DSGEPopulationSpectrum: Computes the population spectrum at various frequencies for a DSGE model.
%
% USAGE:
%
%       [SyOmega,Omega,SxiOmega,Fi,Fmi,ei,emi] = DSGEPopulationSpectrum(H,R,F,B0)
%
% REQUIRED INPUT:  H (rxn) matrix where n is the number of observed variables and r the number of
%                          state variables. This matrix is part of the measurement equation of the
%                          state-space representation for the Kalman filter.
%
%                  R (nxn) matrix with the covariance matrix of the measurement errors.
%
%                  F (rxr) matrix from the state equation.
%
%                  B0 (rxq) matrix, where B0*B0' is the covariance matrix for the shocks to the
%                          state equation.
%
% REQUIRED OUTPUT: SyOmega (1xf) cell array with nxn matrices of the population spectrum of the
%                          observed variables at the f different frequencies from 0 to pi.
%
%                  Omega (1xf) vector with the frequencies from 0 to pi.
%
%                  SxiOmega (1xf) cell array with rxr matrices of the population spectrum of the
%                          state variables at the f frequencies from 0 to pi.
%
%                  Fi (1xf) cell array with rxr matrices of inv(F(exp(i*omega))) at the f
%                          frequencies from 0 to pi.
%
%                  Fmi (1xf) cell array with rxr matrices of inv(F(exp(-i*omega))) at the f
%                          frequencies from 0 to pi.
%
%                  ei (1xf) vector with exp(i*omega) at the f frequencies from 0 to pi.
%
%                  emi (1xf) vector with exp(-i*omega) at the f frequencies from 0 to pi.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   DG-R/MPR
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2017 European Central Bank.
%
%                       First version: May 5, 2010.
%                        This version: March 29, 2017.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 06-05-2010: Changed the Omega variable such that the max(diff(Omega))==min(diff(Omega)).
%
% * 12-05-2010: Made sure that the standard transpose is applied to Fi{k} instead of the conjugate.
%
% * 05-01-2011: Updated the documentation.
%
% * 06-01-2012: Updated the documentation.
%
% * 03-01-2013: Updated the documentation.
%
% * 28-01-2014: Updated the documentation.
%
% * 14-01-2015: Updated the documentation.
%
% * 22-02-2016: Updated the documentation.
%
% * 29-03-2017: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize
%
[r,n] = size(H);
q = size(B0,2);
Omega = (0:pi/299:pi);
f = length(Omega);
SyOmega = cell(1,f);
SxiOmega = cell(1,f);
Fi = cell(1,f);
Fmi = cell(1,f);
ei = exp(i*Omega);
emi = exp(-i*Omega);
%
% loop over all frequencies
%
Ir = eye(r);
Q = B0*B0';
for k=1:f;
   %
   % compute output that is used when differentiating the spectrum
   %
   Fi{k} = (Ir-(F*ei(k)))\Ir;
   Fmi{k} = (Ir-(F*emi(k)))\Ir;
   %
   % spectrum for state variables
   %
   SxiOmega{k} = (1/(2*pi))*Fmi{k}*Q*Fi{k}.';
   %
   % spectrum for observed variables
   %
   SyOmega{k} = (H'*SxiOmega{k}*H)+((1/(2*pi))*R);
   drawnow;
end;

%
% end of DSGEPopulationSpectrum.m
%
