function [MeanPrediction,CovPrediction,Quantiles,KernelX,KernelY] = DSGEPredictionDistribution(n,OutputDirectory,CondStr,ShockMethod,NameOfModel,MaxPH,NumDraws,NumPredPaths,AnnualizeData,TransData,CurrChain,IsPosterior,Weights,QuantileVector,VariableNames,ConfidenceRegionMethod,CurrINI)
% DSGEPredictionDistribution: Computes the mean value of conditional and unconditional prediction paths
%                             as well as quantiles by loading data from disk.
%
% USAGE:
%
%       [MeanPrediction,CovPrediction,Quantiles,KernelX,KernelY] = DSGEPredictionDistribution(n,OutputDirectory,CondStr, ...
%                                 ShockMethod,NameOfModel,MaxPH,NumDraws,NumPredPaths,AnnualizeData,TransData,CurrChain, ...
%                                 IsPosterior,Weights,QuantileVector,VariableNames,ConfidenceRegionMethod,CurrINI)
%
% REQUIRED INPUT:  n (integer) with the number of observed variables.
%
%                  OutputDirectory (string vector) with the full path to the base output directory.
%
%                  CondStr (string vector) that is either "Cond" or "".
%
%                  ShockMethod (string vector) that takes on the values "Direct-", "WaggonerZha-" or "".
%
%                  NameOfModel (string vector) with the name of the model.
%
%                  MaxPH (integer) with the length of the prediction horizon.
%
%                  NumDraws (integer) with the total number of parameter draws.
%
%                  NumPredPaths (integer) with tht total number of prediction paths per parameter draw.
%
%                  AnnualizeData (boolean) that take the value 1 if annualized results should be checked and 0 otherwise.
%
%                  TransData (boolean) that is 1 if the prediction paths should be transformed and 0 otherwise.
%
%                  CurrChain (integer) with the number of the currently used Markov chain.
%
%                  IsPosterior (boolean) that is 1 if draws from the posterior distribution are used and 0
%                              if draws from the prior are supplied.
%
%                  Weights (vector) of dimension NumDraws x 1 with the importance weights from the SMC with
%                                 likelihood tempering posterior sampler.
%
%                  QuantileVector (vector) of dimension 1xNumQuants with percent values for the quantiles,
%                              e.g. [5 10 90 95].
%
%                  VariableNames (string matrix) with the names of the observed variables.
%
%                  ConfidenceRegionMethod (integer) that takes the value 1 for the equal tails method, and
%                              2 for the highest probability density method.
%
%                  CurrINI (structure) with initialization information.
%
% REQUIRED OUTPUT: MeanPrediction (matrix) of dimension nxMaxPH with the mean predictions.
%
%                  CovPrediction (matrix) of dimension nxnxMaxPH with the sample covariance matrix for the predictions.
%
%                  Quantiles (structure) of dimension NumQuants where each has a field called "Quantile" which
%                              is an nxh matrix with the quantiles of the prediction paths. The second field
%                              is "percent" that gives the percentile value.
%
%                  KernelX (3D matrix) of dimension n x 2^8 x MaxPH with the x-axis values.
%
%                  KernelY (3D matrix) of dimension n x 2^8 x MaxPH with the y-axis values.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   DG-R/MPR
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2017 European Central Bank.
%
%                       First version: February 23, 2007.
%                        This version: August 24, 2017.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 12-03-2007: Added the input "CondStr" to handle conditional predictive distributions.
%
% * 13-11-2007: Updated the documentation.
%
% * 19-12-2007: Added the input "ShockMethod".
%
% * 20-12-2007: Hunting an annualization bug.
%
% * 10-01-2008: Changed the last drawnow call to pause(0.02).
%
% * 05-05-2008: Added the input variable "TransData".
%
% * 23-05-2008: Updated the documentation.
%
% * 22-01-2009: Updated the documentation.
%
% * 03-03-2009: Added the output variables "KernelX" and "KernelY".
%
% * 26-03-2009: Added the input variable "IsPosterior".
%
% * 24-04-2009: Added figure refresh code for wait dialog.
%
% * 31-07-2009: Added the output variable "CovPrediction".
%
% * 03-08-2009: Updated the documentation.
%
% * 05-01-2010: Updated the documentation.
%
% * 08-06-2010: Speeded up the sorting of quantile values by skipping looping over the number of
%               observed variables. The computation speed of mean predictions and path-based forecast
%               uncertainty has also been improved.
%
% * 05-01-2011: Updated the documentation.
%
% * 13-01-2011: The function now supports unix and mac paths.
%
% * 01-03-2011: Made sure that "CovPrediction" is numerically positive semidefinite.
%
% * 06-01-2012: Updated the documentation.
%
% * 29-08-2012: Added support for higest probability density confidence bands.
%
% * 03-01-2013: Updated the documentation.
%
% * 03-09-2013: Added "try-catch" code around possibly too large matrices.
%
% * 28-01-2014: Updated the documentation.
%
% * 14-01-2015: Updated the documentation.
%
% * 22-02-2016: Updated the documentation.
%
% * 29-03-2017: Updated the documentation.
%
% * 24-08-2017: Added the Weights input variable.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize
%
MeanPrediction = zeros(n,MaxPH);
CovPrediction = zeros(n,n,MaxPH);
NumQuants = length(QuantileVector);
KernelX = zeros(n,2^8,MaxPH);
KernelY = KernelX;
if isempty(Weights)==1;
   Weights = ones(NumDraws,1);
end;
%
% Initialize quantiles output
%
for i=1:NumQuants;
   Quantiles(i).Quantile = zeros(n,MaxPH);
   Quantiles(i).percent = QuantileVector(i);
end;
%
if isempty(CondStr)==1;
   HeadStr = 'Predictive Distributions';
   txtStr = '';
else;
   HeadStr = 'Conditional Predictive Distributions';
   txtStr = 'conditional ';
end;
if IsPosterior==0;
   HeadStr = [HeadStr ' - Prior'];
end;
%
% set up a wait dialog
%
txt = ['Please wait while YADA computes the mean ' txtStr 'predictions for the observed variables. The data are loaded sequentially from disk and it may therefore take some time.'];
WaitHandle = WaitDLG(txt,'information',HeadStr,500,150,CurrINI,0);
WaitControls = get(WaitHandle,'UserData');
drawnow;
pause(0.02);
%
% compute the mean prediction
%
TotalPaths = NumPredPaths*NumDraws;
for ParamIndex=1:NumDraws;
   if IsPosterior==1;
      if AnnualizeData==0;
         if TransData==0;
            file = FixFilePath([OutputDirectory '\predictions\DSGE-' CondStr 'Prediction-Original-' ShockMethod NameOfModel '-' num2str(CurrChain,'%0.0f') '-' num2str(MaxPH,'%0.0f') '-' num2str(ParamIndex,'%0.0f') '.' num2str(NumDraws,'%0.0f') '.' num2str(NumPredPaths,'%0.0f') '.mat']);
         else;
            file = FixFilePath([OutputDirectory '\predictions\DSGE-' CondStr 'Prediction-Transformed-' ShockMethod NameOfModel '-' num2str(CurrChain,'%0.0f') '-' num2str(MaxPH,'%0.0f') '-' num2str(ParamIndex,'%0.0f') '.' num2str(NumDraws,'%0.0f') '.' num2str(NumPredPaths,'%0.0f') '.mat']);
         end;
      else;
         file = FixFilePath([OutputDirectory '\predictions\DSGE-' CondStr 'Prediction-Annual-' ShockMethod NameOfModel '-' num2str(CurrChain,'%0.0f') '-' num2str(MaxPH,'%0.0f') '-' num2str(ParamIndex,'%0.0f') '.' num2str(NumDraws,'%0.0f') '.' num2str(NumPredPaths,'%0.0f') '.mat']);
      end;
   else;
      if AnnualizeData==0;
         if TransData==0;
            file = FixFilePath([OutputDirectory '\predictions\DSGE-' CondStr 'Prediction-Original-Prior-' ShockMethod NameOfModel '-' num2str(MaxPH,'%0.0f') '-' num2str(ParamIndex,'%0.0f') '.' num2str(NumDraws,'%0.0f') '.' num2str(NumPredPaths,'%0.0f') '.mat']);
         else;
            file = FixFilePath([OutputDirectory '\predictions\DSGE-' CondStr 'Prediction-Transformed-Prior-' ShockMethod NameOfModel '-' num2str(MaxPH,'%0.0f') '-' num2str(ParamIndex,'%0.0f') '.' num2str(NumDraws,'%0.0f') '.' num2str(NumPredPaths,'%0.0f') '.mat']);
         end;
      else;
         file = FixFilePath([OutputDirectory '\predictions\DSGE-' CondStr 'Prediction-Annual-Prior-' ShockMethod NameOfModel '-' num2str(MaxPH,'%0.0f') '-' num2str(ParamIndex,'%0.0f') '.' num2str(NumDraws,'%0.0f') '.' num2str(NumPredPaths,'%0.0f') '.mat']);
      end;
   end;
   %
   % load the data
   %
   PredData = load(file);
   PredPaths = PredData.PredPaths;
   MeanPrediction = MeanPrediction+((1/TotalPaths)*sum(PredPaths*Weights(ParamIndex),3));
   for j=1:MaxPH;
      PP = shiftdim(PredPaths(:,j,:),2)';
      CovPrediction(:,:,j) = CovPrediction(:,:,j)+((Weights(ParamIndex)/TotalPaths)*PP*PP');
   end;
   clear('PredData');
   drawnow;
end;
%
% compute the central second moment from the non-central
%
for j=1:MaxPH;
   CovPrediction(:,:,j) = CovPrediction(:,:,j)-(MeanPrediction(:,j)*MeanPrediction(:,j)');
   CovPrediction(:,:,j) = diag(abs(diag(CovPrediction(:,:,j))))+tril(CovPrediction(:,:,j),-1)+tril(CovPrediction(:,:,j),-1)';
end;
%
% compute all the quantile values
%
for j=1:MaxPH;
   if ConfidenceRegionMethod==1;
      txt = ['Please wait while YADA computes quantile values and kernel density estimates for the ' txtStr 'predictions of the observed variables. Current forecast horizon ' int2str(j) '.'];
   else;
      txt = ['Please wait while YADA computes confidence/credible regions and kernel density estimates for the ' txtStr 'predictions of the observed variables. Current forecast horizon ' int2str(j) '.'];
   end;
   set(WaitControls.text,'String',txt);
   drawnow;
   %
   % initialzie a temporary variable
   %
   try;
      TempVar = zeros(TotalPaths,n);
      for ParamIndex=1:NumDraws;
         if IsPosterior==1;
            if AnnualizeData==0;
               if TransData==0;
                  file = FixFilePath([OutputDirectory '\predictions\DSGE-' CondStr 'Prediction-Original-' ShockMethod NameOfModel '-' num2str(CurrChain,'%0.0f') '-' num2str(MaxPH,'%0.0f') '-' num2str(ParamIndex,'%0.0f') '.' num2str(NumDraws,'%0.0f') '.' num2str(NumPredPaths,'%0.0f') '.mat']);
               else;
                  file = FixFilePath([OutputDirectory '\predictions\DSGE-' CondStr 'Prediction-Transformed-' ShockMethod NameOfModel '-' num2str(CurrChain,'%0.0f') '-' num2str(MaxPH,'%0.0f') '-' num2str(ParamIndex,'%0.0f') '.' num2str(NumDraws,'%0.0f') '.' num2str(NumPredPaths,'%0.0f') '.mat']);
               end;
            else;
               file = FixFilePath([OutputDirectory '\predictions\DSGE-' CondStr 'Prediction-Annual-' ShockMethod NameOfModel '-' num2str(CurrChain,'%0.0f') '-' num2str(MaxPH,'%0.0f') '-' num2str(ParamIndex,'%0.0f') '.' num2str(NumDraws,'%0.0f') '.' num2str(NumPredPaths,'%0.0f') '.mat']);
            end;
         else;
            if AnnualizeData==0;
               if TransData==0;
                  file = FixFilePath([OutputDirectory '\predictions\DSGE-' CondStr 'Prediction-Original-Prior-' ShockMethod NameOfModel '-' num2str(MaxPH,'%0.0f') '-' num2str(ParamIndex,'%0.0f') '.' num2str(NumDraws,'%0.0f') '.' num2str(NumPredPaths,'%0.0f') '.mat']);
               else;
                  file = FixFilePath([OutputDirectory '\predictions\DSGE-' CondStr 'Prediction-Transformed-Prior-' ShockMethod NameOfModel '-' num2str(MaxPH,'%0.0f') '-' num2str(ParamIndex,'%0.0f') '.' num2str(NumDraws,'%0.0f') '.' num2str(NumPredPaths,'%0.0f') '.mat']);
               end;
            else;
               file = FixFilePath([OutputDirectory '\predictions\DSGE-' CondStr 'Prediction-Annual-Prior-' ShockMethod NameOfModel '-' num2str(MaxPH,'%0.0f') '-' num2str(ParamIndex,'%0.0f') '.' num2str(NumDraws,'%0.0f') '.' num2str(NumPredPaths,'%0.0f') '.mat']);
            end;
         end;
         %
         % load the data
         %
         PredData = load(file);
         PredPaths = PredData.PredPaths;
         for p=1:NumPredPaths;
            CurrPath = p+(NumPredPaths*(ParamIndex-1));
            TempVar(CurrPath,:) = PredPaths(:,j,p)';
         end;
         clear('PredData');
         drawnow;
      end;
      %
      % TempVar is now fully loaded and may
      %
      TempVar = sort(TempVar);
      if ConfidenceRegionMethod==1;
         for k=1:NumQuants;
            if QuantileVector(k)<50;
               QuantVal = ceil((QuantileVector(k)/100)*TotalPaths);
            else;
               QuantVal = floor((QuantileVector(k)/100)*TotalPaths);
            end;
            Quantiles(k).Quantile(:,j) = TempVar(QuantVal,:)';
         end;
      else;
         for k=1:floor(NumQuants/2);
            ConfDraws = floor((QuantileVector(NumQuants-k+1)-QuantileVector(k))*TotalPaths/100);
            for t=1:n;
               Bounds = HighestProbabilityDensityBand(TempVar(:,t),ConfDraws);
               Quantiles(k).Quantile(t,j) = Bounds(1);
               Quantiles(NumQuants-k+1).Quantile(t,j) = Bounds(2);
            end;
         end;
      end;
   catch;
      for t=1:n;
         TempVar = zeros(TotalPaths,1);
         for ParamIndex=1:NumDraws;
            if IsPosterior==1;
               if AnnualizeData==0;
                  if TransData==0;
                     file = FixFilePath([OutputDirectory '\predictions\DSGE-' CondStr 'Prediction-Original-' ShockMethod NameOfModel '-' num2str(CurrChain,'%0.0f') '-' num2str(MaxPH,'%0.0f') '-' num2str(ParamIndex,'%0.0f') '.' num2str(NumDraws,'%0.0f') '.' num2str(NumPredPaths,'%0.0f') '.mat']);
                  else;
                     file = FixFilePath([OutputDirectory '\predictions\DSGE-' CondStr 'Prediction-Transformed-' ShockMethod NameOfModel '-' num2str(CurrChain,'%0.0f') '-' num2str(MaxPH,'%0.0f') '-' num2str(ParamIndex,'%0.0f') '.' num2str(NumDraws,'%0.0f') '.' num2str(NumPredPaths,'%0.0f') '.mat']);
                  end;
               else;
                  file = FixFilePath([OutputDirectory '\predictions\DSGE-' CondStr 'Prediction-Annual-' ShockMethod NameOfModel '-' num2str(CurrChain,'%0.0f') '-' num2str(MaxPH,'%0.0f') '-' num2str(ParamIndex,'%0.0f') '.' num2str(NumDraws,'%0.0f') '.' num2str(NumPredPaths,'%0.0f') '.mat']);
               end;
            else;
               if AnnualizeData==0;
                  if TransData==0;
                     file = FixFilePath([OutputDirectory '\predictions\DSGE-' CondStr 'Prediction-Original-Prior-' ShockMethod NameOfModel '-' num2str(MaxPH,'%0.0f') '-' num2str(ParamIndex,'%0.0f') '.' num2str(NumDraws,'%0.0f') '.' num2str(NumPredPaths,'%0.0f') '.mat']);
                  else;
                     file = FixFilePath([OutputDirectory '\predictions\DSGE-' CondStr 'Prediction-Transformed-Prior-' ShockMethod NameOfModel '-' num2str(MaxPH,'%0.0f') '-' num2str(ParamIndex,'%0.0f') '.' num2str(NumDraws,'%0.0f') '.' num2str(NumPredPaths,'%0.0f') '.mat']);
                  end;
               else;
                  file = FixFilePath([OutputDirectory '\predictions\DSGE-' CondStr 'Prediction-Annual-Prior-' ShockMethod NameOfModel '-' num2str(MaxPH,'%0.0f') '-' num2str(ParamIndex,'%0.0f') '.' num2str(NumDraws,'%0.0f') '.' num2str(NumPredPaths,'%0.0f') '.mat']);
               end;
            end;
            %
            % load the data
            %
            PredData = load(file);
            PredPaths = PredData.PredPaths;
            for p=1:NumPredPaths;
               CurrPath = p+(NumPredPaths*(ParamIndex-1));
               TempVar(CurrPath,1) = PredPaths(t,j,p)';
            end;
            clear('PredData');
            drawnow;
         end;
         %
         % TempVar is now fully loaded and may
         %
         TempVar = sort(TempVar);
         if ConfidenceRegionMethod==1;
            for k=1:NumQuants;
               if QuantileVector(k)<50;
                  QuantVal = ceil((QuantileVector(k)/100)*TotalPaths);
               else;
                  QuantVal = floor((QuantileVector(k)/100)*TotalPaths);
               end;
               Quantiles(k).Quantile(t,j) = TempVar(QuantVal,1);
            end;
         else;
            for k=1:floor(NumQuants/2);
               ConfDraws = floor((QuantileVector(NumQuants-k+1)-QuantileVector(k))*TotalPaths/100);
               Bounds = HighestProbabilityDensityBand(TempVar,ConfDraws);
               Quantiles(k).Quantile(t,j) = Bounds(1);
               Quantiles(NumQuants-k+1).Quantile(t,j) = Bounds(2);
            end;
         end;
      end;
   end;
   drawnow;
   %
   for i=1:n;
      if TempVar(1,i)+1.0e-6<TempVar(TotalPaths,i);
         [x,f] = YADAMHDensEst(TempVar(:,i),2^8,0,'gaussian',[]);
         KernelX(i,:,j) = x';
         KernelY(i,:,j) = f';
      end;
   end;
end;
%
% close the wait dialog
%
set(WaitControls.text,'UserData','done');
delete(WaitHandle);
drawnow;
pause(0.02);

%
% end of DSGEPredictionDistribution.m
%
