function SVEs = DSGESimSmoothDistribution(DSGEModel,CurrINI,VarType,SelectedParameters,NumDraws,TotalDraws,CurrChain,TotalPaths,IsPosterior,Weights)
% DSGESimSmoothDistribution: Computes the mean and quantile values of unobserved variables based on simulation smoothing.
%
% USAGE:
%
%       SVEs = DSGESimSmoothDistribution(DSGEModel,CurrINI,VarType,SelectedParameters,NumDraws,TotalDraws,CurrChain,...
%                                        TotalPaths,IsPosterior,Weights)
%
% REQUIRED INPUT:  DSGEModel (structure) with fields containing all the information about a DSGE Model for YADA.
%
%                  CurrINI (structure) with initialization information.
%
%                  VarType (integer) which determines the type of unobserved variable that should be computed.
%                          1 is state variable, 2 is economic shock, and 3 is measurement error.
%
%                  SelectedParameters (vector) of dimension NumParam which indicates which parameters can vary from
%                                     one draw to the next (1) and which parameters are fixed at posterior mode (0).
%
%                  NumDraws (integer) with the number of effective draws from the posterior distribution of the
%                                     parameters.
%
%                  TotalDraws (integer) with the number of draws from the posterior distribution of the parameters
%                                     that could have been used.
%
%                  CurrChain (integer) with the number for the current Markov chain.
%
%                  TotalPaths (integer) with the total number of paths to simulate per parameter value.
%
%                  IsPosterior (boolean) that is 1 if draws from the posterior distribution are used and 0
%                                 if draws from the prior are supplied.
%
%                  Weights (vector) of dimension NumDraws x 1 with the importance weights from the SMC with
%                                 likelihood tempering posterior sampler.
%
% REQUIRED OUTPUT: SVEs (structure) with with fields that depend on VarType. For the state variable estimates the field
%                                     is: "KsitT". This is the mean estimates of the smooth estimates of the states.
%                                     For the state shock estimates the field is: "etatT", the smooth estimates of the
%                                     shocks. Moreover, the field "ShockNames" gives the names of the shocks that have
%                                     a non-zero effect on the observed variables. For the measuremente rrors the field
%                                     is: "wtT", the smooth estimates of the measurement errors. Moreover, the field
%                                     "ErrorIndicator" is a vector that indicates which equations have a positive
%                                     measurement error variance (1) and which don't (0). In addition, the vector
%                                     field "Quantiles" is used, with subfields "percent" and field names determined
%                                     by the VarType.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   DG-R/MPR
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2017 European Central Bank.
%
%                       First version: October 28, 2010.
%                        This version: August 24, 2017.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 29-10-2010: Updated the documentation.
%
% * 05-01-2011: Updated the documentation.
%
% * 12-01-2011: The function now supports unix and mac paths.
%
% * 08-04-2011: The function now uses "ShockAliases" instead of "StateShockNames".
%
% * 06-01-2012: Updated the documentation.
%
% * 28-08-2012: Added support for higest probability density confidence bands.
%
% * 04-01-2013: Updated the documentation.
%
% * 03-09-2013: Added "try-catch" code around possibly too large matrices.
%
% * 28-01-2014: Updated the documentation.
%
% * 14-01-2015: Updated the documentation.
%
% * 23-02-2016: Updated the documentation.
%
% * 29-03-2017: Updated the documentation.
%
% * 24-08-2017: Added the Weights input variable.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize
%
SVEs = [];
if isempty(Weights)==1;
   Weights = ones(NumDraws,1);
end;
%
% fix the information needed to setup the file names
%
SelParamStr = strrep(num2str(SelectedParameters),' ','');
%
NumFiles = ceil(NumDraws/1);
NumQuants = length(DSGEModel.Percentiles);
NumPaths = TotalPaths/4;
%
% we need to first determine the size of the data
%
if IsPosterior==1;
   file = FixFilePath([DSGEModel.OutputDirectory '\state\DSGE-SimSmooth-' DSGEModel.NameOfModel '-' int2str(CurrChain) '-' SelParamStr '-1-' int2str(NumFiles) '.' int2str(NumDraws) '-' int2str(TotalDraws) '.' int2str(TotalPaths) '.mat']);
else;
   file = FixFilePath([DSGEModel.OutputDirectory '\state\DSGE-SimSmooth-Prior-' DSGEModel.NameOfModel '-' SelParamStr '-1-' int2str(NumFiles) '.' int2str(TotalDraws) '.' int2str(TotalPaths) '.mat']);
end;
SVEData = load(file);
SVEs.DateVector = SVEData.DateVector;
%
PriorHeader = 'Posterior';
if IsPosterior==0;
   PriorHeader = 'Prior';
end;
%
if VarType==1;
   %
   % state variables
   %
   [r,T] = size(SVEData.SimSmooth(1,1).KsitT);
   SVEs.KsitT = zeros(r,T);
   for i=1:NumQuants;
      SVEs.Quantiles(i).KsitT = zeros(r,T);
      SVEs.Quantiles(i).percent = DSGEModel.Percentiles(i);
   end;
   %
   % open a wait dialog
   %
   txt = ['Please wait while YADA computes the percentile values for the state variable paths from the simulation smoother. The data are loaded sequentially from disk and it may therefore take some time. Current variable: ' StringTrim(DSGEModel.StateVariableNames(1,:)) '.'];
   WaitHandle = WaitDLG(txt,'information',['Simulation Smoother - ' PriorHeader ' Distribution'],500,200,CurrINI,0);
   WaitControls = get(WaitHandle,'UserData');
   drawnow;
   pause(0.02);
   %
   Weight = kron(Weights,ones(TotalPaths,1));
   for i=1:r;
      %
      % update the wait dialog
      %
      txt = ['Please wait while YADA computes the percentile values for the state variable paths from the simulation smoother. The data are loaded sequentially from disk and it may therefore take some time. Current variable: ' StringTrim(DSGEModel.StateVariableNames(i,:)) '.'];
      set(WaitControls.text,'String',txt);
      drawnow;
      %
      try;
         TempT = zeros(NumDraws*TotalPaths,T);
         CurrSaves = 0;
         ThisSave = 1;
         for ThisSave=1:NumFiles;
            if IsPosterior==1;
               file = FixFilePath([DSGEModel.OutputDirectory '\state\DSGE-SimSmooth-' DSGEModel.NameOfModel '-' int2str(CurrChain) '-' SelParamStr '-' int2str(ThisSave) '-' int2str(NumFiles) '.' int2str(NumDraws) '-' int2str(TotalDraws) '.' int2str(TotalPaths) '.mat']);
            else;
               file = FixFilePath([DSGEModel.OutputDirectory '\state\DSGE-SimSmooth-Prior-' DSGEModel.NameOfModel '-' SelParamStr '-' int2str(ThisSave) '-' int2str(NumFiles) '.' int2str(TotalDraws) '.' int2str(TotalPaths) '.mat']);
            end;
            SVEData = load(file);
            SimSmooth = SVEData.SimSmooth;
            %
            % load data from CurrSaves+1 to CurrSaves+TotalPaths
            %
            for s=1:NumPaths;
               TempT(CurrSaves+1+(4*(s-1)),:) = SimSmooth(s,1).KsitT(i,:);
               TempT(CurrSaves+2+(4*(s-1)),:) = SimSmooth(s,2).KsitT(i,:);
               TempT(CurrSaves+3+(4*(s-1)),:) = SimSmooth(s,3).KsitT(i,:);
               TempT(CurrSaves+4+(4*(s-1)),:) = SimSmooth(s,4).KsitT(i,:);
            end;
            CurrSaves = CurrSaves+TotalPaths;
            drawnow;
         end;
         %
         % sort each column
         %
         SVEs.KsitT(i,:) = (1/(NumDraws*TotalPaths))*sum(TempT.*(Weight*ones(1,T)),1);
         TempT = sort(TempT);
         if DSGEModel.ConfidenceRegionMethod==1;
            for l=1:NumQuants;
               if DSGEModel.Percentiles(l)<50;
                  QuantVal = ceil((DSGEModel.Percentiles(l)/100)*(NumDraws*TotalPaths));
               else;
                  QuantVal = floor((DSGEModel.Percentiles(l)/100)*(NumDraws*TotalPaths));
               end;
               SVEs.Quantiles(l).KsitT(i,:) = TempT(QuantVal,:);
            end;
         else;
            for l=1:floor(NumQuants/2);
               ConfDraws = floor((DSGEModel.Percentiles(NumQuants-l+1)-DSGEModel.Percentiles(l))*NumDraws*TotalPaths/100);
               for t=1:T;
                  Bounds = HighestProbabilityDensityBand(TempT(:,t),ConfDraws);
                  SVEs.Quantiles(l).KsitT(i,t) = Bounds(1);
                  SVEs.Quantiles(NumQuants-l+1).KsitT(i,t) = Bounds(2);
               end;
            end;
         end;
      catch;
         for t=1:T;
            TempT = zeros(NumDraws*TotalPaths,1);
            CurrSaves = 0;
            ThisSave = 1;
            for ThisSave=1:NumFiles;
               if IsPosterior==1;
                  file = FixFilePath([DSGEModel.OutputDirectory '\state\DSGE-SimSmooth-' DSGEModel.NameOfModel '-' int2str(CurrChain) '-' SelParamStr '-' int2str(ThisSave) '-' int2str(NumFiles) '.' int2str(NumDraws) '-' int2str(TotalDraws) '.' int2str(TotalPaths) '.mat']);
               else;
                  file = FixFilePath([DSGEModel.OutputDirectory '\state\DSGE-SimSmooth-Prior-' DSGEModel.NameOfModel '-' SelParamStr '-' int2str(ThisSave) '-' int2str(NumFiles) '.' int2str(TotalDraws) '.' int2str(TotalPaths) '.mat']);
               end;
               SVEData = load(file);
               SimSmooth = SVEData.SimSmooth;
               %
               % load data from CurrSaves+1 to CurrSaves+TotalPaths
               %
               for s=1:NumPaths;
                  TempT(CurrSaves+1+(4*(s-1)),1) = SimSmooth(s,1).KsitT(i,t);
                  TempT(CurrSaves+2+(4*(s-1)),1) = SimSmooth(s,2).KsitT(i,t);
                  TempT(CurrSaves+3+(4*(s-1)),1) = SimSmooth(s,3).KsitT(i,t);
                  TempT(CurrSaves+4+(4*(s-1)),1) = SimSmooth(s,4).KsitT(i,t);
               end;
               CurrSaves = CurrSaves+TotalPaths;
               drawnow;
            end;
            %
            % sort 
            %
            SVEs.KsitT(i,t) = (1/(NumDraws*TotalPaths))*sum(TempT.*Weight,1);
            TempT = sort(TempT);
            if DSGEModel.ConfidenceRegionMethod==1;
               for l=1:NumQuants;
                  if DSGEModel.Percentiles(l)<50;
                     QuantVal = ceil((DSGEModel.Percentiles(l)/100)*(NumDraws*TotalPaths));
                  else;
                     QuantVal = floor((DSGEModel.Percentiles(l)/100)*(NumDraws*TotalPaths));
                  end;
                  SVEs.Quantiles(l).KsitT(i,t) = TempT(QuantVal,1);
               end;
            else;
               for l=1:floor(NumQuants/2);
                  ConfDraws = floor((DSGEModel.Percentiles(NumQuants-l+1)-DSGEModel.Percentiles(l))*NumDraws*TotalPaths/100);
                  Bounds = HighestProbabilityDensityBand(TempT,ConfDraws);
                  SVEs.Quantiles(l).KsitT(i,t) = Bounds(1);
                  SVEs.Quantiles(NumQuants-l+1).KsitT(i,t) = Bounds(2);
               end;
            end;
         end;
      end;
      drawnow;
   end;
elseif VarType==2;
   %
   % economic shocks
   %
   ShockNames = DSGEModel.ShockAliases(SVEData.KeepShocks,:);
   for j=1:size(ShockNames,1);
      ShockNames(j,:) = strrep(ShockNames(j,:),'_',' ');
   end;
   SVEs.ShockNames = ShockNames;
   [q,T] = size(SVEData.SimSmooth(1,1).etatT);
   SVEs.etatT = zeros(q,T);
   for i=1:NumQuants;
      SVEs.Quantiles(i).etatT = zeros(q,T);
      SVEs.Quantiles(i).percent = DSGEModel.Percentiles(i);
   end;
   %
   % open a wait dialog
   %
   txt = ['Please wait while YADA computes the percentile values for the state shock paths from the simulation smoother. The data are loaded sequentially from disk and it may therefore take some time. Current variable: ' StringTrim(ShockNames(1,:)) '.'];
   WaitHandle = WaitDLG(txt,'information',['Simulation Smoother - ' PriorHeader ' Distribution'],500,200,CurrINI,0);
   WaitControls = get(WaitHandle,'UserData');
   drawnow;
   pause(0.02);
   %
   Weight = kron(Weights,ones(TotalPaths,1));
   for i=1:q;
      %
      % update the wait dialog
      %
      txt = ['Please wait while YADA computes the percentile values for the state shock paths from the simulation smoother. The data are loaded sequentially from disk and it may therefore take some time. Current variable: ' StringTrim(ShockNames(i,:)) '.'];
      set(WaitControls.text,'String',txt);
      drawnow;
      %
      try;
         TempT = zeros(NumDraws*TotalPaths,T);
         CurrSaves = 0;
         ThisSave = 1;
         for ThisSave=1:NumFiles;
            if IsPosterior==1;
               file = FixFilePath([DSGEModel.OutputDirectory '\state\DSGE-SimSmooth-' DSGEModel.NameOfModel '-' int2str(CurrChain) '-' SelParamStr '-' int2str(ThisSave) '-' int2str(NumFiles) '.' int2str(NumDraws) '-' int2str(TotalDraws) '.' int2str(TotalPaths) '.mat']);
            else;
               file = FixFilePath([DSGEModel.OutputDirectory '\state\DSGE-SimSmooth-Prior-' DSGEModel.NameOfModel '-' SelParamStr '-' int2str(ThisSave) '-' int2str(NumFiles) '.' int2str(TotalDraws) '.' int2str(TotalPaths) '.mat']);
            end;
            SVEData = load(file);
            SimSmooth = SVEData.SimSmooth;
            %
            % load data from CurrSaves+1 to CurrSaves+TotalPaths
            %
            for s=1:NumPaths;
               TempT(CurrSaves+1+(4*(s-1)),:) = SimSmooth(s,1).etatT(i,:);
               TempT(CurrSaves+2+(4*(s-1)),:) = SimSmooth(s,2).etatT(i,:);
               TempT(CurrSaves+3+(4*(s-1)),:) = SimSmooth(s,3).etatT(i,:);
               TempT(CurrSaves+4+(4*(s-1)),:) = SimSmooth(s,4).etatT(i,:);
            end;
            CurrSaves = CurrSaves+TotalPaths;
            drawnow;
         end;
         %
         % sort each column
         %
         SVEs.etatT(i,:) = (1/(NumDraws*TotalPaths))*sum(TempT.*(Weight*ones(1,T)),1);
         TempT = sort(TempT);
         if DSGEModel.ConfidenceRegionMethod==1;
            for l=1:NumQuants;
               if DSGEModel.Percentiles(l)<50;
                  QuantVal = ceil((DSGEModel.Percentiles(l)/100)*(NumDraws*TotalPaths));
               else;
                  QuantVal = floor((DSGEModel.Percentiles(l)/100)*(NumDraws*TotalPaths));
               end;
               SVEs.Quantiles(l).etatT(i,:) = TempT(QuantVal,:);
            end;
         else;
            for l=1:floor(NumQuants/2);
               ConfDraws = floor((DSGEModel.Percentiles(NumQuants-l+1)-DSGEModel.Percentiles(l))*NumDraws*TotalPaths/100);
               for t=1:T;
                  Bounds = HighestProbabilityDensityBand(TempT(:,t),ConfDraws);
                  SVEs.Quantiles(l).etatT(i,t) = Bounds(1);
                  SVEs.Quantiles(NumQuants-l+1).etatT(i,t) = Bounds(2);
               end;
            end;
         end;
      catch;
         for t=1:T;
            TempT = zeros(NumDraws*TotalPaths,1);
            CurrSaves = 0;
            ThisSave = 1;
            for ThisSave=1:NumFiles;
               if IsPosterior==1;
                  file = FixFilePath([DSGEModel.OutputDirectory '\state\DSGE-SimSmooth-' DSGEModel.NameOfModel '-' int2str(CurrChain) '-' SelParamStr '-' int2str(ThisSave) '-' int2str(NumFiles) '.' int2str(NumDraws) '-' int2str(TotalDraws) '.' int2str(TotalPaths) '.mat']);
               else;
                  file = FixFilePath([DSGEModel.OutputDirectory '\state\DSGE-SimSmooth-Prior-' DSGEModel.NameOfModel '-' SelParamStr '-' int2str(ThisSave) '-' int2str(NumFiles) '.' int2str(TotalDraws) '.' int2str(TotalPaths) '.mat']);
               end;
               SVEData = load(file);
               SimSmooth = SVEData.SimSmooth;
               %
               % load data from CurrSaves+1 to CurrSaves+TotalPaths
               %
               for s=1:NumPaths;
                  TempT(CurrSaves+1+(4*(s-1)),1) = SimSmooth(s,1).etatT(i,t);
                  TempT(CurrSaves+2+(4*(s-1)),1) = SimSmooth(s,2).etatT(i,t);
                  TempT(CurrSaves+3+(4*(s-1)),1) = SimSmooth(s,3).etatT(i,t);
                  TempT(CurrSaves+4+(4*(s-1)),1) = SimSmooth(s,4).etatT(i,t);
               end;
               CurrSaves = CurrSaves+TotalPaths;
               drawnow;
            end;
            %
            % sort
            %
            SVEs.etatT(i,t) = (1/(NumDraws*TotalPaths))*sum(TempT.*Weight,1);
            TempT = sort(TempT);
            if DSGEModel.ConfidenceRegionMethod==1;
               for l=1:NumQuants;
                  if DSGEModel.Percentiles(l)<50;
                     QuantVal = ceil((DSGEModel.Percentiles(l)/100)*(NumDraws*TotalPaths));
                  else;
                     QuantVal = floor((DSGEModel.Percentiles(l)/100)*(NumDraws*TotalPaths));
                  end;
                  SVEs.Quantiles(l).etatT(i,t) = TempT(QuantVal,1);
               end;
            else;
               for l=1:floor(NumQuants/2);
                  ConfDraws = floor((DSGEModel.Percentiles(NumQuants-l+1)-DSGEModel.Percentiles(l))*NumDraws*TotalPaths/100);
                  Bounds = HighestProbabilityDensityBand(TempT,ConfDraws);
                  SVEs.Quantiles(l).etatT(i,t) = Bounds(1);
                  SVEs.Quantiles(NumQuants-l+1).etatT(i,t) = Bounds(2);
               end;
            end;
         end;
      end;
      drawnow;
   end;
else;
   %
   % measurement errors
   %
   SVEs.ErrorIndicator = SVEData.ErrorIndicator;
   [n,T] = size(SVEData.SimSmooth(1,1).wtT);
   SVEs.wtT = zeros(n,T);
   for i=1:NumQuants;
      SVEs.Quantiles(i).wtT = zeros(n,T);
      SVEs.Quantiles(i).percent = DSGEModel.Percentiles(i);
   end;
   %
   % open a wait dialog
   %
   txt = ['Please wait while YADA computes the percentile values for the measurement error paths from the simulation smoother. The data are loaded sequentially from disk and it may therefore take some time. Measurement error for variable: ' StringTrim(DSGEModel.VariableNames(1,:)) '.'];
   WaitHandle = WaitDLG(txt,'information',['Simulation Smoother - ' PriorHeader ' Distribution'],500,200,CurrINI,0);
   WaitControls = get(WaitHandle,'UserData');
   drawnow;
   pause(0.02);
   %
   Weight = kron(Weights,ones(TotalPaths,1));
   for i=1:n;
      if SVEs.ErrorIndicator(i)==1;
         %
         % update the wait dialog
         %
         txt = ['Please wait while YADA computes the percentile values for the measurement error paths from the simulation smoother. The data are loaded sequentially from disk and it may therefore take some time. Measurement error for variable: ' StringTrim(DSGEModel.VariableNames(i,:)) '.'];
         set(WaitControls.text,'String',txt);
         drawnow;
         %
         try;
            TempT = zeros(NumDraws*TotalPaths,T);
            CurrSaves = 0;
            ThisSave = 1;
            for ThisSave=1:NumFiles;
               if IsPosterior==1;
                  file = FixFilePath([DSGEModel.OutputDirectory '\state\DSGE-SimSmooth-' DSGEModel.NameOfModel '-' int2str(CurrChain) '-' SelParamStr '-' int2str(ThisSave) '-' int2str(NumFiles) '.' int2str(NumDraws) '-' int2str(TotalDraws) '.' int2str(TotalPaths) '.mat']);
               else;
                  file = FixFilePath([DSGEModel.OutputDirectory '\state\DSGE-SimSmooth-Prior-' DSGEModel.NameOfModel '-' SelParamStr '-' int2str(ThisSave) '-' int2str(NumFiles) '.' int2str(TotalDraws) '.' int2str(TotalPaths) '.mat']);
               end;
               SVEData = load(file);
               SimSmooth = SVEData.SimSmooth;
               %
               % load data from CurrSaves+1 to CurrSaves+TotalPaths
               %
               for s=1:NumPaths;
                  TempT(CurrSaves+1+(4*(s-1)),:) = SimSmooth(s,1).wtT(i,:);
                  TempT(CurrSaves+2+(4*(s-1)),:) = SimSmooth(s,2).wtT(i,:);
                  TempT(CurrSaves+3+(4*(s-1)),:) = SimSmooth(s,3).wtT(i,:);
                  TempT(CurrSaves+4+(4*(s-1)),:) = SimSmooth(s,4).wtT(i,:);
               end;
               CurrSaves = CurrSaves+TotalPaths;
               drawnow;
            end;
            %
            % sort each column
            %
            SVEs.wtT(i,:) = (1/(NumDraws*TotalPaths))*sum(TempT.*(Weight*ones(1,T)),1);
            TempT = sort(TempT);
            if DSGEModel.ConfidenceRegionMethod==1;
               for l=1:NumQuants;
                  if DSGEModel.Percentiles(l)<50;
                     QuantVal = ceil((DSGEModel.Percentiles(l)/100)*(NumDraws*TotalPaths));
                  else;
                     QuantVal = floor((DSGEModel.Percentiles(l)/100)*(NumDraws*TotalPaths));
                  end;
                  SVEs.Quantiles(l).wtT(i,:) = TempT(QuantVal,:);
               end;
            else;
               for l=1:floor(NumQuants/2);
                  ConfDraws = floor((DSGEModel.Percentiles(NumQuants-l+1)-DSGEModel.Percentiles(l))*NumDraws*TotalPaths/100);
                  for t=1:T;
                     Bounds = HighestProbabilityDensityBand(TempT(:,t),ConfDraws);
                     SVEs.Quantiles(l).wtT(i,t) = Bounds(1);
                     SVEs.Quantiles(NumQuants-l+1).wtT(i,t) = Bounds(2);
                  end;
               end;
            end;
         catch;
            for t=1:T;
               TempT = zeros(NumDraws*TotalPaths,1);
               CurrSaves = 0;
               ThisSave = 1;
               for ThisSave=1:NumFiles;
                  if IsPosterior==1;
                     file = FixFilePath([DSGEModel.OutputDirectory '\state\DSGE-SimSmooth-' DSGEModel.NameOfModel '-' int2str(CurrChain) '-' SelParamStr '-' int2str(ThisSave) '-' int2str(NumFiles) '.' int2str(NumDraws) '-' int2str(TotalDraws) '.' int2str(TotalPaths) '.mat']);
                  else;
                     file = FixFilePath([DSGEModel.OutputDirectory '\state\DSGE-SimSmooth-Prior-' DSGEModel.NameOfModel '-' SelParamStr '-' int2str(ThisSave) '-' int2str(NumFiles) '.' int2str(TotalDraws) '.' int2str(TotalPaths) '.mat']);
                  end;
                  SVEData = load(file);
                  SimSmooth = SVEData.SimSmooth;
                  %
                  % load data from CurrSaves+1 to CurrSaves+TotalPaths
                  %
                  for s=1:NumPaths;
                     TempT(CurrSaves+1+(4*(s-1)),1) = SimSmooth(s,1).wtT(i,t);
                     TempT(CurrSaves+2+(4*(s-1)),1) = SimSmooth(s,2).wtT(i,t);
                     TempT(CurrSaves+3+(4*(s-1)),1) = SimSmooth(s,3).wtT(i,t);
                     TempT(CurrSaves+4+(4*(s-1)),1) = SimSmooth(s,4).wtT(i,t);
                  end;
                  CurrSaves = CurrSaves+TotalPaths;
                  drawnow;
               end;
               %
               % sort
               %
               SVEs.wtT(i,t) = (1/(NumDraws*TotalPaths))*sum(TempT.*Weight,1);
               TempT = sort(TempT);
               if DSGEModel.ConfidenceRegionMethod==1;
                  for l=1:NumQuants;
                     if DSGEModel.Percentiles(l)<50;
                        QuantVal = ceil((DSGEModel.Percentiles(l)/100)*(NumDraws*TotalPaths));
                     else;
                        QuantVal = floor((DSGEModel.Percentiles(l)/100)*(NumDraws*TotalPaths));
                     end;
                     SVEs.Quantiles(l).wtT(i,t) = TempT(QuantVal,1);
                  end;
               else;
                  for l=1:floor(NumQuants/2);
                     ConfDraws = floor((DSGEModel.Percentiles(NumQuants-l+1)-DSGEModel.Percentiles(l))*NumDraws*TotalPaths/100);
                     Bounds = HighestProbabilityDensityBand(TempT,ConfDraws);
                     SVEs.Quantiles(l).wtT(i,t) = Bounds(1);
                     SVEs.Quantiles(NumQuants-l+1).wtT(i,t) = Bounds(2);
                  end;
               end;
            end;
         end;
         drawnow;
      end;
   end;
end;
%
% close the wait dialog
%
set(WaitControls.text,'UserData','done');
delete(WaitHandle);
drawnow;
pause(0.02);

%
% end of DSGESimSmoothDistribution.m
%
