function [SpecDec,status,kalmanstatus] = DSGESpectralDecompTheta(theta,thetaPositions,ModelParameters,VarStr,DSGEModel,CurrINI)
% DSGESpectralDecompTheta: Computes a spectral decomposition of the contemporaneous covariance matrix
%                          for the state-space model. The decomposition is made in terms of all structural
%                          shocks and the measurement errors.
%
% USAGE:
%
%       [SpecDec,status,kalmanstatus] = DSGESpectralDecompTheta(theta,thetaPositions,ModelParameters,VarStr,DSGEModel,CurrINI)
%
% REQUIRED INPUT:  theta (vector) with values for the original parameters of the DSGE model.
%
%                  thetaPositions (structure) with fields parameter and distribution. This function
%                                 needs the former field to know which parameter name corresponds to
%                                 position i in theta.
%
%                  ModelParameters (structure) whose fields have names given by the parameter names in the
%                                 DSGE model.
%
%                  VarStr (string vector) that takes on the values "Observed Variables" and "State Variables".
%
%                  DSGEModel (structure) whose fields contain all required information
%                                 about the model, such as input files, variable and sample
%                                 data. Actual observations are not stored in this file.
%
%                  CurrINI (structure) with initialization information.
%
% REQUIRED OUTPUT: SpecDec (structure) with fields "Frequencies", "States", "Shocks", "MeasureError", "Y",
%                                 "SigmaXi", "SigmaY", "OriginalVariance", "R", "ShockNames", "ShockGroups", "ShockGroupNames",
%                                 and "ShockGroupColors". If annualized output should also be prepared, the
%                                 additional fields are "AnnualY", "AnnualMeasureError", "AnnualYStates",
%                                 "AnnualYShocks", "AnnualVariance", "ShockAliases", and "ShockColors".
%
% OPTIONAL OUTPUT: status (integer) Indicates whether the solution of the model is unique or not.
%                                 mcode is 1 when unique, 0 if not computed, and various other values when
%                                 there are problems with the solution. Specifically:
%
%                                      mcode = 2:      Roots not correctly computed by real_schur
%                                      mcode = 3:      Too many big roots
%                                      mcode = 35:     Too many big roots, and q(:,right) is singular
%                                      mcode = 4:      Too few big roots
%                                      mcode = 45:     Too few big roots, and q(:,right) is singular
%                                      mcode = 5:      q(:,right) is singular
%                                      mcode = 61:     Too many exact shiftrights
%                                      mcode = 62:     Too many numeric shiftrights
%                                      mcode = 7:      The a matrix has infinite or NaN values. Eigenvalues
%                                                      cannot be calculated.
%                                      mcode = 8:      The function "compute_aim_matrices" returns complex
%                                                      numbers.
%                                      mcode = -1:     No stable solution.
%                                      mcode = -2:     Too many large generalized eigenvalues.
%                                      mcode = -3:     Too few large generalized eigenvalues.
%                                      else            Return code not properly specified
%
%                  kalmanstatus (integer) indicating the covariance stationarity of the state variables. If all the
%                                 eigenvalues of F are inside the unit circle, then kalmanstatus=0 and the log-likelihood function
%                                 is calculated. If not, then status=1 and initP is set to 0. In addition, kalmanstatus can be
%                                 -1 if ths forecast covariance matrix is not positive definite for some time period.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   DG-R/MPR
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2017 European Central Bank.
%
%                       First version: October 5, 2009.
%                        This version: March 29, 2017.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 06-10-2009: Added the annualized output.
%
% * 07-10-2009: Made sure that "CreateDefaultColors" receives an integer as input.
%
% * 08-10-2009: The variances of the annualized observed variables are now computed in "AnnualVariance". In addition,
%               the variances of the original variables are stored in the field "OriginalVariance".
%
% * 05-01-2010: Updated the documentation.
%
% * 05-05-2010: Updated the documentation.
%
% * 06-05-2010: Omega now satisfies max(diff(Omega))==min(diff(Omega)).
%
% * 28-05-2010: Fixed a bug when annualizing the variance for monthly data. Also, the annualized variance
%               computations now take the annualscale field into account.
%
% * 17-12-2010: Took into account that "SolveDSGEModel" takes the input variable "OrderQZ" from the CurrINI
%               structure.
%
% * 19-12-2010: Updated the documentation.
%
% * 20-12-2010: Took into account which model solver is used when an error message is provided from a failed
%               run of the "SolveDSGEModel" function.
%
% * 05-01-2011: Updated the documentation.
%
% * 10-01-2011: Changed all dos copy calls to "CopyFile" calls.
%
% * 05-04-2011: Added the fields "ShockAliases" and "ShockColors" to "SpecDec".
%
% * 18-04-2011: "ShockColors" can now also be initialized from the color of the "ShockGroups" the different
%               shocks belong to.
%
% * 06-01-2012: Made sure that ShockGroupColors is only used when it has as many rows as there are shock groups.
%
% * 02-01-2013: Took into account that "SolveDSGEModel" now supports "ErrMsg".
%
% * 28-01-2014: Updated the documentation.
%
% * 29-01-2014: Switched the "inv" function for "\".
%
% * 14-02-2014: Fixed a bug where "InvIrFexpm" was erroneously called "nvIrFexpm".
%
% * 14-01-2015: Updated the documentation.
%
% * 18-03-2015: Switched to the new IsHandle function.
%
% * 22-02-2016: Updated the documentation.
%
% * 29-03-2017: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize
%
SpecDec = [];
if nargout==1;
   %
   % copy files to the tmp directory for posterior mode values
   %
   [stat,msg] = CopyFile([GetPath(DSGEModel.AIMDataFile) 'compute_aim_matrices.m'],[pwd '\tmp']);
   [stat,msg] = CopyFile(DSGEModel.MeasurementEquationFile,[pwd '\tmp']);
   if FileExist(DSGEModel.UpdateParameterFile)==1;
      [stat,msg] = CopyFile(DSGEModel.UpdateParameterFile,[pwd '\tmp']);
   end;
else;
   status = 1;
   kalmanstatus = 0;
end;
%
% load the AIMData file
%
ErrorStr = '';
try;
   AIMData = load(DSGEModel.AIMDataFile);
catch;
   ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' lasterr];
end;
if isempty(ErrorStr)==0;
   if nargout>1;
      status = 0;
      kalmanstatus = 0;
   end;
   About(ErrorStr,'information','YADA - Bad MAT File',120,500,CurrINI);
   return;
end;
%
% try to solve the model
%
ErrorStr = '';
try;
   [A,H,R,F,B0,mcode,ErrMsg] = SolveDSGEModel(DSGEModel,theta,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
   if isempty(ErrMsg)==0;
      ErrorStr = ['YADA caught an error when trying to solve the DSGE model. Message caught is:' ErrMsg];
   end;
catch;
   ErrorStr = ['YADA caught an error when trying to solve the DSGE model. Message caught is:' lasterr];
end;
if isempty(ErrorStr)==0
   About(ErrorStr,'error','YADA Error',CurrINI.scrsz(4)-50,500,CurrINI);
   drawnow;
   return;
end;
[B0,KeepVar] = RemoveRedundantColumns(B0);
[r,q] = size(B0);
if q==0;
   txt = 'The DSGE model doesn''t have any structural shocks. YADA is therefore unable to compute any spectral decompostions.';
   About(txt,'information','YADA - No Shocks',200,500,CurrINI);
   return;
end;
n = DSGEModel.n;
%
% check if the solution is unique and convergent
%
if nargout>1;
   status = mcode;
   if mcode~=1;
      if mcode==2;
         mcodeStr = 'Roots are not correctly computed by real_schur.'
      elseif mcode==3;
         mcodeStr = 'Too many big roots.';
      elseif mcode==35;
         mcodeStr = 'Too many big roots, and q(:,right) is singular.';
      elseif mcode==4;
         mcodeStr = 'Too few big roots.';
      elseif mcode==45;
         mcodeStr = 'Too few big roots, and q(:,right) is singular.';
      elseif mcode==5;
         mcodeStr = 'q(:,right) is singular.';
      elseif mcode==61;
         mcodeStr = 'Too many exact shiftrights.';
      elseif mcode==62;
         mcodeStr = 'Too many numeric shiftrights.';
      elseif mcode==7;
         mcodeStr = 'Infinite or NaN values detected.';
      elseif mcode==8;
         mcodeStr = 'The function "compute_aim_matrices" returns complex numbers.';
      elseif mcode==-1;
         mcodeStr = 'No stable solution.';
      elseif mcode==-2;
         mcodeStr = 'Too many large generalized eigenvalues.';
      elseif mcode==-3;
         mcodeStr = 'Too few large generalized eigenvalues.';
      else;
         mcodeStr = 'Return code not properly specified.';
      end;
      if DSGEModel.ModelSolver==1;
         SolverStr = 'AiM';
      elseif DSGEModel.ModelSolver==2;
         SolverStr = 'Klein';
      elseif DSGEModel.ModelSolver==3;
         SolverStr = 'Gensys';
      end;
      txt = ['The ' SolverStr ' solver provided the return code: ' int2str(mcode) ', i.e., "' mcodeStr '"'];
      About(txt,'information',['YADA - ' SolverStr ' Solver Error'],200,500,CurrINI);
      return;
   end;
end;
%
% check if measurement equations are properly defined for all time periods
%
if length(size(H))==3;
   if size(H,3)<DSGEModel.T;
      ErrorStr = ['The number of time periods for the time-varying measurement matrix H (' int2str(size(H,3)) ') is less than the number of observations (T = ' int2str(DSGEModel.T) '). YADA has therefore aborted from the spectral decompositions.'];
      About(ErrorStr,'error','Error - Measurement Equation',200,500,CurrINI);
      return;
   end;
   %
   % check the sample (if needed for Kalman filter)
   %
   [FirstPeriod,LastPeriod,SubSampleStrMat] = CreateSubSample(DSGEModel);
   %
   % compute the average H-matrix
   %
   H = H(:,:,FirstPeriod+DSGEModel.KalmanFirstObservation-1:LastPeriod);
   H = (1/size(H,3))*sum(H,3);
end;
%
% compute the population covariances
%
[SigmaXi,stat] = DoublingAlgorithmLyapunov(F,B0*B0',DSGEModel.DAMaximumIterationsValue*100,10^(-(DSGEModel.DAToleranceValue+1)));
SpecDec.SigmaY = (H'*SigmaXi*H)+R;
SpecDec.SigmaXi = SigmaXi;
%
% Determine proper frequencies
%
Omega = (0:pi/299:pi);
SpecDec.Frequencies = Omega;
f = length(Omega);
SpecDec.States = cell(1,f);
SpecDec.Shocks = cell(q,f);
SpecDec.Y = cell(1,f);
SpecDec.MeasureError = (1/(2*pi))*R;
SpecDec.ShockNames = DSGEModel.StateShockNames(KeepVar,:);
SpecDec.ShockGroups = DSGEModel.ShockGroups(KeepVar);
SpecDec.ShockGroupNames = DSGEModel.ShockGroupNames(unique(SpecDec.ShockGroups),:);
if (isempty(DSGEModel.ShockGroupColors)==0)&(size(DSGEModel.ShockGroupColors,1)==length(unique(SpecDec.ShockGroups)));
   SpecDec.ShockGroupColors = DSGEModel.ShockGroupColors(unique(SpecDec.ShockGroups),:);
else;
   SpecDec.ShockGroupColors = CreateDefaultColors(length(unique(SpecDec.ShockGroups)));
end;
SpecDec.ShockAliases = DSGEModel.ShockAliases(KeepVar,:);
if isempty(DSGEModel.ShockColors)==0;
   SpecDec.ShockColors = DSGEModel.ShockColors(KeepVar,:);
else;
   if max(DSGEModel.ShockGroups)==length(DSGEModel.StateShockPositions);
      SpecDec.ShockColors = CreateDefaultColors(length(KeepVar));
   else;
      SpecDec.ShockColors = CreateShockColors(DSGEModel.ShockGroups(KeepVar),DSGEModel.ShockGroupColors);
   end;
end;
Ir = eye(r);
FT = F';
%
% check if we should compute decompositions for observed variables
% or state variables
%
if strcmp(lower(VarStr),'observed variables')==1;
   IsObsVar = 1;
   SpecDec.SigmaXi = H'*SpecDec.SigmaXi*H;
   SpecDec.R = R;
   SpecDec.OriginalVariance = diag(SpecDec.SigmaY);
else;
   IsObsVar = 0;
   SpecDec.OriginalVariance = diag(SpecDec.SigmaXi);
end;
%
% check if we should compute the annual filter
%
if (max(DSGEModel.annual)>1)&(IsObsVar==1);
   ComputeAnnual = 1;
   SpecDec.AnnualY = cell(1,f);
   SpecDec.AnnualMeasureError = cell(1,f);
   SpecDec.AnnualYStates = cell(1,f);
   SpecDec.AnnualYShocks = cell(q,f);
   %
   % compute the annualized variances of the observed variables
   %
   if max(DSGEModel.annual)==4;
      Cov1 = H'*F*SigmaXi*H;
      Cov2 = H'*F*F*SigmaXi*H;
      Cov3 = H'*F*F*F*SigmaXi*H;
      AnnCov = (4*SpecDec.SigmaY)+(3*(Cov1+Cov1'))+(2*(Cov2+Cov2'))+(Cov3+Cov3');
      SpecDec.AnnualVariance = ((DSGEModel.annual==1)' .* diag(SpecDec.SigmaY))+((DSGEModel.annual==4)' .* DSGEModel.annualscale' .* DSGEModel.annualscale' .* diag(AnnCov));
   elseif max(DSGEModel.annual)==12;
      Cov1 = H'*F*SigmaXi*H;
      Cov2 = H'*(F^2)*SigmaXi*H;
      Cov3 = H'*(F^3)*SigmaXi*H;
      Cov4 = H'*(F^4)*SigmaXi*H;
      Cov5 = H'*(F^5)*SigmaXi*H;
      Cov6 = H'*(F^6)*SigmaXi*H;
      Cov7 = H'*(F^7)*SigmaXi*H;
      Cov8 = H'*(F^8)*SigmaXi*H;
      Cov9 = H'*(F^9)*SigmaXi*H;
      Cov10 = H'*(F^10)*SigmaXi*H;
      Cov11 = H'*(F^11)*SigmaXi*H;
      AnnCov = (12*SpecDec.SigmaY)+(11*(Cov1+Cov1'))+(10*(Cov2+Cov2'))+(9*(Cov3+Cov3'))+(8*(Cov4+Cov4'))+(7*(Cov5+Cov5'))+ ...
               (6*(Cov6+Cov6'))+(5*(Cov7+Cov7'))+(4*(Cov8+Cov8'))+(3*(Cov9+Cov9'))+(2*(Cov10+Cov10'))+(Cov11+Cov11');
      SpecDec.AnnualVariance = ((DSGEModel.annual==1)' .* diag(SpecDec.SigmaY))+((DSGEModel.annual==12)' .* DSGEModel.annualscale' .* DSGEModel.annualscale' .* diag(AnnCov));
   end;
else;
   ComputeAnnual = 0;
end;
%
if n>=7;
   %
   % initialize the wait dialog
   %
   if nargout>1;
      EstType = 'Initial Values';
   else;
      EstType = 'Posterior Mode';
   end;
   txt = ['Please wait while YADA computes spectral decompositions of the ' lower(VarStr) '.'];
   WaitHandle = WaitDLG(txt,'information',['Spectral Decompositions - ' EstType],500,200,CurrINI,0);
   WaitControls = get(WaitHandle,'UserData');
end;
%
% compute the decompositions for one frequency at a time
%
for k=1:f;
   %
   % compute the term for each shock
   %
   omega = Omega(k);
   if ComputeAnnual==1;
      [AnnMatrix,ConjAnnMatrix] = AnnualFilter(DSGEModel.annual,DSGEModel.annualscale,omega);
      SpecDec.AnnualMeasureError{1,k} = (1/(2*pi))*AnnMatrix*R*ConjAnnMatrix;
   end;
   %
   expmiomega = exp(-i*omega);
   expiomega = exp(i*omega);
   InvIrFexpm = (Ir-(F*expmiomega))\Ir;
   InvIrFexp = (Ir-(FT*expiomega))\Ir;
   %
   for j=1:q;
      B0j = B0(:,j);
      ShockCov = B0j*B0j';
      if IsObsVar==1;
         SpecDec.Shocks{j,k} = (1/(2*pi))*H'*InvIrFexpm*ShockCov*InvIrFexp*H;
      else;
         SpecDec.Shocks{j,k} = (1/(2*pi))*InvIrFexpm*ShockCov*InvIrFexp;
      end;
      if ComputeAnnual==1;
         SpecDec.AnnualYShocks{j,k} = AnnMatrix*SpecDec.Shocks{j,k}*ConjAnnMatrix;
      end;
   end;
   if IsObsVar==1;
      SpecDec.States{1,k} = (1/(2*pi))*H'*InvIrFexpm*B0*B0'*InvIrFexp*H;
      SpecDec.Y{1,k} = SpecDec.States{k}+SpecDec.MeasureError;
   else;
      SpecDec.States{1,k} = (1/(2*pi))*InvIrFexpm*B0*B0'*InvIrFexp;
      SpecDec.Y{1,k} = (H'*SpecDec.States{k}*H)+SpecDec.MeasureError;
   end;
   if ComputeAnnual==1;
      SpecDec.AnnualYStates{1,k} = AnnMatrix*SpecDec.States{1,k}*ConjAnnMatrix;
      SpecDec.AnnualY{1,k} = AnnMatrix*SpecDec.Y{1,k}*ConjAnnMatrix;
   end;
   %
   drawnow;
end;
%
% close the wait dialog
%
if n>=7;
   if IsHandle(WaitHandle)==1;
      set(WaitControls.text,'UserData','done');
      delete(WaitHandle);
      drawnow;
      pause(0.02);
   end;
end;

%
% end of DSGESpectralDecompTheta.m
%
