function [A,H,R,F,B0] = DSGEStateSpaceMoments(theta,thetaDraws,thetaPositions,ModelParameters,IsPosterior,Weights,TotalDraws,CurrChain,DSGEModel,CurrINI)
% DSGEStateSpaceMoments: Computes the mean and standard deviation of draws from the prior or posterior
%                        distribution of the state-space matrices of the DSGE model.
%
% USAGE:
%
%       [A,H,R,F,B0] = DSGEStateSpaceMoments(theta,thetaDraws,thetaPositions,ModelParameters,IsPosterior,...
%                                            Weights,TotalDraws,CurrChain,DSGEModel,CurrINI)
%
% REQUIRED INPUT:  theta (vector) of dimension NumParam with the posterior mode estimates or the initial
%                        values of the original DSGE model parameters.
%
%                  thetaDraws (matrix) of dimension NumDrawsxNumParam with values of the parameters of the
%                        DSGE model.
%
%                  thetaPositions (structure) where thetaPositions(i).parameter is a string vector giving the name
%                        of the model parameter in position i of theta. The field "distribution" has a string
%                        vector giving the assumed prior distribution of the parameter.
%
%                  ModelParameters (structure) whose fields have names given by the parameter names in the
%                        DSGE model.
%
%                  IsPosterior (boolean) that is 1 if draws from the posterior distribution are used and 0
%                        if draws from the prior are supplied.
%
%                  Weights (vector) of dimension NumDraws x 1 with the importance weights from the SMC with
%                        likelihood tempering posterior sampler.
%
%                  TotalDraws (integer) with the total number of parameter draws that could have been used.
%
%                  CurrChain (integer) with the number for the current Markov chain.
%
%                  DSGEModel (structure) whose fields contain all required information about the model,
%                        such as input files, variable and sample data.
%
%                  CurrINI (structure) with initialization information.
%
% REQUIRED OUTPUT: A (structure) with fields "Mean" and "Std" of dimension kxn holding the mean and standard
%                        deviation of the parameters on the exogenous variables in the measurement equation.
%
%                  H (structure) with fields "Mean" and "Std" of dimension rxn holding the mean and standard
%                        deviation of the parameters on the state variables in the measurement equation.
%
%                  R (structure) with fields "Mean" and "Std" of dimension nxn holding the mean and standard
%                        deviation of the measurement error covariance matrix.
%
%                  F (structure) with fields "Mean" and "Std" of dimension rxr holding the mean and standard
%                        deviation of the transition matrix parameters in the state equation.
%
%                  B0 (structure) with fields "Mean" and "Std" of dimension rxq holding the mean and standard
%                        deviation of the parameters on the structural shocks in the state equation.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   DG-R/MPR
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2017 European Central Bank.
%
%                       First version: March 2, 2012.
%                        This version: August 25, 2017.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 19-03-2012: Updated the documentation.
%
% * 02-01-2013: Added try-catch code around the call to "SolveDSGEModel".
%
% * 28-01-2014: Updated the documentation.
%
% * 14-01-2015: Updated the documentation.
%
% * 22-02-2016: Updated the documentation.
%
% * 29-03-2017: Updated the documentation.
%
% * 25-08-2017: Added the "Weights" input variable.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize
%
A = [];
H = [];
R = [];
F = [];
B0 = [];
%
%
% copy files to the tmp folder
%
[stat,msg] = CopyFile([GetPath(DSGEModel.AIMDataFile) 'compute_aim_matrices.m'],[pwd '\tmp']);
[stat,msg] = CopyFile(DSGEModel.MeasurementEquationFile,[pwd '\tmp']);
if FileExist(DSGEModel.UpdateParameterFile)==1;
   [stat,msg] = CopyFile(DSGEModel.UpdateParameterFile,[pwd '\tmp']);
end;
%
% load the AIMData file
%
ErrorStr = '';
try;
   AIMData = load(DSGEModel.AIMDataFile);
catch;
   ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' lasterr];
end;
if isempty(ErrorStr)==0;
   About(ErrorStr,'information','YADA - Bad MAT File',120,500,CurrINI);
   return;
end;
%
% create the output directory
%
stat = MakeDir(DSGEModel.OutputDirectory,'statespace');
if stat~=1;
   if ispc==1;
      txt = ['YADA was for some reason unable to create the directory "' DSGEModel.OutputDirectory '\statespace". The computation of the mean and standard deviation for the state-space form has therefore been aborted.'];
   else;
      txt = ['YADA was for some reason unable to create the directory "' DSGEModel.OutputDirectory '/statespace". The computation of the mean and standard deviation for the state-space form has therefore been aborted.'];
   end;
   About(txt,'information','YADA - Directory Creation Problem',180,500,CurrINI);
   return;
end;
%
[NumDraws,NumParam] = size(thetaDraws);
if isempty(Weights)==1;
   Weights = ones(NumDraws,1);
end;
%
% get default values for the state-space form
%
[Amode,Hmode,Rmode,Fmode,B0mode,status] = SolveDSGEModel(DSGEModel,theta,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
%
n = DSGEModel.n;
k = DSGEModel.k;
r = length(DSGEModel.StateVariablePositions);
q = length(DSGEModel.StateShockPositions);
%
% intialize mean and standard deviation values
%
if k>0;
   Amean = zeros(k,n);
   Astd = zeros(k,n);
else;
   Amean = [];
   Astd = [];
end;
Hmean = zeros(r,n);
Hstd = zeros(r,n);
Rmean = zeros(n,n);
Rstd = zeros(n,n);
Fmean = zeros(r,r);
Fstd = zeros(r,r);
B0mean = zeros(r,q);
B0std = zeros(r,q);
%
if IsPosterior==1;
   DistStr = 'Posterior';
else;
   DistStr = 'Prior';
end;
%
%
% setup a wait dialog while checking the properties of the prior draws
%
txt = ['Please wait while YADA solves the DSGE model for ' int2str(NumDraws) ' values from the ' lower(DistStr) ' distribution. Computations started at: ' StringTrim(datestr(now,14)) '.'];
WaitHandle = WaitDLG(txt,'information',['State-Space Form - ' int2str(NumDraws) ' ' DistStr ' Draws'],500,300,CurrINI,1);
WaitControls = get(WaitHandle,'UserData');
drawnow;
%
StopEstimation = 0;
BadDraws = 0;
for i=1:NumDraws;
   %
   % check if the cancel button was pressed
   %
   if strcmp(get(WaitControls.cancel,'UserData'),'cancel')==1;
      StopEstimation = 1;
      break;
   end;
   mcode = 1;
   try;
      [A,H,R,F,B0,mcode] = SolveDSGEModel(DSGEModel,thetaDraws(i,:)',thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
   catch;
      mcode = 0;
   end;
   if mcode==1;
      %
      % compute the mean and the non-central variance
      %
      if k>0;
         Amean = Amean+((Weights(i)/NumDraws)*A);
         Astd = Astd+((Weights(i)/NumDraws)*(A.*A));
      end;
      if length(size(H))==2;
         Hmean = Hmean+((Weights(i)/NumDraws)*H);
         Hstd = Hstd+((Weights(i)/NumDraws)*(H.*H));
      else;
         Hmean = Hmean+((Weights(i)/NumDraws)*H(:,:,size(H,3)));
         Hstd = Hstd+((Weights(i)/NumDraws)*(H(:,:,size(H,3)).*H(:,:,size(H,3))));
      end;
      Rmean = Rmean+((Weights(i)/NumDraws)*R);
      Rstd = Rstd+((Weights(i)/NumDraws)*(R.*R));
      Fmean = Fmean+((Weights(i)/NumDraws)*F);
      Fstd = Fstd+((Weights(i)/NumDraws)*(F.*F));
      B0mean = B0mean+((Weights(i)/NumDraws)*B0);
      B0std = B0std+((Weights(i)/NumDraws)*(B0.*B0));
   elseif (mcode~=1)&(status==1);
      %
      % compute the mean and the non-central variance
      %
      if k>0;
         Amean = Amean+((Weights(i)/NumDraws)*Amode);
         Astd = Astd+((Weights(i)/NumDraws)*(Amode.*Amode));
      end;
      if length(size(H))==2;
         Hmean = Hmean+((Weights(i)/NumDraws)*Hmode);
         Hstd = Hstd+((Weights(i)/NumDraws)*(Hmode.*Hmode));
      else;
         Hmean = Hmean+((Weights(i)/NumDraws)*Hmode(:,:,size(Hmode,3)));
         Hstd = Hstd+((Weights(i)/NumDraws)*(Hmode(:,:,size(Hmode,3)).*Hmode(:,:,size(Hmode,3))));
      end;
      Rmean = Rmean+((Weights(i)/NumDraws)*Rmode);
      Rstd = Rstd+((Weights(i)/NumDraws)*(Rmode.*Rmode));
      Fmean = Fmean+((Weights(i)/NumDraws)*Fmode);
      Fstd = Fstd+((Weights(i)/NumDraws)*(Fmode.*Fmode));
      B0mean = B0mean+((Weights(i)/NumDraws)*B0mode);
      B0std = B0std+((Weights(i)/NumDraws)*(B0mode.*B0mode));
   else;
      BadDraws = BadDraws+1;
   end;
   drawnow;
end;
%
% close the wait dialog
%
set(WaitControls.text,'UserData','done');
delete(WaitHandle);
drawnow;
pause(0.02);
%
if StopEstimation==1;
   return;
end;
%
% setup the output
%
if BadDraws>0;
   if k>0;
      Amean = (NumDraws/(NumDraws-BadDraws))*Amean;
      Astd = (NumDraws/(NumDraws-BadDraws))*Astd;
   end;
   Hmean = (NumDraws/(NumDraws-BadDraws))*Hmean;
   Hstd = (NumDraws/(NumDraws-BadDraws))*Hstd;
   Rmean = (NumDraws/(NumDraws-BadDraws))*Rmean;
   Rstd = (NumDraws/(NumDraws-BadDraws))*Rstd;
   Fmean = (NumDraws/(NumDraws-BadDraws))*Fmean;
   Fstd = (NumDraws/(NumDraws-BadDraws))*Fstd;
   B0mean = (NumDraws/(NumDraws-BadDraws))*B0mean;
   B0std = (NumDraws/(NumDraws-BadDraws))*B0std;
end;
A.Mean = Amean;
Astd = abs(Astd-(Amean.*Amean));
scale = max(max(abs(Amean)));
Astd(Astd/max(scale,1)<1e-6) = 0;
A.Std = sqrt(Astd);
H.Mean = Hmean;
Hstd = abs(Hstd-(Hmean.*Hmean));
scale = max(max(abs(Hmean)));
Hstd(Hstd/max(scale,1)<1e-6) = 0;
H.Std = sqrt(Hstd);
R.Mean = Rmean;
Rstd = abs(Rstd-(Rmean.*Rmean));
scale = max(max(abs(Rmean)));
Rstd(Rstd/max(scale,1)<1e-6) = 0;
R.Std = sqrt(Rstd);
F.Mean = Fmean;
Fstd = abs(Fstd-(Fmean.*Fmean));
scale = max(max(abs(Fmean)));
Fstd(Fstd/max(scale,1)<1e-6) = 0;
F.Std = sqrt(Fstd);
B0.Mean = B0mean;
B0std = abs(B0std-(B0mean.*B0mean));
scale = max(max(abs(B0mean)));
B0std(B0std/max(scale,1)<1e-6) = 0;
B0.Std = sqrt(B0std);
%
% save to file
%
if IsPosterior==1;
   MatFile = [DSGEModel.OutputDirectory '\statespace\DSGE-StateSpace-PosteriorMoments-' DSGEModel.NameOfModel '-' int2str(CurrChain) '-' int2str(NumDraws) '.' int2str(TotalDraws) '.mat'];
else;
   MatFile = [DSGEModel.OutputDirectory '\statespace\DSGE-StateSpace-PriorMoments-' DSGEModel.NameOfModel '-' int2str(NumDraws) '.mat'];
end;
save(MatFile,'A','H','R','F','B0','BadDraws');

%
% end of DSGEStateSpaceMoments.m
%
