function [PValues,OmegaSim,OmegaObs] = OneStepAheadPredictiveChecks(DSGEModel,SelectedParameters,NumDraws,TotalDraws,CurrChain,IsPosterior,Weights,lambda)
% OneStepAheadPredictiveChecks: Computes p-values for the one-step-ahead forecast error covariances being greater than the
%                               observed values, where the influence of parameters is integrated out.
%
% USAGE:
%
%       [PValues,OmegaSim,OmegaObs] = OneStepAheadPredictiveChecks(DSGEModel,SelectedParameters,NumDraws,TotalDraws, ...
%                                                                  CurrChain,IsPosterior,Weights,lambda)
%
% REQUIRED INPUT:  DSGEModel (structure) with fields containing all the information about a DSGE Model for YADA.
%
%                  SelectedParameters (vector) of dimension NumParam which indicates which parameters can vary from
%                                     one draw to the next (1) and which parameters are fixed at posterior mode (0).
%
%                  NumDraws (integer) with the number of effective draws from the posterior distribution of the
%                                     parameters.
%
%                  TotalDraws (integer) with the number of draws from the posterior distribution of the parameters
%                                     that could have been used.
%
%                  CurrChain (integer) with the number for the current Markov chain.
%
%                  IsPosterior (boolean) that is 1 if draws from the posterior distribution are used and 0
%                                     if draws from the prior are supplied.
%
%                  Weights (vector) of dimension NumDraws x 1 with the importance weights from the SMC with
%                                     likelihood tempering posterior sampler.
%
%                  lambda (scalar) with the value of the lambda hyperparameter for the DSGE-VAR.
%
% REQUIRED OUPPUT: PValues (vector) of dimension n*(n+1)/2 with the probabilities that the one-step-ahead covariances
%                                    are greater than the observed.
%
%                  OmegaSim (matrix) of dimension NumDraws*(n(n+1)/2) with the simulated values of the one-step-ahead
%                                    covariances for the NumDraws parameter draws.
%
%                  OmegaObs (matrix) of dimension NumDraws*(n(n+1)/2) with the observed values of the one-step-ahead
%                                    covariances for the NumDraws parameter draws.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   DG-R/MPR
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2017 European Central Bank.
%
%                       First version: August 15, 2013.
%                        This version: August 25, 2017.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 16-08-2013: Updated the documentation.
%
% * 19-08-2013: Updated the documentation.
%
% * 20-08-2013: Updated the documentation.
%
% * 28-01-2014: Updated the documentation.
%
% * 14-01-2015: Updated the documentation.
%
% * 23-02-2016: Updated the documentation.
%
% * 29-03-2017: Updated the documentation.
%
% * 25-08-2017: Added the input variable "Weights".
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize
%
if isempty(Weights)==1;
   Weights = ones(NumDraws,1);
end;
n = DSGEModel.n;
p = DSGEModel.BVARLags;
nn1 = n*(n+1)/2;
PValues = [];
OmegaSim = zeros(NumDraws,nn1);
OmegaObs = zeros(NumDraws,nn1);
%
% fix the information needed to setup the file names
%
SelParamStr = strrep(num2str(SelectedParameters),' ','');
NumFiles = ceil(NumDraws/min(NumDraws,1000));
LastValue = 0;
for ThisSave=1:NumFiles;
   if isempty(lambda)==1;
      if IsPosterior==1;
         file = FixFilePath([DSGEModel.OutputDirectory '\predcheck\DSGE-OSA-' DSGEModel.NameOfModel '-' num2str(CurrChain,'%0.0f') '-' SelParamStr '-' num2str(ThisSave,'%0.0f') '-' num2str(NumFiles,'%0.0f') '.' num2str(NumDraws,'%0.0f') '-' num2str(TotalDraws,'%0.0f') '.mat']);
      else;
         file = FixFilePath([DSGEModel.OutputDirectory '\predcheck\DSGE-OSA-Prior-' DSGEModel.NameOfModel '-' SelParamStr '-' num2str(ThisSave,'%0.0f') '-' num2str(NumFiles,'%0.0f') '.' num2str(NumDraws,'%0.0f') '.mat']);
      end;
   else;
      if IsPosterior==1;
         file = FixFilePath([DSGEModel.OutputDirectory '\predcheck\DSGE-VAR-OSA-' num2str(lambda,'%0.4f') '-' int2str(p) '-' DSGEModel.NameOfModel '-' num2str(CurrChain,'%0.0f') '-' num2str(ThisSave,'%0.0f') '-' num2str(NumFiles,'%0.0f') '.' num2str(NumDraws,'%0.0f') '-' num2str(TotalDraws,'%0.0f') '.mat']);
      else;
         file = FixFilePath([DSGEModel.OutputDirectory '\predcheck\DSGE-VAR-OSA-Prior-' num2str(lambda,'%0.4f') '-' int2str(p) '-' DSGEModel.NameOfModel '-' num2str(ThisSave,'%0.0f') '-' num2str(NumFiles,'%0.0f') '.' num2str(NumDraws,'%0.0f') '.mat']);
      end;
   end;
   PCData = load(file);
   Omega = PCData.PredCheck.Omega;
   NDraws = length(Omega);
   clear('PCData');
   for i=1:NDraws;
      OmegaSim(i+LastValue,:) = Omega(i).Sim;
      OmegaObs(i+LastValue,:) = Omega(i).Obs;
   end;
   LastValue = LastValue+NDraws;
end;
PValues = (1/NumDraws)*sum((OmegaSim>OmegaObs).*(Weights*ones(1,nn1)),1);

%
% end of OneStepAheadPredictiveChecks.m
%
