function [status,ZLBYPaths,ZLBKsiPaths,KsitT,CenterShocks,Data,file] = ZLBDSGEPredictionPathsTheta(theta,thetaPositions,ModelParameters,DSGEModel,CurrINI,Shocks,MeasError,HistState,ExtraDraws,etaMean,KsiTMean,IsConditional,A,H,R,F,B0,X,NameStr)
% ZLBDSGEPredictionPathsTheta: Calculates prediction paths for a DSGE model when the monetary policy rate is
%                              subject to a zero lower bound.
%
% USAGE:
%
%       [status,ZLBYPaths,ZLBKsiPaths,KsitT,CenterShocks,Data,file] = ZLBDSGEPredictionPathsTheta(theta,thetaPositions, ...
%                                                                        ModelParameters,DSGEModel,CurrINI,Shocks,MeasError, ...
%                                                                        HistState,ExtraDraws,etaMean,KsiTMean,IsConditional, ...
%                                                                        A,H,R,F,B0,X,NameStr)
%
% REQUIRED INPUT:  theta (vector) with values of the original DSGE model parameters.
%
%                  thetaPositions (structure) with fields parameter and distribution. This function
%                                 needs the former field to know which parameter name corresponds to
%                                 position i in theta.
%
%                  ModelParameters (structure) whose fields have names given by the parameter names in the
%                                 DSGE model.
%
%                  DSGEModel (structure) with fields containing all the information about a DSGE Model for YADA.
%
%                  CurrINI (structure), with initialization information.
%
%                  Shocks (matrix) of dimension qxhxNumPaths with draws of structural shocks.
%
%                  MeasError (matrix) of dimension nxhxNumPaths with draws of the measurement errors.
%
%                  HistState (matrix) of dimension rxNumPaths with draws of the state variables for the last
%                                 period of historical data.
%
%                  ExtraDraws (structure) with fields "Shocks", "MeasError" and "HistState" containing NumExtra
%                                 additional draws of the shocks, states and measurement errors in the event
%                                 that an original draw fails the existence test for the ZLB calculation.
%
%                  etaMean (matrix) of dimension qxh with the population mean of the structural shocks over
%                                 the forecast horizon.
%
%                  KsiTMean (vector) of dimension rx1 with the population mean of the state variables in the
%                                 last historical time period.
%
%                  IsConditional (boolean) that takes the value 1 if the predictions are conditional on a
%                                 particular path for the observables and 0 otherwise.
%
%                  A (kxn) matrix with coefficients on the exogenous variables in the measurement equation.
%
%                  H (rxnxT) matrix with cofficients on the state variables in the measurement equation.
%
%                  R (nxn) matrix with the covariances of the measurement errors.
%
%                  F (rxr) transition matrix for the state variables.
%
%                  B0 (rxq) matrix with coefficients on the economic shocks in the state equation.
%
%                  X (matrix) of dimension kxh with the values of the deterministic variables over the
%                                 prediction sample.
%
%                  NameStr (string vector) that is sent to the progress dialog name after the string
%                                 "Zero Lower Bound".
%
% REQUIRED OUTPUT: status (boolean) that takes the value 1 if the calculations were successful and 0 otherwise.
%
%                  ZLBYPaths (matrix) of dimension nxhxNumPaths with zero lower bound consistent prediction
%                                 paths for the observed variables.
%
%                  ZLBKsiPaths (matrix) of dimension rxhxNumPaths with zero lower bound consistent prediction
%                                 paths for the state variables.
%
%                  KsitT (matrix) of dimension rxTh with the smooth estimates of the state variables taking
%                                 data from the prediction sample into account.
%
%                  CenterShocks (boolean) that takes the value 1 if shocks and states are have recentered and
%                                 0 otherwise.
%
%                  Data (structure) with fields "Shocks", "UsedShocks", "HistState", "UsedHistState", "MeasError",
%                                 and "UsedMeasError".
%
%                  file (string) with the name of the output file.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   DG-R/MPR
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2017 European Central Bank.
%
%                       First version: August 19, 2014.
%                        This version: March 29, 2017.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 20-08-2014: Continued work on the function. Finished after completing the code for running
%               the Kalman smoother using actual data from the forecast sample.
%
% * 21-08-2014: Continued work on the function.
%
% * 22-08-2014: Continued working on the function after the draws of shocks, states and measurement
%               errors had been recentered.
%
% * 25-08-2014: Continued working on the function.
%
% * 26-08-2014: Took into account that the function "ZLBKleinSolver" takes the input variable "REqPosition".
%
% * 27-08-2014: Took into account the changes for the output of the ZLBKleinSolver function.
%
% * 01-09-2014: Checking code only for the ZLBKleinSolver function.
%
% * 02-09-2014: Continued work on the function from the ZLBKleinSolver call.
%
% * 03-09-2014: Continued work on the function with the inputs for the forward-back shooting algorithm.
%
% * 04-09-2014: Continued work on the stochastic simulations part of the function. Added the input variables
%               "etaMean" and "KsiTMean" with the population mean shock values and the population mean
%               state value in the last historical period.
%
% * 05-09-2014: Added the output variable "CenterShocks" and saving the mean original shocks to file.
%
% * 10-09-2014: Took into account that the name of "file" now depends on the method for controlling the shocks
%               when the shocks are based on conditional forecasting. The output variable "KsitT" was
%               also added.
%
% * 11-09-2014: Took into account that the name of "file" depends on the selected sample, and that "RhatPosition"
%               has been changed to "RtildePosition".
%
% * 16-09-2014: Took into account that the output "AlgorithmInformation" now has 5 elements. Added the output variable
%               "Data" which contains all the stochastic information about shocks, initial state, and mreasurement
%               errors in terms of suggested ones and used ones.
%
% * 23-10-2014: Fixed a bug where X can have fewer data points than hmax, since h can be greater than DSGEModel.ZLBSampleT+1.
%
% * 14-01-2015: Updated the documentation.
%
% * 25-02-2015: Made some changes based on Riksbank (Ingvar Strid, thanks) wishes.
%
% * 09-06-2015: The code takes the CurrINI.GraphicsRoot field into account.
%
% * 02-09-2015: Fixed the information displayed when the kalmanstatus is -1.
%
% * 22-02-2016: Updated the documentation.
%
% * 29-03-2017: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize
%
status = 0;
ZLBYPaths = [];
ZLBKsiPaths = [];
file = '';
CenterShocks = 0;
Data = [];
[q,h,NumPaths] = size(Shocks);
r = size(HistState,1);
n = DSGEModel.n;
KsitT = [];
%
% create the output directory
%
stat = MakeDir(DSGEModel.OutputDirectory,'predictions');
if stat~=1;
   if ispc==1;
      txt = ['YADA was for some reason unable to create the directory "' DSGEModel.OutputDirectory '\predictions". The computation of predictive distributions has therefore been aborted.'];
   else;
      txt = ['YADA was for some reason unable to create the directory "' DSGEModel.OutputDirectory '/predictions". The computation of predictive distributions has therefore been aborted.'];
   end;
   About(txt,'information','YADA - Directory Creation Problem',180,500,CurrINI);
   return;
end;
%
% Make sure that the ModelParameter structure takes the correct values.
% This is not necessary to do when initial values are given to the function.
% In that case, more substantial checking is required!
%
ModelParameters = ThetaToModelParameters(ModelParameters,theta,thetaPositions);
%
% copy files to the tmp folder
%
[stat,msg] = CopyFile([GetPath(DSGEModel.AIMDataFile) 'compute_aim_matrices.m'],[pwd '\tmp']);
[stat,msg] = CopyFile(DSGEModel.MeasurementEquationFile,[pwd '\tmp']);
if FileExist(DSGEModel.UpdateParameterFile)==1;
   [stat,msg] = CopyFile(DSGEModel.UpdateParameterFile,[pwd '\tmp']);
end;
%
% Update any parameteters that need updating
%
if FileExist(DSGEModel.UpdateParameterFile)==1;
   eval(['ModelParameters = ' GetName(DSGEModel.UpdateParameterFile) '(ModelParameters);']);
end;
%
% load the AIMData file
%
ErrorStr = '';
try;
   AIMData = load(DSGEModel.AIMDataFile);
catch;
   ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' lasterr];
end;
if isempty(ErrorStr)==0;
   if nargout>2;
      mcode = 0;
   end;
   About(ErrorStr,'information','YADA - Bad MAT File',120,500,CurrINI);
   return;
end;
%
% Get the sub-sample
%
[DSGEModel.FirstPeriod,DSGEModel.LastPeriod] = CreateSubSample(DSGEModel);
%
% try to solve the original model
%
if isempty(H)==1;
   try;
      if DSGEModel.ModelSolver==1;
         [mcode,MaxAbsError,BMatrix,SMatrix] = AiMSolver(ModelParameters,AIMData.neq,AIMData.nlag,AIMData.nlead,DSGEModel.AIMTolerance);
      elseif DSGEModel.ModelSolver==2;
         [F,B0,mcode] = KleinSolver(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,CurrINI.OrderQZ);
      elseif DSGEModel.ModelSolver==3;
         [F,B0,mcode] = SimsSolver(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,CurrINI.OrderQZ);
      end;
   catch;
      if DSGEModel.ModelSolver==1;
         ErrorStr = ['There was an error running AiM. YADA received the following message: ' lasterr];
      elseif DSGEModel.ModelSolver==2;
         ErrorStr = ['There was an error running the Klein solver. YADA received the following message: ' lasterr];
      elseif DSGEModel.ModelSolver==3;
         ErrorStr = ['There was an error running Gensys. YADA received the following message: ' lasterr];
      end;
      About(ErrorStr,'error','YADA Error',CurrINI.scrsz(4)-50,500,CurrINI);
      drawnow;
      return;
   end;
   if mcode~=1;
      if mcode==2;
         mcodeStr = 'Roots are not correctly computed by real_schur.'
      elseif mcode==3;
         mcodeStr = 'Too many big roots.';
      elseif mcode==35;
         mcodeStr = 'Too many big roots, and q(:,right) is singular.';
      elseif mcode==4;
         mcodeStr = 'Too few big roots.';
      elseif mcode==45;
         mcodeStr = 'Too few big roots, and q(:,right) is singular.';
      elseif mcode==5;
         mcodeStr = 'q(:,right) is singular.';
      elseif mcode==61;
         mcodeStr = 'Too many exact shiftrights.';
      elseif mcode==62;
         mcodeStr = 'Too many numeric shiftrights.';
      elseif mcode==7;
         mcodeStr = 'Infinite or NaN values detected.';
      elseif mcode==8;
         mcodeStr = 'The function "compute_aim_matrices" returns complex numbers.';
      elseif mcode==-1;
         mcodeStr = 'No stable solution.';
      elseif mcode==-2;
         mcodeStr = 'Too many large generalized eigenvalues.';
      elseif mcode==-3;
         mcodeStr = 'Too few large generalized eigenvalues.';
      else;
         mcodeStr = 'Return code not properly specified.';
      end;
      if DSGEModel.ModelSolver==1;
         SolverStr = 'AiM';
      elseif DSGEModel.ModelSolver==2;
         SolverStr = 'Klein';
      elseif DSGEModel.ModelSolver==3;
         SolverStr = 'Gensys';
      end;
      txt = ['The ' SolverStr ' solver provided the return code: ' num2str(mcode) ', i.e., "' mcodeStr '"'];
      About(txt,'information',['YADA - ' SolverStr ' Solver Error'],200,500,CurrINI);
      return;
   end;
   %
   % Convert to the state equation matrices
   %
   if DSGEModel.ModelSolver==1;
      [F,B0] = AiMtoStateSpace(BMatrix,SMatrix,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions);
   end;
   [S,KeepShocks] = RemoveRedundantColumns(B0);
   if isempty(S)==1;
      txt = 'The model doesn''t have any structural shocks. YADA has aborted from the lower bound computations.';
      About(txt,'information',['Zero Lower Bound - ' NameStr],200,500,CurrINI);
      return;
   end;
   B0 = S;
   %
   % obtain the measurement equation matrices
   %
   try;
      eval(['[A,H,R] = ' GetName(DSGEModel.MeasurementEquationFile) '(ModelParameters,AIMData.endog(DSGEModel.StateVariablePositions,:),DSGEModel.VariableNames,DSGEModel.XVariableNames,n,r,DSGEModel.k);']);
   catch;
      %
      % display an error message
      %
      ErrorStr = ['Error found in "' GetName(DSGEModel.MeasurementEquationFile) '.m". Message caught is: ' lasterr];
      About(ErrorStr,'error','Error - Measurement Equation',150,500,CurrINI);
      return;
   end;
   %
   if length(size(H))==3;
      if size(H,3)<DSGEModel.LastPeriod+h;
         %
         % too few data points in the 3D H matrix.
         %
         ErrorStr = ['The number of time periods for the time-varying measurement matrix H (' int2str(size(H,3)) ') is lower than the number of observations plus forecasts (T+h = ' int2str(DSGEModel.LastPeriod+h) '). YADA has therefore aborted from the conditional predictions.'];
         About(ErrorStr,'error','Error - Measurement Equation',200,500,CurrINI);
         return;
      end;
   end;
else;
   [B0,KeepShocks] = RemoveRedundantColumns(B0);
end;
%
% test if the measurement equation for the policy rate maps to the correct state variables only
%
if length(size(H))==2;
   ZLBLink = H(DSGEModel.RtildePosition,DSGEModel.ZLBPosition);
   NonZLBLink = H(setdiff((1:r),DSGEModel.RtildePosition),DSGEModel.ZLBPosition);
   if r>1;
      TestValue = isequal(NonZLBLink,zeros(r-1,1));
   else;
      TestValue = isempty(NonZLBLink);
   end;
   txt = '';
   BadValue = 0;
   if ZLBLink==0;
      txt = ['The observed monetary policy interest rate (' StringTrim(DSGEModel.VariableNames(DSGEModel.ZLBPosition,:)) ') doesn''t map to the claimed unrestricted monetary policy rate (' StringTrim(DSGEModel.StateVariableNames(DSGEModel.RtildePosition,:)) ') in the DSGE model. YADA is therefore unable to use the lower bound and the steady-state interest rate and will abort.'];
      BadValue = 1;
   elseif TestValue==0;
      txt = ['The measurement equation for ' StringTrim(DSGEModel.VariableNames(DSGEModel.ZLBPosition,:)) ' maps to other state variables than the claimed unrestricted monetary policy rate (' StringTrim(DSGEModel.StateVariableNames(DSGEModel.RtildePosition,:)) ') in the DSGE model. Are you sure you want to use this mapping?'];
   end;
   if isempty(txt)==0;
      if BadValue==1;
         About(txt,'information',['Zero Lower Bound - ' NameStr],300,600,CurrINI);
         return;
      else;
         answer = Query(txt,'question',300,['Zero Lower Bound - ' NameStr],600,'no',CurrINI);
         if strcmp(lower(answer),'no')==1;
            return;
         end;
      end;
   end;
else;
   Hlength = size(H,3);
   ZLBLink = shiftdim(H(DSGEModel.RtildePosition,DSGEModel.ZLBPosition,DSGEModel.LastPeriod:Hlength),2);
   ZLBLTest = min(ZLBLink~=zeros(Hlength-DSGEModel.LastPeriod+1,1));
   NonZLBLink = shiftdim(H(setdiff((1:r),DSGEModel.RtildePosition),DSGEModel.ZLBPosition,DSGEModel.LastPeriod:Hlength),2);
   if r>1;
      TestValue = isequal(NonZLBLink,zeros(Hlength-DSGEModel.LastPeriod+1,r-1))
   else;
      TestValue = isempty(NonZLBLink);
   end;
   txt = '';
   BadValue = 0;
   if ZLBLink==0;
      txt = ['The observed monetary policy interest rate (' StringTrim(DSGEModel.VariableNames(DSGEModel.ZLBPosition,:)) ') doesn''t map to the claimed unrestricted monetary policy rate (' StringTrim(DSGEModel.StateVariableNames(DSGEModel.RtildePosition,:)) ') in the DSGE model. YADA is therefore unable to use the lower bound and the steady-state interest rate and will abort.'];
      BadValue = 1;
   elseif TestValue==0;
      txt = ['The measurement equation for ' StringTrim(DSGEModel.VariableNames(DSGEModel.ZLBPosition,:)) ' maps to other state variables than the claimed unrestricted monetary policy rate (' StringTrim(DSGEModel.StateVariableNames(DSGEModel.RtildePosition,:)) ') in the DSGE model for some time periods. Are you sure you want to use this mapping?'];
   end;
   if isempty(txt)==0;
      if BadValue==1;
         About(txt,'information',['Zero Lower Bound - ' NameStr],300,600,CurrIN);
         return;
      else;
         answer = Query(txt,'question',300,['Zero Lower Bound - ' NameStr],600,'no',CurrINI);
         if strcmp(lower(answer),'no')==1;
            return;
         end;
      end;
   end;
end;
%
% determine the steady-state interest rate path.
%
if DSGEModel.k>0;
   Ax = A'*X;
   if size(DSGEModel.X,2)>DSGEModel.LastPeriod;
      X = DSGEModel.X(:,DSGEModel.LastPeriod+1:size(DSGEModel.X,2));
      LastValue = X(:,size(X,2));
   else;
      X = [];
      LastValue = DSGEModel.X(:,DSGEModel.LastPeriod);
   end;
   if isempty(DSGEModel.PredictedX)==0;
      X = [X DSGEModel.PredictedX];
   end;
   Tx = size(X,2);
   Xold = X;
   if Tx>DSGEModel.ZLBSampleT+1;
      X = X(:,1:DSGEModel.ZLBSampleT+1);
   elseif Tx<DSGEModel.ZLBSampleT+1;
      X = [X LastValue*ones(1,DSGEModel.ZLBSampleT+1-Tx)];
   end;
   Rbar = A(:,DSGEModel.ZLBPosition)'*X;
else;
   Ax = zeros(n,h);
   Rbar = zeros(1,DSGEModel.ZLBSampleT+1);
end;
%
% setup the zero lower bound path
%
Tzlb = length(DSGEModel.ZLBData);
if Tzlb==1;
   Rzlb = DSGEModel.ZLBData*ones(1,DSGEModel.ZLBSampleT+1);
else;
   if Tzlb>DSGEModel.LastPeriod;
      Rzlb = DSGEModel.ZLBData(1,DSGEModel.LastPeriod:Tzlb);
      Tr = length(Rzlb);
      if Tr>DSGEModel.ZLBSampleT+1;
         Rzlb = Rzlb(1,1:DSGEModel.ZLBSampleT+1);
      elseif Tr<DSGEModel.ZLBSampleT;
         Rzlb = [Rzlb Rzlb(1,Tr)*ones(1,DSGEModel.ZLBSampleT+1-Tr)];
      end;
   else;
      Rzlb = DSGEModel.ZLBData(1,Tzlb)*ones(1,DSGEModel.ZLBSampleT+1);
   end;
end;
%
% Transform the steady-state interest rate and the lower bound to the same units
% as the unrestricted monetary policy rate
%
if length(size(H))==2;
   Hvalue = H(DSGEModel.RtildePosition,DSGEModel.ZLBPosition);
   Rzlb = (1/Hvalue)*Rzlb;
   Rbar = (1/Hvalue)*Rbar;
else;
   Hvalue = shiftdim(H(DSGEModel.RtildePosition,DSGEModel.ZLBPosition,DSGEModel.LastPeriod:Hlength),2)';
   Th = length(Hvalue);
   if Th>DSGEModel.ZLBSampleT;
      Hvalue = Hvalue(1,1:DSGEModel.ZLBSampleT);
   elseif Th<DSGEModel.ZLBSampleT;
      Hvalue = [Hvalue Hvalue(1,Th)*ones(1,DSGEModel.ZLBSampleT-Th)];
   end;
   Rzlb = Rzlb./Hvalue;
   Rbar = Rbar./Hvalue;
end;
%
% set the InitialStateVector value
%
if DSGEModel.UseOwnInitialState==1;
   if length(DSGEModel.InitialStateValues)==size(AIMData.endog,1);
      DSGEModel.InitialStateVector = DSGEModel.InitialStateValues(DSGEModel.StateVariablePositions);
   else;
      DSGEModel.InitialStateVector = zeros(length(DSGEModel.StateVariablePositions),1);
   end;
else;
   DSGEModel.InitialStateVector = zeros(length(DSGEModel.StateVariablePositions),1);
end;
%
% We can now run the Kalman filter
%
try;
   [logLikeValue,stat,lnLt,Yerror,Yhat,SigmaSqRoot,MSEY,InvMSEY,KalmanGain,Ksitt1,Ptt1] = RunKalmanFilter(DSGEModel.Y,DSGEModel.X,DSGEModel.FirstPeriod,min(DSGEModel.LastPeriod+h,DSGEModel.T),A,H,F,B0,R,DSGEModel.InitialStateVector,DSGEModel.UseDoublingAlgorithm,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,1,DSGEModel.AllowUnitRoot,DSGEModel.StateCovConstValue,GetPosition(DSGEModel.StateVariablePositions,DSGEModel.UnitRootStates),DSGEModel.YNaNs,DSGEModel.KalmanAlgorithm);
catch;
   txt = ['YADA caught the following error message from the Kalman filter routine: ' lasterr];
   About(txt,'error','Error - Kalman Filter',200,500,CurrINI);
   return;
end;
if stat~=0;
   if stat==1;
      txt = 'The state variables are not stationary at the initial values. Issue recorded while running the Kalman filter on the linear DSGE model.';
   else;
      txt = 'The covariance matrix of the forecast of the observed variables in period t conditional on data until t-1 is not positive definite for some time period t. This implies that the log-likelihood value is a NaN (or a complex number) in Matlab. Issue recorded while running the Kalman filter on the linear DSGE model.';
   end;
   About(txt,'information',['Zero Lower Bound - ' NameStr],200,500,CurrINI);
   return;
end;
%
% check if there are numerical issues with diffuse initialization of the Kalman filter
%
if DSGEModel.UseDoublingAlgorithm==4;
   txt = '';
   if KalmanGain.d==KalmanGain.T;
      txt = 'The diffuse initialization of the Kalman filter didn''t converge within the selected sample. This may be due to non-stationarity or near non-stationarity of some state variable which isn''t observed. YADA therefore recommends that you choose a different initialization method.';
   elseif (KalmanGain.d<KalmanGain.T)&(max(diff(sum(SigmaSqRoot,1)))>0);
      txt = ['Diffuse initialization of the Kalman filter converged within the selected sample in period ' int2str(KalmanGain.d) ' relative to period ' int2str(KalmanGain.T) ', but the rank of the Sigma_Inf matrix has increased over the initialization sample. This indicates a numerical issue with the procedure and it may be due to non-stationarity or near non-stationarity of a state variable which is not observed. YADA therefore recommends that you choose a different initialization method.'];
   end;
   if isempty(txt)==0;
      answer = Query(strvcat(txt,' ','Are you sure you want to continue?'),'question',300,['Zero Lower Bound - ' NameStr],500,'no',CurrINI);
      if strcmp(lower(answer),'no')==1;
         return;
      end;
   end;
end;
%
% run the Kalman smoother for g periods
%
try;
   [Ksitt,Ptt,KsitT,PtT,Ksit1t] = RunKalmanSmoother(DSGEModel.Y,DSGEModel.FirstPeriod,min(DSGEModel.LastPeriod+h,DSGEModel.T),Yhat,Ksitt1,Ptt1,H,F,R,B0,Yerror,SigmaSqRoot,InvMSEY,KalmanGain,DSGEModel.YNaNs,DSGEModel.KalmanAlgorithm,DSGEModel.UseDoublingAlgorithm==4);
catch;
   txt = ['YADA caught the following error message from the Kalman smoother: ' lasterr];
   About(txt,'error','Error - Kalman Smoother',200,500,CurrINI);
   return;
end;
%
% ask if the user wants to center the original forecasts from the linear model on the actual data
%
if DSGEModel.T>DSGEModel.LastPeriod;
   txt = 'Would you like to recenter the structural shocks and initial states such that the predictive distribution of the DSGE model without anticipated shocks is centered on the observed data?';
   answer = Query(txt,'question',300,['Zero Lower Bound - ' NameStr],600,'no',CurrINI);
   if strcmp(lower(answer),'yes')==1;
      CenterShocks = 1;
      %
      Txi = DSGEModel.LastPeriod-DSGEModel.FirstPeriod+1;
      hmax = min(DSGEModel.LastPeriod+h,DSGEModel.T)-DSGEModel.LastPeriod;
      Txold = size(Xold,2);
      if Txold<hmax;
         Xold = [Xold Xold(:,Txold)*ones(1,hmax-Txold)];
      end;
      %
      % the state variables in the last historical period
      %
      KsiTTh = KsitT(:,Txi);
      %
      % now we can calculate the shocks
      %
      wTTh = zeros(n,h);
      etaTTh = zeros(q,h);
      etaTTh(KeepShocks,1:hmax) = ((B0'*B0)\B0')*(KsitT(:,Txi+1:Txi+hmax)-(F*KsitT(:,Txi:Txi+hmax-1)));
      %
      % and the measurement errors if need be
      %
      if rank(R)>0;
         Yh = DSGEModel.Y(:,DSGEModel.LastPeriod+1:DSGEModel.LastPeriod+hmax);
         if DSGEModel.k>0;
            Ahx = A'*Xold(:,1:hmax);
         else;
            Ahx = zeros(n,hmax);
         end;
         if length(size(H))==2;
            wTTh(:,1:hmax) = Yh-Ahx-(H'*KsitT(:,Txi+1:Txi+hmax));
         else;
            for i=1:hmax;
               wTTh(:,i) = Yh(:,i)-Ahx(:,i)-(H(:,:,DSGEModel.LastPeriod+i)'*KsitT(:,Txi+1))
            end;
         end;
      end;
      %
      % compute the mean shocks, state variables, and measurement errors from the draws and the additional draws
      %
      ShockMean = (1/NumPaths)*sum(Shocks,3);
      MeasErrorMean = (1/NumPaths)*sum(MeasError,3);
      HistStateMean = (1/NumPaths)*sum(HistState,2);
      %
      % and the additional draws
      %
      if isempty(ExtraDraws)==0;
         NumExtraPaths = size(ExtraDraws.Shocks,3);
         ExtraDraws.ShockMean = (1/NumExtraPaths)*sum(ExtraDraws.Shocks,3);
         ExtraDraws.MeasErrorMean = (1/NumExtraPaths)*sum(ExtraDraws.MeasError,3);
         ExtraDraws.HistStateMean = (1/NumExtraPaths)*sum(ExtraDraws.HistState,2);
      end;
      %
      % re-center the shocks, state variables, and the measurement errors
      %
      HistState = HistState-((HistStateMean-KsiTTh)*ones(1,NumPaths));
      for i=1:NumPaths;
         Shocks(:,:,i) = Shocks(:,:,i)-ShockMean+etaTTh;
         if rank(R)>0;
            MeasError(:,:,i) = MeasError(:,:,i)-MeasErrorMean+wTTh;
         end;
      end;
      if isempty(ExtraDraws)==0;
         ExtraDraws.HistState = ExtraDraws.HistState-((ExtraDraws.HistStateMean-KsiTTh)*ones(1,NumExtraPaths));
         for i=1:NumExtraPaths;
            ExtraDraws.Shocks(:,:,i) = ExtraDraws.Shocks(:,:,i)-ExtraDraws.ShockMean+etaTTh;
            if rank(R)>0;
               ExtraDraws.MeasError(:,:,i) = ExtraDraws.MeasError(:,:,i)-ExtraDraws.MeasErrorMean+wTTh;
            end;
         end;
      end;
   end;
end;
%
% Try to solve the model subject to the anticipated shocks
%
[G,C1,P,mcode] = ZLBKleinSolver(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.ZLBSampleT,DSGEModel.RtildePosition,DSGEModel.REqPosition,DSGEModel.AIMTolerance,CurrINI.OrderQZ);
%
% notice that the row r+1 of P corresponds to the unrestricted interest rate
% and row DSGEModel.RtildePosition to the actual policy interest rate
%
if mcode~=1;
   if mcode==8;
      mcodeStr = 'The function "compute_aim_matrices" returns complex numbers.';
   elseif mcode==-1;
      mcodeStr = 'No stable solution.';
   elseif mcode==-2;
      mcodeStr = 'Too many large generalized eigenvalues.';
   elseif mcode==-3;
      mcodeStr = 'Too few large generalized eigenvalues.';
   else;
      mcodeStr = 'Return code not properly specified.';
   end;
   txt = ['The Klein solver with anticipated shocks for handling the zero lower bound provided the return code: ' num2str(mcode) ', i.e., "' mcodeStr '"'];
   About(txt,'information',['YADA - Klein Solver Error'],200,500,CurrINI);
   return;
end;
%
% setup the parameters that determine the projected restricted policy rate
%
e_pr = zeros(r+1,1);
e_pr(DSGEModel.RtildePosition) = 1;
%
% setup ePG for conmputing the restricted policy rate projections
%
ePG = zeros(DSGEModel.ZLBSampleT+1,size(G,2));
ePG(1,:) = e_pr'*P(1:r+1,:);
for tau=1:DSGEModel.ZLBSampleT;
   ePG(tau+1,:) = ePG(tau,:)*G;
end;
if DSGEModel.ZLBSampleT>1;
   M = [zeros(DSGEModel.ZLBSampleT,1) eye(DSGEModel.ZLBSampleT);zeros(1,DSGEModel.ZLBSampleT+1)];
else;
   M = 0;
end;
%
% compute the mean predictions
%
ZLBKsiMean = zeros(r,h);
ZLBYMean = zeros(n,h);
%
At = zeros(DSGEModel.ZLBSampleT+1,1);
Xt = [etaMean(:,1);KsiTMean;KsiTMean(DSGEModel.RtildePosition)];
for t=1:h;
   [At,status,AlgorithmInformation] = ForwardBackShootingAlgorithm(Xt,At,ePG,Rzlb,Rbar,DSGEModel.AIMTolerance);
   if status==1;
      Ksit = P(1:r,:)*[Xt;At];
      rhat = P(r+1,:)*[Xt;At];
      ZLBKsiMean(:,t) = Ksit;
      if length(size(H))==2;
         ZLBYMean(:,t) = Ax(:,t)+(H'*Ksit);
      else;
         ZLBYMean(:,t) = Ax(:,t)+(H(:,:,DSGEModel.LastPeriod+t)'*Ksit);
      end;
      if t+1<=h;
         At = M*At;
         Xt = [etaMean(:,t+1);Ksit;rhat];
      end;
   else;
      break;
   end;
end;
%
% check if the mean shocks and state failed to find the anticipated shocks.
%
if status==0;
   if AlgorithmInformation(1)==1;
      AddTxt = 'YADA has recorded an invertibility problem when attempting to solve for the anticipated shocks.';
   elseif AlgorithmInformation(2)==1;
      AddTxt = 'YADA has recored that a solution cannot be found, i.e., Step 3 (ii) holds (see the YADA Manual).';
   elseif AlgorithmInformation(3)==1;
      AddTxt = 'YADA has recorded that the anticipated shocks are nonnegative, that the restricted policy rate satisfies the zero lower bound, but the complementary slackness condition is violated.';
   else;
      AddTxt = ['YADA has recored that the maximum number of attempts (' int2str(AlgorithmInformation(5)) ') to find the anticipated shocks was reached.'];
   end;
   txt = ['The forward-back shooting algorithm failed to solve for the anticipated shocks for period ' int2str(t) ' using the population means of the structural shocks and the state variables in the last historical period. ' AddTxt ' Are you sure you want to continue?'];
   answer = Query(txt,'question',300,['Zero Lower Bound - ' NameStr],600,'no',CurrINI);
   if strcmp(lower(answer),'no')==1;
      return;
   end;
end;
%
% check if we should setup a wait handle
%
if DSGEModel.ShowProgress==1;
   ProgressStructure.title = ['Progress for ' int2str(NumPaths) ' Prediction Paths'];
   ProgressStructure.facecolor = CurrINI.progress_facecolor;
   ProgressStructure.startfacecolor = CurrINI.progress_startfacecolor;
   ProgressStructure.edgecolor = CurrINI.progress_edgecolor;
   ProgressStructure.bgcolor = CurrINI.progress_bgcolor;
   ProgressStructure.stop = 0;
   ProgressStructure.clock = DSGEModel.ShowProgressClock;
   ProgressStructure.label = 'Path Number:';
   %
   if IsConditional==1;
      ProgressStructure.name = ['Conditional Zero Lower Bound Forecasts - ' NameStr];
   else;
      ProgressStructure.name = ['Zero Lower Bound Forecasts - ' NameStr];
   end;
   ProgressStructure.CurrINI = CurrINI;
   WaitHandle = ProgressDLG(0,ProgressStructure);
   set(WaitHandle,'Color',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'));
   drawnow;
end;
%
% start the iterations
%
if isempty(ExtraDraws)==0;
   NumExtraPaths = size(ExtraDraws.Shocks,3);
else;
   NumExtraPaths = 0;
end;
abort = '';
BadDraws = 0;
BadDrawInformation = [];
NumExtra = 1;
NeedMoreShocks = 0;
ZLBYPaths = zeros(n,h,NumPaths);
ZLBKsiPaths = zeros(r,h,NumPaths);
Meaneta = zeros(q,h);
Data.Shocks = Shocks;
Data.HistState = HistState;
Data.MeasError = MeasError;
Data.UsedShocks = zeros(q,h,NumPaths);
Data.UsedHistState = HistState;
Data.UsedMeasError = zeros(n,h,NumPaths);
Data.UsedExtraData = 0;
MeanOrigShocks = zeros(q,h);
for it=1:NumPaths;
   %
   % check if we should update the progress dialog
   %
   if DSGEModel.ShowProgress==1;
      abort = get(WaitHandle,'UserData');
      if (strcmp(abort,'cancel')==1)|(strcmp(abort,'done')==1);
         ZLBYPaths = [];
         ZLBKsiPaths = [];
         break;
      else;
         ProgressDLG([it/NumPaths it]);
      end;
   end;
   %
   % Step 1: initialize anticipated shocks
   %
   At = zeros(DSGEModel.ZLBSampleT+1,1);
   %
   % step 2: initialize predetermined variable
   %
   etaMatrix = zeros(q,h);
   wMatrix = zeros(n,h);
   Ksi = HistState(:,it);
   eta = Shocks(:,:,it);
   MeanOrigShocks = MeanOrigShocks+((1/NumPaths)*eta);
   w = MeasError(:,:,it);
   wt = w(:,1);
   etat = eta(:,1);
   Xt = [etat;Ksi;Ksi(DSGEModel.RtildePosition)];
   UsedExtraData = 0;
   %
   % start the loop over the prediction horizon
   %
   t = 1;
   while t<=h;
      %
      % step 3: compute the equilibrium deviation with the forward-back shooting algorithm
      %
      AtOld = At;
      [At,status,AlgorithmInformation] = ForwardBackShootingAlgorithm(Xt,At,ePG,Rzlb,Rbar,DSGEModel.AIMTolerance);
      %
      % check if we have valid anticipated shocks
      %
      if status==1;
         %
         % update the model variables z_t
         %
         Ksit = P(1:r,:)*[Xt;At];
         rhat = P(r+1,:)*[Xt;At];
         ZLBKsiPaths(:,t,it) = Ksit;
         %
         % now compute the path value for the observed variables
         %
         if length(size(H))==2;
            ZLBYPaths(:,t,it) = Ax(:,t)+(H'*Ksit)+wt;
         else;
            ZLBYPaths(:,t,it) = Ax(:,t)+(H(:,:,DSGEModel.LastPeriod+t)'*Ksit)+wt;
         end;
         etaMatrix(:,t) = etat;
         wMatrix(:,t) = wt;
         %
         % compute the initial value of A(t+1)
         %
         t = t+1;
         if t<=h;
            At = M*At;
            etat = eta(:,t);
            Xt = [etat;Ksit;rhat];
            wt = w(:,t);
         end;
      else;
         %
         % get new shocks to redo certain calculations
         %
         UsedExtraData = 1;
         BadDraws = BadDraws+1;
         BadDrawInformation = [BadDrawInformation;BadDraws it t AlgorithmInformation];
         if (IsConditional==1)&(NumExtraPaths>0);
            t = 1;
            if NumExtra<=NumExtraPaths;
               Ksit = ExtraDraws.HistState(:,NumExtra);
               Data.UsedHistState(:,it) = Ksit;
               rhat = Ksit(DSGEModel.RtildePosition);
               eta = ExtraDraws.Shocks(:,:,NumExtra);
               etat = eta(:,1);
               w = ExtraDraws.MeasError(:,:,NumExtra);
               wt = w(:,1);
               At = zeros(DSGEModel.ZLBSampleT+1,1);
               NumExtra = NumExtra+1;
            else;
               %
               % there are no more extra draws
               %
               NeedMoreShocks = 1;
            end;
         elseif (IsConditional==0)&(CenterShocks==1);
            %
            % we need to get new shocks
            %
            t = 1;
            if NumExtraN<=NumExtraPaths;
               Ksit = ExtraDraws.HistState(:,NumExtra);
               Data.UsedHistState(:,it) = Ksit;
               rhat = Ksit(DSGEModel.RtildePosition);
               eta = ExtraDraws.Shocks(:,:,NumExtra);
               etat = eta(:,1);
               w = ExtraDraws.MeasError(:,:,NumExtra);
               wt = w(:,1);
               At = zeros(DSGEModel.ZLBSampleT+1,1);
               NumExtra = NumExtra+1;
            else;
               NeedMoreShocks = 1;
            end;
         else;
            %
            % we can draw shocks freely
            %
            etat = randn(q,1);
            At = AtOld;
         end;
         Xt = [etat;Ksit;rhat];
      end;
      if NeedMoreShocks==1;
         break;
      end;
      drawnow;
   end;
   if NeedMoreShocks==1;
      break;
   end;
   Meaneta = Meaneta+((1/NumPaths)*etaMatrix);
   Data.UsedShocks(:,:,it) = etaMatrix;
   Data.MeasError(:,:,it) = wMatrix;
   Data.UsedExtraData = Data.UsedExtraData+UsedExtraData;
   drawnow;
end;
%
% close any wait dialog
%
if DSGEModel.ShowProgress==1;
   set(WaitHandle,'UserData','done');
   close(WaitHandle);
   drawnow;
   pause(0.02);
end;
%
if NeedMoreShocks==1;
   ZLBYPaths = [];
   ZLBKsiPaths = [];
   txt = ['YADA requires additional shock draws to compute the predictive distribution subject to the zero lower bound. The total number of available extra draws is equal to ' int2str(NumExtraPaths) '.'];
   About(txt,'information',['Zero Lower Bound - ' NameStr],200,500,CurrINI);
   return;
end;
%
% save results to file
%
if (strcmp(abort,'cancel')==0);
   if IsConditional==1;
      file = FixFilePath([DSGEModel.OutputDirectory '\predictions\ZLB-CondPrediction-' DSGEModel.NameOfModel '-' strrep(NameStr,' ','') '-' int2str(CenterShocks) '-' int2str(DSGEModel.FirstPeriod) '.' int2str(DSGEModel.LastPeriod) '-' int2str(DSGEModel.ZLBSampleT) '.' int2str(DSGEModel.RtildePosition) '.' int2str(DSGEModel.REqPosition) '-' int2str(DSGEModel.ShockControlMethod) '-' int2str(h) '-' int2str(NumPaths) '.' int2str(DSGEModel.AdditionalDraws) '.mat']);
   else;
      file = FixFilePath([DSGEModel.OutputDirectory '\predictions\ZLB-Prediction-' DSGEModel.NameOfModel '-' strrep(NameStr,' ','') '-' int2str(CenterShocks) '-' int2str(DSGEModel.FirstPeriod) '.' int2str(DSGEModel.LastPeriod) '-' int2str(DSGEModel.ZLBSampleT) '.' int2str(DSGEModel.RtildePosition) '.' int2str(DSGEModel.REqPosition) '-' int2str(h) '-' int2str(NumPaths) '.' int2str(DSGEModel.AdditionalDraws) '.mat']);
   end;
   save(file,'ZLBYPaths','ZLBKsiPaths','ZLBYMean','ZLBKsiMean','KsitT','etaMean','Meaneta','MeanOrigShocks','BadDrawInformation','Data');
else;
   status = 0;
end;

%
% end of ZLBDSGEPredictionPathsTheta.m
%
